define([
		"WFEditor/core/error/ErrorItem",
		"WFEditor/core/error/ErrorLevel",
		"utils/Observable"
	],
	function (
		ErrorItem,
		ErrorLevel,
		Observable
	) {
		"use strict";

		/**
		 * This handler can be used for error reporting in the application.
		 * @exports ErrorHandler
		 * @class
		 * @mixes Observable
		 */
		var ErrorHandler = function () {
			Observable.mixin(this, "errorhandler");
		};

		/**
		 * @private
		 * Create an error of the given level.
		 * @param {ErrorLevel} errorLevel - The error level severity
		 * @param {Array.<Any>} args - Arguments of the helper functions
		 */
		ErrorHandler.prototype._createError = function (errorLevel, args) {
			var e = new Error(args[0]);
			var i = Array.prototype.slice.call(args, 1);
			var errorItem = new ErrorItem(errorLevel, e, i);
			this._emit("error", errorItem);
		};

		/**
		 * Create a SEVERE error
		 * @param {String} message - The error message
		 * @param {Any} otherInfo - Other info to add
		 */
		ErrorHandler.prototype.severe = function (message, otherInfo) {
			var args = arguments;
			this._createError(ErrorLevel.SEVERE, args);
		};

		/**
		 * Create a WARNING error
		 * @param {String} message - The error message
		 * @param {Any} otherInfo - Other info to add
		 */
		ErrorHandler.prototype.warning = function (message, otherInfo) {
			var args = arguments;
			this._createError(ErrorLevel.WARNING, args);
		};

		/**
		 * Create an INFO error
		 * @param {String} message - The error message
		 * @param {Any} otherInfo - Other info to add
		 */
		ErrorHandler.prototype.info = function (message, otherInfo) {
			var args = arguments;
			this._createError(ErrorLevel.INFO, args);
		};

		/**
		 * Create a FINE error
		 * @param {String} message - The message
		 * @param {Any} otherInfo - Other info to add
		 */
		ErrorHandler.prototype.fine = function (message, otherInfo) {
			var args = arguments;
			this._createError(ErrorLevel.FINE, args);
		};

		/**
		 * Create a FINER error
		 * @param {String} message - The message
		 * @param {Any} otherInfo - Other info to add
		 */
		ErrorHandler.prototype.finer = function (message, otherInfo) {
			var args = arguments;
			this._createError(ErrorLevel.FINER, args);
		};

		/**
		 * Create a FINEST error
		 * @param {String} message - The message
		 * @param {Any} otherInfo - Other info to add
		 */
		ErrorHandler.prototype.finest = function (message, otherInfo) {
			var args = arguments;
			this._createError(ErrorLevel.FINEST, args);
		};

		return ErrorHandler;
	});