define([],
	function () {
		"use strict";

		/**
		 * This object represents an error. You should never create an instance yourself. Use ErrorHandler for that.
		 * @exports ErrorItem
		 * @class
		 * @param {ErrorHandler.ErrorLevel} The level of severity of the error
		 * @param {Error} The error object
		 * @param {Array.<Any>} Any additional info for the error
		 */
		var ErrorItem = function (level, error, info) {
			/**
			 * @member {ErrorHandler.ErrorLevel} Error level
			 */
			this._level = level;
			/**
			 * @member {Error} Error object
			 */
			this._error = error;
			/**
			 * @member {Array.<Any>} Any additional info for the error
			 */
			this._info = info;
		};

		/**
		 * Get the error level of severity
		 * @returns {ErrorHandler.ErrorLevel}
		 */
		ErrorItem.prototype.getLevel = function () {
			return this._level;
		};

		/**
		 * Get the stack where the error occured
		 * @returns {String}
		 */
		ErrorItem.prototype.getStack = function () {
			return this._error.stack;
		};

		/**
		 * Get the message of the error
		 * @returns {String}
		 */
		ErrorItem.prototype.getMessage = function () {
			return this._error.message;
		};

		/**
		 * Get a String representation of the error and additional info
		 * @returns {String}
		 */
		ErrorItem.prototype.toString = function () {
			var s = this.getLevel().name;
			s += " " + this._error.toString();
			if (_.size(this._info) > 0) {
				s += ", Details:";
				_.each(this._info, function (info) {
					s += ", " + info;
				});
			}
			return s;
		};

		return ErrorItem;
	});