define(["component/modal/Modal",
		"component/button/Button",
		"utils/webi18n!WFEditorWCR",
		"underscore",
		"jquery",

	],
	function (
		Modal,
		Button,
		i18n,
		_,
		$) {

		"use strict";

		/**
		 * Popup used to clear a workflow
		 * @exports ClearWorkflowPopup
		 * @class
		 */
		var ClearWorkflowPopup = function (wfEditorWCR) {
			this._wfEditorWCR = wfEditorWCR;

			// Popup content
			this._content = $("<div />").text(i18n.t("workflowClearWarning"));
			this._clearWarningPopup = new Modal(this._content);
			this._clearWarningPopup.setTitle(i18n.t("workflowClearTooltip"));

			this._clearWarningPopup.onClose(_.bind(function(){
				this.destroy();
			}, this._clearWarningPopup));
			
			this._cancelButton = new Button(null, null, false, "Cancel", i18n.t("configurationCancel"), true, _.bind(function () {
				this._clearWarningPopup.close();
			}, this));
			this._cancelButton.addClass("btn btn-default btn-md");

			this._clearButton = new Button(null, null, false, "OK", i18n.t("configurationOK"), true, _.partial(function (popup) {
				this.setEnabled(false);
				$.when(popup._wfEditorWCR.doClearWorkflow()).always(function () {
					// errors already logged
					popup._clearWarningPopup.close();
				});
			}, this));
			this._clearButton.addClass("btn btn-primary btn-md");

			this._buttonGroup = $("<div>").addClass("btn-group pull-right");
			this._buttonGroup.append(this._cancelButton.element).append(this._clearButton.element);
			this._clearWarningPopup.setFooter(this._buttonGroup);
		};


		/**
		 * Open the popup
		 */
		ClearWorkflowPopup.prototype.open = function () {
			this._clearWarningPopup.open();
			this._clearButton.setEnabled(true);
		};

		return ClearWorkflowPopup;
	});