define(["component/modal/Modal",
		"component/button/Button",
		"utils/webi18n!WFEditorWCR",
		"Template",
		"tpl!WFEditorWCR/actionPopup/ImportWorkflowPopupContent",
		"underscore",
		"jquery",

	],
	function (
		Modal,
		Button,
		i18n,
		Template,
		tpl,
		_,
		$) {

		"use strict";

		/**
		 * Popup used to import a workflow
		 * @exports ImportWorkflowPopup
		 * @class
		 */
		var ImportWorkflowPopup = function (wfEditorWCR) {
			this._wfEditorWCR = wfEditorWCR;

			// Confirm popup
			this._confirmContent = $("<div>").text(i18n.t("workflowImportConfirm"));
			this._importConfirm = new Modal(this._confirmContent);
			this._importConfirm.setTitle(i18n.t("workflowImportTooltip"));

			this._importConfirmCancel = new Button(null, null, false, "Cancel", i18n.t("configurationCancel"), true, _.bind(function () {
				this._doNotDestroyPopup = true;
				this._importConfirm.close();
				this._importPopup.open();
				this._importButton.setEnabled(true);
			}, this));
			this._importConfirmCancel.addClass("btn btn-default btn-md");

			this._importConfirm.onClose(_.bind(function () {
				if (!this._doNotDestroyPopup) {
					this._importConfirm.destroy();
					this._importPopup.destroy();
				}
				this._doNotDestroyPopup = false;
			}, this));

			this._importConfirmOK = new Button(null, null, false, "Cancel", i18n.t("configurationOK"), true, _.bind(function (popup) {
				wfEditorWCR.doImportWorkflow();
				this._importConfirm.close();
			}, this));
			this._importConfirmOK.addClass("btn btn-primary btn-md");
			this._importConfirmButtonGroup = $("<div>").addClass("btn-group pull-right");
			this._importConfirmButtonGroup.append(this._importConfirmCancel.element).append(this._importConfirmOK.element);
			this._importConfirm.setFooter(this._importConfirmButtonGroup);

			// Popup
			this._content = $("<div>");

			// Template for popup content
			this.t = new Template(i18n, tpl);
			//set the context for the template, this is the content of the popup
			this.t.setContext(this._content);
			//register change event
			this.t.bind("change", ".datafile", _.partial(this._dataFileChanged, this));
			this._importPopup = new Modal(this._content);
			this._importPopup.setTitle(i18n.t("workflowImportTooltip"));

			this._importPopup.onClose(_.bind(function () {
				if (!this._doNotDestroyPopup) {
					this._importConfirm.destroy();
					this._importPopup.destroy();
				}
				this._doNotDestroyPopup = false;
			}, this));

			this._cancelButton = new Button(null, null, false, "Cancel", i18n.t("configurationCancel"), true, _.bind(function () {
				this._importPopup.close();
			}, this));
			this._cancelButton.addClass("btn btn-default btn-md");
			this._importButton = new Button(null, null, false, "OK", i18n.t("configurationOK"), true, _.partial(function (popup) {
				this.setEnabled(false);
				popup._doNotDestroyPopup = true;
				popup._importPopup.close();
				popup._importConfirm.open();
			}, this));
			this._importButton.addClass("btn btn-primary btn-md");
			this._importButton.setEnabled(false);

			this._buttonGroup = $("<div>").addClass("btn-group pull-right");
			this._buttonGroup.append(this._cancelButton.element).append(this._importButton.element);
			this._importPopup.setFooter(this._buttonGroup);
		};

		/**
		 * Executed when data file is changed
		 */
		ImportWorkflowPopup.prototype._dataFileChanged = function (popup, elem) {
			var fileName = $(elem).val();
			popup._importButton.setEnabled(fileName);
		};

		/**
		 * Open the popup
		 */
		ImportWorkflowPopup.prototype.open = function () {
			this.t.d.taskTypeID = this._wfEditorWCR.getTaskTypeID();
			this.t.render();
			this._importPopup.open();
			this._importButton.setEnabled(false);
		};

		return ImportWorkflowPopup;
	});
