define([
		"WFEditor/components/Popup",
		"utils/webcss!WFEditorWCR/core/wfeditorwcr-style.css"
	],
	function (
		Popup
	) {

		"use strict";

		/**
		 * 
		 * @class
		 * @augments Popup
		 * @exports WFEditorWCRPopup
		 * @param {String} name - name used for the observable id
		 * @param {JQueryObject} containment - dom object to which this popup will be added
		 * @param {String} title - will be set in the popup header
		 */
		var WFEditorWCRPopup = function (name, containment, title) {
			Popup.call(this, name, containment);

			this.setTitle(title);

			this._closeBtn.element.on("mousedown", function (e) {
				e.stopPropagation();
			});
			
			this.element.addClass("WFEditorWCRPopup")

		};


		WFEditorWCRPopup.prototype = Object.create(Popup.prototype);
		WFEditorWCRPopup.prototype.constructor = Popup;

		/**
		 * @override
		 */
		WFEditorWCRPopup.prototype.fitContent = function () {
			// We don't want to fit content
		};

		/**
		 * @override
		 */
		WFEditorWCRPopup.prototype._initializeResizableDraggable = function () {
			Popup.prototype._initializeResizableDraggable.call(this);
			this.element.resizable("option", "containment", "document");
			this.element.draggable("option", "containment", "document");
		};

		/**
		 * get the JQuery element of the footer
		 * @returns {JQueryObject}
		 */
		WFEditorWCRPopup.prototype.getFooter = function () {
			return this._footer;
		};

		/**
		 * get the JQuery element of the header
		 * @returns {JQueryObject}
		 */
		WFEditorWCRPopup.prototype.getHeader = function () {
			return this._header;
		};


		/**
		 * The close button was pressed
		 * @protected
		 */
		WFEditorWCRPopup.prototype._closeButtonPressed = function (e) {
			var canCloseObj = {
				canClose: true
			};
			// send event to see if we can close the popup
			this._emit("closed", canCloseObj);
			if (canCloseObj.canClose) {
				this.hide();
			}
			e.stopPropagation();
		};

		return WFEditorWCRPopup;

	});
