define(["underscore", "wcr/sdrop"], function (_) {
	"use strict";
	/**
	 * @class
	 * Static Data Provider that can have custom sorting and use data object with specific name getter
	 * It uses the promise to fetch data.
	 * 
	 */
	var MapPromiseDataProvider = new Class({
		Extends: SDropDataProvider,
		options: {
			nameGetter: "getName",
			sortMethod: function (a, b) {
				if (a.name.toLowerCase() < b.name.toLowerCase()) {
					return -1;
				} else if (a.name.toLowerCase() > b.name
					.toLowerCase()) {
					return 1;
				} else {
					return 0;
				}
			},
			promise: null
		},
		reset: function () {
			this.flushDataStore(this.options.datastoreCapacity);
		},
		setData: function (dataSet) {
			var newList = _.map(dataSet.asObject(), function (item, id) {
				return {
					id: id,
					name: item[this.options.nameGetter](),
					item: item
				};
			}, this);

			this.addToStore(newList);
		},

		loadByID: function (id, delegate) {
			// notify listeners of loading
			if (delegate) {
				delegate.dataproviderIsLoading();
			}

			var pattern = this.registerPattern("###BYID###" + id + "####");
			pattern.loading = true;

			this.options.promise.done(_.bind(function (map) {
				pattern.loading = false;
				var item = map.get(id);
				if (item) {
					this.flushDataStore(1, pattern);
					var newobjs = this._addToStore([{
						id: id,
						name: item[this.options.nameGetter](),
						item: item
						}]);
					pattern.truncatedResults = false;

					if (delegate) {
						delegate.dataproviderLoadCompleted();
					}
				} else {
					if (delegate) {
						delegate.dataproviderLoadFailed("Could not find element with id " + id);
					}
				}
			}, this)).fail(_.bind(function () {
				if (delegate) {
					delegate.dataproviderLoadFailed("Could not find element with id " + id);
				}
			}, this));
		},

		load: function (textPattern, delegate) {
			if (delegate) {
				delegate.dataproviderIsLoading();
			}

			// check if pattern needs to be loaded
			if (!this.isPatternLoaded(textPattern, delegate)) {
				// pattern is not yet loaded, or is not loading yet
				var pattern = this.registerPattern(textPattern);
				pattern.loading = true;
				if (delegate) {
					pattern.delegates = [delegate];
				}

				this.options.promise.done(_.bind(function (map) {
					var resultList = [];
					var truncated = false;
					_.each(map.asObject(), function (item, id) {
						if (!truncated) {
							var name = item[this.options.nameGetter]();
							if (pattern.regex.test(name)) {
								if (resultList.length < this.options.truncate) {
									resultList.push({
										id: id,
										name: name,
										item: item
									});
								} else {
									truncated = true;
								}
							}
						}
					}, this);

					this.flushDataStore(resultList.length, pattern);

					var newObjs = this._addToStore(resultList);
					pattern.loaded = true;
					pattern.truncatedResults = truncated;

					this.setPatternLoadCompleted(pattern);


				}, this)).fail(_.bind(function () {
					this.loadFailed(pattern, "Error while loading data");
				}, this));
			}
		},

		getData: function (patternText, truncate) {
			return this.datastore
				.filter(function (item, index) {
					return this.regex.test(item.name);
				}, this.registerPattern(patternText))
				.sort(this.options.sortMethod);
		}
	});

	return MapPromiseDataProvider;

});