/**
 * ContentStatementElement object
 * @author HUBU
 */
define(["utils/XMLUtils",
		"./ContentConstants",
		"underscore"
	],
	function (XMLUtils,
		ContentConstants,
		_) {
		"use strict";


		/**
		 * @class Represents a content statement ELEMENT (part of a content statement).
		 * @exports ContentStatementElement
		 * @constructor
		 */
		var ContentStatementElement = function (json, parentStatement) {
			
			if (json) {
				this.master = json.master;
				this.content = json.content;
				this.state = json.state;
				this.language = json.language;
				this.created = json.created;
				this.updated = json.updated;
				this.originalApproval = json.approvals;
				this.approval = [];
				this.comment = json.comments;
				this.deleted = json.deleted;
				this.isEditable = json.isEditable;
				this.whyNonEditable = json.whyNonEditable;
				this.isReviewable = json.isReviewable;
				this.whyNonReviewable = json.whyNonReviewable;
				this.myApprovalStatus = json.myApprovalStatus;
				this.parentStatement = parentStatement;
			} else {
				this.master = false;
				this.content;
				this.state;
				this.language = null;
				this.created = null;
				this.updated = null;
				this.originalApproval = [];
				this.approval = [];
				this.comment = [];
				this.deleted = false;
				this.isEditable = "false";
				this.whyNonEditable = "NO_PCM_PERM_NODEV";
				this.isReviewable = "false";
				this.whyNonReviewable = "NO_PCM_PERM_NODEV";
				this.myApprovalStatus;
				this.parentStatement = parentStatement;
			}
		};
		
		/**
		 * Clone this ContentStatementElement.
		 * 
		 * @returns {ContentStatementElement} A perfect clone of this ContentStatementElement.
		 */
		ContentStatementElement.prototype.clone = function () {
			var clone = new ContentStatementElement();
			clone.master = this.master;
			clone.content = this.content;
			clone.state = this.state;
			clone.language = this.language;
			clone.created = this.created;
			clone.updated = this.updated;
			clone.originalApproval = this.originalApproval;
			clone.approval = this.approval;
			clone.comment = this.comment;
			clone.contentChanged = this.contentChanged;
			clone.commentChanged = this.commentChanged;
			clone.approvalChanged = this.approvalChanged;
			clone.doneApproveORReject = this.doneApproveORReject;
			clone.deleted = this.deleted;
			clone.isEditable = this.isEditable;
			clone.whyNonEditable = this.whyNonEditable;
			clone.isReviewable = this.isReviewable;
			clone.whyNonReviewable = this.whyNonReviewable;
			clone.myApprovalStatus = this.myApprovalStatus;
			clone.parentStatement = this.parentStatement;
			return clone;
		};
		
		
		/**
		 * A quick helper function, only for testing purposes.
		 *
		 * @param {String} master   True or false, if it is a master or not
		 * @param {String} content  Text content
		 * @param {String} state    State of the element
		 * @param {String} language Language string
		 */
		ContentStatementElement.prototype.initContentStatementElement = function (master, content, state, language, parentStatement) {
			this.master = master;
			this.content = content;
			this.state = state;
			this.language = language;
			this.parentStatement = parentStatement;
		};
		
		/**
		 * Revert back to the original approval change.
		 */
		ContentStatementElement.prototype.cancelStatusChange = function () {
			// Only do if a change occured.
			if (this.approvalChanged) {
				this.approval = [];
				// Reset approval array
				this.approvalChanged = false;
				this.doneApproveORReject = false;
			}
		};
		
		/**
		 * Revert back to the original approval change.
		 */
		ContentStatementElement.prototype.isMaster = function () {
			// Only do if a change occurred.
			if (typeof this.master == "boolean") {
				return this.master;
			} else if (typeof this.master == "string") {
				return this.master == "true";
			}
		};
		
		/**
		 * Set content for the element and update contentChanged flag to true.
		 * This will consider that element needs to be saved while saving the document.
		 * 
		 * @param {Object} content - content object which needs to be set in this content. {"content": "text content"}
		 */
		ContentStatementElement.prototype.setContent = function (content) {
			// First check whether content is not same.
			if (this.content.content != content.content) {
				this.contentChanged = true;
				this.content.content = content.content;
			}
		};
		
		/**
		 * Revert back to the original approval change.
		 */
		ContentStatementElement.prototype.isChanged = function () {
			return this.contentChanged || this.commentChanged || this.approvalChanged;
		};

		// DO NOT PUT ANY ADDITIONAL BUSINESS LOGIC IN THIS OBJECT.  THE LOGIC GOES INTO CONTENTMANAGER.JS

		return ContentStatementElement;
	});