/**
 * ContentStatementPermission object
 * @author RAHM
 */
define(["underscore"],
function(_){
    "use strict";
	
	
	/**
	 * @class Represents a content statement Permissions (part of a content statement).
	 * @exports ContentStatementPermission
	 * @constructor
	 */
	var ContentStatementPermission =  function(json){
		
		if (json) {
			this.role = json.role;
			this.tcp = json.tcp;
	        this.lang = json.lang;
		} else {
			this.role = null;
			this.tcp = null;
	        this.lang = null;
		}
	};
	
	/**
	 * Clone this ContentStatementPermission
	 * 
	 * @returns {ContentStatementPermission} A perfect clone of this ContentStatementPermission.
	 */
	ContentStatementPermission.prototype.clone = function () {
		var clone = new ContentStatementPermission();
		clone.role = this.role;
		clone.tcp = this.tcp;
        clone.lang = this.lang;
        return clone;
	};
	
	/**
	 * A quick helper function, only for testing purposes.
	 * 
	 * @param {String} role   Role
	 * @param {String} tcp  TextContentPermission (Editor, Content Manager, Approver)
	 * @param {String} language Language string
	 */
	ContentStatementPermission.prototype.initContentStatementPermission = function(role, tcp, language){
        this.role = role;
		this.tcp = tcp;
		this.lang = language;
	};
    
	return ContentStatementPermission;
});