define(["jquery",
        "underscore",
        "packaging-content-management/LanguagePanelRow",
        "utils/webi18n!TextContentStrings"],
		function($,
				_,
				LanguagePanelRow,
				TextContentStrings) {
	"use strict";
	
	var LanguagePanel = function (projectid) {
		this._languageIndexSequence = 1;
		this._languageRowCount = 0;
		this._projectid = projectid;
		this._targetLanguagePanelRows = [];
		this._languageIndexStore = [];
		this._languageIndexStore[1] = 1;
		this._wrapper = $('<div>').attr({'name': 'target_language_panel'});
		this._constructLanguagePanel();
	};
	
	/**
	 *  Method used to create Language panel View
	 */
	LanguagePanel.prototype._constructLanguagePanel = function() {
		
		this._wrapper.empty();
		// Create Statement field
		var languagePanelView_first = new LanguagePanelRow(this._languageIndexSequence, this._projectid, ++this._languageRowCount);
		this._addLangIndexChangeEvent(languagePanelView_first);
		
		// store the statement row HTML element in array
		this._targetLanguagePanelRows.push(languagePanelView_first);
		
		this._wrapper.append($('<table>').addClass("table-full-width").append(
								$("<thead>").append($("<tr>").append(
										$("<th>").text(TextContentStrings.t("PCM_GS1_LANGUAGE_INDEX")),
										$("<th>").text(TextContentStrings.t("PCM_GS1_LANGUAGE"))
								)
							)));
		this._addButtonRow = $("<tr>").append(
									$("<td>").addClass("add-padding-top").attr({'colspan': '3'}).append(
											$('<button>').attr({'type': 'button', 'name': 'addbutton', 'title': TextContentStrings.t("PCM_ADD")}).addClass("btn btn-primary btn-sm").append(
													$('<i>').addClass("icon icon-small icon-WCR_Actions-Add")
											)
									)
							);
						
			
		var languageRowJQElement = languagePanelView_first.getJQElement();
		$("table", this._wrapper).append(
				$("<tbody>").append(
						languageRowJQElement, this._addButtonRow
				)
		);
		
		this._showOrHideStatementActionButtons();
		
		this._addNewRow();
		this._removeStatementButton(languageRowJQElement);
	};
	
	/**
	 * Method to add new row 
	 * @param langPanelRow
	 */
	LanguagePanel.prototype._addNewRow = function() {
			
		 var addButton = this._addButtonRow.find('button[name="addbutton"]');
 	 
			var addButtonHandler = _.bind(function(event) {
				
				var langIndex = this._getAvailableIndex();
				var newLanguagePanelRow = new LanguagePanelRow(langIndex, this._projectid, ++this._languageRowCount);
				this._addLangIndexChangeEvent(newLanguagePanelRow);
				
				var newRow = newLanguagePanelRow.getJQElement();
				
				$(newRow).insertBefore(this._addButtonRow);
				
				// store the statement row HTML element in array
				this._targetLanguagePanelRows.push(newLanguagePanelRow);
				
				this._removeStatementButton(newLanguagePanelRow.getJQElement());
				
				this._showOrHideStatementActionButtons();
				
			}, this);
			
			addButton.click(addButtonHandler);
		};
		
		/**
		 * Method to Handle delete current row
		 * @param langPanelRow
		 */
		LanguagePanel.prototype._removeStatementButton = function(langPanelRow) {
			
			var removeButton =  langPanelRow.find('i[name="removebutton"]');
				
			var removeButtonHandler = _.bind(function(event) {
				this._targetLanguagePanelRows = $.grep(this._targetLanguagePanelRows, _.bind(function(row){
					if (row.getRowName() == event.attr('name')) {
						this._languageIndexStore[row._languageIndex] = this._languageIndexStore[row._languageIndex] - 1;
						return false;
					} else {
						return true;
					}
				}, this));
				
				this._showOrHideStatementActionButtons();
				event.remove();
				
			}, this, langPanelRow);
			
			removeButton.click(removeButtonHandler);
			
		};
	
		LanguagePanel.prototype._showOrHideStatementActionButtons = function() {
			if(this._targetLanguagePanelRows.length == 1 ) {
				var langPanelRow = this._targetLanguagePanelRows[0].getJQElement();
				langPanelRow.find('i[name="removebutton"]').hide();
			}else{
				var langPanelRow = this._targetLanguagePanelRows[0].getJQElement();
				langPanelRow.find('i[name="removebutton"]').show();
			}
		};
		
		/**
		 * Method to create Json Object for the all selected Language index and Language
		 * 
		 * @returns
		 */
		LanguagePanel.prototype.getTargetLanguagesJson = function() {
			var jsonObj = {"gs1_target_langs": []};
			
			for(var i=0; i<this._targetLanguagePanelRows.length;i++) {
				jsonObj.gs1_target_langs.push(this._targetLanguagePanelRows[i].getSelectedValues());
			}
			
			var jsonString = JSON.stringify(jsonObj);
			return jsonString;
		};
		
		
		LanguagePanel.prototype.getJQElement = function() {
			return this._wrapper;
		};
		
		/**
		 * This method used to get the available language index
		 * 
		 * @returns {Number}
		 */
		LanguagePanel.prototype._getAvailableIndex = function(requiredIndex) {
			if (requiredIndex) {
				if (!this._languageIndexStore[requiredIndex]) {
					this._languageIndexStore[requiredIndex] = 1;
				} else {
					this._languageIndexStore[requiredIndex] = this._languageIndexStore[requiredIndex] + 1;
				}
				return requiredIndex;
			} else {
				var highIndex = 1;
				$.each(this._languageIndexStore, _.bind(function (languageIndex) {
					if (0 == languageIndex) {
						highIndex--;
					} else if (!this._languageIndexStore[languageIndex] || 0 == this._languageIndexStore[languageIndex]) {
						highIndex = languageIndex;
						return false;
					}
					highIndex++;
				}, this));
				if (this._languageIndexStore[highIndex]) {
					this._languageIndexStore[highIndex] = this._languageIndexStore[highIndex] + 1;
				} else {
					this._languageIndexStore[highIndex] = 1;
				}
				return highIndex;
			}
		};
		
		/**
		 * Event Listener method for Language index Input field
		 * 
		 * @param langRowObj
		 */
		LanguagePanel.prototype._addLangIndexChangeEvent = function (langRowObj) {
			langRowObj.addLangIndexChangeEvent(_.bind(function () {
				var indexValue = langRowObj.getJQIndexInputField().val();
				var oldIndexValue = langRowObj.getJQIndexInputField().attr('old-value');
				if(typeof indexValue !== 'number' && (indexValue % 1) !== 0) {
					langRowObj.getJQIndexInputField().val('');
				    alert(TextContentStrings.t("PCM_VALID_LANGUAGE_INDEX"));
				} else {
					var index = this._getAvailableIndex(indexValue);
					langRowObj.getJQIndexInputField().attr('old-value', index);
					this._languageIndexStore[oldIndexValue] = this._languageIndexStore[oldIndexValue] - 1;
				}
			}, this));
		};
		
		
	return LanguagePanel;
});