define(["jquery",
        "underscore",
        "./text-content-utils/LanguageUtils",
        "./text-content-services/languageService",
        "utils/webi18n!TextContentStrings"],
		function($,
				_,
				LanguageUtils,
				languageService,
				TextContentStrings) {
	"use strict";
	
	var LanguagePanelRow = function (languageIndexSequence, projectid, languageRowCount) {
		this._languageIndex = languageIndexSequence;
		this._projectid = projectid;
		this._onIndexChageCallbacks = [];
		this._languagePanelRowName = 'LanguagePanelRow_'+ languageRowCount.toString();
		this._wrapper = $('<tr>').attr({'name': this._languagePanelRowName});
		this._constructView();
		this._addLanguageOptions();
	};

	/**
	 *  Create Language Panel Row View for the field Language Index and Language 
	 */
	LanguagePanelRow.prototype._constructView = function() {
		
		this._languageIndexInputField = $('<input>').attr({"type": "text", "name": "languageIndex", "value": this._languageIndex, "old-value": this._languageIndex}).addClass("form-control");
		this._languageIndexInputField.on('change', _.bind(function () {
			this._executeOnChageCallbacks();
		}, this));
		
		this._languageDropdown = $('<select>').attr({"name": "languageDropdown"}).addClass("form-control");
		
		var indexInput = $("<td>").append(this._languageIndexInputField);
		
		var languageSelect = $("<td>").append(this._languageDropdown);
		
		var deleteButton = $("<td>").append($('<i>').attr({'name': 'removebutton', 'title': TextContentStrings.t("PCM_REMOVE")}).addClass("icon icon-normal icon-WCR_Actions-Delete12"));
		
		this._wrapper.append(indexInput, languageSelect, deleteButton);
	};
	
	/**
	 * Add Languages to Language option filed Drop-down
	 */
	LanguagePanelRow.prototype._addLanguageOptions = function() {
		
		languageService.getLanguages(this._projectid, function (projMasterLangs, projTransLangs, allMasterLangs, allTransLangs) {
			this.projMasterLangs = projMasterLangs;
    		this.projTransLangs = projTransLangs;
    		this.allMasterLangs = allMasterLangs;
    		this.allTransLangs = allTransLangs;
		
			if(this.projMasterLangs.length == 0 && this.projTransLangs.length == 0 ) {
				this.combinedLang = LanguageUtils.mergeBothLanguagesTogether(this.allMasterLangs, this.allTransLangs);
			} else {
				this.combinedLang = LanguageUtils.mergeBothLanguagesTogether(this.projMasterLangs, this.projTransLangs);
			}
			
			this._createLangList();
			
		}.bind(this));
	};
	
	/**
	 * Add languages option for Given Language node (Source or target Languages)
	 * @param langNode
	 */
	LanguagePanelRow.prototype._createLangList = function () {
		_.each(this.combinedLang.values(), _.bind(function(langObj) {
			var code = langObj.langCode;
			var description = langObj.langDesc;
			this._languageDropdown.append($('<option>').attr({ "value" : code }).text(description));
		}, this));
	};
	
	/**
	 * get Selected Value of Language Index and Language
	 * @returns
	 */
	LanguagePanelRow.prototype.getSelectedValues = function() {
		var selected = {};
		selected.index = this._languageIndexInputField.val();
		selected.language = this._languageDropdown.val();
		return selected;
	};
	
	LanguagePanelRow.prototype.getRowName = function() {
    	 return this._languagePanelRowName;
     };

	LanguagePanelRow.prototype.getJQElement = function() {
		return this._wrapper;
	};
	
	LanguagePanelRow.prototype.getJQIndexInputField = function() {
		return this._languageIndexInputField;
	};
	
	LanguagePanelRow.prototype._executeOnChageCallbacks = function() {
		$.each(this._onIndexChageCallbacks, _.bind(function (index, callback) {
			callback();
		}, this));
	};
	
	LanguagePanelRow.prototype.addLangIndexChangeEvent = function(callback) {
		this._onIndexChageCallbacks.push(callback);
	};
	
	return LanguagePanelRow;
});