define(["jquery",
        "underscore",
        "packaging-content-management/ContentConstants",
        "utils/webi18n!TextContentSearchStrings"],
function($,
		_,
		ContentConstants,
		webi18n) {
	"use strict";
	
	/**
	 * Constructor to prepare statement field for text content and status.
	 */
	var TextContentSearchStatementFields = function() {
		this._wrapper = '';
		this._textContentElement = '';
		this._textStatusElement = '';
		this._constructTextSearchFields();
	};
	
	/**
	 * This private method invokes from the constructor of this class to create text and status fields.
	 */
	TextContentSearchStatementFields.prototype._constructTextSearchFields = function() {
		var divParameterFieldGroup = $('<div>').addClass('input-group full-width input-statement-field');

		// Create the text label
		var textLabel = $("<span>").addClass("input-group-addon textcontent-textaddon");
		textLabel.text(webi18n.t('STATEMENT_TEXT'));
		divParameterFieldGroup.append(textLabel);
		
		// Create text content input field
		this._textContentElement = $('<input>').addClass('form-control attachleft search-input-lg');
		this._textContentElement.attr({
			'aria-label': '...',
			'type' : 'text',
		});
		
		divParameterFieldGroup.append(this._textContentElement);
		
		// Create status drop down
		var divStatusDropDown = $('<div>').addClass('input-group-btn');
		
		this._buttonStatusDropDown = $('<button>').addClass('btn btn-default dropdown-toggle');
		this._buttonStatusDropDown.addClass('btn-itm-height');
		this._buttonStatusDropDown.attr({
			'type' : 'button',
			'data-toggle': 'dropdown',
			'aria-haspopup': 'true',
			'aria-expanded': 'false',
		});
		
		// Create label for status button as "Any" to be default
		this._buttonStatusDropDown.append(this._getIconForStatus(""));
		this._buttonStatusDropDown.append($('<span>',{"class" : "caret"}));
		
		divStatusDropDown.append(this._buttonStatusDropDown);
		
		// Prepare drop down option for the status
		var listStatusOptions = $('<ul>').addClass('dropdown-menu dropdown-menu-right');
		
		// Add the empty entry.
		var emptyLabel =  $("<a>",{"href":"#"}).append(this._getIconForStatus("")).append($("<span>").text(webi18n.t("ANY")));
		var emptyItem =  $('<li>').append(emptyLabel);
		emptyItem.on("click", function () {
			this._changeStatus("");
		}.bind(this));
		listStatusOptions.append(emptyItem);
		
		// Create status drop down list items.
		$.each(ContentConstants.SearchState, _.bind(function (statusKey, statusValue) {
			var entry = $("<a>", { "href" : "#" });
			entry.append(this._getIconForStatus(statusValue));
			entry.append($("<span>").text(webi18n.t(statusKey)));
			
			var item = $('<li>').append(entry);
			item.on('click', _.bind(function (event) {
				this._changeStatus(statusValue);
			}, this));
			
			// Add to options
			listStatusOptions.append(item);
		}, this));
		
		// Create a hidden input to hold the status.
		this._textStatusElement = $("<input>", {"type" : "hidden"});

		divStatusDropDown.append(listStatusOptions);
		divStatusDropDown.append(this._textStatusElement);
		divParameterFieldGroup.append(divStatusDropDown);
				
		this._wrapper = $('<table>').addClass('table-full-width').append(
			$('<tbody>').append(
				$('<tr>').append(
					$('<td>').append(divParameterFieldGroup)
				)
			)
		);
	};
	
	/**
	 * Method is to get icon for specific status to prepend in option of drop down and to display on the button.
	 */
	TextContentSearchStatementFields.prototype._getIconForStatus = function(status) {
		var icon = $("<i>").addClass("statement-status");
		if (status == ContentConstants.SearchState.DRAFT) {
			icon.addClass("icon icon-WCR_Text-Status-Draft draft");
		} else if (status == ContentConstants.SearchState.READY_FOR_APPROVAL) {
			icon.addClass("icon icon-WCR_Text-Status-Pending pending");
		} else if (status == ContentConstants.SearchState.SIGNED_OFF) {
			icon.addClass("icon icon-WCR_Text-Status-Approved approved");
		} else if (status == ContentConstants.SearchState.REJECT) {
			icon.addClass("icon icon-WCR_Text-Reject rejected");
		} else if (status == "") {
			icon.addClass("icon icon-WCR_More-horizontal statement-status");
		} else {
			console.log(status);
		}
		
		return $("<span>").append(icon);
	};
	
	/**
	 * Public method to set status on the drop down.
	 * Should be called to set value other then default value.
	 * For example ParameterField_textcontent is using this method for setting status from data value.
	 */
	TextContentSearchStatementFields.prototype.setStatusValue = function(status) {
		this._textStatusElement.val(status);
		this._buttonStatusDropDown.empty();
		this._buttonStatusDropDown.append(this._getIconForStatus(status));
		this._buttonStatusDropDown.append($("<span>",{"class" : "caret"}));
	};
	
	// Event to manage drop down option and selection to show on drop down button.
	TextContentSearchStatementFields.prototype._changeStatus = function(status) {
		this.setStatusValue(status);
		
		// Manually triggering for change, so parameter field can update data values
		this._textStatusElement.trigger("change");
	};
	
	/**
	 * @returns JQuery element for text content and status drop down as a single group.
	 */
	TextContentSearchStatementFields.prototype.getJQElement = function() {
		return this._wrapper;
	};

	/**
	 * @returns text content input field.
	 */
	TextContentSearchStatementFields.prototype.getTextContentElement = function() {
		return this._textContentElement;
	};

	/**
	 * @returns status hidden field.
	 */
	TextContentSearchStatementFields.prototype.getTextStatusElement = function() {
		return this._textStatusElement;
	};
	
	return TextContentSearchStatementFields;
});
