define(["jquery",
        "packaging-content-management/ContentStatement",
        "packaging-content-management/dyn-content/TextDynContent"], 
        function ($, 
        		ContentStatement,
        		TextDynContent) {
	
	
	
	/**
	 * 
	 */
	var DynContentService = function () {

		this._isFetchingDynContentFor = {};
		
		this._dynContentsByDocVerID = [];
	};
	
	/**
	 * 
	 * @param docVersionId
	 */
	DynContentService.prototype.getDynContentFor = function(docVersionId, forceUpdate) {

		// force the update of the cache
		if (forceUpdate) this._dynContentsByDocVerID[docVersionId] = null;

		var cacheItem = this._dynContentsByDocVerID[docVersionId];

		if (!cacheItem) {
			// create a new cache item for the data to be loaded
			cacheItem = {};
			this._dynContentsByDocVerID[docVersionId] = cacheItem;
		}

		if (!cacheItem.request) {
			// create a new Deferred object representing the result of this method (not only the AJAX call!)
			cacheItem.request = $.Deferred();
		} else {
			// return the Promise object in the cache (representing the request), which already contains 
			// the result or is waiting for the executed AJAX call to finish
			return cacheItem.request.promise();
		}

		// The parameters of the call
		var params = {
				"ajax": "1",
				"docverid": docVersionId
		};
		// The baseUrl of the service.
		var jspBaseUrl = window.webcenterConfig.jspBaseUrl;
		var url = jspBaseUrl+"GetDynContent.jsp?json=1&" + $.param(params, true);

			// Get the ajax JSON.
		$.getJSON(url, function(json) {

			// json = {
			//			textDynContents: [
			//				{
			//					contentStatement: <json>,
			//					locations: {
			//						fr-FR: [...],
			//						en-US: [...],
			//						...
			//					}
			//				},
			//				{
			//					...
			//				}, ...
			//			];
			// }

			var dynContents = [];

			// Parse and generate the text dynamic content objects. 
			for (var i=0; i < json.length; i++) {
				var contentStatement = new ContentStatement(json[i]);
				for (var j=0; j < contentStatement.element.length; j++) {
					var textElement = contentStatement.element[j];
					var locations = [];
					if (contentStatement.locationList && contentStatement.locationList[textElement.language]) {
						locations = contentStatement.locationList[textElement.language];
					}
					var textDynContent = new TextDynContent(textElement, locations);
					dynContents.push(textDynContent);
				}
			}

			// return the parsed value for the request.done() method.
			cacheItem.request.resolve(dynContents);

		}.bind(this)).fail(function(jqXHR, textStatus, errorThrown) {
			// TODO
		});

		// value not yet prepared in the cache (the loading is in progress) - return a Promise object
		// to be able to register callbacks in the caller(s)
		return cacheItem.request.promise();
	};
	
	return new DynContentService();
});
