/**
 * @author KADA
 */

define(["jquery",
        "underscore",
        "../ContentStatementElement",
        "utils/webcss!./LanguageIndicatorStyle.css"],
		function($, _, ContentStatementElement){
	
	"use strict";
	
	/**
	 * This class is the visual representation of a single language in the LanguageIndicator.
	 */
	var LanguageIndicatorEntry = function(language, isAvailable, showCompact, disable) {
		
		// Save the language
		this._language = language;
		
		// By default indicate that this flag is available.
		this._isAvailable = isAvailable;
		
		// By default is this entry not editable.
		this._editable = false;
		
		if('undefined' !== typeof disable && disable) {
			this._disable = disable;
		} else {
			this._disable = false;
		}
		
		// All the callbacks that should be executed when a flag is clicked.
		this._onClicked = [];
		
		// Create the root 'wrapper' element for this widget.
		this._wrapper = $("<div>",{'class': 'LanguageIndicatorEntry',
									'data-lang-code':language.langCode});
		// In compact mode, we should add a class to this wrapper.
		if (showCompact) {
			this._wrapper.addClass('LanguageIndicatorEntry-Compact');
		}
		
		// Create the flag image.
		var flagFile = language.langFlag ? language.langFlag : "_unknown.png";
		this._flag = $("<div>", {'class':'LanguageIndicatorEntry-Flag'});
		this._flag.css('background-image', 'url("'+webcenterConfig.imageBaseUrl+'country_flags/big/'+flagFile+'")');
		this._flagOverlayIcon = $("<i>", {'class':'LanguageIndicatorEntry-flagOverlay icon'});
		this._flag.append(this._flagOverlayIcon);
		this._wrapper.append(this._flag);
		
		// In non compact mode (e.g. 'all', also show the name of the language.
		this._description;
		if (!showCompact) {
			this._description = $("<div>",{'class': 'LanguageIndicatorEntry-Description'}).text(language.langDesc);
			this._wrapper.append(this._description);
		}
		
		// If this language is not available, hide this entry.
		if (!this._isAvailable) {
			this._wrapper.addClass('LanguageIndicatorEntry-NotAvailable');
		}
		
	};
	
	/**
	 * Set whether or not this should be shown in read only mode.
	 */
	LanguageIndicatorEntry.prototype.getEditable = function () {
		return this._editable;
	};
	
	/**
	 * Set whether or not this should be shown in read only mode.
	 */
	LanguageIndicatorEntry.prototype.setEditable = function () {
		
		if(this._disable) {
			// This entry is not editable.
			this._editable = false;
			
			// Start showing this widget.
			this._wrapper.show();
			
			return;
		}
		
		// This entry is editable.
		this._editable = true;
		
		// Start showing this widget.
		this._wrapper.show();
		
		// Add a class to enable hover effects
		this._wrapper.addClass("Editable");
		
		// Change the indicator on hover
		this._wrapper.hover(_.bind(function () {
			if (this._isAvailable) {
				this._flagOverlayIcon.attr("class", "LanguageIndicatorEntry-flagOverlay icon icon-WCR_Filter-Remove12");
			} else {
				this._flagOverlayIcon.attr("class", "LanguageIndicatorEntry-flagOverlay icon icon-WCR_Filter-Add12");
			}
		}, this), _.bind(function () {
			this._flagOverlayIcon.attr("class", "LanguageIndicatorEntry-flagOverlay icon");
		}, this));

		// Now link the callback to the click event of the wrapping jQuery element.
		this._wrapper.on("click", _.bind(function(){
			$.each(this._onClicked, _.bind(function (index, callback){
				callback(this._language.langCode);
			}, this));
		}, this));
	};
	
	/**
	 * Return the code of the language that this entry represents.
	 */
	LanguageIndicatorEntry.prototype.getLangCode = function () {
		return this._language.langCode;
	};
	
	/**
	 * Return the language that this entry represents.
	 */
	LanguageIndicatorEntry.prototype.getLanguage = function () {
		return this._language;
	};
	
	/**
	 * Return whether this entry is available or not.
	 */
	LanguageIndicatorEntry.prototype.isAvailable = function () {
		return this._isAvailable;
	};
	
	/**
	 * When this languageEntry is clicked.
	 */
	LanguageIndicatorEntry.prototype.onLanguageClicked = function (callback) {
		this._onClicked.push(callback);
	};
	
	/**
	 * Get the wrapping jQuery element.
	 */
	LanguageIndicatorEntry.prototype.getJQElement = function () {
		return this._wrapper;
	};
	
	return LanguageIndicatorEntry;
});