define(["underscore",
        "jquery",
        "./ContentStatementElementWidgetView",
        "../ContentConstants",
        "utils/webi18n!TextContentStrings"],
	function (_,
			$,
			ContentStatementElementWidgetView,
			ContentConstants,
			TextContentStrings) {
	
		"use strict";

		/**
		 * Create the ViewerContentStatementElementWidgetView.
		 * 
		 * @param {Object} text - The content object from the ContentStatementElementWidget represented by this widget.
		 * @param {Array} comments - The array of Comment objects from the ContentStatementElementWidget represented by this widget.
		 * @param {String} state - The state of the ContentStatementElementWidget represented by this widget.
		 * @param {String} language - The language object of the ContentStatementElementWidget represented by this widget.
		 * @param {Boolean} canEdit - Whether or not the ContentStatementElementWidget represented by this widget can be edited by the current user.
		 * @param {Boolean} whyCanNotEdit - Why the ContentStatementElementWidget represented by this widget can't be edited by the current user.
		 * @param {Boolean} canReview - Whether or not the ContentStatementElementWidget represented by this widget can be reviewed by the current user.
		 * @param {Boolean} whyCanNotReview - Why the ContentStatementElementWidget represented by this widget can't be reviewed by the current user.
		 * @param {Boolean} isContentManagerView - Specifies whether the current user is a Content_Manager
		 * @param {String} myApprovalStatus - Approval status of TextContent element for a Approver
		 * @param {Object} [viewOptions] - An object containing some specific variables indicating how the widget will be rendered.
		 * @param {String} markedApprovalState - Approval action taken if any, Current status otherwise
		 * @param {String} type - text type as "richtext" or "simple".
		 */
		var ViewerContentStatementElementWidgetView = function (text, isMaster, comments, state, languageCode, canEdit, whyCanNotEdit, canReview, whyCanNotReview, isContentManagerView, myApprovalStatus, viewOptions, markedApprovalState, type) {
			
			// call the parent prototype object to inherit properties
			ContentStatementElementWidgetView.call(this, text, isMaster, comments, state, languageCode, canEdit, whyCanNotEdit, canReview, whyCanNotReview, isContentManagerView, myApprovalStatus, viewOptions, markedApprovalState, type);
			
			// Do not show tooltips in the viewer.
			this._showTooltips = false;
			
			// call the overridden function to render the doctablex view of the widget
			this._constructView();
		};
		
		ViewerContentStatementElementWidgetView.prototype = Object.create(ContentStatementElementWidgetView.prototype);
		ViewerContentStatementElementWidgetView.prototype.constructor = ViewerContentStatementElementWidgetView;
		ViewerContentStatementElementWidgetView.prototype.parent = ContentStatementElementWidgetView.prototype;
		
		/**
		 * First override the content height before executing the initializeDOMElements method.
		 */
		ViewerContentStatementElementWidgetView.prototype._initializeDOMElements = function () {
			this._contentHeight = 76;
			ContentStatementElementWidgetView.prototype._initializeDOMElements.call(this);
		};
		
		/**
		 * Construct the HTML elements of this view.
		 * 
		 */
		ViewerContentStatementElementWidgetView.prototype._constructView = function () {
			this._wrapper.empty();
			
			this._initializeDOMElements();
			
			this._wrapper.addClass("Viewer");
			this._wrapper.addClass("enableTextSelection");
			
			// create widget elements and append to the wrapper
			this._wrapper.append(
					this._constructApprovalDiv(),
					$('<div>').addClass("TextContent-ContentCol").append(
							$('<div>').addClass("TextContent-InnerContentCol").append(this._constructTextDiv())
					)
			);
		};
		
		return ViewerContentStatementElementWidgetView;
		
});