define(["jquery"], function ($) {

	
	
	var ElementTypeService = function () {
		
		// These private variables will cache the returned languages and smartnames
        this._cachedElementTypes = undefined;
        
        this._cachedElementTypesWithoutDelimiter = undefined;
        
        // These private variables take care of preventing duplicate requests.
        this._elementTypesAreBeingRetrieved = false;
        
        this._onElementTypesRetrieved = [];
	};
	
	/**
	 * Method is to get readable format of the element type code without worried about delimiter "_" present in element type code string.
	 * 
	 * @param {String} elementTypeCode : Element type code for which readable format is required.
	 */
	ElementTypeService.prototype.getReadableElementType = function (elementTypeCode, callback) {
		if (elementTypeCode) {
			elementTypeCode = elementTypeCode.trim().replace(/\_/g, '');
			
			if (this._cachedElementTypesWithoutDelimiter) {
				if (this._cachedElementTypesWithoutDelimiter[elementTypeCode]) {
					callback(this._cachedElementTypesWithoutDelimiter[elementTypeCode]);
				} else {
					callback(elementTypeCode);
				}
			} else {
				this.getElementTypes(function (elementTypes) {
					var elementTypeReadable = elementTypes[elementTypeCode];
					if (elementTypeReadable) {
						callback(elementTypeReadable);
					} else {
						callback(elementTypeCode);
					}
				});
			}
		}
	};
	
	/**
	 * 
	 */
	ElementTypeService.prototype.getElementTypes = function (callback) {
		
		// Check if we have cached element types.
		if (this._cachedElementTypes) {
			
			callback(this._cachedElementTypes);
			
		} else {
			
			this._onElementTypesRetrieved.push(callback);
			
			if (!this._elementTypesAreBeingRetrieved) {
				
				this._elementTypesAreBeingRetrieved = true;
				
				// The baseUrl of the service.
		        var jspBaseUrl = window.webcenterConfig.jspBaseUrl;
				
				$.ajax({
					url: jspBaseUrl+"GetRowsFromList.jsp?listType=WCR_DB_List&indexed=1",
					type: "GET",
					dataType : "text",
					data : {
						outputType:"JSON",
						list : "ELEMENT_TYPES",
						maxrows : 200,
						maxcolrows : 200
					},
					success: function(jsonContent) {
						
						this._cachedElementTypes = {};
						this._cachedElementTypesWithoutDelimiter = {};
						
						try {
							var jsonObj;
							try {
								jsonObj = jsonParse(jsonContent);
							} catch(err) {			// if jsonParse fails, fall back on eval (which is slower)
								jsonObj = eval('(' + jsonContent + ')');
							}
							if (!jsonObj.success || jsonObj.success == "0") {
								callback('Cannot get values ' + jsonObj.error.code + ' : '+ jsonObj.error.message);
							} else {
								
								var elementTypesValues = [];
								var elementTypesColIndex = -1;
								var humanReadableValues = [];
								var humanReadableColIndex = -1;
								var elementCategoryValues = [];
								var elementCategoryColIndex = -1;
								var colorValues = [];
								var colorColIndex = -1;

								for (var i=0; i < jsonObj.columns.length; i++) {
									if (jsonObj.columns[i].name == "ELEMENT_TYPES") {
										elementTypesValues = jsonObj.columns[i].values;
										elementTypesColIndex = i;
									} else if (jsonObj.columns[i].name == "HUMAN_READABLE_FORM") {
										humanReadableValues = jsonObj.columns[i].values;
										humanReadableColIndex = i;
									} else if (jsonObj.columns[i].name == "ELEMENT_CATEGORY") {
										elementCategoryValues = jsonObj.columns[i].values;
										elementCategoryColIndex = i;
									} else if (jsonObj.columns[i].name == "COLOR") {
										colorValues = jsonObj.columns[i].values;
										colorColIndex = i;
									} 
								}
								var combinations = jsonObj.combinations;
								
								$.each(combinations, function(index, value) {
									
									var elementDetails = {};
									var combinationsArr = value.split('~');

									var indexElemType = combinationsArr[elementTypesColIndex];
									this._cachedElementTypes[elementTypesValues[indexElemType]] = elementDetails;
									
									// Adding element type details for element type code without delimiter.
									this._cachedElementTypesWithoutDelimiter[elementTypesValues[indexElemType].trim().replace(/\_/g, '')] = elementDetails;
									
									if (humanReadableColIndex >= 0) {
										elementDetails.elementHumanReadable = humanReadableValues[combinationsArr[humanReadableColIndex]];
									}
									
									if (elementCategoryColIndex >= 0) {
										elementDetails.elementCat = elementCategoryValues[combinationsArr[elementCategoryColIndex]];
									}
									
									if (colorColIndex >= 0) {
										elementDetails.color = colorValues[combinationsArr[colorColIndex]];
									}
									
								}.bind(this));
							}
						} catch (err) {
							alert(err);
						}
						
						// Execute the onLangsRetrieved callbacks.
		    			for (var i=0; i < this._onElementTypesRetrieved.length; i++) {
		    				this._onElementTypesRetrieved[i](this._cachedElementTypes);
		    			}
		    			
					}.bind(this)
				});
			}
		}
	};
	
	return new ElementTypeService();
	
});