define(["utils/Map",
        "underscore",
        "jquery"],
	function (Map,
			_,
			$) {
	
		"use strict";
		
		var LanguageUtils = function () {
			
		};
		
		
		LanguageUtils.prototype.removeFrom = function (langs, langsToRemove) {
			for (var i=0; i <langs.length; i++) {
				for (var j=0; j <langsToRemove.length; j++) {
					if (langs[i].langCode === langsToRemove[j].langCode) {
						langs.splice(i, 1); // remove the entry
					}
				}
			}
		};
		
		LanguageUtils.prototype.getLangForDescription = function (langs, langDescription) {
			for (var i=0; i <langs.length; i++) {
				if (langs[i].langDesc === langDescription) {
					return langs[i];
				}
			}
		};
		
		LanguageUtils.prototype.getLangForCode = function (langs, langCode) {
			for (var i=0; i <langs.length; i++) {
				if (langs[i].langCode === langCode) {
					return langs[i];
				}
			}
		};
		
		LanguageUtils.prototype.getLangsForCodes = function (langs, langCodes) {
			var matchedLangs = [];
			for (var i=0; i <langCodes.length; i++) {
				var lang = this.getLangForCode(langs, langCodes[i]);
				if (lang) {
					matchedLangs.push(lang);
				}
			}
			return matchedLangs;
		};
		
		LanguageUtils.prototype.isAProjectAttrLang = function(projectLangs, langCode) {
			for(var i=0; i<projectLangs.length; i++) {
				if(projectLangs[i].langCode === langCode) {
					return true;
					break;
				}
			}
			return false;
		};
		
		/**
		 * Combines two array of language objects in single Map. <br>
		 * This method is being used basically for merging master and translation languages.
		 * @param masterLanguages {Array of objects}	- 	{langCode : "en-US", langDesc : "English - United States", langFlag : "US.png", langIndex : "1"}
		 * @param translationLanguages {Array of objects}	-	{langCode : "nl-BE", langDesc : "Dutch - Belgium", langFlag : "BE.png", langIndex : "2"}
		 * @returns {Map} - Map  with language code as key and language object as value
		 * 	E.g. key = "en-US" and value = {langDesc : "English - United States", langFlag : "US.png", langIndex : "1"}
		 */
		LanguageUtils.prototype.mergeBothLanguagesTogether = function (masterLanguages, translationLanguages) {
			var completeBothLanguage = new Map();
			
			$(masterLanguages).each(function(lang) {
				completeBothLanguage.add(masterLanguages[lang].langCode, {langCode : masterLanguages[lang].langCode, langDesc : masterLanguages[lang].langDesc, langFlag : masterLanguages[lang].langFlag, langIndex : masterLanguages[lang].langIndex});
			});
			
			$(translationLanguages).each(function(lang) {
				completeBothLanguage.add(translationLanguages[lang].langCode, {langCode : translationLanguages[lang].langCode, langDesc : translationLanguages[lang].langDesc, langFlag : translationLanguages[lang].langFlag, langIndex : translationLanguages[lang].langIndex});
			});
			return completeBothLanguage;
		};
		
		/**
		 * This method will search for language code in map of Languages and returns lang object 
		 * {langCode : "en-US", langDesc : "English - United States", langFlag : "US.png", langIndex : "1"} 
		 * @param langsMap - Map of languages where language code is key and language object is value
		 * @ returns LangObject
		 */
		LanguageUtils.prototype.getLangForCodeFromMap = function (langsMap, languageCode) {
			if(languageCode == "")
				return;
			var langObj ; 
			_.each(langsMap.entries(), _.bind(function(langCode){
				//if languageCode is found in map, set the langObj and return false to break the loop
				if(langCode == languageCode){
					langObj = langsMap.get(langCode);
					return false;
				}
			}));
			
			return langObj;
		};
		
		return new LanguageUtils();
		
});