define(["jquery",
        "underscore",
        "utils/ConvertUtils",
        "packaging-content-management/ContentConstants",
        "packaging-content-management/ContentStatementElement",
        "packaging-content-management/text-content-element-widget/ContentStatementElementWidget",
        "packaging-content-management/language-indicator/LanguageIndicator",
        "packaging-content-management/text-content-services/languageService",
        "packaging-content-management/text-content-services/elementTypeService",
        "component/passwordpopup/PasswordValidationPopup",
        "utils/webcss!./InitTextInformationView.css"],
		function($,
				_,
				ConvertUtils,
				ContentConstants,
				ContentStatementElement,
				ContentStatementElementWidget,
				LanguageIndicator,
				languageService,
				elementTypeService,
				PasswordValidationPopup) {
			
			"use strict";
			
			var InitTextInformationView = function(webi18n) {
				
				this.webi18n = webi18n;
				this.languageMasterIndicator = null;
				this.languageTranslationIndicator = null;
				this._masterTextWidgets = [];
				this._translationsWidgets = [];
				
				this._onSaveClicked = [];
				this._onInvariantChange = [];
				this._onElementTypeChange = [];
				this._onApprovalTypeChange = [];
				this._onTextTypeChange = [];
				this._onLangChange = [];
				
				// Create a Container to hold the top buttons of the page.
				this._topWarningContainer = $("<div/>", {'class':'WarningContainer'});
				$("#text-information").append(this._topWarningContainer);
				
				// Create a Container to hold the properties of the text content file.
				this._propertiesContainer = $("<div/>", {'class':'PropertiesContainer' });
				this._propertiesTable = $("<table>", {'class' : 'PropertiesTable'});
				var propertiesPanelBody = $("<div>", {'class':'panel-body'}).append(this._propertiesTable);
				var propertiesPanel = $("<div>", {'class':'panel panel-default'}).append(propertiesPanelBody);
				this._lastUpdatedContainer = $("<div/>", {'class' :'LastUpdatedContainer'});
				this._propertiesContainer.append(this._lastUpdatedContainer, propertiesPanel);
				$("#text-information").append(this._propertiesContainer);
				
				// Create a Container to hold the master elements.
				this._masterElementsContainer = $("<div/>", {'class': 'MasterTextContainer'});
				$("#text-information").append(this._masterElementsContainer);
				
				// Create a Container to hold the translation widgets.
				this._translationElementsContainer = $("<div/>", {'class':'TranslationsContainer'});
				$("#text-information").append(this._translationElementsContainer);
				
				// Create a Container to hold the bottom buttons of the page.
				this._bottomWarningContainer = $("<div/>", {'class':'WarningContainer'});
				$("#text-information").append(this._bottomWarningContainer);
				
				this._topErrorContainer = $("<div>").addClass("ErrorContainer").hide();
				this._topErrorContainer.insertAfter("#text-information .WarningContainer:first-child");
				
				this._topLoadingErrorContainer = $("<div>").addClass("LoadingErrorContainer");
				this._topLoadingErrorContainer.insertAfter("#text-information .ErrorContainer");
			};
			
			/**
			 * Register the callbacks for this view.
			 */
			InitTextInformationView.prototype.onSaveClicked = function(callback) {
				this._onSaveClicked.push(callback);
			};
			InitTextInformationView.prototype.onInvariantChange = function(callback) {
				this._onInvariantChange.push(callback);
			};
			InitTextInformationView.prototype.onElementTypeChange = function(callback) {
				this._onElementTypeChange.push(callback);
			};
			InitTextInformationView.prototype.onApprovalTypeChange = function(callback) {
				this._onApprovalTypeChange.push(callback);
			};
			InitTextInformationView.prototype.onTextTypeChange = function(callback) {
				this._onTextTypeChange.push(callback);
			};
			InitTextInformationView.prototype.onLangChange = function(callback) {
				this._onLangChange.push(callback);
			};

			
			/**
			 * Hide the loading message.
			 */
			InitTextInformationView.prototype.hideLoadingMessage = function () {
				$("#loadingtext").hide();
				this._propertiesContainer.show();
			};
			
			/**
			 * Displays last updated date of text content document
			 * @param updated
			 */
			InitTextInformationView.prototype.getTextUpdatedDate = function(updated) {			
				if(updated) {
					this._lastUpdatedContainer.append(
							$('<span>').addClass("text-updatedDate").text(this.webi18n.t("PCM_LAST_UPDATED") + ': ' 
													+ ConvertUtils.getLangFormattedDateAndTime(this.webi18n, new Date(updated)))
					);
				}
			};
			
			/**
			 * Display a dismissable warning message.
			 * 
			 * @param message
			 */
			InitTextInformationView.prototype.displaySaveWarning = function(message, isDismissable) {
				
				// Clear the error containers
				this._topErrorContainer.empty().hide();
				
				var warning = $("<div>", {'class':'alert alert-warning', 'role' : 'alert'});
				if (isDismissable) {
					var warningDismiss = $("<button>", {'type':'button', 'class':'close', 'data-dismiss' : 'alert'});
					warningDismiss.append($("<span>").html("&times;"));
					warning.append(warningDismiss);
					warning.addClass("alert-dismissible");
				}
				var warningText = $("<span>").text(message);
				var saveButton = $("<button>", {'class':'btn btn-primary btn-sm pull-right SaveButton'}).text(this.webi18n.t("SAVE"));
				var saveCell = $("<td>",{'class' : 'SaveCell'}).append(saveButton);
				var warningInnerRow = $("<tr>").append($("<td>").append(warningText), saveCell);
				var warningInnerTable = $("<table>", {'class' : 'WarningTable'}).append(warningInnerRow);
				warning.append(warningInnerTable);
				
				var bottomWarning = warning.clone();
				var warnings = $().add(warning).add(bottomWarning);
				
				var saveButtonHandler = _.bind(function() {
					for(var i=0; i < this._onSaveClicked.length; i++) {
						this._onSaveClicked[i]();
					}
					$(".SaveButton", warnings).hide();
					$(".SaveCell", warnings).append($("<img/>",{'src':'./images/loaderTransparent.gif'}));
				}, this);
				
				$(".SaveButton", warnings).on("click", function () {
					if(this.isPasswordRequiredFunc()) {
						this._passwordPopup = new PasswordValidationPopup();
						this._passwordPopup.onSubmit(saveButtonHandler);
						this._passwordPopup.open();
					} else {
						saveButtonHandler();
					}
				}.bind(this));
				
				this._topWarningContainer.empty();
				this._bottomWarningContainer.empty();
				this._topWarningContainer.append(warning);
				this._bottomWarningContainer.append(bottomWarning);
				
			};
			
			
			/**
			 * Display a dismissable error message.
			 * 
			 * @param message
			 */
			InitTextInformationView.prototype.displaySaveError = function(message, isDismissable) {
				// Clear the warning containers
				this._topWarningContainer.empty();
				this._bottomWarningContainer.empty();
				
				var error = $("<div>", {'class':'alert alert-danger', 'role' : 'alert'});
				
				if (isDismissable) {
					var errorDismiss = $("<button>", {'type':'button', 'class':'close', 'data-dismiss' : 'alert'});
					errorDismiss.append($("<span>").html("&times;"));
					error.append(errorDismiss);
					error.addClass("alert-dismissible");
				}
				
				var errorText = $("<span>").text(message);
				var errorInnerRow = $("<tr>").append($("<td>").append(errorText));
				var errorInnerTable = $("<table>", {'class' : 'ErrorTable'}).append(errorInnerRow);
				error.append(errorInnerTable);
				
				this._topErrorContainer.empty();
				this._topErrorContainer.append(error).show();
				
			};
			
			/**
			 * Display a dismissable error message when viewing text content.
			 * 
			 * @param message
			 */
			InitTextInformationView.prototype.displayLoadError = function(message, isDismissable) {
				
				var error = $("<div>", {'class':'alert alert-danger', 'role' : 'alert'});
				
				if (isDismissable) {
					var errorDismiss = $("<button>", {'type':'button', 'class':'close', 'data-dismiss' : 'alert'});
					errorDismiss.append($("<span>").html("&times;"));
					error.append(errorDismiss);
					error.addClass("alert-dismissible");
				}
				
				var errorText = $("<span>").text(message);
				var errorInnerRow = $("<tr>").append($("<td>").append(errorText));
				var errorInnerTable = $("<table>", {'class' : 'ErrorTable'}).append(errorInnerRow);
				error.append(errorInnerTable);
				
				this._propertiesContainer.hide();
				$("#loadingtext").hide();
				this._topLoadingErrorContainer.empty();
				this._topLoadingErrorContainer.append(error);
				
			};
			
			
			/**
			 * Displays Project Attribute Languages selected
			 * @param contentManagerView
			 * @param textInfo
			 */
			InitTextInformationView.prototype.getProjectLanguagesRow = function(contentStatement){
				
				var contentManagerView = contentStatement.contentManagerView;
				
				var options = {showLabels:false};
				if ('true' !== contentManagerView) {
					options.readOnly = true;
				}
				
				this.languageMasterIndicator = new LanguageIndicator(contentStatement, true, options);
				var jqMasterElement = this.languageMasterIndicator.getJQElement();
				
				this.languageTranslationIndicator = new LanguageIndicator(contentStatement, false, options);
				var jqTranslationElement = this.languageTranslationIndicator.getJQElement();				
				
				var masterLangRow = $("<tr>", {'class':''});
				var masterLangLabelCell = $("<td>", {'class' : 'labelCell'});
				masterLangLabelCell.text(this.webi18n.t("PCM_MASTER_LANGUAGE"));
				var masterLangCell = $("<td>", {'class' : ''});
				masterLangCell.append(jqMasterElement);
				masterLangRow.append(masterLangLabelCell, masterLangCell);
				
				var transLangRow = $("<tr>", {'class':'TranslationLangRow'});
				var transLangLabelCell = $("<td>", {'class' : 'labelCell'});
				transLangLabelCell.text(this.webi18n.t("PCM_TRANSLATION_LANGUAGE"));
				var transLangCell = $("<td>", {'class' : ''});
				transLangCell.append(jqTranslationElement);
				transLangRow.append(transLangLabelCell, transLangCell);
				
				if(contentStatement.invariant === 'true') {
					transLangRow.hide();
				}
				
				languageService.getLanguages(contentStatement.projectId, function (projMastLangs, projTransLangs) {
					
					if (projMastLangs.length > 0 && projTransLangs.length > 0) {
						
						//If both source and target languages are defined show both source and target language list
						this._propertiesTable.append(masterLangRow);
						this._propertiesTable.append(transLangRow);
					
					} else if (projMastLangs.length > 0) {
						
						this._propertiesTable.append(masterLangRow);
						
					} else if (projTransLangs.length > 0) {
						
						this._propertiesTable.append(transLangRow);
					}
					
				}.bind(this));

				// What needs to happen when a language was added/removed.
				var langChangeCallback = function () {
					// Execute the necessary callbacks.
					for (var i=0; i < this._onLangChange.length; i++) {
						this._onLangChange[i]();
					}
					// Display the save warning.
					this.displaySaveWarning(this.webi18n.t("PCM_CONTENT_CHANGED"));
				}.bind(this);
				
				pcmEventBus.on("PCM.ContentStatement.langAdded", function(contentStatement, textElement){
					langChangeCallback();
					this.languageMasterIndicator.update();
					this.languageTranslationIndicator.update();
				}.bind(this));
				
				// When this is a new (e.g. copy/new) document statement
				pcmEventBus.on("PCM.ContentStatement.langRemoved", function(contentStatement, textElement){
					langChangeCallback();
					this.languageMasterIndicator.update();
					this.languageTranslationIndicator.update();
				}.bind(this));
				
				
			};
		
			/**
			 * Displays element type field and Mark as invariant
			 * @param contentManagerView
			 */
			InitTextInformationView.prototype.getMetaDataRow = function(contentManagerView, elementType, invariant) {

				/*
				 * Create a property entry that controls whether the text content file is invariant or not.
				 */ 
				
				var invariantRow = $("<tr>", {'class':''});
				var invariantLabel = $("<label>");
				var invariantInput = $("<input>", {'type':'checkbox'});
				if (invariant === "true") {
					invariantInput.attr('checked','checked');
				}
				var invariantLabelText = $("<span>", {'class': 'checkBoxLabel'}).text(this.webi18n.t("PCM_MAKE_INVARIANT"));
				invariantLabel.append(invariantInput, invariantLabelText);
				var invariantCell = $("<td>", {'colspan': '2'});
				invariantCell.append(invariantLabel);
				invariantRow.append(invariantCell);
				
				// If we do not have the permissions to change this, do not show it.
				if ('true' === contentManagerView) {
					this._propertiesTable.append(invariantRow);
				}
				
				// When somebody changes the value of the input.
				invariantInput.on("change", function() {
					
					// Execute the necessary callbacks.
					for (var i=0; i < this._onInvariantChange.length; i++) {
						
						this._onInvariantChange[i](invariantInput.is(":checked"));
					}
					
					// Display the save warning.
					this.displaySaveWarning(this.webi18n.t("PCM_CONTENT_CHANGED"));
					
				}.bind(this));
				
				
				/*
				 * Create a property entry that controls the element type of the text content file.
				 */ 
				
				var elementTypeRow = $("<tr>", {'class':''});
				var elementTypeLabelCell = $("<td>", {'class' : 'labelCell'});
				elementTypeLabelCell.text(this.webi18n.t("PCM_ELEMENT_TYPE"));
				var elementTypeInput = $("<select>", {'class':'', 'name': 'elementType'});
				var elementTypeInputCell = $("<td>", {'class' : ''});
				elementTypeService.getElementTypes(function (elementTypes) {
					//If element type is custom and not available in universal ElementType list,
					//add custom element type to drop down. Once user changes the element type, it wont be available again
					if(typeof elementTypes[elementType] === 'undefined'){
						var option = new Option(elementType, elementType);							
						$(option).attr("selected", "selected");
						elementTypeInput.append(option);
					}	
					
					// Populate the dropdown.
					for (var key in elementTypes) {
						if (elementTypes.hasOwnProperty(key)){
							var option = new Option(elementTypes[key].elementHumanReadable, key);
							if (key === elementType) {
								$(option).attr("selected", "selected");
							}
						}
						elementTypeInput.append(option);
					}
					// Depending on our permissions, add the input or the readonly value.
					if ('true' === contentManagerView) {
						elementTypeInputCell.append(elementTypeInput);
					} else {
						var text;
						if (elementTypes[elementType]) {
							text = elementTypes[elementType].elementHumanReadable;
						} else {
							text = elementType;
						}
						elementTypeInputCell.text(text);
					}
				});

				elementTypeRow.append(elementTypeLabelCell, elementTypeInputCell);
				this._propertiesTable.append(elementTypeRow);
				
				// When somebody changes the value of the input.
				elementTypeInput.on("change", function() {
					
					// Execute the necessary callbacks.
					for (var i=0; i < this._onElementTypeChange.length; i++) {
						this._onElementTypeChange[i](elementTypeInput.val());
					}
					
					// Display the save warning.
					this.displaySaveWarning(this.webi18n.t("PCM_CONTENT_CHANGED"));
					
				}.bind(this));
				
			};
			
			
			/**
			 * gets the approval setup row
			 * @param contentManagerView
			 */
			InitTextInformationView.prototype.getApprovalSetupRow = function(contentManagerView, approvalType) {
				var self = this;
				
					var approvalTypeString=null;
					var approvalTypeCode=null;
					
					if(approvalType == ContentConstants.ApprovalSetup.DEFAULT || approvalType == null || approvalType == "" || typeof approvalType ==='undefined')
					{
						approvalTypeString = self.webi18n.t("PCM_SETUP_DEFAULT");
						approvalTypeCode = ContentConstants.ApprovalSetup.DEFAULT;
						
						
					}else if(approvalType == ContentConstants.ApprovalSetup.ONE_OF_GROUP)
					{
						approvalTypeString = self.webi18n.t("PCM_ALLOFROLE_ONEPERGROUP");
						approvalTypeCode   = ContentConstants.ApprovalSetup.ONE_OF_GROUP;
					}else if(approvalType == ContentConstants.ApprovalSetup.ONE_OF_ROLE)
					{
						approvalTypeString = self.webi18n.t("PCM_ONE_OF_ROLE");
						approvalTypeCode   = ContentConstants.ApprovalSetup.ONE_OF_ROLE;
					}

					var approvalTypeRow = $("<tr>", {'class':''});
					var approvalTypeLabelCell = $("<td>", {'class' : 'labelCell'});
					approvalTypeLabelCell.text(this.webi18n.t("PCM_APPROVAL_SETUP"));
					var approvalTypeInput = $("<select>", {'class':'', 'name': 'approvalType'});
					approvalTypeInput.append(new Option(this.webi18n.t("PCM_SETUP_DEFAULT"), ContentConstants.ApprovalSetup.DEFAULT));
					approvalTypeInput.append(new Option(this.webi18n.t("PCM_ALLOFROLE_ONEPERGROUP"), ContentConstants.ApprovalSetup.ONE_OF_GROUP));
					approvalTypeInput.append(new Option(this.webi18n.t("PCM_ONE_OF_ROLE"), ContentConstants.ApprovalSetup.ONE_OF_ROLE));
					approvalTypeInput.val(approvalTypeCode);
					var approvalTypeInputCell = $("<td>", {'class' : ''});
					approvalTypeInputCell.append(approvalTypeInput);
					// Depending on our permissions, add the input or the readonly value.
					if ('true' === contentManagerView) {
						approvalTypeInputCell.append(approvalTypeInput);
					} else {
						approvalTypeInputCell.text(approvalTypeString);
					}
					approvalTypeRow.append(approvalTypeLabelCell, approvalTypeInputCell);
					this._propertiesTable.append(approvalTypeRow);
					
					// When somebody changes the value of the input.
					approvalTypeInput.on("change", function() {
						
						// Execute the necessary callbacks.
						for (var i=0; i < this._onApprovalTypeChange.length; i++) {
							this._onApprovalTypeChange[i](approvalTypeInput.val());
						}
						
						// Display the save warning.
						this.displaySaveWarning(this.webi18n.t("PCM_CONTENT_CHANGED"));
						
					}.bind(this));
					
			};
			
			/**
			 * gets the text type row
			 * @param contentManagerView {String} - 'true' if user is having content manager permission
			 * @param textType {String} - text type to identify for rich text or simple text
			 */
			InitTextInformationView.prototype.getTextTypeRow = function(contentManagerView, textType) {
				var textTypeRow = $("<tr>");
				this._propertiesTable.append(textTypeRow);

				var textTypeLabelCell = $("<td>", {'class' : 'labelCell'});
				textTypeLabelCell.text(this.webi18n.t("PCM_TEXT_TYPE"));
				
				var textTypeInputCell = $("<td>");
				textTypeRow.append(textTypeLabelCell, textTypeInputCell);
				
				// If type of the document is not defined to rich or simple text, then it considard as simple text.
				if (ContentConstants.TextType.RICHTEXT !== textType && ContentConstants.TextType.SIMPLE !== textType) {
					textType = ContentConstants.TextType.SIMPLE;
				}
				
				// Depending on our permissions, add the input or the readonly value.
				if ('true' === contentManagerView) {
					this._textTypeInput = $("<select>", {'name': 'textType'});
					this._textTypeInput.append(new Option(this.webi18n.t("PCM_SIMPLE_TEXT"), ContentConstants.TextType.SIMPLE));
					this._textTypeInput.append(new Option(this.webi18n.t("PCM_RICH_TEXT"), ContentConstants.TextType.RICHTEXT));
					this._textTypeInput.val(textType);
					
					textTypeInputCell.append(this._textTypeInput);
					
					// When somebody changes the value of the input.
					this._textTypeInput.on("change", _.bind(function() {
						var textTypeUpdated = false;
						
						// Execute the necessary callbacks.
						for (var i=0; i < this._onTextTypeChange.length; i++) {
							textTypeUpdated = this._onTextTypeChange[i](this._textTypeInput.val());
						}
						
						if (textTypeUpdated) {
							// Display the save warning.
							this.displaySaveWarning(this.webi18n.t("PCM_CONTENT_CHANGED"));
						}
						
					}, this));
				} else {
					textTypeInputCell.text(textType);
				}
			};
			
			/**
			 * This method is to set selection value of type drop down as rich text.
			 * Which will help user to roll back if user is not confirmed for type change as simple.
			 * 
			 * @param textType {String} - Text content documents text type as simple or rich.
			 */
			InitTextInformationView.prototype.setTextType = function(textType) {
				this._textTypeInput.val(textType);
			};
			
			/**
			 * Renders the text content element rows
			 * @param elements {Array} - All the elements available in the text content document.
			 * @param elementTypeCode {String} - element type code of current text content document.
			 * @param invariant {String} - true as string if document is marked as invariant.
			 */
			InitTextInformationView.prototype.getTextContentRows = function(elements, invariant) {
				var options = {};
				options.viewOptions = {widgetview: 'default'};

				/*
				 * Master Text
				 */
				this._masterElementsContainer.empty();
				
				// Extract the master elements
				var masterElements = $.grep(elements, function(element, index){
					return element.master == "true" && element.deleted !== true && element.state !== "EMPTY";
				});
				
				// Add a header to indacte the master statements.
				var masterTextTitle = $("<h2>").text(this.webi18n.t('PCM_MASTER_TEXT'));
				this._masterElementsContainer.append(masterTextTitle);
				
				// Add the widgets
				for (var i=0; i < masterElements.length; i++) {
					var widget = new ContentStatementElementWidget(masterElements[i], options);
					this._masterTextWidgets.push(widget);
					this._masterElementsContainer.append(widget.getJQElement());					
					widget.onChange(_.bind(function() {
						this.displaySaveWarning(this.webi18n.t("PCM_CONTENT_CHANGED"), false);
					}, this));
				}
				
				
				/*
				 * Translations
				 */
				this._translationElementsContainer.empty();
				
				// Extract the master elements
				var translationElements = $.grep(elements, function(element, index){
					return element.master != "true" && element.deleted !== true && element.state !== "EMPTY";
				});
				
				// Add a header to indacte the master statements.
				var translationsTitle = $("<h2>").text(this.webi18n.t('PCM_TRANSLATED'));
				this._translationElementsContainer.append(translationsTitle);
								
				// Add the widgets
				for (var i=0; i < translationElements.length; i++) {
					var widget = new ContentStatementElementWidget(translationElements[i], options);					
					this._translationsWidgets.push(widget);
					this._translationElementsContainer.append(widget.getJQElement());
					widget.onChange(_.bind(function() {
						this.displaySaveWarning(this.webi18n.t("PCM_CONTENT_CHANGED"), false);
					}, this));
				}
				
				// Hide translation's section if document is marked as invariant or does not contain any translation.
				if(invariant === 'true' || translationElements.length == 0) {
					this._translationElementsContainer.hide();
				}
			};
			
			InitTextInformationView.prototype.setIsPasswordRequiredFunc = function(isPasswordRequiredFunc) {
				this.isPasswordRequiredFunc = isPasswordRequiredFunc;
			};
									
			return InitTextInformationView;
		}
);

