/**
 * Text Information view generation.
 * @author PASH, RAHM
 */
define(["jquery",
        "underscore",
        "packaging-content-management/text-content-services/contentStatementService",
        "packaging-content-management/text-content-services/languageService",
        "packaging-content-management/text-content-utils/LanguageUtils",
        "packaging-content-management/ContentStatementPermissions",
        "packaging-content-management/ContentConstants",
        "utils/webi18n!TextContentStrings",
        "packaging-content-management/AlertPopup",
        "packaging-content-management/ContentStatement"], 
function($,
		_,
		contentStatementService,
		languageService,
		languageUtils,
		ContentStatementPermissions,
		ContentConstants,
		webi18n,
		AlertPopup,
		ContentStatement){
	
	"use strict";
	
	/**
	 * Constructor for TextInformationView.
	 * 
	 * @param documentVersionId {documentVersionId} a reference to the documentVersionId
	 */
	var TextContentPermissions = function(menuFile, documentVersionId, projectID, rolesToManageList, showActions, langEditorRolePrefix, langApproverRolePrefix) {
		
		this.menuFile = menuFile;
		this.projectID = projectID;
		this.documentVersionId = documentVersionId;
		this.rolesToManageList = rolesToManageList;
		this.langEditorRolePrefix = langEditorRolePrefix;
		this.langApproverRolePrefix = langApproverRolePrefix;
		this.isEditMode = false;
		
		this.showActions = showActions;
		this.permissionsTable = null;
		this.editPermissionsTable = null;		
		this.totalRows = 0;
		this.wrapper = null;
		this.permissionsArray = null;
		this._alertpopup = null;
		this._contentStatement = null;
		
		// Create a Container to hold the top buttons of the page.
		this._topWarningContainer = $("<div/>", {'class':'WarningContainer'});
         
        // JSP call to load text content
        contentStatementService.getContentStatement(documentVersionId, function(contentStatement) {
        	this._contentStatement = contentStatement;
			this.permissions = contentStatement.permission;
			
			languageService.getLanguages(projectID, function (projMasterLangs, projTransLangs, allMasterLangs, allTransLangs) {
				this.combinedLang = languageUtils.mergeBothLanguagesTogether(allMasterLangs, allTransLangs);
				this.projMasterLangs = projMasterLangs;
        		this.projTransLangs = projTransLangs;
        		this.allMasterLangs = allMasterLangs;
        		this.allTransLangs = allTransLangs;
				
        		this.totalLangSelected = this.projMasterLangs.length  + this.projTransLangs.length;
        		
        		this.permissionsArray = {
        				"SELECT": webi18n.t("PCM_SELECT"), 
                        "CONTENT MANAGER": webi18n.t("PCM_CONTENT_MANAGER"), 
                        "EDITOR": webi18n.t("PCM_EDITOR"),
                        "APPROVER" : webi18n.t("PCM_APPROVER")
                        };
        		
        		this.initDisplayView();
				
			}.bind(this));
			
		}.bind(this));
			
		this.addEvents();
	};
	
	/**
	 * Create Read-only Permissions table with header row
	 */
	TextContentPermissions.prototype.createROPermissionsTable = function() {
		this.permissionsTable = document.createElement("table");		
		this.permissionsTable.setAttribute('class', "table");
		this.permissionsTable.setAttribute('id',"permissionControllerTable");
		
		var header = this.permissionsTable.createTHead();
		var tr = document.createElement('tr'); header.appendChild(tr); 
		var th2 = document.createElement('th'); tr.appendChild(th2); th2.align = "left"; th2.innerHTML = webi18n.t("PCM_CONTENT_PERMISSION");
		var th3 = document.createElement('th'); tr.appendChild(th3); th3.align = "left"; th3.innerHTML = webi18n.t("PCM_LANGUAGE");
		var th4 = document.createElement('th'); tr.appendChild(th4); th4.align = "left"; th4.innerHTML = webi18n.t("PCM_ROLE");
		var th5 = document.createElement('th'); tr.appendChild(th5); th5.align = "left"; th5.className = "lastHeaderCol"; th5.innerHTML = webi18n.t("PCM_ASSIGNEE");
	};
	
	/**
	 * Create Editable Permission Table 
	 */
	TextContentPermissions.prototype.createEditPermissionsTable = function() {
		this.editPermissionsTable = document.createElement("table");		
		this.editPermissionsTable.setAttribute('class', "table");
		this.editPermissionsTable.setAttribute('id',"editPermissionTable");
		
		//header row creation
		var header = this.editPermissionsTable.createTHead();
		var tr = document.createElement('tr'); header.appendChild(tr); 
		var th2 = document.createElement('th'); tr.appendChild(th2); th2.align = "left"; th2.innerHTML = webi18n.t("PCM_CONTENT_PERMISSION");
		var th3 = document.createElement('th'); tr.appendChild(th3); th3.align = "left"; th3.innerHTML = webi18n.t("PCM_LANGUAGE");
		var th4 = document.createElement('th'); tr.appendChild(th4); th4.align = "left"; th4.innerHTML = webi18n.t("PCM_ROLE");
		var th5 = document.createElement('th'); tr.appendChild(th5); th5.className = "lastHeaderCol";
		
		var row = this.editPermissionsTable.insertRow(-1); row.setAttribute("id","editlastrow"); 
		var c2 = $(row.insertCell(0)); 
		var addButton = $("<a/>", {'href':'#', 'class':'btn btn-default addButton'});
		addButton.append(this.addPlusAction());
		addButton.append($("<span>").text(webi18n.t("PCM_ADD_PERMISSION")));
		c2.append(addButton);
		c2.attr("colspan","4");
		c2.addClass("darktext_greenlink_12_bold nobr");
		
		//For every permissions in the Text Content document create corresponding row in the table
		for (var permissionsCount = 0; permissionsCount < this.permissions.length; permissionsCount++){
			this.addEditPermissionsTableRow(this.permissions[permissionsCount]);
		}
	};
	
	/** <Wrapper>
	 * 	<table>
	 * 		<r1> blank </r1>
	 * 		<r2> Edit button </r2>
	 * 		<r3> change and cancel buttons </r3> //visible only for users who have "ShowActions" set to "True"
	 * 		<r4> blank </r4>
	 * 		<r5> Comment </r5>
	 * 		<r6> no permissions configured </r6>
	 * 		<r7> blank </r7>
	 * 		<r8> Read Only Permissions table </r8> //Available for all users
	 * 		<r9> Edit Permission table </r9> //visible only for users who have "ShowActions" set to "True"
	 * 	</table> 
	 */
	TextContentPermissions.prototype.createWrapper = function() {
		
		this.wrapper = document.createElement("div");
		this.wrapper.setAttribute('id', "managetcpform");
		
		var table = document.createElement("table");
		table.style.width = "100%";
		$(table).css({'max-width': '65%'});
		
		//blank row
		var r1 = table.insertRow(-1);		
		var r1c1 = r1.insertCell(0); r1c1.innerHTML = '<img src="images/clearpixel.gif" width="2" height="5" />';
		r1.insertCell(1);
		
		//Edit/save/cancel is available only for PM/Admin/Content Manager
		if(this.showActions !== 'false' &&  this._contentStatement.contentManagerView == 'true'){
			this.addEditButtonToTable(table);
			this.addSaveAndCancelButtonsToTable(table);//creates buttons row
			
		}
		
		if(this.permissions && this.permissions.length == 0) {
			
			var blankrow = table.insertRow(-1); blankrow.className = "white_background";
			var blankrowc1 = blankrow.insertCell(0);
			blankrowc1.innerHTML = '<img src="images/clearpixel.gif" width="2" height="5" />';
			blankrow.insertCell(1);//blank cell
			
			var r6 = table.insertRow(-1); r6.className = "white_background"; 
			r6.setAttribute('id',"nopermissionscommentrow"); r6.className = "white_background";
			var r6c1 = r6.insertCell(0);  r6c1.innerHTML = '<img src="images/clearpixel.gif" width="2" height="5" />';
			var r6c2 = r6.insertCell(1);r6c2.innerHTML = '<div class="darktext_darklink_11">'+webi18n.t("PCM_NO_PERMISSIONS_STATEMENT")+'</div>';			
		}
		
		//Create r7 blank row
		var r7 = table.insertRow(-1); r7.className = "white_background";
		var r7c1 = r7.insertCell(0);
		r7c1.innerHTML = '<img src="images/clearpixel.gif" width="2" height="5" />';
		r7.insertCell(1);//blank cell
		
		if(this.permissions && this.permissions.length > 0) {
			//Read Only Permissions table row
			var r8 = table.insertRow(-1); r8.className = "white_background"; r8.setAttribute("id", "readonlypermissions");
			var r8c1 = r8.insertCell(0);
			r8c1.innerHTML = '<img src="images/clearpixel.gif" width="2" height="5" />'; // offset
			var r8c2 = r8.insertCell(1);
			r8c2.align = "left";		
			this.createROPermissionsTable();
			
			var panel = $('<div>').addClass("panel panel-default").append($('<div>').addClass("panel-body").append(this.permissionsTable));
			$(r8c2).append(panel);
		}
		
		//Edit Permissions table row
		var r9 = table.insertRow(-1); r9.className = "white_background"; r9.setAttribute("id", "editpermissionsrow"); r9.setAttribute("style", "display:none");
		var r9c1 = r9.insertCell(0);
		r9c1.innerHTML = '<img src="images/clearpixel.gif" width="2" height="5" />'; // offset
		var r9c2 = r9.insertCell(1);
		r9c2.align = "left";		
		this.createEditPermissionsTable();
		
		var panel = $('<div>').addClass("panel panel-default").append($('<div>').addClass("panel-body").append(this.editPermissionsTable));
		$(r9c2).append(panel);
		
		this.wrapper.appendChild(table);
		
		
	};
		
	/**
	 * Create a row with 'Manage Permissions' Button. This button is available only if "ShowActions" variable is true
	 */
	TextContentPermissions.prototype.addEditButtonToTable = function(table) {		
		var r2 = table.insertRow(-1); r2.className = "white_background"; 
		r2.setAttribute('id',"editbuttonrow"); r2.className = "white_background";
		var r2c1 = r2.insertCell(0); r2c1.innerHTML = '<img src="images/clearpixel.gif" width="2" height="5" />';
		var r2c2 = r2.insertCell(1);
		
		var i1 = document.createElement("button");
		i1.setAttribute('type',"button");i1.setAttribute('name',"editpermissions");i1.setAttribute('id', "editpermissions");
		i1.setAttribute('value',webi18n.t("PCM_MANAGE_PERMISSIONS"));i1.className = "btn btn-primary";i1.innerHTML = webi18n.t("PCM_MANAGE_PERMISSIONS");
		
		r2c2.appendChild(i1);
	};	
	
	/**
	 * Create a row with 'Change' and 'Cancel' Button. This button is available only if "ShowActions" variable is true
	 */
	TextContentPermissions.prototype.addSaveAndCancelButtonsToTable = function(table) {		
				
		var r3 = table.insertRow(-1); r3.className = "white_background"; 
		r3.setAttribute('id',"savecancelbuttonrow"); r3.setAttribute('style',"display:none");	r3.className = "white_background";
		var r3c1 = r3.insertCell(0); r3c1.innerHTML = '<img src="images/clearpixel.gif" width="2" height="5" />';
		var r3c2 = r3.insertCell(1);

		// Initiate waring page section
		$(r3c2).append(this._topWarningContainer);
		this.displaySaveWarning(webi18n.t("PCM_CONTENT_CHANGED"));
		this.showOrHideSaveButton(false);
		
		//Create r4 blank row
		var r4 = table.insertRow(-1);r4.setAttribute('style',"display:none"); r4.setAttribute('id', "blankrow");
		r4.className = "white_background";
		var r4c1 = r4.insertCell(0);
		r4c1.innerHTML = '<img src="images/clearpixel.gif" width="2" height="5" />';
		r4.insertCell(1);//blank cell
		
		//Create r5 comment row
		var r5 = table.insertRow(-1);r5.setAttribute('style',"display:none");r5.setAttribute('id','commentrow');
		r5.className = "white_background";
		var r5c1 = r5.insertCell(0);
		r5c1.innerHTML = '<img src="images/clearpixel.gif" width="2" height="5" />';
		var r5c2 = r5.insertCell(1);//blank cell
		
		var infoText = document.createElement("div");
		infoText.className = "darktext_darklink_11";
		infoText.innerHTML = webi18n.t("PCM_PERMISSION_COMMENT");
		infoText.innerHTML += "</br>"+webi18n.t("PCM_PERMISSION_COMMENT1");
		infoText.innerHTML += "<a href=\"projdetailsassignroles.jsp?&menu_file="+this.menuFile+"&projectID="+ this.projectID +"&assignAllRoles=1&sourcePageDocVerId="+this.documentVersionId+"\">" +webi18n.t("PCM_PERMISSION_COMMENT2") + "</a>"; 
		
		var panel = $('<div>').addClass("panel panel-default").append($('<div>').addClass("panel-body").append($(infoText)));
		$(r5c2).append(panel);
	};
	
	/**
	 *  This method will prepare hyper link for the details of role, if role exists in WebCenter. 
	 */
	TextContentPermissions.prototype.getLinkString = function(id, name, linkIdentifier) {
		'use strict';
		var aElement = document.createElement("a");
		if (linkIdentifier || linkIdentifier == 0) {			
			aElement.setAttribute("href","javascript:ShowGroupDetails(\'" + id + "\', \'" + this.projectID + "\',\'" + linkIdentifier + "\')");
			aElement.innerHTML = name;
			
		} else {
			aElement.setAttribute("href","javascript:ShowUserDetails(\'" + id + "\')");
			aElement.innerHTML = name;
		}
		
		
		return aElement;
	};
		
	/**
	 * Creates one single row of Read Only mode Permissions with 7 columns
	 * 1. blank
	 * 2. TCP (text content permission)
	 * 3. Language
	 * 4. WebCenter Role
	 * 5. Group assigned
	 * 6. User assigned
	 * 7. blank
	 */
	TextContentPermissions.prototype.getROPermissionsTableRow = function(permission, role, language, langFlag) {
		this.totalRows++;
		
		var row = this.permissionsTable.insertRow(-1); row.setAttribute("id","row"+this.totalRows);
		if(this.totalRows%2 != 0)
			row.className = "alt";
		
		var c2 = row.insertCell(0); c2.className = "darktext_darklink_bold_11"; c2.innerHTML = permission;
		
		//Language column
		var c3 = row.insertCell(1); c3.className = "darktext_darklink_11"; 
		if(null != langFlag)
		{
			var imgElem = this.getImageElement("country_flags/"+langFlag);
			imgElem.setAttribute('style', "position: relative; margin-bottom: -4px");
			c3.appendChild(imgElem);
		}
		
		// to display master language as bold text
		var availableMasterLanguages = {};
		for ( var languageIndex in this.projMasterLangs) {
			if (this.projMasterLangs.hasOwnProperty(languageIndex)) {
				availableMasterLanguages[this.projMasterLangs[languageIndex].langDesc] = true;
			}
		}
		var isMaster = availableMasterLanguages[language];
		 if(isMaster){
			 c3.innerHTML += "&nbsp;<b>"+language+"</b>";
		 }else{
			 c3.innerHTML += "&nbsp;"+language;
		 }
		

		//Role Column
		var c4 = row.insertCell(2); c4.className = "darktext_darklink_11"; 
		if (role.id) {
			c4.appendChild(this.getImageElement("role.png"));c4.innerHTML += "&nbsp;";
			c4.appendChild(this.getLinkString(role.id, role.name, 1));
		} else if(role){
			c4.appendChild(this.getImageElement("role.png"));c4.innerHTML += "&nbsp;";
			c4.innerHTML += role;
		}
		
		//Assignee(s) column
		var c5 = row.insertCell(3); c5.className = "darktext_darklink_11"; 
		if (role.assignedUsers && role.assignedUsers.length > 0) {
			for ( var userCount = 0; userCount < role.assignedUsers.length; userCount++) {
				if (role.assignedUsers && role.assignedUsers[userCount]) {
					c5.appendChild(this.getImageElement("user.gif"));c4.innerHTML += "&nbsp;";
					if (role.assignedUsers[userCount].id) {
						var linkString = this.getLinkString(role.assignedUsers[userCount].id, role.assignedUsers[userCount].name);
						c5.appendChild(linkString);
					} else {
						c5.appendChild(role.assignedUsers[userCount].name);
					}
					var br = document.createElement("br");
					c5.appendChild(br);
				}
			}
		}
		
		if (role.assignedGroups && role.assignedGroups.length > 0) {
			for ( var groupCount = 0; groupCount < role.assignedGroups.length; groupCount++) {				
				if (role.assignedGroups && role.assignedGroups[groupCount]) {
					c5.appendChild(this.getImageElement("group.gif"));c5.innerHTML += "&nbsp;";
					if (role.assignedGroups[groupCount].id) {
						var linkString = this.getLinkString(role.assignedGroups[groupCount].id, role.assignedGroups[groupCount].name, 0);
						c5.appendChild(linkString);
					} else {
						c5.appendChild(role.assignedGroups[groupCount].name);
					}
					var br = document.createElement("br");
					c5.appendChild(br);
				}				
			}
		}		
		
	};
	
	TextContentPermissions.prototype.getImageElement = function(imageName){
		var image = document.createElement("img");
		image.setAttribute('src',"images/"+imageName);
		image.setAttribute('border', "0");
		
		return image;
	};
	
	/**
	 * Add the Minus action to this widget.
	 */
	TextContentPermissions.prototype.addMinAction = function() {		
		if (this.showActions) {
			var minAction = $("<i/>", {'class':'icon icon-normal activIcon icon-WCR_Reject', 'id':'removeButton', 'style' : 'cursor: pointer; margin: 2px;' });
			return minAction;
		}
		
	};
	
	/**
	 * Add the Plus action to this widget.
	 */
	TextContentPermissions.prototype.addPlusAction = function() {		
		if (this.showActions) {
			var plusAction = $("<i/>", {'class':'icon icon-normal activIcon  icon-WCR_Actions-Add', 'id':'addButton', 'style' : 'cursor: pointer; margin: 2px;' });
			return plusAction;
		}
	};
	
	/**
	 * Prepares the initial view for user with read-only permissions table
	 */
	TextContentPermissions.prototype.initDisplayView = function() {
		this.createWrapper();
		
		if(this.permissions){
			/** For Content Manager */
			var arrManagers = $.grep(this.permissions, function(permission, index){
				return permission.tcp == "CONTENT MANAGER";
			}.bind(this));
			this.getPermissionRows(webi18n.t("PCM_CONTENT_MANAGER"), arrManagers);
			
			/** For Editor */
			var arrEditor = $.grep(this.permissions, function(permission, index){
				return permission.tcp == "EDITOR";
			}.bind(this));
			this.getPermissionRows(webi18n.t("PCM_EDITOR"), arrEditor);
			
			/** For Approver */
			var arrApprovers = $.grep(this.permissions, function(permission, index){
				return permission.tcp == "APPROVER";
			}.bind(this));
			this.getPermissionRows(webi18n.t("PCM_APPROVER"), arrApprovers);
		}
		
		var permissionsTableHolder = $('#textContentPermissionsHolder');
		
		//Add form to the div
		permissionsTableHolder.html(this.wrapper);
	};
	
	/**
	 * Calculates the permissions, roles and prepares readonly permission row
	 * @param permType
	 * @param permission
	 */
	TextContentPermissions.prototype.getPermissionRows = function(permType, permission) {
		var langRoles = this.getResolvedSmartRoles(permType, permission);
		for (var i = 0; i < langRoles.length; i++){
			
			var langPermission = langRoles[i];
			var rolesFromProject = $.grep(this.rolesToManageList, function(roleFromProject, index) {
				return roleFromProject.name == langPermission.role; 
			});
			
			//To generate the Read-only row, we will use complete source language list.
			var langDetails = languageUtils.getLangForCodeFromMap(this.combinedLang, langPermission.lang);
			var langDesc = null;
			var langFlag = null;
			if (langDetails) {
				langDesc = langDetails.langDesc;
				langFlag = langDetails.langFlag;
			} else {
				langDesc = langPermission.lang;
				langFlag = null;
			}
			
			if (rolesFromProject.length == 0) {
				this.getROPermissionsTableRow(permType, langPermission.role, langDesc, langFlag);
			} else {
				for ( var rolesCount = 0; rolesCount < rolesFromProject.length; rolesCount++) {
					this.getROPermissionsTableRow(permType, rolesFromProject[rolesCount], langDesc, langFlag);
				}
			}
		}
	};
	
	/**
	 * Create one editable row of permission. If permission object is not null, each column is set with the values in permission object.
	 * If permission object is null, all the columns will have default value.
	 *  
	 * @param permission
	 */
	TextContentPermissions.prototype.addEditPermissionsTableRow = function(permission) {
		this.totalRows++;
		var editTableRows = document.getElementById("editPermissionTable") ? document.getElementById("editPermissionTable").rows.length-1 : this.totalRows;
		
		var row = this.editPermissionsTable.insertRow(editTableRows); 
		row.className = "altern editrow"; row.setAttribute("id","row"+this.totalRows);
		
		//Permissions drop down
		var c2 = row.insertCell(0); c2.className = "darktext_darklink_bold_11"; 
		
		var permissionSelect = document.createElement("select"); permissionSelect.className = "permissionSelect";
		$(permissionSelect).addClass('form-control');
		permissionSelect.id = "permissionSelect"+this.totalRows;
		c2.appendChild(permissionSelect);
		 
		//Create and append the options
		for (var perm in this.permissionsArray){
			if(this.permissionsArray.hasOwnProperty(perm)){
			    var option = document.createElement("option");
			    option.value = perm;
			    option.text = this.permissionsArray[perm];
			    permissionSelect.appendChild(option);
			    
				if(null != permission && permission.tcp === perm){
					option.setAttribute('selected','selected');
				}
			}
		}
		
		//Languages drop down - Languages selected by user
		var c3 = row.insertCell(1); c3.className = "darktext_darklink_11";
		var langSelect = this.createLangSelect(permission);
		c3.appendChild(langSelect);
				
		//Roles selected at Project Level
		var c4 = row.insertCell(2); c4.className = "darktext_darklink_11";

		var rolesSelect = document.createElement("select"); rolesSelect.className = "rolesSelect";
		$(rolesSelect).addClass('form-control');
		rolesSelect.id = "rolesSelect"+this.totalRows;
		c4.appendChild(rolesSelect); 
			
		var firstOption = document.createElement("option");
		firstOption.value = "blank";
		firstOption.text = "";
		rolesSelect.appendChild(firstOption);
		
		//Create and append the options
		for (var i = 0; i < this.rolesToManageList.length; i++) {
		    var option = document.createElement("option");
		    option.value = rolesToManageList[i].id;
		    option.text = rolesToManageList[i].name;
		    rolesSelect.appendChild(option);
		    
		    if(null != permission && permission.role === rolesToManageList[i].name){
				option.setAttribute('selected','selected');
			}
		}
		
		//Actions buttons - Add button should be added only for last row
		var c5 = $(row.insertCell(3));
		var removeButton = $("<a/>", {'href':'#', 'style':'#749412', 'class':'removeButton'}).append(this.addMinAction());
		c5.append(removeButton);
			
	};
	
	/**
	 * This will create a select drop down with the languages available at project level. Drop down will contain all the Master and Target Languages
	 * If there are no languages selected at project level, all the language codes in the text content document will be shown.
	 * 
	 * @param permission
	 * @returns {___langSelect56}
	 */
	TextContentPermissions.prototype.createLangSelect = function(permission){
		var langSelect = document.createElement("select"); langSelect.className = "langSelect";
		$(langSelect).addClass('form-control');
		langSelect.id = "langSelect"+this.totalRows;

		if(null != permission && permission.tcp && permission.tcp == "CONTENT MANAGER"){
			$(langSelect).hide();
		}
		
		var firstOption = document.createElement("option");
		firstOption.value = "blank";
		firstOption.text = "";
		langSelect.appendChild(firstOption);
		
		var secondOption = document.createElement("option");
		secondOption.value = "ALL LANGUAGES";
		secondOption.text = webi18n.t("PCM_SELECT_ALL");
		langSelect.appendChild(secondOption);
		
		if(null != permission && permission.lang && permission.lang.indexOf('[LANGUAGE]')>=0){
			secondOption.setAttribute('selected', 'selected');
		}
		// group the Master Languages
		var masterGroup =$('<optgroup>');
		if(this.projMasterLangs.length > 0){
			 masterGroup.attr('label', webi18n.t("PCM_MASTER_LANGUAGE"));
			$(langSelect).append(masterGroup);
		}
		
		
		for (var i=0; i < this.projMasterLangs.length; i++){
			var option = document.createElement("option");
		    option.value = this.projMasterLangs[i].langCode;
		    option.text = this.projMasterLangs[i].langDesc;
		    masterGroup.append(option);
		    
		    if(null != permission && permission.lang === this.projMasterLangs[i].langCode){
				option.setAttribute('selected', 'selected');
			}
		}
		// group the Translation Languages
		var translationGroup = $('<optgroup>');
		if(this.projTransLangs.length > 0){
			translationGroup.attr('label', webi18n.t("PCM_TRANSLATION_LANGUAGE"));
			$(langSelect).append(translationGroup);
		}
		
		for (var i=0; i < this.projTransLangs.length; i++){
			var option = document.createElement("option");
		    option.value = this.projTransLangs[i].langCode;
		    option.text = this.projTransLangs[i].langDesc;
		    translationGroup.append(option);
		    
		    if(null != permission && permission.lang === this.projTransLangs[i].langCode){
				option.setAttribute('selected', 'selected');
			}
		}
				
		return langSelect;
	};
	
	/**
	 * If permissions.role is a smart name, this method will resolve all the smart names
	 * 
	 * @param permissions
	 * @returns {Array}
	 */
	TextContentPermissions.prototype.getResolvedSmartRoles = function(permType, permissions) {
		var resolvedRoles = [];
		var avoidLangInSmartName = [];
		for (var langCount = 0; langCount < permissions.length; langCount++) {
			if(permissions[langCount].lang) {
				avoidLangInSmartName[permissions[langCount].lang] = true;  // Creating an array which should not be used for smart name resolving.
			}
		}
		
		for ( var permissionCount = 0; permissionCount < permissions.length; permissionCount++) {
			
			if (permissions[permissionCount].lang && permissions[permissionCount].lang.indexOf("[LANGUAGE]") >= 0) {
				//Check for total languages selected in Project Attribute Category. Resolving smartname makes sense only when user has selected languages
				//either as master or translation. If there are no languages selected, show [ALL LANGUAGES] in the language column.
				
				if(this.totalLangSelected > 0){
					for (var i=0; i < this.projMasterLangs.length; i++){
						//if user has selected particular language and assigned a role, we dont apply smartname for that language
						if(!avoidLangInSmartName[this.projMasterLangs[i].langCode]) {
							var permission = {};
							permission.role = permissions[permissionCount].role;
							
							if(permType === webi18n.t("PCM_EDITOR")){
								if(permission.role.length == 0){
									permission.role = (this.langEditorRolePrefix+" "+this.projMasterLangs[i].langCode).toUpperCase().trim();
								}	
							}
							else if(permType === webi18n.t("PCM_APPROVER")){
								if(permission.role.length == 0){
									permission.role = (this.langApproverRolePrefix+" "+this.projMasterLangs[i].langCode).toUpperCase().trim();
								}
							}							
							
							permission.lang = this.projMasterLangs[i].langCode;
							resolvedRoles.push(permission);	
						}
					}
					
					for (var i=0; i < this.projTransLangs.length; i++){
						if(!avoidLangInSmartName[this.projTransLangs[i].langCode]) {
							var permission = {};
							permission.role = permissions[permissionCount].role;
							
							if(permType === webi18n.t("PCM_EDITOR")){
								if(permission.role.length == 0){
									permission.role = (this.langEditorRolePrefix+" "+this.projTransLangs[i].langCode).toUpperCase().trim();
								}	
							}
							else if(permType === webi18n.t("PCM_APPROVER")){
								if(permission.role.length == 0){
									permission.role = (this.langApproverRolePrefix+" "+this.projTransLangs[i].langCode).toUpperCase().trim();
								}
							}							
							
							permission.lang = this.projTransLangs[i].langCode;
							resolvedRoles.push(permission);
						}
					}
				}else{
					var permission = {};
					permission.role = permissions[permissionCount].role;
					permission.lang = webi18n.t("PCM_SELECT_ALL");
					resolvedRoles.push(permission);
				}
			} else if(permissions[permissionCount].lang){
				var permission = {};
				permission.role = permissions[permissionCount].role.toUpperCase();
				permission.lang = permissions[permissionCount].lang;
				resolvedRoles.push(permission);
			} else {
				var permission = {};
				permission.role = permissions[permissionCount].role.toUpperCase();
				permission.lang = '';
				resolvedRoles.push(permission);
			}
			
		}
		return resolvedRoles;
	};

	/**
	 * when user clicks on "Change" button, all the rows in the table are read and Content Statement object is prepared for saving 
	 * 
	 * @param documentVersionId
	 */
	TextContentPermissions.prototype.addUnsavedPermissions = function(documentVersionId){
		var errorState = false;
		var noPermissionset = true; //All permissions rows are deleted
		var permissionsArray = [];
		
		$("tr.editrow").each(function() {
			var tcp =  $(this).find('td:nth-child(1) select option:selected').val();
			var lang = $(this).find('td:nth-child(2) select option:selected').val();
			var role = $(this).find('td:nth-child(3)  select option:selected').text();
			
			if(tcp === 'CONTENT MANAGER' && role === ''){
				this._alertpopup = new AlertPopup("errorpopup",webi18n.t("ERROR", this._languageBundleName),webi18n.t("PCM_PERMISSION_SAVE_ERROR3", this._languageBundleName));
				this._alertpopup.open();
				errorState = true;
				return false;
			}
			
			//Validation to check if user has selected language in the 2nd column, it cannot be blank
			if((tcp === 'EDITOR' || tcp === 'APPROVER') && (lang === 'blank')){
				this._alertpopup = new AlertPopup("errorpopup",webi18n.t("ERROR", this._languageBundleName),webi18n.t("PCM_PERMISSION_SAVE_ERROR", this._languageBundleName));
				this._alertpopup.open();
				errorState = true;
				return false;
			}
			
			if((tcp === 'EDITOR' || tcp === 'APPROVER') && (lang !== 'ALL LANGUAGES' && role === '')){
				this._alertpopup = new AlertPopup("errorpopup",webi18n.t("ERROR", this._languageBundleName),webi18n.t("PCM_PERMISSION_SAVE_ERROR2", this._languageBundleName));
				this._alertpopup.open();
				errorState = true;
				return false;
			}
			
			if(tcp !== "SELECT"){ //From TCP drop down if user has selected default 'Select' then ignore
				var isExist = false;
				for(var i=0; i<permissionsArray.length; i++) {
					if(permissionsArray[i].role === role && permissionsArray[i].tcp === tcp && permissionsArray[i].lang === lang) {
						isExist = true;
						break;
					}
				}
				
				if(!isExist) {
					var csPermission = new ContentStatementPermissions();
					csPermission.initContentStatementPermission(role, tcp, lang);
					permissionsArray.push(csPermission);
					noPermissionset = false; //There is atleast one permissions in the row
				}
				
			}
		});
		
		if(errorState)
			return false;
		
		//Incase there are no permissions selected, send empty permissions object 
		//"permission":[{"role":null,"tcp":null,"lang":null}] --> This way, in backend all the permissions will be deleted
		if(noPermissionset){
			var csPermission = new ContentStatementPermissions();
			permissionsArray.push(csPermission);
		}
		
		this._contentStatement.isPermissionsChanged = true;
		this._contentStatement.permission = permissionsArray;
		
		return true;
	};
	
	/**
	 * This will hide all the read only mode elements when user is in "Edit" mode
	 */
	TextContentPermissions.prototype.showEditMode = function(){
		$('#editbuttonrow').hide();
		$('#savecancelbuttonrow').show();
		$('#readonlypermissions').hide();
		$('#editpermissionsrow').show();
		$('#blankrow').show();
		$('#commentrow').show();
		$('#nopermissionscommentrow').hide();
	};
	
	/**
	 * This method is used show or hide Save button based on flag 
	 * 
	 * @param boolean - flag 
	 */
	
	TextContentPermissions.prototype.showOrHideSaveButton = function(flag){
		
		if (flag) {
			$(this._topWarningContainer).find('td.SaveCell button.SaveButton').show();
			$('span', $(this._topWarningContainer).find('td.WarningLabel')).show();
		}else{
			$(this._topWarningContainer).find('td.SaveCell button.SaveButton').hide();
			$('span', $(this._topWarningContainer).find('td.WarningLabel')).hide();
		}
	};
	
	
	
	/**
	 * Display a dismissable warning message.
	 * 
	 * @param message
	 */
	TextContentPermissions.prototype.displaySaveWarning = function(message, isDismissable) {
		
		var warning = $("<div>", {'class':'alert alert-warning', 'role' : 'alert'});
		if (isDismissable) {
			var warningDismiss = $("<button>", {'type':'button', 'class':'close', 'data-dismiss' : 'alert'});
			warningDismiss.append($("<span>").html("&times;"));
			warning.append(warningDismiss);
			warning.addClass("alert-dismissible");
		}
		var warningText = $("<span>").text(message);
		
		var saveButton = $("<button>", {'class':'btn btn-primary btn-sm pull-right SaveButton'}).text(webi18n.t("SAVE"));
		var saveCell = $("<td>",{'class' : 'SaveCell'}).append(saveButton);
		
		
		var cancelButton = $("<button>", {'class':'btn btn-default btn-sm pull-right CancelButton'}).text(webi18n.t("CANCEL"));
		var cancelCell = $("<td>",{'class' : 'CancelCell'}).append(cancelButton);
		
		var space = document.createElement("span");
		space.innerHTML = "&nbsp;&nbsp;";
		
		var warningInnerRow = $("<tr>").append($("<td>").append(warningText).addClass('WarningLabel').css({'width':'100%'}), saveCell,space,cancelCell);
		var warningInnerTable = $("<table>", {'class' : 'WarningTable'}).append(warningInnerRow);
		warning.append(warningInnerTable);
		
		
		$(".SaveButton", warning).on("click", function () {
			var addStatus = this.addUnsavedPermissions(this.documentVersionId);
			
			//If addStatus is false, it means there was an error state. So dont execute save operation
			if(addStatus){
				contentStatementService.saveContentStatements([this._contentStatement], function (updatedContentStatements, failedContentStatements, errorMsgs) {
					var updatedContentStatementIds = Object.keys(updatedContentStatements);
					var errorMsgIds = Object.keys(errorMsgs);
					if (updatedContentStatementIds.length > 0) {
						// Get the updated content statement.
						var updatedContentStatement = updatedContentStatements[updatedContentStatementIds[0]];
						// Generate the new query string and redirect to it.
						var query = jQuery.param({'menu_file': this.menuFile, 'docVerID': updatedContentStatement.docVersionId});
						window.location.replace("./doctypetextpermissions.jsp?"+query);
					} else if (errorMsgIds.length > 0) {
						window.alert(errorMsgs[errorMsgIds[0]]);
					}
				}.bind(this));
			}
		}.bind(this));
		
		$(".CancelButton", warning).on("click", function () {
			window.location.reload(true);
		}.bind(this));
		
		this._topWarningContainer.empty();
		this._topWarningContainer.append(warning);
		
	};
	
	
	/**
	 * Registering event listeners
	 */
	TextContentPermissions.prototype.addEvents = function(){
		$(document).on("click",".addButton", function(event,ui) {
			this.addEditPermissionsTableRow(null);
		}.bind(this));
		
		//This method doesnt have a bind, since we just want to remove the entire row from the table
		var self = this;
		$(document).on("click",".removeButton",function(event, ui){
			self.showOrHideSaveButton(true);
			$(this).parent().parent().remove();
			
		});

		$(document).on("change",".permissionSelect",function(event, ui){
			if($(event.target).val() == 'CONTENT MANAGER'){
				$(event.target).parent().siblings().find('.langSelect').hide();
			}
			else{
				$(event.target).parent().siblings().find('.langSelect').show();
			}
			this.showOrHideSaveButton(true);
		}.bind(this));
		
		$(document).on("change",".langSelect",function(event, ui){
			this.showOrHideSaveButton(true);
		}.bind(this));
		
		$(document).on("change",".rolesSelect",function(event, ui){
			this.showOrHideSaveButton(true);
		}.bind(this));
		
		
		$(document).on("click", "#editpermissions", function(event, ui){
			this.showEditMode();
		}.bind(this));
		
	};
	
	return TextContentPermissions;
});

