
/**
 * <p>The cart badge... This extends the normal IconBadge because we want to automatically update when ajax calls changed the content of the cart.</p>
 * 
 * <p>Attach this to the element surrounding the icon/thing...</p>
 * 
 * @author JEPE
 */
define('pages/cartbadge/CartBadge',["component/badge/IconBadge"], function(IconBadge) {
	/**
	 * Constructor
	 */
	var CartBadge = function(div){
		// store the element
		this.$div = $(div);
		
		//
		// Cart Badge => Outsource to re-usable component
		//
		
		// make the actual badge (which shows it)
		this.badge = new IconBadge(div);
		
		
		// 
		// Cart Tooltip
		// 
		// read initial data
		this.tooltipstr = this.$div.attr("data-cartbadgetooltip");
		this.documentcount = this.$div.attr("data-cartdocuments");
		this.projectcount = this.$div.attr("data-cartprojects");
		this.clearsetting = this.$div.attr("data-langstrclearsetting");
			
		// set the tooltip
		this.setTooltip();
		
		//
		// Update cart badge on events...
		//
		
		// listen to the cartupdated event (in case we need to update the numbers...)
		// this event is thrown if the page updated the cart using ajax but did not refresh the page.
		$(document.body).on("cartupdated", this.updateByAjax.bind(this));
	};
	
	/**
	 * When the cart changes without the page being refreshed, 
	 * get the new number of items in the cart using ajax and update.
	 */
	CartBadge.prototype.updateByAjax = function() {
		$.ajax("docartactions.jsp?actionVal=CartCountUpdate", {
			context:this,
			cache:false,
			dataType:"xml",
			success:function(data, textStatus, jqXHR){
				var $data = $("cartcontentcountinfo", data);
				this.projectcount = $data.attr("projects");
				this.documentcount = $data.attr("documents");
				this.badge.setCount($data.attr("total"));
				this.setTooltip();
			},
			error:function(jqXHR, textStatus, errorThrown){
				// very rare occasion...
				this.badge.setCount("?");
				this.projectcount = "?";
				this.documentcount = "?";
				this.setTooltip();
			}
		});
	};
	
	/**
	 * Set the tooltip 
	 */
	CartBadge.prototype.setTooltip = function() {
		var tooltip = this.tooltipstr.replace("{0}", this.documentcount).replace("{1}", this.projectcount);
		this.$div.prop("title", tooltip+"\n("+this.clearsetting+")");
	};
	
	/**
	 * Initialize as a component
	 */
	CartBadge.wcrinit = function(div) {
		new CartBadge(div);
	};
	
	return CartBadge;
});
