define(["jquery", "underscore","utils/webi18n!Migration","wcr/webcenter_xml" ], function($, _ , i18n) {
"use strict";        


	var ImportItems = function(div) {
		this.div = $(div);
		
		// import button
		$(".submitbutton", this.div)
			.click(_.bind(this.importItems, this));
		
		// return button
		$(".returnbutton", this.div)
			.click(_.bind(this.returnToSourcePage, this));
		
		// add confirmation message
		$(".importdocs", this.div)
		.click(_.bind(this.importDocsClicked, this));
		
		// make processing queue
		this.queue = [];
		
		// make counter for the document upload 
		this.docCounter = 0 ;
		this.projectIndex = 0 ;
		this.docUploadSuccess = 0 ;
		this.docUploadFailed = 0 ;
		this.totalDocCounter = 0 ;		
		
	};
	
	/**
	 * Populate the queue with items to be processed
	 * */
	ImportItems.prototype.addToQueue = function(importType, importIndex) {
		this.queue.push({
			importType:importType,
			importIndex:importIndex
		});
	};

	/** 
	* When disabling the checkbox to import documents, show a warning message  
	* */
	ImportItems.prototype.importDocsClicked = function(){
		
		if (!$(".importdocs")[0].checked){
			return confirm( i18n.t("NODOCSCONF"));			
		}
	};
	
	
	/** 
	* Return to source page
	* */
	ImportItems.prototype.returnToSourcePage = function(){
		
		var returnPage  = $(".returnbutton").attr('returnpage');
		window.open(returnPage, '_top');
		return true;
	};
	
	/**
	 * import items  
	 * @param elementsToBeImported
	 * @param callback
	 */
	ImportItems.prototype.importTypes = function(elementsToBeImported, callback){
		if (elementsToBeImported == undefined)
			return ;
		
		$.each(elementsToBeImported, _.bind(function(i, checkbox) {
			if (checkbox.checked){        	
				this.addToQueue(checkbox.getAttribute('importtype'), checkbox.attributes["index"].value);
        	} 
        	else{
        		checkbox.disabled='disabled';
        	}
		}, this));
	};
	
	/**
	 * Do some preprocessing work: add items to the queue in correct order, then process the items
	 * 
	 */
	ImportItems.prototype.importItems = function()
    {
		$(".submitbutton").hide();
		$(".returnbutton").hide();   
		$(".importdocs").prop("disabled", true);
		 
		// import the groups and roles, categories, task types and projects in this order 
		this.importTypes($(".tobeimported[importtype='RestrictedSet']") );
		this.importTypes($(".tobeimported[importtype='Group']") );
		this.importTypes($(".tobeimported[importtype='Category']") );
		this.importTypes($(".tobeimported[importtype='TaskType']") );
		this.importTypes($(".tobeimported[importtype='SavedSearch']") );
		this.importTypes($(".tobeimported[importtype='Project']") );
		
		this.processQueue();
    };
    
    /**
     * This method will call the import item jsp call for each doucment to be imported.
     * Reason for doing so is to be able to give progress bar on the import of projects
     */
    ImportItems.prototype.importProjectDocs = function (projectIndex, nrOfDocuments){
    	
    	// initialize the counters 
    	this.docCounter 		= nrOfDocuments ; // will iterate over the documents
    	this.totalDocCounter 	= nrOfDocuments ; // total nr of docs in this project
    	this.projectIndex 		= projectIndex;   // Project index item
    	this.docUploadFailed 	= 0 ;			  // nr of documents that failed	
		this.docUploadSuccess   = 0 ;			  // nr of documents that uploaded successfull	
		
    	// show the import document feedback items 
    	$("." + projectIndex + "_PB").removeClass("hiddenElement");
    
    	// process this document
    	this.processDocument();    	
    };
    
    
    /**
     * This method will process the next document.
     * When there are documents to be processed, the next document is imported
     */
    ImportItems.prototype.processDocument = function(){
    	
    	// de-activate the previous progress bar
		if (this.projectIndex > 0){
			$("."+ (this.projectIndex-1) +"_PB_SUCCESS").removeClass("active");
			$("."+ (this.projectIndex-1) +"_PB_FAILED").removeClass("active");
		}
    	
		if(this.docCounter >0) {
			// decrease counter and process next document
			this.docCounter--;
			this.importDocument();
			
		} else {
			// process next element 
			this.processQueue();
		}
		return ;
	};
	
	
	/**
	 * This method will process the XML returned when uploading a document.
	 * 
	 * At this moment, we know that there is at least one error code in the XML  
	 */
	ImportItems.prototype.processUploadDocError = function(xml){
		
	   // Update the progress bar:
	   // increase the nr of failed documents and update the ratio of failed documents 
 	   this.docUploadFailed++ ; 
 	   var newRatio = (this.docUploadFailed / this.totalDocCounter) * 100 ;
 	   var newWidth = parseInt(newRatio)+ "%"; 
	   $("."+ this.projectIndex +"_PB_FAILED").css("width",  newWidth).text(i18n.t("NRDOCSFAILED",this.docUploadFailed));
	   
	   // show the tip on failed uploading of doucments 
	   $(".PB_FAILED_TIP").removeClass("hiddenElement");
	
	    // Get the name of the document and add it to the GUI/Log 
	   var docNameElem =  xml.getElementsByTagName("name");
	   if (docNameElem!= undefined && docNameElem.length > 0 ){
		   var docName= WCXML.GetElementText(xml,"name", true);
		   
		   // add the failed document to the list of documents, only when the nr of failed docs > max_nr_failed_GUI
		   $("."+ this.projectIndex +"_PB_FAILED_DOCLIST").removeClass("hiddenElement");
		   $("."+ this.projectIndex +"_PB_FAILED_DOCLIST .overflowcontainer").append($("<div>").text(docName));   
		   
		   // always log the document that failed to upload 
		   log("Failed to upload document " + docName  );		   
	   }
	      
	   // take out the minumim width for the success  
	   $("."+ this.projectIndex +"_PB_SUCCESS").css("min-width",  "");
	   
	   // Update the project status icon -> set it to warning
	   var warningIcon = "icon-orange icon-WCR_Msg-Warning";
	   $("#"+this.projectIndex + "_Project .icon").prop("class",warningIcon).prop("title",i18n.t("WARNING"));
	   
	};

	/**
	 * This method is called when uploading a document was successful.
	 * 
	 *  It will update the progress bar with the nr of successfull uploaded documents 
	 */
	ImportItems.prototype.processUploadDocSuccess = function(xml){
		// update the progress bar
		this.docUploadSuccess++ ; 
		var newRatio = (this.docUploadSuccess / this.totalDocCounter) * 100 ;
		var newWidth = parseInt(newRatio)+ "%"; 
		$("."+ this.projectIndex +"_PB_SUCCESS").css("width",  newWidth).text(i18n.t("NRDOCSSUCCESS", this.docUploadSuccess));
		
		// check whether there are warnings, if so, add the warning message to the feedback
		var warningNodes=  xml.getElementsByTagName("warning");
		if (warningNodes != undefined && warningNodes.length > 0 ){
			this.processReturnXML(xml,this.projectIndex,'Project');
		}
		
	};

	/**
	 * Make an ajax call for importing the documents 
	 */
	ImportItems.prototype.importDocument = function(){
		
		// Put the document index parameter
    	var inclDocs = "&includedocs=1&docindex="+this.docCounter; 
    	
    	// make an ajax call to the SDK to import this document
    	var importItemURL = 'importItem.jsp?importType=Project&importindex=' +this.projectIndex + inclDocs; 
		log('Importing Document - URL : ' +importItemURL);
		
		
		try {
			$.ajax({
	    		url: importItemURL,
				type: "GET",
		        data: "xml",
	    		success: _.bind(function(xml) 
	    		{
	    			// check whether an error occured 
	    			var errorElem =  xml.getElementsByTagName("error");
	    			if (errorElem != undefined && errorElem.length > 0 ){
	    				this.processUploadDocError(xml);
	    			}
	    			else{
	    				this.processUploadDocSuccess(xml);
	    			}
	    			
	    			// upload next Document
					this.processDocument();
										
	    		}, this),
		       error: function(status,objHdl, description) 
				{
		    	   this.processUploadDocError(xml);
		    	   objHdl.processDocument();
					
		    	 },
				objHdl: this     
	      		});
		} catch (err) {	
			alert("Import Item caused an error. Details:" + err);
		};		
    

	};
	/**
	 * Take items from the queue, one by one, and make a call to the import method 
	 */
	ImportItems.prototype.processQueue = function(){
		
		if(this.queue.length>0) {
			var queueItem = this.queue[0];
			this.queue.shift();
			this.importItem(queueItem.importType, queueItem.importIndex);
			
		} else {
			
			$(".progress-bar").removeClass("active");
			$(".upload_doc_string").addClass("hiddenElement");
			$(".finishedMessage").removeClass("hiddenElement");
			$(".progress-bar").removeClass("progress-bar-striped");
						
			$(".returnbutton").show();
			$(".returnbutton").addClass("btn-primary");	
		}
		return ;
	};
    
	/**
	 *  Import an item.
	 *  Build an URL and make an ajax call to import the item to the server
	 *   
	 */
	ImportItems.prototype.importItem = function(importType, importIndex){
    
    	// check the import document checkbox
    	var inclDocs = "";
    	inclDocs = "&includedocs=0"; 
    	
	    // make an ajax call to the SDK to import this document
    	var importItemURL = 'importItem.jsp?importType=' + importType + '&importindex=' +importIndex + inclDocs; 
		log('Importing Item - URL: ' +importItemURL);

		var checkBoxSpanID= "#"+importIndex + "_" + importType + "_ID";
		$(checkBoxSpanID).addClass("hiddenElement");			
		
		
		try {
			$.ajax({
	    		url: importItemURL,
				type: "GET",
		        data: "xml",
	    		success: _.bind(function(xml) 
	    		{
	    			// check whether an error and/or warning occurred
	    			this.processReturnXML(xml,importIndex, importType);
	    			
					// if we are uploading a project, upload the documents one by one
	    			var doImportDocs = $("#importdocs") != undefined && $("#importdocs").length>0 && $("#importdocs")[0].checked ;
					if (importType == 'Project' && doImportDocs){
						// get the number of documents of the project
						var nrDocs = $("#" + importIndex+ "_Project_CB").attr("nr_docs");
						this.importProjectDocs(importIndex,nrDocs);
					}else{
						// go to the next element
						this.processQueue();						
					}					

	    		}, this),
		       error: function(status,objHdl, description) 
				{
		    	    objHdl.processReturnXML(xml,importIndex, importType);
		    	   
					// go to the next element
					objHdl.processQueue();
		    	 },
				objHdl: this     
	      		});
		} catch (err) {	
			alert("Import Item caused an error. Details:" + err);
		};		
    
    };
    
    
    /**
     * Display the error message.
     * Run over the list of nodes, take the text out of the node and display it 
     */
    ImportItems.prototype.displayMessage = function (className , nodeList){
    
    	// for each node, get the text and add it to the feedback part
    	for(var node = 0; node < nodeList.length ; node++) {
			var messageNode = nodeList[node];
			var message=WCXML.GetElementSelfTextContent(messageNode);
			$(className +" .overflowcontainer").append($("<div>").text(message)); 
			
			// when the warning is about missing items in the task type, show a tip that they might want to rerun the import tool
			var wfIncompletePrefix = "Workflow incomplete";
			var indexOfPrefix = message.indexOf(wfIncompletePrefix);
			if (indexOfPrefix > -1 ){
				$(".taskTypeHasMissingItems").removeClass("hiddenElement");
			}
		}
    };
    
    
    /**
     * Process the return XML , show the error, warning, success icon dependint on the XML 
     */
    ImportItems.prototype.processReturnXML = function(xml,importIndex, importType){
    	
    	// check whether there are errors and/or warnings
    	var errorElem =  xml.getElementsByTagName("error");
		var warningElem =  xml.getElementsByTagName("warning");
		var titleText = i18n.t("SUCCESS");
		
		// get the warning messages and show it in the GUI
		var feedback_warning_class =  "."+ importIndex + "_" +importType+ "_Feedback_Warning";
		// get the error messages and show it in the GUI
		var feedback_error_class =  "."+ importIndex + "_" +importType+ "_Feedback_Error";
		
		// when an error ocuured, show the error icon
		var iconClass = "icon-green icon-normal icon-WCR_Checked" ; 
		if (errorElem != undefined && errorElem.length > 0 ){
			iconClass = "icon-red icon-danger icon-WCR_Msg-Error";
			
			// get the text and set it to the feedback
			$(feedback_error_class).removeClass("hiddenElement");
			
			// display the error message to the correct control
			this.displayMessage(feedback_error_class, errorElem);
			
			//in case there were warnings encountered along with errors, also show those
			if (warningElem!= undefined && warningElem.length > 0) {
				$(feedback_warning_class).removeClass("hiddenElement");
				this.displayMessage(feedback_warning_class, warningElem);
			}
			
			titleText = i18n.t("ERROR");
		}
		// check whether there are warnings, if so, show the warning icon
		else if (warningElem!= undefined && warningElem.length > 0){
			iconClass = "icon-orange  icon-WCR_Msg-Warning";
			
			// get the text and set it to the feedback
			$(feedback_warning_class).removeClass("hiddenElement");
			this.displayMessage(feedback_warning_class, warningElem);
			
			titleText = i18n.t("WARNING");	
		}
		
		// we have set the correct class, now show the icon
		var feedbackId= "#"+importIndex + "_" + importType ;
    	var feedbackIcon =  feedbackId + " .icon";
    	
    	$(feedbackId).removeClass("hiddenElement");
    	$(feedbackIcon).addClass(iconClass).prop("title",titleText );    	
    		        		
	};
        
    
    /**
	 * Log function
	 */
	function log(msg)  {
        if ( (typeof console != 'undefined') && console.log){
            console.log(msg);
        }
    };
    
	/**
	 * Init method 
	 * 
	 * @param div
	 */
    ImportItems.wcrinit = function(div) {
    	new ImportItems(div);
    };
        
    return ImportItems;
});