
/**
 * This is the javascript code needed to make the rows in the 
 *    Project Creation => Create From Project => Popup => Copy Tasks Rows 
 * work...
 * 
 * Used by CopyProjectOptionsTable
 * 
 * @author JEPE
 */
define('pages/projectcreation/CopyProjectOptionsTableTaskCopyRows',["jquery"], function ($) {
	/**
	 * Constructor of the component managing the task copy rows
	 */
	function CopyProjectOptionsTableTaskCopyRows(rows){
		// Save the rows with the components we control...
		this.$trA = rows.filter(".taskCopyRow.rowA");
		this.$trB = rows.filter(".taskCopyRow.rowB");
		this.$trBerr = rows.filter(".taskCopyRow.rowError");
		
		// Find all the interacting components
		// Because I do not use id's this component could in theory appear multiple times on the page and still work.
		// Should probably be more selective in the jquery selectors (use classes)
		this.$checkbox = $("input[type='checkbox']",this.$trA);
		this.$copytype = $("select",this.$trA);
		this.$templateselector = $("select",this.$trB);
		this.$sdroploadericon = $("img", this.$trB);
		
		// Initialize
		this._initialize();
	}
	
	/**
	 * (public) 
	 * Set the id of the project to copy from
	 * @param projectId the id of the project to copy from
	 */
	CopyProjectOptionsTableTaskCopyRows.prototype.setProjectId = function(projectId) {
		this._resetToDefault();
		if(this._templateSDrop) {
			// destroy the sdrop if it exist when selecting a different projectId
			this._templateSDrop.destroy();
			this._templateSDrop = null;
			// clear the actual select also
			this.$templateselector.empty();
		}
		this._currentProjectId = projectId;
		// load the template id to preselect in the SDrop
		this._loadTemplateSelectorDefaultValue((function(result){
			this._initSDropSelector(result);
		}).bind(this));
	};
	
	/**
	 * (public)
	 * Collect the data from the rows to submit to the server...
	 * 
	 * @param data an object in which the parameters will be added.
	 * @returns {Boolean} true if the validation was successful.
	 */
	CopyProjectOptionsTableTaskCopyRows.prototype.verifyAndCollectData = function (data) {
		var validation = true;
		if(this.$checkbox.prop("checked")) {
			// whether we are a project or a template
			data[this.$copytype.attr("name")] = this.$copytype.val();
			
			// which template in case of a template
			if(this.$copytype.val()==="template") {
				var selectedTemplateId = this.$templateselector.val();
				if(selectedTemplateId) {
					data[this.$templateselector.prop("name")] = this.$templateselector.val();
					this._showerror(false);
				}else {
					this._showerror(true);
					validation = false;
				}
			}
		}
		return validation;
	};
	
	/**
	 * (public) 
	 * Restore the data for the rows to a previous state...
	 * 
	 * @param data an object in which the parameters that need to be restored can be found
	 */
	CopyProjectOptionsTableTaskCopyRows.prototype.restoreData = function (data) {
		if(this.$checkbox.prop("checked")) {
			// whether we are a project or a template
			this.$copytype.val(data[this.$copytype.attr("name")]);
			
			// which template in case of a template
			if(this.$copytype.val()==="template") {
				this.$templateselector.val(data[this.$templateselector.prop("name")]);
			}
		}
		this._showerror(false);

		// update (some change events might not be thrown...)
		this._onChangeEnabled();
		this._onChangeSource();
	};
	
	/**
	 * Show the error message
	 */
	CopyProjectOptionsTableTaskCopyRows.prototype._showerror = function(show) {
		this.$trBerr.toggle(show);
		var sdrop = $(".sdrop", this.$trB);
		sdrop.css("border", "1px solid "+(show?"#F00":"#000"));
	};
	
	/**
	 * Initialize the functionality of the rows
	 */
	CopyProjectOptionsTableTaskCopyRows.prototype._initialize = function() {
		// register listener - update disable look when changing checkbox
		this.$checkbox.change(this._onChangeEnabled.bind(this));
		
		// register listener - show/hide
		this.$copytype.change(this._onChangeSource.bind(this));
		
		// set values to defaults
		this._resetToDefault();
	};
	
	/**
	 * Reset all values to it's defaults...
	 */
	CopyProjectOptionsTableTaskCopyRows.prototype._resetToDefault = function() {
		this.$copytype.val("project");
		this._onChangeEnabled();
		this._onChangeSource();
		this._showerror(false);
	};

	/**
	 * When the copyOptionTasks checkbox changes
	 * 
	 * Change the fields to disabled...
	 */
	CopyProjectOptionsTableTaskCopyRows.prototype._onChangeEnabled = function(){
		var checked = this.$checkbox.prop("checked");
		this.$copytype.prop("disabled", !checked);
		this.$templateselector.prop("disabled", !checked);
	};
	
	/**
	 * When the copyOptionTasksSource dropdown changes
	 * 
	 * Show the second row and redraw the SDrop
	 */
	CopyProjectOptionsTableTaskCopyRows.prototype._onChangeSource = function(){
		var isTemplate = this.$copytype.val()==='template';
		this.$trB.toggle(isTemplate);
		if(isTemplate) {
			if(this._templateSDrop) {
				// Redraw
				// Correct the width, zindex, ...
				this._templateSDrop.redraw();
			}
		}
	};
	
	/**
	 * Initializes the SDrop you see when you select Copy Tasks from template in the popup you see when you select "Copy from existing project"
	 * 
	 * @param setInitialValue the callback which is called with the initial value, or with null if something went wrong or there is no initial value (function(String):void)
	 */
	CopyProjectOptionsTableTaskCopyRows.prototype._loadTemplateSelectorDefaultValue = function(setInitialValue){
		// hide the original select
		this.$templateselector.hide();
		
		// do ajax call to get the initial value for the SDrop
		if(!this._currentProjectId) {
			// oops, no projectId...
			this.$templateselector.show();
			setInitialValue(null);
		}else{
			var projectID = this._currentProjectId;
			// do the ajax call
			$.ajax("GetProjectCreationInfo.jsp?"+$.param({action:"CopyProjectFindSourceTemplateId", projectID:projectID}),
				{
					context:this,
					dataType:"xml",
					success:function(data, textStatus, jqXHR){
						// Get the originalSourceTemplate from the xml...
						var templateId = $("originalSourceTemplate", data).text();
						// Do something
						if(templateId){
							this.$templateselector.show();
							setInitialValue(templateId);
						}else{
							this.$templateselector.show();
							setInitialValue(null);
						}
					},
					error:function(jqXHR, textStatus, errorThrown){
						this.$templateselector.show();
						setInitialValue(null);
					}
				});
		}
	};
	
	/**
	 * Initialize the SDrop (after we know the initial value)
	 */
	CopyProjectOptionsTableTaskCopyRows.prototype._initSDropSelector = function(initialValue){
		if(!this.templateSDrop) {
			// Get the width we want the SDrop to be...
			var width = this.$templateselector.attr("data-sdropselectorwidth");
			
			// Z-index, need to show above popup
			var zindex = 10000;
			
			// The options used when creating the SDrop
			var sdropOptions = {
					'autoload': true,
					'zIndex': zindex,
					'minWidth': width,
					'maxWidth': width,
					'initializeDataProvider': false};
			
			// If we have an initial value, add it to the options
			if(initialValue!==null) {
				sdropOptions["initialValue"]=initialValue;
			}
			
			// Then, make the SDrop...
			this._templateSDrop = new SDrop(
				this.$templateselector, 
				this.$sdroploadericon,
				new SDropDataProvider({'dataSource':"GetNameIDList.jsp?list=Templates"}),
				sdropOptions
				);
		}
	};
    
    return CopyProjectOptionsTableTaskCopyRows;
});

/**
 * This is the javascript code needed to make the fields in the table in the 
 *    Project Creation => Create From Project => Popup => Table 
 * work...
 * 
 * Why do I do it this way?
 *  - This component can contain multiple times on a page (I don't use id's)
 *  - I don't create any globals... (not polluting the global namespace)
 *    (I have a lot of internal functions)
 *  - Not possible to forget including the javascript as the javascript is attached to the html using utils/init.
 *    (The page does not need to know if the table is used.) 
 *    (The javascript is loaded automatically.)
 * 
 * @author JEPE
 */
define('pages/projectcreation/CopyProjectOptionsTable',["jquery", "underscore", "./CopyProjectOptionsTableTaskCopyRows"], function ($, _, CopyProjectOptionsTableTaskCopyRows) {
	/**
	 * Constructor of the component managing the project copy settings table
	 */
	function CopyProjectOptionsTable(table){
		// the <table> in which all the checkboxes and other html is located
		this.$table = $(table);
		
		// store this object
		this.$table.get(0).CopyProjectOptionsTable = this;
		
		// Find the task copy rows... (those rows have more logic)
		var $taskCopyRows = $(".taskCopyRow", this.$table);
		
		// Make a separate controller for those rows
		// But first check if those rows exist (could be that we don't have task license...)
		if($taskCopyRows.length>0) {
			this._taksrowController = new CopyProjectOptionsTableTaskCopyRows($taskCopyRows);
		} else {
			this._taksrowController = null;
		}
		
		this._initializeDependencyListeners();
	}
	
	/**
	 * (public!)
	 * When a project id is selected, the table get's initialized correctly...
	 * @param projectId The projectId we are copying from
	 */
	CopyProjectOptionsTable.prototype.setProjectId = function(projectId) {
		// if we selected a different project, reset all values...
		if(!this._projectId || this._projectId!==projectId) {
			this._projectId=projectId;
			this._resetToDefaults();
			this.validateAndSave();
			if(this._taksrowController!==null){
				this._taksrowController.setProjectId(projectId);
			}
		}
	};
	
	/**
	 * (public!)
	 * Collects and returns the options selected in the dialog as a string name=value&name=value
	 * 
	 * !!! MAKE SURE validateAndSave was called before calling this method.
	 */
	CopyProjectOptionsTable.prototype.getSelectedOptions = function() {
		return $.param(this.lastValidatedValues);
	};
	
	/**
	 * (public!)
	 * If a template is chosen, send some default values...
	 */
	CopyProjectOptionsTable.prototype.getTemplateCopyOptions = function() {
		var data = {};
		
		// Send all values for a template
		$(":input[data-valuefortemplate]", this.$table).each(function() {
			data[$(this).attr("name")] = $(this).attr("data-valuefortemplate");
		});
		
		return $.param(data);
	};
	
	/**
	 * (public!)
	 * Validate the values the user entered. (Do this before closing the popup)
	 * After calling this method you can call "getSelectedOptions" to get the settings
	 * @returns true if the user entered correct values. False if validation (and saving failed)
	 */
	CopyProjectOptionsTable.prototype.validateAndSave = function() {
		var data = {};
		var validation = this._verifyAndCollectValues(data);
		if(validation) {
			this.lastValidatedValues = data;
		}
		return validation;
	};
	
	/**
	 * (public!)
	 * If the user cancels, go back to the last valid state.
	 */
	CopyProjectOptionsTable.prototype.cancelAndRestore = function(){
		if(this.lastValidatedValues) {
			this._restoreValues(this.lastValidatedValues);
		} else {
			this.resetToDefaults();
			this.validateAndSave();
		}
	};
	
	/**
	 * Create all listeners for checkboxes which depend on another checkbox
	 */
	CopyProjectOptionsTable.prototype._initializeDependencyListeners = function() {
		// Put all checkboxes in the form in a map of name->$obj
		var allCheckboxesByName = _.reduce($("input[type='checkbox']"), function(obj, item) {
			var $item = $(item);
			obj[$item.prop("name")] = $item;
			return obj;
		}, {});
		var self = this;
		
		// Loop through all checkboxes which depend on another checkbox and add the required listeners...
		$("input[type='checkbox'][data-copyoptiondependson]", this.$table).each(function() {
			var $checkbox = $(this);
			var nameOfDependency = $checkbox.attr("data-copyoptiondependson");
			var dependency = allCheckboxesByName[nameOfDependency];
			if(dependency && dependency instanceof jQuery) {
				var callback = self._checkboxWithDependingCheckboxClicked.bind(self, dependency, $checkbox);
				dependency.click(callback);// for IE
				dependency.change(callback);
			}
		});
	};
	
	/**
	 * A listener which is invoked when a checkbox is clicked which has a depending checkbox.
	 * 
	 * When the original checkbox is not checked, also uncheck the depending checkbox and disable it.
	 * 
	 * @param $clickedCheckbox the checkbox on which the other checkbox depends
	 * @param $dependingCheckbox the checkbox with the dependency
	 */
	CopyProjectOptionsTable.prototype._checkboxWithDependingCheckboxClicked = function($clickedCheckbox, $dependingCheckbox) {
		var checked = $clickedCheckbox.prop("checked");
		$dependingCheckbox.prop("disabled", !checked);
		if(!checked) {
			$dependingCheckbox.prop("checked", false);
		}
		// also update dependencies of this checkbox
		$($dependingCheckbox).triggerHandler("change");
	};
	
	/**
	 * Reset all checkboxes to their original state...
	 */
	CopyProjectOptionsTable.prototype._resetToDefaults = function() {
		$("input[type='checkbox']", this.$table).each(function() {
			$(this).prop("checked", $(this).prop("defaultChecked"));
		});
		
		// Update all disable/enable params...
		$(":input[type='checkbox']", this.$table).each(function() {
			$(this).triggerHandler("change");
		});
	};
	
	/**
	 * Collect all the values of the inputs...
	 * 
	 * @param {Object} data the object in which the values will be put
	 * @returns {Boolean} true if the form validates correctly
	 */
	CopyProjectOptionsTable.prototype._verifyAndCollectValues = function(data) {
		var validation = true;
		
		// Collect all checkboxes...
		$(":checked[type='checkbox']", this.$table).each(function() {
			data[$(this).attr("name")] = "on";
		});
		
		// Collect the information from the task copy rows
		if(this._taksrowController){
			validation = validation & this._taksrowController.verifyAndCollectData(data);
		}
		return validation;
	};
	
	/**
	 * Restore values...
	 * 
	 * @param data the values to restore
	 */
	CopyProjectOptionsTable.prototype._restoreValues = function(data) {
		// Set all checkboxes...
		$(":input[type='checkbox']", this.$table).each(function() {
			if(data[$(this).attr("name")]==="on"){
				$(this).prop("checked", true);
			}
		});
		
		// Update all disable/enable params...
		$(":input[type='checkbox']", this.$table).each(function() {
			$(this).triggerHandler("change");
		});
		
		// Set the information from the task copy rows
		if(this._taksrowController){
			this._taksrowController.restoreData(data);
		}
	};
	
	/**
	 * Implemented as a component...
	 * 
	 * Called when initializing the component
	 * 
	 * @param table the table containing the copy table content.
	 */
	CopyProjectOptionsTable.wcrinit = function (table){
		new CopyProjectOptionsTable(table);
	};
	
	/**
	 * Implemented as a component...
	 * 
	 * Called when destroying the component
	 * 
	 * @param table the table containing the copy table content.
	 */
	CopyProjectOptionsTable.wcrdestroy = function (table){};
    
    return CopyProjectOptionsTable;
});
