define(["attributecategory/AttributeCategory",
    "component/helpIcon/HelpIcon",
	"jquery",
	"utils/webi18n!ProjectCreation"
], function (AttributeCategory,
	HelpIcon,
	$,
	i18n) {

	"use strict";

	// Base url for attribute category
	var baseURL = "GetAttributeCategoryXML.jsp?categoryid=";

	/**
	 * display warning if project creation view is available in attribute category
	 */
	var checkForProjectCreationView = function (projectID, projectCategoryID, modTimeStamp, warningContext) {
		if(projectCategoryID){
			var readURL = baseURL + projectCategoryID;
			readURL += modTimeStamp ? "&lastmodified="+modTimeStamp :  "";
			var atCat;
			var attrObj = {
				ACatObj: "projectCreationWarningContext",
				readURL: readURL,
				projectID: projectID,
				callbackOnLoad: function () {
					checkAttributeCategoryForProjectCreationView(atCat, warningContext);
				}
			};

			atCat = new AttributeCategory(attrObj);
		}
	};

	/**
	 * callback fn when atcat is loaded
	 */
	var checkAttributeCategoryForProjectCreationView = function (atCat, warningContext) {
		var projectCreationView = atCat.getViewByName("ProjectCreation");
		if (projectCreationView) {
			var warning = createWarningMessage();
			warningContext.prepend(warning);
		}
	};

	/**
	 * creates the warning mesage
	 */
	var createWarningMessage = function () {
		var span = $("<span/>").text(i18n.t("warningProjectCreation")).prepend($("<i/>").addClass("icon icon-warning icon-WCR_Msg-Warning"));
		var help = HelpIcon.getInlineHelpTopicJQuery("/common/wc/topic/to_wc_projectcreationsettings");
		return $("<div class='alert alert-warning'/>").append(span).append(help);
	};

	return {
		checkForProjectCreationView: checkForProjectCreationView
	};

});
