/**
 * An SDrop in which you can select attributes (or specifications).
 * 
 * When you select an attribute, a certain form will be submitted with a list of previously selected id's + the new id.
 * 
 * That is all this component does...
 */
define(["jquery", "wcr/sdrop"], function($, sdropmod) {
	
	/**
	 * Constructor...
	 */
	var SearchAttributeSDropSelector = function($div, initialselectedids, form, formactioninput, formidinput, listname, automatedtestingid, filter) {
		// store all information
		this.$div = $div;
		this.selectedids = initialselectedids;
		this.form = form;
		this.formactioninput = formactioninput;
		this.formidinput = formidinput;
		this.listname = listname?listname:"Specifications";
		this.automatedtestingid = automatedtestingid;
		this.filter = filter;
		
		// make the SDrop
		this.makeSDrop();
	};
	
	/**
	 * Make the SDrop
	 */
	SearchAttributeSDropSelector.prototype.makeSDrop = function(){
		// First, make the select...
		this.sdropselect = $("<select>", {
			width: 500,
			id: this.automatedtestingid
		});
		this.$div.append(this.sdropselect);
		
		// Do we need to filter the list with a comma delimited string of attribute (specification) Ids ?
		var filterParm = '';
		if (this.filter !== undefined && this.filter !== null && this.filter.length > 0) {
			filterParm = '&filter='+this.filter;
		}
		
		// Then, create the SDrop		
		this.sdrop = new SDrop(
				this.sdropselect,
                $(".loadingplaceholder", this.$div),
                new SDropDataProvider({'dataSource': "GetNameIDList.jsp?list="+this.listname+filterParm}),
                { autoWidth:false }
		);
		
		// When the user selects something in the sdrop, call add()
		this.sdrop.addEvent("changedByUser", function(){
			this.add();
		}.bind(this));
	};
	
	/**
	 * Called when the user selected something in the sdrop
	 */
	SearchAttributeSDropSelector.prototype.add = function() {
		// What did he select?
		var id = this.sdrop.getSelectedItemID();
		
		// > WORKAROUND FOR BUG IN SDROP - REMOVE WHEN SDROP IS FIXED
		//   (workaround needed for automated testing)
		//   Remove when Jira issue WCR-9078 is fixed
		if(!id) { id=this.sdrop.dropbox.val(); }
		// < END WORKAROUND
		
		// Add it to the previously selected id's
		this.selectedids.push(id);
		
		// And... submit the form
		this.submit(this.selectedids.join(";"));
	};
	
	/**
	 * Submits the form with a certain value for "idlist".
	 * 
	 * @param idlist some string value
	 */
	SearchAttributeSDropSelector.prototype.submit = function(idlist) {
		// set the action
		this.formactioninput.val("AddOneCriterium");
		
		// set the id's
		this.formidinput.val(idlist);
		
		// submit...
		this.form.get(0).submit();
	};
	
	/**
	 * Read from the element and create a new instance.
	 * (Used by data-init... But you can also simply call the constructor)
	 * 
	 * @param div
	 */
	SearchAttributeSDropSelector.wcrinit = function(div){
		var $div = $(div);
		var selectedids = $div.attr("data-alreadyshownattributeids").split(";");
		var form = $($div.attr("data-formselector"));
		var formactioninput = $($div.attr("data-formactionselector"));
		var formidinput = $($div.attr("data-formidselector"));
		var listname = $div.attr("data-listname");
		var automatedtestingid = $div.attr("data-idforautomatedtesting");
		var filter = $div.attr("data-filter");
		
		new SearchAttributeSDropSelector($div, selectedids, form, formactioninput, formidinput, listname, automatedtestingid, filter);
	};
	
	// return the constructor (for manual creation of the component)
	return SearchAttributeSDropSelector;
});