
/**
 * TunnelMonitor View
 *
 * @author KADA
 * @since 9.2.2015
 */
define('pages/tunnelmonitor/tunnelmonitor-util',["jquery"], function ($) {
	
	"use strict";
	
	var TunnelMonitorUtil = {	

		/**
		 * 
		 * @param bps
		 */
		formatBps : function (bps) {
			 
			var unit = "b/s";
			var num = bps;
			
			var Kbs = bps/1000;
			var Mbs = Kbs/1000;
			var Gbs = Mbs/1000;
			
			if (Gbs > 1) {
				num = parseFloat(Math.round(Gbs * 100) / 100).toFixed(2);
				unit = "Gb/s";
			} else if (Mbs > 1){
				num = parseFloat(Math.round(Mbs * 100) / 100).toFixed(2);
				unit = "Mb/s";
			} else if (Kbs > 1){
				num = parseFloat(Math.round(Kbs * 100) / 100).toFixed(2);
				unit = "Kb/s";
			} else {
				num = parseFloat(Math.round(bps * 100) / 100).toFixed(2);
				unit = "b/s";
			}			
			return  $("<span class='VariableNumber'>"+num+"</span>&nbsp;<span class='VariableUnit'>"+unit+"</span>");
		},
	
		/**
		 * 
		 * @param ps
		 */
		formatPs : function (ps) {
			return  $("<span>(<span class='VariableNumber'>"+ps+"</span>&nbsp;<span class='VariableUnit'>p/s</span>)</span>");
		}
	};
	
	return TunnelMonitorUtil;
	
});

/**
 * ConnectionInfoView
 *
 * @author KADA
 * @since 9.2.2015
 */
define('pages/tunnelmonitor/view/connectioninfoitem',["jquery",
        "../tunnelmonitor-util",
        "utils/webi18n!TunnelMonitor"], function ($, TunnelMonitorUtil, webI18n) {
	
	"use strict";
	
	/**
	 * 
	 */
	var ConnectionInfoItem = function (connectionId) {
		
		// The connection ID
		this._connectionId = connectionId;
		
		// The wrapping element;
        this._wrapper = $("<div></div>");
        
        // Create boxes to hold errors and infos.
        this._msgBox = $("<li></li>");
        
        // These fields will contain the connection specific values.
        this._idTextWrap = $("<span class='Name ConnectionName'></span>");
        this._bwUpTextWrap = $("<span>", {'class':'VariableField', 'title': webI18n.t('BPS')});
        this._bwDownTextWrap = $("<span>", {'class':'VariableField', 'title': webI18n.t('BPS')});
        
        // These labels will hold a translation.
        var connectionLabel = $("<b>", {'text':webI18n.t('CONNECTION_LABEL')});
        var upLabel = $("<span>", {'text':webI18n.t('UP_LABEL')});
        var downLabel = $("<span>", {'text':webI18n.t('DOWN_LABEL')});
        
        // This box will hold all information related to the connection.
        var wrapperTable = $("<table class='Item-ConnectionWrapTable'><tr><td width='55%'/><td width='20%'/><td width='25%'/></tr></table>");
        $(wrapperTable[0].rows[0].cells[0]).append(connectionLabel);
        $(wrapperTable[0].rows[0].cells[0]).append(this._idTextWrap);
        $(wrapperTable[0].rows[0].cells[1]).append(upLabel);
        $(wrapperTable[0].rows[0].cells[1]).append(this._bwUpTextWrap);
        $(wrapperTable[0].rows[0].cells[2]).append(downLabel);
        $(wrapperTable[0].rows[0].cells[2]).append(this._bwDownTextWrap);
        this._connectionInfoBox = $("<div></div>");
        this._connectionInfoBox.append(wrapperTable);
        
        // Add it to the wrapper
        this._wrapper.append(this._connectionInfoBox);
        // Wrap it in a li element
        this._wrapper = this._wrapper.wrap("<li></li>").parent();
        // Add the message box
        this._wrapper.append(this._msgBox.wrap("<ul></ul>").parent());
        
	};

	
	/**
	 * 
	 */
	ConnectionInfoItem.prototype.updateInfo = function (connectionInfo) {
		
		// Hide the message box.
		this._msgBox.hide();
		
		this._bwUpTextWrap.empty();
		this._bwUpTextWrap.append(TunnelMonitorUtil.formatBps(connectionInfo.getBandWidthUp()));
		this._bwDownTextWrap.empty();
		this._bwDownTextWrap.append(TunnelMonitorUtil.formatBps(connectionInfo.getBandWidthDown()));
		this._idTextWrap.text(connectionInfo.getId());
	};
	
	/**
	 * 
	 * @param errorMsg
	 */
	ConnectionInfoItem.prototype.showError = function (errorMsg) {
		this._msgBox.show();
		this._msgBox.empty();
		var errorSpan = $("<span class='ErrorMsg'></span>");
		errorSpan.text(errorMsg);
		this._msgBox.append(errorSpan);
	};
	
	/**
	 * 
	 */
	ConnectionInfoItem.prototype.destroy = function() {
		this._wrapper.remove();
	};
	
	/**
	 * 
	 */
	ConnectionInfoItem.prototype.getJQElement = function() {
		return this._wrapper;
	};
	
	return ConnectionInfoItem;
});

define('lib/text!pages/tunnelmonitor/view/tunnelmonitor-style.css',[],function () { return '.TunnelMonitor {\r\n\tposition: relative;\r\n\tfont-size: 11px;\r\n\tvertical-align: text-bottom;\r\n\twidth: 700px;\r\n}\r\n.TunnelMonitor-List-Item-Outer .TunnelInfo {\r\n\tpadding-top: 15px;\r\n}\r\n.TunnelMonitor .Item-HeaderTable {\r\n\twidth: 100%;\r\n\tfont-size: 11px;\r\n}\r\n.TunnelMonitor .Item-HeaderRightCell {\r\n\ttext-align: right;\r\n}\r\n.TunnelMonitor .UserInfoWrappingTable {\r\n\tfont-size: 11px;\r\n\twidth: 100%;\r\n}\r\n.TunnelMonitor .Item-BandWidthTable {\r\n\tfont-size: 11px;\r\n}\r\n.TunnelMonitor .Item-BandWidthTable td {\r\n\tpadding: 1px 8px 1px 8px;\r\n}\r\n.TunnelMonitor .Item-AddressTable {\r\n\tfont-size: 11px;\r\n\tmargin-left: 15px;\r\n}\r\n.TunnelMonitor .Item-AddressTable td {\r\n\tpadding: 1px 8px 1px 8px;\r\n}\r\n.TunnelMonitor .Item-ConnectionWrapTable {\r\n\tfont-size: 11px;\r\n\twidth: 100%;\r\n}\r\n.TunnelMonitor span.Name {\r\n\t-moz-border-radius: 3px;\r\n    -webkit-border-radius: 3px;\r\n    -khtml-border-radius: 3px;\r\n    border-radius: 3px;\r\n\tpadding: 2px;\r\n}\r\n.TunnelMonitor .VariableNumber {\r\n\tfont-size: 12px;\r\n}\r\n.TunnelMonitor .LinkButton {\r\n\tcursor: Pointer;\r\n}\r\n.TunnelMonitor .PauseResumeButton {\r\n\tdisplay: inline-block;\r\n}\r\n.TunnelMonitor .HideButton {\r\n\tdisplay: none;\r\n}\r\n.TunnelMonitor .LinkButtonContainer {\r\n\tpadding: 5px;\r\n}\r\n.TunnelMonitor .TopMenuContainer {\r\n\twhite-space: nowrap;\r\n}\r\n.TunnelMonitor .TopMenuContainerCell {\r\n\ttext-align: right;\r\n}\r\n.TunnelMonitor .VariableUnit {\r\n\tfont-size: 9px;\r\n}\r\n.TunnelMonitor .VariableField {\r\n\tfont-style: italic;\r\n\tcursor: help;\r\n}';});

/**
 * When a dependency is made to this module, the "tunnelmonitor-style.css" will get inserted in the DOM.
 */

define('pages/tunnelmonitor/view/tunnelmonitor-style',['require','exports','module','lib/text!./tunnelmonitor-style.css','lib/dom'],function(require, exports, module) {
	exports.cssText = require("lib/text!./tunnelmonitor-style.css");
	var dom = require("lib/dom");
	dom.importCssString(exports.cssText);
});

/**
 * BindingInfoItem
 *
 * @author KADA
 * @since 9.2.2015
 */
define('pages/tunnelmonitor/view/bindinginfoitem',["jquery",
        "component/tree/treeitem", 
        "lib/oop",
        "utils/webi18n!TunnelMonitor",
        "./connectioninfoitem",
        "../tunnelmonitor-util",
        "./tunnelmonitor-style"
        ], function ($, TreeItem, oop, webI18n, ConnectionInfoItem, TunnelMonitorUtil) {
	
	"use strict";
	
	/**
	 * 
	 */
	var BindingInfoItem = function (bindingId) {
		
		// Extend TreeItem.
		TreeItem.call(this);
		
		// The binding id
		this._bindingId = bindingId;
		
		// Private variables of this component.
		this._onMonitoringToggledCallbacks = [];
		this._isMonitoringPaused = false;
        
        /*
         * Build the rest of the component.
         */
        
        // These fields will contain the binding specific values.
        this._nameTextWrap = $("<span class='Name BindingName'></span>");
        this._statusTextWrap = $("<span></span>");
        this._bwUpTextWrap = $("<span>", {'class':'VariableField', 'title':webI18n.t('BPS')});
        this._bwDownTextWrap = $("<span>", {'class':'VariableField', 'title': webI18n.t('BPS')});
        this._packetsUpTextWrap = $("<span>", {'class':'VariableField', 'title': webI18n.t('PPS')});
        this._packetsDownTextWrap = $("<span>", {'class':'VariableField', 'title': webI18n.t('PPS')});
        this._isUpstreamTextWrap = $("<span></span>");
        this._remoteAdressTextWrap = $("<span>", {'class':'VariableField', 'title': webI18n.t('REMOTE_ADDRESS')});
        this._localAdressTextWrap = $("<span>", {'class':'VariableField', 'title': webI18n.t('LOCAL_ADDRESS')});
        
        // These labels will hold a translation.
        var bindingLabel = $("<b>", {'text':webI18n.t('BINDING_LABEL')});
        var upLabel = $("<span>", {'text':webI18n.t('UP_LABEL')});
        var downLabel = $("<span>", {'text':webI18n.t('DOWN_LABEL')});
        var remoteLabel = $("<span>", {'text':webI18n.t('REMOTE_LABEL')});
        var localLabel = $("<span>", {'text':webI18n.t('LOCAL_LABEL')});
        
        // Create the title
        var titleSpan = $("<span></span>");
        titleSpan.append(bindingLabel);
        titleSpan.append(this._nameTextWrap);
        var headerTable = $("<table class='Item-HeaderTable'><tr><td class='Item-HeaderLeftCell'/><td class='Item-HeaderRightCell'/></tr></table>");
        headerTable.find(".Item-HeaderLeftCell").append(titleSpan);
        headerTable.find(".Item-HeaderRightCell").append(this._statusTextWrap);
        this.getView().setTitleElement(headerTable);
        
        
        // Create the actual content.
        var bandWidthTable = $("<table class='Item-BandWidthTable'><tr><td/><td/><td/></tr><tr><td/><td/><td/></tr></table>");
        $(bandWidthTable[0].rows[0].cells[0]).append(upLabel);
        $(bandWidthTable[0].rows[1].cells[0]).append(downLabel);
        $(bandWidthTable[0].rows[0].cells[1]).append(this._bwUpTextWrap);
        $(bandWidthTable[0].rows[1].cells[1]).append(this._bwDownTextWrap);
        $(bandWidthTable[0].rows[0].cells[2]).append(this._packetsUpTextWrap);
        $(bandWidthTable[0].rows[1].cells[2]).append(this._packetsDownTextWrap);
        
        var addressTable = $("<table class='Item-AddressTable'><tr><td/><td/></tr><tr><td/><td/></tr></table>");
        $(addressTable[0].rows[0].cells[0]).append(remoteLabel);
        $(addressTable[0].rows[1].cells[0]).append(localLabel);
        $(addressTable[0].rows[0].cells[1]).append(this._remoteAdressTextWrap);
        $(addressTable[0].rows[1].cells[1]).append(this._localAdressTextWrap);
        
        var wrapTable = $("<table><tr><td/><td/></tr></table>");
        $(wrapTable[0].rows[0].cells[0]).append(bandWidthTable);
        $(wrapTable[0].rows[0].cells[1]).append(addressTable);  
        this.getView().setContentElement(wrapTable);
		
        // Update with no info to put in right state.
        this.updateConnectionsInfo([]);
        
        // Expand by default.
        this.expand();
	};
	
	/**
	 * Extend TreeItem (all the methods).
	 */
	oop.inherits(BindingInfoItem, TreeItem);
	
	
	/**
	 * 
	 */
	BindingInfoItem.prototype.updateInfo = function (bindingInfo) {
		
		this._bwUpTextWrap.empty();
		this._bwUpTextWrap.append(TunnelMonitorUtil.formatBps(bindingInfo.getBandWidthUp()));
		this._bwDownTextWrap.empty();
		this._bwDownTextWrap.append(TunnelMonitorUtil.formatBps(bindingInfo.getBandWidthDown()));
		this._packetsUpTextWrap.empty();
		this._packetsUpTextWrap.append(TunnelMonitorUtil.formatPs(bindingInfo.getPacketsUp()));
		this._packetsDownTextWrap.empty();
		this._packetsDownTextWrap.append(TunnelMonitorUtil.formatPs(bindingInfo.getPacketsDown()));
		this._statusTextWrap.text(bindingInfo.getStatus());
		this._nameTextWrap.text(bindingInfo.getName());
		this._remoteAdressTextWrap.text(bindingInfo.getRemoteAdress());
		this._localAdressTextWrap.text(bindingInfo.getLocalAdress());
		this.updateConnectionsInfo(bindingInfo.getConnectionsInfo());
	};
	
	/**
	 * 
	 * @param connectionsInfo
	 */
	BindingInfoItem.prototype.updateConnectionsInfo = function (connectionInfos) {
		
		// Check if we have to show active bindings.
		if (connectionInfos.length == 0) {
			this.getView().showMessage("No active connections");
			return;
		} else {
			this.getView().switchToDisplayingChildren();
		}
		
		// Update all the bindingInfoItems
		var passedConnectionIds = [];
		for (var i=0; i < connectionInfos.length; i++) {
			
			var connectionId = connectionInfos[i].getId();
			passedConnectionIds.push(connectionId);
			var connectionInfoItem = undefined;
			for (var j=0; j < this._children.length; j++) {
				if (this._children[j]._connectionId == connectionId) {
					connectionInfoItem = this._children[j];
				}
			}
			
			// If no view element existst for this binding, make one.
			if (!connectionInfoItem) {
				// Create it.
				connectionInfoItem = new ConnectionInfoItem(connectionId);
				this.addChild(connectionInfoItem);
			}

			// Update the view
			connectionInfoItem.updateInfo(connectionInfos[i]);
		}
		
		// Remove the connectionInfoItems who are no longer mentioned in bindingInfos.
		for (var i=0; i < this._children.length; i++) {
			var connectionId = this._children[i]._connectionId;
		    if ($.inArray(connectionId, passedConnectionIds) < 0) {
		    	this.removeChild(this._children[i]);
		    }
		}
		
	};
	
	return BindingInfoItem;
	
});

/**
 * TunnelMonitorView
 *
 * @author KADA
 * @since 9.2.2015
 */
define('pages/tunnelmonitor/view/tunnelinfoitem',["jquery",
        "component/tree/treeitem", 
        "lib/oop",
        "utils/webi18n!TunnelMonitor",
        "./bindinginfoitem",
        "../tunnelmonitor-util",
        "./tunnelmonitor-style"
        ], function ($, TreeItem, oop, webI18n, BindingInfoItem, TunnelMonitorUtil) {
	
	"use strict";
	
	/**
	 * 
	 */
	var TunnelInfoItem = function (tunnelId) {
		
		// Extend TreeItem.
		TreeItem.call(this);
		
		// The Tunnel Id
		this._tunnelId = tunnelId;
		
		// Private variables of this component.
		this._onMonitoringToggledCallbacks = [];
		this._isMonitoringPaused = false;
        
        /*
         * Build the rest of the component.
         */
        
        // These fields will contain the tunnel specific values.
        this._nameTextWrap = $("<span class='Name TunnelName'></span>");
        this._statusTextWrap = $("<span></span>");
        this._bwUpTextWrap = $("<span>", {'class':'VariableField', 'title': webI18n.t('BPS')});
        this._bwDownTextWrap = $("<span>", {'class':'VariableField', 'title': webI18n.t('BPS')});
        this._packetsUpTextWrap = $("<span>", {'class':'VariableField', 'title': webI18n.t('PPS')});
        this._packetsDownTextWrap = $("<span>", {'class':'VariableField', 'title': webI18n.t('PPS')});
        
        // These labels will hold a translation.
        var tunnelLabel = $("<b>", {'text':webI18n.t('TUNNEL_LABEL')});
        var upLabel = $("<span>", {'text':webI18n.t('UP_LABEL')});
        var downLabel = $("<span>", {'text':webI18n.t('DOWN_LABEL')});

        
        // Create the title
        var titleSpan = $("<span></span>");
        titleSpan.append(tunnelLabel);
        titleSpan.append(this._nameTextWrap);
        var headerTable = $("<table class='Item-HeaderTable'><tr><td class='Item-HeaderLeftCell'/><td class='Item-HeaderRightCell'/></tr></table>");
        headerTable.find(".Item-HeaderLeftCell").append(titleSpan);
        headerTable.find(".Item-HeaderRightCell").append(this._statusTextWrap);
        this.getView().setTitleElement(headerTable);
        
        
        // Create the actual content.
        var bandWidthTable = $("<table class='Item-BandWidthTable'><tr><td/><td/><td/></tr><tr><td/><td/><td/></tr></table>");
        $(bandWidthTable[0].rows[0].cells[0]).append(upLabel);
        $(bandWidthTable[0].rows[1].cells[0]).append(downLabel);
        $(bandWidthTable[0].rows[0].cells[1]).append(this._bwUpTextWrap);
        $(bandWidthTable[0].rows[1].cells[1]).append(this._bwDownTextWrap);
        $(bandWidthTable[0].rows[0].cells[2]).append(this._packetsUpTextWrap);
        $(bandWidthTable[0].rows[1].cells[2]).append(this._packetsDownTextWrap);
        this.getView().setContentElement(bandWidthTable);
		
        // Update with no info to put in right state.
        this.updateBindingsInfo([]);
        
        // Expand by default.
        this.expand();
	};
	
	/**
	 * Extend TreeItem (all the methods).
	 */
	oop.inherits(TunnelInfoItem, TreeItem);
	
	/**
	 * 
	 */
	TunnelInfoItem.prototype.updateInfo = function (tunnelInfo) {
		
		this._bwUpTextWrap.empty();
		this._bwUpTextWrap.append(TunnelMonitorUtil.formatBps(tunnelInfo.getBandWidthUp()));
		this._bwDownTextWrap.empty();
		this._bwDownTextWrap.append(TunnelMonitorUtil.formatBps(tunnelInfo.getBandWidthDown()));
		this._packetsUpTextWrap.empty();
		this._packetsUpTextWrap.append(TunnelMonitorUtil.formatPs(tunnelInfo.getPacketsUp()));
		this._packetsDownTextWrap.empty();
		this._packetsDownTextWrap.append(TunnelMonitorUtil.formatPs(tunnelInfo.getPacketsDown()));
		this._statusTextWrap.text(tunnelInfo.getStatus());
		this._nameTextWrap.text(tunnelInfo.getName());
		this.updateBindingsInfo(tunnelInfo.getBindingsInfo());
		
		if (tunnelInfo.getErrorMsg() && tunnelInfo.getErrorMsg().length > 0) {
			this.showError(tunnelInfo.getErrorMsg());
		}
	};
	
	/**
	 * 
	 * @param bindingInfos
	 */
	TunnelInfoItem.prototype.updateBindingsInfo = function (bindingInfos) {
		
		// Check if we have to show active bindings.
		if (bindingInfos.length == 0) {
			this.getView().showMessage("No active bindings");
			return;
		} else {
			this.getView().switchToDisplayingChildren();
		}
		
		// Update all the bindingInfoItems
		var passedBindingIds = [];
		for (var i=0; i < bindingInfos.length; i++) {
			
			var bindingId = bindingInfos[i].getId();
			passedBindingIds.push(bindingId);
			var bindingInfoItem = undefined;
			for (var j=0; j < this._children.length; j++) {
				if (this._children[j]._bindingId == bindingId) {
					bindingInfoItem = this._children[j];
				}
			}
			
			// If no view element existst for this binding, make one.
			if (!bindingInfoItem) {
				// Create it.
				bindingInfoItem = new BindingInfoItem(bindingId);
				this.addChild(bindingInfoItem);
			}

			// Update the view
			bindingInfoItem.updateInfo(bindingInfos[i]);
		}
		
		// Remove the bindingInfoItems who are no longer mentioned in bindingInfos.
		for (var i=0; i < this._children.length; i++) {
			var bindingId = this._children[i]._bindingId;
			if ($.inArray(bindingId, passedBindingIds) < 0) {
		    	this.removeChild(this._children[i]);
		    }
		}
	};
	
	return TunnelInfoItem;
	
});

/**
 * TunnelMonitor View
 *
 * @author KADA
 * @since 9.2.2015
 */
define('pages/tunnelmonitor/view/tunnelmonitor-view',["jquery",
        "component/tree/treeitem",
        "lib/oop",
        "utils/webi18n!TunnelMonitor",
        "./tunnelinfoitem",
        "utils/dateformatter",
        "./tunnelmonitor-style"
        ], function ($, TreeItem, oop, webI18n, TunnelInfoItem, DateFormatter) {
	
	"use strict";
	
	/**
	 * 
	 */
	var TunnelMonitorView = function (element, userName) {
		
		// If not wrapped by jQuery, do it.
        if (!(element instanceof jQuery)) {
        	element = $(element);
        }
        
		// Extend TreeItem.
		TreeItem.call(this);
        
        // Private variables of this component.
        this._onMonitoringToggledCallbacks = [];
        
        // Create the view.
        var self = this;
        
        // These labels will hold a translation.
        var userLabel = $("<b/>").text(webI18n.t("USER_LABEL"));
        var expandAllButton = $("<a class='LinkButton'/>").text(webI18n.t("EXPAND"));
        var collapseAllButton = $("<a  class='LinkButton'/>").text(webI18n.t("COLLAPSE"));
        this._pauseButton = $("<a class='LinkButton PauseResumeButton'/>").text(webI18n.t("PAUSE"));
        this._resumeButton = $("<a class='LinkButton PauseResumeButton HideButton'/>").text(webI18n.t("RESUME"));
        
        // Create the title.
        
        // Create the action buttons of this view.
        
        // The "Expand" Button
        expandAllButton.on("click",function () {
        	self.deepExpand();
        });
        // The "Collapse" Button
        collapseAllButton.on("click",function () {
        	self.deepCollapse();
        });
        // The "Pause/Resume" Button
        this._pauseButton.on("click",function () {
        	
        	// Show Resume Button
        	self._pauseButton.toggleClass('HideButton');
        	self._resumeButton.toggleClass('HideButton');
        	
        	// Execute the callbacks that should be called when monitoring is toggled
        	for (var i=0; i < self._onMonitoringToggledCallbacks.length; i++) {
        		self._onMonitoringToggledCallbacks[i](true);
        	};
        });
        this._resumeButton.on("click",function () {
        	
        	// Show Pause Button
        	self._pauseButton.toggleClass('HideButton');
        	self._resumeButton.toggleClass('HideButton');
        	
        	// Execute the callbacks that should be called when monitoring is toggled
        	for (var i=0; i < self._onMonitoringToggledCallbacks.length; i++) {
        		self._onMonitoringToggledCallbacks[i](false);
        	};
        });
        // The timestamp field
        this._timeStampField = $("<span></span>");
        
        // Create the title widget
        var titleSpan = $("<span class='RootSpan'></span>");
        titleSpan.append(userLabel);
        titleSpan.append($("<span class='Name UserName'>"+userName+"</span>"));
        var buttonContainer = $("<div class='TopMenuContainer'/>");
        buttonContainer.append(expandAllButton);
        buttonContainer.append("<span class='LinkButtonContainer'>|</span>");
        buttonContainer.append(collapseAllButton);
        buttonContainer.append("<span class='LinkButtonContainer'>|</span>");
        buttonContainer.append(this._pauseButton);
        buttonContainer.append(this._resumeButton);
        buttonContainer.append("<span class='LinkButtonContainer'>|</span>");
        buttonContainer.append(this._timeStampField);
        var titleTable = $("<table class='UserInfoWrappingTable'><tr><td></td><td class='darktext_greenlink_12 TopMenuContainerCell'></td></tr></table>");
        $(titleTable[0].rows[0].cells[0]).append(titleSpan);
        $(titleTable[0].rows[0].cells[1]).append(buttonContainer);
        this.getView().setTitleElement(titleTable);
        
        // Set this item as the root.
        this.getView().setRoot();
        
        // Wrap this item 
        this._wrapper = this.getView().getJQElement().wrap("<div class='TunnelMonitor'></div>").parent();
        
        // Add this wrapper to the element.
        element.append(this._wrapper);
        
        // Update with no info to put in right state.
        this.updateTunnelInfos([]);
        
        // Expand by default.
        this.expand();
	};
	
	/**
	 * Extend TreeItem (all the methods).
	 */
	oop.inherits(TunnelMonitorView, TreeItem);
	
	/**
	 * 
	 * @param tunnelInfos
	 */
	TunnelMonitorView.prototype.updateTunnelInfos = function (tunnelInfos, timeStamp) {
		
		// Update the timeStamp
		if (typeof timeStamp === "number") {
			this._timeStampField.text(DateFormatter.formatDate(new Date(timeStamp), DateFormatter.TimeFormatWithSeconds));
		} else {
			this._timeStampField.text(DateFormatter.formatDate(new Date(), DateFormatter.TimeFormatWithSeconds));
		}
		
		
		// Check if we have to show active tunnels.
		if (tunnelInfos.length == 0) {
			this.getView().showMessage("No active tunnels");
			return;
		} else {
			this.getView().switchToDisplayingChildren();
		}

		// Update all the TunnelInfoViews
		var passedTunnelIds = [];
		for (var i=0; i < tunnelInfos.length; i++) {
			
			var tunnelId = tunnelInfos[i].getId();
			passedTunnelIds.push(tunnelId);
			var tunnelInfoItem = undefined;
			for (var j=0; j < this._children.length; j++) {
				if (this._children[j]._tunnelId == tunnelId) {
					tunnelInfoItem = this._children[j];
				}
			}
			
			// If no view element existst for this tunnel, make one.
			if (!tunnelInfoItem) {
				// Create it.
				tunnelInfoItem = new TunnelInfoItem(tunnelId);
				this.addChild(tunnelInfoItem);
			}

			// Update the view
			tunnelInfoItem.updateInfo(tunnelInfos[i]);
		}
		
		// Remove the tunnelInfoItems who are no longer mentioned in tunnelInfos.
		for (var i=0; i < this._children.length; i++) {
			var tunnelId = this._children[i]._tunnelId;
		    if ($.inArray(tunnelId, passedTunnelIds) < 0) {
		    	this.removeChild(this._children[i]);
		    }
		}
	};
	
	/**
	 * 
	 * @param callback
	 */
	TunnelMonitorView.prototype.onMonitoringToggled = function (callback) {
		this._onMonitoringToggledCallbacks.push(callback);
	};
 	
	/**
	 * 
	 */
	TunnelMonitorView.prototype.getJQElement = function () {
		return this._wrapper;
	};
	
	return TunnelMonitorView;
});

/**
 * ConnectionInfo
 */


define('pages/tunnelmonitor/data/connectioninfo',[], function() {
	"use strict";
	
	var ConnectionInfo = function (json) {
		this._id = json.id;
		this._timeStamp = json.timeStamp;
		this._bwUp = json.bwUp;
		this._bwDown = json.bwDown;
		this._packetsUp = json.packetsUp;
		this._packetsDown = json.packetsDown;
	};
	
	ConnectionInfo.prototype.getId = function () {
		return this._id;
	};
	
	ConnectionInfo.prototype.getTimeStamp = function () {
		return this._timeStamp;
	};
	
	ConnectionInfo.prototype.getBandWidthDown = function () {
		return this._bwDown;
	};
	
	ConnectionInfo.prototype.getBandWidthUp = function () {
		return this._bwUp;
	};
	
	ConnectionInfo.prototype.getPacketsUp = function () {
		return this._packetsUp;
	};
	
	ConnectionInfo.prototype.getPacketsDown = function () {
		return this._packetsDown;
	};
	
	return ConnectionInfo;
});
	

/**
 * BindingInfo
 */


define('pages/tunnelmonitor/data/bindinginfo',["./connectioninfo"], function(ConnectionInfo) {
	"use strict";
	
	var BindingInfo = function (json) {
		this._id = json.id;
		this._name = json.name;
		this._status = json.status;
		this._remoteAdress = json.remoteAdress;
		this._localAdress = json.localAdress;
		this._isUpStream = json.isUpStream;
		this._connectionsInfo = [];
		for (var i=0; i < json.connections.length; i++) {
			this._connectionsInfo.push(new ConnectionInfo(json.connections[i]));
		}
	};
	
	BindingInfo.prototype.getId = function () {
		return this._id;
	};
	
	BindingInfo.prototype.getName = function () {
		return this._name;
	};
	
	BindingInfo.prototype.getStatus = function () {
		return this._status;
	};
	
	BindingInfo.prototype.getRemoteAdress = function () {
		return this._remoteAdress;
	};
	
	BindingInfo.prototype.getLocalAdress = function () {
		return this._localAdress;
	};
	
	BindingInfo.prototype.getIsUpstream = function () {
		return this._isUpstream;
	};
	
	BindingInfo.prototype.getConnectionsInfo = function () {
		return this._connectionsInfo;
	};
	
	BindingInfo.prototype.getBandWidthDown = function () {
		var sum = 0;
		for (var i=0; i < this._connectionsInfo.length; i++) {
			sum += this._connectionsInfo[i].getBandWidthDown();
		}
		return sum;
	};
	
	BindingInfo.prototype.getBandWidthUp = function () {
		var sum = 0;
		for (var i=0; i < this._connectionsInfo.length; i++) {
			sum += this._connectionsInfo[i].getBandWidthUp();
		}
		return sum;
	};
	
	BindingInfo.prototype.getPacketsUp = function () {
		var sum = 0;
		for (var i=0; i < this._connectionsInfo.length; i++) {
			sum += this._connectionsInfo[i].getPacketsUp();
		}
		return sum;
	};
	
	BindingInfo.prototype.getPacketsDown = function () {
		var sum = 0;
		for (var i=0; i < this._connectionsInfo.length; i++) {
			sum += this._connectionsInfo[i].getPacketsDown();
		}
		return sum;
	};
	
	return BindingInfo;
});

/**
 * TunnelInfo
 */


define('pages/tunnelmonitor/data/tunnelinfo',["./bindinginfo"], function(BindingInfo) {
	"use strict";
	
	var TunnelInfo = function (json) {
		this._id = json.id;
		this._name = json.name;
		this._status = json.status;
		this._errorMsg = json.errorMsg;
		this._bindings = [];
		for (var i=0; i < json.bindings.length; i++) {
			this._bindings.push(new BindingInfo(json.bindings[i]));
		}
		
	};
	
	TunnelInfo.prototype.getId = function () {
		return this._id;
	};
	
	TunnelInfo.prototype.getName = function () {
		return this._name;
	};
	
	TunnelInfo.prototype.getStatus = function () {
		return this._status;
	};
	
	TunnelInfo.prototype.getErrorMsg = function () {
		return this._errorMsg;
	};
	
	TunnelInfo.prototype.getBindingsInfo = function () {
		return this._bindings;
	};
	
	TunnelInfo.prototype.getBandWidthDown = function () {
		var sum = 0;
		for (var i=0; i < this._bindings.length; i++) {
			sum += this._bindings[i].getBandWidthDown();
		}
		return sum;
	};
	
	TunnelInfo.prototype.getBandWidthUp = function () {
		var sum = 0;
		for (var i=0; i < this._bindings.length; i++) {
			sum += this._bindings[i].getBandWidthUp();
		}
		return sum;
	};
	
	TunnelInfo.prototype.getPacketsUp = function () {
		var sum = 0;
		for (var i=0; i < this._bindings.length; i++) {
			sum += this._bindings[i].getPacketsUp();
		}
		return sum;
	};
	
	TunnelInfo.prototype.getPacketsDown = function () {
		var sum = 0;
		for (var i=0; i < this._bindings.length; i++) {
			sum += this._bindings[i].getPacketsDown();
		}
		return sum;
	};
	
	return TunnelInfo;
});
    

/**
 * TunnelMonitorService
 *
 * @author KADA
 * @since 9.2.2015
 */
define('pages/tunnelmonitor/tunnelmonitor-service',["jquery","./data/tunnelinfo"], function ($, TunnelInfo) {
	
	"use strict";
	
	var TunnelMonitorService = (function (){
			
			// Instance stores a reference to the Singleton
			var instance;
			
			function init() {
				
				// The baseUrl of the service.
		        var jspBaseUrl = window.webcenterConfig.jspBaseUrl;
				
				return {
					
					getTunnelMonitorInfoFor : function (clientName, onSuccesCallback, onFailCallback) {
						
						// Do the async call to the service
						$.ajaxSetup({ cache: false });
	            		$.getJSON( jspBaseUrl+"tunnelmonitor.wcr?"+$.param({action:"GET_TUNNEL_INFO", client:clientName}), function( data ) {
	            			
	            			var timeStamp = data.timeStamp;
	            			var tunnelInfos = [];
	            			for (var i=0; i < data.tunnelInfos.length; i++) {
	            				tunnelInfos.push(new TunnelInfo(data.tunnelInfos[i]));
	            			}
	            			
	            			onSuccesCallback(tunnelInfos, timeStamp);
	            			
	                	}).fail(function(jqxhr, textStatus, error) {
	                		// Try to parse the error.
	        	    		var msg;
	        	    		try {
	        	    			msg =JSON.parse(jqxhr.responseText).error.message;
	        	    		} catch (e) {
	        	    			msg = error;
	        	    		}
	        	    		// Execute the callback.
	                		onFailCallback(msg);
	                	});
					}
				};
			}
			
			return {
				// Get the Singleton instance if one exists
				// or create one if it doesn't
				getInstance : function() {
					if (!instance) {
						instance = init();
					}
					return instance;
				}
			};
	})();
	    
	return TunnelMonitorService;

});

/**
 * TunnelMonitor
 *
 * @author KADA
 * @since 9.2.2015
 */
define('pages/tunnelmonitor/tunnelmonitor',["jquery",
        "./view/tunnelmonitor-view",
        "./tunnelmonitor-service"
        ], function ($, TunnelMonitorView, TunnelMonitorService) {
	
	"use strict";
	
	/**
	 * Create a TunnelMonitor from a div HTML element.
	 */
	var TunnelMonitor = function (element) {
		
		// Wrap the element with jQuery if this is not the case ...
		if (!(element instanceof jQuery)) {
        	element = $(element);
        }
		
        // Extract the tunnel name from the element.
		this._clientName = element.attr("data-tunnel-client");
		
		// Create the necessary components.
		this._tunnelMonitorService = TunnelMonitorService.getInstance();
        this._tunnelMonitorView = new TunnelMonitorView(element, this._clientName);
        
        
        // What to do when someone toggles pause/resume monitoring
        var self = this;
        this._tunnelMonitorView.onMonitoringToggled(function (isPaused) {
        	if (!isPaused) {
        		self.startMonitoring();
        	} else {
        		self.pauseMonitoring();
        	}
        });
        
        // Indicates whether we're currently monitoring.
        this._isMonitoring = false;
        
		// Start monitoring.
		this.startMonitoring();
	};
	
	
	/**
	 * Start monitoring the tunnels for the passed client.
	 */
	TunnelMonitor.prototype.startMonitoring = function () {
		
		// We should no longer pause monitoring (used for terminating).
		this._pauseMonitoring = false;
		
		// If we are already monitoring ...
		if (this._isMonitoring) {
			return;
		}
		
		var self = this;
		function monitor () {
			// Only do this when not paused.
			if (!self._pauseMonitoring) {
				
				self._isMonitoring = true;
				
				// Do this every 500 sec.
				setTimeout(function(){
					// Request the current tunnelInfo from the TunnelMonitorService
					self._tunnelMonitorService.getTunnelMonitorInfoFor(self._clientName, function (tunnelInfos, timeStamp) {
							// OnSuccess, update the Tunnel Info in the TunnelMonitorView (if not paused).
							if (!self._pauseMonitoring) {
								self._tunnelMonitorView.updateTunnelInfos(tunnelInfos, timeStamp);
							}
							// Request new tunnelMonitorService update.
							monitor();
						}, function (errorMsg) {
							// Something went wrong requesting the tunnelInfo, show the error (if not paused).
							if (!self._pauseMonitoring) {
								self._tunnelMonitorView.showError(errorMsg);
							}
							// Request new tunnelMonitorService update.
							monitor();
					});
				}, 500);
			} else {
				self._isMonitoring = false;
			}
		}
		monitor();
	};
	
	/**
	 * 
	 */
	TunnelMonitor.prototype.pauseMonitoring = function () {
		// Pause the monitoring.
		this._pauseMonitoring = true;
	};
	
	
    /**
     * Method to automatically apply this module to a div element ...
     */
	TunnelMonitor.wcrinit = function(element){
        new TunnelMonitor(element);
    };
	
	return TunnelMonitor;
});
