/**
 * BindingInfoItem
 *
 * @author KADA
 * @since 9.2.2015
 */
define(["jquery",
        "component/tree/treeitem", 
        "lib/oop",
        "utils/webi18n!TunnelMonitor",
        "./connectioninfoitem",
        "../tunnelmonitor-util",
        "./tunnelmonitor-style"
        ], function ($, TreeItem, oop, webI18n, ConnectionInfoItem, TunnelMonitorUtil) {
	
	"use strict";
	
	/**
	 * 
	 */
	var BindingInfoItem = function (bindingId) {
		
		// Extend TreeItem.
		TreeItem.call(this);
		
		// The binding id
		this._bindingId = bindingId;
		
		// Private variables of this component.
		this._onMonitoringToggledCallbacks = [];
		this._isMonitoringPaused = false;
        
        /*
         * Build the rest of the component.
         */
        
        // These fields will contain the binding specific values.
        this._nameTextWrap = $("<span class='Name BindingName'></span>");
        this._statusTextWrap = $("<span></span>");
        this._bwUpTextWrap = $("<span>", {'class':'VariableField', 'title':webI18n.t('BPS')});
        this._bwDownTextWrap = $("<span>", {'class':'VariableField', 'title': webI18n.t('BPS')});
        this._packetsUpTextWrap = $("<span>", {'class':'VariableField', 'title': webI18n.t('PPS')});
        this._packetsDownTextWrap = $("<span>", {'class':'VariableField', 'title': webI18n.t('PPS')});
        this._isUpstreamTextWrap = $("<span></span>");
        this._remoteAdressTextWrap = $("<span>", {'class':'VariableField', 'title': webI18n.t('REMOTE_ADDRESS')});
        this._localAdressTextWrap = $("<span>", {'class':'VariableField', 'title': webI18n.t('LOCAL_ADDRESS')});
        
        // These labels will hold a translation.
        var bindingLabel = $("<b>", {'text':webI18n.t('BINDING_LABEL')});
        var upLabel = $("<span>", {'text':webI18n.t('UP_LABEL')});
        var downLabel = $("<span>", {'text':webI18n.t('DOWN_LABEL')});
        var remoteLabel = $("<span>", {'text':webI18n.t('REMOTE_LABEL')});
        var localLabel = $("<span>", {'text':webI18n.t('LOCAL_LABEL')});
        
        // Create the title
        var titleSpan = $("<span></span>");
        titleSpan.append(bindingLabel);
        titleSpan.append(this._nameTextWrap);
        var headerTable = $("<table class='Item-HeaderTable'><tr><td class='Item-HeaderLeftCell'/><td class='Item-HeaderRightCell'/></tr></table>");
        headerTable.find(".Item-HeaderLeftCell").append(titleSpan);
        headerTable.find(".Item-HeaderRightCell").append(this._statusTextWrap);
        this.getView().setTitleElement(headerTable);
        
        
        // Create the actual content.
        var bandWidthTable = $("<table class='Item-BandWidthTable'><tr><td/><td/><td/></tr><tr><td/><td/><td/></tr></table>");
        $(bandWidthTable[0].rows[0].cells[0]).append(upLabel);
        $(bandWidthTable[0].rows[1].cells[0]).append(downLabel);
        $(bandWidthTable[0].rows[0].cells[1]).append(this._bwUpTextWrap);
        $(bandWidthTable[0].rows[1].cells[1]).append(this._bwDownTextWrap);
        $(bandWidthTable[0].rows[0].cells[2]).append(this._packetsUpTextWrap);
        $(bandWidthTable[0].rows[1].cells[2]).append(this._packetsDownTextWrap);
        
        var addressTable = $("<table class='Item-AddressTable'><tr><td/><td/></tr><tr><td/><td/></tr></table>");
        $(addressTable[0].rows[0].cells[0]).append(remoteLabel);
        $(addressTable[0].rows[1].cells[0]).append(localLabel);
        $(addressTable[0].rows[0].cells[1]).append(this._remoteAdressTextWrap);
        $(addressTable[0].rows[1].cells[1]).append(this._localAdressTextWrap);
        
        var wrapTable = $("<table><tr><td/><td/></tr></table>");
        $(wrapTable[0].rows[0].cells[0]).append(bandWidthTable);
        $(wrapTable[0].rows[0].cells[1]).append(addressTable);  
        this.getView().setContentElement(wrapTable);
		
        // Update with no info to put in right state.
        this.updateConnectionsInfo([]);
        
        // Expand by default.
        this.expand();
	};
	
	/**
	 * Extend TreeItem (all the methods).
	 */
	oop.inherits(BindingInfoItem, TreeItem);
	
	
	/**
	 * 
	 */
	BindingInfoItem.prototype.updateInfo = function (bindingInfo) {
		
		this._bwUpTextWrap.empty();
		this._bwUpTextWrap.append(TunnelMonitorUtil.formatBps(bindingInfo.getBandWidthUp()));
		this._bwDownTextWrap.empty();
		this._bwDownTextWrap.append(TunnelMonitorUtil.formatBps(bindingInfo.getBandWidthDown()));
		this._packetsUpTextWrap.empty();
		this._packetsUpTextWrap.append(TunnelMonitorUtil.formatPs(bindingInfo.getPacketsUp()));
		this._packetsDownTextWrap.empty();
		this._packetsDownTextWrap.append(TunnelMonitorUtil.formatPs(bindingInfo.getPacketsDown()));
		this._statusTextWrap.text(bindingInfo.getStatus());
		this._nameTextWrap.text(bindingInfo.getName());
		this._remoteAdressTextWrap.text(bindingInfo.getRemoteAdress());
		this._localAdressTextWrap.text(bindingInfo.getLocalAdress());
		this.updateConnectionsInfo(bindingInfo.getConnectionsInfo());
	};
	
	/**
	 * 
	 * @param connectionsInfo
	 */
	BindingInfoItem.prototype.updateConnectionsInfo = function (connectionInfos) {
		
		// Check if we have to show active bindings.
		if (connectionInfos.length == 0) {
			this.getView().showMessage("No active connections");
			return;
		} else {
			this.getView().switchToDisplayingChildren();
		}
		
		// Update all the bindingInfoItems
		var passedConnectionIds = [];
		for (var i=0; i < connectionInfos.length; i++) {
			
			var connectionId = connectionInfos[i].getId();
			passedConnectionIds.push(connectionId);
			var connectionInfoItem = undefined;
			for (var j=0; j < this._children.length; j++) {
				if (this._children[j]._connectionId == connectionId) {
					connectionInfoItem = this._children[j];
				}
			}
			
			// If no view element existst for this binding, make one.
			if (!connectionInfoItem) {
				// Create it.
				connectionInfoItem = new ConnectionInfoItem(connectionId);
				this.addChild(connectionInfoItem);
			}

			// Update the view
			connectionInfoItem.updateInfo(connectionInfos[i]);
		}
		
		// Remove the connectionInfoItems who are no longer mentioned in bindingInfos.
		for (var i=0; i < this._children.length; i++) {
			var connectionId = this._children[i]._connectionId;
		    if ($.inArray(connectionId, passedConnectionIds) < 0) {
		    	this.removeChild(this._children[i]);
		    }
		}
		
	};
	
	return BindingInfoItem;
	
});