/**
 * ConnectionInfoView
 *
 * @author KADA
 * @since 9.2.2015
 */
define(["jquery",
        "../tunnelmonitor-util",
        "utils/webi18n!TunnelMonitor"], function ($, TunnelMonitorUtil, webI18n) {
	
	"use strict";
	
	/**
	 * 
	 */
	var ConnectionInfoItem = function (connectionId) {
		
		// The connection ID
		this._connectionId = connectionId;
		
		// The wrapping element;
        this._wrapper = $("<div></div>");
        
        // Create boxes to hold errors and infos.
        this._msgBox = $("<li></li>");
        
        // These fields will contain the connection specific values.
        this._idTextWrap = $("<span class='Name ConnectionName'></span>");
        this._bwUpTextWrap = $("<span>", {'class':'VariableField', 'title': webI18n.t('BPS')});
        this._bwDownTextWrap = $("<span>", {'class':'VariableField', 'title': webI18n.t('BPS')});
        
        // These labels will hold a translation.
        var connectionLabel = $("<b>", {'text':webI18n.t('CONNECTION_LABEL')});
        var upLabel = $("<span>", {'text':webI18n.t('UP_LABEL')});
        var downLabel = $("<span>", {'text':webI18n.t('DOWN_LABEL')});
        
        // This box will hold all information related to the connection.
        var wrapperTable = $("<table class='Item-ConnectionWrapTable'><tr><td width='55%'/><td width='20%'/><td width='25%'/></tr></table>");
        $(wrapperTable[0].rows[0].cells[0]).append(connectionLabel);
        $(wrapperTable[0].rows[0].cells[0]).append(this._idTextWrap);
        $(wrapperTable[0].rows[0].cells[1]).append(upLabel);
        $(wrapperTable[0].rows[0].cells[1]).append(this._bwUpTextWrap);
        $(wrapperTable[0].rows[0].cells[2]).append(downLabel);
        $(wrapperTable[0].rows[0].cells[2]).append(this._bwDownTextWrap);
        this._connectionInfoBox = $("<div></div>");
        this._connectionInfoBox.append(wrapperTable);
        
        // Add it to the wrapper
        this._wrapper.append(this._connectionInfoBox);
        // Wrap it in a li element
        this._wrapper = this._wrapper.wrap("<li></li>").parent();
        // Add the message box
        this._wrapper.append(this._msgBox.wrap("<ul></ul>").parent());
        
	};

	
	/**
	 * 
	 */
	ConnectionInfoItem.prototype.updateInfo = function (connectionInfo) {
		
		// Hide the message box.
		this._msgBox.hide();
		
		this._bwUpTextWrap.empty();
		this._bwUpTextWrap.append(TunnelMonitorUtil.formatBps(connectionInfo.getBandWidthUp()));
		this._bwDownTextWrap.empty();
		this._bwDownTextWrap.append(TunnelMonitorUtil.formatBps(connectionInfo.getBandWidthDown()));
		this._idTextWrap.text(connectionInfo.getId());
	};
	
	/**
	 * 
	 * @param errorMsg
	 */
	ConnectionInfoItem.prototype.showError = function (errorMsg) {
		this._msgBox.show();
		this._msgBox.empty();
		var errorSpan = $("<span class='ErrorMsg'></span>");
		errorSpan.text(errorMsg);
		this._msgBox.append(errorSpan);
	};
	
	/**
	 * 
	 */
	ConnectionInfoItem.prototype.destroy = function() {
		this._wrapper.remove();
	};
	
	/**
	 * 
	 */
	ConnectionInfoItem.prototype.getJQElement = function() {
		return this._wrapper;
	};
	
	return ConnectionInfoItem;
});