/**
 * TunnelMonitorView
 *
 * @author KADA
 * @since 9.2.2015
 */
define(["jquery",
        "component/tree/treeitem", 
        "lib/oop",
        "utils/webi18n!TunnelMonitor",
        "./bindinginfoitem",
        "../tunnelmonitor-util",
        "./tunnelmonitor-style"
        ], function ($, TreeItem, oop, webI18n, BindingInfoItem, TunnelMonitorUtil) {
	
	"use strict";
	
	/**
	 * 
	 */
	var TunnelInfoItem = function (tunnelId) {
		
		// Extend TreeItem.
		TreeItem.call(this);
		
		// The Tunnel Id
		this._tunnelId = tunnelId;
		
		// Private variables of this component.
		this._onMonitoringToggledCallbacks = [];
		this._isMonitoringPaused = false;
        
        /*
         * Build the rest of the component.
         */
        
        // These fields will contain the tunnel specific values.
        this._nameTextWrap = $("<span class='Name TunnelName'></span>");
        this._statusTextWrap = $("<span></span>");
        this._bwUpTextWrap = $("<span>", {'class':'VariableField', 'title': webI18n.t('BPS')});
        this._bwDownTextWrap = $("<span>", {'class':'VariableField', 'title': webI18n.t('BPS')});
        this._packetsUpTextWrap = $("<span>", {'class':'VariableField', 'title': webI18n.t('PPS')});
        this._packetsDownTextWrap = $("<span>", {'class':'VariableField', 'title': webI18n.t('PPS')});
        
        // These labels will hold a translation.
        var tunnelLabel = $("<b>", {'text':webI18n.t('TUNNEL_LABEL')});
        var upLabel = $("<span>", {'text':webI18n.t('UP_LABEL')});
        var downLabel = $("<span>", {'text':webI18n.t('DOWN_LABEL')});

        
        // Create the title
        var titleSpan = $("<span></span>");
        titleSpan.append(tunnelLabel);
        titleSpan.append(this._nameTextWrap);
        var headerTable = $("<table class='Item-HeaderTable'><tr><td class='Item-HeaderLeftCell'/><td class='Item-HeaderRightCell'/></tr></table>");
        headerTable.find(".Item-HeaderLeftCell").append(titleSpan);
        headerTable.find(".Item-HeaderRightCell").append(this._statusTextWrap);
        this.getView().setTitleElement(headerTable);
        
        
        // Create the actual content.
        var bandWidthTable = $("<table class='Item-BandWidthTable'><tr><td/><td/><td/></tr><tr><td/><td/><td/></tr></table>");
        $(bandWidthTable[0].rows[0].cells[0]).append(upLabel);
        $(bandWidthTable[0].rows[1].cells[0]).append(downLabel);
        $(bandWidthTable[0].rows[0].cells[1]).append(this._bwUpTextWrap);
        $(bandWidthTable[0].rows[1].cells[1]).append(this._bwDownTextWrap);
        $(bandWidthTable[0].rows[0].cells[2]).append(this._packetsUpTextWrap);
        $(bandWidthTable[0].rows[1].cells[2]).append(this._packetsDownTextWrap);
        this.getView().setContentElement(bandWidthTable);
		
        // Update with no info to put in right state.
        this.updateBindingsInfo([]);
        
        // Expand by default.
        this.expand();
	};
	
	/**
	 * Extend TreeItem (all the methods).
	 */
	oop.inherits(TunnelInfoItem, TreeItem);
	
	/**
	 * 
	 */
	TunnelInfoItem.prototype.updateInfo = function (tunnelInfo) {
		
		this._bwUpTextWrap.empty();
		this._bwUpTextWrap.append(TunnelMonitorUtil.formatBps(tunnelInfo.getBandWidthUp()));
		this._bwDownTextWrap.empty();
		this._bwDownTextWrap.append(TunnelMonitorUtil.formatBps(tunnelInfo.getBandWidthDown()));
		this._packetsUpTextWrap.empty();
		this._packetsUpTextWrap.append(TunnelMonitorUtil.formatPs(tunnelInfo.getPacketsUp()));
		this._packetsDownTextWrap.empty();
		this._packetsDownTextWrap.append(TunnelMonitorUtil.formatPs(tunnelInfo.getPacketsDown()));
		this._statusTextWrap.text(tunnelInfo.getStatus());
		this._nameTextWrap.text(tunnelInfo.getName());
		this.updateBindingsInfo(tunnelInfo.getBindingsInfo());
		
		if (tunnelInfo.getErrorMsg() && tunnelInfo.getErrorMsg().length > 0) {
			this.showError(tunnelInfo.getErrorMsg());
		}
	};
	
	/**
	 * 
	 * @param bindingInfos
	 */
	TunnelInfoItem.prototype.updateBindingsInfo = function (bindingInfos) {
		
		// Check if we have to show active bindings.
		if (bindingInfos.length == 0) {
			this.getView().showMessage("No active bindings");
			return;
		} else {
			this.getView().switchToDisplayingChildren();
		}
		
		// Update all the bindingInfoItems
		var passedBindingIds = [];
		for (var i=0; i < bindingInfos.length; i++) {
			
			var bindingId = bindingInfos[i].getId();
			passedBindingIds.push(bindingId);
			var bindingInfoItem = undefined;
			for (var j=0; j < this._children.length; j++) {
				if (this._children[j]._bindingId == bindingId) {
					bindingInfoItem = this._children[j];
				}
			}
			
			// If no view element existst for this binding, make one.
			if (!bindingInfoItem) {
				// Create it.
				bindingInfoItem = new BindingInfoItem(bindingId);
				this.addChild(bindingInfoItem);
			}

			// Update the view
			bindingInfoItem.updateInfo(bindingInfos[i]);
		}
		
		// Remove the bindingInfoItems who are no longer mentioned in bindingInfos.
		for (var i=0; i < this._children.length; i++) {
			var bindingId = this._children[i]._bindingId;
			if ($.inArray(bindingId, passedBindingIds) < 0) {
		    	this.removeChild(this._children[i]);
		    }
		}
	};
	
	return TunnelInfoItem;
	
});