/**
 * TunnelMonitor View
 *
 * @author KADA
 * @since 9.2.2015
 */
define(["jquery",
        "component/tree/treeitem",
        "lib/oop",
        "utils/webi18n!TunnelMonitor",
        "./tunnelinfoitem",
        "utils/dateformatter",
        "./tunnelmonitor-style"
        ], function ($, TreeItem, oop, webI18n, TunnelInfoItem, DateFormatter) {
	
	"use strict";
	
	/**
	 * 
	 */
	var TunnelMonitorView = function (element, userName) {
		
		// If not wrapped by jQuery, do it.
        if (!(element instanceof jQuery)) {
        	element = $(element);
        }
        
		// Extend TreeItem.
		TreeItem.call(this);
        
        // Private variables of this component.
        this._onMonitoringToggledCallbacks = [];
        
        // Create the view.
        var self = this;
        
        // These labels will hold a translation.
        var userLabel = $("<b/>").text(webI18n.t("USER_LABEL"));
        var expandAllButton = $("<a class='LinkButton'/>").text(webI18n.t("EXPAND"));
        var collapseAllButton = $("<a  class='LinkButton'/>").text(webI18n.t("COLLAPSE"));
        this._pauseButton = $("<a class='LinkButton PauseResumeButton'/>").text(webI18n.t("PAUSE"));
        this._resumeButton = $("<a class='LinkButton PauseResumeButton HideButton'/>").text(webI18n.t("RESUME"));
        
        // Create the title.
        
        // Create the action buttons of this view.
        
        // The "Expand" Button
        expandAllButton.on("click",function () {
        	self.deepExpand();
        });
        // The "Collapse" Button
        collapseAllButton.on("click",function () {
        	self.deepCollapse();
        });
        // The "Pause/Resume" Button
        this._pauseButton.on("click",function () {
        	
        	// Show Resume Button
        	self._pauseButton.toggleClass('HideButton');
        	self._resumeButton.toggleClass('HideButton');
        	
        	// Execute the callbacks that should be called when monitoring is toggled
        	for (var i=0; i < self._onMonitoringToggledCallbacks.length; i++) {
        		self._onMonitoringToggledCallbacks[i](true);
        	};
        });
        this._resumeButton.on("click",function () {
        	
        	// Show Pause Button
        	self._pauseButton.toggleClass('HideButton');
        	self._resumeButton.toggleClass('HideButton');
        	
        	// Execute the callbacks that should be called when monitoring is toggled
        	for (var i=0; i < self._onMonitoringToggledCallbacks.length; i++) {
        		self._onMonitoringToggledCallbacks[i](false);
        	};
        });
        // The timestamp field
        this._timeStampField = $("<span></span>");
        
        // Create the title widget
        var titleSpan = $("<span class='RootSpan'></span>");
        titleSpan.append(userLabel);
        titleSpan.append($("<span class='Name UserName'>"+userName+"</span>"));
        var buttonContainer = $("<div class='TopMenuContainer'/>");
        buttonContainer.append(expandAllButton);
        buttonContainer.append("<span class='LinkButtonContainer'>|</span>");
        buttonContainer.append(collapseAllButton);
        buttonContainer.append("<span class='LinkButtonContainer'>|</span>");
        buttonContainer.append(this._pauseButton);
        buttonContainer.append(this._resumeButton);
        buttonContainer.append("<span class='LinkButtonContainer'>|</span>");
        buttonContainer.append(this._timeStampField);
        var titleTable = $("<table class='UserInfoWrappingTable'><tr><td></td><td class='darktext_greenlink_12 TopMenuContainerCell'></td></tr></table>");
        $(titleTable[0].rows[0].cells[0]).append(titleSpan);
        $(titleTable[0].rows[0].cells[1]).append(buttonContainer);
        this.getView().setTitleElement(titleTable);
        
        // Set this item as the root.
        this.getView().setRoot();
        
        // Wrap this item 
        this._wrapper = this.getView().getJQElement().wrap("<div class='TunnelMonitor'></div>").parent();
        
        // Add this wrapper to the element.
        element.append(this._wrapper);
        
        // Update with no info to put in right state.
        this.updateTunnelInfos([]);
        
        // Expand by default.
        this.expand();
	};
	
	/**
	 * Extend TreeItem (all the methods).
	 */
	oop.inherits(TunnelMonitorView, TreeItem);
	
	/**
	 * 
	 * @param tunnelInfos
	 */
	TunnelMonitorView.prototype.updateTunnelInfos = function (tunnelInfos, timeStamp) {
		
		// Update the timeStamp
		if (typeof timeStamp === "number") {
			this._timeStampField.text(DateFormatter.formatDate(new Date(timeStamp), DateFormatter.TimeFormatWithSeconds));
		} else {
			this._timeStampField.text(DateFormatter.formatDate(new Date(), DateFormatter.TimeFormatWithSeconds));
		}
		
		
		// Check if we have to show active tunnels.
		if (tunnelInfos.length == 0) {
			this.getView().showMessage("No active tunnels");
			return;
		} else {
			this.getView().switchToDisplayingChildren();
		}

		// Update all the TunnelInfoViews
		var passedTunnelIds = [];
		for (var i=0; i < tunnelInfos.length; i++) {
			
			var tunnelId = tunnelInfos[i].getId();
			passedTunnelIds.push(tunnelId);
			var tunnelInfoItem = undefined;
			for (var j=0; j < this._children.length; j++) {
				if (this._children[j]._tunnelId == tunnelId) {
					tunnelInfoItem = this._children[j];
				}
			}
			
			// If no view element existst for this tunnel, make one.
			if (!tunnelInfoItem) {
				// Create it.
				tunnelInfoItem = new TunnelInfoItem(tunnelId);
				this.addChild(tunnelInfoItem);
			}

			// Update the view
			tunnelInfoItem.updateInfo(tunnelInfos[i]);
		}
		
		// Remove the tunnelInfoItems who are no longer mentioned in tunnelInfos.
		for (var i=0; i < this._children.length; i++) {
			var tunnelId = this._children[i]._tunnelId;
		    if ($.inArray(tunnelId, passedTunnelIds) < 0) {
		    	this.removeChild(this._children[i]);
		    }
		}
	};
	
	/**
	 * 
	 * @param callback
	 */
	TunnelMonitorView.prototype.onMonitoringToggled = function (callback) {
		this._onMonitoringToggledCallbacks.push(callback);
	};
 	
	/**
	 * 
	 */
	TunnelMonitorView.prototype.getJQElement = function () {
		return this._wrapper;
	};
	
	return TunnelMonitorView;
});