define(["wcr/webcenter_html",
        "attributecategory/AttributeCategory",
        "./PageSection",
        "jquery",
        "underscore",
        "wcr/AddDocument",
        "wcr/DocumentsTable",
        "wcr/validate"], function (WCHTML, AttributeCategory, PageSection, $, _) {
	
	/**
	*	AttributesSection constructor (subclass of PageSection)
	* 
	* PageSection serves as a generic superclass of various page sections used within task execution page
	* and others.
	*/
	function AttributesSection(sectionID, expandToggleID, bodyID, data, state) {
		try {
			// call superclass constructor with parameters containing IDs of main elements of the section
			PageSection.call(this, sectionID, expandToggleID, bodyID, state);
		
			// total number of documents
			this.attrsNumber = data.attrsNumber;
			
			this.readOnly = true;
		} catch (err) {
			alert("Problem in AttributesSection constructor: " + err);
		}
	}

	// define AttributesSection as a subclass of PageSection
	AttributesSection.prototype = new PageSection;


	AttributesSection.prototype.initialize = function(name) {
		PageSection.prototype.initialize.call(this, name);
		try {
			// set a flag defining whether this task's section is modifiable
			this.readOnly = (this.parent.isTaskCompleted || !this.parent.isChangeable);

			// reference to attributeCategory object that handles the whole task attributes (specifications) displaying
			this.myAttrCat = null;
		}
		catch (err) {
			alert("Problem in AttributesSection.initialize() method: " + err);
		}
	};


	AttributesSection.prototype.isModified = function(){	
		if (this.myAttrCat) {
			return this.modified || this.myAttrCat.isValidationNeeded(true);
		} else {
			return this.modified;
		}
	};

	
	AttributesSection.prototype.validateFormParams = function(force)
	{
		if (this.myAttrCat) {
			force = (force || ($('#taskStatusInput').val() == "00001_0000000003")); // if status changed to completed manually, it has to force attribute category to be valid (required attributes filled out)
			if (!this.myAttrCat.prepareForSubmit(force)) return false;
		}
		return true;
	};


	AttributesSection.prototype.show = function()
	{
		try {
			PageSection.prototype.show.call(this);
		}
		catch (err) {
			alert("Problem in AttributesSection.show() method: " + err);
		}
	};


	AttributesSection.prototype.generateDynamicContent = function() {
		try {
			// the wctask attributes was already generated - skip it
			if (this.dynamicContentReady) {
				return;
			}
			
			aCatOptions.delayedLoading = true;
			window.aCatOptions.callbackOnLoad = _.bind(function() {
				var aCatObj = this.myAttrCat;
				var view = "Default";
				if(aCatObj.getViewByName('TaskExecution')){
					view = "TaskExecution";
				} else {
					if (this.attrsNumber < 1) {
						this.setState("HIDE");
						this.show(); // repaint the section
					}
				}
				
				aCatObj.activateViewByName(view);
				$("#mainTableId > div").not("#upperButtonRow").removeClass("displayNone");
				$("#LoadingRowId").remove();
			}, this);
			window.aCatOptions.callbackOnFail = function() {
				$("#mainTableId > div").not("#upperButtonRow").removeClass("displayNone");
				$("#LoadingRowId").remove();
			};
			
			this.myAttrCat = new AttributeCategory(window.aCatOptions);
			this.myAttrCat.valueChangedListeners.push(_.bind(function() {
				this.modified = true;
			}, this));

			// dynamic content of this section generated		
			this.dynamicContentReady = true;
		}
		catch (err) {
			alert("Problem in AttributesSection.generateDynamicContent() method: " + err);
		}
	};
	
	return AttributesSection;
});