define(["wcr/webcenter_html",
        "./PageSection",
        "wcr/AddDocument",
        "wcr/DocumentsTable",
        "wcr/validate"], function (WCHTML, PageSection) {
	
	

	/**
	*	DiscussionSection constructor (subclass of PageSection)
	* 
	* PageSection serves as a generic superclass of various page sections used within task execution page
	* and others.
	*/
	function DiscussionSection(sectionID, expandToggleID, bodyID, data, state)
	{
		try {
			// call superclass constructor with parameters containing IDs of main elements of the section
			PageSection.call(this, sectionID, expandToggleID, bodyID, state);
			
			// whether is this section enabled by task type
			this.enabled = data.enabled;
			// total number of messages in discussion
			this.msgNumber = data.msgNumber;
			this.threadID = data.threadID;
			
			this.readOnly = true;
		}
		catch (err) {
			alert("Problem in DiscussionSection constructor: " + err);
		}
	}

	// define DiscussionSection as a subclass of PageSection
	DiscussionSection.prototype = new PageSection;


	DiscussionSection.prototype.initialize = function(name)
	{
		PageSection.prototype.initialize.call(this, name);
		try {
			// set a flag defining whether this task's section is modifiable
			this.readOnly = (this.parent.isTaskCompleted || !this.parent.isChangeable);
		
	    if (this.msgNumber == 0) {
	    	// when there are no messages in discussion:
	  	  if (!this.enabled) {
			   	// hide the whole section if it's disabled by its task type
	  	  	this.setState("HIDE");
	  	  }
	  	  else {
					// slide (collapse) the whole section if there are no messages to show
					this.slide();
				}
			}
		}
		catch (err) {
			alert("Problem in DiscussionSection.initialize() method: " + err);
		}
	};


	DiscussionSection.prototype.show = function()
	{
		try {
			PageSection.prototype.show.call(this);
		}
		catch (err) {
			alert("Problem in DiscussionSection.show() method: " + err);
		}
	};

	/**
	* If the "Reply" (or "Start Discussion") button is clicked, a text area is shown and the button dissapears.
	*/
	DiscussionSection.prototype.replyButtonClicked = function()
	{
		try {
			var replyButton = $("#replyInWcTaskDiscussion");
			var newMessageBody = $("#newDiscussionMessageBodyID");
			
			// first hide the "Reply" button once clicked - it's not allowed to post more than one message at once
			replyButton.hide();
			
			// now show the text area to enter the new discussion message slowly (animated)
			newMessageBody.slideDown("fast");
		}
		catch (err) {
			alert("Problem in PageSection.slide() method: " + err);
		}
		
	};


	DiscussionSection.prototype.print = function()
	{
		try {
			// open a new window with the whole wctask discussion in print-friendly format
			var url = "projdetailswctaskdetailsthreadprint.jsp?curProjectThreadID=" + this.threadID 
					+ "&projectID=" + this.parent.projectID + "&curProjTaskID=" + this.parent.taskID;
			window.open(url);
		}
		catch (err) {
			alert("Problem in DiscussionSection.print() method: " + err);
		}
	};

	return DiscussionSection;
	
});