define(["wcr/webcenter_html",
        "./PageSection",
        "jquery",
        "wcr/AddDocument",
        "wcr/DocumentsTable",
        "wcr/validate"], function (WCHTML, PageSection, $) {
	
	/**
	*	GeneralTaskInfoSection constructor (subclass of PageSection)
	* 
	* PageSection serves as a generic superclass of various page sections used within task execution page
	* and others.
	*/
	function GeneralTaskInfoSection(sectionID, bodyID, data)
	{
		try {
			// call superclass constructor with parameters containing IDs of main elements of the section
			PageSection.call(this, sectionID, null, bodyID, null);
			
	/*	
			// call super.setState(state) to store given section state
			PageSection.prototype.setState.call(this, state);
	*/	

			// flag whether current task can be started manually by user (or just automatically)
			this.allowManualStart = data.allowManualStart == "1";
			// the ID of task status saved in DB. it doesn't change until the whole execution page is submitted.
			this.origTaskStatusID = data.origTaskStatusID;
			// the due date of task currently saved in DB
			this.origDueDateMS = data.origDueDateMS;
			if (isNaN(this.origDueDateMS)) this.origDueDateMS = 0;
			
			// actual due date displayed in GUI (is changed immediately when user set a new due date)
			this.dueDateMS = this.origDueDateMS;
		}
		catch (err) {
			alert("Problem in GeneralTaskInfoSection constructor: " + err);
		}
	}

	// define GeneralTaskInfoSection as a subclass of PageSection
	GeneralTaskInfoSection.prototype = new PageSection;


	GeneralTaskInfoSection.prototype.initialize = function(name)
	{
		PageSection.prototype.initialize.call(this, name);
	};


	GeneralTaskInfoSection.prototype.show = function()
	{
		try {
			PageSection.prototype.show.call(this);
		}
		catch (err) {
			alert("Problem in GeneralTaskInfoSection.show() method: " + err);
		}
	};


	GeneralTaskInfoSection.prototype.validateFormParams = function()
	{
		try {
	/*//JANH task name is read-only in the latest version of TEP!
			if (this.parent.isChangeable) {
			  // check to make sure the task type name is not empty
			  if (isWhitespace($("input[name=taskName]").val()))
			  {
			    alert(this.parent.langArr.BADPROJECTWCTASKNAME);
			    return false;
			  }
	  	}
	*/
			// verify validity of task status change
			if (!this.validateTaskStatusChange()) return false;

		  // check due date 
		  if (!this.validateDueDate()) return false;
		  // and estimated date (if entered)
		  if (!this.validateEstimatedDate()) return false;
		
		  return true;
		}
		catch (err) {
			alert("Problem in GeneralTaskInfoSection.validateFormParams() method: " + err);
		}
	};


	GeneralTaskInfoSection.prototype.validateTaskStatusChange = function()
	{
		try {
			if (this.parent.isStatusEditable) {
				if (!this.allowManualStart) {
					// it's not allowed to start task manually
					if ((this.origTaskStatusID == "00001_0000000001") && 
							(this.origTaskStatusID != $('#taskStatusInput').val())) {
						alert(this.parent.langArr.TASK_MANUAL_START_NOT_ALLOWED_SHORT);
						return false;
					}
				}
			}
			return true;
		}
		catch (err) {
			alert("Problem in GeneralTaskInfoSection.validateTaskStatusChange() method: " + err);
		}
	};


	GeneralTaskInfoSection.prototype.validateDueDate = function()
	{
		try {
			if (!this.parent.taskExecution && this.parent.isChangeable) {
				// if task status is set to "Completed", any notification about overdue does not make sence
				if ($("#taskStatusInput").val() == '00001_0000000003') return true;
				
				if ((this.dueDateMS == 0) || IsEmptyDateSelection('due_date', 'due_periodOfDay'))
				{
					alert(this.parent.langArr.PROJ_TASK_DUEDATE_REQUIRED);
					return false;
				}
				
				// there was no action done by user that effected due date - skip the rest of due date checks
				if (this.dueDateMS == this.origDueDateMS) return true;
				
				var curDateMS = (new Date()).getTime();   // current date in millisenconds
				
				// due date should not be set to past
				if (curDateMS > this.dueDateMS) {
					return confirm(this.parent.langArr.PROJ_TASK_DUEDATE_IN_PAST);
				}
			}
			return true;
		}
		catch (err) {
			alert("Problem in GeneralTaskInfoSection.validateDueDate() method: " + err);
		}
	};

	GeneralTaskInfoSection.prototype.validateEstimatedDate = function()
	{
		try {
			if (this.parent.isChangeable && ($("#estimated_date").length > 0)) {
				var dateVal = $("#estimated_date").val();
				var dateValDeleted = (dateVal == "0NaN/0NaN/NaN");
				if (dateVal == "0NaN/0NaN/NaN") dateVal = "";
				var periodOfDayVal = $("#estimated_periodOfDay").val();
				if (periodOfDayVal == "-1") periodOfDayVal = "";
				
				// check whether the estimated date is fully set. it's not possible to set only time, 
				// but not date part or vice versa (note time is set by default to 18:00 when a date is picked).
				if (((dateVal.length == 0) && (periodOfDayVal.length > 0) && !dateValDeleted) ||
						((dateVal.length > 0) && (periodOfDayVal.length == 0))) {
					alert(this.parent.langArr.ESTIMATED_DATE_NOT_COMPLETE);
					return false;
				}
			}
			return true;
		}
		catch (err) {
			alert("Problem in GeneralTaskInfoSection.validateDueDate() method: " + err);
		}
	};


	/*
	* dueDateChanged(elem, timeElemID) handles date field changes and its counterpart
	* dueDateChangedTime(elem, dateElemeID) handles corresponding time field changes.
	*/
	GeneralTaskInfoSection.prototype.dueDateChanged = function(elem, timeElemID)
	{
		try {
			var d = Date.parse(elem.value);
			var date = new Date(d);
			
			if (timeElemID && (timeElemID.length > 0)) {
				var time = $("#" + timeElemID).val();
				date.setHours(Math.floor(time / 2));
				date.setMinutes((time % 2) * 30);
			}
			this.dueDateMS = date.getTime();

			// mark this section as modified
			this.setModified(true);
		}
		catch (err) {
			alert("Problem in GeneralTaskInfoSection.dueDateChanged() method: " + err);
		}
	};

	/*
	* dueDateChanged(elem, timeElemID) handles date field changes and its counterpart
	* dueDateChangedTime(elem, dateElemeID) handles corresponding time field changes.
	*/
	GeneralTaskInfoSection.prototype.dueDateChangedTime = function(elem, dateElemID)
	{
		try {
			var d = Date.parse($("#" + dateElemID).val());
			var date = new Date(d);

			var time = elem.value;
			date.setHours(Math.floor(time / 2));
			date.setMinutes((time % 2) * 30);

			this.dueDateMS = date.getTime();

			// mark this section as modified
			this.setModified(true);
		}
		catch (err) {
			alert("Problem in GeneralTaskInfoSection.dueDateChanged() method: " + err);
		}
	};

	/*
	* These two methods below handle date field and time field changes of estimated date.
	*/
	GeneralTaskInfoSection.prototype.estimatedDateChanged = function(elem, id)
	{
		// the only needed action is to mark this section as modified
		this.setModified(true);
	};

	GeneralTaskInfoSection.prototype.estimatedDateChangedTime = function(elem, id)
	{
		// the only needed action is to mark this section as modified
		this.setModified(true);
	};

	return GeneralTaskInfoSection;
	
});