define(["underscore", "wcr/sdrop"], function (_) {
	"use strict";
	/**
	 * @class
	 * Task Assignee Data Provider - the elements are sorted in the same order as they are written in the XML.
	 * 
	 * 
	 */
	var TaskAssigneeDataProvider = new Class({
		Extends: SDropDataProvider,
		options: {
			options : {},
					
			sort: function (a, b) {
				if (a.order < b.oder ) {
					return -1;
				} else if (a.order > b.oder ) {
					return 1;
				} else {
					return 0;
				}	
			},			
		
			promise: null
		},
		
		getData : function(patternText, truncate) {
			return this.datastore
					.filter(function(item, index) {
						return this.regex.test(item.name);
					}, this.registerPattern(patternText))
					.sort(function(a, b) {
						if (a.order != undefined){
							if (a.order< b.order) {
								return -1;
							} else if (a.order > b.order) {
								return 1;
							} else {
								return 0;
							}							
						}
						return 0;
					});
		},
		
		// Parse the XML and add the elements to the resultlist
		parseXML: function(pattern, xml){ 
			var resultList = [];

			try {
				var elements = xml.getElementsByTagName("entry");
				if (elements) {
					for ( var i = 0; i < elements.length; i++) {
						// read all the elements, some will be null
						var roleName 	= WCXML.GetElementText(elements[i], "role_name") ;
						var roleID 		= WCXML.GetElementText(elements[i], "role_id") ;
						var groupName 	= WCXML.GetElementText(elements[i], "group_name") ;
						var groupID 	= WCXML.GetElementText(elements[i], "group_id") ;
						var userName 	= WCXML.GetElementText(elements[i], "user_name") ;
						var userID 		= WCXML.GetElementText(elements[i], "user_id") ;
						var type 		= WCXML.GetElementText(elements[i], "type") ;
						var isPM  		= WCXML.GetElementText(elements[i], "isPM") ;
						var level = 0;
						var name ='';
						var id ='';
						
						// Check the role to see where we can take the name from
						if (type == 'CURRENT_USER'){  // Current user is a special case
							name = this.options.langArr.ME  ;  
							id = 'current_user' ;
						}
						else if (type == 'R'){ 
							name = roleName + ' ' + '<'+this.options.langArr.ROLE+ '>' ; 
							id = type + roleID ;
						}
						else if (type == 'G' ){
							if ( roleName != ''){  
								level=1 ;
							}
							name += groupName + ' ' + '<'+this.options.langArr.GROUP+ '>' ;  
							id = type + groupID;
						}
						else if (type == 'U'){
							if ( groupName != '' || roleName != ''){ 
								level=1;
							}
							name += userName ;
							// if the role AND group name are not null, this is a user role assignment via group,
							// Reflect this in the name of the user
							if ( groupName != '' && roleName != ''){
								name += ' @ ' + groupName + ' ' + '<'+this.options.langArr.GROUP+ '>' ;  	
							} else{
								name +=  ' ' + '<'+this.options.langArr.USER+ '>'; 
							}
								
							id = type + userID ;
						}
						
						// we want to indicate the user/group that is the PM of the project
						if (isPM ){
							name += ' ' + '<'+ this.options.langArr.PM+ '>' ;
							level=0;
						}
						
						if (name != undefined){
							if (console && console.log)
								console.log('name ' + name + ' id : '+ id + ' order ' + i);
							
							// Ensure that the IDs are unique. SDrop does not work well with non-unique ids
							if (id != 'current_user'){
								id = id + '_' + i ;
							}
							
			                resultList.push({	'name' : name, 'id' : id , 
												'type' : type, 'order' : i,
												'cssClass': level > 0?'result-indent':''});							
						}
					}
				}
			} catch (exception) {
				this.loadFailed(pattern, "ParseXML failed: " + exception);
				return false;
			}
			
			return resultList ;
		},
		
		itemHasID: function(id, item){
			var itemId = item.id;			
			return id == itemId.substring(0, itemId.lastIndexOf("_"));
		}
	});

	return TaskAssigneeDataProvider;

});