

define('pages/wctaskexecution/PageSection',["wcr/webcenter_html",
        "jquery",
        "wcr/AddDocument",
        "wcr/DocumentsTable",
        "wcr/validate"], function (WCHTML, $) {

	
	
	/**
	*	PageSection constructor
	* 
	* PageSection serves as a generic superclass of various page sections used within task execution page
	* and others.
	*/
	function PageSection(sectionID, expandToggleID, bodyID, state)
	{
		try {
			// set the initial state of the page section (possible values: SHOW, HIDE, COLLAPSE, REQUIRED, ERROR)
			// default for page section is that it's shown (expanded)
			if (!state || (state.length == 0)) state = "SHOW";
			this.setState(state);
			
			// page or other section inside which this section resides
			this.parent = null;
			// flag showing whether any item owned by the section was changed 
			// note it doesn't include state of other sections - they should have their own modified flag!
			this.modified = false;
			
			// IDs of the whole GUI section, its header and body
			this.sectionID = sectionID;
			this.expandToggleID = expandToggleID;
			this.bodyID = bodyID;
			
			// stores the future state of section, if it will be expanded or collapsed
			this.expand = (this.state != "COLLAPSE");

			// flag to store if the dynamic part of page section was already prepared or not (late initialization)
			this.dynamicContentReady = false;
			
//			try {
//				var sectionBody = $("#" + this.bodyID);
//				sectionBody.prepend('<input type="hidden" name="projectThreadID" value=""/>');
//			}
//			catch (ignoredError) {}
		}
		catch (err) {
			alert("Problem in PageSection constructor: " + err);
		}
	}


	/**
	* Method to initialize page section after it's added to task execution page.
	* To be overridden, but this basic version should be always called at first.
	*/
	PageSection.prototype.initialize = function(name)
	{
		this.name = name;
	};


	/**
	* General method to display the section GUI
	*/
	PageSection.prototype.show = function()
	{
		var section = $("#" + this.sectionID);
		var sectionBody = $("#" + this.bodyID);

		section.hide();

		if (this.state == "COLLAPSE") {
			// this is the same as SHOW/REQUIRED, but the initial this.expand state is false (== collapsed), not true
			// and thus there is no need to call this.generateDynamicContent() method...
		}
		else if ((this.state == "SHOW") || (this.state == "REQUIRED")) {
			this.generateDynamicContent();
		}
		else {
			sectionBody.html("Error in section " + this.bodyID + ".");
		}
		
		if (this.state != "HIDE") {
			section.show();
			this.slide(false);
		}
		// (no catching of exceptions here - try/catch block should be in show() methods of subclasses, 
		// from which this method is called !)
	};


	PageSection.prototype.setState = function(state) 
	{
		try {
			// possible values of page section state: SHOW, HIDE, COLLAPSE, REQUIRED, ERROR
			if ((state != "SHOW") && (state != "HIDE") && (state != "COLLAPSE") && 
					(state != "REQUIRED") && (state != "ERROR")) state = "HIDE";
			this.state = state;
		}
		catch (err) {
			alert("Problem in PageSection.setState() method: " + err);
		}
	};


	PageSection.prototype.setModified = function(modified) 
	{	
		try {
			// calling this method without parameter is the same as calling it with parameter == true
			if ((typeof modified == 'undefined') || (modified == null)) modified = true;
			this.modified = modified;
			// when page section changed, invalidate the dynamic content by default
			if (modified) this.dynamicContentReady = false;
		}
		catch (err) {
			alert("Problem in PageSection.setModified() method: " + err);
		}
//		if really modified, display the "change" button and the rest...
		taskExPage.checkIfModifiedAndUpdateButtons();
	};

	PageSection.prototype.isModified = function() 
	{
		return this.modified;
	};


	/**
	* Function to expand and collapse page sections.
	*/
	PageSection.prototype.slide = function(switchStatus)
	{
		try {
			// the page section doesn't support collapsing if expand toggle ID is not set
			if (!this.expandToggleID || (this.expandToggleID.length == 0)) return;
			
			var headerExpander = $("#" + this.expandToggleID);
			var body = $("#" + this.bodyID);

			// invert the state of page section expansion whenever it's not explicitly said not to do it
			if ((switchStatus == null) || switchStatus) this.expand = !this.expand;

			if (this.expand) {
				// prepare the content of the section first
				this.generateDynamicContent();
				if (headerExpander) {
					headerExpander.attr("class", "icon icon-normal icon-WCR_Down");
				}
				// make the section content visible (animated)
				body.slideDown("fast");
			} 
			else {
				if (headerExpander) {
					headerExpander.attr("class", "icon icon-normal icon-WCR_Right");
				}
				body.slideUp("fast");
			}
		}
		catch (err) {
			alert("Problem in PageSection.slide() method: " + err);
		}
	};


	//JANH not used!!!
	/**
	* Method to reset GUI of the section.
	* ??? To be overridden ???
	*/
	PageSection.prototype.reset = function()
	{
		// dynamic content of the page section will be regenerated
		this.dynamicContentReady = false;
	};

	//JANH not used!!!
	/**
	* A generalized method to set all needed/known data related to the section.
	* The input parameter is expected to be an array of (identifier, value) pairs.
	* To be overridden.
	*/
	PageSection.prototype.setSectionData = function(data)
	{
		alert("PageSection.setSectionData() should be implemented in subclass");
	};

	/**
	* Method to validate data entered by user in particular section.
	* To be overridden.
	*/
	PageSection.prototype.validateFormParams = function()
	{
		return true;
	};

	/**
	* Method to generate dynamic content of a page section.
	* To be overridden.
	*/
	PageSection.prototype.generateDynamicContent = function()
	{
		// by default there is no dynamic content in page section
	};
	
	return PageSection;
	
});


define('pages/wctaskexecution/TaskAssigneeDataProvider',["underscore", "wcr/sdrop"], function (_) {
	"use strict";
	/**
	 * @class
	 * Task Assignee Data Provider - the elements are sorted in the same order as they are written in the XML.
	 * 
	 * 
	 */
	var TaskAssigneeDataProvider = new Class({
		Extends: SDropDataProvider,
		options: {
			options : {},
					
			sort: function (a, b) {
				if (a.order < b.oder ) {
					return -1;
				} else if (a.order > b.oder ) {
					return 1;
				} else {
					return 0;
				}	
			},			
		
			promise: null
		},
		
		getData : function(patternText, truncate) {
			return this.datastore
					.filter(function(item, index) {
						return this.regex.test(item.name);
					}, this.registerPattern(patternText))
					.sort(function(a, b) {
						if (a.order != undefined){
							if (a.order< b.order) {
								return -1;
							} else if (a.order > b.order) {
								return 1;
							} else {
								return 0;
							}							
						}
						return 0;
					});
		},
		
		// Parse the XML and add the elements to the resultlist
		parseXML: function(pattern, xml){ 
			var resultList = [];

			try {
				var elements = xml.getElementsByTagName("entry");
				if (elements) {
					for ( var i = 0; i < elements.length; i++) {
						// read all the elements, some will be null
						var roleName 	= WCXML.GetElementText(elements[i], "role_name") ;
						var roleID 		= WCXML.GetElementText(elements[i], "role_id") ;
						var groupName 	= WCXML.GetElementText(elements[i], "group_name") ;
						var groupID 	= WCXML.GetElementText(elements[i], "group_id") ;
						var userName 	= WCXML.GetElementText(elements[i], "user_name") ;
						var userID 		= WCXML.GetElementText(elements[i], "user_id") ;
						var type 		= WCXML.GetElementText(elements[i], "type") ;
						var isPM  		= WCXML.GetElementText(elements[i], "isPM") ;
						var level = 0;
						var name ='';
						var id ='';
						
						// Check the role to see where we can take the name from
						if (type == 'CURRENT_USER'){  // Current user is a special case
							name = this.options.langArr.ME  ;  
							id = 'current_user' ;
						}
						else if (type == 'R'){ 
							name = roleName + ' ' + '<'+this.options.langArr.ROLE+ '>' ; 
							id = type + roleID ;
						}
						else if (type == 'G' ){
							if ( roleName != ''){  
								level=1 ;
							}
							name += groupName + ' ' + '<'+this.options.langArr.GROUP+ '>' ;  
							id = type + groupID;
						}
						else if (type == 'U'){
							if ( groupName != '' || roleName != ''){ 
								level=1;
							}
							name += userName ;
							// if the role AND group name are not null, this is a user role assignment via group,
							// Reflect this in the name of the user
							if ( groupName != '' && roleName != ''){
								name += ' @ ' + groupName + ' ' + '<'+this.options.langArr.GROUP+ '>' ;  	
							} else{
								name +=  ' ' + '<'+this.options.langArr.USER+ '>'; 
							}
								
							id = type + userID ;
						}
						
						// we want to indicate the user/group that is the PM of the project
						if (isPM ){
							name += ' ' + '<'+ this.options.langArr.PM+ '>' ;
							level=0;
						}
						
						if (name != undefined){
							if (console && console.log)
								console.log('name ' + name + ' id : '+ id + ' order ' + i);
							
							// Ensure that the IDs are unique. SDrop does not work well with non-unique ids
							if (id != 'current_user'){
								id = id + '_' + i ;
							}
							
			                resultList.push({	'name' : name, 'id' : id , 
												'type' : type, 'order' : i,
												'cssClass': level > 0?'result-indent':''});							
						}
					}
				}
			} catch (exception) {
				this.loadFailed(pattern, "ParseXML failed: " + exception);
				return false;
			}
			
			return resultList ;
		},
		
		itemHasID: function(id, item){
			var itemId = item.id;			
			return id == itemId.substring(0, itemId.lastIndexOf("_"));
		}
	});

	return TaskAssigneeDataProvider;

});

define('pages/wctaskexecution/WcTaskExecutionPage',["wcr/webcenter_html",
        "./PageSection",
        "utils/EventBasedValidator",
        "jquery",
        "underscore",
        "component/passwordpopup/PasswordValidationPopup",
        "pages/wctaskexecution/TaskAssigneeDataProvider",
        "wcr/AddDocument",
        "wcr/DocumentsTable",
        "wcr/validate","wcr/sdrop"
        ], 
function (WCHTML, 
		PageSection, 
		EventBasedValidator, 
		$,
		_,
		PasswordValidationPopup,
		TaskAssigneeDataProvider) {
	
	
	function WcTaskExecutionPage(data, langStrings)	{
		
		try {
			// ID of currently displayed project task
			if (!data.taskID || (data.taskID.length == 0)) {
				alert("No task to process.");
				return;
			}
			this.taskID = data.taskID;
			this.projectID = data.projectID;
			this.loggedInUserID = data.loggedInUserID;
			this.loggedInUserFullName = data.loggedInUserFullName;
		
			this.isProjectActive = data.isProjectActive;
			this.isTaskEditable = data.isTaskEditable;
			this.isChangeable = data.isChangeable;
			this.isStatusEditable = data.isStatusEditable;
			this.canChangeAssignee = data.canChangeAssignee;
			this.isTaskStarted = data.isTaskStarted;
			this.isTaskCompleted = data.isTaskCompleted;
			// variable for debug purposes - fast switch between task execution and editation modes
			this.taskExecution = data.taskExecution;
			
			// an object to store all inner page sections of task execution page
			this.pageSections = new Object();
			
			// array of language strings read from input XML
			this.langArr = new Array();
			this.setLanguageStrings(langStrings);
			
			// by default, attempts to leave the page will throw a confirm box if the page is modified
			this.exitConfirmNeeded = true;

			//need to omit "leave page" message in the case od document download
			this.downloaDoc = false;
			//when no changes are made, change btn is disabled and Cancel/Return button behaves as cancel. This is to notify the state of button, thus avoid changing it(enabling change btn and change return->cancel btn) if it is already changed
			this.changeBtnEnabled = false;

//			events for various inputs - to detect when to switch page to "changed mode"-> display "change" button and change "return" to "cancel" button
			this.eventNames = "change.changeDetect select.changeDetect keyup.changeDetect";
//			elements needed to catch the previous events for
			this.eventSourceSelector = "textarea, select, input, div";
			
//			special event for image element - don't want to handle click event for any other element, like input - no need, that just consumes resources
			this.eventName_Img = "click.changeDetect";
//			the img element :)
			this.eventSourceSelector_Img = "img";
			
			// confirmExit is a method called whenever user tries to get away from the page
			var thisObj = this;
			window.onbeforeunload = function() {return thisObj.confirmExit();};
		}
		catch (err) {
			alert("Problem in WcTaskExecutionPage constructor: " + err);
		}
	}

	/**
	* Method to set translation strings
	*/
	WcTaskExecutionPage.prototype.setLanguageStrings = function(langStrings) {
		try {
			if (!langStrings) {
				return;
			}
			
			for (var key in langStrings) {
				if (langStrings.hasOwnProperty(key)){
					this.langArr[key] = langStrings[key];
				}
			}
		} catch (err) {
			alert("Problem in WcTaskExecutionPage.setLanguageStrings() method: " + err);
		}
	};

	/**
	* Method to set the SDrop for the Task Assignee
	*/
	WcTaskExecutionPage.prototype.setSDrop = function(sdrop)
	{
		this.SDrop = sdrop; 		
	};

	/**
	*	Method to add a new page section to task execution page under a given name (overwriting is not checked).
	*/
	WcTaskExecutionPage.prototype.addSection = function(section, name)
	{
	/*	
				genTaskInfo : genInfoSection,
				documents : documentsSection,
				checklist : checklistSection,
				attributes : attributesSection,
				discussion : discussionSection
	*/
		try {
			// perform basic initialization of the newly added page section
			section.parent = this;
			section.initialize(name);
			
			// store it in pageSections object under given name
			this.pageSections[name] = section;
		}
		catch (err) {
			alert("Problem in WcTaskExecutionPage.addSection() method: " + err);
		}
	};

	/**
	*	Method to remove a page section from task execution page with given name.
	*/
	WcTaskExecutionPage.prototype.removeSection = function(name)
	{
		try {
			// if exists remove the section property from pageSections object completely
			if (this.pageSections[name]) delete this.pageSections[name];
		}
		catch (err) {
			alert("Problem in WcTaskExecutionPage.removeSection() method: " + err);
		}
	};


	WcTaskExecutionPage.prototype.show = function() {
		try {
			var numberOfVisibleSections = 0;
			for (var item in this.pageSections) {
				if (this.pageSections.hasOwnProperty(item)){
					var pItem = this.pageSections[item];
					if (pItem) {
						pItem.show();
						if (pItem.state != "HIDE") {
							numberOfVisibleSections++;
						}
					}
				}
			}
			// non of "hidable" sections is visible (general info section is visible always) => just a simple 
			// task without docs, checklist, etc. is shown...
			// in this case hide the bottom row of buttons
			if (numberOfVisibleSections > 1) {
				$("#lowerButtonRow").show();
			} else {
				$("#lowerButtonRow").hide();
			}
		} catch (err) {
			alert("Problem in WcTaskExecutionPage.show() method: " + err);
		}
	};

	WcTaskExecutionPage.prototype.reset = function() {
		try {
			for (var item in this.pageSections) {
				if (this.pageSections.hasOwnProperty(item)){
					var pItem = this.pageSections[item];
					if (pItem) {
						pItem.reset();
					}
				}
			}
		} catch (err) {
			alert("Problem in WcTaskExecutionPage.reset() method: " + err);
		}
	};


	WcTaskExecutionPage.prototype.isModified = function() {
		try {
			for (var item in this.pageSections) {
				if (this.pageSections.hasOwnProperty(item)){
					if (this.pageSections[item].isModified()) {
						return true;
					}
				}
			}
		} catch (err) {
			alert("Problem in WcTaskExecutionPage.isModified() method: " + err);
		}
		return false;
	};

	/**
	 * Checks various events(cliciing image, changing select, type to an input, ...) and checks, if the page changed. If so, then unregister the events and change the page to "changed mode"-> display "change" button and change "return" to "cancel" button
	 * @param a - specifically for input element that is created a outside of this page - so to be noticed, when the content of the input changed right after some letter was typed
	 */
	WcTaskExecutionPage.prototype.checkIfModifiedAndUpdateButtons = function(a) 
	{
	/*	the text value is used to test if some text was typed in some ajax inserted input(or input created in some template, where is no desire to modify its event firing) - that is right after pressing some letter. 
		Don't want to wait until "onchanged" event is fired, because that is fired after loosing focus from that input and that is too late */
		var somethingModified = false;
		if (typeof a != 'undefined' && 'target' in a && 'defaultValue' in a.target && 'value' in a.target && a.target.defaultValue !== a.target.value) {
			somethingModified = true;
		}
			
		if ((this.isModified() || somethingModified)&& !this.changeBtnEnabled ) {
			this.updateButtons();
		} 
	};

	WcTaskExecutionPage.prototype.updateButtons = function() 
	{
		$(this.eventSourceSelector).off(this.eventNames);
		$("[name='cancelButton']").val(this.langArr.cancel);
		var changeBtns = $("[name='createWCTask']");
		changeBtns.prop("disabled", false);
		changeBtns.show();
		this.changeBtnEnabled = true;
	};

	/**
	 * This will register all kinds of events - so It can be tested, whether the page has changed and so to change the page to "changed mode" -> display "change" button and change "return" to "cancel" button
	 */
	WcTaskExecutionPage.prototype.registerEventToDetectChanges = function() 
	{
		$(this.eventSourceSelector).on(this.eventNames, this.checkIfModifiedAndUpdateButtons.bind(this));
		$(this.eventSourceSelector_Img).on(this.eventName_Img, this.checkIfModifiedAndUpdateButtons.bind(this));
		
		if(this.pageSections.attributes.myAttrCat){
			this.pageSections.attributes.myAttrCat.valueChangedListeners.push(function() {
				this.updateButtons();
			}.bind(this));
		}
		
		require(["utils/WCEvent"],function(WCEvent){
			WCEvent.on("btnClicked.UploadNewVersion.*", function(){
				this.updateButtons();
			}.bind(this));
		}.bind(this));
	};

	/** 
	* Callback method that is called whenever user navigates away from the page
	* (this == window in this method due to it's a callback!)
	*/
	WcTaskExecutionPage.prototype.confirmExit = function() 
	{
		try {
			if (this.exitConfirmNeeded && this.isModified() && !this.downloaDoc) {		  	
				return this.langArr.PAGE_EXIT_CONFIRM;
			} 
		}
		catch (err) {
			alert("Problem in WcTaskExecutionPage.confirmExit() callback method: " + err);
		}
		//reset the flag, that announces document download => avoiding leave page message
		this.downloaDoc = false;
	};

	WcTaskExecutionPage.prototype.changeButtonClicked = function(force)
	{
		try {
			var isPasswordRequired = false;
			
			for(var item in this.pageSections) {
				var pItem = this.pageSections[item];
				
				// skip empty and not modified sections
				if (pItem && pItem.isModified() && pItem.docTableObj && pItem.docTableObj.checkIfPasswordRequired) {
					// (note checkIfPasswordRequired() is defined only when PCM license is active!)
					isPasswordRequired = pItem.docTableObj.checkIfPasswordRequired();
					
					if(isPasswordRequired) {
						break;
					}
				}
			}
			
			// If password is required - open popup to get password from user , save otherwise
			if(isPasswordRequired) {
				this.passwordValidationPopup = new PasswordValidationPopup();
				this.passwordValidationPopup.onSubmit(_.bind(function() {
					this.validateAndSaveChanges(force);
				}, this, force));
				this.passwordValidationPopup.open();
			} else {
				this.validateAndSaveChanges(force);
			}
			
		} catch (err) {
			alert("Problem in WcTaskExecutionPage.changeButtonClicked() method: " + err);
		}
		
	};
	
	WcTaskExecutionPage.prototype.validateAndSaveChanges = function(force) {
		try {
			// 'modSections' string is used to store space-delimited list of names of modified sections 
			var modSections = "";
			var paramsValid = true;
			
			for (var item in this.pageSections) {
				if (this.pageSections.hasOwnProperty(item)){
					var pItem = this.pageSections[item];
					// skip empty and not modified sections unless the force parameter is set (true) - that
					// is in case of completing the task only!
					if (pItem && (pItem.isModified() || force)) {
						// remember actual section as modified
						modSections += pItem.name + " ";
						// check whether all required fields are filled and values are acceptable
						if (!pItem.validateFormParams(force)) {
							paramsValid = false;
							break;
						}
					}
				}
			}
			
			// when nothing was changed...
			if (modSections.length == 0) {
				alert(this.langArr.NO_CHANGES_MADE);
				paramsValid = false;
			}
			
			// return back without submitting the form because one or more parameters filled by user is/are invalid
			// the appropriate warning/error message is displayed within corresponding 
			// PageSection.validateFormParams() method
			if (!paramsValid) {
				return;
			}
			
			// check unique document names
			var evInst = EventBasedValidator.getInstance();
        	var uniqueNamesObj = evInst.createUniqueNamesObj();
        	evInst.validateNamesGlobally(uniqueNamesObj);
			if(!uniqueNamesObj.isOk()){
				return false;
			}
			
			// store info about what sections are modified to designated input field to be passed to TaskActionsHelper.java
			$("#modifiedSections").val(modSections);
			// finally set the action value and submit the whole page
			$("#actionValId").val("ChangeTaskOnExecutionPage");
			
			// do not prompt user to confirm leaving of the page
			this.exitConfirmNeeded = false;
			
			$("#actionformId").submit();
			
			return true;
		}
		catch (err) {
			alert("Problem in WcTaskExecutionPage.changeButtonClicked() method: " + err);
		}
	};

	/**
	 * Assign To Me action:
	 * initialize the SDrop to 'Me' and recreate the SDROP
	 */
	WcTaskExecutionPage.prototype.assignToMeButtonClicked = function(){
		
		// assign to me option
		var taskAssigneeDP =  this.SDrop.dataProvider;
		var sdropOptions = this.SDrop.options;
		
		taskExPage.pageSections.genTaskInfo.modified = true;
		
		// Destroy the current SDrop and initialize the drop down with the 'Me' value
		this.SDrop.destroy(); 
		$("#taskAssigneeSDrop").empty().append($("<option>", {selected: true, value: "current_user", text: this.langArr['ME']})).val("current_user");
		
		// recreate the new SDrop
		this.SDrop = new SDrop($("#taskAssigneeSDrop"), null, taskAssigneeDP, sdropOptions);
	};
	
	
	WcTaskExecutionPage.prototype.cancelButtonClicked = function()
	{
		try {
			$("#actionValId").val("GoBackFromTaskExecutionPage");

			// do not prompt user to confirm leaving of the page
			this.exitConfirmNeeded = false;
			
			//on this depends, if the page is gonna be redirected back to source page(if no changes are made) or stay on TEP and revert changes(if changes are made)
			$("#pageModifiedId").val(this.isModified()?"1":"0");

			$("#actionformId").submit();
		}
		catch (err) {
			alert("Problem in WcTaskExecutionPage.cancelButtonClicked() method: " + err);
		}
	};

	WcTaskExecutionPage.prototype.startButtonClicked = function()
	{
		try {
			if (confirm(this.langArr.SAVE_AND_START_WCTASK_CONFIRM)) {
				var statusOptions = $("#taskStatusInput option");
				// (the first status of started task is always second in the task status select box - first is "Created")
				statusOptions[1].selected = true;
				// mark the General Info section as modified, because the changed task status is part of it
				this.pageSections.genTaskInfo.modified = true;
				// can't easily submit here because other wctask params could be changed too
				// (changeButtonClicked() method incorporates all validation etc. including submit...)
				this.changeButtonClicked();
			}
		}
		catch (err) {
			alert("Problem in WcTaskExecutionPage.startButtonClicked() method: " + err);
		}
	};

	WcTaskExecutionPage.prototype.completeButtonClicked = function()
	{
		try {
			if (confirm(this.langArr.SAVE_AND_COMPLETE_WCTASK_CONFIRM)) {
				var previousStatus = $("#taskStatusInput").val();
				$("#taskStatusInput").val("00001_0000000003");
				
				// mark the General Info section as modified, because the changed task status is part of it
				this.pageSections.genTaskInfo.modified = true;
				
				// can't easily submit here because other wctask params could be changed too
				// (changeButtonClicked() method incorporates all validation etc. including submit...)
				var result = this.changeButtonClicked(true);
				if(!result){
					$("#taskStatusInput").val(previousStatus);
				}
			}
		}
		catch (err) {
			alert("Problem in WcTaskExecutionPage.completeButtonClicked() method: " + err);
		}
	};


	WcTaskExecutionPage.prototype.editTaskDetailsClicked = function()
	{
		try {
			// set the hidden inputs controlling the outcome of submitting the page to redirect to task details page
			$("input[name=projectTaskID]").val(this.taskID);   //JANH is this necessary???
			$("#actionValId").val("ViewTaskDetails");

			if ($("#actionformId").length > 0) {
				try {				
			$("#actionformId").submit();
		}
				catch (IEerr) {
					// When a page loading action initiated by JavaScript is cancelled by the onbeforeunload event
					// IE will throw an exception. Too bad there's no way to identify this specific exception
					// so ignore exceptions on form submission
				}
			}
		}
		catch (err) {
			alert("Problem in WcTaskExecutionPage.editTaskDetailsClicked() method: " + err);
		}
	};


	WcTaskExecutionPage.prototype.showTaskWorkflowClicked = function()
	{
		try {
			// set the hidden inputs controlling the outcome of submitting the page to redirect to task details page
			$("input[name=projectTaskID]").val(this.taskID);   //JANH is this necessary???
			$("#actionValId").val("ViewTaskWorkflow");

//			// do not prompt user to confirm leaving of the page
//			this.exitConfirmNeeded = false;

			$("#actionformId").submit();
		}
		catch (err) {
			alert("Problem in WcTaskExecutionPage.showTaskWorkflowClicked() method: " + err);
		}
	};

	return WcTaskExecutionPage;
});

