define(["jquery",
        "underscore"], 
        function ($, 
        		_) {
	"use strict";
	
	var PasswordValidationService = function () {
		this._isPasswordValidated = false;
		this._password = null;
		
	};
	
	/**
	 * Function to validate the password of the user
	 * 
	 * @param {String} password - encoded password of the user
	 * @param {function} callback - callback to be executed after successful validation of password
	 */
	PasswordValidationService.prototype.validatePassword = function(password, callback) {
		var isValidationSuccessful = false;
		
		$.when($.ajax({
			type : "POST",
			dataType : "xml",
			cache: false,
			url : "ValidateUserCredential.jsp",
			noAjaxParam: false,
			contentType: "application/x-www-form-urlencoded; charset=UTF-8",
			data : "password=" + password})).then(_.bind(function(data, textStatus, jqXHR) {
			if(data) {
				isValidationSuccessful = $(data).find('password_validation').text() === 'true';
				
				if(isValidationSuccessful) {
					this._isPasswordValidated = true;
					this._password = password;
				}
				
				callback(isValidationSuccessful);
			} else {
				// TODO : error handling
			}
		}, this), _.bind(function() {
			// TODO: Error handling
			alert("An error occured in while validating the password");
		}, this));
		
	};
	
	/**
	 * Method to get the validated password
	 * 
	 * @returns {String} - encoded password
	 */
	PasswordValidationService.prototype.getValidatedPassword = function() {
		return this._password;
	};
	
	/**
	 * Method to check if successful password validation is already done
	 * @returns {Boolean}
	 */
	PasswordValidationService.prototype.isPasswordValidated = function() {
		return this._isPasswordValidated;
	};
	
	
	return new PasswordValidationService();
});