define([], function () {
	"use strict";

	/**
	 * A Command is an action that can be undone and redone.
	 * Commands are used by the UndoRedoController
	 *
	 * @class Command
	 *
	 *@param {Function} doCallback - function to execute the command
	 * @param {Function} undoCallback - function to revert the actions of the doCallback
	 * @param {String} description - description of the command
	 */
	var Command = function (doCallback, undoCallback, description) {

		this._do = doCallback;
		this._undo = undoCallback;
		this._description = description;
	};

	/**
	 * Execute the command
	 */
	Command.prototype.executeDo = function () {
		this._do();
	};

	/**
	 * Revert the actions done by the command
	 */
	Command.prototype.executeUndo = function () {
		this._undo();
	};

	/**
	 * Get the description of the command
	 * returns {String}
	 */
	Command.prototype.getDescription = function () {
		return this._description;
	};

	return Command;
});