define([], function(){
    "use strict";
    var DomUtils = {
        /**
         * Dynamically load CSS file
         * 
         * @param {String}   filename - path to css from requireJS base
         * @param {Function} [onload]   - callback function when css is loaded
         */
        loadCss: function(filename, onload){
            this.loadCssFromUrl(requirejs.toUrl(filename), onload); 
        },
    
        /**
         * Dynamically load CSS file
         * 
         * @param {String}   url - url to css
         * @param {Function} [onload]   - callback function when css is loaded
         */
	    loadCssFromUrl: function(url, onload){
	        var sheet = null;
	        var timerfunc = null;
	        var fileref = document.createElement("link");
	        fileref.setAttribute("rel", "stylesheet");
	        fileref.setAttribute("type", "text/css");
	
	
	        timerfunc = function (){
	            for (var i=0;i<document.styleSheets.length;i++){
	                sheet = document.styleSheets[i].href;
	                if(sheet !== null && sheet.substr(sheet.length-url.length) == url){
	                    if (onload) {
	                        onload();
	                        onload = null; // make sure the callback is only called once
	                    }
	                    return;
	                }
	            }
	            setTimeout(timerfunc,50);  
	        };
	
	        if (document.all){ //Uses onreadystatechange for Internet Explorer
	            fileref.attachEvent('onreadystatechange',function() {
	                if(fileref.readyState == 'complete' || fileref.readyState == 'loaded') {
	                    if (onload) {
	                        onload();
	                        onload = null; // make sure the callback is only called once
	                    }
	                }
	
	            });
	        } else {    //Checks if the stylesheet has been loaded every 50 ms for others
	            setTimeout(timerfunc,50);
	        }
	
	        fileref.setAttribute("href", url);	    
	        document.getElementsByTagName("head")[0].appendChild(fileref);       
	    }
	    
	};

    
    return DomUtils;
});