define(["utils/webi18n!validatorStrings",
        "jquery",
        "underscore",
        "utils/WCEvent"], 
function(ValidatorStrings,
         $,
         _,
         WCEvent){
	"use strict";

	var EventBasedValidator = (function() {


		function init() {
			return {
				onValidateUniqueDocName: function(callback){
					WCEvent.on("validation.uniqueDocName", callback);
				},
				removeListener: function(callback){
					WCEvent.off("validation.uniqueDocName", callback);
				},

				///////////////////////
				//specific functions //
				///////////////////////
				
				//////////////////////////////
				// check unique document name:
				//////////////////////////////
				
				// create validation object
				createUniqueNamesObj: function(){
					return {
			    		errors : [],
			    		newDocs: [],
			    		newDocsDOMElems: {}, 
			    		newDocsMetadata: {}, 
			    		excludedProjects: {}, 
			    		localClashFound: false,
			    		emptyDocsFound: false,
			    		addDoc: function(docNameInput, projectID, metadata, elemToHighlight){
			    			if (!elemToHighlight) {
			    				// by default the DOM element to be highlighted in case of an error is the input field itself
			    				elemToHighlight = docNameInput;
			    			}
			    			var inputId = docNameInput.attr('id');
			    			if(!inputId){
			    				inputId = _.uniqueId();
			    				docNameInput.attr('id', inputId);
			    			}
			    			this.newDocsMetadata[inputId] = metadata;
			    			
			    			// register document for backend name validation and check for duplicate documents on the page 
			    	    	if(docNameInput.val()){
			    	    		// do everything in lower case
			    	    		var docName = docNameInput.val().toLowerCase();
			    	    		if(!this.newDocsDOMElems[projectID + '_' + docName]){
			    	        		// no duplicate document found yet. We'll add it to the list of existing docs
			    	    			this.newDocsDOMElems[projectID + '_' + docName] = elemToHighlight;
			    	    			
			    	    			// add for backend check, but only if there's a project and it's not excluded
			    	    			if(projectID && !this.excludedProjects[projectID]){
			    	    				this.newDocs.push({docName: docName, projectId: projectID});
			    	    			}
			    	        	} else {
			    	        		// document is already added to this project on this page, we have a duplicate
			    	        		// we'll add the element with the other one, so operations (like highlight) will affect both elements
			    	        		this.newDocsDOMElems[projectID + '_' + docName].push(elemToHighlight[0]);
			    	        		
			    	        		// we only want to add the error message once, as it is a general message (not parametrized)
			    	        		if(!this.localClashFound){
			    	        			// add error object, we want the validation to fail
				    	        		var errorObj = {
			    	        				msg : ValidatorStrings.t('PROJ_CREATION_SPECIFY_UNIQUE_DOCNAMES'),
			    	        				element : this.newDocsDOMElems[projectID + '_' + docName]
			    	        			};
				    	        		this.errors.splice(0,0,errorObj);
				    	        		this.localClashFound = true;
			    	        		}
			    	        	}
			    	    	} else {
			    	    		if(!this.emptyDocsFound){ // only add error once
			    	    			var errorObj = {
			    	        				msg : ValidatorStrings.t('INVALIDDOCNAMEMSG'),
			    	        				element : elemToHighlight
			    	        			};
					    	    		this.emptyDocsFound = true;
				    	        		this.errors.splice(0,0,errorObj);
			    	    		}
			    	    	}
			    		},
			    		excludeProject: function(projId){
			    			this.excludedProjects[projId] = true;
			    		},
			    		isOk: function(){
			    			return this.errors.length == 0;
			    		},
						getErrorMetadata: function(error){
							var res = [];
							_.each(error.element, _.bind(function(elem){
								if(elem.id && this.newDocsMetadata[elem.id]){
									res.push(this.newDocsMetadata[elem.id]);
								}
							}, this));
							return res;
						}
			    	};
				},
				
				// do the actual validation of unique document names
				validateNamesGlobally: function(validObj, noAlerts){
					// remove highlight
					$('.validationHighlight').removeClass('validationHighlight');
					
					// if no validation element was passed, create a new one
					if(!validObj){
						validObj = this.createUniqueNamesObj();
					}

					// use validation object and send it around to all listeners (which will handle this synchronously)
					WCEvent.emit("validation.uniqueDocName", validObj);
					
					// if backend check is needed, do the ajax call
					if(validObj.newDocs.length > 0){
						// check on backend for duplicates
		                var checkDocnamesUrl = "GetDocumentsByNameAndProjectID.jsp";
		                
		                $.ajax({
		        			url: checkDocnamesUrl,
		        			dataType : "json",
		        			method: "POST",
		        			async: false,
		        			data: { 
		        		        'data': JSON.stringify({docs: validObj.newDocs}) 
		        		    }
		        		}).done(function(jsonObj){
		        			if(jsonObj.docs && jsonObj.docs.length > 0){
		        				for(var i=0 ; i<jsonObj.docs.length ; i++){
		        					// conflict found, add errorObject
		        					var errorObj = {
		        						msg : ValidatorStrings.t('NAMECONFLICTEXCEPTION_ADV_TAG',jsonObj.docs[i].docName, jsonObj.docs[i].projectName),
		        						element : validObj.newDocsDOMElems[jsonObj.docs[i].projectId + '_' + jsonObj.docs[i].docName.toLowerCase()]
		                        	};
		                			validObj.errors.push(errorObj);
		        				}	
		        			}
		                });
					}
	                
					// now handle all errors
					if(!noAlerts){
						var errorMsg = '';
	        			for (var errIdx = 0; errIdx < validObj.errors.length; errIdx++) {
		        			var errObj = validObj.errors[errIdx];
		        			
		        			if (errObj.msg) {
		        				errorMsg += errObj.msg + '\n';
		        			}
		        			if(errObj.element){
		        				errObj.element.addClass('validationHighlight');
		        			}
		        		}
	        			if(errorMsg){
	        				alert(errorMsg);
	        			}
					}
				}
			};

		};

		return {
			// Get the Singleton instance if one exists
			// or create one if it doesn't
			getInstance : function() {
				if (!window.eventBasedValidator) {
					window.eventBasedValidator = init();
				}
				return window.eventBasedValidator;
			}
		};

	})();
	
	return EventBasedValidator;
});