define(["jquery"], function ($) {
	
	"use strict";

	var Modal = function (content, options) {
		
		// Content should always be wrapped by jQuery, do this if not ...
		if (!(content instanceof jQuery)) {
			content = $(content);
        }
		
		// Start parsing the options when specified
		
		// Parse whether we allow the modal to be dismissed by the user.
		if (options && options.allowClose!==undefined) {
			this._allowClose = options.allowClose;
		} else {
			this._allowClose = true;
		}
		
		// Parse the width of the modal.
		if (options && options.width) {
			this._widthClass = options.width;
		} else {
			this._widthClass = '';
		}
		
		// The array that will hold all the functions that need to be called on close.
		this._onCloseFunctions = [];
		this._onOpenFunctions = [];
		
		// Create the content
		this._content = content;
		
		// Create the other elements of the modal
		this._wrapper = $("<div>", {"class":"modal fade"});
		this._modalDialog = $("<div>", {"class":"modal-dialog "+this._widthClass});
		this._modalContent = $("<div>", {"class":"modal-content"});
		this._modalHeader = $("<div>", {"class":"modal-header"});
		this._modalCloseButton = $("<button>", {"class":"close", "type":"button", "data-dismiss":"modal"});
		this._modalHeaderTitle = $("<h4>", {"class":"modal-title"});
		this._modalBody = $("<div>", {"class":"modal-body"});
		this._modalFooter = $("<div>", {"class":"modal-footer"});
		
		// Only add the close button when we allow the user to dismiss this modal.
		if (this._allowClose) {
			this._modalCloseButton.append($("<i>", {"class":"icon icon-WCR_Reject"}));
			this._modalHeader.append(this._modalCloseButton);
		}
		
		this._modalHeader.append(this._modalHeaderTitle);
		this._modalBody.append(this._content);
		
		this._modalContent.append(this._modalHeader);
		this._modalContent.append(this._modalBody);
		
		this._modalDialog.append(this._modalContent);
		this._wrapper.append(this._modalDialog);
		
		
		// Apply some settings to the modal for when we do not allow it to be closed ...
		if (!this._allowClose) {
			this._wrapper.modal({
				backdrop : 'static', // Disable dismissal when backdrop is clicked ...
				keyboard : false, // Disable the dismissal when hitting ESC ...
				show : false
			});
		} else {
			this._wrapper.modal({
				show : false
			});
		}
		
		// When a model hidden event is fired do ... 
		this._wrapper.on('hide.bs.modal', function (e) {
			// execute all callbacks ...
			$.each(this._onCloseFunctions, function (index, callback) {
				callback();
			}.bind(this));
		}.bind(this));
		
		// When a model shown event is fired do ... 
		this._wrapper.on('show.bs.modal', function (e) {
			// execute all callbacks ...
			$.each(this._onOpenFunctions, function (index, callback) {
				callback();
			}.bind(this));
		}.bind(this));
		
	};
	
	Modal.prototype.setTitle = function (newTitle) {
		this._modalHeaderTitle.text(newTitle);
	};
	
	// The footer is optional, when not set, it will not be shown.
	Modal.prototype.setFooter = function (jQElement) {
		this._modalFooter.empty().append(jQElement);
		// If the footer is not yet part of the modal, add it.
		if (this._modalFooter.parent()[0] !== this._modalContent[0]){
			this._modalContent.append(this._modalFooter);
		} else if (jQElement === undefined) {
			// If the passed element is undefined, remove the footer again from the modal.
			this._modalContent.remove(this._modalFooter);
		}
	};
	
	Modal.prototype.close = function () {
		this._wrapper.modal('hide');
	};
	
	Modal.prototype.open = function () {
		this._wrapper.modal('show');
	};
	
	Modal.prototype.update = function () {
		this._wrapper.modal('handleUpdate');
	};
	
	Modal.prototype.onClose = function (onCloseFunction) {
		this._onCloseFunctions.push(onCloseFunction);
	};
	
	Modal.prototype.onOpen = function (onCloseFunction) {
		this._onOpenFunctions.push(onCloseFunction);
	};
	
	Modal.prototype.getJQElement = function () {
		return this._wrapper;
	};
	
	return Modal;

});