define(["jquery", "jqueryUI"], function ($) {
	
	"use strict";

	var Popup = function (content, options) {
		
		// Content should always be wrapped by jQuery, do this if not ...
		if (!(content instanceof jQuery)) {
			content = $(content);
        }
		
		// Make a reference to this object.
		var self = this;
		
		// The array that will hold all the functions that need to be called on close.
		this._onCloseFunctions = [];
		
		// Create the content
		this._content = content.wrap($("<div></div>")).parent();
		
		// Define the default options.
		var defaultOptions = {
				modal: true,
				resizable: false,
				width:'auto',
	            autoOpen: false,
	            close: function( event, ui ) {
	            	// Execute the functions who are listening for on close value.
	                for (var i = 0; i < self._onCloseFunctions.length; i++) {
	                    self._onCloseFunctions[i]();
	                }
	            }
	        };
		
		// Create popup with right options
		if (options !== undefined) {
			
			// Merge the default options with the passed options. 
			$.extend(true, defaultOptions, options);
			
			// Overwrite the close option, to make sure that the onCloseFunctions are always called.
			defaultOptions.close = function( event, ui ) {
            	// Execute the functions who are listening for on close value.
                for (var i = 0; i < self._onCloseFunctions.length; i++) {
                    self._onCloseFunctions[i]();
                }
                // Call also the close function of the passed options (if defined).
                if (options.close !== undefined) {
                	options.close();
                }
            }
			
			// Make the popup.
			this._content.dialog(defaultOptions);
		} else {
			
			// Make the popup.
			this._content.dialog(defaultOptions);
		}
		
	}
	
	Popup.prototype.setTitle = function (newTitle) {
		this._content.dialog({
			title: newTitle
		});
	}
	
	Popup.prototype.close = function () {
		this._content.dialog( "close" );
	}
	
	Popup.prototype.open = function () {
		this._content.dialog( "open" );
	}
	
	Popup.prototype.onClose = function (onCloseFunction) {
		this._onCloseFunctions.push(onCloseFunction);
	}
	
	Popup.prototype.getJQElement = function () {
		return this._content;
	}
	
	return Popup;

});