define(["lib/EventEmitter2"],
	function (EventEmitter2) {
		"use strict";

		/**
		 * Object to create and handle events in WebCenter.
		 * The module immediately returns the instance of the event emitter, so it can be used like this:
		 * 	- WCEvent.on("xxx", function(){...})
		 *  - WCEvent.emit("xxx", arguments...)
		 */
		var WCEvent = (function () {

			function init() {
				var ebus = new EventEmitter2({
					wildcard: true, // should the event emitter use wildcards.
					delimiter: '.'
				});

				var maxListeners = 0;

				//>>excludeStart("PExclude", pragmas.productionExclude);
				// during development we want maximum 20 listeners per channel, in production mode this can be much more
				maxListeners = 20;

				// IE9 fix for EventEmitter (for when we exceed the maxlistener limit)
				if (!window.console) {
					window.console = {};
				}
				if (!window.console.trace) {
					window.console.trace = function () {};
				}
				//>>excludeEnd("PExclude");	

				ebus.setMaxListeners(maxListeners);

				return {
					on: function () {
						ebus.on.apply(ebus, arguments);
					},
					off: function () {
						ebus.off.apply(ebus, arguments);
					},
					emit: function () {
						ebus.emit.apply(ebus, arguments);
					}
				};
			};

			return {
				// Get the Singleton instance if one exists or create one if it doesn't
				getInstance: function () {
					if (!window.WCEvent) {
						window.WCEvent = init();
					}
					return window.WCEvent;
				}
			};

		})();

		return WCEvent.getInstance();
	});