/**
 * Used by webcss to get the css text content...
 * 
 * Same as text, but BUILDS LESS FILES!!!
 * 
 * Note: see webcss-text for the version used in development/in the browser...
 * 
 * This versions compiles the less and writes the less to the javascript "bundle"...
 * 
 * @author JEPE
 */

define(["lib/less-rhino-1.7.5"], function(less){
	var buildMap = {};
	
	/**
	 * Method to read the less/css file... (when optimizing)
	 * 
	 * CODE COPIED...
	 * See also layout/tpl (method is copied from there...)
	 * Or text! where the original code is located...
	 * 
	 * NOTE JEPE: If we were executing from command line, we could have used the build in Rhino method "readFile(path, encoding)"...
	 * (It is only available when run from command line)
	 */
	function readFile(req, name){
		//>>excludeStart("PExclude", pragmas.afterBuildExclude);
		var url = req.toUrl(name);
		var stringBuffer, line,
        encoding = "utf-8",
        file = new java.io.File(url),
        lineSeparator = java.lang.System.getProperty("line.separator"),
        input = new java.io.BufferedReader(new java.io.InputStreamReader(new java.io.FileInputStream(file), encoding)),
        content = '';
        try {
            stringBuffer = new java.lang.StringBuffer();
            line = input.readLine();

            // Byte Order Mark (BOM) - The Unicode Standard, version 3.0, page 324
            // http://www.unicode.org/faq/utf_bom.html

            // Note that when we use utf-8, the BOM should appear as "EF BB BF", but it doesn't due to this bug in the JDK:
            // http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=4508058
            if (line && line.length() && line.charAt(0) === 0xfeff) {
                // Eat the BOM, since we've already found the encoding on this file,
                // and we plan to concatenating this buffer with others; the BOM should
                // only appear at the top of a file.
                line = line.substring(1);
            }

            if (line !== null) {
                stringBuffer.append(line);
            }

            while ((line = input.readLine()) !== null) {
                stringBuffer.append(lineSeparator);
                stringBuffer.append(line);
            }
            //Make sure we return a JavaScript string and not a Java string.
            content = String(stringBuffer.toString()); //String   
        } finally {
            input.close();
        }
        return content;
        //>>excludeEnd("PExclude");
	}
	
	/**
	 * Utility method
	 */
	function endsWith(str, suffix) {
	    return str.indexOf(suffix, str.length - suffix.length) !== -1;
	}
	
	/**
	 * Escaping text in javascript...
	 * 
	 * CODE COPIED...
	 * Copied from: lib/text
	 */
	function jsEscape (content) {
        return content.replace(/(['\\])/g, '\\$1')
            .replace(/[\f]/g, "\\f")
            .replace(/[\b]/g, "\\b")
            .replace(/[\n]/g, "\\n")
            .replace(/[\t]/g, "\\t")
            .replace(/[\r]/g, "\\r")
            .replace(/[\u2028]/g, "\\u2028")
            .replace(/[\u2029]/g, "\\u2029");
    }
	
	var loader = {};
	
	/**
	 * Build the stylesheet (either less or css)
	 * 
	 * NOTE: Similar code in webcss-text & webcss-build !!!
	 * HOWEVER: Different versions of less + less for different environment...
	 */
	function buildStylesheet(req, isLessFile, name, content, config, callback, callbackerror) {
		// check if we did not already build this file...
		if(buildMap.hasOwnProperty(name)) {
			callback(buildMap[name]);
			return;
		}
		
		if(isLessFile) {
			// LESS - Build the less...
			var baseUrl = config.baseUrl;
			var fileUrl = name;
			
			var cfg = config.less || {};
		    cfg.paths = [baseUrl];
		    cfg.filename = baseUrl+fileUrl;
		    cfg.compress = true;
		    
		    // Write a line to the log...
		    console.log("Building less file \""+name+"\"");
		    var parsingdone = false;
		    
		    // NOTE: because we have different versions of less, we cannot compile in the same way...
		    // Here we need to use the less.Parser approach...
		    (new less.Parser(cfg)).parse(content, function(err, root) {
		    	parsingdone = true;
		    	if (err) {
		    		// There is an error in the less file...
		    		console.log(err + ' at ' + err.filename + ', line ' + err.line);
		    		callbackerror();
		    		return;
			    }
		    	
		    	var newContent = root.toCSS(cfg);
		    	
		    	buildMap[name] = newContent;
			    callback(newContent);
		    });
		    if(!parsingdone) {
		    	console.log("Error: something went wrong ... less did not return... error?");
		    	callbackerror();
		    }
		} else {
			// CSS
			buildMap[name] = content;
			callback(content);
		}
	};
	
	/**
	 * CODE COPIED...
	 * Copied from lessc-rhino-1.7.5
	 * 
	 * Required to get the less-rhino-1.7.5 library working.
	 * That library contains everything we need EXCEPT for this one method...
	 * Also, we cannot directly use the one in lessc-rhino as it except the global method readFile, 
	 * but that does not exist if the code is not run from the commandline (I think).
	 * 
	 * So: small changes to use the function readFile defined above...
	 */
	less.Parser.fileLoader = function (file, currentFileInfo, callback, env) {

	    var href = file;
	    if (currentFileInfo && currentFileInfo.currentDirectory && !/^\//.test(file)) {
	        href = less.modules.path.join(currentFileInfo.currentDirectory, file);
	    }

	    var path = less.modules.path.dirname(href);

	    var newFileInfo = {
	        currentDirectory: path + '/',
	        filename: href
	    };

	    if (currentFileInfo) {
	        newFileInfo.entryPath = currentFileInfo.entryPath;
	        newFileInfo.rootpath = currentFileInfo.rootpath;
	        newFileInfo.rootFilename = currentFileInfo.rootFilename;
	        newFileInfo.relativeUrls = currentFileInfo.relativeUrls;
	    } else {
	        newFileInfo.entryPath = path;
	        newFileInfo.rootpath = less.rootpath || path;
	        newFileInfo.rootFilename = href;
	        newFileInfo.relativeUrls = env.relativeUrls;
	    }

	    var j = file.lastIndexOf('/');
	    if(newFileInfo.relativeUrls && !/^(?:[a-z-]+:|\/)/.test(file) && j != -1) {
	        var relativeSubDirectory = file.slice(0, j+1);
	        newFileInfo.rootpath = newFileInfo.rootpath + relativeSubDirectory; // append (sub|sup) directory path of imported file
	    }
	    newFileInfo.currentDirectory = path;
	    newFileInfo.filename = href;

	    var data = null;
	    try {
	    	console.log("Importing file '"+less.modules.path.basename(href)+"'");
	        data = readFile(loader.req, href);
	    } catch (e) {
	        callback({ type: 'File', message: "'" + less.modules.path.basename(href) + "' wasn't found" });
	        return;
	    }

	    try {
	        callback(null, data, href, newFileInfo, { lastModified: 0 });
	    } catch (e) {
	        callback(e, null, href);
	    }
	};

	/**
	 * Implements a custom require loader...
	 * 
	 * See http://requirejs.org/docs/plugins.html for information about implementing custom loaders...
	 * @param name
	 * @param req
	 * @param onload
	 * @param config
	 */
	loader.load = function (name, req, onload, config) {
		loader.req = req;
		
		var isLessFile = endsWith(name.toLowerCase(), ".less");
		
		if(config.isBuild) {
			//
			// We are optimizing...
			//
			
			// 1. read file with css/less
			var content = readFile(req, name);
			
			// 2. build file... (if needed)
			buildStylesheet(req, isLessFile, name, content, config, function(newContent){
				// 3. return
				onload(newContent);
			}, function() {
				// 3. return
				onload.error();
			});
		}else{
			// Should NEVER reach here... Instead, "webcss-text.load(...)" should have been called...
			throw new Error("This should only be executed at build time...");
		}
	};
	
	/**
	 * When optimizing... Do the same as text! require module. BUT always write the CSS content. (even for less files)
	 * 
	 * @param pluginName
	 * @param moduleName
	 * @param write
	 */
	loader.write = function (pluginName, moduleName, write) {
	    if (buildMap.hasOwnProperty(moduleName)) {
            var content = jsEscape(buildMap[moduleName]);
            write.asModule(pluginName + "!" + moduleName,
                           "define(function () { return '" +
                               content +
                           "';});\n");
        }
	};

	return loader;
});