/**
 * Used by webcss to get the css text content...
 * 
 * Same as text, but BUILDS LESS FILES!!!
 * 
 * Note: see webcss-build for the version used during optimization...
 * 
 * @author JEPE
 */

define([], function(){
	var buildMap = {};
	
	/**
	 * Utility method
	 */
	function endsWith(str, suffix) {
	    return str.indexOf(suffix, str.length - suffix.length) !== -1;
	}
	
	
	
	var loader = {};
	
	/**
	 * Build the stylesheet (either less or css)
	 * 
	 * NOTE: Similar code in webcss-text & webcss-build !!!
	 * HOWEVER: Different versions of less + less for different environment...
	 */
	function buildStylesheet(req, isLessFile, name, content, config, callback, callbackerror) {
		// check if we did not already build this file...
		if(buildMap.hasOwnProperty(name)) {
			callback(buildMap[name]);
			return;
		}
		
		if(isLessFile) {
			// LESS - Build the less...
			require(["lib/less"], function(less){
				var baseUrl = config.baseUrl;
				var fileUrl = name;
				
				var cfg = config.less || {};
			    cfg.paths = [baseUrl];
			    cfg.filename = baseUrl+fileUrl;
			    cfg.async = true;
				cfg.syncImport = false;
			    
			    // Should only happen in development... But better add a warning...
			    console.log("Warning - not optimized - compiling less in browser...");
			    less.render(content, cfg, function(err, output) {
			    	if (err) {
			    		// There is an error in the less file...
			    		console.log(err + ' at ' + err.filename + ', line ' + err.line);
				        callbackerror();
				        return;
				    }
			    	
			    	var newContent = output.css;
			    	
			    	buildMap[name] = newContent;
				    callback(newContent);
			    });
			});
		} else {
			// CSS
			buildMap[name] = content;
			callback(content);
		}
	}

	/**
	 * Implements a custom require loader...
	 * 
	 * See http://requirejs.org/docs/plugins.html for information about implementing custom loaders...
	 * @param name
	 * @param req
	 * @param onload
	 * @param config
	 */
	loader.load = function (name, req, onload, config) {
		var isLessFile = endsWith(name.toLowerCase(), ".less");
		
		if(config.isBuild) {
			//
			// We are optimizing...
			//
			// Should NEVER reach here... Instead, "webcss-build.load(...)" should have been called...
		}else{
			//
			// We are just loading without optimization...
			//
			
			// 1. read file with css/less
			req(["lib/text!"+name], function(content){
				// 2. build file... (if needed)
				buildStylesheet(req, isLessFile, name, content, config, function(newContent){
					// 3. return
					onload(newContent);
				}, function(){
					// 3. an error occured...
					onload.error();
				});
			});
		}
	};
	
	loader.pluginBuilder = "utils/webcss-build";

	return loader;
});