define(["web-viewer/annotation/AnnotationPopup",
		"web-viewer/annotation/AnnotationRow",
		"underscore"
	],
	function (AnnotationPopup,
		AnnotationRow,
		_) {
		"use strict";

		/**
		 * Create a new Annotation Popup for existing annotations.
		 * @exports AnnotationPopupExisting
		 * @memberof AnnotationModule
		 * @class
		 * The annotation popup is responsible for an existing annotation popup and its content (the annotation row).
		 * @augments AnnotationModule.AnnotationPopup
		 * @constructor 
		 * @param {Core.ViewerFacade} f - Viewerfacade
		 * @param {String} id - Id of the popup.
		 * @param {Annotation} annotation - Annotation this popup belongs to.
		 * @param {AnnotationModel} annotationModel - Model of the annotations.
		 * @param {JQueryObj} container - The container where the popup should stay in.
		 */
		var AnnotationPopupExisting = function (f, id, annotation, annotationModel, container) {
			AnnotationPopup.call(this, f, id, annotation, container);

			this.annotationModel = annotationModel;
		};

		AnnotationPopupExisting.prototype = Object.create(AnnotationPopup.prototype);
		AnnotationPopupExisting.prototype.constructor = AnnotationPopupExisting;

		/**
		 * @override
		 */
		AnnotationPopupExisting.prototype._initialize = function () {
			// Super call
			AnnotationPopup.prototype._initialize.call(this);

			// Annotation row (content)
			this.row = new AnnotationRow(this.f, this.annotation, this.annotationModel, true);
			this.row.setSelected(true);
			this.annotationExistingContent = $("<div>").addClass("AnnotationTable enableTextSelection").append(this.row.element);

			this.setContent(this.annotationExistingContent);

			// Resize the row along with the popup
			this.on("contentResized", _.bind(function (size) {
				this.row.setAvailableHeight(size.height);
			}, this));
			this.row.setAvailableHeight(this.content.height());
		};

		/**
		 * @override
		 * @param {AnnotationModule.Annotation} refannotation - Referenced annotation used in the row update
		 */
		AnnotationPopupExisting.prototype.repaint = function (refannotation) {
			if (this._isInitialized) {
				// Super call
				AnnotationPopup.prototype.repaint.call(this);
				this.row.annotationUpdated(refannotation);
			}
		};

		return AnnotationPopupExisting;
	});