define(["component/toolbar/ToolbarComponent",
		"component/toolbar/ButtonGroup",
		"web-viewer/layout/component/ViewerToolbarButton",
		"web-viewer/layout/component/DropdownToolbarButton",
		"web-viewer/layout/LayoutConstants",
		"web-viewer/utils/ListenerUtil",
		"underscore"
	],
	function (ToolbarComponent,
		ButtonGroup,
		ViewerToolbarButton,
		DropdownToolbarButton,
		LayoutConstants,
		ListenerUtil,
		_
	) {
		"use strict";
		/**
		 * mobile Toolbar component for approval
		 *
		 * @exports ApprovalMobileToolbarComponent
		 * @memberof ApprovalModule
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {Approval.ApprovalModule} module
		 */
		var ApprovalMobileToolbarComponent = function (f, approvalModule) {
			ToolbarComponent.call(this, LayoutConstants.ToolbarComponent.APPROVAL, f);
			ListenerUtil.mixin(this, f);

			this.f = f;
			this._approvalModule = approvalModule;
			this._registerListener("updated", this.updateComponent, this, this._approvalModule);
			// start by displaying loading message
			this.currentState = "Loading";


			this._icon = $("<i>").addClass("icon");

			this._dropdownMenu = new DropdownToolbarButton(this.f, "approval", this._icon, false);

			this.element.append(this._dropdownMenu.element).addClass("approvalControls");


			this.updateComponent();
		};

		ApprovalMobileToolbarComponent.prototype = Object.create(ToolbarComponent.prototype);
		ApprovalMobileToolbarComponent.prototype.constructor = ApprovalMobileToolbarComponent;
		ApprovalMobileToolbarComponent.toolbarName = LayoutConstants.ToolbarComponent.APPROVAL;


		/**
		 * Update the display of the status in the approval toolbar component
		 * @param {String} status - the approval status that should be displayed
		 */
		ApprovalMobileToolbarComponent.prototype.setStatus = function (status) {
			this._dropdownMenu.element.toggleClass(this.currentState, false).toggleClass(status, true);

			// hide the component if no approval defined
			if (status !== "NotDefined") {
				this.show();
			} else {
				this.hide();
			}
			this.currentState = status;

			this._icon.removeClass().addClass("icon icon-WCR_" + status);
			this.f.i18n.loadBundle("approval", function () {
				this._dropdownMenu.element.text(this.f.i18n.t("CApp" + this.currentState));
				this._dropdownMenu.element.prepend(this._icon.addClass("statusIcon"));
				this._dropdownMenu.element.append($("<i>").addClass("icon icon-WCR_Sort-Down dropdownIndicator"));
			}, this);
		};

		/**
		 * Updates the toolbar component when the approval setup or compare mode has changed
		 */
		ApprovalMobileToolbarComponent.prototype.updateComponent = function () {
			// get the active document
			var docController = this.f.getDocumentController();
			var page = docController.getActivePage();


			if (page) {
				this.show();
				this.setStatus("Loading");
				this.setStatus(page.getApprovalStatus());


				// clear buttons
				this._dropdownMenu.clear();

				// add buttons			
				var actions = this._approvalModule.getAvailableActions();
				for (var i = 0; i < actions.list.length; i++) {
					var action = actions.list[i];

					this._dropdownMenu.addAction({
						name: action.name,
						content: action.text,
						icon: "icon-WCR_" + action.name,
						action: action.action,
						cssClass: action.name,
						enabled: true
					});
				}
				if (actions.list.length === 0) {
					this._dropdownMenu.disableDropdown();
					if (!this._approvalModule.showStatus()) {
						this.hide();
					}
					this.element.toggleClass("clickable", false);
				} else {
					this._dropdownMenu.enableDropdown();
					this.element.toggleClass("clickable", true);
				}

			} else {
				// no document active
				this.buttonGroup.clearButtons();
				this.buttonGroup.hide();
				this.hide();
			}

			this._emit("changed.content");
		};




		return ApprovalMobileToolbarComponent;
	});