
define('doctablex/TextContentView',["jquery",
        "underscore",
        "doctablex/DocTableXUtils",
        "doctablex/AJAXDataCache",
        "utils/webi18n!TextContentStrings",
		"packaging-content-management/ContentConstants",
        "wcr/webcenter_xml",
        "packaging-content-management/PCMEventBus"],
function ($,
    _,
    DocTableXUtils,
    AJAXDataCache,
    webi18n,
    ContentConstants,
    WCXML,
    pcmEventBus) {

    /**
     * TextContentView.js
     *
     * This class implements rendering methods for documents table when in Text Content render mode.
     *
     * @author JANH - February 2015
     */

    "use strict";


    var TextContentView = function() {

		this.__isMixin = true;   //JANH debug then DEL!!!

		
	    this.sortOption = function ($select) {
	    	$select.append($select.find("option").remove().sort(function(a, b) {
	    	    var at = $(a).text(), bt = $(b).text();
	    	    return (at > bt)?1:((at < bt)?-1:0);
	    	}));
	    };
	    
	
	    /**
	     * Populates master and translation language drop down in the table header
	     */
	    this.updateLanguageDropDowns = function (masterLangToSelect, transLangToSelect) {
	    	//Fill Master language dropdown
	    	this._addLangsToDropDowns(this.dm._projMasterLangs, this.languageMasterDropdownMap, masterLangToSelect);
	    	
			//Fill translation language dropdown
	    	this._addLangsToDropDowns(this.dm._projTransLangs, this.languageTranslatedDropdownMap, transLangToSelect);
	
	    };
	
	    
	    /**
	     * 
	     * @param langs
	     * @param languageDropdownMap
	     * @param langToSelect - language code
	     */
	    this._addLangsToDropDowns = function (langs, languageDropdownMap, langToSelect) {
	    	
			for ( var colIndex in languageDropdownMap) {
				if (languageDropdownMap.hasOwnProperty(colIndex)) {
	
					var select = languageDropdownMap[colIndex];
					
					// Get the current selected language before removing the options
					var currentSelectedLang = select.val();
					
					// Select empty.
					select.empty();
					
					//Add change event listener that will change the view
	    			select.change(_.bind(function(e){
	    				this.repaintDocRows();
	    			}, this));
	    			
	    			//Populate the language dropdown
	    			for (var i=0; i < langs.length; i++) {
	    				select.append(new Option(langs[i].langDesc, langs[i].langCode));
	    			}	
	    			
	    			if(langToSelect) {
	    				select.attr("data-default-langcode", langToSelect);
	    			} else if(currentSelectedLang) {
	    				select.attr("data-default-langcode", currentSelectedLang);
	    			}
	    			
	    			// Select the default if available.
	    			var defLangCode = select.attr("data-default-langcode");
	    			select.find('option[value="'+defLangCode+'"]').attr('selected', 'selected');
	    			select.change();
	    			
	    			// Show the dropdown
	    			select.show();
				}
			}
	    	
	    };
	    
		/**
		 * Table header for Master Language
		 */
		this.renderMasterTextHeader = function(phElem, config) {
			this._renderTextHeader(phElem, config, this.languageMasterDropdownMap, this.dm._projMasterLangs);
		};
		
		/**
		 * Table header for Translation Language
		 */
		this.renderTranslationTextHeader = function(phElem, config) {
			this._renderTextHeader(phElem, config, this.languageTranslatedDropdownMap, this.dm._projTransLangs);
		};
		
		/**
		 * Render text languages header, render the header for a text language collumn.
		 * 
		 * @param phElem The element in which the header should be rendered.
		 * @param config The configuration of this collumn.
		 * @param dropdownMap The map that keeps track of possible dropdowns to show in this header.
		 */
		this._renderTextHeader = function(phElem, config, dropdownMap, langs) {
			// Create the header element
			var thElem = $('<th/>').addClass('docrefdoccol');
			phElem.append(thElem);
			// Add the correct label.
			thElem.append('<span>' + DocTableXUtils.getEncodedTranslatedLabel(config.label) + '</span>&nbsp;&nbsp;');
			// Get the appropiate dropdown for this collumn.
			var dropdown = dropdownMap[thElem.index()];
			// If the dropdown does not yet exist ...
			if (!dropdown) {
				//Set margin to reduce the Doctablex header row height for text content table
				// Create it ...
				dropdown = $('<select/>',{"class" : "text-lang-selector"}).css("margin", "0px 5px").attr("data-default-langcode",config.typeDetail).hide();
				dropdownMap[thElem.index()] = dropdown;
				// Fill it with the available language options.
				this._addLangsToDropDowns(langs, dropdownMap);
			}
			
			//Fix for IE browser:: When select element is moved in DOM, options are not available
			//Hence re-add options in the select element
			if ($("option", dropdown).length < langs.length) {
				// Fill it with the available language options.
				this._addLangsToDropDowns(langs, dropdownMap);
			}
			
			// Add the dropdown
			thElem.append(dropdown);
			dropdown.change(_.bind(function(e){
				this.repaintDocRows();
			}, this));
			
		};
		
		/**
		 * Table header for Text Languages
		 */
		this.renderTextLanguagesHeader = function(phElem, config) {
			
			var thElem = $('<th/>').addClass('docrefdoccol');
			phElem.append(thElem);
			thElem.append('<span>' + DocTableXUtils.getEncodedTranslatedLabel(config.label) + '</span>');
		};
		
		
/*//JANH these are not necessary as master and translation texts are not renedered at all when there is no PCM license		
		// UTGO:: Here only create placeholder element. 
		DocTableXView.prototype.renderMasterTextHeader = function(phElem, config) {
			// Actual rendering with language selector should happen in the subclass TextContentView
			var thElem = $('<th/>').addClass('docrefdoccol');
			phElem.append(thElem);
			thElem.append('<span>' + DocTableXUtils.getEncodedTranslatedLabel(config.label) + '</span>&nbsp;&nbsp;');
		};

		// UTGO:: Here only create placeholder element
		DocTableXView.prototype.renderTranslationTextHeader = function(phElem, config) {
			// Actual rendering with language selector should happen in the subclass TextContentView
			var thElem = $('<th/>').addClass('docrefdoccol');
			phElem.append(thElem);
			thElem.append('<span>' + DocTableXUtils.getEncodedTranslatedLabel(config.label) + '</span>&nbsp;&nbsp;');
		};
*/

	};

	
	/**
	 * Mixin constructor/initializer.
	 * (class method)
	 */ 
	TextContentView.__initMixin = function(options) {
    	if (!options) {
    		// nothing to initialize from
    		return this;
    	}

    	// extend the mapping of rendering methods (by text content specific methods)
    	this.renderMethodMap['MasterText'] = this.renderMasterTextHeader;
    	this.renderMethodMap['TranslationText'] = this.renderTranslationTextHeader;
    	this.renderMethodMap['TextLanguages'] = this.renderTextLanguagesHeader;

    	
    	// Stores the Language dropdown element to make it available to rows.
    	this.languageMasterDropdownMap = {};
    	this.languageTranslatedDropdownMap = {};

    	// by default make green links appear at top
    	this.renderOptions.showGreenLinksOnTop = true;
    	this.renderOptions.showGreenLinksAtBottom = false;

    	//The languageBundle necessary for this component.
    	this.i18n = webi18n;
	};

	
	/**
	 * List of overridden methods with their original code.
	 */ 
	TextContentView.__overridden = {
	};

	
    return TextContentView;
});

define('doctablex/TextContent',["jquery",
		"doctablex/DocTableXUtils",
		"doctablex/TextContentView",
		"packaging-content-management/text-content-services/languageService",
		"packaging-content-management/text-content-utils/LanguageUtils"
		],
function ($,
	DocTableXUtils,
	TextContentView,
	languageService,
	LanguageUtils) {
	
	/**
	 * TextContent.js
	 *
	 * Class (mixin) designed to extend DocTableX to support PCM (text content) specific logic and features of text statements. 
	 * Use DocTableXFactory to create instances of DocTableX enhanced by methods and members defined here.
	 *
	 * @author JANH, HUBU - February 2015
	 */

	"use strict";

	var TextContent = function() {

		this.__isMixin = true;   //JANH debug then DEL!!!

		
		/**
		 * Private initialization method using AJAX to load asynchronously required info that is not passed by
		 * 'options' (constructor parameter) from the server.
		 */
		this._initializeMixinFromServer = function() {
//			DocTableX.prototype._initializeFromServer.call(this);   //JANH !!!!!
			
	   
		   if ((DocTableXUtils.getRenderItem(this.renderOptions.columns, 'MasterText') != null) 
				   || (DocTableXUtils.getRenderItem(this.renderOptions.columns, 'TranslationText') != null)
				   || (DocTableXUtils.getRenderItem(this.renderOptions.columns, 'TextLanguages') != null)){
				// Retrieve the languages.
				languageService.getLanguages(this.projectID, function (projMasterLangs, projTransLangs){
					
					// Remember a reference to these project languages.
					this._projMasterLangs = projMasterLangs;
					this._projTransLangs = projTransLangs;
					
					// Update the language dropdowns
					this.view.updateLanguageDropDowns();
					
				}.bind(this));
		   }

			// Whenever a language is added/removed from the project, we need to update the dropdown elements of our doctable.
			// We therefore listen to the "PCM.projMasterLangChanged" event which is fired whenever the "MASTER_TEXT_LANGUAGE"
			// or "TRANSLATION_LANGUAGE" attributes are changed.
			pcmEventBus.on("PCM.projMasterLangChanged", _.bind(
					function (langAttrName, addedLangDesc, removedLangDesc, projectID) {
		
				if (this.projectID != projectID) {
					return;
				}
					
				// Determine wether we need to work on Master/Translation text.
				var isMaster = true;
				var langs = this._projMasterLangs;
				if ('MASTER_TEXT_LANGUAGE' !== langAttrName) {
					isMaster = false;
					langs = this._projTransLangs;
				}
				
				// Let's start by mapping the passed language descriptions to real language objects.
				
				// We need all the languages in WebCenter for this.
				languageService.getAllLanguages(this.projectID, isMaster, function (allLangs) {
					
					// When we have the "All Languages" array ...
					
					// Start searching for the passed language description in the "All Languages" array.
					var addedLang = [];
					var removedLang = [];
					for (var i=0; i < addedLangDesc.length;i++) {
						var lang = LanguageUtils.getLangForDescription(allLangs, addedLangDesc[i]);
						if (lang) {
							addedLang.push(lang);
						}
					}
					for (var i=0; i < removedLangDesc.length;i++) {
						var lang = LanguageUtils.getLangForDescription(allLangs, removedLangDesc[i]);
						if (lang) {
							removedLang.push(lang);
						}
					}
					
					var sortLangsByDesc = function(lang1, lang2) {
						var lang1Desc = lang1.langDesc.toLowerCase();
						var lang2Desc = lang2.langDesc.toLowerCase(); 
						return ((lang1Desc < lang2Desc) ? -1 : ((lang1Desc > lang2Desc) ? 1 : 0));
					};
					
					var masterLangToSelect = null;
					var transLangToSelect = null;
					// Add/remove these languages from our internal project language variables
					if (addedLang.length > 0) {
						if(isMaster) {
							this._projMasterLangs = langs.concat(addedLang);
							this._projMasterLangs.sort(sortLangsByDesc);
							masterLangToSelect = addedLang[addedLang.length-1].langCode;
						} else {
							this._projTransLangs = langs.concat(addedLang);
							this._projTransLangs.sort(sortLangsByDesc);
							transLangToSelect = addedLang[addedLang.length-1].langCode;
						}
					}
					if (removedLang.length > 0) {
						if(isMaster) {
							LanguageUtils.removeFrom(this._projMasterLangs, removedLang);
						} else {
							LanguageUtils.removeFrom(this._projTransLangs, removedLang);
						}
					}
					
					// Add/Remove these languages from the documents.
					this.addLanguageToAllDocs(addedLang, isMaster);
					this.removeLanguageFromAllDocs(removedLang, isMaster);
					
					// Update the dropdowns.
					this.view.updateLanguageDropDowns(masterLangToSelect, transLangToSelect);
				}.bind(this));
				
			}, this));
		};
	
	    /**
		 * Handling added languages from project attributes
		 */
		this.addLanguageToAllDocs = function(addedLangs, isMaster) {
	
			// Add the language to all document rows.
			for (var dr = 0; dr < this.docRows.length; dr++) {
				
				// Only add if the current row has a text content document.
				if (!this.docRows[dr].contentStatement) {
					continue;
				}
				
				// Only add if the current row is not in a remote project.
				if (this.docRows[dr].isRemoteDocument()) {
					continue;
				}
				
				// Only add if the current row is not in a remote project.
				if (this.docRows[dr].contentStatement.contentManagerView !== "true") {
					continue;
				}
				
				// Start adding the languages.
				var contentStatement = this.docRows[dr].contentStatement;
				for (var i=0; i < addedLangs.length; i++) {
					var elementProvider = this.docRows[dr].getTextContentElementProvider();
					contentStatement.addLanguage(addedLangs[i], isMaster, elementProvider);
					this.docRows[dr].setIsChanged();
				}
			}
	    };
	    
	
		/**
		 * Handling removed languages from project attributes
		 */
	    this.removeLanguageFromAllDocs = function(removedLangs, isMaster) {
	    	
	    	// Remove the language to all document rows.
			for (var dr = 0; dr < this.docRows.length; dr++) {
				
				// Only add if the current row has a text content document.
				if (!this.docRows[dr].contentStatement) {
					continue;
				}
				
				// Only add if the current row is not in a remote project.
				if (this.docRows[dr].isRemoteDocument()) {
					continue;
				}
				
				// Only add if the current row is not in a remote project.
				if (this.docRows[dr].contentStatement.contentManagerView !== "true") {
					continue;
				}
				
				// Start removing the languages.
				var contentStatement = this.docRows[dr].contentStatement;
				for (var i=0; i < removedLangs.length; i++) {
					contentStatement.removeLanguage(removedLangs[i], isMaster);
					this.docRows[dr].setIsChanged();
				}
			}
	    };
	    
	    
	    /**
	     * Recursive function to check if any action is performed in any document that requires password validation.
	     * 
	     * @returns {Boolean} true if password is required, otherwise false.
	     */
	    this.checkIfPasswordRequired = function() {
	    	var isPasswordRequired = false;
	    	
	    	// Iterate through each docrow
			_.each(this.docRows, function(docRow) {
				// check if contentstatement is present and password is required for approval
				if(!isPasswordRequired && docRow && docRow.contentStatement) {
					isPasswordRequired = docRow.contentStatement.isPasswordRequiredForApproval();
				}
				// If password is required do not proceed [skip iteration]
				if(!isPasswordRequired && docRow.aCatObj) {
					_.each(docRow.aCatObj.attributes, function(attribute) {
		    			// If password is not required at current category, check  in attribute category of each of its docRow
		    			if(!isPasswordRequired && attribute.docTable) {
		    				isPasswordRequired = attribute.docTable.checkIfPasswordRequired();
		    			}
		    		});
				}
			});
			
			return isPasswordRequired;
	    };
	    
		/**
		 * Get all the project master languages.
		 */
		 this.getProjectMasterLanguages = function() {
			 return this._projMasterLangs;
		 };
	    
		/**
		 * Get all the project master languages.
		 */
		 this.getProjTranslationLanguages = function() {
			 return this._projTransLangs;
		 };
		 
		 /**
		  * 
		  * Handles request to make given remote document local and replace document to 'this.projectID' project.
		  * 
		  * @param {int} rowIdx - Index of the document row, where is the document to be made local.
		  * @param {String} flowType - This param is required for localRemote Flow. Based on this param docrow object initiated.
		  * @param {String} operation - Operation which is clicked during replace.
		  * @param {String} templateName - TemplateName will be passed only in case of 'DocumentTemplate' (NEW) operation.
		  * @param {DocumentRowNew} docRow - Preinitialized document row based on the "SavedSearch" document source.
		  * @param {Boolean} retainTextContentStatus - When a new TextContent object is requested, should the status of the original be retained? 
		  */
		  if (this.localRemoteButtonClicked) { TextContent.__overridden.localRemoteButtonClicked = this.localRemoteButtonClicked; }
		  this.localRemoteButtonClicked = function (rowIdx, flowType, operation, templateName, searchDocRow, retainTextContentStatus) {
			  this.retainTextContentStatus = retainTextContentStatus;
			  
			  	// Get the affected row.
		        var docRow = this.docRows[rowIdx];
			  
		        // Indicate that we should retain the text content status.
		        if (retainTextContentStatus) {
		        	docRow.retainTextContentStatus = true;
		        } else {
		        	docRow.retainTextContentStatus = false;
		        }
		        
		        // Call the supper implementation.
		        TextContent.__overridden.localRemoteButtonClicked.call(this, rowIdx, flowType, operation, templateName, searchDocRow);
		  };
		 
		 return this;
	};
	
	


	/**
	 * Mixin constructor/initializer.
	 * (class method)
	 */ 
	TextContent.__initMixin = function(options) {
        if (!options) {
            // nothing to initialize from
            return;
        }

		// This variable holds the default language code for each collumn.
		this._defaultSelectedLanguageCodes = {};
		
		// These variables contain the master and translation languages of the project.
		this._projMasterLangs = [];
		this._projTransLangs = [];
		
		// TODO: Make this work for not only document versions but also projects......
		//Get the immediate parent doc. ver. Id.
		this.parentDocVerId = this.parentDocVerIDs[this.parentDocVerIDs.length-1];
	
		TextContentView.__initMixin.call(this.view, this.renderOptions);   // initializer of the added view mixin functionality

		// load additional required info from the server via AJAX (if needed)
		// note: this should be called only after the 'this.view' is properly initialized!
		this._initializeMixinFromServer();
	};

	
	/**
	 * List of overridden methods with their original code.
	 */ 
	TextContent.__overridden = {
	};

	
	return TextContent;
});

define('utils/webcss-text!doctablex/DocTableXRenderConfig.css',[],function () { return '.WcrPageContent #DocRenderConfigTableID .DocXColRow {\r\n\tbackground-color: white;\r\n}\r\n.WcrPageContent #DocRenderConfigTableID .PlaceHolderRow {\r\n\tbackground-color: #F7F7F7;\r\n}\r\n.WcrPageContent #DocRenderConfigTableID .SortableCol {\r\n\tcursor: move;\r\n}\r\n.WcrPageContent #DocRenderConfigTableID .ui-sortable-helper {\r\n\tborder: 1px solid #dddddd;\r\n\tborder-top: 0px;\r\n}';});


/*
 * TODO:
 * 
 * - Refactor localization strings - introduce new "application" for this class
 * - Validation is missing! (not regression)
 * - Reference attribute category "on load" callback doesn't work!
 * 
 * 
 */


define('doctablex/DocTableXRenderConfig',["wcr/webcenter_xml",
		"utils/webi18n!ACatStrings",
		"utils/webi18n!ACLayoutStrings",
		"jquery",
		"underscore",
		"doctablex/AJAXDataCache",
		"jqueryUI",
		"utils/webcss!./DocTableXRenderConfig.css"],
function(WCXML,
		ACatStrings,
		ACLayoutStrings,
		$,
		_,
		AJAXDataCache) {

	/**
	 * DocTableXRenderConfig.js
	 * 
	 * This class implements a class handling rendering options and configuration of a documents table (DocTableX class).
	 * 
	 * @author JANH - September 2015
	 */
	
	"use strict";
	
	var DocTableXRenderConfig = function(options) {
	
		this.defaultName = options.defaultName || '';
		
		this.section = options.section || '';
		
		this.phElemID = options.phElemID || 'DocRenderConfigDivID';
		this.functionPrefix = '$(\'#' + this.phElemID + '\')[0].wrapperObj';

		this.defaultAttrCatView = options.defaultAttrCatView;
		if (!this.defaultAttrCatView && (this.section != 'TaskDocuments')) {
			// in task documents, it's not an option to set 'Default' attr. cat. view by default
			this.defaultAttrCatView = 'Default';
		}
		

        this.dltProperties = ['type','typeDetail','label','visibility','thumbnailSize','useCreationView','uploadNewVersion','attributeName','width', 'displayLanguages', 'langShowMode', 'retainTextContentStatus', 'hideDescription', 'hideSource', 'hideFolder', 'hideDocTemplate', 'hide'];
        this.documentActions = ['View','OpenAcad','Open3d','Download','UploadNewVersion'];
		this.dltTypes = ['DocumentName','DocumentActions','DocumentProperty','ReferenceProperty','DocumentAttribute','AttributeFlapout', 'LocalRemote','SequenceNumber','RowSelector','UpDownArrows','ReplaceButton','CloneButton','DeleteButton'];

        // When the PCM license is active, add options to show the text content cols.
        if (window.webcenterConfig.isPCMEnabled) {
    		this.dltTypes = ['DocumentName','DocumentActions','DocumentProperty','ReferenceProperty','DocumentAttribute','AttributeFlapout','MasterText','TranslationText', 'TextLanguages', 'LocalRemote','SequenceNumber','RowSelector','UpDownArrows','ReplaceButton','CloneButton','DeleteButton'];
        }
        
		if (this.section == 'TaskDocuments') {
			this._removePredefinedOption(this.dltTypes, 'ReferenceProperty');
			this._removePredefinedOption(this.dltTypes, 'DocumentAttribute');
		}

        this.dltDocPropertyDetails = ['Description','VersionRevision','Project','ApprovalStatus','ApprovalStage','VersionAuthor','ModifiedDate','Thumbnail'];
        this.dltDocThumbSizes = ['25','50','75','100','150','200'];
		
        
		if (options.configObj) {
			this.configObj = options.configObj;
			// fix potential incomplete configuration object
			if (!this.configObj.name) this.configObj.name = this.defaultName;
			if (this.configObj.hideheader !== true && this.configObj.hideheader !== false) {
				// note hideHeader should be a real boolean
				this.configObj.hideHeader = false;
			}
			if (this.configObj.useCreationView !== false) {
				// by default the useCreationView is enabled
				this.configObj.useCreationView = true;
			}
			if (!this.configObj.columns) this.configObj.columns = [];
		} else {
			this.configObj = this.getDefaultConfiguration();
		}
		
		this.attributes = options.attributes || [];
		this.properties = options.properties || [];
		
		// flag defining whether to correct attributes (and properties) IDs based on names (== true), 
		// or not (== false)
		this.isImported = options.isImported || false;
		
		this.changeCallback = options.changeCallback;
		
		return this;
	};
	

	/**
	 * This private method is used to remove an option from a specified list of options e.g. this.dltTypes,
	 * if it is not supported in a given context.
	 * 
	 * IMPORTANT: Be aware that no checking whether given option is already "in use" by a configuration
	 * is done! The behaviour is not well defined when a removed option is already defined in an existing
	 * configuration and this configuration is loaded!
	 * 
	 * @param {Array.<String>} optionsList - List of options from which the specified option should be removed.
	 * @param {String} option - Option (its name) to be removed from the list. If it doesn't exist in the list,
	 * 		nothing happens. If it's there more than once, only the first occurrence is removed.
	 */
	DocTableXRenderConfig.prototype._removePredefinedOption = function(optionsList, option) {
		var index = optionsList.indexOf(option);
		if (index > -1) {
			optionsList.splice(index, 1);
		}
	};


	DocTableXRenderConfig.prototype.getConfigObj = function() {
		return this.configObj;
	};

	DocTableXRenderConfig.prototype.getName = function() {
		return this.configObj.name;
	};

	DocTableXRenderConfig.prototype.setName = function(newName) {
		this.configObj.name = newName;
	};

	DocTableXRenderConfig.prototype.getHideHeader = function() {
		return this.configObj.hideHeader;
	};

	DocTableXRenderConfig.prototype.setHideHeader = function(value) {
		this.configObj.hideHeader = (value === true);
	};

	DocTableXRenderConfig.prototype.setAttributes = function(attrs) {
		this.attributes = attrs || [];
	};

	DocTableXRenderConfig.prototype.setProperties = function(attrs) {
		// note an attribute and a property is in fact the same object (Attribute)
		this.properties = attrs || [];
	};

	
	/**
	 * Loads the documents table rendering configuration from specified XML string (xmlString).
	 */
	DocTableXRenderConfig.prototype.readFromXMLString = function(xmlString, callback) {
		if (xmlString) {
			// convert the input XML string to XML object supporting the XML search/manipulation functions
			var xmlObj = WCXML.xmlTextToObj(xmlString);
			var docRenderElem = WCXML.GetFirstFirstLevelChild(xmlObj, "docRender");
			if (docRenderElem) {
				this.readFromXML(docRenderElem, callback);
			}
		}
	};
	
	/**
	 * Loads the documents table rendering configuration from specified XML object (xmlRootElem).
	 * Note just the configObj is stored to and loaded from XML (the rest is contextual info).
	 */
	DocTableXRenderConfig.prototype.readFromXML = function(xmlRootElem, callback) {
		// create empty configuration object to be filled by loaded data
		this.configObj = this.getEmptyConfiguration();
		
		if (!xmlRootElem) return;
		
		// start loading data into the prepared empty configuration object
		this.configObj.name = WCXML.GetElementText(xmlRootElem, "name");
		this.configObj.hideHeader = WCXML.GetElementBoolean(xmlRootElem, "hideHeader");
		var dltArr =  WCXML.GetFirstLevelChilds(xmlRootElem, "dlt");
		if (dltArr && (dltArr.length > 0)) {
			for (var dlt = 0; dlt < dltArr.length; dlt++) {
				var dltElem = dltArr[dlt];
				var dltObj = {};
				for (var idx = 0; idx < this.dltProperties.length; idx++) {
					var dltPropName = this.dltProperties[idx];
					dltObj[dltPropName] = WCXML.GetElementText(dltElem, dltPropName);
				}
				if ((dltObj.type == 'DocumentProperty') && (dltObj.typeDetail == 'Thumbnail') && !dltObj.thumbnailSize) {
					// set default thumbnail size to 50 (only does something on old configurations)
					dltObj.thumbnailSize = '50';
				}
				if ((dltObj.type == 'DocumentProperty') && (dltObj.typeDetail == 'VersionRevision')) {
					if (dltObj.uploadNewVersion === 'false') {
						dltObj.uploadNewVersion = false;
					} else {
						// either true or undefined (default value)
						dltObj.uploadNewVersion = true;
					}
				}
				if (this.isImported && ((dltObj.type == 'DocumentAttribute') || (dltObj.type == 'ReferenceProperty'))) {
					// look for attribute by name
					var attObj = this.findItemByNameOrID(this.attributes, dltObj.attributeName);
					
					if (attObj) {
						// adjust typedetail, so attribute ID is correct
						dltObj.typeDetail = attObj.id;
					} else {
						// did not find attribute, go to the server
						var currentDlt = dltObj;
						currentDlt.typeDetail = "";
						var attrRequest = AJAXDataCache.getInstance().getAttribute(dltObj.attributeName);
						attrRequest.done(_.partial(_.bind(function(attr, dltObj) {
							if (attr) {
								// push attributes in attribute list, to be used further in this session
								attr.hidden = true; // don't display in available attributes
								this.attributes.push(attr);
								
								// adjust typedetail, so attribute ID is correct
								dltObj.typeDetail = attr.id;
							}
							// execute the callback (if any)
							if (callback) callback();
						}, this), _, currentDlt));
						attrRequest.fail(_.bind(function() {
							// attribute not found, should be displayed in error state - execute the callback (if any)
							if (callback) callback();
						}, this));
					}
				}
				if (dltObj.type == 'AttributeFlapout') {
					if (dltObj.useCreationView === 'false'){
						dltObj.useCreationView = false;
					} else {
						// either true or undefined (default value)
						dltObj.useCreationView = true;
					}
				}
				this.configObj.columns.push(dltObj);
			}
		}
	};

	
	
//JANH TODO!!! 'attList' parameter should be deleted!!!
	/**
	 * Saves the documents table rendering configuration to XML
	 * Note just the configObj is stored to and loaded from XML (the rest is contextual info).
	 */
	DocTableXRenderConfig.prototype.convertToXML = function() {
		// start the XML generation immediately
		var xmlcode = '<docRender>';
		xmlcode += WCXML.ElementEncode('name', this.getName());
		xmlcode += WCXML.ElementEncodeBoolean('hideHeader', this.getHideHeader());
		for (var dlt = 0; dlt < this.configObj.columns.length; dlt++) {
			var dltObj = this.configObj.columns[dlt];
			if (dltObj) {
				if (dltObj.type == 'DocumentAttribute' || dltObj.type == 'ReferenceProperty') {
					if (!dltObj.attributeName){
						// recovery for categories where name was never added
						var attObj = this.findItemByNameOrID(this.attributes, null, dltObj.typeDetail); // find by ID
						if (attObj) {
							dltObj.attributeName = attObj.name;
						}
					} else if (this.isImported) {
						// fix ID permanently after export/import
						var attObj = this.findItemByNameOrID(this.attributes, dltObj.attributeName); // find by name
						if (attObj) {
							dltObj.typeDetail = attObj.id;
						}
					}
				}
				
				xmlcode += '<dlt>';
				for (var prop in dltObj) {
					if (!dltObj.hasOwnProperty(prop)) continue;
					if ((dltObj[prop] !== undefined) && (dltObj[prop] !== '') && (dltObj[prop] !== null)) {
						// (note the condition is like this, because boolean value 'false' should be saved)
						xmlcode += WCXML.ElementEncode(prop, dltObj[prop]);
					}
				}
				xmlcode += '</dlt>';
			}
		}
		xmlcode += '</docRender>';	
		
		return xmlcode;
	};
	
	
	/**
	 * Displays the whole documents table render configuration options setup
	 */
	DocTableXRenderConfig.prototype.render = function() {

	    try {
	    	var phElem = $('#' + this.phElemID);
	    	if (phElem.length == 0) return;
	    	
		    var htmlcode = '';
			// checkbox to display the header (note the first parameter of changeDocumentTableRow() method
		    // is ignored in case of 'clickHeaderChk')
			var clickMethod = this.functionPrefix + '.changeDocumentTableRow(0,\'clickHeaderChk\');';
			
			htmlcode += '<div class="add-spacing-left add-spacing-bottom"><div class="checkbox"><label>';
			htmlcode += '<input type="checkbox" id="docRefDialog_HeaderChk" onclick="'+ clickMethod + '"';
			if (this.configObj.hideHeader) htmlcode += ' checked="checked" ';
			htmlcode += '/>' + ACLayoutStrings.t('ACLayoutHideHeader');
			htmlcode += '</label></div></div>';
			
			// the table with columns placeholder
			htmlcode += '<div id="DocRenderConfigTableID"></div>';
			phElem.html(htmlcode);
			if (phElem.length == 1) {
	            // if rendering of the documents table rendering configuration succeeded store the current 
				// DocTableXRenderConfig instance (the configuration wrapper object) itself in DOM 
				// to be accessible by action handlers in generated HTML
				phElem[0].wrapperObj = this;
			}
			
			$('#DocRenderConfigTableID').html(this.displayDocRefTableRenderOptions());
			$('#DocRenderConfigTableID tbody').sortable({
				items: "tr.DocXColRow",
				placeholder: "PlaceHolderRow",
				axis: "y",
				forcePlaceholderSize: true,
				start: function (event, ui) {
					// When dragging, set the height of the helper to better indicate where the row will be placed.
					var height = (ui.item.height()+4)+"px";
					ui.placeholder.height(height);
				}.bind(this),
				stop: function (event, ui) {
					var newCols = [];
					$('#DocRenderConfigTableID tbody > tr.DocXColRow').each(function (i, rowElem) {
						var row = $(rowElem);
						var index = parseInt(row.attr("data-col-index"));
						if (index >= 0) {
							newCols.push(this.configObj.columns[index]);
						}
					}.bind(this));
					this.configObj.columns = newCols;
					this.repaint();
				}.bind(this)
			});

	    } catch(err) {
			alert('Problem in DocTableXRenderConfig.render: ' +  err.message);
			throw err;
		}
	    return htmlcode;
	
	};	    

	/**
	 * Method to repaint the whole doc. table rendering configuration table
	 */
	DocTableXRenderConfig.prototype.repaint = function() {
		this.isInRepaint = true;   //JANH TODO!!! (not used for now)
		this.render();
		this.isInRepaint = false;
	};

	
	/**
	 * Displays the table with columns for document links rendered in a table
	 */
	DocTableXRenderConfig.prototype.displayDocRefTableRenderOptions = function() {
	    var htmlcode = '';
	    try {
			var clickMethod = ''; 
	
	        htmlcode += '<table class="table">';  
	        // (don't move out the table to the caller since then you need to update the content of the table instead of the content of the td, which IE can't do)
	        // labels
	        htmlcode += '<thead>';
	        htmlcode += '<tr>';
	        htmlcode += '<th class="td-narrow"/>';
	        htmlcode += '<th class="td-narrow"/>';
	        htmlcode += '<th>' + ACatStrings.t('ACatType') + '</th>';
	        htmlcode += '<th class="td-narrow">' + ACLayoutStrings.t('ACLayoutDetails') + '</th>';		
	        htmlcode += '<th>' + ACLayoutStrings.t('ACLayoutLabel') + '</th>';		
	        htmlcode += '<th>' + ACLayoutStrings.t('ACLayoutVisibility') + '</th>';
	        htmlcode += '<th>' + ACLayoutStrings.t('ACLayoutWidth') + '</th>';	
	        htmlcode += '</tr>';
	        htmlcode += '</thead>';
	
	        htmlcode += '<tbody>';
	
	        // Display the rows
	        for (var row = 0; row < this.configObj.columns.length; row++) {
	        	// object pointing to 1 row in the doc. table rendering configuration table
	            var dltObj = this.configObj.columns[row];
	            if (dltObj) {
	                htmlcode += this.displayDocumentTableRow(dltObj, row, (row == this.configObj.columns.length - 1));
	            }
	        }
	
			// Link to add a row
			clickMethod =  this.functionPrefix + '.changeDocumentTableRow('+row+',\'Add\'); return false;';
			htmlcode += '<tr>';
			htmlcode += '<td colspan="7" id="docRenderDialogLinks">';
			htmlcode += '<a class="btn btn-default" href="#" onclick="'+clickMethod+'">'+ WCXML.xmlencode(ACLayoutStrings.t('ACLayoutAddDocRefAttr')) +'</a>';
			
			
			// Link to revert the configuration to the WebCenter default
			clickMethod =  this.functionPrefix + '.resetToDefault(true); return false;';
			htmlcode += '<a class="btn btn-default" href="#" onclick="'+clickMethod+'">' + WCXML.xmlencode(ACLayoutStrings.t('ACLayoutRevertToWebCenterDefault')) + '</a>';
			htmlcode += '</td>';
			htmlcode += '</tr>';
	
	        htmlcode += '</tbody>';
	        htmlcode += '</table>';
	        htmlcode += '</td></tr>';
	
	    } catch(err) {
			alert('Problem in DocTableXRenderConfig.displayDocRefTableRenderOptions: ' +  err.message);
			throw err;
		}
	    return htmlcode;
	
	};
	

	/**
	 * Reverts a doc reference table layout configuration to the WebCenter default layout
	 */
	DocTableXRenderConfig.prototype.resetToDefault = function(repaint) {
		this.configObj = this.getDefaultConfiguration();
		
		if (this.changeCallback) {
			// notify about the configuration being changed
			this.changeCallback();
		}
		if (repaint) {
			this.repaint();
		}
	};
	
	/** 
	 * Adds a line to the list of document sources
	 * 
	 * @param dltObj: the object having the data to add
	 * @param row: index in the table
	* @param last: indicates that it's the last row in the table : then don't put the arrow down icon
	*/
	DocTableXRenderConfig.prototype.displayDocumentTableRow = function(dltObj, row, last) {
	    var htmlcode = '';
	    try {	
	        var options;
	
	        // Check if this is a currently supported collumn type
	        if ($.inArray(dltObj.type, this.dltTypes) < 0) {
	        	// This type is not supported, don't generate a row for it.
	        	return htmlcode;
	        }
	        
	        // construct a row
	        htmlcode += '<tr class="DocXColRow" data-col-index="'+row+'">';
	
	        // remove icon, but only if there are at least 2 rows. Also cannot remove document name column
	        htmlcode += '<td class="td-narrow">';
	        if ( !(row == 0 && last) && !(dltObj.type == 'DocumentName')) {  
	            var clickMethod = this.functionPrefix + '.changeDocumentTableRow('+row+',\'Remove\'); return false;';
	            htmlcode += '<i class="icon icon-normal icon-WCR_Remove12" onclick="'+clickMethod+'" title="' + WCXML.xmlencode(ACLayoutStrings.t('REMOVE')) + '"/>';
	        }
	        htmlcode += '</td>';
	
	        // up/down icons
	        htmlcode += '<td class="td-narrow SortableCol">';
	        if (row > 0) {
	            clickMethod =  this.functionPrefix + '.changeDocumentTableRow('+row+',\'Up\'); return false;';
	            htmlcode += '<i class="icon icon-normal icon-WCR_Sort-Up" style="cursor:pointer;" onclick="'+clickMethod+'" title="' + WCXML.xmlencode(ACLayoutStrings.t('MOVEUP')) + '"/>';
	        }	
	        htmlcode += '<br>';
	        if (!last) {
	            clickMethod =  this.functionPrefix + '.changeDocumentTableRow('+row+',\'Down\'); return false;';
	            htmlcode += '<i class="icon icon-WCR_Sort-Down" style="cursor:pointer;" onclick="'+clickMethod+'" title="' + WCXML.xmlencode(ACLayoutStrings.t('MOVEDOWN')) + '"/>';
	        } else {
	            htmlcode += '&nbsp;';
	        }
	        htmlcode += '</td>';
	
	        var changeMethod = this.functionPrefix + '.changeDocumentTableDetail(this)';
	
	        // type
	        htmlcode += '<td>';				
	        htmlcode += '<select id="docLinkTableType_'+ row +'" onchange="'+changeMethod+'" class="form-control">';
	        options = this.dltTypes;
	        for (var opt=0; opt<options.length; opt++) {
	        	var option = options[opt];
	        	if ((this.properties.length == 0) && (option == 'ReferenceProperty')) {
	        		// skip 'ReferenceProperty' option if there is no referenceProperty attr. category defined
	        		continue;
	        	} else if (option == 'Count') {
	        		// skip 'Count' as it is not supported for this version
	        		continue;
	        	}
	        	
	        	 // Add seperator before certain options in order to group them  
	            if (option == 'LocalRemote' || option == 'SequenceNumber' || option == 'MasterText' || option == 'ReplaceButton'){
	            	 htmlcode += this.generateSeperatorToTableRow();
	            }   
	        	
	            htmlcode += '<option value="'+option+'" ';
	            if (dltObj.type == option){
	            	htmlcode += ' selected ';
	            }
	            htmlcode += '>' + ACLayoutStrings.t('ACLayout' + option) + '</option>';
	            
	        
	        }
	        htmlcode += '</select>';				
	        
	        htmlcode += '</td>';
	
	        // Details
	        htmlcode += '<td class="td-nowrap">';
	
	        switch (dltObj.type) {
	            case 'DocumentName':
	            	
	            	var wrapper = $('<div>');
	            	
	            	var showSourceInput = $('<input id="docLinkTableDetail_'+row+'_hideSource" type="checkbox" onchange="'+changeMethod+'" name="hideSource">');
	            	var showSourceLabel = $('<span>').text(ACLayoutStrings.t('ACLayoutNameShowDocSource'));
	            	wrapper.append($('<div>').addClass('checkbox').append($('<label>').append(showSourceInput, showSourceLabel)));
	            	
	            	var showDescriptionInput = $('<input id="docLinkTableDetail_'+row+'_hideDescription" type="checkbox" onchange="'+changeMethod+'" name="hideDescription">');
	            	var showDescriptionLabel = $('<span>').text(ACLayoutStrings.t('ACLayoutNameShowDocDescription'));
	            	wrapper.append($('<div>').addClass('checkbox').append($('<label>').append(showDescriptionInput, showDescriptionLabel)));
	            	
	            	var showFolderInput = $('<input id="docLinkTableDetail_'+row+'_hideFolder" type="checkbox" onchange="'+changeMethod+'" name="hideFolder">');
	            	var showFolderLabel = $('<span>').text(ACLayoutStrings.t('ACLayoutNameShowFolder'));
	            	wrapper.append($('<div>').addClass('checkbox').append($('<label>').append(showFolderInput, showFolderLabel)));
	            	
	            	var showDocTemplateInput = $('<input id="docLinkTableDetail_'+row+'_hideDocTemplate" type="checkbox" onchange="'+changeMethod+'" name="hideDocTemplate">');
	            	var showDocTemplateLabel  = $('<span>').text(ACLayoutStrings.t('ACLayoutNameShowDocTemplate'));
	            	wrapper.append($('<div>').addClass('checkbox').append($('<label>').append(showDocTemplateInput, showDocTemplateLabel)));
	            	
	            	// Check the correct inputs.
	            	if (dltObj.hideSource != "true") {
	            		showSourceInput.attr('checked','checked');
	            	}
	            	if (dltObj.hideDescription != "true") {
	            		showDescriptionInput.attr('checked','checked');
	            	}
	            	if (dltObj.hideFolder != "true") {
	            		showFolderInput.attr('checked','checked');
	            	}
	            	if (dltObj.hideDocTemplate != "true") {
	            		showDocTemplateInput.attr('checked','checked');
	            	}
	            	
	            	htmlcode += wrapper.prop('outerHTML');
	                break;
	                
	            case 'DocumentActions':
	            	var addMethod =  this.functionPrefix + '.addDocRefDocAction('+row+'); return false;';
	
	            	var values = dltObj.typeDetail.split("|");
	            	
	            	htmlcode += '<div class="documentactions">';
	            	for(var i=0 ; i<values.length ; i++){
	            		var removeMethod =  this.functionPrefix + '.removeDocRefDocAction('+row+', '+i+'); return false;';
	                	var changeValueMethod =  this.functionPrefix + '.changeDocRefDocAction(this, '+row+', '+i+');';
	                	
	                	htmlcode += '<div class="input-group">';
	                	
	            		htmlcode += '<select onchange="'+changeValueMethod+'" class="form-control">';
	            		htmlcode += '<option value="">---</option>';
	            		for(var j=0 ; j<this.documentActions.length ; j++){
	            			var selectedStr = (this.documentActions[j] == values[i])?'selected="selected" ':'';
	            			htmlcode += '<option '+selectedStr+'value="'+this.documentActions[j]+'">'+WCXML.xmlencode(ACLayoutStrings.t('ACLayoutDocAction'+this.documentActions[j]))+'</option>';
	            		}
	            		htmlcode += '</select>';
	            		if(values.length > 1){
	            			htmlcode += '<a class="input-group-btn add-padding-left" href="#" onclick="'+removeMethod+'"><i class="icon icon-normal icon-WCR_Remove12"></i></a>';
	            		}
	            		
	            		htmlcode += '</div>';
	            	}
	            	
	            	htmlcode += '</div>';
	            	
	            	
	                htmlcode += '<a class="btn btn-default add-spacing-top" href="#" onclick="'+addMethod+'">'+ACLayoutStrings.t('ACLayoutAddAction')+'</a>';
	                break;
	                
	            case 'DocumentProperty':	
	                options = this.dltDocPropertyDetails;
	                htmlcode += '<div class="input-group">';
	                htmlcode += '<select id="docLinkTableDetail_'+row+'" onchange="'+changeMethod+'" class="form-control">';
	                for (var opt=0; opt<options.length; opt++) {
	                    htmlcode += '<option value="'+options[opt]+'" ';
	                    if (dltObj.typeDetail == options[opt]) htmlcode += ' selected ';
	                    htmlcode += '>' + ACLayoutStrings.t('DocRefColLabel_' + options[opt]) + '</option>';
	                }
	                htmlcode += '</select>';	
					
					// thumbnail size
					if (dltObj.typeDetail == 'Thumbnail') {
						options = this.dltDocThumbSizes;
						htmlcode += '<div class="input-group-addon-select"><select id="docLinkTableThumbnail_'+row+'"  onchange="'+changeMethod+'">';					                    
						for (var opt=0; opt<options.length; opt++) {
							htmlcode += '<option value="'+options[opt]+'" ';
							if (dltObj.thumbnailSize == options[opt]) htmlcode += ' selected ';
							htmlcode += '>'+ options[opt] + '</option>';
						}
						htmlcode += '</select></div><label class="input-group-addon" for="docLinkTableThumbnail_'+row+'">px</label>';
						
						htmlcode += '</div>';
						
					} else if (dltObj.typeDetail == 'VersionRevision') {
						// checkbox for upload new version
						
						htmlcode += '</div>';
						
	                    htmlcode += '<div class="checkbox"><label>';
	                    htmlcode += '<input type="checkbox" id="docLinkTableUploadNewVersion_'+row+'" ' 
	                    		+ (dltObj.uploadNewVersion === true ? 'checked="checked"' : '') 
	                    		+ 'onchange="'+changeMethod+'"/>';
	                    htmlcode += ACLayoutStrings.t('ACLayoutAllowUploadNewVersion');
	                    htmlcode += '</label></div>';
	                    break;
					}
					
	                break;
	                
	            case 'AttributeFlapout':
	                if (!dltObj.typeDetail) dltObj.typeDetail = this.defaultAttrCatView;
	                htmlcode += '<div class="input-group">';
	                htmlcode += '<label class="input-group-addon" for="docLinkTableDetail_'+row+'">'+ ACLayoutStrings.t('ACLayoutView') + '</label>';
	                htmlcode += '<input type="text" id="docLinkTableDetail_'+row+'" value="' + dltObj.typeDetail +'" onchange="'+changeMethod+'" class="form-control"/>';
	                htmlcode += '</div>';
	                
	                // checkbox for creation view
	                htmlcode += '<div class="checkbox"><label>';
	                htmlcode += '<input type="checkbox" id="docLinkTableUseCreationView_'+row+'" ' 
	                		+ (dltObj.useCreationView === true ? 'checked="checked"' : '') 
	                		+ 'onchange="'+changeMethod+'"/>';
	                htmlcode += ACLayoutStrings.t('ACLayoutUseCreationView');
	                htmlcode += '</label></div>';
	                break;
	                
	            case 'ReferenceProperty':		
	                htmlcode += '<select id="docLinkTableDetail_'+row+'" onchange="'+changeMethod+'" class="form-control">';					
	                if (this.properties.length > 0) {
	                	var currentAttr = this.findItemByNameOrID(this.properties, dltObj.attributeName, dltObj.typeDetail);
	                    if (!currentAttr){
	                    	currentAttr = {}; // empty object for comparison
	                    }
	                	
	                    options = this.properties;  // linked properties (== attributes of reference properties attribute category)
	                    for (var opt=0; opt<options.length; opt++) {
	                    	var aProp = options[opt];
	                    	if (aProp && !aProp.isDocReference() && !aProp.isRichText()) {
	                            htmlcode += '<option value="'+aProp.id+'" ';
	                            if (currentAttr.id == aProp.id) htmlcode += ' selected ';
	                            htmlcode += '>'+ aProp.name + '</option>';
	                    	}
	                    }
	                }
                    htmlcode += '</select>';	
	                break;
	                
	            case 'DocumentAttribute':	
	                htmlcode += '<select id="docLinkTableDetail_'+row+'"  onchange="'+changeMethod+'" class="form-control">';	
	                
	                if (this.attributes.length > 0) {
		                var currentAttr = this.findItemByNameOrID(this.attributes, dltObj.attributeName, dltObj.typeDetail);
		                if (!currentAttr){
		                	currentAttr = {}; // empty object for comparison
		                }
		
		                for (var opt = 0; opt < this.attributes.length; opt++) {
		                	var anAttr = this.attributes[opt];
		            		// rich text attributes are not supported here - filter them out
		                	if (anAttr && anAttr.isRichText()) continue;
		
		                	htmlcode += '<option value="' + anAttr.id +'" ';
		                    if (currentAttr.id == anAttr.id){
		                    	htmlcode += ' selected ';
		                    }
		                    htmlcode += '>'+ WCXML.xmlencode(anAttr.name) + '</option>';
		                }
	                }
	                htmlcode += '</select>';			
	                break;
	
	            case 'MasterText' :
		            var defLangCode = dltObj.typeDetail ? dltObj.typeDetail : "";
		            htmlcode += '<input placeholder="Default language code" id="docLinkTableDetail_'+row+'" type="text" onchange="'+changeMethod+'" value="'+defLangCode+'" class="form-control">';
	            	break;
	            	
	            case 'TranslationText' :
		            var defLangCode = dltObj.typeDetail ? dltObj.typeDetail : "";
		            htmlcode += '<input placeholder="Default language code" id="docLinkTableDetail_'+row+'" type="text" onchange="'+changeMethod+'" value="'+defLangCode+'" class="form-control">';
	            	break;
	            
	            case 'TextLanguages' :
	            	htmlcode += '<div class="input-group" style="padding-bottom: 2px;">';
	            	htmlcode += '<label class="input-group-addon" for="docLinkTableDetail_'+row+'_show_for">'+WCXML.xmlencode(ACLayoutStrings.t("ACLayoutTextLanguagesShowFor"))+'</label>';
	            	htmlcode += '<select id="docLinkTableDetail_'+row+'_show_for" name="displayLanguages" onchange="'+changeMethod+'" class="form-control">';
	            	if (dltObj.displayLanguages == "master") {
	            		htmlcode += '<option value="master" selected="selected">'+WCXML.xmlencode(ACLayoutStrings.t("ACLayoutTextLanguagesForMaster"))+'</option>';
	            	} else {
	            		htmlcode += '<option value="master">'+WCXML.xmlencode(ACLayoutStrings.t("ACLayoutTextLanguagesForMaster"))+'</option>';
	            	}
	            	if (dltObj.displayLanguages == "translation") {
	            		htmlcode += '<option value="translation" selected="selected">'+WCXML.xmlencode(ACLayoutStrings.t("ACLayoutTextLanguagesForTranslation"))+'</option>';
	            	} else {
	            		htmlcode += '<option value="translation">'+WCXML.xmlencode(ACLayoutStrings.t("ACLayoutTextLanguagesForTranslation"))+'</option>';
	            	}
	            	htmlcode += '</select>';
	            	htmlcode += '</div>';
	            	
	            	htmlcode += '<div class="input-group" style="padding-top: 2px;">';
	            	htmlcode += '<label class="input-group-addon" for="docLinkTableDetail_'+row+'_mode">'+WCXML.xmlencode(ACLayoutStrings.t("ACLayoutTextLanguagesMode"))+'</label>';
	            	htmlcode += '<select id="docLinkTableDetail_'+row+'_mode" name="langShowMode" onchange="'+changeMethod+'" class="form-control">';
	            	if (dltObj.langShowMode == "compact") {
	            		htmlcode += '<option value="compact" selected="selected">'+WCXML.xmlencode(ACLayoutStrings.t("ACLayoutTextLanguagesCompact"))+'</option>';
	            	} else {
	            		htmlcode += '<option value="compact">'+WCXML.xmlencode(ACLayoutStrings.t("ACLayoutTextLanguagesCompact"))+'</option>';
	            	}
	            	if (dltObj.langShowMode == "normal") {
	            		htmlcode += '<option value="normal" selected="selected">'+WCXML.xmlencode(ACLayoutStrings.t("ACLayoutTextLanguagesNormal"))+'</option>';
	            	} else {
	            		htmlcode += '<option value="normal">'+WCXML.xmlencode(ACLayoutStrings.t("ACLayoutTextLanguagesNormal"))+'</option>';
	            	}
	            	htmlcode += '</select>';
	            	htmlcode += '</div>';
	            	break;
	            	
	            case 'LocalRemote' :
	            	
	            	if (window.webcenterConfig.isPCMEnabled) {
	            		htmlcode += '<div class="checkbox">';
		            	htmlcode += '<label>';
		            	if (dltObj.retainTextContentStatus === "true") {
		            		htmlcode += '<input type="checkbox" name="retainTextContentStatus" id="docLinkTableDetail_'+row+'" onchange="'+changeMethod+'" checked="checked" />';		            		
		            	} else {
		            		htmlcode += '<input type="checkbox" name="retainTextContentStatus" id="docLinkTableDetail_'+row+'" onchange="'+changeMethod+'" />';
		            	}
		            	htmlcode += 'Retain text document status';
		            	htmlcode += '</label>';
		            	htmlcode += '</div>';
	            	}
	            	break;
	            	
	            default:
	                // do nothing
	        }
	
	
	        htmlcode += '</td>';
	
	        // Label
	        htmlcode += '<td>';
	        if (dltObj.type == 'AttributeFlapout') {
	        	// not editable '+' symbol
	        	htmlcode += '+';
	        } else if (dltObj.type == 'UpDownArrows' || dltObj.type == 'DeleteButton' || dltObj.type == 'ReplaceButton' || dltObj.type == 'CloneButton'
	        		|| dltObj.type == 'LocalRemote' || dltObj.type == 'RowSelector') {
				// put nothing
	        } else {
	        	// by default the label contains language tag, but customized labels contains the value itself
	        	var labelStr = '';
	        	if (dltObj.label.indexOf('DocRefColLabel_') === 0) {
	        		// 'label' contains a translation tag - use it
	        		labelStr = ACLayoutStrings.t(dltObj.label);
	        	} else if (ACLayoutStrings.tFallback('DocRefColLabel_' + dltObj.label, '')) { // fall back on '' if it doesn't exist, or we will see the label with 'DocRefColLabel_'
	        		// for back-compatibility: try to prepend the translation tag prefix and see if it exists
	        		labelStr = ACLayoutStrings.t('DocRefColLabel_' + dltObj.label);
	        	}
	        	if (!labelStr) {
	        		// not supported translation tag or a custom label
	        		labelStr = dltObj.label;
	        	}
	        	// the label is editable
	        	htmlcode += '<input type="text" id="docLinkTableLabel_'+ row +'" value="'+ WCXML.xmlencode(labelStr) +'" onchange="'+changeMethod+'" class="form-control">';
	        }
	        htmlcode += '</td>';
	
	        // Visibility	
	        htmlcode += '<td>';				
	
	        if (dltObj.type == 'ReferenceProperty' || dltObj.type == 'DocumentAttribute' || dltObj.type == 'Count' 
	        		|| dltObj.type == 'AttributeFlapout' || dltObj.type == 'LocalRemote' 
	        		|| dltObj.type == 'MasterText' || dltObj.type == 'TranslationText' || dltObj.type == 'TextLanguages') {
	            htmlcode += '<select id="docLinkTableVisibility_'+ row +'" onchange="'+changeMethod+'" class="form-control">';
	            if (dltObj.type ==  'AttributeFlapout' || dltObj.type == 'LocalRemote'  
	            	|| dltObj.type == 'MasterText' || dltObj.type == 'TranslationText' || dltObj.type == 'TextLanguages') {
	                options = ['Editable','ReadOnly'];	
	            } else if (dltObj.type == 'ReferenceProperty') {
	                options = ['Editable','Required','ReadOnly','Hidden'];
	            } else {
	                options = ['Editable','Required','ReadOnly'];
	            }
	
	            for (var opt=0; opt<options.length; opt++) {
	                htmlcode += '<option value="'+options[opt]+'" ';
	                if (dltObj.visibility == options[opt]) htmlcode += ' selected ';
	                htmlcode += '>'+ ACLayoutStrings.t('ACLayout' + options[opt]) + '</option>';
	            }
	            htmlcode += '</select>';	
	        } else {
	            dltObj.visibility = 'ReadOnly';  // probably should not be here, mixture of controller with view!
	            htmlcode += ACLayoutStrings.t('ACLayoutReadOnly');
	        } 
	        htmlcode += '</td>';
	        

	        // Width of DocTableX columns settings
	        htmlcode += '<td>';

        	if (dltObj.visibility != 'Hidden') {
		        var widthMode = 'none';
		        switch (dltObj.type) {
			        case 'DocumentName':
			        case 'ReferenceProperty':
			        case 'DocumentAttribute':
			        case 'MasterText':
			        case 'TranslationText':
			        case 'TextLanguages':
			        	if (dltObj.width) {
				        	widthMode = 'editableChanged';
			        	} else {
				        	widthMode = 'editableDefault';
			        	}
			        	break;
		
			        case 'DocumentProperty':
			        	switch (dltObj.typeDetail) {
			        	case 'Description':
			        	case 'Project':
			        	case 'ApprovalStage':
			        	case 'VersionAuthor':
			        	case 'ModifiedDate':
			        	if (dltObj.width) {
					        	widthMode = 'editableChanged';
				        	} else {
					        	widthMode = 'editableDefault';
				        	}
				        	break;
			        	}
			        	break;
		        }
		        // render HTML based on the calculated "mode" of the width edit field
	        	if (widthMode == 'editableChanged') {
			        	htmlcode += '<input type="text" id="docLinkTableWidth_'+ row +'" onchange="'+changeMethod+'" value="'+dltObj.width+'" class="form-control"/>';
	        	} else if (widthMode == 'editableDefault') {
		        		htmlcode += '<input type="text" id="docLinkTableWidth_'+ row +'" onchange="'+changeMethod+'" value="auto" class="form-control"/>';
		        } else {
		        	htmlcode += 'auto';
		        }
        	}
	        htmlcode += '</td>';
	        
	        //Close it up
	        htmlcode += '</tr>';
	
	
	        return htmlcode;
	
	    } catch(err) {
	        alert('Problem in DocTableXRenderConfig.displayDocumentTableRow: ' + err.message);
	        throw err;
	    }
	
	};
	
	
	/**
	 * Add a seperator to the list of options
	 */
	DocTableXRenderConfig.prototype.generateSeperatorToTableRow = function(){
		return "<option disabled>-------------</option>";
	};
	
	
	/**
	* Handles row operations on the document sources table 
	* @param idx: index of the row on which the operation is done (null for operation Add)
	* @param operation: one of 'Add','Remove','Up','Down'
	*/
	DocTableXRenderConfig.prototype.changeDocumentTableRow = function(idx, operation) {
		var configObj = this.configObj;
		if (!configObj || !configObj.columns) {
			configObj = this.getEmptyConfiguration();
		}
        switch (operation) {
            case 'Add':
                var dltObj = {};
                dltObj.type = 'DocumentProperty';	
                dltObj.typeDetail = 'Description';
                dltObj.label = 'DocRefColLabel_Description';	
                dltObj.visibility = 'ReadOnly';
                configObj.columns.push(dltObj);		
                break;

            case 'Remove':
                if (configObj.columns.length > idx) configObj.columns.splice(idx,1);
                break;

            case 'Up':
                if (idx > 0 && configObj.columns.length > idx) {
                    var dltObj = configObj.columns[idx];  // remember the object
                    configObj.columns.splice(idx,1); // take it out of the Array
                    configObj.columns.splice(idx-1,0,dltObj);  // put it back 1 up
                }
                break;

            case 'Down':
                if (configObj.columns.length > idx + 1) {
                    var dltObj = configObj.columns[idx];  // remember the object
                    configObj.columns.splice(idx,1); // take it out of the Array
                    configObj.columns.splice(idx+1,0,dltObj);  // put it back 1 down
                }
                break;

			case 'clickHeaderChk':
				configObj.hideHeader = ($('#docRefDialog_HeaderChk').attr('checked'));
				break;
            default:
                throw new Error ('Unknown operation : ' + operation);
        }

	    try {
			if (operation != 'clickHeaderChk') {
				// repaint the table of the dialog
	            var phElem = $('#' + this.phElemID);
	            if (phElem.length > 0) {
	            	this.repaint();
	            	
	            	// handle dialogs with too many rows to fit on a screen
	            	var dialogElem = $('#docRenderDialog');
	            	if (dialogElem.length > 0) {
		    			// get the height of the browser's window and adapt the max. dialog height according to it.
		            	var viewportHeight = $(window).height();
		            	var maxHeight = viewportHeight - viewportHeight / 10;
		            	if (maxHeight < 400) maxHeight = 400;
		            	if (dialogElem.height() > maxHeight - 100) {
			            	// resize the dialog to maximal height that still fits on the given screen
		            		dialogElem.dialog('option', 'height', maxHeight);
		            	}
		            	if (operation == 'Add') {
			            	// scroll to the very bottom of the dialog content in order to still see the "Add" 
			            	// button, but only if Add button was clicked!
		            		dialogElem.scrollTop(dialogElem[0].scrollHeight);
		            	}
		            	if(!dialogElem.attr('moved')){
			        		dialogElem.dialog("option", "position", "center");
			        	}
	            	}
	            }
			}
	
	        // repaint the preview is not needed since it is repainted when the dialog closes.
	        
			if (this.changeCallback) {
				// notify about the configuration being changed
				this.changeCallback();
			}
	
	    } catch(err) {
	        alert('Problem in DocTableXRenderConfig.changeDocumentTableRow: ' + err.message);
	        throw err;
	    }
	
	};
	
	
	/**
	 * Handles all changes to the document reference table pop-up
	 * @param {DOMObj} elem - the DOM element which was changed. It has an id looking like property_row
	 */
	DocTableXRenderConfig.prototype.changeDocumentTableDetail = function(elem) {
	    try {
	    	var repaintNeeded = false;
	    	if (!elem) throw new Error('elem is not defined');

	    	var configObj = this.configObj;
	    	
	    	// find row and property from the id which looks like 'property_row'
	    	var id = elem.id;
	    	var idArr = id.split('_');		
	    	var property = idArr[0];
	    	var row = idArr[1];  
	
	    	var dltObj = configObj.columns[row];
	    	if (!dltObj) {
	    		throw new Error('No dltObj for row ' + row);
	    	}
	
	    	switch (property) {
	    	case 'docLinkTableType':
	    		dltObj.type = elem.value;
	    		dltObj.typeDetail = '';   // reset type detail
	    		switch (dltObj.type) {
	    			case 'DocumentName':
	    				dltObj.label = 'DocRefColLabel_DocumentSpec';
	    				dltObj.width = 'auto';
	                    break;	
	                case 'DocumentActions':
	                    dltObj.label = 'DocRefColLabel_DocumentActions';  
	                    dltObj.typeDetail = 'Open3d|View|OpenAcad|Download';
	                    break;	
	                case 'AttributeFlapout':
	                	dltObj.typeDetail = this.defaultAttrCatView;
	                	dltObj.label = '';
	                	dltObj.visibility = 'ReadOnly';
	                	dltObj.useCreationView = true;
	                	break;	
	                case 'DocumentProperty':
	                	dltObj.typeDetail = 'Description';
	                	dltObj.label = 'DocRefColLabel_Description';
	                	break;
	                case 'DocumentAttribute':
	                 	// find first non-richtext, non-reference attribute
	                 	var attInd = 0;
	                 	for (attInd = 0; attInd < this.attributes.length; attInd++) {
	                 		var anAttr = this.attributes[attInd];
	                     	if (anAttr && !anAttr.isRichText() && !anAttr.isDocReference()){
	                     		break;
	                     	}
	                 	}
	                 	// fill out typedetail and label of first attribute
	                 	if(attInd < this.attributes.length){
	                 		dltObj.typeDetail = this.attributes[attInd].id;
	                 		dltObj.label = this.attributes[attInd].name;
	                 		dltObj.attributeName = this.attributes[attInd].name;
	                 	}              
	                 	break;	
	                case 'ReferenceProperty':
	                	if (this.properties.length > 0) {
	                		var i = 0;
	                     	// fill out first element from the list, which will be the first non-richtext and non-docref attribute
	                     	while (i < this.properties.length) {
	                     		if (this.properties[i].isDocReference() || this.properties[i].isRichText()) {
	                     			i++;
	                     		} else {
	                     			break;
	                     		}
	                     	}
	                     	if (i < this.properties.length) {
	                     		var firstProp = this.properties[i];
	                         	dltObj.typeDetail = firstProp.id;
	                         	dltObj.label = firstProp.name;
	                         	dltObj.attributeName = firstProp.name;
	                     	}
	                	}
	                	dltObj.visibility = 'Editable';
	                	break;	
	                case 'Count':
	                	dltObj.label = 'DocRefColLabel_Count';  
	                	break;	
	                case 'SequenceNumber':
	                	dltObj.label = '#';  
	                	break;	
	                case 'MasterText':
	                	dltObj.label = 'DocRefColLabel_MasterText';  
	                	dltObj.visibility = 'Editable';
	                	dltObj.width = 'auto';
	                	break;	
	                case 'TranslationText':
	                	dltObj.label = 'DocRefColLabel_TranslationText';  
	                	dltObj.visibility = 'Editable';
	                	dltObj.width = 'auto';
	                	break;
	                case 'TextLanguages':
	                	dltObj.label = 'DocRefColLabel_TextLanguages';  
	                	dltObj.visibility = 'Editable';
	                	break;
	                default:
	                	dltObj.label = '';
	                	break;	
	    		}
	    		repaintNeeded = true; 
	    		break;
	    	case 'docLinkTableDetail':
	    		dltObj.typeDetail = elem.value;	
	    		// set the default label
	    		switch (dltObj.type) {
	    		
	    			case 'DocumentName':
	    				dltObj[elem.name] = !elem.checked+"";
	    				break;	
	    			case 'DocumentProperty':
	    				dltObj.label = 'DocRefColLabel_' + dltObj.typeDetail;
	    				break;		
	    			case 'DocumentAttribute':
	    				if (this.attributes.length > 0) {
	    					var docAttObj = this.findItemByNameOrID(this.attributes, null, dltObj.typeDetail);   // typeDetail contains the attribute ID
	    					dltObj.attributeName = docAttObj.name;
	    					dltObj.label = dltObj.attributeName;
	    				}
	    				break;						
	    			case 'ReferenceProperty':
	    				if (this.properties.length > 0) {
	    					var refAttObj = this.findItemByNameOrID(this.properties, null, elem.value);
	    					dltObj.attributeName = refAttObj.name;
	    					dltObj.label = refAttObj.name;
	    				}			
	    				break;
	                case 'MasterText':
	                	break;	
	                case 'TranslationText':
	                	break;
	    			case 'TextLanguages':
	    				dltObj[elem.name] = elem.value;
	    				break;
	    			case 'LocalRemote' :
	    				dltObj[elem.name] = elem.checked+"";
	    				break;
	    			default:
	    				dltObj.label = '';
	    		}
				// set or reset thumbnail size
				if (dltObj.type == 'DocumentProperty' && dltObj.typeDetail == 'Thumbnail') {
					dltObj.thumbnailSize = '50';
				} else {
					dltObj.thumbnailSize = '';
				}
	            repaintNeeded = true; 
	            break;
			case 'docLinkTableThumbnail':
				dltObj.thumbnailSize = elem.value;					
				break;
			case 'docLinkTableUploadNewVersion':
				dltObj.uploadNewVersion = ($(elem).attr('checked') === "checked");					
				break;
			case 'docLinkTableLabel':
				dltObj.label = elem.value;
				break;
			case 'docLinkTableVisibility':
				if ((dltObj.visibility != elem.value) && ((dltObj.visibility == 'Hidden') || (elem.value == 'Hidden'))) {
					repaintNeeded = true;
				}
				dltObj.visibility = elem.value;
				break;
			case 'docLinkTableUseCreationView':
				dltObj.useCreationView = ($(elem).attr('checked') === "checked");					
				break;
			case 'docLinkTableWidth' :
				// Check if the value is of format '60'
				if(/^([0-9]+)$/.exec(elem.value)) {
					elem.value = elem.value+"px";
					dltObj.width = elem.value;
					break;
				}
				// Check if the value is of format '60px'
				if(/^([0-9]+)px$/.exec(elem.value)) {
					dltObj.width = elem.value;
					break;
				}
				// Check if the value is of format '60%'
				if(/^([0-9]+)%$/.exec(elem.value)) {
					dltObj.width = elem.value;
					break;
				}
				// Check if the value is 'auto'.
				if (elem.value.toUpperCase().trim() === 'AUTO') {
					dltObj.width = elem.value;
					break;
				}
				// Not a valid format, set to auto.
				elem.value = 'auto';
				dltObj.width = elem.value;
				break;
			default:
				throw new Error('unknown property : ' + property);
	    	}

			if (this.changeCallback) {
				// notify about the configuration being changed
				this.changeCallback();
			}
	
	    	// repaint the document source table in the cell details			
	    	if (repaintNeeded) {
	    		this.repaint();
	    	}
	     } catch(err) {
	         alert('Problem in DocTableXRenderConfig.changeDocumentTableDetail: ' + err.message);
	         throw err;
	     }
	
	 };
	 
	/**
	 * Add option to document actions configuration
	 * The actions are pipe-separated, so we're just adding a pipe in the end
	 */
	DocTableXRenderConfig.prototype.addDocRefDocAction = function(row) {
		var dltObj = this.configObj.columns[row];
		if (!dltObj) throw new Error('No dltObj for row ' + row);
		
		dltObj.typeDetail += "|";

		if (this.changeCallback) {
			// notify about the configuration being changed
			this.changeCallback();
		}
		this.repaint();
	};
	 
	/**
	 * change option in document actions configuration
	 */
	DocTableXRenderConfig.prototype.changeDocRefDocAction = function(elem, row, actionIndex) {
	    var dltObj = this.configObj.columns[row];
	    if (!dltObj) throw new Error('No dltObj for row ' + row);
	      
	    var actions = dltObj.typeDetail.split("|");
		if(actions.length > actionIndex){
			actions[actionIndex] = $(elem).val();
		}
		dltObj.typeDetail = actions.join("|");
	
		if (this.changeCallback) {
			// notify about the configuration being changed
			this.changeCallback();
		}
		this.repaint();
	};
	
	/**
	 * Remove an action form the document actions configuration for doc references
	 */
	DocTableXRenderConfig.prototype.removeDocRefDocAction = function(row, actionIndex) {
		var dltObj = this.configObj.columns[row];
		if (!dltObj) throw new Error('No dltObj for row ' + row);
		  
		var actions = dltObj.typeDetail.split("|");
		if(actions.length > actionIndex){
			actions.splice(actionIndex, 1);
		}
		dltObj.typeDetail = actions.join("|");
		
		if (this.changeCallback) {
			// notify about the configuration being changed
			this.changeCallback();
		}
		this.repaint();
	};
	
	
	/**
	 * Creates an empty documents table configuration.
	 */
	DocTableXRenderConfig.prototype.getEmptyConfiguration = function() {
		var emptyConfig = {};
		emptyConfig.name = this.defaultName || '';
		emptyConfig.columns = [];
		emptyConfig.hideHeader = false;
		emptyConfig.useCreationView = true;
		return emptyConfig;
	};

	/**
	 * Creates a default documents table configuration.
	 * To be called each time a new document reference is set to table rendering
	 */
	DocTableXRenderConfig.prototype.getDefaultConfiguration = function() {
		var defaultConfig = this.getEmptyConfiguration();
		var dltObj = {};
		
		if (this.section == 'TaskDocuments') {
			// selector row - specific for TEP page
			dltObj = {};
			dltObj.type = 'RowSelector';			
			dltObj.label = '';
			dltObj.visibility = 'ReadOnly';
			defaultConfig.columns.push(dltObj);	
		}
		
		if ((this.section == 'AttributeCategory') || (this.section == 'TaskDocuments')) {
			//  Attribute +
			dltObj = {};
			dltObj.type = 'AttributeFlapout';			
			dltObj.label = '';	  // label is rendered explicitely as a read-only '+' symbol
			dltObj.typeDetail = this.defaultAttrCatView;
			if (this.section == 'TaskDocuments') {
				// documents linked to a task can be by default unlinked etc. (if logged in user has corresponding rights)
				dltObj.visibility = 'Editable';
			} else {
				dltObj.visibility = 'ReadOnly';
			}
			dltObj.useCreationView = true;
			defaultConfig.columns.push(dltObj);	
			
			if (this.section != 'TaskDocuments') {
				// Up/down arrows
				dltObj = {};
				dltObj.type = 'UpDownArrows';			
				dltObj.label = '';
				dltObj.visibility = 'ReadOnly';
				defaultConfig.columns.push(dltObj);	
			}
			
			// Thumbnail
			dltObj = {};
			dltObj.type = 'DocumentProperty';	
			dltObj.typeDetail = 'Thumbnail';
			dltObj.thumbnailSize = '50'; 	// always a string
			dltObj.label = 'DocRefColLabel_Thumbnail';	
			dltObj.visibility = 'ReadOnly';
			defaultConfig.columns.push(dltObj);
			
			//Document Name
			dltObj = {};
			dltObj.type = 'DocumentName';			
			dltObj.label = 'DocRefColLabel_DocumentSpec';	
			dltObj.visibility = 'ReadOnly';
			defaultConfig.columns.push(dltObj);
			
			//Document Actions
			dltObj = {};
			dltObj.type = 'DocumentActions';			
			dltObj.typeDetail = 'Open3d|View|OpenAcad|Download';
			dltObj.label = 'DocRefColLabel_DocumentActions';	
			defaultConfig.columns.push(dltObj);		
			
			// Version
			dltObj = {};
			dltObj.type = 'DocumentProperty';	
			dltObj.typeDetail = 'VersionRevision';
			dltObj.label = 'DocRefColLabel_VersionRevision';	
			dltObj.visibility = 'ReadOnly';
			dltObj.uploadNewVersion = true;
			defaultConfig.columns.push(dltObj);
	
			// Approval status
			if (this.section == 'TaskDocuments') {
				dltObj = {};
				dltObj.type = 'DocumentProperty';	
				dltObj.typeDetail = 'ApprovalStatus';
				dltObj.label = 'DocRefColLabel_ApprovalStatus';	
				dltObj.visibility = 'ReadOnly';
				defaultConfig.columns.push(dltObj);
			}
	
			// Delete button
			dltObj = {};
			dltObj.type = 'DeleteButton';					
			dltObj.label = '';	
			dltObj.visibility = 'ReadOnly';
			defaultConfig.columns.push(dltObj);
		}
		
		if(this.section == 'UploadTool'){			
			//Document Name
			dltObj = {};
			dltObj.type = 'DocumentName';			
			dltObj.label = 'DocRefColLabel_DocumentSpec';	
			dltObj.visibility = 'ReadOnly';
			defaultConfig.columns.push(dltObj);
			
			// Delete button
			dltObj = {};
			dltObj.type = 'DeleteButton';					
			dltObj.label = '';	
			dltObj.visibility = 'ReadOnly';
			defaultConfig.columns.push(dltObj);
		}
		
		return defaultConfig;
	};


	/**
	 * A general method to find an item in an array of items by name or ID. 
	 * Note name has priority.
	 */
	DocTableXRenderConfig.prototype.findItemByNameOrID = function(items, name, id) {
        var foundItem = null;
        if (name) {
        	var lowerCaseName = name.toLowerCase();
            for (var i = 0; i < items.length; i++) {
                if (items[i].name.toLowerCase() == lowerCaseName) {
                    foundItem = items[i];
                    break; // jump out of the for loop
                }
            }
        }
        if (id && !foundItem) {  // not found, try with the ID
            for (var i = 0; i < items.length; i++) {
                if (items[i].id == id) {
                	foundItem = items[i];
                    break; // jump out of the for loop
                }
            }
        }
        return foundItem;
	};
	

	return DocTableXRenderConfig;
});
    

define('web-viewer/attributeCategory/AttributeCategoryDataExtensions',["web-viewer/webcenter/WebCenterDocumentPage",
		"web-viewer/utils/ViewerUtils",
		"utils/Cache",
		"attributecategory/AttributeCategory",
		"underscore"
	],
	function (WebCenterDocumentPage,
		ViewerUtils,
		Cache,
		AttributeCategory,
		_) {
		"use strict";
		var ATTRIBUTE_DATA_KEY = "attributeCategory";


		/**
		 * AttributeCategoryDataExtensions install methods to WebCenterDocumentPage to manage an attributeCategory for the page
		 *
		 * @exports AttributeCategoryDataExtensions
		 * @memberof AttributeCategoryModule
		 * @param   {ViewerFacade} f [[Description]]
		 */
		var AttributeCategoryDataExtensions = function (f) {

			// Cache used for retrieving the groups and roles a user is assigned to in a project
			var userGroupRolesCache = new Cache(20, true);
			userGroupRolesCache._loadItem = function (projectID) {
				var url = f.getDataAdapter().WebCenter("loadrolesandgroupsforuser.wcr", {
					projectid: projectID
				});

				f.ajax(url, {
					dataType: "JSON",
					success: function (data) {
						if (data && data.result && data.result.rolesAndGroups) {
							// convert to Uppercase for consistency
							var rolesAndGroups = _.map(data.result.rolesAndGroups, function (roleGroup) {
								return roleGroup.toUpperCase();
							});
							userGroupRolesCache._registerItem(projectID, rolesAndGroups);
						} else {
							// we couldn't load usergroupsroles for this project
							userGroupRolesCache._registerItem(projectID);
						}
					},
					error: function () {
						// we couldn't load usergroupsroles for this project
						userGroupRolesCache._registerItem(projectID);
					}
				});
			};


			/**
			 * Get the Data Object for the AttributeCategoryDataExtensions
			 * @returns {Object} data-object, containing the attribute category status, object and DOM elements
			 */
			WebCenterDocumentPage.prototype._getAttributeCategoryData = function () {
				return this.get(ATTRIBUTE_DATA_KEY);
			};

			/**
			 * Execute a callback when the AttributeCategory is loaded (its structure is available, and views can be requested)
			 * @param {Function} callback - callback callend when the attribute category is available, or loading failed. First parameters indicates whether load was successfull. Second parameter can be an errorstring if something failed
			 * @param {Object}   [binding]  - binding for the callback
			 */
			WebCenterDocumentPage.prototype.whenAttributeCategoryAvailable = function (callback, binding) {
				if (binding) {
					callback = _.bind(callback, binding);
				}

				if (!this.hasAttributeCategory()) {
					// No attribute category is defined for this page
					callback(false);
				} else {
					var data = this._getAttributeCategoryData();
					if (data && data.loaded) {
						// The attribute category is already loaded and available, execute the callback
						callback(data.object);
					} else {
						// The category should be initialized, or is still loading
						// register the callback
						this.once("attributeCategory.loaded", callback);
						if (!data || !data.loading) {
							// the category is not yet initialized, initialize and start loading the category
							this._initializeAttributeCategory();
						}
					}
				}
			};

			/**
			 * Bootstrap and initialize the Attribute Category
			 */
			WebCenterDocumentPage.prototype._initializeAttributeCategory = function () {
				// load language strings
				f.loadBundle("attributeCategory", function (bundle) {
					// create the data exetensions data object
					var data = {};


					var document = this.getDocument();
					var docVersionID = this.getDocumentVersion().getDocumentVersionID();

					// generate a unique ID for storing the attribute category in the DOM
					data.id = ViewerUtils.generateUniqueDOMId("aCatObj_");

					// make sure the values are not yet loaded (they should only be fetched when there is a custom Viewer view
					var aCatOptions = {
						delayedLoading: true
					};

					// set the ID for the attribute storage
					aCatOptions.ACatObj = data.id;
					aCatOptions.ACatPanel = data.id + "_panel";

					// create the context div (in this the attribtue category panel, form and iframe to submit will be placed)
					data.contextDiv = $("<div>").hide();

					// iframe for submitting category
					data.iframe = $("<iframe>")
						.attr("name", aCatOptions.ACatObj + "_iframe")
						.css({
							height: 1,
							width: 1,
							position: "absolute",
							top: "-90000px",
							left: "-10000px"
						})
						.appendTo(data.contextDiv);


					// form for submitting category
					data.form = $("<form>")
						.attr("name", "actionform")
						.attr("method", "post")
						.attr("enctype", "multipart/form-data")
						.attr("target", aCatOptions.ACatObj + "_iframe")
						.attr("action", this.f.getDataAdapter().WebCenter("dodocdetailsactions.jsp"))
						.appendTo(data.contextDiv);
					data.form.append($("<input>")
						.attr("type", "hidden")
						.attr("name", "actionVal")
						.val("SaveAttributesEmbedded"));
					data.form.append($("<input>")
						.attr("type", "hidden")
						.attr("name", "docVerID")
						.val(docVersionID));

					// panel div for rendering category
					data.panelDiv = $("<div>")
						.attr("id", aCatOptions.ACatPanel)
						.appendTo(data.form);

					data.contextDiv.appendTo(f.getContext());

					// global object to store category in it
					data.objectDiv = $("<div>").attr("id", aCatOptions.ACatObj).hide();
					data.objectDiv.appendTo(f.getContext());


					// register some callbacks
					aCatOptions.callbackOnLoad = _.bind(this._attributeCategoryFireOnLoad, this);
					aCatOptions.callbackOnFail = _.bind(this._attributeCategoryFireOnFail, this);
					aCatOptions.callbackOnRender = _.bind(this._attributeCategoryFireOnRender, this);

					// we are requesting the category for a document
					aCatOptions.type = "document";
					aCatOptions.projectID = document.getProjectID();
					aCatOptions.documentID = document.getDocumentID();
					aCatOptions.docVersionID = docVersionID;
					if (aCatOptions.docVersionID) {
						// to be used to detect cyclic dependencies in document references
						aCatOptions.docTableParentDocVerIDs = [docVersionID];
					}

					// set the language strings
					aCatOptions.lanugageStrings = bundle;

					aCatOptions.currentUser = _.clone(window.gCurrentUser);
					aCatOptions.currentUser.canEditAttributes = this.canEditAttributes();

					// url for fetching the values
					aCatOptions.valuesURL = f.getDataAdapter().WebCenter("GetAttributes.jsp", {
						docverid: docVersionID
					});

					// indicate that the category is being loaded
					data.loading = true;

					// initialize (not we didn't  provide the readURL option, so the category won't start doing anything yet)
					data.object = new AttributeCategory(aCatOptions);
					data.object.valueChangedListeners.push(_.bind(this._attributeCategoryFireOnChange, this));

					// store the data extensions data object
					this.set(ATTRIBUTE_DATA_KEY, data);

					// start loading category
					data.object.readACatFromXML(f.getDataAdapter().WebCenter("GetAttributeCategoryXML.jsp", {
						categoryid: this.getAttributeCategoryID()
					}));
				}, this);

			};

			/**
			 * Get the attributecategory object for this page
			 * @returns {AttributeCategory}
			 */
			WebCenterDocumentPage.prototype.getAttributeCategory = function () {
				var data = this._getAttributeCategoryData();
				if (data) {
					return data.object;
				}

				// no attribute category available
				return;
			};

			/**
			 * Fire onload event
			 */
			WebCenterDocumentPage.prototype._attributeCategoryFireOnLoad = function () {
				var data = this._getAttributeCategoryData();
				data.loading = false;
				data.loaded = true;
				this._emit("attributeCategory.loaded", true);
			};

			/**
			 * Fire on fail event
			 */
			WebCenterDocumentPage.prototype._attributeCategoryFireOnFail = function (error) {
				this._emit("attributeCategory.loaded", false, error);
			};

			/**
			 * Fire the render event
			 */
			WebCenterDocumentPage.prototype._attributeCategoryFireOnRender = function () {
				this._emit("attributeCategory.render");
			};

			/**
			 * Fire the change event
			 */
			WebCenterDocumentPage.prototype._attributeCategoryFireOnChange = function () {
				this._emit("attributeCategory.change");
			};
			
			/**
			 * Fire the modal popup closed event
			 */
			WebCenterDocumentPage.prototype.attributeCategoryFireModalPopupClosed = function () {
				this._emit("attributeCategory.modalPopupClosed");
			};

			/**
			 * Fire a save finished event
			 * @param {Boolean}       success - is the save operation successful?
			 * @param {String|Object} error   - error information
			 */
			WebCenterDocumentPage.prototype._attributeCategoryFireOnSaveFinished = function (success, error) {
				this._emit("attributeCategory.saveFinished", success, error);
			};

			/**
			 * Get the panel (with form and ifrmae to submit) for display the attribute category
			 * @returns {JQueryObj} a JQuery reference to the DOM element in which the attribtue category panel, form and iframe sit
			 */
			WebCenterDocumentPage.prototype.getAttributeCategoryPanel = function () {
				var data = this._getAttributeCategoryData();
				if (data && data.loaded) {
					return data.contextDiv;
				}
			};


			/**
			 * This is a helper function to make sure all dates are set correctly when saving the category
			 * NOTE: this is legacy WCR code, but it seems to work
			 * JANH NOTE: It doesn't work for any prefixed attribute category! (project creation page,
			 * 		task execution page, all document references) But it seems not needed at those places...
			 *
			 * @param   {Object}  thisForm - form of the attribute category
			 * @returns {Boolean}
			 */
			function assignDate(thisForm) {
				var i, cobj, dateElementName, dElement, setValueOjbName, setValueOjb, isRS;
				for (i = 0; i < thisForm.elements.length; ++i) {
					cobj = thisForm.elements[i];

					if (cobj.tagName == 'INPUT' &&
						(cobj.type == 'text' || cobj.type == 'hidden') &&
						cobj.name.substring(0, 5) == 'adate') {
						dateElementName = cobj.name.substring(5);
						dElement = document.getElementById(dateElementName);
						dElement.value = cobj.value;
					}

					if ((cobj.tagName == 'INPUT' && (cobj.type == 'text' || cobj.type == 'hidden') &&
							cobj.name.substring(0, 2) == 'UN' &&
							cobj.value == '00001_0000000020')) {
						isRS = document.getElementById("isRS" + cobj.name.substring(2));
						if ((isRS != null && isRS.value != "1") || (isRS == null)) {
							// not a restricted set
							dElement = document.getElementById(cobj.name.substring(2));
							setValueOjbName = "adate" + dElement.name;
							setValueOjb = document.getElementById(setValueOjbName);

							dElement.value = setValueOjb.value;
						}
					}
				}
				return true;
			}

			/**
			 * Save the attributes
			 * @param {Function} callback - callback function (first parameter: success, second parameter: error string if any)
			 */
			WebCenterDocumentPage.prototype.saveAttributes = function (callback) {
				var data = this._getAttributeCategoryData();
				// make sure no save operation is ongoing
				if (data && !data.saving && data.loaded && this.canEditAttributes()) {
					data.saving = true;

					// notify that things are changing
					this._attributeCategoryFireOnChange();

					// register listener
					this.once("attributeCategory.saveFinished", callback);

					// make sure the dates are set correctly
					assignDate(data.form.get(0));

					// validate and prepare the attribute category for submit
					if (data.object.prepareForSubmit(false, true)) {
						var iframe = data.iframe.get(0);
						// register onload on the iframe
						iframe.onload = _.bind(function () {
							data.saving = false;

							// the iframe has loaded, handle result
							try {
								if (iframe.contentDocument) {
									if (iframe.contentDocument.getElementsByTagName("ok").length > 0) {
										data.object.attributesChanged = false;
										// save operation was successful

										var attributeCategory = this.getAttributeCategory();
										if (attributeCategory && attributeCategory.hasActiveViewDocReference()) {
											this.f.getDataAdapter().updateDocumentData(this.f.getActivePage());
										}

										this._attributeCategoryFireOnSaveFinished(true);
									} else if (iframe.contentDocument.getElementsByTagName("error").length > 0) {
										// an errr occurred, parse all errors
										var error = "";
										var errorTags = iframe.contentDocument.getElementsByTagName("error");

										for (var i = 0; i < errorTags.length; i++) {
											if (i > 0) {
												error += ", ";
											}
											error += errorTags[i].textContent;
										}
										this._attributeCategoryFireOnSaveFinished(false, error);
									}
								}
							} catch (err) {
								this._attributeCategoryFireOnSaveFinished(false, err);
							}
							this._attributeCategoryFireOnChange();
						}, this);

						// submit the form in the iframe
						data.form.submit();

					} else {
						// attribute category is not valid, user choose not to save, so don't call callback for any error... The attribute category will give alert
						data.saving = false;
						this._attributeCategoryFireOnChange();
					}
				} else {
					// save operation is already ongoing
					data.saving = false;
					callback(false, this.f.t("AttrSaveErrorOngoing"));
					this._attributeCategoryFireOnChange();
				}
			};

			/**
			 * Can the user save attributes? (is the category loaded and does he have edit rights)
			 * @returns {Boolean}
			 */
			WebCenterDocumentPage.prototype.canSaveAttributes = function () {
				var data = this._getAttributeCategoryData();
				return data && this.canEditAttributes();
			};

			/**
			 * Is a save operation ongoing?
			 * @returns {Boolean}
			 */
			WebCenterDocumentPage.prototype.isAttributeCategorySaveOngoing = function () {
				var data = this._getAttributeCategoryData();
				return data && data.saving;
			};

			/**
			 * Are there any unsaved changes on the attribute category
			 * @returns {Boolean} [[Description]]
			 */
			WebCenterDocumentPage.prototype.hasUnsavedAttributeChanges = function () {
				var data = this._getAttributeCategoryData();

				if (data && data.loaded && this.canEditAttributes()) {
					return data.object.attributesChanged;
				}

				return false;
			};

			/**
			 * Is there an attribute category defined on the page
			 * @returns {Boolean}
			 */
			WebCenterDocumentPage.prototype.hasAttributeCategory = function () {
				return this.getAttributeCategoryID() && true;
			};

			/**
			 * Get a set of avialable Roles and Groups that have a view in the category
			 * @param {Function} callback - parameter of the callback is an object with a properties "viewerView" [contains the name of the Viewer view, or undefined if non-existant], "roleSpecificViews" [keys: available roles and values the attribute category view assigned to that role/group], "all": [array of all views]
			 * @example
			 *    {
			 *        viewerView: "Viewer",
			 *        roleSpecificViews: {
			 *             GROUP: "Viewer Group",
			 *             "SOME ROLE": "Viewer Some ROLE"
			 *        },
			 *        all: ["Viewer", "Viewer Group", "Viewer Some ROLE", "Default" , ...]
			 *    }
			 */
			WebCenterDocumentPage.prototype.whenAttributeCategoryViewsAvailable = function (callback) {
				var data = this._getAttributeCategoryData();
				if (data && data.loaded) {

					// check if we already did this operation before
					if (data.views && data.views.loaded) {
						// execute callback, we have the needed data
						callback(data.views);
						return;
					} else {

						// register listener
						this.once("attributeCategory.viewsAvailable", callback);

						if (!data.views) {
							data.views = {};
							data.views.viewerView = null;
							data.views.all = data.object.getViewNames();
							data.views.roleSpecificViews = {};
							data.views.loaded = false;

							// make a list of all attribute category views that could match a Role or Group
							// and detect if the Viewer view is present
							var roleViews = {};
							_.each(data.views.all, function (viewName) {
								var upperCaseView = viewName.toUpperCase();
								if (upperCaseView == "VIEWER") {
									// we found the viewer view
									data.views.viewerView = viewName;
								} else if (upperCaseView.indexOf("VIEWER") === 0) {
									// we found an attribute category view that could match a Role or Group
									roleViews[viewName.toUpperCase().replace(/^VIEWER\s*/, "")] = viewName;
								}
							});


							if (_.size(roleViews) > 0) {
								// there are one or more attribute category views that could mat a role or group, fetch the roles and groups for the active user in the project of this file
								userGroupRolesCache.get(this.getDocument().getProjectID(), _.bind(function (userGroupRoles) {
									data.views.roleSpecificViews = {};
									// run through the roles/groups the user is assigned to, if a match is found with the attribute category views, add it to the available views
									_.each(userGroupRoles, function (groupRole) {
										if (roleViews[groupRole]) {
											data.views.roleSpecificViews[groupRole] = roleViews[groupRole];
										}
									});
									data.views.loaded = true;
									// we have everything we need, execute the callback
									this._emit("attributeCategory.viewsAvailable", data.views);
								}, this));
							} else {
								data.views.loaded = true;
								// no custom views are set, just execute the callback
								this._emit("attributeCategory.viewsAvailable", data.views);
							}

						}
					}
				}
			};

			/**
			 * Is the "Viewer" view defined for this page
			 * @returns {Boolean}
			 */
			WebCenterDocumentPage.prototype.hasViewerView = function () {
				var data = this._getAttributeCategoryData();
				return data && data.views.viewerView;
			};

			/**
			 * Get the name of the Viewer view
			 * @returns {String} name of the viewer view
			 */
			WebCenterDocumentPage.prototype.getViewerView = function () {
				var data = this._getAttributeCategoryData();
				if (data) {
					return data.views.viewerView;
				}
			};

			/**
			 * Get all the views defined on the attribute category of the document
			 * if no views are defined, or the views are not yet loaded, an empty array is returned
			 * @returns {Array.<String>} list of available views
			 */
			WebCenterDocumentPage.prototype.getAllAttributeCategoryViews = function () {
				var data = this._getAttributeCategoryData();
				if (data && data.views) {
					return data.views.all;
				} else {
					return [];
				}
			};

			/**
			 * Reset the attribute category (this destroys the existing data and reloads/reinitializes the category
			 * @param {Function} callback - called when finished
			 * @param {Object}   [binding]  - binding for the callback
			 */
			WebCenterDocumentPage.prototype.resetAttributeCategory = function (callback, binding) {
				var oldData = this._getAttributeCategoryData();

				if (binding) {
					callback = _.bind(callback, binding);
				}

				if (oldData && oldData.object) {
					// remove TinyMCE editors
					oldData.object.destroy();

				}


				// reset the data
				this.set(ATTRIBUTE_DATA_KEY);

				// load category
				this.whenAttributeCategoryAvailable(function () {
					if (oldData) {
						if (oldData.objectDiv) {
							oldData.objectDiv.remove();
						}
						if (oldData.contextDiv) {
							oldData.contextDiv.remove();
						}
					}

					callback();
				});

			};
		};

		return AttributeCategoryDataExtensions;
	});

define('web-viewer/attributeCategory/AttributeCategoryWidget',["web-viewer/layout/AbstractWidget",
		"web-viewer/layout/LayoutConstants",
		"web-viewer/utils/ListenerUtil",
		"web-viewer/layout/component/MiniButton",
		"web-viewer/layout/component/DropdownMiniButton",
		"underscore",
		"jquery"
	],
	function (AbstractWidget,
		LayoutConstants,
		ListenerUtil,
		MiniButton,
		DropdownMiniButton,
		_,
		$) {
		"use strict";

		/**
		 * The AttributeCategoryWidget shows the attribute category for the active document
		 * 
		 * @exports AttributeCategoryWidget
		 * @augments Layout.AbstractWidget
		 * @class The AttributeCategoryWidget shows the attribute category for the active docmetn
		 * @constructor
		 * @mixes {Utils.ListenerUtilMixin}
		 * @param {Core.ViewerFacade} f - viewerFacade
		 */
		var AttributeCategoryWidget = function AttributeCategoryWidget(f, attributeCategorModule) {
			AbstractWidget.call(this, f, LayoutConstants.Widget.ATTRIBUTE_CATEGORY);
			ListenerUtil.mixin(this, f);

			/**
			 * @member {AbstractDocumentPage} the page for which the attributes are shown
			 */
			this._page = null;

			/**
			 * @member {JQueryObj} the DOM element containing all DOM elements for the attributes
			 */
			this._panel = null;

			// intialize Save button
			this._saveButton = new MiniButton(this.f, "saveAttributes", "icon-WCR_Save", true, _.bind(function () {
				this._emit("saveAttributes");
			}, this), "fontIconDisabled icon-WCR_Save");
			this._saveButton.element.addClass("displayNone");
			this._saveButton.setTooltip("viewer", "AttrSaveAttributes");

			// initialize view swichter
			this._viewSwitcher = new DropdownMiniButton(this.f, "viewSwitcher", "icon-WCR_User", ["bottom", "top"]);
			this._viewSwitcher.element.addClass("displayNone");
			this._viewSwitcher.setTooltip("viewer", "AttrSwitchView");




			// settitle
			this.f.loadBundle("viewer", function () {
				this.setTitle(this.f.t("AttributeCategoryWidgetTitle"));
			}, this);

			this.setIcon("icon icon-WCR_Attributes");


			// load CSS to display attributes correctly in the viewer
			this._loadCss = _.once(function () {
				// wrap popitmenu so it is displayed correctly
				$("#popitmenu").wrap($("<div>").addClass("wcr"));

			});
		};

		AttributeCategoryWidget.prototype = Object.create(AbstractWidget.prototype);
		AttributeCategoryWidget.prototype.constructor = AttributeCategoryWidget;
		AttributeCategoryWidget.widgetName = LayoutConstants.Widget.ATTRIBUTE_CATEGORY;



		/**
		 * @override
		 */
		AttributeCategoryWidget.prototype.setContext = function (context, headerExtra) {
			AbstractWidget.prototype.setContext.call(this, context, headerExtra);

			this.context.addClass("enableTextSelection");

			headerExtra.append(this._saveButton.element);
			headerExtra.append(this._viewSwitcher.element);

			if (this._panel) {
				this.context.append(this._panel.show()).addClass("wcr");
			}

			this._loadCss();
		};

		/**
		 * Set the page for the widget
		 * @param {AbstractDocumentPage} [page]            - The page for which the attribute category is shown. If null, the current attribtue category is removed from the widget
		 * @param {String}               [activeGroupRole] - Name of the currently active group/role view
		 * @param {Array.<String>}       [availableViews]  - List of available roles/groups for display the attribute category custom views   
		 */
		AttributeCategoryWidget.prototype.setPage = function (page, activeGroupRole, availableViews) {
			var panel;

			if (page) {
				panel = page.getAttributeCategoryPanel();
			}


			if (this._page && this._page !== page) {
				// remove listeners from the current page
				this._unregisterAllListenersForObject(this._page);
				// move the attribute category context out of the widget
				this.f.getContext().append(this._panel.hide());
			}

			this._page = page;
			this._panel = panel;

			if (this._page) {
				// listen to change and render events of the attribute category
				this._registerListener("attributeCategory.render", this._fireResized, this, this._page);
				this._registerListener("attributeCategory.change", this._updateSaveButton, this, this._page);
			}

			// clear the view switcher
			this._viewSwitcher.clear();
			if (this._page && availableViews && availableViews.length > 1) {
				// there is more than 1 views available, initialize and display the view switcher
				var selectGroupRole = _.bind(function (groupRole) {
					this._emit("changedGroupRole", groupRole);
				}, this);

				// add available views
				this._groupRoleOptions = [];
				_.each(availableViews, function (groupRole) {
					this._viewSwitcher.addOption({
						name: groupRole,
						content: groupRole,
						enabled: true,
						selected: groupRole == activeGroupRole,
						onchange: _.partial(selectGroupRole, groupRole)
					});
					this._groupRoleOptions.push(groupRole);
				}, this);

				this._viewSwitcher.element.removeClass("displayNone");
			} else {
				// hide the view swithcer
				this._viewSwitcher.element.addClass("displayNone");
			}

			if (this.context && this._panel) {
				// append the attribute category panel to the widget
				this.context.append(this._panel.show()).addClass("wcr");
				this._fireResized();
			}

			// render the save button correctly
			this._updateSaveButton();
		};

		/**
		 * Update the view selector dropdown
		 * @param {String} activeGroupRole - the currently active Group/Role
		 */
		AttributeCategoryWidget.prototype.updateGroupRoleSelection = function (activeGroupRole) {
			_.each(this._groupRoleOptions, function (groupRole) {
				this._viewSwitcher.toggleOption(groupRole, false);
			}, this);
			this._viewSwitcher.toggleOption(activeGroupRole, true);
		};

		/**
		 * Update the rendering of the save button
		 */
		AttributeCategoryWidget.prototype._updateSaveButton = function () {
			if (this._page) {
				if (this._page.canSaveAttributes()) {
					// the user can change attributes, show the button and indicate whether any changes are made
					this._saveButton.element.removeClass("displayNone");
					// while saving the button should be disabled
					this._saveButton.setEnabled(!this._page.isAttributeCategorySaveOngoing() && this._page.hasUnsavedAttributeChanges());
					this._saveButton.setBadge(this._page.hasUnsavedAttributeChanges() ? "!" : "");
				} else {
					// hide the save button, the user can't edit attributes
					this._saveButton.element.addClass("displayNone");
				}
			} else {
				// hide the save button, no page is defined
				this._saveButton.element.addClass("displayNone");
			}
		};

		/**
		 * When the widget is being moved, we need to make sure to disable the tinyMCE fields
		 * This method is called before the widget is being moved
		 * @override
		 */
		AttributeCategoryWidget.prototype.movingStarted = function () {
			AbstractWidget.prototype.movingStarted.apply(this, arguments);

			if (this._page) {
				var attributeCategory = this._page.getAttributeCategory();
				if (attributeCategory) {
					attributeCategory.beforeMove();
				}
			}
		};

		/**
		 * When the widget is being moved, we need to make sure to disable the tinyMCE fields
		 * This method is called after the widget was moved, reintialize the tinyMCE instances
		 * @override
		 */
		AttributeCategoryWidget.prototype.movingStopped = function () {
			AbstractWidget.prototype.movingStopped.apply(this, arguments);

			if (this._page) {
				var attributeCategory = this._page.getAttributeCategory();
				if (attributeCategory) {
					attributeCategory.afterMove();
				}
			}
		};

		/**
		 * @override
		 */
		AttributeCategoryWidget.prototype.willFill = function () {
			return true;
		};

		return AttributeCategoryWidget;
	});

define('web-viewer/layout/component/FullscreenModalPopup',["web-viewer/layout/component/ModalPopup",
		"utils/ShapeUtils",
		"underscore",
		"jquery"
	],
	function (ModalPopup,
		ShapeUtils,
		_,
		$) {
		"use strict";

		/**
		 * A FullScreen modal popup is covering the complete viewer, its size depends on the screen size of the user
		 * @exports FullscreenModalPopup
		 * @class A FullScreen modal popup is covering the complete viewer, its size depends on the screen size of the user
		 * @constructor
		 * @param {Core.ViewerFacade} f - Viewer facade
		 * @param {String} id - Id of the popup. It is used in its private channel.
		 * @param {Boolean} [noCancelButton=false] - do no show a cancel button
		 * @param {Number} [size=.95] - percentage of the screen the popup should cover
		 */
		var FullscreenModalPopup = function (f, id, noCancelButton, size) {
			ModalPopup.call(this, f, id, f.getContext(), noCancelButton);

			this.element.addClass("fullscreenModalPopup");

			if (!size) {
				size = 0.95;
			}


			this._size = size;

			this._debouncedRecalculateSize = _.debounce(_.bind(function (e) {
				// http://bugs.jqueryui.com/ticket/7514: resize of popups bubbles to window
				if (e.target === window) {
					this.recalculateSize();
				}
			}, this), 300);

			this.content.append(this._actionBtns);
		};

		FullscreenModalPopup.prototype = Object.create(ModalPopup.prototype);
		FullscreenModalPopup.prototype.constructor = FullscreenModalPopup;

		/**
		 * Override default recalculate size behavior
		 * @Override
		 */
		FullscreenModalPopup.prototype.recalculateSize = function () {

			var width = Math.ceil(this.containment.innerWidth() * this._size);
			var height = Math.ceil(this.containment.innerHeight() * this._size);

			var x = this.containment.innerWidth() / 2 - width / 2;
			var y = this.containment.innerHeight() / 2 - height / 2;

			this.content.css("overflow", "hidden"); //fix for Chrome: dont show unnecessary scrollbar
			this.setRect(new ShapeUtils.Rect(new ShapeUtils.Point(x, y), new ShapeUtils.Size(width, height)));
			this.content.css("overflow", "auto");
		};

		/**
		 * make sure the height of the content and action buttons is correctly set
		 * @override
		 */
		FullscreenModalPopup.prototype._onResized = function () {
			ModalPopup.prototype._onResized.call(this);

			this._popupContentContainer.css({
				height: this.content.innerHeight() - this._actionBtns.outerHeight(true)
			});
		};

		/**
		 * Add window resized listener when popup is shown
		 * @Override
		 */
		FullscreenModalPopup.prototype.show = function () {
			ModalPopup.prototype.show.apply(this, arguments);

			$(window).on("resize", this._debouncedRecalculateSize);
		};

		/**
		 * Remove window resized listener when popup is hidden
		 * @Override
		 */
		FullscreenModalPopup.prototype.hide = function () {
			ModalPopup.prototype.hide.apply(this, arguments);
			$(window).off("resize", this._debouncedRecalculateSize);
		};


		return FullscreenModalPopup;
	});

/**
 * @namespace AttributeCategory
 */
define('web-viewer/attributeCategory/AttributeCategoryModule',["web-viewer/core/AbstractModule",
		"web-viewer/utils/ListenerUtil",
		"web-viewer/attributeCategory/AttributeCategoryDataExtensions",
		"web-viewer/attributeCategory/AttributeCategoryWidget",
		"attributecategory/AttributeCategory",
		"web-viewer/layout/component/ModalPopup",
		"web-viewer/layout/component/FullscreenModalPopup",
		"web-viewer/layout/component/ViewerToolbarButton",
		"underscore",
		"jquery"
	],
	function (AbstractModule,
		ListenerUtil,
		AttributeCategoryDataExtensions,
		AttributeCategoryWidget,
		AttributeCategory,
		ModalPopup,
		FullscreenModalPopup,
		ViewerToolbarButton,
		_,
		$) {
		"use strict";

		/**
		 * Create a new instance of the attribute category module. This enables a widget with the attribute category of the active document
		 *
		 * @exports AttributeCategoryModule
		 * @memberof AttributeCategoryModule
		 * @class
		 * @augments Core.AbstractModule
		 * @constructor
		 *
		 * @param {Core.ViewerFacade} f
		 * @param {Boolean} silenced
		 */
		var AttributeCategoryModule = function (f, silenced) {
			AbstractModule.call(this, f, silenced, "attributeCategory");
			ListenerUtil.mixin(this, f);

			AttributeCategoryDataExtensions(f);


			this._category = null;
			this._page = null;
			this._viewList = [];
			this._activeGroupRole = null;
			this._activeView = null;

			this._viewBeforeModal = null;



			this._publishedWidget = false;

			this._registerListener("intercept.activePageChange", this._interceptActivePageChange, this);
			this._registerListener("intercept.savingCadOverlay", function (context) {
				if (this._category && this._category.hasActiveViewDocReference()) {
					if (this._page.isAttributeCategorySaveOngoing()) {
						// save operation is ongoing
						context.saveCadOverlay = false;

						// when save has finished, save CAD overlay
						this._page.once("attributeCategory.saveFinished", function (success, error) {
							// save operation for attributes has finished, save thfe CAD overlay
							context.callback();
						});


					} else if (this._page.hasUnsavedAttributeChanges()) {
						// no ongoing save operation, but unsaved attributes... prompt to save
						context.saveCadOverlay = false;
						this._promptToSaveUnsavedChanges(_.bind(function () {
							context.callback();
						}, this), _.bind(function () {
							context.callback();
						}, this), true);
					}
				}
			}, this);
			this._registerListener("newLinkedCad", function () {
				if (!this.f.getModule("compare")) {
					this.f.requestModule("compare", undefined, false);
				}
				this.resetAttributeCategory(this._activeView);
			}, this, this.f.getDocumentController());
			this.f.getDocumentController().whenActivePageChangedAndParametersAvailable(this._whenActivePageChangedAndParametersAvailable, this);
		};

		// Inherit from AbstractModule
		AttributeCategoryModule.prototype = Object.create(AbstractModule.prototype);
		AttributeCategoryModule.prototype.constructor = AttributeCategoryModule;


		/**
		 * When the active page has changed and the parameters are available, react on it:
		 *
		 * Load attribute category, roles and groups and display the widget if needed
		 *
		 * @param {AbstractDocumentPage} oldPage - previous active page
		 * @param {AbstractDocumentPage} newPage - new active page
		 */
		AttributeCategoryModule.prototype._whenActivePageChangedAndParametersAvailable = function (oldPage, newPage) {
			this._viewList = [];

			if (this._widget) {
				this._widget.setPage(null);
			}

			// unregister listeners
			this._unregisterAllListenersForObject(oldPage);

			newPage.whenAttributeCategoryAvailable(function (isCategoryAvailable) {
				if (isCategoryAvailable) {
					newPage.whenAttributeCategoryViewsAvailable(_.bind(function (views) {
						var viewToActivate = null;
						if (views && _.size(views.roleSpecificViews) > 0) {
							this._viewList = _.sortBy(_.keys(views.roleSpecificViews), function (groupRole) {
								return groupRole;
							});
							this._activeGroupRole = _.first(this._viewList);
							viewToActivate = views.roleSpecificViews[this._activeGroupRole];

						} else if (newPage.hasViewerView()) {
							viewToActivate = newPage.getViewerView();
						}

						if (viewToActivate) {
							this._activateCategoryWithView(newPage, viewToActivate);
						} else {
							this._disableWidget();
						}
					}, this));
				} else {
					this._disableWidget();
				}
			}, this);
		};


		/**
		 * Activate the Category with a certain view
		 * @param {AbstractDocumentPage} page           - the page for which the attribute category should be activated
		 * @param {String}               viewToActivate - the view that should be activated on the attribute category
		 */
		AttributeCategoryModule.prototype._activateCategoryWithView = function (page, viewToActivate) {
			this._category = page.getAttributeCategory();
			this._page = page;

			if (this._aCatLoadedListener) {
				this._unregisterListener(this._aCatLoadedListener);
				delete this._aCatLoadedListener;
			}

			this._aCatLoadedListener = this._registerListener("attributeCategory.loaded", function (success, error) {
				if (!success) {
					this.f.loadBundle("viewer", function () {
						this._emit("userMessage", "error", this.f.t("AttrErrorWhileLoading", error ? error : ""));
					}, this);
				}
			}, this, page);

			this._activateViewByName(viewToActivate);
			this._displayAttributeCategory();
		};

		/**
		 * Activate a certain view on the category
		 * @param {String} viewName - view to be activated
		 */
		AttributeCategoryModule.prototype._activateViewByName = function (viewName) {
			this._activeView = viewName;
			this._category.activateViewByName(viewName);
		};

		/**
		 * Are there unsaved attributes?
		 * @Override
		 */
		AttributeCategoryModule.prototype.hasUnsavedElements = function () {
			return this._page && this._page.hasAttributeCategory() && this._page.canEditAttributes() && this._page.hasUnsavedAttributeChanges();
		};

		/**
		 * Language string for indicating what is not yet saved (hint, it returns "Attributes" :P)
		 * @returns {String}
		 * @override
		 */
		AttributeCategoryModule.prototype.getUnsavedElementsName = function () {
			// note we don't wait for the bundle to be loaded... but it's not even possible in this situation
			// this language var should normally be loaded already...
			return this.f.t("AttrUnsavedItemName");
		};

		/**
		 * Save the attribute
		 * @override
		 * @param callbackFunction
		 */
		AttributeCategoryModule.prototype.saveAllUnsavedElements = function (callbackFunction) {
			var page = this.f.getActivePage();

			if (page && page.hasAttributeCategory() && page.canEditAttributes() && page.hasUnsavedAttributeChanges()) {
				page.saveAttributes(callbackFunction);
			} else {
				callbackFunction();
			}
		};

		/**
		 * Display the attribute category in the widget (this method will publish the widget if needed)
		 */
		AttributeCategoryModule.prototype._displayAttributeCategory = function () {
			if (this._modalMode) {
				// we are displaying the attribute category in modal mode

				if (!this._modalPopup) {
					this._modalPopup = new FullscreenModalPopup(this.f, "attributeCategoryPopup", false, 0.95);

				}

				this._modalPopup.setContent(this._page.getAttributeCategoryPanel().show().addClass("wcr"));
				this._modalPopup.setHeader($("<span/>").addClass("popupTitle").text(this._modalInfo.popupTitle));

				// save attributes and close popup				
				this._modalPopup.setOkBtnText(this._modalInfo.saveText);
				this._modalPopup.setOkAction(_.bind(function () {
					this.saveActivePageAttributes();
				}, this));
					
				if(!this._page.canEditAttributes()){
					this._modalPopup.toggleButtons(false);
				}

				// on close prompt to save unsaved changes and close the popup
				this._modalPopup.setCloseAction(_.bind(function () {
					this._promptToSaveUnsavedChanges(_.bind(function () {
						this._modalMode = false;
						this.resetAttributeCategory(this._viewBeforeModal);
						this._page.attributeCategoryFireModalPopupClosed();
					}, this), _.bind(function () {
						// the user cancelled close operation, show the popup again
						this._modalPopup.show();
					}, this));
				}, this));

				// the popup is cancelled, revert changes hide popup
				this._modalPopup.setCancelAction(_.bind(function () {
					this._modalMode = false;
					this.resetAttributeCategory(this._viewBeforeModal);
					this._page.attributeCategoryFireModalPopupClosed();
				}, this));

				// show the popup
				this._modalPopup.show();
			} else {
				// display the attributes in the widget

				if (this._modalPopup) {
					this._modalPopup.hide();
				}

				if (!this._publishedWidget) {
					// the widget is not yet published

					this._publishedWidget = true;
					this._publishWidget(AttributeCategoryWidget.widgetName, _.bind(function (layoutType) {
						// widget is allowed, create it, set the page
						this._widget = new AttributeCategoryWidget(this.f, this);
						this._widget.setPage(this._page, this._activeGroupRole, this._viewList);
						this._widget.setEnabled(true);

						this._registerListener("changedGroupRole", this._activateGroupRoleView, this, this._widget);
						this._registerListener("saveAttributes", this.saveActivePageAttributes, this, this._widget);

						return this._widget;
					}, this));
				} else if (this._widget) {
					// widget already exists, just enable it and set the page
					this._widget.setEnabled(true);
					this._widget.setPage(this._page, this._activeGroupRole, this._viewList);
				}
			}

			$("#dropmenudiv").addClass("wcr").appendTo(this.f.getContext());
		};


		/**
		 * Handle the Attribute Category View request of the widget
		 * @param {String} newGroupRole - name of the group/role that should be activated
		 */
		AttributeCategoryModule.prototype._activateGroupRoleView = function (newGroupRole) {
			if (this._page) {
				this._page.whenAttributeCategoryViewsAvailable(_.bind(function (views) {
					if (views && views.roleSpecificViews && views.roleSpecificViews[newGroupRole]) {
						// the view is avaialble on the category
						var oldGroupRole = this._activeGroupRole;

						// if there are unsaved changes, first prompt to save them
						this._promptToSaveUnsavedChanges(_.bind(function () {
							// ok, change the view
							this._activeGroupRole = newGroupRole;
							// we want to completily reload the category
							this.resetAttributeCategory(views.roleSpecificViews[newGroupRole]);

							// make sure the correct group role selection is set
							if (this._widget) {
								this._widget.updateGroupRoleSelection(newGroupRole);
							}
						}, this), _.bind(function () {
							// we didn't activate the new view
							// make sure the correct group role selection is set                        
							if (this._widget) {
								this._widget.updateGroupRoleSelection(oldGroupRole);
							}
						}, this));
					}
				}, this));
			} else {
				// This is an exceptional case, it should normally not occur... we want to log it to detect this case...
				this.f.emit("error.WARNING", "Trying to activate a view when no page is active");
			}
		};

		/**
		 * Disable the widget
		 */
		AttributeCategoryModule.prototype._disableWidget = function () {
			if (this._widget) {
				this._widget.setEnabled(false);
				this._widget.setPage(null);
			}
		};

		/**
		 * Intercept active page changes. If the page is changing, and the user has unsaved changed, we want to prompt this...
		 * @param {Object} context - context object for interception
		 */
		AttributeCategoryModule.prototype._interceptActivePageChange = function (context) {
			var activePage = context.oldActivePage;
			var newActivePage = context.newActivePage;
			if (this._page && this._page === activePage && newActivePage !== activePage && activePage.hasUnsavedAttributeChanges()) {
				// the page has changed, and we have unsaved changed

				// by default prevent the change of page
				context.newActivePage = context.oldActivePage;

				// now prompt the user what he wants to do
				this._promptToSaveUnsavedChanges(_.bind(function () {
					// the user wants to change the page (and if needed attributes have been saved)
					this.f.getDocumentController().changeActivePage(newActivePage);
				}, this), function () {
					// the user cancelled, do nothing
				});
			}
		};

		/**
		 * Validate the Password and Save the attribute for the active page
		 * @param {Function} callback - callback function called when save operation finished, two parameters: first = success, second = error if any
		 */
		AttributeCategoryModule.prototype.saveActivePageAttributes = function (callback) {
			var options = {};
			options.viewOptions = {widgetview: "viewer"};
			options.f = this.f;
			this._category.checkPasswordAndSave(_.bind(function() {
				this.saveAttributesChanges(callback);
			}, this, callback),options);
		};

		/**
		 * Save the attribute for the active page
		 * 
		 * @param callback - callback function called when save operation finished, two parameters: first = success, second = error if any
		 */
		AttributeCategoryModule.prototype.saveAttributesChanges = function (callback) {
			if (this._page) {
				this._page.saveAttributes(_.bind(function (success, error) {
					// show user message for save result
					this.f.loadBundle("viewer", function () {
						if (success) {
							this.f.emit("userMessage", "success", this.f.t("AttrSaveSuccess"), true);
						} else {
							this.f.emit("userMessage", "error", this.f.t("AttrSaveError") + " " + error);
						}
					}, this);

					if (success) {
						// reload the category on success
						this.resetAttributeCategory(this._activeView);
					}

					// execute the callbacks if there are any
					if (callback) {
						callback(success, error);
					}
				}, this));
			}
		};


		/**
		 * Prompt the user the save unsaved attribute changes (if there are unsaved attribute changes)
		 * @param {Function} callbackOnOk     - called when everything is OK (attributes saved)
		 * @param {Function} callbackOnCancel - called when the user cancels the operation (attributes are not saved)
		 * @param {Boolean} [canNotBeCancelled] - indicates the action cannot be cancelled, it is either revert or save
		 * @returns {Boolean} true if a prompt is shown
		 */
		AttributeCategoryModule.prototype._promptToSaveUnsavedChanges = function (callbackOnOk, callbackOnCancel, canNotBeCancelled) {
			if (this._page && this._page.hasUnsavedAttributeChanges()) {
				this.f.loadBundle("viewer", function () {

					var modalPopup = new ModalPopup(this.f, "saveUnsavedAttributeChanges", this.f.getContext(), canNotBeCancelled);
					modalPopup.setRemoveFromDomAfterUsage(true);

					// the user wants to Save attributes before the callbackOnOk is executed
					modalPopup.setOkAction(_.bind(function () {
						// indicate to user that save operation is ongoing
						var loadingIndicator = $("<div>").css({
							'background': "url('" + requirejs.toUrl('web-viewer/layout/img/loaderTransparent.gif') + "') no-repeat 2px center",
							'padding-left': '20px',
							'line-height': '20px'
						}).addClass("unsavedItemsContent").text(this.f.t("AttrSaveOngoing"));
						modalPopup.setContent(loadingIndicator);

						//disable buttons while saving
						modalPopup.toggleButtons(false);
						modalPopup.toggleHeaderIcons(false);

						// save the attributes
						this.saveActivePageAttributes(function (success) {
							if (success) {
								// attributes are saved successfully!
								callbackOnOk();
							} else {
								// save operation failed, act like the save was cancelled, so the user wont lose data
								callbackOnCancel();
							}

							modalPopup.hide();
						});
					}, this));


					// Discard/revert changes
					var discardChanges = new ViewerToolbarButton(this.f, "exit", $("<span/>").text(this.f.t("RevertAnnChanges")), false, _.bind(function () {
						// reset the attribute category, but don't load new data
						this.resetAttributeCategory();
						// execute the callback on ok
						callbackOnOk();
						modalPopup.hide();
					}, this));
					modalPopup.addButton(discardChanges);

					if (!canNotBeCancelled) {
						// cancel the pending action
						modalPopup.setCancelAction(_.bind(function () {
							callbackOnCancel();
						}, this));
						modalPopup.setCancelBtnText(this.f.i18n.t("Cancel"));
					}


					// Header + Content
					modalPopup.setHeader($("<span/>").addClass("popupTitle").text(this.f.t("AttrSavePromptTitle")));
					modalPopup.setContent($("<div/>").addClass("unsavedItemsContent").text(this.f.t("AttrSavePromptContent")));

					modalPopup.setOkBtnText(this.f.i18n.t("Save"));

					modalPopup.show();
				}, this);
				return true;
			} else {
				// everything is OK, go on with OK action
				callbackOnOk();
				return false;
			}
		};


		/**
		 * Reset the category completily
		 * @param {String} [loadView] - if defined, the given view will be activated on the attribute category when its available (this triggers a "safe" reload), if nothing is defined, the attributes are reset, but no new data will be fetched
		 */
		AttributeCategoryModule.prototype.resetAttributeCategory = function (loadView) {
			if (this._widget) {
				this._widget.setPage(null);
			}

			this._viewList = [];

			this._activeView = null;

			var whenCategoryIsReset = _.bind(function (isCategoryAvailable) {
				if (loadView) {
					this._page.whenAttributeCategoryViewsAvailable(_.bind(function (views) {
						this._viewList = _.sortBy(_.keys(views.roleSpecificViews), function (groupRole) {
							return groupRole;
						});
						this._activateCategoryWithView(this._page, loadView);
					}, this));
				}
			}, this);

			if (this._page) {
				this._page.resetAttributeCategory(whenCategoryIsReset);
			} else {
				this._page = this.f.getActivePage();
				whenCategoryIsReset();
			}
		};



		/**
		 * Open a modal popup with a attribute category view for the current document
		 * (Note, if the user cancels saving the current attribute changes, the modal popup won't be opened)
		 * @param {String} viewName - the name of the view you want to activate (this should be an existing view offcourse!
		 * @param {String} [popupTitle=Attributes] -title for the modal popup
		 * @parma {String} [saveText=Save Attributes] - text on save button
		 */
		AttributeCategoryModule.prototype.showViewInModalPopup = function (viewName, popupTitle, saveText) {
			// save the current active view for if the widget is already showing something
			if (!this._modalMode) {
				this._viewBeforeModal = this._activeView;
			}

			this._promptToSaveUnsavedChanges(_.bind(function () {
					this._modalMode = true;
					this._modalInfo = {
						popupTitle: popupTitle || this.f.t("AttributeCategoryWidgetTitle"),
						saveText: saveText || this.f.t("AttrSaveAttributes")
					};

					this.resetAttributeCategory(viewName);
				}, this),

				_.bind(function () {
					// the user cancelled the operation
					this._modalMode = false;
				}, this));
		};

		/**
		 * Method to get the currently active view
		 * @returns {String|Object} _activeView
		 */
		AttributeCategoryModule.prototype.getActiveView = function () {
			return this._activeView;
		};

		return AttributeCategoryModule;
	});
