define(["underscore",
		"web-viewer/compare/CompareMode",
		"web-viewer/compare/CompareConstants"
	],
	function (_,
		CompareMode,
		CompareConst) {
		"use strict";
		/**
		 * @memberof CompareModule
		 * @exports CompareBlinkingMode
		 * @class Compare two documents by switching continuously between the two documents 
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {Compare.CompareModel} compareModel - model for compare
		 */
		var CompareBlinkingMode = function (f, compareModel) {
			CompareMode.call(this, f, CompareConst.mode.BLINKING, compareModel, false, "DiffBlinkingAnnotationsHidden");

			/**
			 * @private the interval for the switch between documents
			 */
			this._blinkInterval = CompareConst.blinkingSpeed;

			/**
			 * @private is the difference visible?
			 */
			this._differenceVisible = false;

			/**
			 * internal function used to execute the blinking mode
			 */
			this._toggleDifference = _.bind(function () {
				this._differenceVisible = !this._differenceVisible;
				// instruct the graphics module
				this._updateCompareModel();

				if (this.isEnabled()) {
					this._timer = setTimeout(this._toggleDifference, this._blinkInterval);
				}
			}, this);

		};

		CompareBlinkingMode.prototype = Object.create(CompareMode.prototype);
		CompareBlinkingMode.prototype.constructor = CompareBlinkingMode;

		/**
		 * Start/stop blinking mode
		 * @param {Boolean} [toggle] - should the mode be blinking or not, if undefined the mode will go to the other state than it is in
		 */
		CompareBlinkingMode.prototype.toggle = function (toggle) {
			CompareMode.prototype.toggle.call(this, toggle);

			if (toggle) {
				this._toggleDifference();
			} else {
				if (this._timer) {
					clearTimeout(this._timer);
				}
				this._differenceVisible = false;

				this._updateCompareModel();
			}
		};

		/**
		 * Update the state of the compare model
		 */
		CompareBlinkingMode.prototype._updateCompareModel = function () {
			var newDiffMode = this.isDifferenceVisible() ? CompareConst.diffMode.DIFFVERSION : CompareConst.diffMode.CURRENTVERSION;
			this._compareModel.setDiffMode(newDiffMode);
		};


		/**
		 * Is the difference visible
		 * @returns {Boolean}
		 */
		CompareBlinkingMode.prototype.isDifferenceVisible = function () {
			return this._differenceVisible;
		};


		/**
		 * Is the current visible
		 * @returns
		 */
		CompareBlinkingMode.prototype.isCurrentVisible = function () {
			return !this._differenceVisible;
		};


		return CompareBlinkingMode;
	});