define(["web-viewer/core/PrivateChannel",
		"web-viewer/layout/component/ModalPopup",
		"Template",
		"tpl!web-viewer/compare/ComparePagelistPopup",
		"web-viewer/compare/CompareConstants",
		"jquery",
		"underscore"
	],
	function (
		PrivateChannel,
		ModalPopup,
		Template,
		tpl,
		CompareConstants,
		$,
		_
	) {
		"use strict";
		/**
		 * Create a popup for selecting comparison setup for pagelists. 
		 * @exports ComparePagelistPopup
		 * @memberof CompareModule
		 * @constructor
		 * @augments Layout.ModalPopup
		 * 
		 * @param {Core.ViewerFacade} f - Viewerfacade
		 * @param {String} popupId - Id of the popup used for its private channel
		 */
		var ComparePagelistPopup = function (f, popupId) {
			ModalPopup.call(this, f, popupId, f.getContext());
			PrivateChannel.installTo(this, f, ['comparePagelist']);

			this._defaultPagelistCompareRange = CompareConstants.pagelistCompareRange.CURRENT;
			this._currentPagelistCompareRange = null;
			this._pagelistComparePage = null;

			this.t = new Template(this.f.i18n, tpl);
			this.t.setContext(this._popupContent);
			this.t.bind("change", "[name=range]", function (source, event) {
				this._currentPagelistCompareRange = parseInt(source.value, 10);
			}, this);

			this.f.i18n.loadBundle("viewer", function () {
				this.setHeader($("<span/>").addClass("popupTitle").html(f.i18n.t("MultipageCompare")));
				this.setOkBtnText(f.i18n.t("OkButton"));
			}, this);

			this.element.addClass("comparePagelistPopup");
		};

		ComparePagelistPopup.prototype = Object.create(ModalPopup.prototype);
		ComparePagelistPopup.prototype.constructor = ComparePagelistPopup;

		/**
		 * Renders the compare pagelist popup
		 */
		ComparePagelistPopup.prototype.render = function () {
			this.t.d.showPage = this._pagelistComparePage && this._pagelistComparePage.getDocumentVersion().isMultiPage();
			this.t.d.page = this._pagelistComparePage.getPagenum() || "";
			this.t.d.range = this._currentPagelistCompareRange;

			this.t.render();
		};

		/**
		 * Reset the popup to default settings
		 */
		ComparePagelistPopup.prototype._reset = function () {
			// reset to default
			this._currentPagelistCompareRange = CompareConstants.pagelistCompareRange.CURRENT;
		};

		/**
		 * Show the pagelist popup
		 */
		ComparePagelistPopup.prototype.show = function () {
			this._reset();
			this.render();
			ModalPopup.prototype.show.call(this);
			this.recalculateSize(300);
		};

		/**
		 * Set the 'compare to' page
		 * @param {AbstractDocumentPage} page
		 */
		ComparePagelistPopup.prototype.setComparePage = function (page) {
			this._pagelistComparePage = page;
		};

		return ComparePagelistPopup;
	});