
define('web-viewer/core/CoreConstants',{
	TOOLS: {
		PAN: "pan",
		ZOOM: "zoom",
		TOUCH: "touch",
		NOTOUCH: "notouch",
		ANNOTATION: "annotationtool",
		DYNCONTENT: "dyncontenttool",
		ANNOTATION_TOUCH_DRAW: "annotationTouchDrawTool",
		ANNOTATION_TOUCH: "annotationTouchTool",
		ANNOTATION_DRAW: "annotationdrawtool",
		ANNOTATION_NOTE_DRAW: "annotationnotetool",
		ANNOTATION_INDICATOR: "annotationindicatortool",
		COMPARE_ANCHOR_ALIGNMENT: "compareAnchorTool",
		MEASURE_BARCODE: "measureBarcodeTool",
		MEASURE_BRAILLE: "measureBrailleTool",
		MEASURE_EYEDROPPER: "measureEyeDropperTool",
		MEASURE_RULER: "measureRulerTool",
		MEASURE_CALIPER: "measureCaliperTool",
		MEASURE_SCREEN_RULING: "measureScreenRulingTool",
		INTERRUPT: "interruptTool",
		NAVIGATIONPAN: "navigationPanTool",
		MOUSE_LOCATOR: "mouseLocatorTool"
	},
	SizeUnits: { // linked to similar EyeDropperUnits
		MM: 0,
		CM: 1,
		INCH: 2,
		PIXELS: 3,
		POINTS: 4
	},
	EyeDropperUnits: { // linked to similar SizeUnits
		SQMM: 0,
		SQCM: 1,
		SQINCH: 2,
		PIXELS: 3,
		PERCENT: 5
	},
	ResolutionUnits: {
		PPI: 0,
		PPMM: 1,
		PPCM: 2
	},
	/**
	 * Mapping array for the unit localized names, see SizeUnits
	 */
	UnitsLocNames: ["Unitmm", "Unitcm", "UnitInchShort", "UnitPixelsShort", "UnitPoint"],
	/**
	 * Mapping array for the unit fixes, see SizeUnits
	 */
	UnitsFixes: [2, 3, 3, 0, 1],

	/**
	 * Mapping array for the resolution unit localized names, see ResolutionUnits
	 */
	ResUnitsLocNames: ["UnitResInch", "UnitResmm", "UnitRescm"],

	/**
	 * Mapping array for the resolution unit fixes, see ResolutionUnits
	 */
	ResUnitsFixes: [3, 2, 3],

	/**
	 * Mapping array for the screen ruling units (localized names), see ResolutionUnits
	 */
	ScreenUnitsLocNames: ["UnitScreenInch", "UnitScreenmm", "UnitScreencm"],
	/**
	 * Mapping array for the screen ruling unit fixes, see ResolutionUnits
	 */
	ScreenUnitsFixes: [2, 1, 2],

	/**
	 * Mapping array for eye dropper unit localized names, see EyeDropperUnits
	 */
	EyeDropperUnitsLocNames: ["Unitmm2", "Unitcm2", "UnitInch2", "UnitPixels", "", "UnitPercent"],
	EyeDropperUnitsShortLocNames: ["Unitmm2", "Unitcm2", "UnitInch2Short", "UnitPixelsShort", "", "UnitPercent"],
	/**
	 * Mapping array for eye dropper unit fixes, see EyeDropperUnits
	 */
	EyeDropperUnitsFixes: [2, 3, 3, 0, 0, 2]

});

/**
 * @memberof layout
 */
define('web-viewer/layout/LayoutConstants',{
	LayoutItemType: {
		WIDGET: 0,
		TOOLBARCOMPONENT: 1
	},
	Widget: {
		CHANNELS: "channelsWidget",
		CAD: "cadWidget",
		APPROVAL: "approvalDetailsWidget",
		HELP: "help/helpWidget",
		ANNOTATION: "annotationWidget",
		COMPARE: "compareWidget",
		PAGELIST: "pagelist/pagelistWidget",
		DEBUG: "debugWidget",
		INFO: "infoWidget",
		VIEWPORT: "ViewportWidget",
		ALIGNWIZARD: "compareAlignmentWizardWidget",
		DOCUMENTINFO: "documentInfo/DocumentInfoWidget",
		MESSAGE: "message/MessageWidget",
		NAVIGATOR: "navigation/NavigatorWidget",
		VIEWCONTROLS: "viewControlsWidget",
		ATTRIBUTE_CATEGORY: "attributeCategoryWidget",
		VIEW3D: "View3DWidget",
		TEXTCONTENT: "textContent/TextContentWidget"
	},
	ToolbarComponent: {
		ANNOTATION: "annotationControls",
		DYNCONTENT: "dynContentControls",
		APPROVAL: "approvalControls",
		COMPARE: "compareControls",
		MEASURE: "measureControls",
		NAVIGATE: "navigationControls",
		PAGELIST: "pagelistControls",
		DOCUMENTSELECTOR: "documentSelector",
		PAGESELECTION: "pageselectionControls",
		VIEW3D: "view3DControls"
	},
	LayoutMode: {
		BASIC: 0,
		RESTRICTED: 1,
		WIZARD: 2,
		MOBILE_VIEW: 3,
		MOBILE_READERSPREAD: 4,
		MOBILE_ANNOTATION: 5,
		MOBILE_COMPARE: 6,
		MOBILE_3D: 7
	},
	LayoutType: {
		AUTOMATIC: "automatic",
		DESKTOP: "desktop",
		MOBILE: "mobile",
		FLEX: "flex"
	},
	MobileLayoutModeBarOrder: [
		3, // MOBILE_VIEW
		6, // MOBILE_COMPARE
		4, // MOBILE_READERSPREAD
		7, // MOBILE_3D
		5 // MOBILE_ANNOTATION
	],
	ViewerStyle: {
		ADVANCED: 0,
		NORMAL: 1,
		RESTRICTED: 2,
		SIMPLIFIED: 3
	}
});

define('web-viewer/core/ViewerFacade',["web-viewer/core/CoreConstants",
		"web-viewer/layout/LayoutConstants"
	],
	function (CoreConstants,
		LayoutConstants) {
		"use strict";
		/**
		 * The ViewerFacade is the API that the core provides to all other application components.
		 *
		 * The facade provides methods for communication over the EventBus, a debugger, a reference to the DocumentController, etc.
		 * @exports ViewerFacade
		 * @memberof Core
		 * @class ViewerFacade The facade provides methods for communication over the EventBus, a debugger, a reference to the DocumentController, etc.
		 * @param {Core.ViewerCore} coreAPI - The Viewer Core
		 * @param {EventEmitter2} ebus - The event bus
		 * @param {Debug} debug - The debug module
		 */
		var ViewerFacade = function (coreAPI, ebus, debug) {
			this.i18n = null; // filled in by the core

			/**
			 * Execute each of the listeners that may be listening for the specified
			 * event name in order with the list of arguments.
			 *
			 * emit(event, [arg1], [arg2], [...])
			 *
			 * @param {(String|Array)} channel
			 * @param {...Any} parameters
			 */
			this.emit = function () {
				// extra security checks can be added here
				ebus.emit.apply(ebus, arguments);
			};

			this.eDelimiter = ebus.delimiter;

			/**
			 * Adds a listener to the end of the listeners array for the specified
			 * event.
			 *
			 * on(event, listener)
			 *
			 * @param {(String|Array)} channel
			 * @param {Function} listener
			 */
			this.on = function () {
				ebus.on.apply(ebus, arguments);
			};

			/**
			 * Remove a listener from the listener array for the specified event.
			 * Caution: changes array indices in the listener array behind the
			 * listener.
			 *
			 * @param {(String|Array)} channel
			 * @param {Function} listener
			 */
			this.off = function () {
				ebus.off.apply(ebus, arguments);
			};

			/**
			 * Adds a one time listener for the event. The listener is invoked only
			 * the first time the event is fired, after which it is removed.
			 *
			 * @param {(String|Array)} channel
			 * @param {...Any} parameters
			 */
			this.once = function () {
				ebus.once.apply(ebus, arguments);
			};

			/**
			 * Adds a listener that will execute n times for the event before being
			 * removed. The listener is invoked only the first time the event is
			 * fired, after which it is removed.
			 *
			 * @param {(String|Array)} channel
			 * @param {Number} timesToListen
			 * @param {...Any} parameters
			 */
			this.many = function () {
				ebus.many.apply(ebus, arguments);
			};

			/**
			 * Returns an array of listeners for the specified event. This array can be manipulated, e.g. to remove listeners.
			 * @param {(String|Array)} channel
			 * @returns {Array.<Function>} List of functions that listen to the specified event
			 */
			this.listeners = function () {
				return ebus.listeners.apply(ebus, arguments);
			};

			/**
			 * Removes all listeners, or those of the specified event.
			 * Should be used with caution!
			 * @param {String|Array} [channel]
			 */
			this.removeAllListeners = function () {
				ebus.removeAllListeners.apply(ebus, arguments);
			};

			/**
			 * Open document(s)
			 * @param documentInformation {any} String or JSON object containing information on which document to load
			 */
			this.openDocuments = function (fileIdentifier) {
				coreAPI.openDocuments(fileIdentifier);
			};

			/**
			 * Get the DocumentController
			 *
			 * @returns {Data.DocumentController}
			 */
			this.getDocumentController = function () {
				return coreAPI.getDocumentController();
			};

			/**
			 * Get the Config
			 *
			 * @returns {Core.AbstractConfig}
			 */
			this.getConfig = function () {
				return coreAPI.getConfig();
			};

			/**
			 * Get the user who is logged in
			 *
			 * @returns {User}
			 */
			this.getCurrentUser = function () {
				return coreAPI.getCurrentUser();
			};

			/**
			 * Debugger
			 */
			this.debug = debug;


			/**
			 * Get configuration property
			 *
			 * @param {String} key - the key of the property
			 * @returns {Any} the value of the property
			 */
			this.getProperty = function (key) {
				return coreAPI.getProperty(key);
			};


			/**
			 * This returns the DataAdapter
			 *
			 * Usage of this adapter should be limited, and only when really needed!
			 * @returns {AbstractDataAdapter} the data adapter
			 */
			this.getDataAdapter = function () {
				return coreAPI.getDataAdapter();
			};

			/**
			 * Use with care! Do not use this for layout components.
			 * @return {JQueryObj} context
			 */
			this.getContext = function () {
				return coreAPI.getAppContext();
			};


			/**
			 * Request a module, if the module is not available yet, it will be loaded in silenced mode
			 * @param {String} name - name of the module
			 * @param {Function} callback - callback which is called when the module is available, the first argument is a reference to the module
			 * @param {Boolean} [silenced] - if undefined, the module is loaded in silenced mode!
			 */
			this.requestModule = function (name, callback, silenced) {
				if (silenced === undefined) {
					silenced = true;
				}
				coreAPI.requestModule(name, callback, silenced);
			};


			/**
			 * If a module is already loaded, this method will return the module, if the module is not loaded it will return undefined
			 *
			 * !!Note!!  this method will never start to load a module
			 * @param {String} name - name of the module
			 * @returns {Core.AbstractModule} module
			 */
			this.getModule = function (name) {
				return coreAPI.getModuleIfAvailable(name);
			};

			/**
			 * Executes the callback when the module with the given name is loaded.
			 * If it is already there, the callback is immediately executed, otherwise it waits until it is ever loaded.
			 *
			 * !! Note !! this method will never start to load a module
			 * @param {String} name - Name of the module
			 * @param {Function} callback - Parameters: moduleInstance
			 * @param {Any} [binding] - Context binding of the callback
			 */
			this.whenModuleAvailable = function (name, callback, binding) {
				return coreAPI.whenModuleAvailable(name, callback, binding);
			};

			/**
			 * Executes the callback when all modules are loaded.
			 * If this is already the case, the callback is immediately executed, otherwise it waits until it is ever loaded.
			 *
			 * @param {Function} callback
			 */
			this.whenAllModulesAreLoaded = function (callback) {
				return coreAPI.whenAllModulesAreLoaded(callback);
			};

			/**
			 * Get the loaded modules
			 */
			this.getLoadedModules = function () {
				return coreAPI.getLoadedModules();
			};

			/**
			 * Inject state to the current viewer. The viewerState is defined as and object with a entry for each module. The value of this entry is specific for
			 * each module.
			 *
			 * @param {Object} viewerState
			 */
			this.injectState = function (state) {
				return coreAPI.injectState(state);
			};

			/**
			 * Get the state of the viewer
			 * @returns {Object}
			 */
			this.getViewerState = function () {
				return coreAPI.getState();
			};

			/**
			 * Do an Ajax call
			 * @param {String} url - The URL to call
			 * @param {Object} options - Object containing the options for the ajax request. See jQuery API for the meaning
			 * @example
			 * defaultOptions = {
			 *	dataType: "xml",
			 *		success: function(data, textStatus, jqXHR) {},
			 *		error: function(jqXHR, textStatus, errorThrown) {},
			 *		cache: false,
			 *		processData: true,
			 *		contentType: "application/x-www-form-urlencoded; charset=UTF-8",
			 *		type: "GET",
			 *		data: ""
			 *	};
			 *
			 */
			this.ajax = function (url, options) {
				return coreAPI.ajax(url, options);
			};

			/**
			 * Start a ViewerMode
			 * @param {Core.ViewerMode} viewerMode - the viewermode which should be started
			 */
			this.startMode = function (viewerMode) {
				return coreAPI.startMode(viewerMode);
			};

			/**
			 * Stop a ViewerMode
			 * @param {Core.ViewerMode} viewerMode - the viewermode which should be stopped
			 */
			this.stopMode = function (viewerMode) {
				return coreAPI.stopMode(viewerMode);
			};

			/**
			 * Stop the currently active viewermode
			 */
			this.stopActiveMode = function () {
				return coreAPI.stopActiveMode();
			};

			/**
			 * Get the currently active mode
			 * @returns {Core.ViewerMode}
			 */
			this.getActiveMode = function () {
				return coreAPI.getActiveMode();
			};

			/**
			 * Return the type of layout we are in
			 * @returns {LayoutConstants.LayoutType}
			 */
			this.getLayoutType = function () {
				return coreAPI.getLayoutType();
			};

			/**
			 * Is the viewer in mobile mode?
			 * @returns {Boolean}
			 */
			this.isMobile = function () {
				return coreAPI.getLayoutType() === LayoutConstants.LayoutType.MOBILE;
			};
			/**
			 * Swith to another layout
			 * @param {LayoutConstants.LayoutType} layoutType - name of the layout to switch to
			 */
			this.switchLayout = function (layoutType) {
				coreAPI.switchLayout(layoutType);
			};

			/**
			 * Get the FeatureDetectorModule
			 * @returns {FeatureDetectorModule}
			 */
			this.getFeatureDetector = function () {
				return coreAPI.getFeatureDetector();
			};

			/**
			 * Is the state of the viewer stable? can fragments be fetched?
			 * @returns {Boolean} true if OK
			 */
			this.canFetchFragments = function () {
				return coreAPI.canFetchFragments();
			};

		};

		/**
		 * Get an option
		 *
		 * @returns value of the option
		 */
		ViewerFacade.prototype.getOption = function (option) {
			return this.getCurrentUser().getOption(option);
		};

		/**
		 * Set an option
		 * @param {String} key - Key of the option
		 * @param {Object} value - Value of the option
		 * @param {Boolean} [doNotPersist] - True if the option should not be persisted
		 */
		ViewerFacade.prototype.setOption = function (key, newValue, doNotPersist) {
			return this.getCurrentUser().setOption(key, newValue, doNotPersist);
		};

		/**
		 * Set an option with the option to wait for when it's saved.
		 * @param {String} key - Key of the option
		 * @param {Object} value - Value of the option
		 * @param {Function} onPersistedCallback - @param {Function} [onPersistedCallback] - Callback executed when the option is persisted. Called with parameter boolean successful (true) saved or not (false).
		 */
		ViewerFacade.prototype.setOptionCallback = function (key, newValue, onPersistedCallback) {
			return this.getCurrentUser().setOption(key, newValue, false, onPersistedCallback);
		};

		/**
		 * Reset an option to its default value
		 * @param option - the option to reset
		 */
		ViewerFacade.prototype.resetOption = function (option) {
			return this.getCurrentUser().resetOption(option);
		};

		/**
		 * Get the active document
		 * @returns {Data.AbstractDocument}
		 */
		ViewerFacade.prototype.getActiveDocument = function () {
			return this.getDocumentController().getActiveDocument();
		};

		/**
		 * Get the active document
		 * @returns {Data.AbstractDocument}
		 */
		ViewerFacade.prototype.getActivePage = function () {
			return this.getDocumentController().getActivePage();
		};

		/**
		 * Get the size units for the current viewer. Tries to read from user preferences, but has a falback to the config.
		 * @returns {CoreConstants.SizeUnits}
		 */
		ViewerFacade.prototype.getUnits = function () {
			var units = this.getCurrentUser().getOption("PREFERENCES_UNIT_SIZE");
			if (isNaN(units)) {
				// fallback on config
				units = this.getProperty("units");
				if (!isNaN(units)) {
					units = parseInt(units, 10);
				} else {
					units = CoreConstants.SizeUnits.MM;
				}
			}
			return units;
		};

		/**
		 * Get the size units localized name
		 * @returns {String} localized name
		 */
		ViewerFacade.prototype.getUnitsLocName = function (index) {
			var units = index;
			if (units === undefined) {
				units = this.getUnits();
			}
			if (units === CoreConstants.SizeUnits.PIXELS && this.getActivePage() && this.getActivePage().hasCad() && !this.getActivePage().hasGraphics()) {
				return CoreConstants.UnitsLocNames[CoreConstants.SizeUnits.POINTS];
			} else {
				return CoreConstants.UnitsLocNames[units];
			}

		};

		/**
		 * Get the fix for the precision of the size unit
		 * @param {CoreConstants.SizeUnits} index - get unit fix or certain type
		 * @returns {Number} number of precision for the size unit
		 */
		ViewerFacade.prototype.getUnitsFix = function (index) {
			if (index !== undefined) {
				return CoreConstants.UnitsFixes[index];
			} else {
				return CoreConstants.UnitsFixes[this.getUnits()];
			}
		};

		/**
		 * Get the resolution units for the current viewer.
		 * @returns {CoreConstants.ResolutionUnits}
		 */
		ViewerFacade.prototype.getResolutionUnits = function () {
			var units = this.getCurrentUser().getOption("PREFERENCES_UNIT_RESOLUTION");
			return units;
		};

		/**
		 * Get the resolution units localized name
		 * @returns {String} localized name
		 */
		ViewerFacade.prototype.getResolutionUnitsLocName = function () {
			return CoreConstants.ResUnitsLocNames[this.getResolutionUnits()];
		};

		/**
		 * Get the fix for the precision of the resolution unit
		 * @param {CoreConstants.ResolutionUnits} index - get unit fix or certain type
		 * @returns {Number} number of precision for the resolution unit
		 */
		ViewerFacade.prototype.getResolutionUnitsFix = function (index) {
			if (index !== undefined) {
				return CoreConstants.ResUnitsFixes[index];
			} else {
				return CoreConstants.ResUnitsFixes[this.getResolutionUnits()];
			}
		};

		/**
		 * Get the screen ruling units for the current viewer.
		 * @returns {CoreConstants.ResolutionUnits}
		 */
		ViewerFacade.prototype.getScreenUnits = function () {
			var units = this.getCurrentUser().getOption("PREFERENCES_UNIT_SCREENRULING");
			return units;
		};

		/**
		 * Get the sceen ruling units localized name
		 * @returns {String} localized name
		 */
		ViewerFacade.prototype.getScreenUnitsLocName = function () {
			return CoreConstants.ScreenUnitsLocNames[this.getScreenUnits()];
		};

		/**
		 * Get the fix for the precision of the screen ruling unit
		 * @param {CoreConstants.ResolutionUnits} [index] - get unit fix or certain type
		 * @returns {Number} number of precision for the resolution unit
		 */
		ViewerFacade.prototype.getScreenUnitsFix = function (index) {
			if (index !== undefined) {
				return CoreConstants.ScreenUnitsFixes[index];
			} else {
				return CoreConstants.ScreenUnitsFixes[this.getScreenUnits()];
			}
		};

		/**
		 * Shortcut for loading a i18n bundle
		 * @param {String|Array.<String>} bundlename - bundlenmame(s)
		 * @param {Function} callback - callback called when bundle available
		 * @param {Any} [binding]
		 */
		ViewerFacade.prototype.loadBundle = function (bundleName, callback, binding) {
			this.i18n.loadBundle(bundleName, callback, binding);
		};

		/**
		 * Shortcut for getting the translation of a certain key
		 *
		 * @param {String} key - key from which you want the translation
		 * @param {...String} [replacements] - if extra parameters are given, they replace {0}, {1}, ... in the translated string
		 * @returns translation - if no translation is found, the key is returned
		 */
		ViewerFacade.prototype.t = function (bundlename, callback, binding) {
			return this.i18n.t.apply(this.i18n, arguments);
		};

		/**
		 * Check if a parameter update is pending or ongoing
		 * @returns {Boolean}
		 */
		ViewerFacade.prototype.isParameterUpdateOngoing = function () {
			return this.getDocumentController().isParameterUpdateOngoing();
		};

		return ViewerFacade;
	});

define('web-viewer/data/DocumentController',["web-viewer/core/PrivateChannel",
		"utils/Map",
		"utils/StringSet",
		"underscore"
	],
	function (PrivateChannel,
		Map,
		StringSet,
		_) {
		"use strict";

		/**
		 * @class The DocumentController is responsible for keeping track of all loaded
		 * documents. It has methods to load new documents, and set the active
		 * document
		 *
		 * The DocumentController uses the doc private channel
		 *
		 * @exports DocumentController
		 * @memberof Data
		 * @constructor
		 * @param f {Core.ViewerFacade}
		 * @param adapter {AbstractDataAdapter} the DataAdapter which should be used by the DocumentController
		 */
		var DocumentController = function (f, adapter) {
			this.f = f;
			PrivateChannel.installTo(this, f, "doc");

			this._nextIID = 0;
			// {Number} iid -> {AbstractDocument} document
			this._documents = new Map();
			/**
			 * Linked documents contains all documents that are linked from another document
			 * they are not available in this._documents
			 */
			this._linkedDocuments = new Map();

			this._adapter = adapter;

			/**
			 * The page that is currently shown in the viewer
			 * {AbstractDocumentPage}
			 */
			this._activePage;

			/**
			 * The reference page that is used by compare
			 * {AbstractDocumentPage}
			 */
			this._differencePage;

			/**
			 * CAD overlay if any
			 */
			this._cadOverlay;

			/**
			 * This keeps track of which parameter updates are pending
			 */
			this._pendingParameterUpdateOrigins = new StringSet();
			/**
			 * This keeps tracks wether an parameter update is ongoing or not...
			 */
			this._parameterUpdateOngoing = true;

			/**
			 * Number of documents that are still loading
			 */
			this._documentsLoading = 0;


			/**
			 * If the page changes, the previous page is stored here until the parameters update has finished
			 */
			/**
			 * {Boolean}
			 */
			this._activePageHasChanged;
			/**
			 * {AbstractDocumentPage}
			 */
			this._previousActivePage;
			/**
			 * {Boolean}
			 */
			this._differencePageHasChanged;
			/**
			 * {AbstractDocumentPage}
			 */
			this._previousDifferencePage;

			/**
			 * Trigger a debounced parameter update
			 */
			this._debouncedExecuteParameterUpdateIfAllowed = _.debounce(_.bind(this._executeParameterUpdateIfAllowed, this), 20);

			// also trigger parametersUpdated when the parameters are changed on the page (without a call to the ViewServer (this is used by CAD)
			var debouncedNotifyParametersUpdated = _.debounce(_.bind(this._notifyParametersUpdated, this), 20);
			this.on("**.page.*.parameters.changed", _.bind(function () {
				if (!this._parameterUpdateOngoing) {
					debouncedNotifyParametersUpdated();
				}
			}, this));

			var debouncedNotifyStateParametersUpdated = _.debounce(_.bind(this._notifyStateParametersUpdated, this), 20);
			this.on("**.page.*.stateParameter.changed", _.bind(function (page) {
				debouncedNotifyStateParametersUpdated(page);
			}, this));

			this.f.on("intercept.fragmentURL", _.bind(this.interceptFragmentURL, this));

		};

		// /////////////////////////////////////
		// EXTERNAL API
		// /////////////////////////////////////
		/**
		 * Open document(s)
		 * @param {any} fileIdentifier - String or JSON object containing information on which document to load
		 */
		DocumentController.prototype.openDocuments = function (fileIdentifier) {
			this._documentsLoading = this._adapter.getNumberOfDocuments(fileIdentifier);
			this._adapter.openDocuments(fileIdentifier, _.bind(function (doc, makeActive) {
				this._documentsLoading--;
				this.addDocument(doc, makeActive);
				if (doc.hasLinkedDocument()) {
					var linkedDocument = doc.getLinkedDocument();
					this._linkedDocuments.add(linkedDocument.getIID(), linkedDocument);
				}
			}, this));
		};

		/**
		 * Creates a copy from the cad overlay and adds this new document page to the linked pages collection.
		 * It sets a link between this new page and the active (Graphics) document page. After creation the new page is set as cadOverlay.
		 * @param {Boolean} [doNotActivate] do not activate the newly created CAD page
		 * @param {AbstractDocumentPage} [cadToLinkFrom] the CAD that will be used to create the link from
		 */
		DocumentController.prototype.createNewLinkedCadFromOverlay = function (doNotActivate, cadToLinkFrom) {
			if (this._cadOverlay || cadToLinkFrom) {
				if (!cadToLinkFrom) {
					cadToLinkFrom = this._cadOverlay;
				}
				var oldLinkedPage = this._activePage.getDocument().getLinkedDocumentPage();
				var newLinkedPage = this._adapter.getDeepCopy(cadToLinkFrom, undefined, undefined, this.getNextIID());
				this._activePage.getDocument().setLinkedDocumentPage(newLinkedPage);

				this._linkedDocuments.add(newLinkedPage.getDocument().getIID(), newLinkedPage.getDocument());
				if (!doNotActivate) {
					this.changeCadOverlay(newLinkedPage);
				}
				this._emit("newLinkedCad", oldLinkedPage, newLinkedPage);

			} else {
				var oldLinkedPage = this._activePage.getDocument().getLinkedDocumentPage();
				this._activePage.getDocument().setLinkedDocumentPage(null);
				this._cadOverlay = null;
				this._emit("newLinkedCad", oldLinkedPage, null);
			}
		};

		/**
		 * Add a newly created document to the linked documents, changes the linked document of the active document page
		 * @param {AbstractDocument} linkedDocument
		 * @param {Boolean} [doNotActivate] do not activate the newly added linked document
		 */
		DocumentController.prototype.addNewLinkedDocument = function (linkedDocument, doNotActivate) {
			var oldLinkedPage = this._activePage.getDocument().getLinkedDocumentPage();
			var linkedPage = linkedDocument.getDocumentVersion().getPage();
			this._activePage.getDocument().setLinkedDocumentPage(linkedPage);
			this._linkedDocuments.add(linkedDocument.getIID(), linkedDocument);
			if (!doNotActivate) {
				this.changeCadOverlay(linkedPage);
			}
			this._emit("newLinkedCad", oldLinkedPage, linkedPage);
		};

		/**
		 * Load all pages for the given docId and version number
		 * @param {Number} docId
		 * @param {Number} versionNumber
		 */
		DocumentController.prototype.loadDocumentPages = function (docId, versionNumber) {
			var document = this.getDocument(docId);
			if (document !== undefined) {
				var documentVersion = document.getDocumentVersion(versionNumber);
				if (documentVersion !== undefined) {
					this._adapter.getPagesOfDocumentVersion(documentVersion, _.bind(function () {
						this._emit("loaded"); //after all pages are loaded emit loaded, so listeners know changes are made to loaded documents
					}, this));
				}
			}
		};

		/**
		 * Returns all linked documents
		 * @returns {Array.<AbstractDocument>}
		 */
		DocumentController.prototype.getLinkedDocuments = function () {
			return this._linkedDocuments.values();
		};

		/**
		 * Returns a document with the docId
		 * @param {Number} docId
		 * @returns {AbstractDocument}
		 */
		DocumentController.prototype.getDocument = function (docId) {
			var document = this._documents.get(docId);
			if (document) {
				return document;
			} else {
				return this._linkedDocuments.get(docId);
			}
		};

		/**
		 * Retuns a document page
		 * @param {String} pageId
		 * @returns {AbstractDocumentPage}
		 */
		DocumentController.prototype.getDocumentPage = function (pageId) {
			var idArray = pageId.split("_");
			var document = this.getDocument(idArray[0]);
			if (document) {
				var version = document.getDocumentVersion(idArray[1]);
				if (version) {
					return version.getPage(idArray[2]);
				}
			}
		};

		/**
		 * Call this method when you will change things that will trigger a parameter update in the end
		 * @param {String} origin - an identifier, you also need to supply in readyForParameterUpdate
		 */
		DocumentController.prototype.willNeedParameterUpdate = function (origin) {
			if (!this._pendingParameterUpdateOrigins.has(origin)) {
				this._pendingParameterUpdateOrigins.add(origin);
				this._parameterUpdateOngoing = true;
				_.each([this._activePage, this._differencePage, this._cadOverlay], function (page) {
					if (page) {
						page.setParametersAvailable(false);
					}
				}, this);
				this._emit("parameters.willChange", origin);
				this._adapter.cancelDocumentParametersUpdate();
			}
		};

		/**
		 * Call this method when you have applied your changes, and are ready for a parameter update
		 * @param {String} origin - an identifier, the same as provided in the willNeedParameterUpdate call
		 * @param {Function} [callback] - callback called when parameters are updated
		 * @param {Object} [binding] - binding for the callback
		 */
		DocumentController.prototype.readyForParameterUpdate = function (origin, callback, binding) {
			var removed = this._pendingParameterUpdateOrigins.remove(origin);
			// only trigger an update if this origin was already registered by the "willNeedParameterUpdate"
			if (removed !== undefined) {
				this._emit("parameters.readyForUpdate", origin);

				if (callback) {
					this.whenParametersAvailable(callback, binding);
				}

				if (this._pendingParameterUpdateOrigins.size() === 0) {
					// no more pending parameter updates...
					this._debouncedExecuteParameterUpdateIfAllowed();
				}
			}
		};

		/**
		 * Execute the parameter update if there are no more parameter updates pending
		 */
		DocumentController.prototype._executeParameterUpdateIfAllowed = function () {
			if (this._pendingParameterUpdateOrigins.size() === 0) {
				this._emit("parameters.startUpdate");
				this._updateParameters(_.bind(function () {
					var oldPage;

					// mark parameters as available
					_.each([this._activePage, this._differencePage, this._cadOverlay], function (page) {
						if (page) {
							page.setParametersAvailable(true);
						}
					}, this);

					// the parameters have been updated, pages have been updated....
					// before the viewport world is verified
					this._emit("parameters.beforeEndUpdate", this._activePageHasChanged, this._differencePageHasChanged, this._cadOverlayHasChanged);
					// the viewport can react on this event, so it can set itself correctly before other changes occur
					this._emit("parameters.endUpdate", this._activePageHasChanged, this._differencePageHasChanged, this._cadOverlayHasChanged);
					// this event can be used for doing changes to a model that should be in place when modules get the "parametersAvailable" event
					this._emit("parameters.afterEndUpdate", this._activePageHasChanged, this._differencePageHasChanged, this._cadOverlayHasChanged);





					// send out all events, based on parameters...				
					var activePage = this.getActivePage();
					var differencePage = this.getDifferencePage();


					if (activePage) {
						this._emit("activePageParametersChange", activePage, differencePage);
					}

					if (this._activePageHasChanged) {
						// the active page has changed
						this._emit("activePageChangedAndParametersAvailable", this._previousActivePage, activePage);
					}

					if (activePage || this._activePageHasChanged) {
						this._emit("activePageChangedAndParametersAvailableOrChange", this._previousActivePage, activePage);
					}


					if (differencePage) {
						this._emit("differencePageParametersChange", activePage, differencePage);
					}

					if (this._differencePageHasChanged) {
						// the difference page has changed
						this._emit("differencePageChangedAndParametersAvailable", this._previousDifferencePage, differencePage);
					}

					if (differencePage || this._differencePageHasChanged) {
						this._emit("differencePageChangedAndParametersAvailableOrChange", this._previousDifferencePage, differencePage);
					}

					if (this._activePageHasChanged || this._differencePageHasChanged) {
						// the active or difference page has changed
						oldPage = this._activePageHasChanged ? this._previousActivePage : this._previousDifferencePage;
						this._emit("activeOrDifferencePageChangedAndParametersAvailable", activePage, differencePage, oldPage, this._activePageHasChanged);
					}

					if (activePage || differencePage || this._activePageHasChanged || this._differencePageHasChanged) {
						if (this._activePageHasChanged) {
							oldPage = this._previousActivePage;
						} else if (this._differencePageHasChanged) {
							oldPage = this._previousDifferencePage;
						} else {
							oldPage = undefined;
						}

						this._emit("activeOrDifferencePageChangedAndParametersAvailableOrChange", activePage, differencePage, oldPage, this._activePageHasChanged);
					}

					if (this._cadOverlayHasChanged) {
						this._emit("cadOverlayChangedAndParametersAvailable", this._previousCadOverlay, this._cadOverlay);
					}

					if (this._activePageHasChanged || this._differencePageHasChanged || this._cadOverlayHasChanged) {
						this._emit("activeOrDifferenceOrCadOverlayChangedAndParametersAvailable", activePage, differencePage, this._cadOverlay);
					}

					this._parameterUpdateOngoing = false;

					// parameters are up to date
					this._emit("parameters.changed", activePage, differencePage);

					this._activePageHasChanged = false;
					this._differencePageHasChanged = false;
					this._cadOverlayHasChanged = false;
					this._previousActivePage = undefined;
					this._previousDifferencePage = undefined;
				}, this));
			}
		};

		/**
		 * Send out Parameters changed evens if no parameter update is ongoing
		 */
		DocumentController.prototype._notifyParametersUpdated = function () {
			if (!this._parameterUpdateOngoing) {
				// send out all events, based on parameters...				
				var activePage = this.getActivePage();
				var differencePage = this.getDifferencePage();


				if (activePage) {
					this._emit("activePageParametersChange", activePage, differencePage);
					this._emit("activePageChangedAndParametersAvailableOrChange", activePage, activePage);
				}

				if (differencePage) {
					this._emit("differencePageParametersChange", activePage, differencePage);
					this._emit("differencePageChangedAndParametersAvailableOrChange", differencePage, differencePage);
				}

				if (activePage || differencePage) {
					this._emit("activeOrDifferencePageChangedAndParametersAvailableOrChange", activePage, differencePage, differencePage, false);
				}
			}
		};

		/**
		 * Notify listeners that the state parameters have been updated
		 * @param {AbstractDocumentPage} page
		 */
		DocumentController.prototype._notifyStateParametersUpdated = function (page) {
			var differencePage = this.getDifferencePage();
			var cadOverlay = this.getCadOverlay();

			if (page === differencePage) {
				this._emit("referencePageStateParametersChanged", this.getActivePage(), this.getDifferencePage());
			} else if (page === cadOverlay) {
				this._emit("cadOverlayPageStateParametersChanged", this.getActivePage(), this.getDifferencePage(), this.getCadOverlay());
			}
		};

		/**
		 * Check if a parameter update is pending or ongoing
		 * @returns {Boolean}
		 */
		DocumentController.prototype.isParameterUpdateOngoing = function () {
			return this._parameterUpdateOngoing;
		};

		/**
		 * Execute a callback when the parameters are available
		 * (if the parameters are already available and no update is pending, the callback is executed immediately)
		 *
		 * @param {Function} callback - function that is called once the parameters are available (with parameters activePage, differencePage)
		 * @param {Any} [binding] - binding for the callback
		 */
		DocumentController.prototype.whenParametersAvailable = function (callback, binding) {
			if (binding) {
				callback = _.bind(callback, binding);
			}
			if (this.isParameterUpdateOngoing()) {
				this.once("parameters.changed", callback);
			} else {
				callback(this.getActivePage(), this.getDifferencePage());
			}
		};

		/**
		 * Update the parameters of the active and difference pages
		 * @param {Function} callback
		 */
		DocumentController.prototype._updateParameters = function (callback) {
			this._adapter.updateDocumentParameters(this._activePage, this._differencePage, this._cadOverlay, callback);
		};

		/**
		 * Get the currently active page
		 * @returns {Data.AbstractPage} page - undefined if no document active
		 */
		DocumentController.prototype.getActivePage = function () {
			return this._activePage;
		};

		/**
		 * @returns {Boolean} True if there is an active document page, false otherwise.
		 */
		DocumentController.prototype.hasActiveDocumentPage = function () {
			return this._activePage !== undefined;
		};


		/**
		 * @returns {Boolean} True if there is an active document page with the parameters available
		 */
		DocumentController.prototype.hasActiveDocumentPageWithParametersAvailable = function () {
			return this._activePage !== undefined && this._activePage.isParametersAvailable();
		};

		/**
		 * @returns {Boolean} true when the cad overlay is set and its parameters are available
		 */
		DocumentController.prototype.hasCadOverlaywWithParametersAvailable = function () {
			return this._cadOverlay && this._cadOverlay.isParametersAvailable();
		};

		/**
		 *
		 * @returns {Boolean} True if there is a difference page set, false otherwise.
		 */
		DocumentController.prototype.hasDifferenceDocumentPage = function () {
			return this._differencePage !== undefined;
		};


		/**
		 * @returns {Boolean} True if there is a difference document page with the parameters available
		 */
		DocumentController.prototype.hasDifferenceDocumentPageWithParametersAvailable = function () {
			return this._differencePage !== undefined && this._differencePage.isParametersAvailable();
		};

		/**
		 * returns the diffence page
		 * @returns {AbstractDocumentPage}
		 */
		DocumentController.prototype.getDifferencePage = function () {
			return this._differencePage;
		};

		/**
		 * Get the next IID id
		 * @returns {Number} Next ID
		 */
		DocumentController.prototype.getNextIID = function () {
			return this._nextIID++;
		};

		/**
		 * Add a document
		 * @param {Data.AbstractDocument} doc - Document to add
		 * @param {Boolean} makeActive - Make the given document the active document or not
		 */
		DocumentController.prototype.addDocument = function (doc, makeActive) {
			this._documents.add(doc.getIID(), doc);

			this._emit("loaded", doc);

			if (makeActive) {
				if (doc.getDocumentVersion() !== undefined && doc.getDocumentVersion().getFirstPage() !== undefined) {
					this.changeActivePage(doc.getDocumentVersion().getFirstPage());
				}
			}
		};

		/**
		 * Returns the current activated document
		 *
		 * @returns {Data.AbstractDocument} Get the active document or null if none is active
		 */
		DocumentController.prototype.getActiveDocument = function () {
			if (this.hasActiveDocumentPage()) {
				var doc = this._activePage.getDocument();
				if (!this.hasDocument(doc)) {
					this.f.emit("error.WARNING",
						"Trying to get an already removed document");
					return null;
				}
				return doc;
			} else {
				return null;
			}
		};

		/**
		 * Returns the current difference document
		 *
		 * @returns {Data.AbstractDocument} Get the difference document or null if none is active
		 */
		DocumentController.prototype.getDifferenceDocument = function () {
			if (this.hasDifferenceDocumentPage()) {
				var doc = this._differencePage.getDocument();
				if (!this.hasDocument(doc)) {
					this.f.emit("error.WARNING",
						"Trying to get an already removed document");
					return null;
				}
				return doc;
			} else {
				return null;
			}
		};

		/**
		 * Returns the current activated document version
		 *
		 * @returns {Data.AbstractDocumentVersion} Get the active document version or null if none is active
		 */
		DocumentController.prototype.getActiveDocumentVersion = function () {
			if (this.hasActiveDocumentPage()) {
				var docVersion = this._activePage.getDocumentVersion();
				if (!this.hasDocument(docVersion.getDocument())) {
					this.f.emit("error.WARNING",
						"Trying to get an already removed document version");
					return null;
				}
				return docVersion;
			} else {
				return null;
			}
		};

		/**
		 * Returns the current difference document version
		 *
		 * @returns {Data.AbstractDocumentVersion} Get the difference document version or null if none is active
		 */
		DocumentController.prototype.getDifferenceDocumentVersion = function () {
			if (this.hasDifferenceDocumentPage()) {
				var docVersion = this._differencePage.getDocumentVersion();
				if (!this.hasDocument(docVersion.getDocument())) {
					this.f.emit("error.WARNING",
						"Trying to get an already removed document version");
					return null;
				}
				return docVersion;
			} else {
				return null;
			}
		};

		/**
		 * check if document exists in the collection
		 * @param document {AbstractDocument} - check if this document is part of the collection of documents
		 * @returns {Boolean}
		 */
		DocumentController.prototype.hasDocument = function (document) {
			return this._documents.has(document.getIID());
		};

		/**
		 * change the active page
		 * @param {AbstractPage} page
		 */
		DocumentController.prototype.changeActivePage = function (page) {
			if (!page) {
				this.f.emit("error.WARNING", "Trying to change the active page to undefined");
			} else {
				var document = page.getDocument();
				var oldPage = this._activePage;

				var oldDocumentVersion = null;
				if (oldPage) {
					oldDocumentVersion = oldPage.getDocumentVersion();
				}
				if (this.hasDocument(document)) {
					var context = {
						oldActivePage: this._activePage,
						newActivePage: page,
						diffPage: this._differencePage
					};
					this.f.emit("intercept.activePageChange", context);

					if (context.diffPage !== this._differencePage) {
						this.changeDifferencePage(context.diffPage);
					}

					this._activePage = context.newActivePage;
					var newDocumentVersion = this._activePage.getDocumentVersion();

					if (oldPage != this._activePage) {
						this.willNeedParameterUpdate("ACTIVE_PAGE_CHANGE");
						this._activePageHasChanged = true;
						this._previousActivePage = oldPage;

						//reset the state
						if (oldPage) {
							oldPage.resetState();
						}

						var finalPartOfActivePageChange = _.bind(function () {
							// check whether we should set a CAD overlay or not
							if (this._activePage.hasLinkedPage()) {
								// the page has a linked CAD file, set it as cadOverlay
								var cadOverlay = this._activePage.getLinkedPage();
								this.changeCadOverlay(cadOverlay);
							} else {
								// reset cadOverlay
								this.changeCadOverlay();
							}

							this._emit("page.active", oldPage, this._activePage);

							if (oldDocumentVersion != newDocumentVersion) {
								this._emit("documentVersion.active", oldDocumentVersion, newDocumentVersion);
							}

							this.readyForParameterUpdate("ACTIVE_PAGE_CHANGE");
						}, this);

						if (this._activePage.needsDataUpdate()) {
							this._adapter.updateDocumentData(this._activePage, _.bind(function () {
								finalPartOfActivePageChange();
							}, this));
						} else {
							finalPartOfActivePageChange();
						}
						return true;
					} else {
						return false;
					}
				}
			}
		};

		/**
		 * Reset the difference page to undefined
		 */
		DocumentController.prototype.resetDifferencePage = function () {
			var oldPage = this._differencePage;
			if (oldPage !== undefined) {
				this.willNeedParameterUpdate("RESET_DIFFERENCE_PAGE");
				this._differencePageHasChanged = true;
				this._previousDifferencePage = oldPage;
				this._differencePage = undefined;

				oldPage.resetState();

				this._emit("page.difference", oldPage, this._differencePage);

				this.readyForParameterUpdate("RESET_DIFFERENCE_PAGE");
			}
		};

		/**
		 * Change the reference page
		 * @param {AbstractPage} page
		 */
		DocumentController.prototype.changeDifferencePage = function (page) {
			if (page === undefined) {
				this.resetDifferencePage();
			} else {
				var document = page.getDocument();
				var oldPage = this._differencePage;

				if (this.hasDocument(document)) {
					this._differencePage = page;

					if (oldPage != this._differencePage) {
						this.willNeedParameterUpdate("DIFFERENCE_PAGE_CHANGE");
						this._differencePageHasChanged = true;
						this._previousDifferencePage = oldPage;

						// reset the state
						if (oldPage) {
							oldPage.resetState();
						}

						this._emit("page.difference", oldPage, this._differencePage);

						this.readyForParameterUpdate("DIFFERENCE_PAGE_CHANGE");
					}
				}
			}
		};

		/**
		 * Register a callback for when the active document version changes, if there is already a page active,  the callback is called  with the currently active documentversion
		 * @param {Function} callback - Called when active document version changes (oldDoc, newDoc)
		 * @param {Object} [binding] - bind the callback to "binding"
		 */
		DocumentController.prototype.whenActiveDocumentVersionChanged = function (callback, binding) {
			if (binding) {
				callback = _.bind(callback, binding);
			}

			var activeDocument = this.getActiveDocumentVersion();
			if (activeDocument) {
				callback(undefined, activeDocument);
			}

			this.on("documentVersion.active", callback);
		};

		/**
		 * Register a callback for when new documents are loaded. If all documents are already loaded the callback is called immediately.
		 * There is already an active page set when this callback is executed.
		 * @param {Function} callback
		 * @param {Object} binding
		 * @param {Boolean} keepListening - indicate if the callback needs to be called for every "loaded"-event
		 */
		DocumentController.prototype.whenDocumentsLoaded = function (callback, binding, keepListening) {
			if (binding) {
				callback = _.bind(callback, binding);
			}
			if (this._documentsLoading === 0 && this._documents.size() > 0) {
				// documents are loaded
				callback(this.getAllDocuments());
			}

			if (keepListening || this._documentsLoading !== 0) {
				// document are not loaded yet
				this.on("loaded", _.bind(function (doc) {
					if (this._documentsLoading === 0) {
						//call the callback with all loaded documents
						callback(this.getAllDocuments());
					}
				}, this));
			}
		};

		/**
		 * Register a callback for when the documents collection changes. First checks if the documents are loaded if so register listener to "loaded"-event.
		 * @param {Function} callback
		 * @param {Object} binding
		 * @param {Boolean} keepListening - indicate if the callback needs to be called for every "loaded"-event
		 */
		DocumentController.prototype.whenDocumentsLoadedOrChange = function (callback, binding) {
			this.whenDocumentsLoaded(callback, binding, true);
		};

		/**
		 * When changes to loaded documents are made, call this function to let listeners know (needed for synchronisation problems)
		 */
		DocumentController.prototype.loadedDocumentChanged = function () {
			this._emit("loaded");
		};


		/**
		 * Change the CAD overlay
		 * @param {AbstractPage} page
		 */
		DocumentController.prototype.changeCadOverlay = function (page) {
			var oldPage = this._cadOverlay;

			if (page && this._activePage.hasCad()) {
				this.f.debug.error("Setting a CAD overlay is not allowed when the active document is a CAD document");
				page = undefined;
			}

			if (oldPage !== page) {
				this._cadOverlay = page;
				this.willNeedParameterUpdate("CAD_OVERLAY_CHANGE");
				this._cadOverlayHasChanged = true;
				this._previousCadOverlay = oldPage;

				// reset the state
				if (oldPage) {
					oldPage.resetState();
				}

				this._emit("page.cadOverlay", oldPage, this._cadOverlay);

				this.readyForParameterUpdate("CAD_OVERLAY_CHANGE");
			}
		};


		/**
		 * Get CAD Overlay document (if any)
		 * @returns {AbstractDocumentPage} the CAD overlay page
		 */
		DocumentController.prototype.getCadOverlay = function () {
			return this._cadOverlay;
		};
		/**
		 * Register a callback for when the active page changes, if there is already a page active, the callback is called with the currently active page
		 * @param {Function} callback - parameters: (oldPage, newPage)
		 * @param {Object} [binding] - bind the callback to "binding"
		 */
		DocumentController.prototype.whenActivePageChanged = function (callback, binding) {
			if (binding) {
				callback = _.bind(callback, binding);
			}

			var page = this.getActivePage();
			if (page) {
				callback(undefined, page);
			}

			this.on("page.active", callback);
		};

		/**
		 * Register a callback for when the difference page changes, if there is already a page active, the callback is called with the currently active page
		 * @param {Function} callback - parameters: (oldPage, newPage)
		 * @param {Object} [binding] - bind the callback to "binding"
		 */
		DocumentController.prototype.whenDifferencePageChanged = function (callback, binding) {
			if (binding) {
				callback = _.bind(callback, binding);
			}

			var page = this.getDifferencePage();
			if (page) {
				callback(undefined, page);
			}

			this.on("page.difference", callback);
		};

		/**
		 * Register a callback for when the active or difference page changes. if there is already an active or difference page, the callback is called immediatily.
		 * This callback occurs before the parameters are available or are being fetched!
		 * @param {Function} callback - the callback, this callback has no parameters
		 * @param {Object} [binding] - binds the callback to binding
		 */
		DocumentController.prototype.whenActiveOrDifferencePageChanged = function (callback, binding) {
			if (binding) {
				callback = _.bind(callback, binding);
			}

			if (this.getActivePage() || this.getDifferencePage()) {
				callback();
			}

			this.on("page.difference", callback);
			this.on("page.active", callback);
		};

		/**
		 * call the callback when the active pages' parameters change
		 * @param {Function} callback - parameters: activePage, diffPage
		 * @param {Object} [binding] - bind the callback to "binding"
		 */
		DocumentController.prototype.whenActivePageParametersChange = function (callback, binding) {
			if (binding) {
				callback = _.bind(callback, binding);
			}

			this.on("activePageParametersChange", callback);
		};

		/**
		 * call the callback when the reference pages' state parameters change
		 * @param {Function} callback - parameters: activePage, diffPage
		 * @param {Object} [binding] - bind the callback to "binding"
		 */
		DocumentController.prototype.whenReferencePageStateParametersChange = function (callback, binding) {
			if (binding) {
				callback = _.bind(callback, binding);
			}

			this.on("referencePageStateParametersChanged", callback);
		};

		/**
		 * call the callback when the cad overlay pages' state parameters change
		 * @param {Function} callback - parameters: activePage, diffPage
		 * @param {Object} [binding] - bind the callback to "binding"
		 */
		DocumentController.prototype.whenCadOverlayPageStateParametersChange = function (callback, binding) {
			if (binding) {
				callback = _.bind(callback, binding);
			}

			this.on("cadOverlayPageStateParametersChanged", callback);
		};

		/**
		 * call the callback when the difference pages' parameters change
		 * @param {Function} callback - parameters: acitvePage, diffPage
		 * @param {Object} [binding] - bind the callback to "binding"
		 */
		DocumentController.prototype.whenDifferencePageParametersChange = function (callback, binding) {
			if (binding) {
				callback = _.bind(callback, binding);
			}

			this.on("differencePageParametersChange", callback);
		};

		/**
		 * Register a callback which is called when the active page has changed and its parameters are available
		 * @param {Function} callback - parameters: oldPage, page
		 * @param {Object} [binding] - bind the callback to "binding"
		 */
		DocumentController.prototype.whenActivePageChangedAndParametersAvailable = function (callback, binding) {
			if (binding) {
				callback = _.bind(callback, binding);
			}

			this.on("activePageChangedAndParametersAvailable", callback);
			if (this.hasActiveDocumentPageWithParametersAvailable()) {
				callback(undefined, this.getActivePage());
			} else if (this.isParameterUpdateOngoing() && !this._activePageHasChanged) {
				this.whenParametersAvailable(function (activePage, differencePage) {
					callback(undefined, activePage);
				});
			}
		};

		/**
		 * Register a callback which is called when the active page has changed and its parameters are available
		 * or when the parameters of the active page are changed
		 * @param {Function} callback - parameters: oldPage, activePage
		 * @param {Object} [binding] - bind the callback to "binding"
		 */
		DocumentController.prototype.whenActivePageChangedAndParametersAvailableOrChange = function (callback, binding) {
			if (binding) {
				callback = _.bind(callback, binding);
			}

			this.on("activePageChangedAndParametersAvailableOrChange", callback);
			if (this.hasActiveDocumentPageWithParametersAvailable()) {
				callback(undefined, this.getActivePage());
			}
		};


		/**
		 * Register a callback which is called when the difference page has changed and its parameters are available
		 * @param {Function} callback - parameters: oldDiffPage, diffPage
		 * @param {Object} [binding] - bind the callback to "binding"
		 */
		DocumentController.prototype.whenDifferencePageChangedAndParametersAvailable = function (callback, binding) {
			if (binding) {
				callback = _.bind(callback, binding);
			}

			this.on("differencePageChangedAndParametersAvailable", callback);
			if (this.hasDifferenceDocumentPageWithParametersAvailable()) {
				callback(undefined, this.getCadOverlay());
			} else if (this.isParameterUpdateOngoing() && this._differencePage && !this._differencePageHasChanged) {
				this.whenParametersAvailable(function (activePage, differencePage) {
					callback(undefined, differencePage);
				});
			}
		};


		/**
		 * Register a callback which is called when the difference page has changed and its parameters are available
		 * or when the parameters of the difference page are changed
		 * @param {Function} callback - parameters: oldDiffPage,diffPage
		 * @param {Object} [binding] - bind the callback to "binding"
		 */
		DocumentController.prototype.whenDifferencePageChangedAndParametersAvailableOrChange = function (callback, binding) {
			if (binding) {
				callback = _.bind(callback, binding);
			}


			this.on("differencePageChangedAndParametersAvailableOrChange", callback);
			if (this.hasDifferenceDocumentPageWithParametersAvailable()) {
				callback(undefined, this.getDifferencePage());
			}
		};

		/**
		 * Register a callback to be called when the cadOverlay is available and changes
		 * @param {Function} callback - parameters : oldCadOverlay, newCadOverlay
		 * @param {[[Type]]} binding  [[Description]]
		 */
		DocumentController.prototype.whenCadOverlayChanged = function (callback, binding) {
			if (binding) {
				callback = _.bind(callback, binding);
			}
			this.on("page.cadOverlay", callback);

			if (this._cadOverlay) {
				callback(undefined, this._cadOverlay);
			}
		};

		/**
		 * Register a callback which is called when the CAD overlay page has changed and its parameters are available
		 * @param {Function} callback  - parameters: oldCadOverlayPage, cadOverlayPage
		 * @param {Object}   [binding] - bind the callback to "binding"
		 */
		DocumentController.prototype.whenCadOverlayChangedAndParametersAvailable = function (callback, binding) {
			if (binding) {
				callback = _.bind(callback, binding);
			}

			this.on("cadOverlayChangedAndParametersAvailable", callback);
			if (this.hasCadOverlaywWithParametersAvailable()) {
				callback(undefined, this.getCadOverlay());
			} else if (this.isParameterUpdateOngoing() && this._cadOverlay && !this._cadOverlayHasChanged) {
				this.whenParametersAvailable(function (activePage, differencePage) {
					callback(undefined, this._cadOverlay);
				}, this);
			}
		};


		/**
		 * Register a callback which is called when the difference page or active page has changed and its parameters are available
		 * @param {Function} callback - parameters: activePage, diffPage, oldPage (either active or diff, depending on last boolean), activePageHasChanged
		 * @param {Object} [binding] - bind the callback to "binding"
		 */
		DocumentController.prototype.whenActiveOrDifferencePageChangedAndParametersAvailable = function (callback, binding) {
			if (binding) {
				callback = _.bind(callback, binding);
			}

			this.on("activeOrDifferencePageChangedAndParametersAvailable", callback);
			if (this.hasActiveDocumentPageWithParametersAvailable() || this.hasDifferenceDocumentPageWithParametersAvailable()) {
				callback(this.getActivePage(), this.getDifferencePage(), undefined, true);
			} else if (this.isParameterUpdateOngoing() && !this._activePageHasChanged && !this._differencePageHasChanged) {
				this.whenParametersAvailable(function (activePage, differencePage) {
					callback(activePage, differencePage, undefined, true);
				});
			}
		};

		/**
		 * Callback is triggered when either active, difference or cad over has changed and the parameters are available
		 * @param {Function} callback - parameters: activePage, differencePage, cadOverlay
		 * @param {Object} [binding] - bind the callback to "binding"
		 */
		DocumentController.prototype.whenActiveOrDifferenceOrCadOverlayChangedAndParametersAvailable = function (callback, binding) {
			if (binding) {
				callback = _.bind(callback, binding);
			}

			this.on("activeOrDifferenceOrCadOverlayChangedAndParametersAvailable", callback);
			if (this.hasActiveDocumentPageWithParametersAvailable() || this.hasDifferenceDocumentPageWithParametersAvailable() || this.hasCadOverlaywWithParametersAvailable()) {
				callback(this.getActivePage(), this.getDifferencePage(), this.getCadOverlay());
			} else if (this.isParameterUpdateOngoing() && !this._activePageHasChanged && !this._differencePageHasChanged && !this._cadOverlayHasChanged) {
				this.whenParametersAvailable(function (activePage, differencePage) {
					callback(this._activePage, this._differencePage, this._cadOverlay);
				}, this);
			}
		};
		/**
		 * Register a callback which is called when the active or difference page has changed and its parameters are available or change
		 * @param {Function} callback - parameters: activePage, diffPage, oldPage (either active or diff, depending on last boolean), activePageHasChanged
		 * @param {Object} [binding] - bind the callback to "binding"
		 */
		DocumentController.prototype.whenActiveOrDifferencePageChangedAndParametersAvailableOrChange = function (callback, binding) {
			if (binding) {
				callback = _.bind(callback, binding);
			}

			this.on("activeOrDifferencePageChangedAndParametersAvailableOrChange", callback);
			if (this.hasActiveDocumentPageWithParametersAvailable() || this.hasDifferenceDocumentPageWithParametersAvailable()) {
				callback(this.getActivePage(), this.getDifferencePage(), undefined, true);
			}
		};

		/**
		 * Let the active document fill out the needed parameters
		 * @param context
		 */
		DocumentController.prototype.interceptFragmentURL = function (context) {
			if (this._activePage) {
				this._activePage.interceptFragmentURL(context);
			}
		};

		/**
		 * Check is the document with documentId has multiple versions
		 * @param {Number} document IID
		 * @returns {Boolean}
		 */
		DocumentController.prototype.hasDocumentMultipleVersions = function (documentIID) {
			return this._documents.get(documentIID).hasMultipleVersions();
		};


		/**
		 * Get the number of documents
		 * @returns {Number}
		 */
		DocumentController.prototype.getDocumentCount = function () {
			return this._documents.size();
		};

		/**
		 * Get all loaded documents as a non-sorted array
		 * @returns {Array.<AbstractDocument>} list of all loaded documents
		 */
		DocumentController.prototype.getDocuments = function () {
			return this._documents.values();
		};

		/**
		 * Get all documents
		 * @returns {Array.<AbstractDocument>}
		 */
		DocumentController.prototype.getAllDocuments = function () {
			var documents = _.sortBy(this._documents.values(), function (document) {
				if (document.get("docname")) {
					return document.get("docname").toLowerCase();
				} else {
					return document.getIID();
				}
			});
			return documents;
		};

		return DocumentController;
	});

define('web-viewer/core/Extendable',[],
	function () {
		/**
		 * Utility tool to enable easy overriding of methods (with possibility to call the overriden method in the new implementation)
		 * 
		 * Usage: install Extendable to an object that you want to be easily extenable
		 * 
		 * Extendable.makeExtendable(obj)
		 * 
		 * To override a method 
		 * @exports Extendable
		 * @memberof Core
		 * @mixin 
		 * @deprecated proper prototyping should be used. Extendable is no longer needed
		 * @example
		 * this._override("methodName", function(x,y,z){
		 * 		//new implementation
		 * 		this._super(x,y,z); //call to original implementation of methodName
		 * 		// further new implementation
		 * });
		 * 
		 * var Abstract = function(){
		 * 		Extendable.makeExtendable(this);
		 * 		this.someAbstractMethod = function(){
		 * 			this.doSomething();
		 * 			// other stuff
		 * 		}
		 * };
		 * 
		 * var SpecificAbstract = function(){
		 * 		Abstract.call(this);
		 * 		
		 * 		this._override("someAbstractMethod", function(){
		 * 			//first call old implementation
		 * 			this._super();
		 * 			// new implementation
		 * 			this.doSomethingElse();
		 * 		});
		 * };
		 * SpecificAbstract.prototype = new Abstract();
		 * SpecificAbstract.prototype.constructor = SpecificAbstract;
		 */
		var Extendable = {
			/**
			 * Install the Extendable mixin to obj
			 * @static
			 * @param {Object} obj - The object that should become extendable
			 */
			installTo: function (obj) {
				/**
				 * Override an existing method
				 * 
				 * @alias Extendable~_override
				 * @param {String} method - name of the method to override
				 * @param {Function} override - the new implementation for method. Previous implementation is still available through this._super 
				 */
				obj._override = function (method, override) {
					var superMethod = this[method];
					if (superMethod === undefined) {
						superMethod = function () {};
					}
					this[method] = function () {
						var old_super = this._super;
						this._super = superMethod;
						var result = override.apply(this, arguments);
						this._super = old_super;
						return result;
					};
				};
			}
		};

		return Extendable;
	});

define('web-viewer/data/AbstractDocument',["web-viewer/core/Extendable",
		"web-viewer/core/PrivateChannel",
		"underscore"
	],
	function (Extendable,
		PrivateChannel
	) {
		"use strict";
		/**
		 * @class Abstract representation of a Document that can be loaded in the viewer. No
		 * data is fetched in the AbstractClass. Extend this class to have a working
		 * version fetching the correct information
		 * 
		 * each document has an internal id: documentID
		 * 
		 * if a change occurs events should be sent on the channel
		 * doc.[documentID]
		 * 
		 * @exports AbstractDocument
		 * @memberof Data
		 * @constructor
		 * @param {Core.ViewerFacade} f 
		 * @param {Any} documentID - Internal ID of the document
		 * @param {Data.DataConstants.documentPageType} type - type of the document
		 */
		var AbstractDocument = function (f, documentID, type) {
			PrivateChannel.installTo(this, f, ["doc", documentID]);

			/**
			 * @private
			 */
			this.f = f;
			/**
			 * @private
			 * @constant
			 */
			this.documentID = documentID;

			/**
			 * @protected
			 */
			this._documentType = type;

			/**
			 * collection of document versions
			 * !note index 0 won't have version, version numbers start at 1
			 * @private
			 */
			this._documentVersions = [];

			/**
			 * Object containing all data of the document.
			 * 
			 * This object should not be accessed externally!
			 * @private
			 */
			this._data = {};

			/**
			 * A document linked to this document
			 * @private
			 */
			this._linkedDocumentPage = null;
		};

		/**
		 * Get the documentID of the document
		 * @returns {Number} documentID of the document
		 */
		AbstractDocument.prototype.getIID = function () {
			return this.documentID;
		};

		/**
		 * Get a property of the document
		 * @param {String} key - name of the property
		 * @returns {Any} Property value for the given key, or undefined if not found
		 */
		AbstractDocument.prototype.get = function (key) {
			if (this._data[key] !== undefined) {
				return this._data[key];
			} else {
				return undefined;
			}
		};

		/**
		 * Set a linked document Page (link from Graphics to CAD)
		 * @param {AbstractDocumentPage} doc
		 */
		AbstractDocument.prototype.setLinkedDocumentPage = function (doc) {
			this._linkedDocumentPage = doc;
		};

		/**
		 * Get a reference to the linked document
		 * @returns {AbstractDocument} the linked document
		 */
		AbstractDocument.prototype.getLinkedDocument = function () {
			return this._linkedDocumentPage ? this._linkedDocumentPage.getDocument() : undefined;
		};

		/**
		 * returns the linked document page, if no versionNumber is given it selects the latest version. It selects the first
		 * page of the document version. This should be a CAD document which only has one page.
		 * @returns {Data.AbstractDocumentPage}
		 */
		AbstractDocument.prototype.getLinkedDocumentPage = function () {
			return this._linkedDocumentPage;
		};

		/**
		 * has the document a linked document
		 * @returns {Boolean}
		 */
		AbstractDocument.prototype.hasLinkedDocument = function () {
			return this._linkedDocumentPage !== null && this._linkedDocumentPage !== undefined;
		};

		/**
		 * Returns the type of the document
		 * @returns {Data.DataConstants.documentPageType}
		 */
		AbstractDocument.prototype.getDocumentType = function () {
			return this._documentType;
		};

		/**
		 * Change a property of the document
		 * 
		 * @param {String} key - name of the property
		 * @param {Any} newValue - new value
		 * @returns {Boolean} True if success
		 */
		AbstractDocument.prototype.set = function (key, newValue) {
			var oldValue = this._data[key];
			if (oldValue === undefined || (oldValue != undefined && oldValue != newValue)) {
				this._data[key] = newValue;
			}
			return true;
		};

		/**
		 * Adds a documentVersion to the document
		 * @param {Number} versionId - the id of the documentVersion
		 * @param {AbstractDocumentVersion} documentVersion
		 */
		AbstractDocument.prototype.addDocumentVersion = function (documentVersion) {
			this._documentVersions[documentVersion.getVersionId()] = documentVersion;
		};

		/**
		 * Add a document page to a document version
		 * @param {Number} documentVersionId
		 * @param {AbstractDocumentPage} documentPage
		 */
		AbstractDocument.prototype.addDocumentPage = function (documentVersionId, documentPage) {
			this._documentVersions[documentVersionId].addPage(documentPage.getPagenum(), documentPage);
		};

		/**
		 * Get the documentVersion, if versionId is not specified returns the last documentVersion
		 * @param {Number} [versionId]
		 * @returns {Data.AbstractDocumentVersion} undefined if the version is not found 
		 */
		AbstractDocument.prototype.getDocumentVersion = function (versionId) {
			if (versionId === undefined && this._documentVersions.length > 0) {
				return _.last(this._documentVersions);
			} else if (this._documentVersions[versionId]) {
				return this._documentVersions[versionId];
			} else {
				return undefined;
			}
		};

		/**
		 * Does the document have the documentVersion
		 * @param {Number} versionId
		 * @returns {Boolean}
		 */
		AbstractDocument.prototype.hasDocumentVersion = function (versionId) {
			return this._documentVersions[versionId] !== undefined;
		};

		/**
		 * Checks if there are multiple versions
		 * @returns {Boolean}
		 */
		AbstractDocument.prototype.hasMultipleVersions = function () {
			return _.size(this._documentVersions) > 2;
		};

		/**
		 * Get all the versions of the document
		 * @returns
		 */
		AbstractDocument.prototype.getAllDocumentVersions = function () {
			return this._documentVersions.slice(0);
		};

		/**
		 * Loads document pages of previous versions with he same document page number
		 * @param {AbstractDocumentPage} documentPage 
		 * @param {Function} callback
		 */
		AbstractDocument.prototype.loadVersionsForDocumentPage = function (documentPage, callback) {
			//to be implemented
		};

		return AbstractDocument;
	});

define('web-viewer/data/AbstractDocumentVersion',["web-viewer/core/Extendable",
		"web-viewer/core/PrivateChannel",
		"underscore"
	],
	function (Extendable, PrivateChannel) {
		"use strict";
		/**
		 * @class Abstract representation of a DocumentVersion that can be loaded in the viewer. No
		 * data is fetched in the AbstractClass. Extend this class to have a working
		 * version fetching the correct information
		 *
		 * @exports AbstractDocumentVersion
		 * @memberof Data
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {AbstractDocument} doc - reference to the document
		 * @param {Number} versionId - id of the version of the document
		 */
		var AbstractDocumentVersion = function (f, doc, versionId) {
			PrivateChannel.installTo(this, f, ["doc", doc.getIID(), "version", versionId]);
			/**
			 * @private
			 */
			this._f = f;

			/**
			 * reference to the document
			 * @private
			 */
			this._doc = doc;

			/**
			 * Number of the version of the document
			 * @private
			 */
			this._versionId = versionId;

			/**
			 * Specific data of the document version
			 * @private
			 */
			this._versionData = {};

			/**
			 * list of all the pages in this documentversion
			 * @private
			 */
			this._pages = [];
		};

		/**
		 * Get a property of the document
		 * @param {String} key - name of the property
		 * @returns {Any} Property value for the given key, or undefined if not found
		 */
		AbstractDocumentVersion.prototype.get = function (key) {
			if (this._versionData[key] !== undefined) {
				return this._versionData[key];
			} else {
				return undefined;
			}
		};

		/**
		 * Change a property of the document
		 *
		 * @param {String} key - name of the property
		 * @param {Any} newValue - new value
		 * @returns {Boolean} True if success
		 */
		AbstractDocumentVersion.prototype.set = function (key, newValue) {
			var oldValue = this._versionData[key];
			if (oldValue === undefined || (oldValue !== undefined && oldValue != newValue)) {
				this._versionData[key] = newValue;
			}
			return true;
		};

		/**
		 * returns the document of this page
		 * @returns {AbstractDocument}
		 */
		AbstractDocumentVersion.prototype.getDocument = function () {
			return this._doc;
		};

		/**
		 * Get the version number of the documentVersion
		 * @returns {Number}
		 */
		AbstractDocumentVersion.prototype.getVersionId = function () {
			return this._versionId;
		};

		/**
		 * Request a certain page
		 *
		 * If the page is not loaded yet, load it, when the page is available, the callback is called with the page as first argument
		 *
		 * @param {Number} pagenum - number of needed page
		 * @param {Function} callback - callback-function which is called when page is available
		 */
		AbstractDocumentVersion.prototype.requestPage = function (pagenum, callback) {
			var page = this.getPage(pagenum);
			if (page === undefined) {
				this.once(["page", pagenum, "loaded"], _.bind(function () {
					callback(this.getPage(pagenum));
				}, this));
				this.loadPage(pagenum);
			} else if (page === null) {
				this.once(["page", pagenum, "loaded"], _.bind(function () {
					callback(this.getPage(pagenum));
				}, this));
			} else {
				callback(page);
			}
		};

		/**
		 * Load a new page
		 *
		 * @param {Number} pagenum - number of needed page
		 * @param {Function} callback - callback-function which is called when page is available
		 */
		AbstractDocumentVersion.prototype.loadPage = function (pagenum) {
			this._pages[pagenum] = null; //if it fails loading it should place this._pages[pagenum] back to undefined
			this._emit("page.loading", pagenum);
			this._pages[pagenum] = undefined; //= page couldn't be loaded
			// to be implemented by implementation of AbstractDocumentVersion
		};

		/**
		 * Add a page to the document version
		 * @param {Number} pagenum
		 * @param {AbstractDocumentPage} page
		 */
		AbstractDocumentVersion.prototype.addPage = function (pagenum, page) {
			this._pages[pagenum] = page;
			this._emit("page." + pagenum + ".loaded", page);
		};

		/**
		 * Get the page object of a certain page
		 * @param {Number} [pagenum] - Number of the page (if undefined the first page will be returned)
		 * @returns {Data.AbstractDocumentPage} Page at requested pagenum or undefined if the requested page does not exist
		 */
		AbstractDocumentVersion.prototype.getPage = function (pagenum) {
			if (pagenum === undefined) {
				return this.getFirstPage();
			}
			return this._pages[pagenum];
		};

		/**
		 * Check if the document version has a page with pagenum
		 * @param {Number} pagenum
		 * @returns {Boolean}
		 */
		AbstractDocumentVersion.prototype.hasPage = function (pagenum) {
			return this._pages[pagenum] !== undefined;
		};

		/**
		 * Page count of the document
		 * @returns {Number} Page count of the document
		 */
		AbstractDocumentVersion.prototype.getPageCount = function () {
			//to be implemented by specific document version
		};

		/**
		 * Checks if this document version is the latest version
		 * @returns {Boolean}
		 */
		AbstractDocumentVersion.prototype.isLatestVersion = function () {
			return this._doc.getDocumentVersion().getVersionId() === this._versionId;
		};

		/**
		 * Check if the document version has multiple pages
		 * @returns {Boolean}
		 */
		AbstractDocumentVersion.prototype.isMultiPage = function () {
			return this.getPageCount() > 1;
		};

		/**
		 * Check if the document version is a page list
		 * @returns {Boolean}
		 */
		AbstractDocumentVersion.prototype.isPageList = function () {
			// pagelist when pageindex is defined in the pages
			return this.get("isPageList") !== undefined && this.get("isPageList");
		};

		/**
		 * Get the first page
		 * If the first page is an empty page, take the first real page.
		 * @returns {Data.AbstractDocumentPage} The first page
		 */
		AbstractDocumentVersion.prototype.getFirstPage = function () {
			var i = 0;
			while (i < this._pages.length && !this._pages[i]) {
				i++;
			}
			return this._pages[i];
		};

		/**
		 * Get the first not blank/empty page in the document version
		 *
		 * @returns {Data.AbstractDocumentPage} the first not blank/empty page, returns undefined if no page is found
		 */
		AbstractDocumentVersion.prototype.getFirstNotBlankPage = function () {
			var i = 0;
			while (i < this._pages.length && (!this._pages[i] || this._pages[i].isBlankPage())) {
				i++;
			}
			return this._pages[i];
		};

		/**
		 * Get the last page
		 * @returns {Data.AbstractDocumentPage} The last page
		 */
		AbstractDocumentVersion.prototype.getLastPage = function () {
			return _.last(this._pages);
		};

		/**
		 * Get all pages of this version
		 * @returns {Array.<AbstractDocumentPage>} All pages of the version.
		 */
		AbstractDocumentVersion.prototype.getAllPages = function () {
			return _.without(this._pages.slice(0), null, undefined);
		};

		return AbstractDocumentVersion;
	});

define('web-viewer/layout/LayoutItem',["web-viewer/layout/LayoutConstants"],
	function (LayoutConstants) {

		/**
		 * @class LayoutItem A Layout item is a representation of a layout item
		 * @memberof Layout
		 * 
		 * @constructor
		 * @param {String} name - the name of the layout item
		 * @param {LayoutConstants.LayoutItemType} type - type of the layout item
		 * @param {Function} factory - factory function which instantiates the layout item (this function gets a parameter: LayoutType)
		 */
		var LayoutItem = function (name, type, factory) {
			this.name = name;
			this.type = type;

			this._factory = factory;
			this._item = null;
			this._itemLayoutType = null;
		};

		/**
		 * Get the item
		 * @param layoutType
		 * @returns {AbstractWidget|AbstractToolbarComponent} initialized item
		 */
		LayoutItem.prototype.getItem = function (layoutType) {
			if (this._item === null || this._itemLayoutType != layoutType) {
				// we might to some try catching in the future here? :)
				this._item = this._factory(layoutType);
				this._itemLayoutType = layoutType;
			}

			return this._item;
		};

		/**
		 * Is the layout item a widget
		 * @returns {Boolean}
		 */
		LayoutItem.prototype.isWidget = function () {
			return this.type === LayoutConstants.LayoutItemType.WIDGET;
		};

		/**
		 * Is the layout item a toolbar component
		 * @returns {Boolean}
		 */
		LayoutItem.prototype.isToolbarComponent = function () {
			return this.type === LayoutConstants.LayoutItemType.TOOLBARCOMPONENT;
		};


		return LayoutItem;

	});

define('web-viewer/layout/LayoutModel',["web-viewer/core/PrivateChannel",
		"web-viewer/layout/LayoutConstants",
		"web-viewer/layout/LayoutItem"
	],
	function (
		PrivateChannel,
		LayoutConstants,
		LayoutItem
	) {
		"use strict";

		/**
		 * 
		 * @exports LayoutModel
		 * @memberof Layout
		 */
		var LayoutModel = function (f) {
			PrivateChannel.installTo(this, f, ["layoutModel"]);

			this.f = f;

			/**
			 * Maps the name of the layoutItem on a reference to the layoutItem (typically widget, toolbarcomponent)
			 */
			this._items = {};

			/**
			 * Holds all the layoutmodes that are available for the loaded layout.
			 */
			this._layoutModes = {};
		};

		/**
		 * Checks if the layoutItem with itemName is part of the layoutModel
		 * @param {String} itemName
		 * @returns {Boolean}
		 */
		LayoutModel.prototype.hasItem = function (itemName) {
			return this._items[itemName] !== undefined;
		};

		/**
		 * Add a widget to the layoutModel	 
		 * @param {LayoutConstants.Widget} name - the name of the widget
		 * @param {Function} widgetFactory - factory function to instantiate the widget (with 1 parameter: LayoutConstants.LayoutType)
		 */
		LayoutModel.prototype.addWidget = function (name, widgetFactory) {
			var newItem = new LayoutItem(name, LayoutConstants.LayoutItemType.WIDGET, widgetFactory);
			this._items[newItem.name] = newItem;

			this._emit("itemAdded", newItem);
		};

		/**
		 * Add a toolbarComponent to the layoutModel
		 * @param {LayoutConstants.ToolbarComponent} name - the name the toolbarcomponent
		 * @param {Function} toolbarComponentFactory - factory function to instantiate the toolbarcomponent (with 1 parameter: LayoutConstants.LayoutType)
		 */
		LayoutModel.prototype.addToolbarComponent = function (name, toolbarComponentFactory) {
			var newItem = new LayoutItem(name, LayoutConstants.LayoutItemType.TOOLBARCOMPONENT, toolbarComponentFactory);
			this._items[newItem.name] = newItem;

			this._emit("itemAdded", newItem);
		};



		/**
		 * Get the layoutType of the layoutItem
		 * @param {String} itemName
		 * @returns {LayoutConstants.LayoutItemType}
		 */
		LayoutModel.prototype.getLayoutItemType = function (itemName) {
			if (this.hasItem(itemName)) {
				return this._items[itemName].type;
			} else {
				return undefined;
			}
		};

		/**
		 * Get the layout item.
		 * @param {String} itemName
		 * @returns {Object}
		 */
		LayoutModel.prototype.getLayoutItem = function (itemName) {
			if (this.hasItem(itemName)) {
				return this._items[itemName].getItem();
			} else {
				return undefined;
			}
		};

		/**
		 * Register a layoutMode to the model. All available mode for the loaded layout are saved in the model.
		 * @param {LayoutMode} mode
		 */
		LayoutModel.prototype.registerLayoutMode = function (mode) {
			this._layoutModes[mode.getName()] = mode;
		};

		/**
		 * Get the LayoutMode.
		 * @param {LayoutConstants.LayoutMode} modeName
		 * @returns {LayoutMode}
		 */
		LayoutModel.prototype.getLayoutMode = function (modeName) {
			return this._layoutModes[modeName];
		};

		return LayoutModel;
	});

define('web-viewer/layout/LayoutManager',["domReady!",
		"web-viewer/utils/ListenerUtil",
		"web-viewer/layout/LayoutModel",
		"web-viewer/layout/LayoutConstants",
		"utils/DomUtils",
		"underscore",
		"jquery"
	],
	function (
		doc,
		ListenerUtil,
		LayoutModel,
		LayoutConstants,
		DomUtils,
		_,
		$
	) {
		"use strict";

		/**
		 * @class
		 * The LayoutManager is responsible for the layout of the application.
		 *
		 * The LayoutManager is the controller for the layout, it adds widgets, toolbarcomponents to the LayoutModel and lets the layout re-render.
		 *
		 * @exports LayoutManager
		 * @memberof Layout
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {JQueryObj} context - the container in which the viewer application should be drawn
		 * @param {LoadinginIndicator} loadingIndicator - the loading indicator, which was created before all requests started
		 * @param {JQueryObj} initialLoader - the JQuery reference to the element indicating the initial loader (needed to safely remove it without emptying the context)
		 * @param {Function} whenInitialized - callback called once the layout manager is ready to work
		 */

		var LayoutManager = function (f, context, loadingIndicator, initialLoader, whenInitialized) {
			ListenerUtil.mixin(this, f);

			this.f = f;

			/**
			 * Layout model saves the added layoutItems with a reference to the those items (widgets, toolbarcomponents,...)
			 */
			this._layoutModel = new LayoutModel(this.f);

			this._registerListener("layout.loadCSS", this.loadCSS, this);

			/**
			 * Active layout, is set during initialization of the viewer
			 */
			this._layout = null;

			/**
			 * The type of the layout
			 */
			this._layoutType = this.f.getLayoutType();


			var state = this._getLayoutStatePreference();

			// which layout should we load?
			var layoutImplementation = "";
			// which state should be injected on initialization?
			var layoutState = null;


			if (this._layoutType === LayoutConstants.LayoutType.MOBILE) {
				layoutImplementation = "web-viewer/layout/mobile/MobileLayout";
				layoutState = state.layout[LayoutConstants.LayoutType.MOBILE];

			} else if (this._layoutType === LayoutConstants.LayoutType.FLEX) {
				layoutImplementation = "web-viewer/layout/flex/FlexLayout";
				layoutState = state.layout[LayoutConstants.LayoutType.FLEX];
			} else {
				// default to DESKTOP layout
				layoutImplementation = "web-viewer/layout/desktop/DesktopLayout";
				layoutState = state.layout[LayoutConstants.LayoutType.DESKTOP];
			}

			// Temp array used to save widgets that get added before the layout has been loaded
			var widgetsAdded = [],
				toolbarComponentsAdded = [];

			// Add listeners for widgets and toolbarComponents and add them to the temporary array to load them when the layout has been loaded
			var widgetListenerObject = this._registerListener("layout.addWidget", function (name, factory) {
				widgetsAdded.push({
					name: name,
					factory: factory
				});
			});
			var toolbarComponentListenerObject = this._registerListener("layout.addToolbarComponent", function (name, factory) {
				toolbarComponentsAdded.push({
					name: name,
					factory: factory
				});
			});


			// dynamically load the correct layout implementation
			requirejs([layoutImplementation], _.bind(function (Layout) {

				this._layout = new Layout(this.f, this._layoutModel, context, layoutState, loadingIndicator, initialLoader);

				// Add all the widgets and toolbarComponents to the layoutModel that were added prior to layout load
				_.each(widgetsAdded, function (widget) {
					if (!this._layoutModel.hasItem(widget.name)) {
						this._layoutModel.addWidget(widget.name, widget.factory);
					}
				}, this);
				_.each(toolbarComponentsAdded, function (widget) {
					if (!this._layoutModel.hasItem(widget.name)) {
						this._layoutModel.addToolbarComponent(widget.name, widget.factory);
					}
				}, this);

				// Remove the listeners
				this._unregisterListener(widgetListenerObject);
				this._unregisterListener(toolbarComponentListenerObject);


				//Register a listener to when widget needs to be added to the layout
				this._registerListener("layout.addWidget", function (name, factory) {
					//check if the widget wasn't added before, it needs to be unique
					if (!this._layoutModel.hasItem(name)) {
						//add the new widget to the layoutModel
						this._layoutModel.addWidget(name, factory);
					}
				}, this, this.f);

				//register a listener for when a toolbarcomponent needs to be added to the layout
				this._registerListener("layout.addToolbarComponent", function (name, factory) {
					//check if the widget wasn't added before, it needs to be unique
					if (!this._layoutModel.hasItem(name)) {
						//add the new widget to the layoutModel
						this._layoutModel.addToolbarComponent(name, factory);
					}
				}, this, this.f);

				this._registerListener("layout.changed", this._persistLayout, this, this._layout);
				this._registerListener("layout.resetToDefault", _.partial(this._persistLayout, true), this, this.f);

				// notify that the layout is initialized
				if (whenInitialized) {
					whenInitialized(this);
				}
			}, this));

		};

		/**
		 * Dynamically load CSS file
		 *
		 * @param {String} filename - path to css from requireJS base
		 * @param {Function} onload - callback function when css is loaded
		 */
		LayoutManager.prototype.loadCSS = function (filename, onload) {
			DomUtils.loadCss(filename, onload);
		};

		/**
		 * Persist the state of the layout. It will check if the layout needs to be persisted.
		 */
		LayoutManager.prototype._persistLayout = _.debounce(function (reset) {
			if (this._layout.isLayoutStatePersisted()) {
				// we don't wanna overwrite the other states
				var state = this._getLayoutStatePreference();

				if (reset) {
					state.layout[this._layoutType] = undefined;
				} else {
					state.layout[this._layoutType] = this._layout.getLayoutState();
				}
				this.f.setOption("LAYOUT_STATE", JSON.stringify(state));
			}
		}, 100);


		/**
		 * Get the currently saved layout state
		 * @returns {Object} layout state object
		 */
		LayoutManager.prototype._getLayoutStatePreference = function () {
			//get the layoutstate from the user options
			var state = this.f.getOption("LAYOUT_STATE");
			if (state !== undefined && state !== "") {
				try {
					state = $.parseJSON(state); //parse the JSON string to an Object

					// BACKWARDS COMPATIBILITY CODE
					if (!state.layout) {
						// before the LAYOUT_STATE just contained the state information for the DESKTOP layout
						var desktopState = state;
						state = {
							layout: {}
						};
						state.layout[LayoutConstants.LayoutType.DESKTOP] = desktopState;
					}
				} catch (err) {
					// could not parse layout state
					this.f.emit("error.SEVERE", "Could not restore layout configuration", err);
					state = {
						layout: {}
					};
				}
			} else {
				state = {
					layout: {}
				};
			}

			return state;
		};

		/**
		 * Switch the the other layout
		 * @param {LayoutConstants.Layout} layoutType
		 */
		LayoutManager.prototype.switchLayout = function (layoutType) {
			if (this._layoutType != layoutType) {
				var modalBackground = $("<div/>").show().addClass("modalOverlay").text(this.f.t("Loading"));
				this.f.getContext().append(modalBackground);
				this.f.setOptionCallback("LAYOUT_PREFERENCE", layoutType, _.bind(function (success) {
					if (success) {
						this.f.emit("viewer.request.close", window.location, "SaveAndSwitchLayout", "SwitchLayoutWithoutSave");
					}
					modalBackground.remove();
				}, this));
			}
		};

		return LayoutManager;
	});

define('web-viewer/core/AbstractModule',["web-viewer/core/Extendable",
		"web-viewer/core/PrivateChannel",
		"underscore"
	],
	function (Extendable,
		PrivateChannel,
		_) {
		"use strict";

		/**
		 * Initialize the module
		 * @exports AbstractModule
		 * @memberof Core
		 * @class
		 * AbstractModule is an abstract class for defining a module
		 * 
		 * A module is a piece of functionality that is loaded once, only 1 instance of the module is available in the system
		 * The AbstractModule creates a private channel ("module.[name]") on which messages can be put. And foresees some methods for basic interaction
		 * 
		 * The name of a module is unique in the system, a module is a singleton (and can thus only be instantiated once!)
		 * @constructor
		 * @mixes Core.PrivateChannel
		 * @mixes Core.Extendable
		 * @param {Core.ViewerFacade} f	  
		 * @param {Boolean} silenced - if true, the module shouldn't publish tools/widgets/toolbarcomponents 
		 * @param {String} name - unique name of the module
		 */
		var AbstractModule = function (f, silenced, name) {
			this.f = f;
			this.silenced = silenced || false;
			this.name = name;
			// in here all publish events reside until the module is unsilenced
			this._onUnsilence = [];

			Extendable.installTo(this);
			PrivateChannel.installTo(this, f, "module." + name);
		};

		/**
		 * Publish an event as soon as the module is unsilenced
		 * 
		 * @private
		 * @param {String|Array} channel
		 * @param {...Any} arguments
		 */
		AbstractModule.prototype._publish = function () {
			if (this.silenced) {
				this._onUnsilence.push(arguments);
			} else {
				this.f.emit.apply(this.f, arguments);
			}
		};

		/**
		 * Unsilence the module
		 * 
		 * All outstanding events are emitted
		 * 
		 * @private
		 */
		AbstractModule.prototype._unSilence = function () {
			// Publish all waiting events
			_.each(this._onUnsilence, _.bind(function (args) {
				this.f.emit.apply(this.f, args);
			}, this));
			// Clear the waiting events list
			this._onUnsilence = [];
			this.silenced = false;
		};

		/**
		 * Set the silenced mode of the module
		 * If a module is silenced, it won't add widgets/toolbarcomponents/tools
		 * @param {Boolean} silenced
		 */
		AbstractModule.prototype.setSilenced = function (silenced) {
			if (silenced) {
				this.silenced = true;
			} else {
				this._unSilence();
			}
		};

		/**
		 * Publish a widget as soon as the module is unsilenced
		 * @protected
		 * @param {LayoutConstants.Widget} name - the name of the widget
		 * @param {Function} widgetFactory - factory function to instantiate the widget (with 1 parameter: LayoutConstants.LayoutType)
		 */
		AbstractModule.prototype._publishWidget = function (name, widgetFactory) {
			this._publish("layout.addWidget", name, widgetFactory);
		};


		/**
		 * Publish a toolbarcomponent as soon as the module is unsilenced
		 * @protected
		 * @param {LayoutConstants.ToolbarComponent} name - the name the toolbarcomponent
		 * @param {Function} toolbarComponentFactory - factory function to instantiate the toolbarcomponent (with 1 parameter: LayoutConstants.LayoutType)
		 */
		AbstractModule.prototype._publishToolbarComponent = function (name, toolbarComponentFactory) {
			this._publish("layout.addToolbarComponent", name, toolbarComponentFactory);
		};

		/**
		 * Publish a tool as soon as the module is unsilenced
		 * @protected
		 * @param {AbstractTool} tool - The tool should be published
		 */
		AbstractModule.prototype._publishTool = function (tool) {
			this._publish("toolManager.addTool", tool);
		};

		/**
		 * Unpublish a tool
		 * @protected
		 * @param {String} tool - name of the tool
		 */
		AbstractModule.prototype._unpublishTool = function (tool) {
			this._publish("toolManager.removeTool", tool);
		};

		/**
		 * Check if the module has unsaved elements, return false by default
		 * @returns {Boolean} 
		 */
		AbstractModule.prototype.hasUnsavedElements = function () {
			return false;
		};

		/**
		 * The (localized) string representing the unsaved elements
		 * @returns {String}
		 */
		AbstractModule.prototype.getUnsavedElementsName = function () {
			return this.name;
		};

		/**
		 * Saves all unsaved elements of a module
		 * @param {Function} [callback] - called when the save action is complete
		 */
		AbstractModule.prototype.saveAllUnsavedElements = function (callback) {
			if (callback) {
				callback();
			}
		};

		/**
		 * Get the state of a module so it can be restored afterwards via setState.
		 * This are typically filters of the widget, options of the models, etc.
		 * @returns {Object} object containing the state of the module
		 */
		AbstractModule.prototype.getState = function () {
			// to be implemented by the module
		};

		/**
		 * Restore the state of a module as obtained by getState
		 * @param {Object} state - The state to restore of the module
		 */
		AbstractModule.prototype.setState = function (state) {
			// to be implemented by the module
		};

		return AbstractModule;

	});

define('web-viewer/featureDetector/FeatureDetectorModule',["web-viewer/core/AbstractModule",
		"modernizr"
	],
	function (AbstractModule, Modernizr) {

		/**
		 * The FeatureDetector module detects which features are available on a device
		 * @exports FeatureDetectorModule
		 * @memberof FeatureDetectorModule
		 * @class FeatureDetectorModule A Module to detect features of the browser
		 * @augments Core.AbstractModule
		 * @param {Core.ViewerFacade} f
		 * @param {Boolean} silenced
		 */
		var FeatureDetectorModule = function (f, silenced) {
			AbstractModule.call(this, f, silenced, "featureDetector");
		};

		FeatureDetectorModule.prototype = Object.create(AbstractModule.prototype);
		FeatureDetectorModule.prototype.constructor = FeatureDetectorModule;


		/**
		 * Check if the device is a mobile device
		 * @returns {Boolean}
		 */
		FeatureDetectorModule.prototype.isMobile = function () {
			return this.f.getProperty("isMobile") === true;
		};

		/**
		 * Check if the currently used device is an iOS device
		 * @returns
		 */
		FeatureDetectorModule.prototype.isIOS = function () {
			return /(iPad|iPhone|iPod)/i.test(navigator.userAgent);
		};


		/**
		 * Check if device has touch
		 * @returns {Boolean}
		 */
		FeatureDetectorModule.prototype.hasTouch = function () {
			// the last check is to detect Metro IE (IE on touch devices)... The surface2 isn't detected as "Modernizr.touch"
			return Modernizr.touch || (navigator.userAgent.indexOf("Trident") >= 0 && navigator.userAgent.indexOf("Touch") >= 0);
		};

		/**
		 * Check if the device supports canvas
		 * @returns {Boolean}
		 */
		FeatureDetectorModule.prototype.hasCanvas = function () {
			return Modernizr.canvas;
		};


		return FeatureDetectorModule;
	});

define('web-viewer/layout/AbstractWidget',["web-viewer/core/Extendable",
		"web-viewer/core/PrivateChannel",
		"jquery",
		"utils/ShapeUtils",
		"Mousetrap",
		"underscore"
	],
	function (Extendable,
		PrivateChannel,
		$,
		ShapeUtils,
		Mousetrap,
		_) {
		"use strict";

		/**
		 * A widget is an interface element. It get's injected with its context (a
		 * div in which it should display itself)
		 *
		 * @exports AbstractWidget
		 * @memberof Layout
		 * @mixes Core.Extendable
		 * @mixes Core.PrivateChannel
		 * @constructor
		 *
		 * @param {Core.ViewerFacade} f
		 * @param {string} ID - the unique name for the widget
		 */
		var AbstractWidget = function (f, ID) {

			Extendable.installTo(this);
			PrivateChannel.installTo(this, f, ["widget", ID]);

			this.collapsed = false;

			this.visible = true;

			this._isModal = false;

			this.f = f;

			this._enabled = true;

			/**
			 * The title of the widget
			 * @access private
			 */
			this._title = ID;

			/**
			 * The icon of the widget
			 * @access private
			 */
			this._icon = "icon icon-WCR_Cont-Circle";

			/**
			 * The cursor that should be displayed when hovering over the widget
			 * @access private
			 */
			this._currentCursor = "cursor-default";

			/**
			 * In _options all user preferences are stored. This state could be
			 * persisted by a LayoutManager
			 *
			 * All changes to the _options should be done with AbstractWidget.setOption so the
			 * correct events are sent out
			 *
			 * @access private
			 */
			this._options = {};

			/**
			 * Get the unique name of the widget.
			 *
			 * This is a unique identifier for the widget, the widget sends events
			 * on widget.[ID].* channels
			 */
			this.getID = function () {
				return ID;
			};

			/**
			 * Change an option of the widget
			 *
			 * If an option is changed, an event is send on
			 * <widget.[ID].options.changed>
			 *
			 * @param {String} key
			 * @param {Object} newValue
			 */
			this.setOption = function (key, newValue) {
				var oldValue = this._options[key];
				if (oldValue !== newValue) {
					this._options[key] = newValue;
					this._emit("options.changed." + key, oldValue, newValue);
				}
			};

			/**
			 * Get an option of the widget based on the given key.
			 *
			 * @param {String} key - Key of the option
			 * @returns {Object} Option of the widget for the given key or undefined if not found
			 */
			this.getOption = function (key) {
				return this._options[key];
			};

			/**
			 * Get the options of the widget. This method can be used to save the
			 * state of a widget
			 *
			 * @returns {Object} object with all options of the Widget
			 */
			this.getOptions = function () {
				return _.clone(this._options);
			};

		};


		/**
		 * Change multiple options of the widget. This can be used to inject the
		 * state of the widget
		 *
		 * @param {Object} options - object with all options to be set (i.e. {key1: value1, key2: value2, ...}
		 */
		AbstractWidget.prototype.setOptions = function (options) {
			_.each(options, function (value, key) {
				this.setOption(key, value);
			}, this);
		};

		/**
		 * Get the title of the widget
		 * @returns {String} current title of the widget
		 */
		AbstractWidget.prototype.getTitle = function () {
			return this._title;
		};

		/**
		 * Set the title of the widget. Listeners get notified automatically
		 * @param {String} title - the new title for the widget
		 */
		AbstractWidget.prototype.setTitle = function (title) {
			if (this._title !== title) {
				this._title = title;
				this._emit("titlechanged", title);
			}
		};

		/**
		 * Get the icon of the widget
		 * @returns {String} icon string
		 */
		AbstractWidget.prototype.getIcon = function () {
			return this._icon;
		};

		/**
		 * Change the icon of the widget
		 * @param {String} icon
		 */
		AbstractWidget.prototype.setIcon = function (icon) {
			if (this._icon !== icon) {
				this._icon = icon;
				this._emit("iconchanged", icon);
			}
		};




		/**
		 * Set te context of the widget. The widget gets injected with a DOM element in which it should draw itself
		 *
		 * @params {JQueryObj} context - the element in which the widget should draw itself
		 * @params {JQueryObj} headerExtra - a div in which extra icons for the header of the widget can be but (preferably MiniButtons)
		 */
		AbstractWidget.prototype.setContext = function (context, headerExtra) {
			if (!context) {
				this.f.emit("error.WARNING", "Can't call setContext without context!");
				return;
			}

			// set the context of the widget
			this.context = context;
			this.headerExtra = headerExtra;

			this.setCursor(this._currentCursor);
			this._emit("contextAvailable");
		};

		/**
		 * The widget is destroyed
		 */
		AbstractWidget.prototype.destroy = function () {
			this._emit("destroyed");
		};

		/**
		 * Check if the context of the widget has been set
		 * @returns {Boolean} true if the context is set
		 */
		AbstractWidget.prototype.isContextAvailable = function () {
			return this.context !== undefined;
		};

		/**
		 * Execute some code as soon as the context is available. If the context is already available execute the code immediately
		 * @param {Function} callback - callback function to execute when the context of the widget is available
		 * @param {Object} [binding] - bind the callback to "binding"
		 */
		AbstractWidget.prototype.whenContextAvailable = function (callback, binding) {
			if (binding) {
				callback = _.bind(callback, binding);
			}

			if (this.isContextAvailable()) {
				callback();
			} else {
				this.once("contextAvailable", callback);
			}
		};

		/**
		 * Make the widget aware that it is visible or not
		 * !! NOTE !! should only be called by Layout, not by any other module/component
		 * to make a widget visible use the setEnabled property
		 * @param {Boolean} isVisible - True if visible, false otherwise
		 */
		AbstractWidget.prototype.setVisible = function (visible) {
			if (this.visible !== visible) {
				this.visible = visible;
				this._emit("visible", visible);
			}
		};

		/**
		 * Check if the widget is visible or not
		 * @returns {Boolean} True if the widget is visible, false otherwse
		 */
		AbstractWidget.prototype.isVisible = function () {
			return this.visible;
		};

		/**
		 * Notify the widget that is collapsed
		 * @param {Boolean} collapsed
		 */
		AbstractWidget.prototype.setCollapsed = function (collapsed) {
			if (this.collapsed !== collapsed) {
				this.collapsed = collapsed;
				this._emit("collapsed", collapsed);
			}
		};

		/**
		 * is the widget collapsed
		 * @returns {Boolean}
		 */
		AbstractWidget.prototype.isCollapsed = function () {
			return this.collapsed;
		};

		/**
		 * Notify the widget that it has become modal/not-modal
		 * @param {Boolean} isModal
		 */
		AbstractWidget.prototype.setModal = function (isModal) {
			this._isModal = isModal;
		};

		/**
		 * This method notifies the widget that it has been resized
		 */
		AbstractWidget.prototype.resized = function () {

		};

		/**
		 * Call this when the contents of the widget have changed, and the layout manager should try to adapt the layout to the new size
		 * @protected
		 */
		AbstractWidget.prototype._fireResized = function () {
			this._emit("resized");
		};

		/**
		 * Fire an event when the modal widget wrapper has been resized, used to persist this in a user preference
		 */
		AbstractWidget.prototype.fireModalResize = function () {
			this._emit("modalResize");
		};

		/**
		 * Call this when the widget needs to react to a resizing event (from widgetwrapper)
		 */
		AbstractWidget.prototype.resizing = function () {
			//implement in specific widget	
		};

		/**
		 * Change the cursor of the widget
		 *
		 * @param {String} cursor - the class for the cursor that should be displayed
		 */
		AbstractWidget.prototype.setCursor = function (cursor) {
			this.whenContextAvailable(_.bind(function () {
				if (cursor != this._currentCursor) {
					this.context.removeClass(this._currentCursor);
					this.context.addClass(cursor);
					this._currentCursor = cursor;
				}
			}, this));
		};

		/**
		 * request the type of the widget
		 *
		 * @returns {"dockable"|"topbar"|"maincontent" | "messagebar" } widgettype
		 */
		AbstractWidget.prototype.getWidgetType = function () {
			return "dockable";
		};


		/**
		 * Get the minimum size of the widget content
		 *
		 * note it is possible that this size is not respected by the layoutmanager
		 *
		 * @returns {ShapeUtils.Size}
		 */
		AbstractWidget.prototype.getMinimumSize = function () {
			return new ShapeUtils.Size(100, 30);
		};

		/**
		 * Get the preferred size of the widget content
		 *
		 * note it is possible that this size is not respected by the layoutmanager
		 *
		 * @returns {ShapeUtils.Size} if not overridden the widget will determine its size dynamically
		 */
		AbstractWidget.prototype.getPreferredSize = function () {
			var clone = this.context.clone(false);
			clone.css({
				position: "absolute",
				visibility: "hidden",
				overflow: "auto",
				height: "auto",
				width: this.context.width() - 2
			});
			this.f.getContext().append(clone);
			clone.css({
				overflow: "hidden"
			});
			clone.css({
				overflow: "auto"
			});
			var size = new ShapeUtils.Size(clone.outerWidth(true), clone.outerHeight(true));
			clone.remove();
			return size;
		};


		/**
		 * This method returns true if the widget can handle extra space above the preferred size
		 * Credit to BAMT for this great method name!
		 * @returns {Boolean} true if the widget is expandable and will make use of the extra gained space
		 */
		AbstractWidget.prototype.willFill = function () {
			return false;
		};

		/**
		 * Get the maximum dimension of the widget content
		 *
		 * @returns {ShapeUtils.Size} if undefined the widget can be made as big as the user wants
		 */
		AbstractWidget.prototype.getMaximumSize = function () {
			return undefined;
		};

		/**
		 * Called when moving in the DOM of the widget started
		 */
		AbstractWidget.prototype.movingStarted = function () {
			this._emit("movingStarted");
		};

		/**
		 * Called when moving in the DOM of the widget stopped
		 */
		AbstractWidget.prototype.movingStopped = function () {
			this._emit("movingStopped");
		};

		/**
		 * Called when widget is dragged to the specified top and left
		 */
		AbstractWidget.prototype.dragged = function (top, left) {
			this._emit("dragged", top, left);
		};

		/**
		 * Binds event of a key to a function (can be keypress, keyup or keydown)
		 * @param {Array.<Array>} array - 2D array where each record on the first level has three records on the second: the first is an array of the keys
		 * (or combination of keys) to listen to, the second is the function that is called when the event happens and the third is optional
		 * and can be the the actual event to listen to (keyup, keydown or keypress) by default this will be automatically use the most correct one
		 */
		AbstractWidget.prototype.bindKeys = function (array) {
			for (var i = 0; i < array.length; i++) {
				Mousetrap.bind(array[i][0], array[i][1], array[i][2]);
			}
		};

		/**
		 * unbinds the events of keys
		 * @param {Array.<Array>} array
		 */
		AbstractWidget.prototype.unbindKeys = function (array) {
			for (var i = 0; i < array.length; i++) {
				Mousetrap.unbind(array[i][0], array[i][1], array[i][2]);
			}
		};

		/**
		 * Enable or disable the widget. If a widget is disabled it won't be visible in the layout
		 * @param {Boolean} enabled
		 */
		AbstractWidget.prototype.setEnabled = function (enabled) {
			if (this._enabled !== enabled) {
				this._enabled = enabled;
				this._emit("enabled", enabled);
			}
		};

		/**
		 * Is the widget enabled?
		 * @returns {Boolean}
		 */
		AbstractWidget.prototype.isEnabled = function () {
			return this._enabled;
		};


		/**
		 * If the widget is collapsed or disabled, it will be uncollapsed and enabled.
		 * the widget emits its intention to be visible, the layout manager should make sure the widget is visible
		 * (even if the widget is in a collapsed side panel or such)
		 */
		AbstractWidget.prototype.makeVisible = function () {
			if (!this.isEnabled()) {
				this.setEnabled(true);
			}

			this._didRequestMakeVisible = true;

			this._emit("makeVisible");
		};

		/**
		 * Did the widget made a request to become visible that is not yet executed/accepted?
		 * @returns {Boolean} true if the widget requested to become visible, but the request was not accepted
		 */
		AbstractWidget.prototype.didRequestMakeVisible = function () {
			return this._didRequestMakeVisible;
		};

		/**
		 * Set the "makeVisible" request as accepted
		 */
		AbstractWidget.prototype.setMakeVisibleRequestAccepted = function () {
			this._didRequestMakeVisible = false;
		};

		return AbstractWidget;
	});

define('web-viewer/message/MessageWidgetWrapper',["web-viewer/layout/AbstractWidget",
		"web-viewer/layout/LayoutConstants",
		"component/message/MessageComponent",
		"jquery",
		"underscore"
	],
	function (AbstractWidget,
		LayoutConstants,
		MessageComponent,
		$,
		_) {

		"use strict";

		/**
		 * MessageWidget a wrapper around the message widget component specifically for the viewer to conform to the AbstractWidget interface.
		 * @exports MessageWidgetWrapper
		 * @memberof MessageModule
		 * @constructor
		 * @augments Layout.AbstractWidget
		 * @param {Core.ViewerFacade} f
		 * @param {MessageModel} messageModel - The message model where the widget should listen to
		 */
		var MessageWidgetWrapper = function (f, messageModel) {
			AbstractWidget.call(this, f, LayoutConstants.Widget.MESSAGE);

			/**
			 * @member {MessageModel} The message model
			 */
			this._messageModel = messageModel;

			/**
			 * @member {MessageWidget} The message widget
			 */
			this._messageWidget = null;
		};

		MessageWidgetWrapper.prototype = Object.create(AbstractWidget.prototype);
		MessageWidgetWrapper.prototype.constructor = MessageWidgetWrapper;
		MessageWidgetWrapper.widgetName = LayoutConstants.Widget.MESSAGE;

		/**
		 * @override
		 * Get the widget type
		 * @returns {String} "messagebar"
		 */
		MessageWidgetWrapper.prototype.getWidgetType = function () {
			return "messagebar";
		};

		/**
		 * @override
		 * Set the context of the widget
		 * @alias AbstractWidget~setContext
		 */
		MessageWidgetWrapper.prototype.setContext = function (context) {
			AbstractWidget.prototype.setContext.call(this, context);
			if (this._messageWidget == null) {
				this._messageWidget = new MessageComponent.MessageWidget(this._messageModel, context);
			}
			// we want to allow selection of the message
			this.context.addClass("enableTextSelection");
		};

		return MessageWidgetWrapper;
	});

/**
 * @namespace MessageModule
 */
define('web-viewer/message/MessageModule',["web-viewer/core/AbstractModule",
		"web-viewer/message/MessageWidgetWrapper",
		"component/message/MessageComponent",
		"underscore"
	],
	function (AbstractModule,
		MessageWidgetWrapper,
		MessageComponent,
		_) {
		"use strict";

		/**
		 * EVENTS
		 *
		 * userMessage
		 *
		 * 		Listen to user messages to show.
		 *
		 * 		@param {Message|String} message - Message to show. If message is string, type is info.
		 * 		OR
		 * 		@param {String} type - Type of the message (error|warn|info|success).
		 * 		@param {String} message - Message to show.
		 * 		OR
		 * 		@param {String} type - Type of the message (error|warn|info|success).
		 * 		@param {String} message - Message to show.
		 * 		@param {Boolean} autoHide - True if the message should be auto hidden, false otherwise
		 * 		OR
		 * 		@param {String} type - Type of the message (error|warn|info|success).
		 * 		@param {String} message - Message to show.
		 * 		@param {Function | String} action - Function executed when clicking on the message or if string URL accessed when clicking on the message.
		 * 		OR
		 * 		@param {String} type - Type of the message (error|warn|info|success).
		 * 		@param {String} message - Message to show.
		 * 		@param {Function | String} action - Function executed when clicking on the message or if string URL accessed when clicking on the message.
		 * 		@param {Boolean} autoHide - True if the message should be auto hidden, false otherwise
		 * 		OR
		 * 		@param {String} type - Type of the message (error|warn|info|success).
		 * 		@param {String} message - Message to show.
		 *		@param {Function | String} action - Function executed when clicking on the message or if string URL accessed when clicking on the message.
		 * 		@param {Boolean} autoHide - True if the message should be auto hidden, false otherwise
		 * 		@param {String} tag - this is a label for the message, it can be used to make all messages with a certain label obsolete
		 * 		OR
		 * 		@param {String} type - Type of the message (error|warn|info|success).
		 * 		@param {String} message - Message to show.
		 * 		@param {Function | String} action - Function executed when clicking on the message or if string URL accessed when clicking on the message.
		 * 		@param {Boolean} autoHide - True if the message should be auto hidden, false otherwise
		 *		@param {String} tag - this is a label for the message, it can be used to make all messages with a certain label obsolete
		 * 		@param {Boolean} doNotAddIfTagExists - do not add the message if another message with the same tag is being shown or is in the queue
		 *
		 *
		 * userMessage.markObsolete
		 * 		Mark all messages having a certain tag as obsolete
		 * 		@param {String} tag
		 *
		 * error.SEVERE
		 *
		 * 		Listen to severe error messages to show
		 * 		@param {String} message - Message of the error
		 *
		 *
		 * error.WARNING
		 *
		 * 		Listen to warning error messages to show
		 * 		@param {String} message - Message of the warning
		 */

		/**
		 * MessageModule is responsible for notifying the user on certain events (errors, warnings, info etc).
		 * @exports MessageModule
		 * @memberof MessageModule
		 * @constructor
		 * @augments Core.AbstractModule
		 * @param {Core.ViewerFacade} f
		 * @param {Boolean} silenced
		 */
		var MessageModule = function MessageModule(f, silenced) {
			AbstractModule.call(this, f, silenced, "message");

			this._messageModel = new MessageComponent.MessageModel();

			// Message widget
			this._publishWidget(MessageWidgetWrapper.widgetName, _.bind(function (layoutType) {
				this._messageWidgetWrapper = new MessageWidgetWrapper(f, this._messageModel);
				return this._messageWidgetWrapper;
			}, this));

			/**
			 * Listen to userMessage events
			 */
			this.f.on("userMessage", _.bind(this._handleMessageEvent, this));

			/**
			 * Listen to markAsObsolete events
			 */
			this.f.on("userMessage.markObsolete", _.bind(this._markObsolete, this));

			/**
			 * Listen to severe error messages to show
			 * @param {String} message - Message of the error
			 */
			this.f.on("error.SEVERE", _.bind(function (message) {
				this._addToMessageQueue(new MessageComponent.Message("error", message));
			}, this));

			/**
			 * Listen to warning error messages to show
			 * @param {String} message - Message of the warning
			 * @param {Boolean} shouldAutoHide - should the warning message be auto hidden
			 */
			this.f.on("error.WARNING", _.bind(function (message, shouldAutoHide) {
				this._addToMessageQueue(new MessageComponent.Message("warn", message, undefined, shouldAutoHide));
			}, this));
		};

		MessageModule.prototype = Object.create(AbstractModule.prototype);
		MessageModule.prototype.constructor = MessageModule;


		/**
		 * Add to Message Queue and alert the widget that we have a new message
		 * @param {Message} message - Message to add to the queue
		 */
		MessageModule.prototype._addToMessageQueue = function (message) {
			this.f.debug.log("Message received", message);
			this._messageModel.addToMessageQueue(message);
		};


		/**
		 * @param {Message|String} message - Message to show. If message is string, type is info.
		 * OR
		 * @param {String} type - Type of the message (error|warn|info|success).
		 * @param {String} message - Message to show.
		 * OR
		 * @param {String} type - Type of the message (error|warn|info|success).
		 * @param {String} message - Message to show.
		 * @param {Boolean} autoHide - True if the message should be auto hidden, false otherwise
		 * OR
		 * @param {String} type - Type of the message (error|warn|info|success).
		 * @param {String} message - Message to show.
		 * @param {Function | String} action - Function executed when clicking on the message or if string URL accessed when clicking on the message.
		 * OR
		 * @param {String} type - Type of the message (error|warn|info|success).
		 * @param {String} message - Message to show.
		 * @param {Function | String} action - Function executed when clicking on the message or if string URL accessed when clicking on the message.
		 * @param {Boolean} autoHide - True if the message should be auto hidden, false otherwise
		 * OR
		 * @param {String} type - Type of the message (error|warn|info|success).
		 * @param {String} message - Message to show.
		 * @param {Function | String} action - Function executed when clicking on the message or if string URL accessed when clicking on the message.
		 * @param {Boolean} autoHide - True if the message should be auto hidden, false otherwise
		 * @param {String} tag - this is a label for the message, it can be used to make all messages with a certain label obsolete
		 * OR
		 * @param {String} type - Type of the message (error|warn|info|success).
		 * @param {String} message - Message to show.
		 * @param {Function | String} action - Function executed when clicking on the message or if string URL accessed when clicking on the message.
		 * @param {Boolean} autoHide - True if the message should be auto hidden, false otherwise
		 * @param {String} tag - this is a label for the message, it can be used to make all messages with a certain label obsolete
		 * @param {Boolean} doNotAddIfTagExists - do not add the message if another message with the same tag is being shown or is in the queue
		 */
		MessageModule.prototype._handleMessageEvent = function () {
			var nrOfArgs = arguments.length;
			if (nrOfArgs === 1) {
				if (_.isString(arguments[0])) {
					this._addToMessageQueue(new MessageComponent.Message("info", arguments[0]));
				} else {
					this._addToMessageQueue(arguments[0]);
				}
			} else if (nrOfArgs === 2) {
				this._addToMessageQueue(new MessageComponent.Message(arguments[0], arguments[1]));
			} else if (nrOfArgs === 3) {
				if (_.isBoolean(arguments[2])) {
					this._addToMessageQueue(new MessageComponent.Message(arguments[0], arguments[1], false, arguments[2]));
				} else {
					this._addToMessageQueue(new MessageComponent.Message(arguments[0], arguments[1], arguments[2], false));
				}
			} else if (nrOfArgs === 4) {
				this._addToMessageQueue(new MessageComponent.Message(arguments[0], arguments[1], arguments[2], arguments[3]));
			} else if (nrOfArgs === 5) {
				this._addToMessageQueue(new MessageComponent.Message(arguments[0], arguments[1], arguments[2], arguments[3], arguments[4]));
			} else if (nrOfArgs === 6) {
				var tag = arguments[4];
				var doNotAddIfTagExists = arguments[5];
				// first check no message with the same tag exists in the queue or is shown in the widget
				if (!doNotAddIfTagExists || (!_.some(this._messageQueue, function (msg) {
						return msg.tag === tag;
					}) && (!this._messageWidget || !this._messageWidget.isShownMessageWithTag(tag)))) {
					this._addToMessageQueue(new MessageComponent.Message(arguments[0], arguments[1], arguments[2], arguments[3], arguments[4]));
				}

			} else {
				// unrecognized event
				this.f.debug.debug("Unrecognized userMessage event", arguments);
			}
		};

		/**
		 * Mark all messages with "tag" as tag, obsolete
		 * @param tag
		 */
		MessageModule.prototype._markObsolete = function (tag) {
			this._messageModel.markObsolete(tag);
		};

		return MessageModule;
	});

/**
 * @namespace Utils
 */
define('web-viewer/utils/ConversionConstants',{
	MM_TO_INCH: 25.4,
	MM_TO_CM: 10.0,

	DEFAULT_POSTSCRIPT_PPI: 72.0,
	DEFAULT_SCREEN_PPI: 96.0,

	FIXED_BARCODE_DPMM: 300 / 25.4,
	POINTS_TO_MM_RATIO: 25.4 / 72.0,
	MM_TO_POINT_RATIO: 72.0 / 25.4,
	PX_TO_MM_RATIO: 25.4 / 96.0,
	MM_TO_PX_RATIO: 96.0 / 25.4
});

define('web-viewer/utils/GEOUtils',["underscore",
		"utils/ShapeUtils",
		"web-viewer/utils/ConversionConstants"
	],
	function (_,
		ShapeUtils,
		ConversionConstants) {
		var GEOUtils = {
			/**
			 * Convert a ps points number to mm
			 * @param {Number} n - number in ps points to convert to mm
			 * @returns {Number} n converted to mm number
			 */
			convertPSPointsToMM: function (n) {
				// PostScript point was defined as 72 points to the inch (1 point = 1⁄72 inches = 25.4⁄72 mm = 0.3527 mm)
				return n * ConversionConstants.POINTS_TO_MM_RATIO;
			},
			/**
			 * Convert a mm number to ps points
			 * @param {Number} n - number in mm to convert to ps points
			 * @returns {Number} n converted to ps points number
			 */
			convertMMToPSPoints: function (n) {
				// PostScript point was defined as 72 points to the inch (1 point = 1⁄72 inches = 25.4⁄72 mm = 0.3527 mm)
				return n * ConversionConstants.MM_TO_POINT_RATIO;
			}
		};

		return GEOUtils;
	});

define('web-viewer/utils/ViewerUtils',["utils/ShapeUtils",
		"web-viewer/utils/GEOUtils",
		"web-viewer/core/CoreConstants",
		"web-viewer/utils/ConversionConstants",
		"underscore",
		"jquery"
	],
	function (ShapeUtils,
		GEOUtils,
		CoreConstants,
		ConversionConstants,
		_,
		$) {
		"use strict";

		var ViewerUtils = {
			/**
			 * Convert rgb to hex
			 * @param {Number} R - red value in RGB [0-255]
			 * @param {Number} G - green value in RGB [0-255]
			 * @param {Number} B - blue value in RGB [0-255]
			 * @returns {String} rgb in hex format
			 */
			rgbToHex: function (R, G, B) {
				return this.toHex(R) + this.toHex(G) + this.toHex(B);
			},
			/**
			 * Convert a number to hex
			 * @param {Number} n - number to convert to hex format
			 * @returns {String} n in hex format
			 */
			toHex: function (n) {
				n = parseInt(n, 10);
				if (isNaN(n)) {
					return "00";
				}
				n = Math.max(0, Math.min(n, 255));
				return "0123456789ABCDEF".charAt((n - n % 16) / 16) + "0123456789ABCDEF".charAt(n % 16);
			},
			/**
			 * Convert a number encoded RGB color to hex format
			 * @param {Number} color - Number encoded color to convert to hex format
			 * @returns {String} hex format of the number encoded RGB color
			 */
			hexFromInt: function (color) {
				var r = (color >> 16) & 0xFF;
				var g = (color >> 8) & 0xFF;
				var b = (color >> 0) & 0xFF;
				return this.rgbToHex(r, g, b);
			},
			/**
			 * Convert a hex RGB color to a number encoded RGB color
			 * @param {String} hex - Hex encoded RGB color
			 * @returns {Number} number encoded RGB color of the given hex RGB color
			 */
			intFromHex: function (hex) {
				var rgb = this.rgbColor(hex);
				return ((rgb.r & 0xFF) << 16) | //red
					((rgb.g & 0xFF) << 8) | //green
					((rgb.b & 0xFF) << 0); //blue
			},
			/**
			 * Convert an RGB number encoded color to BGR number encoded color
			 * @param {Number} color - RGB number encoded color
			 * @returns {Number} BGR number encoded color from the RGB number encoded color
			 */
			rgbIntToBgrInt: function (color) {
				var r = this.toHex((color >> 16) & 0xFF);
				var g = this.toHex((color >> 8) & 0xFF);
				var b = this.toHex((color >> 0) & 0xFF);
				return parseInt(b + g + r, 16);
			},
			/**
			 * Convert an RGB hex color String to an object
			 * @param {String} string - hex formatted color string
			 * @returns {Object} o.r, o.g, o.b contains respectively the R, G and B value. Returns null if parsing failed.
			 */
			rgbColor: function (string) {
				var result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(string);
				return result ? {
					r: parseInt(result[1], 16),
					g: parseInt(result[2], 16),
					b: parseInt(result[3], 16)
				} : null;
			},
			/**
			 * Trim a String
			 * @param {String} s - String to trim
			 * @returns {String} trimmed string
			 */
			trim: function (s) {
				s = s.replace(/(^\s*)|(\s*$)/gi, "");
				s = s.replace(/[ ]{2,}/gi, " ");
				s = s.replace(/\n /, "\n");
				return s;
			},
			/**
			 * Convert value in mm to another unit with a fixed number of digits
			 * @param {Number} valueInMM - Value to convert in mm
			 * @param {CoreConstants.SizeUnits} units - Unit to convert to (0 for mm, 1 for cm, 2 for inches).
			 * @param {Number} [fix] - Number of digits
			 * @param {Number} [resolution = 72] - resolution for pixel conversion. Defaults to 72 ppi.
			 * @returns {String} Converted value of valueInMM to units with fix number of digits
			 */
			convertFromMmValue: function (valueInMM, units, fix, resolution) {
				var value = valueInMM;
				if (units == CoreConstants.SizeUnits.INCH) { //inch
					value = valueInMM / ConversionConstants.MM_TO_INCH;
				} else if (units == CoreConstants.SizeUnits.CM) { //cm
					value = valueInMM / ConversionConstants.MM_TO_CM;
				} else if (units == CoreConstants.SizeUnits.PIXELS || units == CoreConstants.SizeUnits.POINTS) {
					if (!isNaN(resolution)) {
						value = valueInMM * resolution;
					} else {
						value = valueInMM * ConversionConstants.MM_TO_POINT_RATIO;
					}
				}
				if (fix !== undefined) {
					return value.toFixed(fix);
				} else {
					return value.toString();
				}
			},
			/**
			 * Convert value in mm to another unit with a fixed number of digits
			 * @param {Number} valueInMM - Value to convert in mm
			 * @param {CoreConstants.ResolutionUnits} units - Unit to convert from (0 for mm, 1 for cm, 2 for inches).
			 * @param {Number} [fix] - Number of digits
			 * @param {Number} [resolution = 72] - resolution for pixel conversion. Defaults to 72 ppi.
			 * @returns {String} Converted value of valueInMM to units with fix number of digits
			 */
			convertToMmValue: function (valueInUserUnit, units, fix, resolution) {
				var value = valueInUserUnit;
				if (units == CoreConstants.SizeUnits.INCH) { //inch
					value = valueInUserUnit * ConversionConstants.MM_TO_INCH;
				} else if (units == CoreConstants.SizeUnits.CM) { //cm
					value = valueInUserUnit * ConversionConstants.MM_TO_CM;
				} else if (units == CoreConstants.SizeUnits.PIXELS || units == CoreConstants.SizeUnits.POINTS) {
					if (!isNaN(resolution)) {
						value = valueInUserUnit / resolution;
					} else {
						value = valueInUserUnit * ConversionConstants.POINTS_TO_MM_RATIO;
					}
				}
				if (fix !== undefined) {
					return value.toFixed(fix);
				} else {
					return value.toString();
				}
			},
			/**
			 * Convert from inches to mm
			 * @param {Number} valueInInch - Value to convert in inches
			 * @param {Number} fix - Number of digits
			 * @returns {String} Converted value of valueInInch to mm with fix number of digits
			 */
			convertFromInchToMm: function (valueInInch, fix) {
				var value = valueInInch;
				return (value * ConversionConstants.MM_TO_INCH).toFixed(fix);
			},
			/**
			 * Convert resolution value in ppmm to another unit with a fixed number of digits
			 * @param {Number} value - Value to convert in ppmm
			 * @param {CoreConstants.ResolutionUnits} toMode - Unit to convert to (0 for ppmm, 1 for ppcm, 2 for ppi).
			 * @param {Number} [fix] - Number of digits
			 * @returns {String} Converted value in ppmm to units with fixed number of digits
			 */
			convertResolutionFromPpmmValue: function (value, toMode, fix) {
				if (toMode === CoreConstants.ResolutionUnits.PPMM) {
					value = value * 1; // safety for when we get numbers as strings
				} else if (toMode === CoreConstants.ResolutionUnits.PPCM) {
					value = value * ConversionConstants.MM_TO_CM;
				} else if (toMode === CoreConstants.ResolutionUnits.PPI) {
					value = value * ConversionConstants.MM_TO_INCH;
				}
				if (fix !== undefined) {
					return value.toFixed(fix);
				} else {
					return value.toString();
				}
			},
			/**
			 * Convert resolution value of another unit to ppmm with a fixed number of digits
			 * @param {Number} value - Value to convert to ppmm
			 * @param {CoreConstants.ResolutionUnits} fromMode - Unit to convert from (0 for ppmm, 1 for ppcm, 2 for ppi).
			 * @param {Number} [fix] - Number of digits
			 * @returns {String} Converted value to ppmm with fixed number of digits
			 */
			convertResolutionToPpmmValue: function (value, fromMode, fix) {
				if (fromMode === CoreConstants.ResolutionUnits.PPMM) {
					value = value / 1; // safety for when we get numbers as strings
				} else if (fromMode === CoreConstants.ResolutionUnits.PPCM) {
					value = value / ConversionConstants.MM_TO_CM;
				} else if (fromMode === CoreConstants.ResolutionUnits.PPI) {
					value = value / ConversionConstants.MM_TO_INCH;
				}
				if (fix !== undefined) {
					return value.toFixed(fix);
				} else {
					return value.toString();
				}
			},
			/**
			 * Find the closest angle of a section of the given angle if you divide 360 in numSections
			 * @param {Number} angle - Angle in degrees of which you want to lookup the section
			 * @param {Number} numSections - Number of sections to divide 360 in
			 * @returns {Number} Closest angle of a section of the given angle if you divide 360 in numSections
			 */
			closestAngle: function (angle, numSections) {
				var closest = (Math.round((angle / 360) * numSections)) * 360 / numSections;
				if (closest >= 360) {
					closest -= 360;
				}
				return closest;
			},
			/**
			 * Get the point on a circle distance
			 * @param {Number} radius - radius
			 * @param {Number} angle - angle in degrees
			 * @param {ShapeUtils.Point} center - center of the circle
			 * @returns {ShapeUtils.Point} point on a circle distance
			 */
			pointAtLength: function (radius, angle, center) {
				var a = angle * Math.PI / 180;

				var x = center.x + radius * Math.cos(a);
				var y = center.y + radius * Math.sin(a);

				return new ShapeUtils.Point(x, y);
			},

			/**
			 * Check if the string starts with a certain other string
			 * @param {String} string - String to check
			 * @param {String} startString - String string should start with
			 * @returns {Boolean} True if string starts with startString, false if not
			 */
			startsWith: function (string, startString) {
				return string.indexOf(startString) === 0;
			},
			/**
			 * Check if the string ends with a certain other string
			 * @param {String} string    - The string to check
			 * @param {String} endString - String string should end with
			 * @returns {Boolean} True if string ends with endString, false if not
			 */
			endsWith: function (string, endString) {
				var position = string.length - endString.length;
				var lastIndex = string.indexOf(endString, position);
				return lastIndex !== -1 && lastIndex === position;
			},
			/**
			 * Generate a unique DOM ID with the given prefix
			 * e.g. "input_45610" if prefix was "input_"
			 * @param {String} prefix - Prefix of the unique DOM ID
			 * @returns {String} unique DOM ID with the given prefix
			 */
			generateUniqueDOMId: function generateUniqueDOMId(prefix) {
				var nr = Math.floor(Math.random() * 10000);
				var id = prefix + nr;
				while (document.getElementById(id)) {
					return generateUniqueDOMId(prefix);
				}
				return id;
			},

			/**
			 * Convert the  date string to an actual date object
			 * @param {String} date - XFDF date string
			 * @returns {Date} date object of the xfdf date string
			 */
			dateFromString: function (date) {
				//D:20121016171755+02'00'
				var year = date.substring(2, 6);
				var month = date.substring(6, 8);
				var day = date.substring(8, 10);
				var hour = date.substring(10, 12);
				var minute = date.substring(12, 14);
				var second = date.substring(14, 16);
				var offsethour = date.substring(16, 19);
				var offsetminute = date.substring(20, 22);

				//"2012-01-17T12:55:00.000+01:00"
				return new Date(year + '-' + month + '-' + day + 'T' + hour + ':' + minute + ':' + second + '.000' + offsethour + ':' + offsetminute);
			},
			/**
			 * < n will be converted to 0n
			 * @param {Number} n - Number to convert
			 * @returns formatted number to two digits
			 */
			twoDigits: function (n) {
				return n > 9 ? "" + n : "0" + n;
			},
			/**
			 * Get a XFDF date String from a Date object
			 * @param {Date} date - date object to convert
			 * @returns {String} XFDF date String from the given date object
			 */
			stringFromDate: function (date) {
				//D:20121016171755+02'00'

				var offset = date.getTimezoneOffset();
				var timezoneString = offset <= 0 ? "+" : "-";

				offset = Math.abs(offset);
				var timezonehours = Math.floor(offset / 60);
				var timezoneminutes = offset - (timezonehours * 60);
				timezoneString += this.twoDigits(timezonehours) + "'" + this.twoDigits(timezoneminutes) + "'";

				return "D:" + date.getUTCFullYear() + this.twoDigits(date.getUTCMonth() + 1) + this.twoDigits(date.getUTCDate()) + this.twoDigits(date.getHours()) + this.twoDigits(date.getMinutes()) + this.twoDigits(date.getUTCSeconds()) + timezoneString;
			},
			/**
			 * Generate a random string of length characters in [0-9a-zA-Z]
			 * @param {Number} length - Number of characters
			 * @returns {String} random string of length characters in [0-9a-zA-Z]
			 */
			randomString: function (length) {
				var chars = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
				var result = '';
				for (var i = length; i > 0; --i) {
					result += chars[Math.round(Math.random() * (chars.length - 1))];
				}
				return result;
			},
			/**
			 * Convert a ShapeUtils.Rect to a String "left, top, right, bottom" in PS points
			 * @param {ShapeUtils.Rect} rect - Rect (in mm) to convert to String
			 * @returns {String} a String "left, top, right, bottom" in PS points
			 */
			rectToString: function (rect) {
				var x1 = GEOUtils.convertMMToPSPoints(rect.origin.x);
				var y1 = GEOUtils.convertMMToPSPoints(rect.origin.y);
				var x2 = GEOUtils.convertMMToPSPoints(rect.right());
				var y2 = GEOUtils.convertMMToPSPoints(rect.bottom());

				return x1 + "," + y1 + "," + x2 + "," + y2;
			},

			/**
			 * Parse a float regardless language settings
			 * @param {String | Number} string - String to parse
			 * @returns {Number} the original number it was already a number, otherwise a parsed number (or NaN) if it was a string, else undefined
			 */
			parseFloat: function (string) {
				if (_.isString(string)) {
					string = string.trim();
					if (string) {
						return +(string.replace(/,/, '.'));
					}
				} else if (_.isNumber(string)) {
					return string;
				}
				return undefined;
			},
			/**
			 * Parse the first found float regardless language settings
			 * @param {String | Number} string - String to parse
			 * @returns {Number} the original number it was already a number, otherwise a parsed number (or NaN) if it was a string, else undefined
			 */
			parseFirstFloat: function (string) {
				if (_.isString(string)) {
					string = string.trim();
					if (string) {
						return parseFloat(string.replace(/,/, '.'));
					}
				} else if (_.isNumber(string)) {
					return string;
				}
				return undefined;
			},
			/**
			 * Is the number close to another number with a given precision
			 * @param   {Number} number
			 * @param   {Number} closeTo
			 * @param   {Number} precision
			 * @returns {Boolean}
			 */
			isCloseTo: function (number, closeTo, precision) {
				return Math.abs(closeTo - number) < (Math.pow(10, -precision) / 2);
			}
		};

		return ViewerUtils;
	});

define('lib/AnimationOptions',["underscore"], 
function(_) {		
"use strict";

	/**
	 * Create a new AnimationOptions object of the given type
	 * see http://greweb.me/2012/02/bezier-curve-based-easing-functions-from-concept-to-implementation/
	 * 
	 * MIT License
	 *
	 * KeySpline - use bezier curve for transition easing function
	 * Copyright (c) 2012 Gaetan Renaudeau <renaudeau.gaetan@gmail.com>
	 *
	 * Permission is hereby granted, free of charge, to any person obtaining a
	 * copy of this software and associated documentation files (the "Software"),
	 * to deal in the Software without restriction, including without limitation
	 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
	 * and/or sell copies of the Software, and to permit persons to whom the
	 * Software is furnished to do so, subject to the following conditions:
	 *
	 * The above copyright notice and this permission notice shall be included in
	 * all copies or substantial portions of the Software.
	 *
	 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
	 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
	 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
	 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
	 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
	 * DEALINGS IN THE SOFTWARE.
	 * 
	 * Notice: changed by Esko to become prototyped
	 */

	/**
	 * @param {String} type - One of "ease", "linear", "ease-in", "ease-out", or "ease-in-out"
	 */
	var AnimationOptions = function(type){
	
		this._types = {
			    "ease":        [0.25, 0.1, 0.25, 1.0], 
			    "linear":      [0.00, 0.0, 1.00, 1.0],
			    "ease-in":     [0.42, 0.0, 1.00, 1.0],
			    "ease-out":    [0.00, 0.0, 0.58, 1.0],
			    "ease-in-out": [0.42, 0.0, 0.58, 1.0]
		};
		
		this._options = this._types[type];
		if(!this._options){
			this._options = this._types["linear"];
		}
		
		this._mX1 = this._options[0];
		this._mY1 = this._options[1];
		this._mX2 = this._options[2];
		this._mY2 = this._options[3];				
	};
	
	/**
	 * Get the current position on the bezier based on a [0 1] value
	 * @param {Number} aX - value in [0-1]
	 * @returns {Number} The current position on the bezier based on a [0 1] value
	 */
	AnimationOptions.prototype.get = function(aX) {
		if (this._mX1 == this._mY1 && this._mX2 == this._mY2) return aX; // linear
		return this._calcBezier(this._getTForX(aX), this._mY1, this._mY2);
	};
	 
	/**
	 * @private
	 * @param aA1
	 * @param aA2
	 * @returns {Number}
	 */
	AnimationOptions.prototype._a = function(aA1, aA2) { 
		return 1.0 - 3.0 * aA2 + 3.0 * aA1; 
	};
	
	/**
	 * @private
	 * @param aA1
	 * @param aA2
	 * @returns {Number}
	 */
	AnimationOptions.prototype._b = function(aA1, aA2) { 
		return 3.0 * aA2 - 6.0 * aA1; 
	};
	
	/**
	 * @private
	 * @param aA1
	 * @returns {Number}
	 */
	AnimationOptions.prototype._c = function(aA1){ 
		return 3.0 * aA1;
	};
	 
	/**
	 * @private
	 * Returns x(t) given t, x1, and x2, or y(t) given t, y1, and y2.
	 * @param aT
	 * @param aA1
	 * @param aA2
	 * @returns {Number}
	 */
	AnimationOptions.prototype._calcBezier = function(aT, aA1, aA2) {
		 return ((this._a(aA1, aA2)*aT + this._b(aA1, aA2))*aT + this._c(aA1))*aT;
	};
	 
	/**
	 * @private
	 * Returns dx/dt given t, x1, and x2, or dy/dt given t, y1, and y2.
	 * @param aT
	 * @param aA1
	 * @param aA2
	 * @returns
	 */
	AnimationOptions.prototype._getSlope = function(aT, aA1, aA2) {
		return 3.0 * this._a(aA1, aA2)*aT*aT + 2.0 * this._b(aA1, aA2) * aT + this._c(aA1);
	};
	 
	/**
	 * @private
	 * @param aX
	 * @returns
	 */
	AnimationOptions.prototype._getTForX = function(aX) {
		// Newton raphson iteration
		var aGuessT = aX;
		for (var i = 0; i < 4; ++i) {
		  var currentSlope = this._getSlope(aGuessT, this._mX1, this._mX2);
		  if (currentSlope == 0.0) return aGuessT;
		  var currentX = this._calcBezier(aGuessT, this._mX1, this._mX2) - aX;
		  aGuessT -= currentX / currentSlope;
		}
		return aGuessT;
	};
	
	return AnimationOptions;
});

define('web-viewer/viewport/ViewportWorld',["underscore",
		"utils/Transformation",
		"utils/ShapeUtils",
		"web-viewer/core/PrivateChannel",
		"web-viewer/utils/ViewerUtils",
		"lib/AnimationOptions"
	],
	function (_,
		Transformation,
		ShapeUtils,
		PrivateChannel,
		ViewerUtils,
		AnimationOptions) {
		"use strict";

		/**
		 * @exports ViewportWorld
		 * @memberof ViewportModule
		 * @class The world keeps track of the actual size (WorldSize) of the viewport content  (in its own units, e.g. mm) and the size of the viewport within the screen (SceneSize)
		 * @constructor
		 * @param {Core.ViewerFacade} f - Viewer Facade
		 * @param {String} IID - internal id
		 * @param {Number} [iz=1] - initial zoom percentage (e.g. "0.9" will show the first view at 90%)
		 */
		var ViewportWorld = function (f, IID, iz) {
			PrivateChannel.installTo(this, f, ['transform', IID]);

			/**
			 * @member {Object} - see transform options
			 */
			this._initialTransformation = undefined;
			this._transform = undefined;
			this._maxZoom = undefined;
			this._minZoom = undefined;
			//is initial zoom, Minimum of SceneX/WorldX and SceneY/WorldY when the document is completely visible
			this._initialStartZoom = undefined;
			//initialZoom is the initial zoom percentage when view is created
			this._initialZoom = (iz ? iz : 1);

			/* Animation Options */
			this._allowAnimation = true;
			this._animationFramerate = 45;
			this._defaultAnimationDuration = 250;
			this._currentAnimation = null;

			/* Transformation Options */
			this._rotation = 0; // in degrees
			this._offsetSize = new ShapeUtils.Size();
			this._zoom = 1;
			this._flip = false;
			this._flipViewFromBack = false;

			/* Sizes */
			this._worldSize = new ShapeUtils.Size();
			this._sceneSize = new ShapeUtils.Size();

			this._worldOrigin = new ShapeUtils.Point();

			// this is the resolution of the document (ppmm) 
			this._worldResolution = {
				x: 0,
				y: 0
			};

			this._didChangeSinceLastZoomToFit = false;

			// timer used while animating
			this._animationTimer = null;
		};

		/**
		 * Recalculate the initial start zoom of the transformation and return it immediately
		 * @returns {Number} the start zoom of the transformation, based on the world/scene size
		 */
		ViewportWorld.prototype.recalculateInitialStartZoom = function () {

			//if the worldSize is 0 dont calculate the startzoom ==> will give Infinity
			if (this._worldSize.width === 0 || this._worldSize.height === 0) {
				return this._zoom;
			}

			//rotation needs to be fixed
			var rotate = true;
			if (this._rotation % 180 === 0) {
				rotate = false;
			}
			var fullWidthZoom = (rotate ? this._sceneSize.height : this._sceneSize.width) / this._worldSize.width;
			var fullHeightZoom = (rotate ? this._sceneSize.width : this._sceneSize.height) / this._worldSize.height;

			var z = Math.min(fullWidthZoom, fullHeightZoom);
			z *= this._initialZoom;

			var minZoom = this.getIntegerZoom(-this.getNumberOfMinZoomLevels());
			var maxZoom = this.getIntegerZoom(this.getNumberOfMaxZoomLevels());
			if (this._minZoom !== minZoom || this._maxZoom !== maxZoom) {
				this._minZoom = minZoom;
				this._maxZoom = maxZoom;
				this._emit("minMaxZoomChanged", this, this._minZoom, this._maxZoom);
			}

			this._initialStartZoom = z;
			return z;
		};

		/**
		 * Check whether the current zoom is full view
		 * @returns {Boolean} True if the current zoom is the initial start zoom (= full view)
		 */
		ViewportWorld.prototype.isFullView = function () {
			return this._zoom === this._initialStartZoom;
		};

		/**
		 * Check whether the current zoom is smaller than the full view
		 * @returns {Boolean} True if the current view is smaller than the full view (so zoomed out)
		 */
		ViewportWorld.prototype.isSmallerThanFullView = function () {
			return this._zoom < this._initialStartZoom;
		};

		/**
		 * Get the initial start zoom
		 * This is the initial zoom as minimum of SceneX/WorldX and SceneY/WorldY when the document is completely visible
		 * @returns {Number} The initial start zoom
		 */
		ViewportWorld.prototype.getInitialStartZoom = function () {
			return this._initialStartZoom;
		};

		/**
		 * Get the initial zoom
		 * It is the initial zoom percentage when viewport is created
		 * @returns {Number} the initial zoom of the viewport world
		 */
		ViewportWorld.prototype.getInitialZoom = function () {
			return this._initialZoom;
		};

		/**
		 * Zoom the world to fit in the world
		 * @param {Boolean} animated - True if it should be animated
		 */
		ViewportWorld.prototype.zoomToFit = function (animated) {
			this._offsetSize = new ShapeUtils.Size();
			this.transform({
				fixedzoom: this.recalculateInitialStartZoom(),
				animated: animated
			});

			this._didChangeSinceLastZoomToFit = false;
		};

		/**
		 * Zoom the world to fit width in the world
		 * @param {Boolean} animated - True if it should be animated
		 */
		ViewportWorld.prototype.zoomToFitWidth = function (animated) {
			this._reportChange();
			var rotate = true;
			if (this._rotation % 180 === 0) {
				rotate = false;
			}

			var denominator = null;
			if (rotate) {
				if (this._worldSize.height !== 0) {
					denominator = this._worldSize.height;
				}
			} else {
				if (this._worldSize.width !== 0) {
					denominator = this._worldSize.width;
				}
			}

			this._offsetSize = new ShapeUtils.Size();

			if (denominator !== null) {
				var fullWidthZoom = this._sceneSize.width / denominator;
				fullWidthZoom *= this._initialZoom;
				this.transform({
					fixedzoom: fullWidthZoom,
					animated: animated
				});
			} else {
				this.updateTransformation(animated);
			}
		};

		/**
		 * Zoom the world to fit height in the world
		 * @param {Boolean} animated - True if it should be animated
		 */
		ViewportWorld.prototype.zoomToFitHeight = function (animated) {
			this._reportChange();
			var rotate = true;
			if (this._rotation % 180 === 0) {
				rotate = false;
			}

			var denominator = null;
			if (rotate) {
				if (this._worldSize.width !== 0) {
					denominator = this._worldSize.width;
				}
			} else {
				if (this._worldSize.height !== 0) {
					denominator = this._worldSize.height;
				}
			}

			this._offsetSize = new ShapeUtils.Size();

			if (denominator !== null) {
				var fullHeightZoom = this._sceneSize.height / denominator;
				fullHeightZoom *= this._initialZoom;
				this.transform({
					fixedzoom: fullHeightZoom,
					animated: animated
				});
			} else {
				this.updateTransformation(animated);
			}
		};

		/**
		 * Zoom the world to 1:1
		 * @param {Boolean} animated - True if it should be animated
		 * @param {Number} [worldResolution] - Pixels per mm. Define if you want to use another resolution than the default world resolution
		 */
		ViewportWorld.prototype.zoomOneToOne = function (animated, worldResolution) {
			this._reportChange();

			var zoom = worldResolution;
			if (zoom === undefined) {
				if (this._worldResolution && this._worldResolution.x !== 0) {
					zoom = this._worldResolution.x;
				}
			}

			this.transform({
				fixedzoom: zoom,
				animated: animated
			});
		};

		/**
		 * Get the zoom value for the transform
		 * @returns {Number} the current zoom level
		 */
		ViewportWorld.prototype.getZoom = function () {
			return this._zoom;
		};

		/**
		 * Get the current zoom percentage
		 * @returns {Number} The current zoom percentage
		 */
		ViewportWorld.prototype.getZoomPercentage = function () {
			if (this._worldResolution && this._worldResolution.x !== 0) {
				return this._zoom / this._worldResolution.x * 100;
			}
			return 0;
		};

		/**
		 * Get the number of zoom levels
		 * @returns {Number} Number of zoom levels
		 */
		ViewportWorld.prototype.getNumberOfZoomLevels = function () {
			return this.getNumberOfMinZoomLevels() + this.getNumberOfMaxZoomLevels() + 1;
		};

		/**
		 * Get the number of zoom levels to zoom out
		 * @returns {Number}
		 */
		ViewportWorld.prototype.getNumberOfMinZoomLevels = function () {
			// until 65536:1
			return 16;
		};

		/**
		 * Get the number of zoom levels to zoom in
		 * @returns {Number}
		 */
		ViewportWorld.prototype.getNumberOfMaxZoomLevels = function () {
			// until 1:512
			return 9;
		};

		/**
		 * Get the max zoom of the world
		 * @returns {Number}
		 */
		ViewportWorld.prototype.getMaxZoom = function () {
			return this._maxZoom;
		};

		/**
		 * Get the min zoom of the world
		 * @returns {Number}
		 */
		ViewportWorld.prototype.getMinZoom = function () {
			return this._minZoom;
		};

		/**
		 * Zoom to a scene rectangle, takes a rotation of the document page into account
		 * @param {ShapeUtils.Rect} rect - Rectangle to zoom to
		 * @param {Boolean} animated - True if it should be animated
		 */
		ViewportWorld.prototype.zoomToSceneRect = function (rect, animated) {
			this._reportChange();
			var worldRect = this.getWorldRectFromScene(rect);
			var fullWidthZoom = Math.abs(this._sceneSize.width / worldRect.size.width);
			var fullHeightZoom = Math.abs(this._sceneSize.height / worldRect.size.height);
			var rotation = this.getRotation() % 360;
			if (rotation === 90 || rotation === 270) {
				fullWidthZoom = Math.abs(this._sceneSize.width / worldRect.size.height);
				fullHeightZoom = Math.abs(this._sceneSize.height / worldRect.size.width);
			}

			this.transform({
				fixedzoom: Math.min(fullWidthZoom, fullHeightZoom),
				middle: worldRect.center(),
				animated: animated
			});
		};

		/**
		 * Set the size of the scene
		 * @param {ShapeUtils.Size} s - size of the scene (screen)
		 */
		ViewportWorld.prototype.setSceneSize = function (s) {
			this._sceneSize = s;
			this.updateTransformation(false, true);
		};

		/**
		 * Get the current scene size
		 * @returns {ShapeUtils.Size} the size of the scene
		 */
		ViewportWorld.prototype.getSceneSize = function () {
			return this._sceneSize;
		};

		/**
		 * Get the bounds of the scene
		 * @returns {ShapeUtils.Rect} the bounds of the scene
		 */
		ViewportWorld.prototype.getSceneBounds = function () {
			return new ShapeUtils.Rect(null, this._sceneSize.copy());
		};

		/**
		 * Get the current size of the world
		 * @returns {ShapeUtils.Size} the size of the world
		 */
		ViewportWorld.prototype.getWorldSize = function () {
			return this._worldSize;
		};

		/**
		 * Get the current bounds of the world
		 * @returns {ShapeUtils.Rect} the bounds of the world
		 */
		ViewportWorld.prototype.getWorldBounds = function () {
			return new ShapeUtils.Rect(this._worldOrigin.copy(), this._worldSize.copy());
		};


		/**
		 * Set the world bounds.
		 * This updates the world size and world origin.
		 * @param {ShapeUtils.Rect} bounds - New world bounds
		 */
		ViewportWorld.prototype.setWorldBounds = function (bounds) {
			this._worldSize = bounds.size.copy();
			this._worldOrigin = bounds.origin.copy();
			this.updateTransformation(false);
			this._emit("worldBoundsChanged");
		};

		/**
		 * Check whether the world is busy with an animation
		 * @returns {Boolean} True if the world is doing an animation
		 */
		ViewportWorld.prototype.isAnimating = function () {
			return this._currentAnimation != null && !this._currentAnimation.done;
		};

		/**
		 * Check whether the world is busy with a rotation animation
		 * @returns {Boolean} True if the world is doing a rotation animation
		 */
		ViewportWorld.prototype.isAnimatingRotation = function () {
			var t1 = this.getTransformation();
			var t2 = this.calculateCurrentTransformation();

			var rotation1 = t1.getRotation();
			var rotation2 = t2.getRotation();

			return rotation1 != rotation2;
		};

		/**
		 * Gets the current rotation (in degrees)
		 * @returns {Number} the current rotation in degrees
		 */
		ViewportWorld.prototype.getRotation = function () {
			return this._rotation;
		};

		/**
		 * Get the transformation Matrix
		 * @returns {Transformation} the transformation of the world
		 */
		ViewportWorld.prototype.getTransformation = function () {
			if (!this._transform) {
				this.updateTransformation();
			}
			return this._transform.copy();
		};

		/**
		 * Calculate the rubber value
		 * @param {Number} value - current value
		 * @param {Number} rest - overflow
		 * @param {Number} max - maximum rubber
		 * @returns {Number} converted number based on the rubber value
		 */
		ViewportWorld.prototype.rubber = function (value, rest, max) {
			var stretch = (value - rest) / (max - rest);
			if (stretch <= 0) {
				return value;
			}
			stretch = stretch / (1 + stretch);
			return rest + (max - rest) * stretch;
		};

		/**
		 * Constrain the zoom
		 * @param {Boolean} rubberBand - if true, it will allow a rubber band, which will give a hint that the zoom is being constrained
		 * @returns {Boolean} if the zoom was constrained
		 */
		ViewportWorld.prototype.constrainZoom = function (rubberBand) {
			if (this._zoom > this._maxZoom) {
				if (rubberBand) {
					this._zoom = this.rubber(this._zoom, this._maxZoom, this._maxZoom * 1.6);
				} else {
					this._zoom = this._maxZoom;
				}
				return true;
			} else if (this._zoom < this._minZoom) {
				if (rubberBand) {
					this._zoom = this.rubber(this._zoom, this._minZoom, this._minZoom / 1.6);
				} else {
					this._zoom = this._minZoom;
				}
				return true;
			} else {
				return false;
			}
		};

		/**
		 * Constrain the rotation
		 * @param {Boolean} hard - if hard is true, it will put a hard 90 degrees constrain
		 */
		ViewportWorld.prototype.constrainRotation = function (hard) {
			if (hard && this._rotation % 90 !== 0) {
				this._rotation = parseInt(this._rotation, 10) % 360;
				if (this._rotation < 0) {
					this._rotation += 360;
				}

				this._rotation = ViewerUtils.closestAngle(this._rotation, 4);
			}
		};

		/**
		 * Constrain panning
		 * @param {Boolean} rubberBand - if true, it will allow a rubber band, which will give a hint that the zoom is being constrained
		 * @returns {Boolean} if the pan was constrained
		 */
		ViewportWorld.prototype.constrainPan = function (rubberBand) {
			var newMiddle = this.getMiddle();
			//TODO: implementation is not as it should be, newMiddle has rotated coordinates => not correct! see known issues 
			if (this.isSmallerThanFullView()) {
				//Don't allow panning when the view is smaller than the start view
				this._offsetSize = new ShapeUtils.Size();
				return;
			} else {
				var cSize = this._worldSize.copy();
				var isRotated = false;
				//checks if the document has been rotated
				if (this._rotation % 180 === 90) {
					isRotated = true;
				}
				var izoom = this.recalculateInitialStartZoom(); //is initial zoom, Minimum of SceneX/WorldX and SceneY/WorldY when the document is completly visible
				if (this._zoom < izoom) { //zoom is the value for the current zoom
					izoom = this._zoom * this._initialZoom; //initialZoom is the initial zoom percentage when view is created
				}
				//original calculations to constrain pan
				//this._sceneSize.width / izoom is the width of the document+the background-border at zoom-to fit of the document
				var left = ((cSize.width - this._sceneSize.width / izoom + this._sceneSize.width / this._zoom) * 0.5);
				var right = ((cSize.width + this._sceneSize.width / izoom - this._sceneSize.width / this._zoom) * 0.5);
				var top = ((cSize.height - this._sceneSize.height / izoom + this._sceneSize.height / this._zoom) * 0.5);
				var bottom = ((cSize.height + this._sceneSize.height / izoom - this._sceneSize.height / this._zoom) * 0.5);
				//
				if (isRotated) { //when rotated the bounds are different
					left = ((cSize.height - this._sceneSize.width / izoom - this._sceneSize.width / this._zoom) * 0.5);
					right = ((cSize.height + this._sceneSize.width / izoom + this._sceneSize.width / this._zoom) * 0.5);
					top = ((cSize.width - this._sceneSize.height / izoom - this._sceneSize.height / this._zoom) * 0.5);
					bottom = ((cSize.width + this._sceneSize.height / izoom + this._sceneSize.height / this._zoom) * 0.5);
				}
				//the coordinates of newMiddle are rotated!! convert to normal coordinates with origin in top left corner
				var tempMiddle = newMiddle.copy();
				//first make sure we don't include the origin of the world in our calculations (this would make a lot harder!)
				newMiddle.offset(new ShapeUtils.Size(-this._worldOrigin.x, -this._worldOrigin.y));
				tempMiddle.offset(new ShapeUtils.Size(-this._worldOrigin.x, -this._worldOrigin.y));

				if (this._rotation % 360 === 90) {
					tempMiddle = new ShapeUtils.Point(cSize.height - newMiddle.y, newMiddle.x);
				} else if (this._rotation % 360 === 180) {
					tempMiddle = new ShapeUtils.Point(cSize.width - newMiddle.x, cSize.height - newMiddle.y);
				} else if (this._rotation % 360 === 270) {
					tempMiddle = new ShapeUtils.Point(newMiddle.y, cSize.width - newMiddle.x);
				}

				//checks if the middle point lays in the bounds
				if (tempMiddle.x < left) {
					tempMiddle.x = left;
				}
				if (tempMiddle.x > right) {
					tempMiddle.x = right;
				}
				if (tempMiddle.y < top) {
					tempMiddle.y = top;
				}
				if (tempMiddle.y > bottom) {
					tempMiddle.y = bottom;
				}


				// also check if the document is still within the view bounds
				if (isRotated) {
					left = 0;
					right = cSize.height;
					top = 0;
					bottom = cSize.width;
				} else {
					left = 0;
					right = cSize.width;
					top = 0;
					bottom = cSize.height;
				}


				if (tempMiddle.x < left) {
					tempMiddle.x = left;
				}
				if (tempMiddle.x > right) {
					tempMiddle.x = right;
				}
				if (tempMiddle.y < top) {
					tempMiddle.y = top;
				}
				if (tempMiddle.y > bottom) {
					tempMiddle.y = bottom;
				}
				//convert the coordinates back to set middle if needed
				newMiddle = tempMiddle;
				if (this._rotation % 360 === 90) {
					newMiddle = new ShapeUtils.Point(tempMiddle.y, cSize.height - tempMiddle.x);
				} else if (this._rotation % 360 === 180) {
					newMiddle = new ShapeUtils.Point(cSize.width - tempMiddle.x, cSize.height - tempMiddle.y);
				} else if (this._rotation % 360 === 270) {
					newMiddle = new ShapeUtils.Point(cSize.width - tempMiddle.y, tempMiddle.x);
				}

				newMiddle.offset(new ShapeUtils.Size(this._worldOrigin.x, this._worldOrigin.y));
			}

			var middle;
			if (!rubberBand) {
				middle = this.getMiddle();
				if (!middle.equals(newMiddle)) {
					this.setMiddle(newMiddle);
					return true;
				} else {
					return false;
				}
			} else {
				middle = this.getMiddle();
				var delta = new ShapeUtils.Point();
				delta.x = Math.round(middle.x - newMiddle.x);
				delta.y = Math.round(middle.y - newMiddle.y);
				middle = newMiddle;
				var stretch = Math.sqrt(delta.x * delta.x + delta.y * delta.y);

				if (stretch < 0.01) {
					return false;
				}

				var newStretch = this.rubber(stretch, 0, 200 / this._zoom);
				delta.x *= newStretch / stretch;
				delta.y *= newStretch / stretch;

				middle.x += delta.x;
				middle.y += delta.y;
				this.setMiddle(middle);
				return true;
			}
		};

		/**
		 * Sets the middle of the world
		 * @param {ShapeUtils.Point} middle - the new middle
		 */
		ViewportWorld.prototype.setMiddle = function (middle) {
			this._reportChange();
			var center = this.getWorldBounds().center();

			this._offsetSize = new ShapeUtils.Size(center.x - middle.x, center.y - middle.y);
		};

		/**
		 * Get the middle of the world
		 * @returns {ShapeUtils.Point} - the middle of the world
		 */
		ViewportWorld.prototype.getMiddle = function () {
			var center = this.getWorldBounds().center();
			return new ShapeUtils.Point(center.x - this._offsetSize.width, center.y - this._offsetSize.height);
		};

		/**
		 * Update the transformation matrix
		 * @param {Boolean | Object} animation - either "false" or "true"
		 * @param hardrotation - hard rotation constrain
		 * @param {Boolean} [tmpDisableConstraints] - if true constraints are disabled temporarily
		 */
		ViewportWorld.prototype.updateTransformation = function (animation, hardrotation, tmpDisableConstraints) {
			if (!tmpDisableConstraints) {
				this.constrainZoom(false);
				this.constrainPan(false); //rubber doesn't work correctly with current constrainPan

				this.constrainRotation(hardrotation);
			}


			var newtransform = this.calculateCurrentTransformation();
			if (this._transform === undefined) {
				this._transform = newtransform;
			}

			if (newtransform.equals(this._transform)) { //transforms are the same. Do not animate
				animation = false;
			}

			if (!this.isAnimationAllowed()) {
				animation = false;
			}

			if (animation) {
				if (_.isBoolean(animation)) {
					animation = {};
				}
				if (!animation.duration) {
					animation.duration = this._defaultAnimationDuration;
				}
				//animation.duration=5000; // uncomment this line for slower animations (makes it easier to debug) (slow animationspeed)
				var gain = (100 / (animation.duration / this._animationFramerate)) / 100;

				if (!animation.type) {
					animation.type = "ease-in-out";
				}
				var animationOption = new AnimationOptions(animation.type);

				animationOption.from = (animation.interrupt && this._currentAnimation && !this._currentAnimation.done ? this._currentAnimation.to : this._transform);
				animationOption.to = newtransform;
				animationOption.gain = gain;
				animationOption.progress = 0;
				animationOption.done = false;
				this.updateTransformationAnimated(animationOption);

				if (this._currentAnimation) {
					this._currentAnimation.done = true;
				}
				this._currentAnimation = animationOption;
			} else {
				if (this.isAnimating()) {
					this.cancelAnimation(true);
				}
				this._transform = newtransform;
				this.fireTransformChanged();
			}

		};

		/**
		 * Create the current transformation matrix
		 * @returns {Transformation} the current transformation matrix
		 */
		ViewportWorld.prototype.calculateCurrentTransformation = function () {
			var worldSceneBounds = new ShapeUtils.Rect(null, new ShapeUtils.Size(this._sceneSize.width / this._zoom, this._sceneSize.height / this._zoom));
			var offsetX = (worldSceneBounds.size.width - this._worldSize.width) / 2;
			var offsetY = (worldSceneBounds.size.height - this._worldSize.height) / 2;

			var t = new Transformation();

			t.scale(this._zoom, this._zoom);

			t.translate(worldSceneBounds.size.width / 2, worldSceneBounds.size.height / 2);
			t.rotate(this._rotation * (Math.PI / 180));
			if (this.isFlip()) {
				t.scale(-1, 1);
			}
			if (this.isFlipViewFromBack()) {
				if (this._rotation % 180 === 0) {
					// not rotated
					t.scale(-1, 1);
				} else {
					// rotated, but flip horizontally
					t.scale(1, -1);
				}
			}
			t.translate(-worldSceneBounds.size.width / 2, -worldSceneBounds.size.height / 2);

			t.translate(offsetX, offsetY);
			if (this._worldOrigin) {
				t.translate(-this._worldOrigin.x, -this._worldOrigin.y);
			}
			t.translate(this._offsetSize.width, this._offsetSize.height);

			// round to integer values so we only draw on pixels
			t.e = Math.floor(t.e);
			t.f = Math.floor(t.f);

			return t;
		};

		/**
		 * Update the transformation
		 * @param {AnimationOption} animationOptions
		 */
		ViewportWorld.prototype.updateTransformationAnimated = _.throttle(function (animationOption) {
			if (animationOption.done) {
				return; //not longer the latest animation
			}
			if (animationOption.progress > 0.9999 || animationOption.progress >= 1) {
				animationOption.progress = 1;
			}
			if (animationOption.from.equals(animationOption.to)) {
				animationOption.progress = 1;
			}


			var p = animationOption.get(animationOption.progress);
			var newtr = new Transformation();

			newtr.a = animationOption.from.a * (1 - p) + animationOption.to.a * p;
			newtr.b = animationOption.from.b * (1 - p) + animationOption.to.b * p;
			newtr.c = animationOption.from.c * (1 - p) + animationOption.to.c * p;
			newtr.d = animationOption.from.d * (1 - p) + animationOption.to.d * p;
			newtr.e = animationOption.from.e * (1 - p) + animationOption.to.e * p;
			newtr.f = animationOption.from.f * (1 - p) + animationOption.to.f * p;

			this._transform = newtr;


			if (animationOption.progress >= 1) {
				animationOption.done = true;
				if (this._currentAnimation == animationOption) {
					this._currentAnimation = null;
				}
				this.fireTransformChanged();
			} else {
				this.fireTransformChanged();

				this._animationTimer = setTimeout(_.bind(function () {
					animationOption.progress += animationOption.gain;
					this.updateTransformationAnimated(animationOption);
				}, this), this._animationFramerate);
			}
		}, 0);


		/**
		 * Set the transform
		 * @param {Object} options
		 * 	- rotate: add a rotation to the current rotation
		 *  - offsetX: add offset to the X axis
		 *  - offsetY: add offset to the Y axis
		 *  - flip: flip the world horizontally
		 *  - flipViewFromBack: toggle flip for view from back (without interfering with the normal flip setting)
		 *  - zoom: add a zoom to the current zoom in steps (1 is one zoomlevel, 2 is two zoomlevels, -1 is zoom out a zoomlevel)
		 *  - fixedzoom: set the zoom to a fixed zoom
		 *  - fixedoffset: set the offset to a fixed size
		 *  - fixedrotation: set the rotation to a fixed number
		 *  - middle: set the middle to a fixed middle
		 *  - animated: identify if the transformation needs to be animated
		 *  - rubber: constrain the zoom/pan with a rubber (this will give an indication if the zoom/pan goes beyond the bounds)
		 *  - constrainRotation: constrains the rotation to steps of 90 degrees (so 0,90,180,270)
		 * @param {boolean} skipUpdate - means we're in a temporary state in which we don't want to do an update
		 */
		ViewportWorld.prototype.transform = function (options, skipUpdate) {
			this._reportChange();

			if (options.rotate) {
				this._rotation = (this._rotation + options.rotate + 360) % 360;
				this._emit("rotated");
			}
			if (options.fixedrotation) {
				this._rotation = options.fixedrotation % 360;
				this._emit("rotated");
			}
			if (options.flip) {
				this._flip = !this._flip;
				this._emit("flipped");
			}

			if (options.offsetX || options.offsetY) {
				var ox = (options.offsetX ? options.offsetX : 0);
				var oy = (options.offsetY ? options.offsetY : 0);
				this._offsetSize.width += ox;
				this._offsetSize.height += oy;
			}

			if (options.flipViewFromBack !== undefined) {
				this._flipViewFromBack = options.flipViewFromBack;
				this._emit("flippedViewFromBack");
			}

			if (options.zoom) {
				if (this._worldResolution && this._worldResolution.x !== 0) {
					// we want to zoom to the levels for which a rasterized image exists
					// the zoomlevels should be in this row:
					// x = worldResolution * n
					// or
					// x = wordResolution / (2^n)
					//
					// zoomInZoomOut = -1 for zoom out, +1 or +2 for zoom in
					// worldResolution * 2 ^ (round(log2(zoom/worldResolution)) + zoomInZoomOut)
					var integerZoomLevel = Math.round(Math.log(this._zoom / this._worldResolution.x) / Math.log(2));
					this._zoom = this.getIntegerZoom(integerZoomLevel + options.zoom);
				} else {
					// just divide by two or multiply by 2
					this._zoom *= Math.pow(2, options.zoom);
				}
			}

			if (options.fixedzoom) {
				this._zoom = options.fixedzoom;
			}

			//if the viewport shouldn't animate anymore, cancel the current animation
			if (!this.isAnimationAllowed()) {
				this.cancelAnimation();
			}

			if (options.fixedoffset) {
				this._offsetSize = options.fixedoffset;
			}

			if (options.middle) {
				this.setMiddle(options.middle);
			}

			if (!skipUpdate) {
				this.updateTransformation(options.animated, options.constrainRotation, options.tmpDisableConstraints);
			}
		};

		/**
		 * Get the zoom for the given integer zoom level
		 * @return {Number}
		 */
		ViewportWorld.prototype.getIntegerZoom = function (integerZoomLevel) {
			return this._worldResolution.x * Math.pow(2, integerZoomLevel);
		};

		/**
		 * Set the initial transformation for the viewport world (used for setting initial state)
		 * @param {Object} transform
		 */
		ViewportWorld.prototype.setInitialTransformation = function (transform) {
			this._reportChange();
			this._initialTransformation = transform;
		};

		/**
		 * Get the initial transformation of the viewport world
		 * @returns {Object}
		 */
		ViewportWorld.prototype.getInitialTransformation = function () {
			return this._initialTransformation;
		};

		/**
		 * Fit in a rectangle
		 * @param {ShapeUtils.Rect} rect - the rectangle to fit into
		 * @param {Boolean} animated - if the transition needs to be animated
		 * @param {Boolean} constrain - true to constrain the zoom
		 */
		ViewportWorld.prototype.fitInRect = function (rect, animated, constrain) {
			this._reportChange();
			var fullWidthZoom = Math.abs(this._sceneSize.width / rect.size.width);
			var fullHeightZoom = Math.abs(this._sceneSize.height / rect.size.height);
			var zoom = Math.min(fullWidthZoom, fullHeightZoom);

			if (constrain) {
				zoom *= 0.8;
				zoom = Math.min(zoom, this._maxZoom);
			}

			this.transform({
				fixedzoom: zoom,
				middle: rect.center(),
				animated: animated
			});
		};

		/**
		 * Gets the offset size of the viewport world
		 * @returns {ShapeUtils.Size}
		 */
		ViewportWorld.prototype.getOffsetSize = function () {
			return this._offsetSize.copy();
		};

		/**
		 * Set the offset size of the world
		 * @param {ShapeUtils.Size} os - offset size
		 * @param {Boolean} animated - true for animation
		 */
		ViewportWorld.prototype.setOffsetSize = function (os, animated) {
			this._reportChange();
			this._offsetSize = os;
			this.updateTransformation(animated);
		};

		/**
		 * Get the viewport displayed with the current transform
		 * @param {Boolean} donotconstrain - True for not constraining
		 * @returns {ShapeUtils.Rect} the viewport rect
		 */
		ViewportWorld.prototype.getTargetViewPort = function (donotconstrain) {
			var rect = this.getWorldRectFromScene(this.getSceneBounds(), this.calculateCurrentTransformation());
			if (!donotconstrain) {
				rect = rect.calculateOverlap(this.getWorldBounds());
			}

			return rect;
		};

		/**
		 * Get the rect of the viewport for when the animation is finished.
		 *
		 * if no animation is running, just return the targetviewport
		 * @returns {ShapeUtils.Rect} the viewport rect for after the animation
		 */
		ViewportWorld.prototype.getAnimationDestinationTargetViewPort = function () {
			if (this.isAnimating()) {
				var rect = this.getWorldRectFromScene(this.getSceneBounds(), this._currentAnimation.to);
				rect = rect.calculateOverlap(this.getWorldBounds());
				return rect;
			} else {
				return this.getTargetViewPort();
			}
		};
		/**
		 * Converts the size of a scene to the world
		 * @param {ShapeUtils.Size} sceneSize - the scene size to convert to world
		 * @param {Transformation} [t] - optionally the transformation
		 * @returns {ShapeUtils.Size} the converted size in world coordinates
		 */
		ViewportWorld.prototype.getWorldSizeFromScene = function (sceneSize, t) {
			if (!t) {
				t = this.getTransformation();
			}
			var p1 = this.getWorldPointFromScene(new ShapeUtils.Point(), t);
			var p2 = this.getWorldPointFromScene(new ShapeUtils.Point(sceneSize.width, sceneSize.height), t);
			var worldSize = new ShapeUtils.Size(p2.x - p1.x, p2.y - p1.y);
			return worldSize;
		};

		/**
		 * Converts the point of a scene to the world
		 * @param {ShapeUtils.Point} scenePoint - the scene point to convert
		 * @param {Transformation} [t] - optionally the transformation
		 * @returns {ShapeUtils.Point} the converted point in world coordinates
		 */
		ViewportWorld.prototype.getWorldPointFromScene = function (scenePoint, t) {
			if (!t) {
				t = this.getTransformation();
			}
			return t.createInverse().transformedPoint(scenePoint);
		};

		/**
		 * Converts the rect of a scene to the world
		 * @param {ShapeUtils.Rect} sceneRect - the scene rect to convert to world
		 * @param {Transformation} t - optionally the transformation
		 * @returns {ShapeUtils.Rect} the converted rect in world coordinates
		 */
		ViewportWorld.prototype.getWorldRectFromScene = function (sceneRect, t) {
			if (!t) {
				t = this.getTransformation();
			}
			var worldRect = t.createInverse().transformedRect(sceneRect);
			var zoom = this.getZoom();
			//Due to floating point calculations in JavaScript, round origin points of the visible rectangle (caued alignment issues in Grid)
			if (1 - (worldRect.origin.y * zoom) < 0.000001) { //if there is a small epsilon difference (~.99999999998) add an larger epsilon value to make sure it rounds correctly after calculations
				worldRect.origin.y += (0.000001 / zoom); // add 0.000001 pixel to the origin
			}
			if (1 - (worldRect.origin.x * zoom) < 0.000001) {
				worldRect.origin.x += (0.000001 / zoom);
			}
			return worldRect;
		};

		/**
		 * Converts the rect of a world to scene coordinates
		 * @param {ShapeUtils.Rect} worldRect - the world rect to convert to scene
		 * @param {Transformation} t - optionally the transformation
		 * @returns {ShapeUtils.Rect} the converted rect in scene coordinates
		 */
		ViewportWorld.prototype.getSceneRectFromWorld = function (worldRect, t) {
			if (!t) {
				t = this.getTransformation();
			}
			return t.transformedRect(worldRect);
		};

		/**
		 * Converts the point of the world to the scene
		 * @param {ShapeUtils.Point} worldPoint - the world point to convert
		 * @param {Transformation} t - optionally the transformation
		 * @returns {ShapeUtils.Point} the converted point in scene coordinates
		 */
		ViewportWorld.prototype.getScenePointFromWorld = function (worldPoint, t) {
			if (!t) {
				t = this.getTransformation();
			}
			return t.transformedPoint(worldPoint);
		};

		/**
		 * Alert the listeners that the world changed
		 */
		ViewportWorld.prototype.fireTransformChanged = function () {
			this._emit("transformChanged", this);
		};

		/**
		 * Can animations be used?
		 * @param {Boolean} toggle - True if animations can be used, false otherwise
		 */
		ViewportWorld.prototype.toggleAnimation = function (toggle) {
			this._allowAnimation = toggle;
			if (!toggle) {
				this.cancelAnimation();
			}
		};

		/**
		 * Is flipped horizontally?
		 * @returns {Boolean} True if flipped horizontally
		 */
		ViewportWorld.prototype.isFlip = function () {
			return this._flip;
		};

		/**
		 * Check if flipviewfromback is set
		 * @returns {Boolean}
		 */
		ViewportWorld.prototype.isFlipViewFromBack = function () {
			return this._flipViewFromBack;
		};

		/**
		 * Set the world resolution (the resolution of the original document)
		 * @param {Object} newWorldResolution - the resolution of the document
		 * {x: {Number} x-resolution,
		 *  y: {Number} y-resolution }
		 */
		ViewportWorld.prototype.setWorldResolution = function (newWorldResolution) {
			this._worldResolution = newWorldResolution;
		};

		/**
		 * Get the resolution of the document
		 * @returns {Object}
		 * {x: {Number} x-resolution,
		 *  y: {Number} y-resolution }
		 */
		ViewportWorld.prototype.getWorldResolution = function () {
			return this._worldResolution;
		};

		/**
		 * Is the zoom level higher than 1-16
		 * @returns {Boolean} True if higher than 1-16 zoom
		 */
		ViewportWorld.prototype.isDeepZoom = function () {
			return (this._zoom / this._worldResolution.x) >= 16 || (this._zoom / this._worldResolution.y) >= 16;
		};

		/**
		 * Internal method for marking the world as changed since the last zoom to fit
		 */
		ViewportWorld.prototype._reportChange = function () {
			this._didChangeSinceLastZoomToFit = true;
		};
		/**
		 * Is the transform of the world changed since the last zoom to fit operation?
		 * @returns {Boolean}
		 */
		ViewportWorld.prototype.didChangeTransformSinceLastZoomToFit = function () {
			return this._didChangeSinceLastZoomToFit;
		};


		/**
		 * Cancel the currently ongoing animation
		 * @param {Boolean} [cancelResult] - if true, the destination transformation of the animation will be ignored, by default, the destation transformation of the animation is set as transform of the viewport
		 */
		ViewportWorld.prototype.cancelAnimation = function (cancelResult) {
			if (this._animationTimer && this._currentAnimation) {
				clearTimeout(this._animationTimer);

				if (!this._currentAnimation.done && !cancelResult) {
					this._transform = this._currentAnimation.to;
					this.fireTransformChanged();
				}

				this._currentAnimation.done = true;
				this._currentAnimation = null;
			}
		};

		/**
		 * Is animation of the viewport allowed?
		 * @returns {Boolean}
		 */
		ViewportWorld.prototype.isAnimationAllowed = function () {
			return this._allowAnimation && this.getZoomPercentage() < 1000;
		};

		/**
		 * Reset the current orientation options of the world (flipped, rotated and view from back)
		 */
		ViewportWorld.prototype.resetOrientation = function () {
			this._flip = false;
			this._emit("flipped");
			this._rotation = 0;
			this._emit("rotated");
			this._flipViewFromBack = false;
			this._emit("flippedViewFromBack");
		};


		return ViewportWorld;
	});

define('web-viewer/viewport/tools/AbstractTool',["underscore",
		"utils/ShapeUtils",
		"web-viewer/core/Extendable",
		"web-viewer/core/PrivateChannel",
		"Mousetrap"
	],
	function (_,
		ShapeUtils,
		Extendable,
		PrivateChannel,
		Mousetrap) {
		"use strict";

		/**
		 * @exports AbstractTool
		 * @memberof ViewportModule
		 * @class An abstract tool can interact with the user on a widget (presumably the viewport widget)
		 *
		 * Tools are managed by the toolmanager, and can be passivated. If a tool is passivated, it can listen to events and request activation based on these events
		 *
		 * @param f {Core.ViewerFacade}
		 * @param IID {String} internal unique ID
		 */
		var AbstractTool = function AbstractTool(f, IID) {
			Extendable.installTo(this);
			PrivateChannel.installTo(this, f, ['tool', IID]);

			this.f = f;

			var active = false;

			/**
			 * @returns true if the tool is active
			 */
			this.isActive = function () {
				return active;
			};

			/**
			 * @returns the unique internal id of the tool
			 */
			this.getIID = function () {
				return IID;
			};


			/**
			 * Ask the toolmanager to be activated
			 */
			this.requestActivation = function () {
				f.emit("toolManager.request.activation", this);
			};

			/**
			 * Ask the toolmanager to be deactivated
			 */
			this.requestDeactivation = function () {
				f.emit("toolManager.request.deactivation", this);
			};

			/**
			 * It can be very straightforward to define the needed eventlisteners in these two objects,
			 * The events are registered and unregistered at the correct moments
			 */

			/**
			 * events needed when passivated
			 */
			this.passiveEvents = {};
			// structure: per key combo array with two elements ["combo", function(){}] or [[combo1, combo2], function(){}]
			this.passiveKeyEvents = [];

			/**
			 * events needed when activated;
			 */
			this.activeEvents = {};
			this.activeKeyEvents = [];

			// these keypress are always active (both in passive and active mode)
			this.persistentKeyEvents = [];

			/**
			 * This method is called when the tool is activated
			 * @param {Boolean} interruptRequest - set to true if the activation is requested by an interrupt event
			 */
			this.activate = function (interruptRequest) {
				active = true;

				//only activate the tool when the widget is actived
				this._widget.whenContextAvailable(_.bind(function () {
					this.doActivation();
					if (this.isActive()) {
						this._emit("activated", interruptRequest);
					}
				}, this));
			};


			/**
			 * Deactivate the tool
			 *
			 * !!should only be called by the toolmanager
			 * @param {Boolean} interruptRequest - set to true if the activation is requested by an interrupt event
			 */
			this.deactivate = function (interruptRequest) {
				this.doDeactivation();
				active = false;
				this._emit("deactivated", interruptRequest);
			};

			/**
			 * State management
			 *
			 * A state is a way of handling events. At any time a tool can change state, which will be in charge of handling the events
			 */
			var states = {}; // list of registered states (by calling registerState)
			var currentState; //name of the current state

			/**
			 * Register all possible active events for this tool
			 */
			this.registerActiveEvents = function (events) {
				_.each(events, function (event) {
					this.activeEvents[event] = _.bind(function (e) {
						this.getState(currentState)[event](e);
					}, this);
				}, this);
			};

			/**
			 * Register all possible active events for this tool
			 */
			this.registerPassiveEvents = function (events) {
				_.each(events, function (event) {
					this.passiveEvents[event] = _.bind(function (e) {
						this.getState(currentState)[event](e);
					}, this);
				}, this);
			};

			/**
			 * Register a state with a name and a function
			 */
			this.registerState = function (stateName, stateFunction) {
				states[stateName] = stateFunction;
			};

			/**
			 * Change the state of the tool to a new one
			 */
			this.changeState = function (stateName) {
				if (currentState !== stateName) {
					var previousState = currentState;
					var nextState = stateName;

					// execute leave state if set
					if (states[previousState] && states[previousState].leaveState) {
						states[previousState].leaveState(nextState);
					}
					currentState = nextState;
					this._emit("State changed to \"" + nextState + "\"");
					// execute enter state if set
					if (states[nextState] && states[nextState].enterState) {
						states[nextState].enterState(previousState);
					}
					this.cursorShouldChange();
				}
			};

			/**
			 * Get the function of a state-name
			 */
			this.getState = function (stateName) {
				return states[stateName];
			};

			/**
			 * Get the current state
			 */
			this.getCurrentState = function () {
				return this.getState(currentState);
			};

			/**
			 * Get the current statename
			 */
			this.getCurrentStateName = function () {
				return currentState;
			};

		};

		/**
		 * Convert a point in the widget (scene) to a point in the world
		 */
		AbstractTool.prototype.sceneToWorld = function (point) {
			if (this._widget) {
				return this._widget.getWorld().getWorldPointFromScene(point);
			} else {
				this.f.emit("error.SEVERE", "Requesting world point with no widget present");
				return point;
			}
		};

		/**
		 * Convert a rectangle from scene to world
		 * @param {ShapeUtils.Rect} rect
		 * @returns
		 */
		AbstractTool.prototype.sceneToWorldRect = function (rect) {
			if (this._widget) {
				return this._widget.getWorld().getWorldRectFromScene(rect);
			} else {
				this.f.emit("error.SEVERE", "Requesting world rectangle with no widget present");
				return null;
			}
		};

		/**
		 * Convert a size from scene to world
		 * @param {ShapeUtils.Size} size
		 * @returns
		 */
		AbstractTool.prototype.sceneToWorldSize = function (size) {
			if (this._widget) {
				return this._widget.getWorld().getWorldSizeFromScene(size);
			} else {
				this.f.emit("error.SEVERE", "Requesting world rectangle with no widget present");
				return null;
			}
		};

		/**
		 * Get the world resolution
		 * @returns Null or {Object}
		 * {x: {Number} x-resolution,
		 *  y: {Number} y-resolution }
		 */
		AbstractTool.prototype.getWorldResolution = function () {
			if (this._widget) {
				return this._widget.getWorld().getWorldResolution();
			} else {
				this.f.emit("error.SEVERE", "Requesting world resolution with no widget present");
				return null;
			}
		};

		/**
		 * Converts a point to the closest point inside the world
		 *
		 * @param {ShapeUtils.Point} point - expects a worldpoint
		 * @param {Boolean} returnScenePoint - is a worldPoint or a scenePoint returned
		 * @returns {ShapeUtils.Point} a scenepoint/worldpoint
		 */
		AbstractTool.prototype.keepPointWithinWorldBounds = function (point, returnScenePoint) {
			var worldRect = this._widget.getWorld().getWorldBounds();
			return this.keepPointWithinRect(point, worldRect, returnScenePoint);
		};

		/**
		 * Converts a point to the closest point inside a rect (in world coordinates)
		 *
		 * @param {ShapeUtils.Point} point - expects a worldpoint
		 * @param {ShapeUtils.Rect} rect - the rectangle to keep the point in
		 * @param {Boolean} returnScenePoint - is a worldPoint or a scenePoint returned
		 * @returns {ShapeUtils.Point} a scenepoint/worldpoint
		 */
		AbstractTool.prototype.keepPointWithinRect = function (point, rect, returnScenePoint) {
			var newPointX = 0;
			//check if x is outside of bounds
			if (point.x < rect.left() || point.x > rect.right()) {
				//if outside check is falling left or right world bounds
				newPointX = point.x < rect.left() ? rect.left() : rect.right();
			} else {
				//inside bounds
				newPointX = point.x;
			}
			var newPointY = 0;
			if (point.y < rect.top() || point.y > rect.bottom()) {
				//if outside check is falling above or beneath world bounds
				newPointY = point.y < rect.top() ? rect.top() : rect.bottom();
			} else {
				//inside bounds
				newPointY = point.y;
			}
			if (returnScenePoint) {
				//convert worldpoint to scenepoint
				return this._widget.getWorld().getScenePointFromWorld(new ShapeUtils.Point(newPointX, newPointY));
			} else {
				return new ShapeUtils.Point(newPointX, newPointY);
			}
		};

		/**
		 * Keep a move size within the bounds of the rectangle
		 * @param   {ShapeUtils.Rect} boudingRect - rectangle the move will be bounded to
		 * @param   {ShapeUtils.Rect} movingRect - the rectangle that will be moved over a given size
		 * @param   {ShapeUtils.Size}   movesize - the size the rectangle will be moved
		 * @returns {ShapeUtils.Size}
		 */
		AbstractTool.prototype.keepMoveSizeWithinRect = function (boundingRect, movingRect, movesize) {
			var left = movingRect.left() + movesize.width;
			var right = movingRect.right() + movesize.width;
			var top = movingRect.top() + movesize.height;
			var bottom = movingRect.bottom() + movesize.height;

			var newMoveSize = new ShapeUtils.Size();
			if (Math.min(left, right) >= boundingRect.left() && Math.max(left, right) <= boundingRect.right() && Math.min(top, bottom) >= boundingRect.top() && Math.max(top, bottom) <= boundingRect.bottom()) {
				newMoveSize.width = movesize.width;
				newMoveSize.height = movesize.height;
			}
			return newMoveSize;
		};

		/**
		 * When the tool is active, and another tool requests activation, the ToolManager will first check if the current active tool can be interrupted
		 *
		 * If no interruption is allowed, return false
		 * @param {AbstractTool} tool - The tool that wants to interrupt the current tool
		 * @returns {Boolean}
		 */
		AbstractTool.prototype.canBeInterrupted = function (tool) {
			return true;
		};

		/**
		 * A tool should always allow activation, but sometimes it has some conditions to be activated
		 * 	e.g.:
		 * 		- the annotation draw tool can only be activated if it follows the annotationmode logic
		 * 		- licensing
		 *
		 * The tool can't be activated, return false
		 * @returns {Boolean}
		 */
		AbstractTool.prototype.canBeActivated = function () {
			return true;
		};


		/**
		 * Change the cursor
		 * @param cursor
		 */
		AbstractTool.prototype.setCursor = function (cursor) {
			if (this._widget) {
				this._widget.setCursor(cursor);
			}
		};


		/**
		 * Request the cursor to change
		 */

		AbstractTool.prototype.cursorShouldChange = function () {
			if (this.isActive()) {
				this.setCursor(this.getDefaultCursor());
			}
		};

		/**
		 * Returns the default cursor for the tool
		 * @returns {String}
		 */
		AbstractTool.prototype.getDefaultCursor = function () {
			return "cursor-default";
		};

		/**
		 * When the tool is initialized, the passivate function is executed
		 * 

		 */
		AbstractTool.prototype.passivate = function () {
			this._widget.whenContextAvailable(_.bind(function () {
				this.doPassivation();
				this._emit("passivated");
			}, this));
		};

		/**
		 * In this function the tool can register itself
		 * (preferably on the context of the widget)
		 *
		 * if you use this.passiveEvents there's no need to override this method
		 */
		AbstractTool.prototype.doPassivation = function () {
			this._context = this._widget.context;
			this._context.on(this.passiveEvents);
			this.bindKeys(this.persistentKeyEvents);
			this.bindKeys(this.passiveKeyEvents);

		};


		/**
		 * When the tool is depassivated, it should no longer listen to events and unregister itself
		 *
		 * if you use this.passiveEvents there's no need to override this method
		 */
		AbstractTool.prototype.depassivate = function () {
			if (this._context) {
				this._context.off(this.passiveEvents);
			}
			this.unbindKeys(this.passiveKeyEvents);

			this._emit("depassivated");
		};


		/**
		 * Register for the needed events to be activated
		 */
		AbstractTool.prototype.doActivation = function () {
			this._context = this._widget.context;

			this._context.off(this.passiveEvents);
			this._context.on(this.activeEvents);
			this.unbindKeys(this.passiveKeyEvents);
			this.bindKeys(this.activeKeyEvents);

		};

		/**
		 * Unregister all active events, and possible start passive events
		 */
		AbstractTool.prototype.doDeactivation = function () {
			if (this._context) {
				this._context.off(this.activeEvents);
				this._context.on(this.passiveEvents);
			}
			this.unbindKeys(this.activeKeyEvents);
			this.bindKeys(this.passiveKeyEvents);
		};

		/**
		 * Request the current view based on the mouseposition
		 * @param event
		 * @returns
		 */
		AbstractTool.prototype.requestView = function (event) {
			return this._widget.requestView(event);
		};

		/**
		 * Request a canvas layer to draw on
		 * @param event
		 */
		AbstractTool.prototype.requestCanvasLayer = function (event) {
			this._viewForActiveCanvasLayer = this.requestView(event);
			if (this._viewForActiveCanvasLayer) {
				this._layer = this._viewForActiveCanvasLayer.addLayer(this.getIID());
			}
		};

		/**
		 * release the requested canvaslayer
		 */
		AbstractTool.prototype.releaseCanvasLayer = function () {
			if (this._layer) {
				this._viewForActiveCanvasLayer.removeLayer(this.getIID());
				this._layer = undefined;
			}
		};

		/**
		 * Returns the drawing context of the canvas for the tool
		 * @returns
		 */
		AbstractTool.prototype.getCanvasCtx = function () {
			if (this._layer) {
				return this._layer.getCanvasContext();
			}
		};

		/**
		 * Get the size of the Canvas
		 * @returns {ShapeUtils.Size}
		 */
		AbstractTool.prototype.getCanvasSize = function () {
			if (this._layer) {
				return new ShapeUtils.Size(this._layer.getCanvas()[0].width, this._layer.getCanvas()[0].height);
			}
		};

		/**
		 * Get the scenepoint given a (mouse) event. This method takes the icon offset into account as returned by getIconOffset (by default nothing)
		 * @param {Object} e - Event object
		 * @returns {ShapeUtils.Point}
		 */
		AbstractTool.prototype.getScenePoint = function (e) {
			var iconOffset = this.getIconOffset();

			var mousePoint;
			if (e.pageX !== undefined) {
				if (iconOffset) {
					mousePoint = new ShapeUtils.Point(e.pageX + iconOffset.width, e.pageY + iconOffset.height);
				} else {
					mousePoint = new ShapeUtils.Point(e.pageX, e.pageY);
				}
			} else if (e.clientX !== undefined) {
				if (iconOffset) {
					mousePoint = new ShapeUtils.Point(e.clientX + iconOffset.width, e.clientY + iconOffset.height);
				} else {
					mousePoint = new ShapeUtils.Point(e.clientX, e.clientY);
				}
			} else {
				mousePoint = new ShapeUtils.Point();
			}

			var view = this._widget.requestView(mousePoint);
			if (view) {
				var offset = view.getViewContext().offset();
				mousePoint.offsetXY(-offset.left, -offset.top);
				return mousePoint;
			} else {
				return new ShapeUtils.Point();
			}
		};

		/**
		 * Get the icon offset of the tool
		 * @returns {ShapeUtils.Size} Icon offset of the tool
		 */
		AbstractTool.prototype.getIconOffset = function () {
			// To be implemented
			return null;
		};

		/**
		 * Get the worldpoint given a (mouse) event
		 * @param e
		 * @returns {ShapeUtils.Point}
		 */
		AbstractTool.prototype.getWorldPoint = function (e) {
			var point = this.sceneToWorld(this.getScenePoint(e));

			return point;
		};

		/**
		 * Bind keys using MouseTrap
		 * @param {Array} array - array of arrays [keycombo, function, keyup|keydown|keypress]
		 */
		AbstractTool.prototype.bindKeys = function (array) {
			for (var i = 0; i < array.length; i++) {
				Mousetrap.bind(array[i][0], array[i][1], array[i][2]);
			}
		};

		/**
		 * Unbind keys
		 * @param array
		 */
		AbstractTool.prototype.unbindKeys = function (array) {
			for (var i = 0; i < array.length; i++) {
				Mousetrap.unbind(array[i][0], array[i][1], array[i][2]);
			}
		};

		/**
		 * Get the panel at a certain (mouse) event)
		 * @param event
		 * @returns {ViewportPanel}
		 */
		AbstractTool.prototype.getPanelAt = function (event) {
			var point;

			if (event.pageX) {
				point = new ShapeUtils.Point(event.pageX, event.pageY);
			} else {
				point = new ShapeUtils.Point(event.clientX, event.clientY);
			}
			return this._widget.getPanelAt(point);
		};



		/**
		 * Initialize the tool for the given widget
		 * @param widget {Layout.AbstractWidget} the widget on which the tool is activated
		 * @param toolManager {ToolManager} the tool manager on which the tool is added
		 */
		AbstractTool.prototype.initialize = function (widget, toolManager) {
			this._widget = widget;
			this._toolManager = toolManager;
		};


		return AbstractTool;
	});

define('web-viewer/viewport/tools/InterruptTool',["web-viewer/viewport/tools/AbstractTool",
		"web-viewer/core/CoreConstants"
	],
	function (AbstractTool,
		CoreConstants) {
		/**
		 * @exports InterruptTool
		 * @memberof ViewportModule
		 * @class 
		 * This tool does nothing but is used to block all other tools in the tool manager.
		 * It can be temporary activated and again deactivated by the toolManager.interruptTools event.
		 * @constructor
		 * @param {Core.ViewerFacade} f - Viewer facade
		 */
		var InterruptTool = function (f) {
			AbstractTool.call(this, f, CoreConstants.TOOLS.INTERRUPT);
		};

		InterruptTool.prototype = Object.create(AbstractTool.prototype);
		InterruptTool.prototype.constructor = InterruptTool;

		/**
		 * @override
		 * Returns false because this tool can only be controlled (activation/deactivation) by the toolManager.interruptTools event.
		 * @returns {Boolean} false
		 */
		InterruptTool.prototype.canBeInterrupted = function () {
			return false;
		};

		return InterruptTool;
	});

define('web-viewer/viewport/tools/ToolManager',["web-viewer/core/PrivateChannel",
		"web-viewer/viewport/tools/InterruptTool",
		"web-viewer/core/CoreConstants",
		"web-viewer/layout/LayoutConstants",
		"underscore"
	],
	function (PrivateChannel,
		InterruptTool,
		CoreConstants,
		LayoutConstants,
		_) {

		"use strict";

		/**
		 * @exports ToolManager
		 * @memberof ViewportModule
		 * @class The tool manager is responsible to manage the tools used on the viewport
		 * It keeps track of the current tools and has a stack of previous tools (in case a tool deactivates itself, the previous one can be restored)
		 * Communication is done via events.
		 * @example
		 * It reacts on the following events:
		 * 	"toolManager.addTool": Add a tool to the tool manager.
		 * 		,{AbstractTool} tool - Tool to add
		 *
		 *  "toolManager.request.activation": Request to activate the given tool. This request can be denied if the active tool is in a "Do not interrupt"-state.
		 *   	,{String | AbstractTool} tool - Tool name or tool to activate
		 *
		 *  "toolManager.request.deactivation" - Request to deactivate the given tool or the current active tool if no tool specified.
		 *   	,{String | AbstractTool} [tool] - Tool name or tool to deactivate.
		 *
		 *  "toolManager.activateTool": Activate the given tool without asking permission to the active tool
		 *   	,{String | AbstractTool} tool - Tool name or tool to activate
		 *
		 *  "toolManager.deactivateTool": Deactivate the given tool or the current active tool if no tool specified.
		 *   	,{String | AbstractTool} [tool] - Tool name or tool to deactivate.
		 *
		 *  "toolManager.activeTool": Get the current active tool.
		 *   	,callback {AbstractTool} The current active tool or undefined if no tool is active.
		 *
		 *  "toolManager.requestTool": Request the tool with the given name.
		 *   	,{String} tool - Tool name.
		 *   	,callback {AbstractTool} - The tool with the given name or undefined if not found.
		 *
		 *  "toolManager.interruptTools":  Interrupt active tools.
		 * 		,{Boolean} interrupt - True if you want to interrupt the active tools, false if you want to restore them.
		 *
		 * @param {Core.ViewerFacade} f - Viewer facade
		 * @param {Layout.AbstractWidget} widget - The widget where the tools of this tool manager are activated on.
		 */
		var ToolManager = function (f, widget) {
			PrivateChannel.installTo(this, f, "toolManager");

			this.f = f;

			this._widget = widget;

			this._defaultTool = (this.f.getLayoutType() === LayoutConstants.LayoutType.MOBILE) ? CoreConstants.TOOLS.TOUCH : CoreConstants.TOOLS.ZOOM;

			this._loadedTools = {};
			this._toolStack = [];

			this._allowedTools = null;
			this._disallowedTools = null;

			/**
			 * Create the interrupt tool and add it to the tool manager
			 */
			this._interruptTool = new InterruptTool(f);
			this._addTool(this._interruptTool);

			/**
			 * Add a tool to the tool manager.
			 * @param {AbstractTool} tool - Tool to add
			 */
			this.on("addTool", _.bind(function (tool) {
				this._addTool(tool);
			}, this));

			/**
			 * Remove a tool from the tool manager
			 * @param {String | AbstractTool} tool - IID of the tool
			 */
			this.on("removeTool", _.bind(function (tool) {
				this._removeTool(tool);
			}, this));

			/**
			 * Request to activate the given tool. This request can be denied if the active tool is in a "Do not interrupt"-state.
			 * @param {String | AbstractTool} tool - Tool IID or tool to activate
			 */
			this.on("request.activation", _.bind(function (tool) {
				this._requestActivation(tool);
			}, this));

			/**
			 * Request to deactivate the given tool or the current active tool if no tool specified.
			 * @param {String | AbstractTool} [tool] - Tool IID or tool to deactivate.
			 */
			this.on("request.deactivation", _.bind(function (tool) {
				this._deactivateToolImpl(tool, false, true, false);
			}, this));

			/**
			 * Deactivate the given tool or the current active tool if no tool specified.
			 * @param {String | AbstractTool} [tool] - Tool IID or tool to deactivate.
			 */
			this.on("deactivateTool", _.bind(function (tool) {
				this._deactivateToolImpl(tool, false, true);
			}, this));

			/**
			 * Get the current active tool
			 * @param {Function} callback - Callback executed with parameter {AbstractTool} tool - The current active tool or undefined if no tool is active.
			 */
			this.on("activeTool", _.bind(function (callback) {
				callback(this.getActiveTool());
			}, this));

			/**
			 * Activate the given tool without asking permission to the active tool
			 * @param {String | AbstractTool} tool - Tool IID or tool to activate
			 * @param {Boolean} [keepHistory] - If true, the current active tool stays on the tool stack, otherwise it's popped (default)
			 */
			this.on("activateTool", _.bind(function (tool, keepHistory) {
				if (keepHistory === undefined) {
					keepHistory = false;
				}

				this._activateToolImpl(tool, keepHistory, false);
			}, this));

			/**
			 * Request the tool with the given IID.
			 * @param {String} tool - Tool IID.
			 * @param {Function} callback - Callback executed with parameter {AbstractTool} tool - The tool with the given IID or undefined if not found.
			 */
			this.on("requestTool", _.bind(function (tool, callback) {
				callback(this._loadedTools[tool]);
			}, this));

			/**
			 * Interrupt active tools.
			 * @param {Boolean} interrupt - True if you want to interrupt the active tools, false if you want to restore them.
			 */
			this.on("interruptTools", _.bind(function (interrupt) {
				if (interrupt) {
					this._activateInterruptTool();
				} else {
					this._deactivateInterruptTool();
				}
			}, this));

			this.on("request.allowAllTools", _.bind(function () {
				this.allowAllToolsExcept(null);
			}, this));

			this.on("request.allowAllToolsExcept", _.bind(function (disallowedTools) {
				this.allowAllToolsExcept(disallowedTools);
			}, this));

			this.on("request.disallowAllToolsExcept", _.bind(function (allowedTools) {
				this.disallowAllToolsExcept(allowedTools);
			}, this));
		};

		/**
		 * Add a tool to the toolmanager. The tool will be "passivated".
		 * @param {AbstractTool} tool - Tool to add.
		 */
		ToolManager.prototype._addTool = function (tool) {
			if (tool) {
				this._loadedTools[tool.getIID()] = tool;
				tool.initialize(this._widget, this);

				tool.passivate();

				// if the tool is the default tool, activate it
				if (tool.getIID() == this._defaultTool && this._isToolAllowed(this._defaultTool)) {
					this._activateTool(tool);
				}
			}
		};

		/**
		 * Remove a tool from the tool manager
		 * @param {String|AbstractTool} tool - IID of the tool
		 */
		ToolManager.prototype._removeTool = function (tool) {
			if (_.isString(tool)) {
				tool = this._loadedTools[tool];

				if (!tool) {
					this.f.emit("error.WARNING", "Trying to deactivate a tool which isn't loaded");
					return;
				}
			}

			if (tool.isActive()) {
				this._deactivateToolImpl(tool, false, true, false);
			}

			this._toolStack = _.without(this._toolStack, tool);

			tool.depassivate();
			delete this._loadedTools[tool.getIID()];
		};

		/**
		 * Activate the given tool. It will not keep the previous tool on the stack.
		 * @param {String|AbstractTool} tool - Tool IID or tool to activate.
		 */
		ToolManager.prototype._activateTool = function (tool) {
			this._activateToolImpl(tool, false, false);
		};

		/**
		 * Activate a tool
		 * @param {String | AbstractTool} tool - Tool IID or tool to activate
		 * @param {Boolean} keepHistory - If true, the current active tool stays on the tool stack, otherwise it's popped (default)
		 * @param {Boolean} skipDeactivation - If true, the deactivation of the previous tool is skipped.
		 * @param {Boolean} interruptRequest - If true, the activation is requested by an interrupt event
		 */
		ToolManager.prototype._activateToolImpl = function (tool, keepHistory, skipDeactivation, interruptRequest) {
			if (!this._isToolAllowed(tool)) {
				this._emit("request.denied", tool);
				return;
			}

			if (_.isString(tool)) {
				tool = this._loadedTools[tool];

				if (!tool) {
					this.f.emit("error.WARNING", "Trying to activate a tool which isn't loaded");
					return;
				}
			}

			if (tool) {
				if (!tool.canBeActivated()) {
					this.f.debug.warn("Trying to activate tool which refuses to activate", tool);
					return;
				}

				var currentActive = this.getActiveTool();
				if (currentActive && !skipDeactivation) {
					if (currentActive == tool) {
						return; // the tool is already active!
					}

					// deactivate the current activated tool
					this._deactivateToolImpl(undefined, keepHistory, false, interruptRequest);
				}

				// make sure the last tool was not the same tool...
				if (_.last(this._toolStack) != tool) {
					this._toolStack.push(tool);
				}

				tool.activate(interruptRequest);
				
				if (this.getActiveTool() === tool){
					this._widget.setCursor(tool.getDefaultCursor());
					this.f.emit("toolManager.activated", tool);
				}
			} else {
				this.f.emit("error.SEVERE", "Can't activate tool.", tool);
			}

		};

		/**
		 * Deactivate a tool
		 * @param {String|AbstractTool} tool - Tool IID or tool to deactivate.
		 * @param {Boolean} keepHistory : If true, the current active tools stays on the stack (note do not call deactiveTool with stack = true from outside!)
		 * @param {Boolean} restorePreviousTool - If true, the previous tools needs to be re-activated
		 * @param {Boolean} interruptRequest - If true, the activation is requested by an interrupt event
		 */
		ToolManager.prototype._deactivateToolImpl = function (tool, keepHistory, restorePreviousTool, interruptRequest) {
			if (_.isString(tool)) {
				tool = this._loadedTools[tool];

				if (!tool) {
					this.f.emit("error.WARNING", "Trying to deactivate a tool which isn't loaded");
					return;
				}
			}

			if (!tool) {
				tool = this.getActiveTool();
			} else if (tool != this.getActiveTool()) {
				this.f.emit("error.WARNING", "Trying to deactive a tool which is not the current active tool", tool);
				return;
			}

			if (tool) {
				tool.deactivate(interruptRequest);

				if (!keepHistory) {
					this._toolStack.pop();
				}

				this.f.emit("toolManager.deactivated", tool);

				if (restorePreviousTool) {
					var toolToRestore = _.last(this._toolStack);

					while (toolToRestore && !this._isToolAllowed(toolToRestore)) {
						this._toolStack.pop();
						toolToRestore = _.last(this._toolStack);
					}
					// make sure we remove the tool from the stack before activating it again
					this._toolStack.pop();

					if (!toolToRestore) {
						// we are at the end of the toolstack, try the default tool
						if (this._isToolAllowed(this._defaultTool)) {
							toolToRestore = this._loadedTools[this._defaultTool];
						}
					}

					if (toolToRestore) {
						// activate the tool
						this._activateToolImpl(toolToRestore, false, true);
					}
				}
			}
		};

		/**
		 * Activate interrupt tool
		 */
		ToolManager.prototype._activateInterruptTool = function () {
			this._activateToolImpl(this._interruptTool, true, false, true);
		};

		/**
		 * Deactivate interrupt tool
		 */
		ToolManager.prototype._deactivateInterruptTool = function () {
			this._deactivateToolImpl(this._interruptTool, false, true, false);
		};

		/**
		 * Get the current active tool.
		 * @returns {AbstractTool} The current active tool or undefined if no tool is active.
		 */
		ToolManager.prototype.getActiveTool = function () {
			return _.last(this._toolStack);
		};

		/**
		 * Request to activate the given tool. This request can be denied if the active tool is in a "Do not interrupt"-state.
		 * @param {String | AbstractTool} tool - Tool IID or tool to activate
		 */
		ToolManager.prototype._requestActivation = function (tool) {
			// check if tool can be activated
			if (!this._isToolAllowed(tool)) {
				// the tool is not allowed
				this._emit("request.denied", tool);
				return;
			}
			//when we are requesting activation, first check if the current active tool can be interrupted
			var activeTool = this.getActiveTool();
			if (activeTool && activeTool != tool) {
				if (!activeTool.canBeInterrupted(tool)) {
					this._emit("request.denied", tool);
					return;
				}
			}
			this._activateToolImpl(tool, true, false, true);
		};

		/**
		 * Check if a tool is allowed to be activated
		 * @param {String | AbstractTool} tool - Tool IID or tool to activate
		 * @returns {Boolean}
		 */
		ToolManager.prototype._isToolAllowed = function (tool) {
			if (!_.isString(tool)) {
				tool = tool.getIID();
			}

			// interrupt tool is always allowed
			if (tool === CoreConstants.TOOLS.INTERRUPT) {
				return true;
			}

			// by default allow the tool
			var allowed = true;

			if (this._allowedTools) {
				// only these tools are allowed
				allowed = _.contains(this._allowedTools, tool);
			}

			if (this._disallowedTools) {
				// all these tools are disallowed
				if (_.contains(this._disallowedTools, tool)) {
					// the tool is disallowed
					allowed = false;
				}
			}

			return allowed;
		};

		/**
		 * Allow all tools, but disallow a specific list of tools
		 * @param {Array.<String>} disallowedTools - the list of toolnames that should be disallowedZ
		 */
		ToolManager.prototype.allowAllToolsExcept = function (disallowedTools) {
			this._allowedTools = null;
			this._disallowedTools = disallowedTools;

			this._emit("allowedAllToolsExcept", disallowedTools);
			this._enforceToolAllowance();
		};

		/**
		 *
		 * @param allowedTools
		 */
		ToolManager.prototype.disallowAllToolsExcept = function (allowedTools) {
			this._allowedTools = allowedTools;
			this._disallowedTools = null;
			this._emit("disallowedAllToolsExcept", allowedTools);
			this._enforceToolAllowance();
		};


		/**
		 * Internal function to run over all tools and all
		 */
		ToolManager.prototype._enforceToolAllowance = function () {
			// run over all loaded tools and enforce the allow/disallow
			var activeTool = this.getActiveTool();
			if (activeTool && !this._isToolAllowed(activeTool)) {
				this._deactivateToolImpl(activeTool, false, true, false);
			}

			// restore default tool if no tool is active
			if (!activeTool && this._isToolAllowed(this._defaultTool)) {
				this._activateTool(this._defaultTool);
			}
		};

		return ToolManager;
	});

/**
 * @memberof ViewportModule
 */
define('web-viewer/viewport/ViewportConstants',{
	ViewsLayoutMode: {
		AUTOMATIC: 0,
		HORIZONTAL: 1,
		VERTICAL: 2
	},
	PanelsLayoutMode: {
		AUTOMATIC: 0,
		VERTICAL: 1,
		HORIZONTAL: 2
	},
	PanelBorderWidth: 2
});

define('web-viewer/utils/DrawUtils',["underscore",
		"utils/ShapeUtils",
		"web-viewer/utils/ViewerUtils",
		"utils/Cache",
		"utils/DashUtils"
	],
	function (_,
		ShapeUtils,
		ViewerUtils,
		Cache,
		DashUtils) {
		"use strict";

		var debugOverlay = false;

		/**
		 * Image cache used for caching loaded image information
		 */
		var imageCache = new Cache(30, false);
		imageCache._loadItem = function (name) {
			var img = new Image();
			img.onload = _.bind(function () {
				this._registerItem(name, img);
			}, this);
			img.src = name;
		};

		/**
		 * Tinted icon cache
		 *
		 * name --> url|r|g|b
		 *
		 * loads image with imageCache, and then makes a tinted version of it
		 */
		var iconCache = new Cache(30, false);
		iconCache._loadItem = function (name) {
			var icon = name.split("|");
			var color = {
				r: icon[1],
				g: icon[2],
				b: icon[3]
			};

			var generateIcon = _.bind(function (img, color) {
				var result = new Image();

				result.onload = _.bind(function () {
					this._registerItem(name, result);
				}, this);

				generateTintImage(img, color.r, color.g, color.b, result);

			}, this);

			// get image from cache
			var image = imageCache.get(icon[0], function (img) {
				generateIcon(img, color);
			});

			if (image) {
				// the image was in the cache, generate icon
				generateIcon(image, color);
			}
		};


		/**
		 * Make a tinted version of an image
		 *
		 * @param {Image} img - the source image
		 * @param {Number} red - red tint
		 * @param {Number} green - green tint
		 * @param {Number} blue - blue tint
		 * @param {Image} [resultImage] - result image, if provided, this image will be used to store the tinted image in (useful to bind callbacks if needed)
		 */
		function generateTintImage(img, red, green, blue, resultImage) {
			var w = img.width;
			var h = img.height;

			var canvas = document.createElement("canvas");
			canvas.width = w;
			canvas.height = h;

			var ctx = canvas.getContext("2d");
			ctx.drawImage(img, 0, 0, w, h);

			var pixels = ctx.getImageData(0, 0, w, h);

			for (var i = 0; i < pixels.data.length; i += 4) {
				pixels.data[i] = pixels.data[i] * (red / 255); //R
				pixels.data[i + 1] = pixels.data[i + 1] * (green / 255); //G
				pixels.data[i + 2] = pixels.data[i + 2] * (blue / 255); //B
				// we don't need to do anything for alpha
			}

			ctx.putImageData(pixels, 0, 0);

			if (resultImage === undefined) {
				resultImage = new Image();
			}

			resultImage.src = canvas.toDataURL();

			return resultImage;
		}

		/**
		 * @memberof Utils
		 * @exports DrawUtils
		 */
		var DrawUtils = {
			LWTYPE_RECT: -1,
			LWTYPE_ELLIPSE: -2,
			LWTYPE_LINEARRAY: -3,
			LWTYPE_LINE: -4,
			LWTYPE_ARROW: -5,
			LWTYPE_IMAGE: -7,
			LWTYPE_TEXT: -21,
			LWTYPE_DASHEDBOX: -100,
			LWTYPE_DASHEDLINE: -101,
			LWANCHOR_TOPLEFT: 1,
			LWANCHOR_TOPRIGHT: 2,
			LWANCHOR_TOPCENTER: 3,
			LWANCHOR_BOTTOMLEFT: 4,
			LWANCHOR_BOTTOMRIGHT: 5,
			LWANCHOR_BOTTOMCENTER: 6,
			LWANCHOR_CENTERLEFT: 7,
			LWANCHOR_CENTERRIGHT: 8,
			LWANCHOR_CENTERCENTER: 9,
			LWANCHOR_CENTERFREE: 10,
			LWANCHOR_HORIZONTAL: 11,
			LWANCHOR_VERTICAL: 12,
			LWANCHOR_START: 13,
			LWANCHOR_END: 14,

			/**
			 * Draw an overlay of linework
			 * @param {Core.ViewerFacade} f
			 * @param {CanvasRenderingContext2D} ctx - canvas context
			 * @param {Array<LineworkWrapper>} linework - array of linework
			 * @param {ShapeUtils.Rect} viewport - rectangle representing viewport
			 * @param {Function} imagecallback - callback called everytime a linework item is of type LWTYPE_IMAGE and when item becomes available (e.g. after loading if not in cache)
			 * @param {Number} [zoom = 1] - current zoom level
			 * @param {Transformation} t - transformation
			 * @param {String} [color = linework[i].color] - The linework color "#RGB"
			 * @param {Number} [alpha = 1] - transparency of the overlay
			 */
			drawOverlay: function (f, ctx, linework, viewport, imagecallback, zoom, t, color, alpha) {
				var lineworksDrawn = 0;
				if (!zoom) zoom = 1;

				ctx.save();

				if (viewport !== undefined) {
					//set the start location so we will draw on pixels, and make sure to translate by +0.5 pixels
					var left = viewport.left(); //mm, distance between the left side of the document and left bound of the visible rect
					var top = viewport.top(); //mm, distance between the top side of the document and top bound of the visible rect
					//first get the mm to pixels with zoom (ppmm) and only get the part that's not a full pixel, to draw lines with linewidth 1 on exactly
					//1 pixel make sure to draw at +0.5 pixel. The result is the amount of pixels the coordinates in the canvas have to be shifted

					var leftPx = left * zoom; //translate the mm coordinates to pixels
					var topPx = top * zoom;
					if (Math.abs(leftPx % 1) < 0.000001 || Math.abs(1 - (leftPx % 1)) < 0.000001) { //if there's a small difference (rounding error), round to nearest integer (happens on graphics files and makes the grid corrupt)
						leftPx = Math.round(leftPx);
					}
					if (Math.abs(topPx % 1) < 0.000001 || Math.abs(1 - (topPx % 1)) < 0.000001) {
						topPx = Math.round(topPx);
					}

					var xShift = 0.5 + (leftPx % 1); //pixels, amount the canvas needs to shift, compensate the rest of a hole pixel and 0.5 pixel 
					var yShift = 0.5 + (topPx % 1); //pixels, amount the canvas needs to shift, compensate the rest of a hole pixel and 0.5 pixel 

					//since coordinate system is working in mm and not pixels, convert the pixel back to mm
					ctx.translate(xShift / zoom, yShift / zoom);
				}

				if (alpha) {
					ctx.globalAlpha = alpha;
				} else {
					ctx.globalAlpha = 1;
				}
				for (var i = 0; i < linework.length; i++) {
					var wrap = linework[i];
					if (wrap == null) {
						continue;
					}
					if (debugOverlay) {
						ctx.save();
						ctx.lineWidth = .01;
						ctx.fillStyle = 'red';
						this.drawRect(ctx, wrap.rect.origin.x, wrap.rect.origin.y, wrap.rect.size.width, wrap.rect.size.height, 0, 'red', 'red');
						ctx.restore();
					}

					var viewrect = wrap.rect.copy();
					if (wrap.staticsize) {
						var center = viewrect.center();
						var size = new ShapeUtils.Size(viewrect.size.width / zoom, viewrect.size.height / zoom);
						viewrect = new ShapeUtils.Rect(new ShapeUtils.Point(center.x - size.width / 2, center.y - size.height / 2), size);
					}

					if (viewport && viewrect && !ShapeUtils.rectIsNull(viewrect)) {
						//check if the rect is in the current viewport. If not, don't worry about drawing it
						if (!viewport.rectIntersect(wrap.rect) && wrap.type != this.LWTYPE_TEXT) {
							continue;
						}
					}

					lineworksDrawn++;

					ctx.save();

					var linethickness = Math.floor(wrap.linewidth);
					if (linethickness === 0) {
						linethickness = 1;
					}

					if (!wrap.staticwidth) {
						if (zoom > 1) {
							linethickness /= Math.ceil(zoom);
						} else {
							linethickness /= zoom;
						}

					}

					var strokecolor = wrap.color;
					var fillcolor = wrap.fillcolor;
					if (color !== null && color !== undefined) {
						strokecolor = color;
						if (fillcolor) {
							fillcolor = color;
						}
					}

					var fill = (fillcolor ? true : false);
					var stroke = (strokecolor ? true : false);
					if (strokecolor) {
						ctx.strokeStyle = strokecolor;
					}
					if (fill) {
						if (color) { //general color for entire overlay was defined, use this color with an alpha
							var colorObj = ViewerUtils.rgbColor(fillcolor);
							ctx.fillStyle = "rgba(" + colorObj.r + ", " + colorObj.g + ", " + colorObj.b + ", 0.5)";
						} else {
							ctx.fillStyle = fillcolor;
						}
					}
					if (linethickness) {
						ctx.lineWidth = linethickness;
					}

					switch (wrap.type) {
					case this.LWTYPE_RECT: //rect
						var radius;
						if (wrap.cornerRadius) {
							radius = wrap.cornerRadius;
						} else {
							radius = 0;
						}
						this.drawRect(ctx, viewrect.origin.x, viewrect.origin.y, viewrect.size.width, viewrect.size.height, radius, fill, stroke);

						break;
					case this.LWTYPE_ELLIPSE: //elipse
						this.drawEllipse(ctx, viewrect.origin.x, viewrect.origin.y, viewrect.size.width, viewrect.size.height, fill, stroke);
						break;
					case this.LWTYPE_IMAGE:
						var ref = wrap.imageRef;
						var img = imageCache.get(ref, imagecallback);
						if (!img) continue;
						var tintedImg = undefined;

						if (wrap.color) {
							var rgb = wrap.rgbColor();
							tintedImg = iconCache.get([ref, rgb.r, rgb.g, rgb.b].join("|"), imagecallback);
						}

						var imgx = viewrect.origin.x;
						var imgy = viewrect.origin.y;

						imgy -= img.naturalHeight / zoom;

						if (viewrect.size.width == 0 || viewrect.size.height == 0) {
							ctx.save();
							// no width/height is defined in the linework wrapper, draw the image in its real pixel size

							var transformedRect = viewrect;
							if (t) {
								var tmpT = t.copy();
								tmpT.translate(xShift / zoom, yShift / zoom);
								transformedRect = tmpT.transformedRect(transformedRect);
							}

							ctx.setTransform(1, 0, 0, 1, 0, 0);

							// if .imageOffset is defined, add the offest to the image (in pixels)
							var offsetX = 0,
								offsetY = 0;
							if (wrap.imageOffset) {
								offsetX = wrap.imageOffset.width;
								offsetY = wrap.imageOffset.height;
							}


							// make sure to draw on pixels (Math.floor)
							ctx.drawImage(tintedImg ? tintedImg : img, Math.ceil(transformedRect.origin.x - 0.5) - offsetX, Math.ceil(transformedRect.origin.y - 0.5) - offsetY);
							ctx.restore();
						} else {
							ctx.drawImage(tintedImg ? tintedImg : img, imgx, imgy, viewrect.size.width, viewrect.size.height);
						}


						break;
					case this.LWTYPE_TEXT: //text
						if (!fillcolor) {
							ctx.fillStyle = strokecolor;
						}
						var textH = ViewerUtils.parseFloat(wrap.textoptions['font-size']);
						if (wrap.staticsize) {
							textH *= zoom;
						}
						var fs = textH + 'px ' + wrap.textoptions['font-family'];
						ctx.font = fs;

						var text = ViewerUtils.trim(wrap.textoptions['text']);
						ctx.textAlign = wrap.textoptions['textAlignment'];

						if (viewport) {
							var textW = ctx.measureText(text).width;
							var textSize = new ShapeUtils.Size(textW, textH);
							var textRect = ShapeUtils.rectMake(viewrect.origin.x - textSize.width / 2, viewrect.origin.y - textSize.height / 2, textSize.width, textSize.height);
							if (!viewport.rectIntersect(textRect)) {
								//continue;
							}
						}

						ctx.save();
						if (wrap.staticsize) {
							var transformedRect = viewrect;
							if (t) {
								transformedRect = t.transformedRect(transformedRect);
							}

							ctx.setTransform(1, 0, 0, 1, 0, 0);

							ctx.fillText(text, transformedRect.origin.x, transformedRect.origin.y);

							ctx.restore();
						} else {
							//Compensate the rotation
							ctx.translate(viewrect.origin.x, viewrect.origin.y);
							ctx.rotate(wrap.textoptions.rotate * Math.PI / 180);
							ctx.translate(-viewrect.origin.x, -viewrect.origin.y);

							//Fill the text
							ctx.fillText(text, viewrect.origin.x, viewrect.origin.y);
						}
						ctx.restore();

						break;
					case this.LWTYPE_DASHEDBOX:
						var dash = wrap.dash;

						ctx.save();
						//					    	if(t){
						//					    		viewrect = t.transformedRect(viewrect);
						//					    	}
						//					    	ctx.setTransform(1,0,0,1,0,0);
						ctx.lineWidth = linethickness;
						DashUtils.drawDashedRect(ctx, viewrect, dash, (!wrap.staticwidth ? zoom : null));

						ctx.restore();

						break;
					case this.LWTYPE_DASHEDLINE:
						var dash = wrap.dash;

						ctx.beginPath();
						if (viewrect.size.width == 0) {
							//Vertical
							DashUtils.drawDash(ctx, viewrect.origin.x, viewrect.origin.y, viewrect.origin.x, viewrect.origin.y + viewrect.size.height, dash);
						} else {
							//Horizontal
							DashUtils.drawDash(ctx, viewrect.origin.x, viewrect.origin.y, viewrect.origin.x + viewrect.size.width, viewrect.origin.y, dash);
						}

						ctx.stroke();

						break;
					default:
						var arrow = (wrap.type == this.LWTYPE_ARROW ? true : false);
						var points = wrap.pointArray;
						var dash = wrap.dash;
						ctx.beginPath();
						if (points.length > 0) {
							this.drawLine(f, ctx, points, arrow, t, zoom, fill, dash);
						}
						break;
					}

					if (wrap.showIndicator) {
						/**
						 * Show an indicator of the object
						 * By default, this is placed on the top left corner of the linework
						 *
						 * Unless:
						 * 	- Array of points: show the indicator at the first point
						 *  - Ellipse: top center
						 */

						ctx.save();
						ctx.setTransform(1, 0, 0, 1, 0, 0);

						var textH = 14;
						ctx.font = textH + 'px Arial';
						ctx.textAlign = 'center';

						var text = wrap.indicatorText;

						var boxwidth = ctx.measureText(text).width;
						var textHeight = textH * 72 / 96;
						var boxheight = textHeight;

						var offset = boxheight * .5;
						boxwidth += offset;
						boxheight += offset;

						var indicationRect = viewrect;
						if (t) {
							indicationRect = t.transformedRect(indicationRect);
						}
						var indicationPoint = indicationRect.origin; // new ShapeUtils.Point(indicationRect.left(), indicationRect.top());
						if (wrap.type == this.LWTYPE_ELLIPSE) {
							indicationPoint.x = indicationRect.center().x - boxwidth / 2;
						} else if (wrap.pointArray && wrap.pointArray.length > 0) {
							indicationPoint = t.transformedPoint(wrap.pointArray[0]);
						}

						var textBox = new ShapeUtils.Rect(new ShapeUtils.Point(indicationPoint.x, indicationPoint.y), new ShapeUtils.Size(boxwidth, boxheight));

						ctx.fillStyle = wrap.indicatorColor;
						ctx.strokeStyle = 'black';
						ctx.lineWidth = 2;

						var radius = textBox.size.height / 2;
						radius = 0;
						this.drawRect(ctx, textBox.origin.x, textBox.origin.y, textBox.size.width, textBox.size.height, radius, true, true);

						ctx.fillStyle = wrap.indicatorTextColor;
						ctx.fillText(text, textBox.center().x, indicationPoint.y + boxheight / 2 + textHeight / 2);

						ctx.restore();
					}
					ctx.restore();
				}
				ctx.restore();
				if (debugOverlay) console.log('endOverlay (' + lineworksDrawn + ')');
			},
			generateTintImage: generateTintImage,
			/**
			 * Draws a rounded rectangle using the current state of the canvas.
			 * If you omit the last three params, it will draw a rectangle
			 * outline with a 5 pixel border radius
			 * @param {CanvasRenderingContext2D} ctx - Canvas context
			 * @param {Number} x - The top left x coordinate
			 * @param {Number} y - The top left y coordinate
			 * @param {Number} width - The width of the rectangle
			 * @param {Number} height - The height of the rectangle
			 * @param {Number} radius - The corner radius. Defaults to 5;
			 * @param {Boolean} fill - Whether to fill the rectangle. Defaults to false.
			 * @param {Boolean} stroke - Whether to stroke the rectangle. Defaults to true.
			 */
			drawRect: function (ctx, x, y, width, height, radius, fill, stroke) {
				if (typeof stroke == "undefined") {
					stroke = true;
				}
				if (typeof radius === "undefined") {
					radius = 5;
				}
				ctx.save();
				ctx.beginPath();
				ctx.moveTo(x + radius, y);
				ctx.lineTo(x + width - radius, y);
				ctx.quadraticCurveTo(x + width, y, x + width, y + radius);
				ctx.lineTo(x + width, y + height - radius);
				ctx.quadraticCurveTo(x + width, y + height, x + width - radius, y + height);
				ctx.lineTo(x + radius, y + height);
				ctx.quadraticCurveTo(x, y + height, x, y + height - radius);
				ctx.lineTo(x, y + radius);
				ctx.quadraticCurveTo(x, y, x + radius, y);
				ctx.closePath();
				if (stroke) {
					ctx.stroke();
				}
				if (fill) {
					ctx.fill();
				}
				ctx.restore();
			},
			/**
			 * Draw a line between points
			 * @param {ViewerFacade} f - Viewer facade
			 * @param {CanvasRenderingContext2D} ctx - Canvas context
			 * @param {Array<ShapeUtils.Point>} points - Array of points to draw lines in between
			 * @param {Boolean} arrow - True to make arrow between the first two points
			 * @param {Transformation} t - Transformation
			 * @param {Number} zoom - Zoom level
			 * @param {Boolean} fill - True to fill area in between the lines
			 * @param {Boolean} dash - True to dash line, false otherwise
			 */
			drawLine: function (f, ctx, points, arrow, t, zoom, fill, dash) {
				var debugdrawing = false;
				ctx.beginPath();
				for (var i = 0; i < points.length; i++) {
					var pointObject = points[i];

					var p = new ShapeUtils.Point(pointObject.x, pointObject.y);

					//round the points so we are drawing right on the pixels to avoid that linewidth 1 spreads over 2 pixels
					var x = Math.ceil(pointObject.x * zoom) / zoom;
					var y = Math.ceil(pointObject.y * zoom) / zoom;

					if (this.dashedLineWarning === undefined) {
						this.dashedLineWarning = _.once(function () {
							f.i18n.loadBundle("viewer", function () {
								f.emit("error.WARNING", f.i18n.t("DashedLineSupport"));
							});
						});
					}

					var dashSizes = dash;

					if (dash && zoom) { //if zoom is defined, draw dashsizes in pixel instead of mm
						dashSizes = _.map(dash, function (value) { //map all dashsizes from mm to pixels
							return value / zoom;
						});
					}

					var supportsLineDash = true; //are the lines dashed or not
					if (dashSizes) { //if there are dashed lines
						if (ctx.setLineDash !== undefined) { //check if setLineDash exists and dashed lines are supported
							ctx.setLineDash(dashSizes);
						} else if (ctx.mozDash !== undefined) { //for older versions of firefox
							ctx.mozDash = dashSizes;
						} else {
							supportsLineDash = false; //dashed lines are not supported
						}
					}

					switch (pointObject.type) {
					case 0:
						ctx.moveTo(x, y);
						break;
					case 1:
						if (supportsLineDash) { //if dashed lines are supported make use of this feature
							ctx.lineTo(x, y);
						} else { //dashed lines are not supported draws the dashed lines
							var prPoint = points[i - 1];
							var previousPoint = new ShapeUtils.Point(prPoint.x, prPoint.y);
							DashUtils.drawDash(ctx, previousPoint.x, previousPoint.y, x, y, dashSizes);
						}
						if (debugdrawing) ctx.stroke();
						break;
					case 2:
						var p2 = new ShapeUtils.Point(pointObject.v1, pointObject.v2);
						var p3 = new ShapeUtils.Point(pointObject.v3, pointObject.v4);

						if (!supportsLineDash) { //show the dashed line warning if the browser doesn't support them
							this.dashedLineWarning();
						}

						ctx.bezierCurveTo(p.x, p.y, p2.x, p2.y, p3.x, p3.y);

						if (debugdrawing) ctx.stroke();
						break;
					case 3:
						var prPoint = points[i - 1];
						var previousPoint = new ShapeUtils.Point(prPoint.x, prPoint.y);
						var rx = pointObject.v1;
						var angle = pointObject.v3;
						var largeArcFlag = pointObject.v4;
						var sweepFlag = pointObject.v5;

						if (!supportsLineDash) {
							this.dashedLineWarning();
						}

						this.drawArcFrom(ctx, previousPoint, p, rx, largeArcFlag, sweepFlag, angle);

						if (debugdrawing) ctx.stroke();
						break;
					case 4: //nick
						//First point of the nick
						var p = new ShapeUtils.Point(pointObject.x, pointObject.y);

						//Second point of the nick is the same as the first point, minus the coordinates given in the nick definition
						var p2 = p.copy();
						p2.x -= pointObject.v3 * 10;
						p2.y -= pointObject.v2 * 10;

						//The text should be a bit away from the second point
						var p3 = p2.copy();
						p3.x -= pointObject.v3 * 5;
						p3.y -= pointObject.v2 * 5;

						//The rotation is the rotation between point 1 and 2 + 90 degrees
						var rotation = Math.atan2(p2.y - p.y, p2.x - p.x) + Math.PI / 2;

						//Move
						ctx.moveTo(p.x, p.y);
						//Line
						ctx.lineTo(p2.x, p2.y);

						//Save the context and rotate to draw the text
						ctx.save();

						ctx.translate(p3.x, p3.y);
						ctx.rotate(rotation);
						ctx.translate(-p3.x, -p3.y);


						var textH = 8;
						ctx.font = textH + 'px Arial';
						ctx.textAlign = 'center';
						ctx.fillStyle = 'blue';
						var text = ViewerUtils.parseFloat(pointObject.v1).toFixed(2);
						ctx.fillText(text, p3.x, p3.y);

						//Restore the context
						ctx.restore();


						break;
					default:
						console.log('a line that is not supported: ' + pointObject.type);
						break;
					}

				}
				if (!debugdrawing && !fill) {
					ctx.stroke();
				}
				if (arrow && points.length > 1) {
					ctx.beginPath();
					// Find out an exellent tutorial: http://www.dbp-consulting.com/tutorials/canvas/CanvasArrow.html
					// Find out an exellent tutorial: http://www.dbp-consulting.com/tutorials/canvas/CanvasArcTo.html
					var startP = points[0];
					var endP = points[1];

					var endPoint;
					var startPoint;
					if (endP.type == 2) {
						startPoint = new ShapeUtils.Point(endP.v1, endP.v2);
						endPoint = new ShapeUtils.Point(endP.v3, endP.v4);
					} else {
						startPoint = new ShapeUtils.Point(startP.x, startP.y);
						endPoint = new ShapeUtils.Point(endP.x, endP.y);
					}
					var start = startPoint;
					var end = endPoint;
					if (t) {
						start = t.transformedPoint(start);
						end = t.transformedPoint(end);
						ctx.save();
						ctx.setTransform(1, 0, 0, 1, 0, 0);
					}

					var u = new ShapeUtils.Point(end.x - start.x, end.y - start.y);
					var length = Math.sqrt(u.x * u.x + u.y * u.y);

					if (length === 0) {
						u.x = 0;
						u.y = 0;
					} else {
						u.x /= length;
						u.y /= length;
					}

					var v = new ShapeUtils.Point(-u.y, u.x);

					var arrowSize = 8;

					u.x *= arrowSize;
					u.y *= arrowSize;
					v.x *= arrowSize / 1.0;
					v.y *= arrowSize / 1.0;
					var p = new ShapeUtils.Point(end.x, end.y);
					p.x += -u.x + v.x;
					p.y += -u.y + v.y;
					var p2 = new ShapeUtils.Point(p.x, p.y);
					p2.x -= 2 * v.x;
					p2.y -= 2 * v.y;

					ctx.moveTo(p.x, p.y);
					ctx.lineTo(end.x, end.y);
					ctx.lineTo(p2.x, p2.y);

					if (debugdrawing) ctx.stroke();

					if (t) {
						ctx.restore();
					}
				}
				if (fill) {
					ctx.fill();
				} else if (!debugdrawing) {
					ctx.stroke();
				}
			},
			/**
			 * Draw an arc
			 * @param {CanvasRenderingContext2D} ctx - Canvas context
			 * @param {ShapeUtils.Point} from - connecting begin point
			 * @param {ShapeUtils.Point} to - connecting end point
			 * @param {Number} radius - radius of the circle
			 * @param {Boolean} bigArc - ?
			 * @param {Boolean} left - True if left, false otherwise
			 * @param {Number} angle - angle of the arc
			 */
			drawArcFrom: function (ctx, from, to, radius, bigArc, left, angle) {

				//Temp solution: when the 2 X values are the same (with accuracy of 2), just draw a line instead of an arch
				var xDiff = Math.abs(from.x - to.x).toFixed(2);
				if (xDiff == 0) {
					ctx.lineTo(to.x, to.y);
				} else {
					var pointDiff = new ShapeUtils.Point(to.x - from.x, to.y - from.y);
					var pointDiffSquare = pointDiff.x * pointDiff.x + pointDiff.y * pointDiff.y;
					var distance = Math.sqrt(pointDiffSquare); //distance between to and from
					var middle = new ShapeUtils.Point((to.x + from.x) / 2, (to.y + from.y) / 2); //middle between to and from
					var squareRoot = Math.sqrt((radius * radius) - ((distance / 2) * (distance / 2))); //tmp var for the calculation of the centre of the circle arc
					if (!left) { //when not left, invert x
						squareRoot = -squareRoot;
					}
					//Calculate the center of the circle arc
					var center = new ShapeUtils.Point(middle.x - (squareRoot * (from.y - to.y) / distance), middle.y - (squareRoot * (to.x - from.x) / distance));
					var startCos = (from.x - center.x) / radius; //cosinus of the angle of the starting point of the circle arc
					var endCos = (to.x - center.x) / radius; //cosinus of the end angle of the circle arc
					startCos = startCos > 1 ? 1 : startCos; //cosinus cannot be bigger than 1 or smaller than -1
					startCos = startCos < -1 ? -1 : startCos;
					endCos = endCos > 1 ? 1 : endCos;
					endCos = endCos < -1 ? -1 : endCos;
					var startAngle = Math.acos(startCos);
					var endAngle = Math.acos(endCos);
					if (from.y < center.y) { //make sure to choose the correct angle if in doubt (- or +)
						startAngle = -startAngle;
					}
					if (to.y < center.y) {
						endAngle = -endAngle;
					}

					ctx.lineTo(from.x, from.y); //make the move to the starting point
					ctx.arc(center.x, center.y, radius, startAngle, endAngle, left); //draw the arc
					ctx.lineTo(to.x, to.y); //make the move to the end point, to make sure we restart the path from the correct position
				}
			},
			/**
			 * Draw a dashed line between two points
			 * @param {CanvasRenderingContext2D} ctx - Canvas context
			 * @param {Number} x - x coordinate of the first point
			 * @param {Number} y - y coordinate of the first point
			 * @param {Number} x2 - x coorindate of the second point
			 * @param {Number} y2 - x coordinate of the second point
			 * @param {Array<Number>} dashArray - dash lenghts
			 */
			drawDash: DashUtils.drawDash,
			/**
			 * Draw an ellipse
			 * @param {CanvasRenderingContext2D} ctx - Canvas context
			 * @param {Number} x - x coordinate (top left of the ellipse)
			 * @param {Number} y - y coordinate (top left of the ellipse)
			 * @param {Number} w - width of the ellipse
			 * @param {Number} h - height of the ellipse
			 * @param {Boolean} fill - True to fill the ellipse
			 * @param {Boolean} stroke - True to stroke the ellipse
			 */
			drawEllipse: function (ctx, x, y, w, h, fill, stroke) {
				var kappa = .5522848;
				var ox = (w / 2) * kappa; // control point offset horizontal
				var oy = (h / 2) * kappa; // control point offset vertical
				var xe = x + w; // x-end
				var ye = y + h; // y-end
				var xm = x + w / 2; // x-middle
				var ym = y + h / 2; // y-middle

				ctx.beginPath();
				ctx.moveTo(x, ym);
				ctx.bezierCurveTo(x, ym - oy, xm - ox, y, xm, y);
				ctx.bezierCurveTo(xm + ox, y, xe, ym - oy, xe, ym);
				ctx.bezierCurveTo(xe, ym + oy, xm + ox, ye, xm, ye);
				ctx.bezierCurveTo(xm - ox, ye, x, ym + oy, x, ym);
				ctx.closePath();
				if (fill) {
					ctx.fill();
				}
				if (stroke) {
					ctx.stroke();
				}
			},
			/**
			 * Calculate the distance from a point to a segment with a max of maxDistance
			 * @param {ShapeUtils.Point} pnt - Point
			 * @param {ShapeUtils.Point} from - Begin point of segment
			 * @param {ShapeUtils.Point} to - End point of segment
			 * @param {Number} maxDistance - maximum distance between point and segment (will return maxDistance + 1 if exceeded)
			 * @returns {Number} the distance between the point and the segment with a max of maxDistance + 1
			 */
			distanceFromPointToSegment: function (pnt, from, to, maxDistance) {
				// Quick bounding box check
				if (from.x > to.x) {
					if (pnt.x < to.x - maxDistance) return maxDistance + 1;
					if (pnt.x > from.x + maxDistance) return maxDistance + 1;
				} else {
					if (pnt.x < from.x - maxDistance) return maxDistance + 1;
					if (pnt.x > to.x + maxDistance) return maxDistance + 1;
				}
				if (from.y > to.y) {
					if (pnt.y < to.y - maxDistance) return maxDistance + 1;
					if (pnt.y > from.y + maxDistance) return maxDistance + 1;
				} else {
					if (pnt.y < from.y - maxDistance) return maxDistance + 1;
					if (pnt.y > to.y + maxDistance) return maxDistance + 1;
				}

				// Distance point to infinite line
				var s = new ShapeUtils.Point();
				s.x = to.x - from.x;
				s.y = to.y - from.y;
				var p = new ShapeUtils.Point();
				p.x = pnt.x - from.x;
				p.y = pnt.y - from.y;
				if ((s.x != 0) || (s.y != 0)) {
					var segmentLength2 = s.x * s.x + s.y * s.y;
					// Test position along segment
					var dotS = s.x * p.x + s.y * p.y;
					if ((dotS > 0) && (dotS < segmentLength2)) {
						// We are inside the segment
						var dotN = s.y * p.x - s.x * p.y;
						if (dotN * dotN > maxDistance * maxDistance * segmentLength2) return maxDistance + 1; // too far away from infinite line
						return Math.abs(dotN / Math.sqrt(segmentLength2));
					}
				}
				var dFrom = p.x * p.x + p.y * p.y;
				if (dFrom > maxDistance * maxDistance) return maxDistance + 1;
				else return Math.sqrt(dFrom);
				var p2 = new ShapeUtils.Point();
				p2.x = point.x - to.x;
				p2.y = point.y - to.y;
				var dTo = p2.x * p2.x + p2.y * p2.y;
				if (dTo > maxDistance * maxDistance) return maxDistance + 1;
				else return sqrt(dTo);

				return maxDistance + 1; // just to avoid compiler warnings
			},
			/**
			 * Calculate the distance from a point to an ellipse
			 * @param {ShapeUtils.Point} p - point
			 * @param {ShapeUtils.Size} r - rectangle around the ellipse
			 * @param {Number} max - maximum distance (will return max + 1 if exceeded)
			 * @returns {Number} distance from point to an ellipse
			 */
			distanceFromPointToEllipse: function (p, r, max) {
				if (r.size.width == 0) return max + 1;
				if (r.size.height == 0) return max + 1;

				// Simple bounding box test
				if (p.x < r.origin.x - max) return max + 1;
				if (p.y < r.origin.y - max) return max + 1;
				if (p.x > r.origin.x + r.size.width + max) return max + 1;
				if (p.y > r.origin.y + r.size.height + max) return max + 1;

				var ctr = new ShapeUtils.Point(0, 0);
				ctr.x = r.origin.x + r.size.width / 2.0;
				ctr.y = r.origin.y + r.size.height / 2.0;

				// This is not mathematically correct, but accurate enough for this purpose
				var d = new ShapeUtils.Point();
				d.x = p.x - ctr.x;
				d.y = p.y - ctr.y;
				d.x *= 2.0 / r.size.width;
				d.y *= 2.0 / r.size.height;
				var length = Math.sqrt(d.x * d.x + d.y * d.y);
				if (length < 0.0001) return 0;
				d.x *= r.size.width * (1 - length) / (2.0 * length);
				d.y *= r.size.height * (1 - length) / (2.0 * length);
				length = d.x * d.x + d.y * d.y;
				if (length > max * max) return max + 1;
				return Math.sqrt(length);
			},
			/**
			 * Calculate distance between lineworkwrapper and point
			 * @param {LineworkWrapper} lineworkWrapper
			 * @param {ShapeUtils.Point} point
			 * @param {Transformation} transform
			 * @param {Number} [fatfinger = 6] - max distance
			 * @returns {Number} distance between lineworkwrapper and point
			 */
			distanceOfScenePoint: function (lineworkWrapper, point, transform, fatfinger) {
				if (!fatfinger) {
					fatfinger = 6;
				}
				var g = fatfinger;


				var dist = -1;

				var points = lineworkWrapper.pointArray;

				// check by points
				if (points && points.length != 0) {
					if (points.length == 1) {
						//only one point
						var lwPoint = transform.transformedPoint(points[0].point);
						var refPoint = new ShapeUtils.Point(lwPoint.x, lwPoint.y);
						if (Math.abs(refPoint.x - point.x) < g && Math.abs(refPoint.y - point.y) < g) {
							dist = 999;
						}
					} else {
						for (var j = 0; j < points.length - 1; j++) {
							var lwPoint = transform.transformedPoint(points[j]);
							var lwPoint2 = transform.transformedPoint(points[j + 1]);
							var refPoint = new ShapeUtils.Point(lwPoint.x, lwPoint.y);
							var refPoint2 = new ShapeUtils.Point(lwPoint2.x, lwPoint2.y);
							var refdist = this.distanceFromPointToSegment(point, refPoint, refPoint2, g);
							if (refdist <= g) {
								dist = refdist;
							}
						}
					}

				} else { // check by rect
					var r = transform.transformedRect(lineworkWrapper.rect);
					if (lineworkWrapper.staticsize) {
						//because it's a static size, use the actual size of the rectangle
						var center = r.center();
						var size = new ShapeUtils.Size(lineworkWrapper.rect.size.width, lineworkWrapper.rect.size.height);
						r = new ShapeUtils.Rect(new ShapeUtils.Point(center.x - size.width / 2, center.y - size.height / 2), size);
					}

					var outsideRect = new ShapeUtils.Rect(new ShapeUtils.Point(r.origin.x - g, r.origin.y - g), new ShapeUtils.Size(r.size.width + (g * 2), r.size.height + (g * 2)));

					r.normalize();
					outsideRect.normalize();

					var refdist;
					if (lineworkWrapper.type == this.LWTYPE_ELLIPSE) {
						if (lineworkWrapper.fillcolor) {
							if (r.containsPoint(point)) {
								dist = 1;
							} else if (outsideRect.containsPoint(point)) {
								dist = 999;
							}
						} else {
							refdist = this.distanceFromPointToEllipse(point, r, g);
							if (refdist <= g) {
								dist = refdist;
							}
						}
					} else {
						if (lineworkWrapper.type == this.LWTYPE_IMAGE && lineworkWrapper.imagetouchspot) { //image
							var touchSpot = lineworkWrapper.imagetouchspot;
							var tX = touchSpot.origin.x;
							var tY = touchSpot.origin.y;
							var tWidth = touchSpot.size.width;
							var tHeight = touchSpot.size.height;
							outsideRect = new ShapeUtils.Rect(new ShapeUtils.Point(outsideRect.origin.x + tX, outsideRect.origin.y + tY), new ShapeUtils.Size(tWidth + (g * 2), tHeight + (g * 2)));
						}

						if (outsideRect.containsPoint(point)) {
							if ((lineworkWrapper.fillcolor || lineworkWrapper.type == this.LWTYPE_IMAGE) && !lineworkWrapper.treatAsOutline) {
								dist = 999;
							} else {
								var insideRect = new ShapeUtils.Rect(new ShapeUtils.Point(r.origin.x + g, r.origin.y + g), new ShapeUtils.Size(r.size.width - (g * 2), r.size.height - (g * 2)));
								if (!insideRect.containsPoint(point)) { //rectangle
									dist = 999;
								}
							}
						}
					}
				}
				return dist;
			},
			/**
			 * Draw a dashed line
			 * @param {Context2D} ctx - canvas context
			 * @param {Number} x - start x
			 * @param {Number} y - start y
			 * @param {Number} x2 - end x
			 * @param {Number} y2 - end y
			 * @param {Array.<Number>} dashArray - dash pattern [px draw, px empty, px draw, px empty, ...]
			 */
			drawDashedLine: DashUtils.drawDashedLine,
			/**
			 * Draw a dashed rectangle
			 * @param {Context2D} ctx - canvas context
			 * @param {ShapeUtils.Rect} rect - rectangle to draw
			 * @param {Array.<Number>} dashArray - dash pattern
			 * @param {Number} [zoom] - which zoomlevel is used (use this if you want to define the dasharray in pixels and not world coordinates)
			 */
			drawDashedRect: DashUtils.drawDashedRect
		};
		return DrawUtils;
	});

define('web-viewer/utils/Fragment',["utils/Cache",
		"underscore"
	],
	function (Cache, _) {

		// empty image
		var emptyImage = new Image();


		/**
		 * @exports Fragment
		 * @memberof Utils
		 * @class
		 * A fragment is a dynamically loaded image
		 * @constructor
		 * @param {ShapeUtils.Rect} viewrect - rectangle of the fragment
		 * @param {Number} z - zoom level of the fragment
		 */
		var Fragment = function (viewrect, z) {
			this.rect = viewrect;
			this.zoom = z;
			this.image = new Image();

			this.fetched = false;
			this.dirty = false;
		};

		/**
		 * Start fetching the fragment if needed
		 * @param {String} url - the viewserver url of the image
		 * @param {Function} onfinish - callback when the image was loaded successfully
		 * @param {Function} onerror - callback when there was an error while loading the image
		 */
		Fragment.prototype.fetchImage = function (fragmentCache, url, onfinish, onerror) {
			//temporarily put in an empty image
			this.image = emptyImage;

			// fetch the image from the cache, if already in cache, callbacks will be called immediatly
			fragmentCache.get(url, _.bind(function (img) {
				if (img === undefined || img.error) {
					// something went wrong while loading image, remove it from the cache
					onerror.call(undefined);
				} else {
					// everything went ok, set the fragment as fetched and execute callback
					this.image = img;
					this.fetched = true;
					onfinish.call(img);
				}
			}, this));
		};

		/**
		 * get a reference to the image object
		 * @returns {Image}
		 */
		Fragment.prototype.getImage = function () {
			return this.image;
		};

		/**
		 * Set the image for the fragment
		 * @param {Image} image
		 */
		Fragment.prototype.setImage = function (image) {
			this.image = $("<img>").attr("src", image)[0];
			this.fetched = true;
		};

		/**
		 * @returns {Boolean} is the fragment fetched already?
		 */
		Fragment.prototype.isFetched = function () {
			return this.fetched;
		};

		/**
		 * @returns {Boolean} is the fragment dirty? (should it be thrown away if there are new fragments)
		 */
		Fragment.prototype.isDirty = function () {
			return this.dirty;
		};

		/**
		 * Mark the fragment as dirty
		 */
		Fragment.prototype.makeDirty = function () {
			this.dirty = true;
		};

		/**
		 * Get the rectangle of the fragment
		 * @returns {ShapeUtils.Rect}
		 */
		Fragment.prototype.getRect = function () {
			return this.rect;
		};

		/**
		 * Get the zoom level of the fragment
		 * @returns {Number}
		 */
		Fragment.prototype.getZoom = function () {
			return this.zoom;
		};

		/**
		 * Get the real width (in pixels) of the image
		 * @returns {Number} width in pixels
		 */
		Fragment.prototype.getNaturalWidth = function () {
			return this.image.naturalWidth;
		};

		/**
		 * Get the real height (in pixels) of the image
		 * @returns {Number} height in pixels
		 */
		Fragment.prototype.getNaturalHeight = function () {
			return this.image.naturalHeight;
		};


		return Fragment;
	});

define('web-viewer/viewport/model/ViewportFragmentManager',["underscore",
		"utils/ShapeUtils",
		"web-viewer/utils/Fragment"
	],
	function (_, ShapeUtils, Fragment) {
		"use strict";

		var MAX_FRAGMENTS_COUNT = 5;
		var DEBUG = false;
		var COMPARE_THRESHOLD = 0.00001;

		function thresholdEquals(value1, value2) {
			return Math.abs(value1 - value2) <= COMPARE_THRESHOLD;
		};

		/**
		 * @exports ViewportFragmentManager
		 * @memberof ViewportModule
		 * @class ViewportFragmentManager is repsonsible for managing all frames shown in a certain view. When a frame is request, the fragmentCache is used
		 * @param {Core.ViewerFacade} f - the ViewerFacade
		 * @param {FragmentManagerDelegate} delegate - the delegate needed for the fragmentmanager
		 * @example
		 *
		 * The FragmentManagerDelegate type has following methods:
		 * - {String} getURLForFragment({Utils.Fragment}) - generate a URL for the a specific fragment
		 * - {void} fragmentsChanged() - the fragment definition has changed, re render
		 *
		 */
		var ViewportFragmentManager = function (f, delegate, world) {
			this.f = f;
			this.delegate = delegate;
			this.world = world;

			// a list of available fragments, ordered by their zoom level
			this.fragments = [];
			this.initialFragment = null;
			this.dirtyInitialFragment = null;
			this.fragmentCache = this.f.getModule("viewport").getImageCache();
		};

		/**
		 * Get a list of all fragments
		 * @returns {Array<Utils.Fragment>}
		 */
		ViewportFragmentManager.prototype.getAllFragments = function () {
			var array = this.fragments.slice(0);
			array.prepend(this.intialFragment);
			return array;
		};

		/**
		 * Get the initial fragment
		 *
		 * @returns {Utils.Fragment|null} the initial fragment (fragment with initial zoom) or null if no fragment is available
		 */
		ViewportFragmentManager.prototype.getInitialFragment = function () {
			if (!this.initialFragment || !this.initialFragment.isFetched()) {
				return this.dirtyInitialFragment;
			}
			return this.initialFragment;
		};

		/**
		 * Check if a rectangle is the same or covered/contained by referenceRect
		 * @param {ShapeUtils.Rect} rect
		 * @param {ShapeUtils.Rect} referenceRect
		 * @returns {Boolean}
		 */
		ViewportFragmentManager.prototype.isRectCoveredBy = function (rect, referenceRect) {
			return rect.rectEquals(referenceRect) || referenceRect.containsRect(rect);
		};

		/**
		 * Check if a fragment has the same resolution and is covered/contained by another fragment
		 * @param {Utils.Fragment} fragment
		 * @param {Utils.Fragment} reference
		 * @returns {Boolean}
		 */
		ViewportFragmentManager.prototype.isFragmentCoveredBy = function (fragment, reference) {
			return thresholdEquals(fragment.getZoom(), reference.getZoom()) && this.isRectCoveredBy(fragment.getRect(), reference.getRect());
		};

		/**
		 * Request to include a fragment. The function will see if the fetch is worthwhile (if no other fragment is already there for the current frame + resolution)
		 *
		 * @param {ShapeUtils.Rect} viewrect - Rect to fetch from the viewserver
		 * @param {float} zoom - zoom/resolution of the fragment to fetch
		 */
		ViewportFragmentManager.prototype.preloadFragment = function (fragmentRect, zoomLevel) {
			if (fragmentRect.size.width === 0 || fragmentRect.size.height === 0) {
				// don't preload empty fragments
				return;
			}

			var newFragment = new Fragment(fragmentRect, zoomLevel);

			var initialFragment = this.getInitialFragment();
			if (initialFragment && !initialFragment.isDirty() && this.isFragmentCoveredBy(newFragment, initialFragment)) {
				// we are requesting something similar to the original fragment... ignore :)
				return;
			}


			var i;
			// first check if we really need to fetch this fragment
			for (i = 0; i < this.fragments.length; i++) {
				var reference = this.fragments[i];
				if (!reference.isDirty() && this.isFragmentCoveredBy(newFragment, reference)) {
					//>>excludeStart("PExclude", pragmas.productionExclude);
					if (DEBUG) {
						this.f.debug.debug("Fragment available, shouldn't fetch new data");
					}
					//>>excludeEnd("PExclude");
					return;
				}
			}


			// add the fragment at the correct location
			i = 0;
			while (i < this.fragments.length &&
				newFragment.getZoom() >= this.fragments[i].getZoom()) {
				i++;
			}

			this.fragments.splice(i, 0, newFragment);

			//>>excludeStart("PExclude", pragmas.productionExclude);
			if (DEBUG) {
				this.f.debug.debug("Fetch for view port: " + fragmentRect.toString() + " and zoom: " + zoomLevel + " insert at index: " + i);
			}
			//>>excludeEnd("PExclude");

			// a new fragment starts at age 0
			newFragment.age = 0;
			newFragment.fetchImage(this.fragmentCache,
				this.getUrlForFragment(newFragment),
				_.bind(this._fragmentFetched, this),
				_.bind(this._removeFragment, this, newFragment)
			);
		};

		/**
		 * Get applicable fragments, these are the fragments that should be drawn in the viewport
		 * note, this function will also try to remove unnecessary fragments
		 * @param {ShapeUtils.Rect} viewRectangle
		 * @param {Number} resolution
		 * @returns {Array.<Utils.Fragment>}
		 */
		ViewportFragmentManager.prototype.getApplicableFragments = function (viewRectangle, resolution) {
			var finalFragments = [];
			var i;
			var perfectFit = null;
			var initialFragment = this.getInitialFragment();

			if (initialFragment && initialFragment.isFetched()) {
				if (!initialFragment.isDirty() && viewRectangle.rectEquals(initialFragment.getRect()) && thresholdEquals(initialFragment.getZoom(), resolution)) {
					initialFragment.perfectFit = true;
					finalFragments.push(initialFragment);
					return finalFragments;
				} else {
					initialFragment.perfectFit = false;
				}
			} else {
				initialFragment = null;
			}

			// cleanFragments contains the applicable fragments that are clean
			var cleanFragments = [];
			// dirtyFragments contains the applicable fragments that are dirty
			var dirtyFragments = [];
			var totalFetchedFrames = 0;
			var totalDirtyFrames = 0;
			// inline helper function to add the fragment to the correct array (dirty or non-dirty)
			var addFragment = function (fragment) {
				if (fragment.isDirty()) {
					dirtyFragments.push(fragment);
				} else {
					cleanFragments.push(fragment);
				}
				// once a fragment is applicable in some way, it gets 1 (year) younger
				fragment.age--;
			};

			// calculate the overlap of the document with the view rectangle, this is needed to detect a perfect fit
			var overlap = viewRectangle.calculateOverlap(this.world.getWorldBounds());

			// run through all frames
			for (i = 0; i < this.fragments.length; i++) {
				var fragment = this.fragments[i];
				// everytime we draw, a frame gets 3 (years) older
				fragment.age += 3;
				if (!perfectFit) {
					if (fragment.isFetched()) {
						totalFetchedFrames++;

						if (fragment.isDirty()) {
							totalDirtyFrames++;
						}

						if (fragment.getZoom() <= resolution) {
							// the fragment is of lower (or equal) resolution than needed						
							if (ShapeUtils.rectIntersection(fragment.getRect(), viewRectangle)) {
								// there is some kind of overlap, this fragment will be drawn						
								if (!initialFragment || !(fragment.getZoom() < initialFragment.getZoom() && initialFragment.getZoom() > resolution)) {
									// if there is an initial fragment, we only draw fragments that have a higher resolution than the initialfragment, unless the required drawing resolution is lower than the the initial fragments resolution
									if (thresholdEquals(fragment.getZoom(), resolution) && this.isRectCoveredBy(overlap, fragment.getRect())) {
										// we have a perfect fit (the fragment covers the view rectangleoverlap and is in the same resolution
										fragment.perfectFit = true;
										if (!fragment.isDirty()) {
											perfectFit = fragment;
											//>>excludeStart("PExclude", pragmas.productionExclude);
											if (DEBUG) {
												this.f.debug.debug("Perfect (nondirty) was found!");
											}
											//>>excludeEnd("PExclude");
										}
									} else {
										fragment.perfectFit = false;
									}

									addFragment(fragment);
								}
							}
						} else if (this.world.isAnimating() && fragment.getZoom() * 3 <= resolution) {
							// during animations we can also draw fragments that are sharper than needed					
							if (ShapeUtils.rectIntersection(fragment.getRect(), viewRectangle)) {
								addFragment(fragment);
							}
						}
					}
				}
			}

			if (perfectFit) {
				// we found a perfect fit, so only draw the initial fragment (if any) and the perfect fit
				if (initialFragment) {
					finalFragments.push(initialFragment);
				}
				// the perfect fit frame gets 3 year younger
				perfectFit.age = -3;
				finalFragments.push(perfectFit);
			} else {
				// build up the list of fragments to draw, first draw dirty fragments till we have clean fagments
				var dirtyI = 0,
					cleanI = 0;
				var currentDirty = dirtyFragments[dirtyI];
				var currentClean = cleanFragments[cleanI];
				while (currentDirty && (!currentClean || currentDirty.getZoom() < currentClean.getZoom())) {
					//if a fragment is drawn it gets one year younger
					currentDirty.age -= 1;
					finalFragments.push(currentDirty);
					currentDirty = dirtyFragments[++dirtyI];
				}
				// draw all the clean fragments, but don't draw fragments for which the viewable area is completily overdrawn by one the following frame
				while (cleanI < cleanFragments.length) {
					var nextFragment = cleanFragments[cleanI + 1];
					if (!nextFragment || !this.isRectCoveredBy(viewRectangle.calculateOverlap(cleanFragments[cleanI].getRect()), nextFragment.getRect())) {
						// if a fragment gets drawn it gets 1 year younger
						cleanFragments[cleanI].age -= 1;
						finalFragments.push(cleanFragments[cleanI]);
					}
					cleanI++;
				}
			}

			// if there are no fragments and an initial fragment or we are zoomed in more than the initial fragment, add the initial fragment
			if (initialFragment && (finalFragments.length == 0 || initialFragment.getZoom() <= resolution)) {
				// find the correct location for the initial fragment
				i = 0;
				while (i < finalFragments.length && finalFragments[i].getZoom() < initialFragment.getZoom()) {
					i++;
				}

				finalFragments.splice(i, 0, initialFragment);
			}


			// now we need to do some clean up		
			var toRemove = totalFetchedFrames - totalDirtyFrames - MAX_FRAGMENTS_COUNT;
			var before = this.fragments.length;

			if (toRemove > 0) {
				// we store to many clean/fetched fragments, clean up!

				// order the fragments by age (and moment added (more recent fragments first))
				var orderedByAge = _.sortBy(this.fragments, function (fragment, index, list) {
					return 100 * fragment.age + (list.length - index);
				});

				//>>excludeStart("PExclude", pragmas.productionExclude);
				if (DEBUG) {
					this.f.debug.debug("Removing fragment, available ages:", _.map(this.fragments, function (fragment, index, list) {
						return 100 * fragment.age + (list.length - index);
					}));
					this.f.debug.debug("age ordering", _.map(orderedByAge, function (fragment, index, list) {
						return 100 * fragment.age + index;
					}));
				}
				//>>excludeEnd("PExclude");


				// now lets remove, try the oldest fragments first
				for (i = orderedByAge.length - 1; i >= 0 && toRemove > 0; i--) {
					var fragment = orderedByAge[i];
					if (fragment !== initialFragment && fragment.isFetched() && !fragment.isDirty() && !_.contains(finalFragments, fragment)) {
						//>>excludeStart("PExclude", pragmas.productionExclude);
						if (DEBUG) {
							this.f.debug.debug("REMOVE fragment with age", fragment.age);
						}
						//>>excludeEnd("PExclude");
						this._removeFragment(fragment);
						toRemove--;
					}
				}
			}


			//>>excludeStart("PExclude", pragmas.productionExclude);
			if (DEBUG) {
				if (before > this.fragments.length) {
					this.f.debug.debug("CLEANED fragments: #", before - this.fragments.length);
				}

				this.f.debug.debug("drawing fragments: #", finalFragments.length);
			}
			//>>excludeEnd("PExclude");

			return finalFragments;
		};



		/**
		 * Get the URL for a fragment
		 * @param {Utils.Fragment} fragment - the fragment for which you want the url
		 * @returns {String}
		 */
		ViewportFragmentManager.prototype.getUrlForFragment = function (fragment) {
			var url = this.delegate.getURLForFragment(fragment);

			//>>excludeStart("PExclude", pragmas.productionExclude);
			if (DEBUG) {
				this.f.debug.debug("Fetching url: " + url);
			}
			//>>excludeEnd("PExclude");

			return url;
		};

		/**
		 * A fragment was fetched, clean up dirty frames and inform the delegate
		 */
		ViewportFragmentManager.prototype._fragmentFetched = function () {
			this.cleanupDirtyFragments();
			this.delegate.fragmentsChanged();
		};

		/**
		 * Remove a fragment
		 * @private
		 * @param {Utils.Fragment} fragmentToBeRemoved
		 */
		ViewportFragmentManager.prototype._removeFragment = function (fragmentToBeRemoved) {
			var index = _.indexOf(this.fragments, fragmentToBeRemoved);
			if (index >= 0) {
				this.removeFragmentAtIndex(index);
			}
		};

		/**
		 * Remove a fragment at a certain index
		 * @param index
		 */
		ViewportFragmentManager.prototype.removeFragmentAtIndex = function (index) {
			this.fragments.splice(index, 1);
		};

		/**
		 * Clean out all dirty fragments if there is at least 1 fetched non-dirty fragment
		 */
		ViewportFragmentManager.prototype.cleanupDirtyFragments = function () {
			var cleanFragments = _.filter(this.fragments, function (fragment) {
				return !fragment.isDirty();
			});

			// if the cleanFragments and fragments differ in length, we should try to clean
			if (cleanFragments.length != this.fragments.length) {
				// only clean up if there is at least one fetched clean fragment available
				if (_.some(cleanFragments, function (fragment) {
						return fragment.isFetched();
					})) {
					this.fragments = cleanFragments;
				}
			}
			this.dirtyInitialFragment = null;
		};

		/**
		 * Mark all available fragments as dirty
		 */
		ViewportFragmentManager.prototype.markAllAsDirty = function () {
			//first clean up
			this.cleanupDirtyFragments();
			_.each(this.fragments, function (fragment) {
				fragment.makeDirty();
			});
			this.dirtyInitialFragment = this.initialFragment;
			if (this.dirtyInitialFragment) {
				this.dirtyInitialFragment.makeDirty();
			}

			this.initialFragment = null;
		};

		/**
		 * Cleanup all fragments
		 */
		ViewportFragmentManager.prototype.cleanupFragments = function () {
			this.fragments = [];
			this.initialFragment = null;
			this.dirtyInitialFragment = null;
		};


		/**
		 * Get the document rectangle, but make sure the width and height are integer numbers in the pixel domain
		 * @returns {ShapeUtils.Rect} the document rect
		 */
		ViewportFragmentManager.prototype.calculateRoundedWorldBounds = function () {
			var worldBounds = this.world.getWorldBounds();
			var zoom = this.world.getInitialStartZoom();
			// make sure width/height are integer values in the pixel domain
			worldBounds.size.width = Math.floor(worldBounds.size.width * zoom) / zoom;
			worldBounds.size.height = Math.floor(worldBounds.size.height * zoom) / zoom;
			return worldBounds;
		};

		/**
		 * Update the initial fragment if the world has changed or no initial fragment is set
		 */
		ViewportFragmentManager.prototype.updateInitialFragment = function () {
			var worldBounds = this.calculateRoundedWorldBounds();
			if (this.world.getInitialStartZoom() > 0 &&
				(!this.initialFragment ||
					(!thresholdEquals(this.initialFragment.getZoom(), this.world.getInitialStartZoom()) ||
						!this.initialFragment.getRect().rectEquals(worldBounds)))) {


				//>>excludeStart("PExclude", pragmas.productionExclude);
				if (DEBUG) {
					this.f.debug.debug("Fetching initial fragment");
				}
				//>>excludeEnd("PExclude");
				this.dirtyInitialFragment = this.initialFragment;

				this.initialFragment = new Fragment(worldBounds, this.world.getInitialStartZoom());
				this.initialFragment.isInitial = true;
				this.initialFragment.fetchImage(this.fragmentCache,
					this.getUrlForFragment(this.initialFragment),
					_.bind(this._fragmentFetched, this),
					_.bind(function () {
						this.initialFragment = null;
					}, this)
				);
			}
		};

		return ViewportFragmentManager;
	});

define('web-viewer/viewport/ViewportLayer',["jquery",
		"underscore",
		"web-viewer/utils/DrawUtils",
		"utils/ShapeUtils",
		"utils/Transformation",
		"web-viewer/utils/ListenerUtil",
		"web-viewer/utils/Fragment",
		"web-viewer/utils/ConversionConstants",
		"web-viewer/viewport/model/ViewportFragmentManager",
		"utils/HTTPUtils",
		"component/popover/Popover"
	],
	function ($,
		_,
		DrawUtils,
		ShapeUtils,
		Transformation,
		ListenerUtil,
		Fragment,
		ConversionConstants,
		ViewportFragmentManager,
		HTTPUtils,
		Popover) {
		"use strict";

		/**
		 * Rendering speed
		 * @constant
		 */
		var RENDER_SPEED = 40;

		/**
		 * Create a ViewportLayer
		 *
		 * @exports ViewportLayer
		 * @memberof ViewportModule
		 * @class A layer is responsible to render the content of one specific part of a view
		 * @constructor
		 * @mixes Utils.ListenerUtilMixin
		 *
		 * @param {Core.ViewerFacade} f
		 * @param {ViewportModule.viewportView} viewportView - view of the layer
		 * @param {ViewportModule.ViewportWorld} world - the world of the Viewport
		 * @param {Transformation} viewTransform - transformation dictated by the view
		 * @param {String} id - the ID of the Layer
		 */
		var ViewportLayer = function (f, viewportView, world, viewTransform, id) {
			this.f = f;
			ListenerUtil.mixin(this, f);

			/**
			 * Unique ID of the canvas
			 * this also the ID in the DOM
			 * @protected
			 */
			this._id = id;

			/**
			 * Main canvas object that is used for rendering
			 * @protected
			 */
			this._canvas = $("<canvas>").css({
				'position': 'absolute',
				'top': 0,
				'left': 0
			}).attr("id", this._id);


			/**
			 * The layerModel keeps track of the content of the layer
			 * @protected
			 * @type {ViewportModule.ViewportLayerModel}
			 */
			this._layerModel = null;

			/**
			 * The view this is layer is part of
			 * @private
			 */
			this._view = viewportView;

			/**
			 * @protected
			 * @type {Boolean}
			 */
			this._visible = true;
			this._canvasIsShown = true;

			/**
			 * This keeps track of all listeners registered to the model
			 */
			this._listeners = [];

			/**
			 * The world for the ViewPort
			 * @private
			 */
			this._world = world;

			/**
			 * The currently used transformation
			 * @private
			 */
			this._transform = viewTransform;

			/**
			 * Is the buffer enabled?
			 * NOTE this buffer doesn't work correctly at the moment, should be repaired if it is necessary to have it enabled
			 * @private
			 */
			this._enableBuffer = false;

			/**
			 * Should the current linework be buffered (when the buffer is enabled)
			 * @private
			 */
			this._shouldBuffer = false;

			/**
			 * Buffer image canvas
			 * @private
			 */
			this._buffer = null;

			/**
			 * Debug the buffer functionality?
			 * @private
			 */
			this._debugBuffer = false;

			/**
			 * Used to now whether some ImageDataRequest should be send out (this is to now whethere all fragments are loaded)
			 * @protected {Boolean}
			 */
			this._dirty = false;


			/**
			 * Interactive mode
			 * @private {Boolean}
			 */
			this._interactive = false;
			/**
			 * Render the layer in a throttled way
			 */
			this.throttledRender = _.throttle(_.bind(this.render, this), RENDER_SPEED);

			/**
			 * Repaint the layer in a throttled way
			 */
			this.debouncedRepaint = _.debounce(_.bind(function () {
				var wasDirty = this._dirty;
				this._dirty = false;
				this.preloadViewportFragment();
				this.throttledRender();

				if (wasDirty) {
					this.f.emit("viewportLayer.finishedImageDataRequest", this._id);
				}
			}, this), 40);

			/**
			 * This manages the fragments shown in
			 * @protected
			 */
			this._fragmentManager = new ViewportFragmentManager(this.f, {
				getURLForFragment: _.bind(function (fragment) {
					return this.generateFragmentURL(fragment);
				}, this),
				fragmentsChanged: this.throttledRender
			}, this._world);


			// extra overwrite options on top of url parameters
			this._overwriteOptions = {};

		};

		/**
		 * Get the layerModel of the layer
		 * @returns {ViewportModel.ViewportLayerModel}
		 */
		ViewportLayer.prototype.getLayerModel = function () {
			return this._layerModel;
		};

		/**
		 * Set the layermodel. This will register listeners to the model and unregister listeners to the old model
		 * @param layerModel
		 */
		ViewportLayer.prototype.setLayerModel = function (layerModel) {
			this._unregisterAllListeners();

			this._layerModel = layerModel;


			this._registerListener("modelChanged", this.onLayerModelChange, this, this._layerModel);
			this._registerListener("popoverAdded", this.onPopoverAdded, this, this._layerModel);
			this._registerListener("popoverRemoved", this.onPopoverRemoved, this, this._layerModel);
			this._registerListener("repaint", this.repaint, this, this._layerModel);
			this._registerListener("fragmentsDirty", this.markAllFragmentsAsDirty, this, this._layerModel);
			this._registerListener("cleanupFragments", this.cleanupFragments, this, this._layerModel);
			this._registerListener("overwriteOption.**", this.markAllFragmentsAsDirty, this, this._layerModel);
			this._registerListener("changed.visible", function () {
				this.repaint();
			}, this, this._layerModel);

			this.repaint();
		};

		/**
		 * Destroy the layer (unregister listeners, remove popups/popovers
		 */
		ViewportLayer.prototype.destroy = function () {
			this._unregisterAllListeners();

			this.hideAllPopovers();
			this._canvas.remove();
		};

		/**
		 * When the layerModel has changed, rerender the layer
		 */
		ViewportLayer.prototype.onLayerModelChange = function () {
			this.throttledRender();
		};




		/**************************************************************
		 *LAYER SETTINGS
		 **************************************************************/

		/**
		 * Start interactive mode
		 * All fetching is disabled, buffering is used (if enabled)
		 * All popovers and popups are hidden
		 */
		ViewportLayer.prototype.startInteractive = function () {
			this._shouldBuffer = true;
			this._interactive = true;
			this.hideAllNonInteractivePopovers();
			this.hideAllPopups();
		};

		/**
		 * Stop interactive mode
		 * Re-enable fragment fetching
		 * Stop buffering
		 * repaint
		 */
		ViewportLayer.prototype.stopInteractive = function () {
			this._shouldBuffer = false;
			this._interactive = false;

			if (!this._world.isAnimating()) {
				this.drawPopovers();
				this.drawPopups();
				this.preloadViewportFragment();
			}
		};

		/**
		 * Hide/show the layer
		 * @param {Boolean} hide - hide/show the layer
		 */
		ViewportLayer.prototype.setVisible = function (visible) {
			this._visible = visible;
			this.repaint();
		};

		/**
		 * Set the size for the canvas
		 * @param {ShapeUtils.Size} dim - dimensions of the canvas
		 */
		ViewportLayer.prototype.setSize = function (dim) {
			if (this._canvas[0].width != dim.width || this._canvas[0].height != dim.height) {
				this._canvas[0].width = dim.width;
				this._canvas[0].height = dim.height;

				this.render(); // we have to render immediately otherwise there could be flickering (when size of canvas is set, all pixels are reset)
			}
		};

		/**
		 * @returns {JQueryObj} the canvas
		 */
		ViewportLayer.prototype.getCanvas = function () {
			return this._canvas;
		};

		/**
		 * Get the drawing context of the layer
		 * @returns {Context2D}
		 */
		ViewportLayer.prototype.getCanvasContext = function () {
			return this._canvas[0].getContext("2d");
		};

		/**
		 * Set the overwrite options for the layer (not the model)
		 * @param {Object}  options
		 */
		ViewportLayer.prototype.setOverwriteOptions = function (options) {
			this._overwriteOptions = options;
			this.markAllFragmentsAsDirty();
		};

		/**
		 * Is the layer visible (this checks both the setting in the layer and in the model)
		 * @returns {Boolean}
		 */
		ViewportLayer.prototype.isVisible = function () {
			return this._visible && (!this._layerModel || this._layerModel.isVisible());
		};

		/**************************************************************
		 * CALCULATIONS/MEASUREMENTS
		 **************************************************************/

		/**
		 * Calculate the closest linework given a certain point
		 * @param {ShapeUtils.Point} point - Point that needs to be verified
		 * @param {Number} [fatfinger] - Optional parameter to be able to work with fat fingers :) it searches less strict (with an diameter of fatfinger)
		 * @returns {LineworkWrapper}  linework - Linework that is closest to the point. null if no object was found
		 */
		ViewportLayer.prototype.getClosestLineworkFromPoint = function (point, fatfinger) {
			if (this._layerModel) {
				var closest = null;
				var closestDist = -1;
				var t = this.getTransformMatrix();
				var viewRect = this._world.getWorldRectFromScene(this._world.getSceneBounds(), t);

				var linework = this._layerModel.getLinework();
				for (var i = 0; i < linework.length; i++) {
					var wrap = linework[i];
					if (wrap.rect && wrap.rect.size.width > 0 && wrap.rect.size.height > 0) {
						//check if the rect is actually in the viewport
						if (!viewRect.rectIntersect(wrap.rect)) {
							continue;
						}
					}
					var dist = DrawUtils.distanceOfScenePoint(wrap, point, t, fatfinger);
					if (dist > 0 && (closestDist < 0 || dist <= closestDist)) {
						closest = wrap;
						closestDist = dist;
					}
				}
				return closest;
			}
		};

		/**
		 * Calculate the closest point given a certain point
		 * @param {ShapeUtils.Point} refpoint - Point that needs to be verified
		 * @returns {ShapeUtils.Point} point - Point that is closest to the point. null if no object was found
		 */
		ViewportLayer.prototype.getClosestLineworkPointFromPoint = function (refpoint, distance) {
			var closest = null;
			if (this._layerModel !== null) {
				var transform = this.getTransformMatrix();
				var viewRect = this._world.getWorldRectFromScene(this._world.getSceneBounds(), transform);
				var size = new ShapeUtils.Size(distance, distance);


				var referenceRect = new ShapeUtils.Rect(new ShapeUtils.Point(refpoint.x - (distance / 2), refpoint.y - (distance / 2)), size);
				var referencePointInWorld = this.getWorldPointFromScene(refpoint);
				var referenceRectInWorld = this.getWorldRectFromScene(referenceRect);

				var linework = this._layerModel.getLinework();
				for (var i = 0; i < linework.length; i++) {
					var wrap = linework[i];

					if (wrap.snapable && wrap.pointArray && wrap.pointArray.length > 0) {
						// the wrapper is snappable and there is a pointarray with some points in it

						// if there is a rectangle, we only want to find the closest point if the rectangle is in the visible area
						if (!(wrap.rect && wrap.rect.size.width > 0 && wrap.rect.size.height > 0 && !viewRect.rectIntersect(wrap.rect))) {
							var points = wrap.pointArray;
							for (var j = 0; j < points.length; j++) {
								var lwPoint = points[j];
								if (referenceRectInWorld.containsPoint(lwPoint)) {
									var candidate = new ShapeUtils.Point(lwPoint.x, lwPoint.y);
									if (!closest || candidate.getDistance(referencePointInWorld) < closest.getDistance(referencePointInWorld)) {
										closest = candidate;
									}
								}
							}
						}
					}
				}
			}

			if (closest) {
				return this.getScenePointFromWorld(closest);

			}
			return null;
		};


		/**
		 * Get the World-coordinates for a scene point
		 * @param {ShapeUtils.Point} point - Scene point
		 * @returns {ShapeUtils.Point} World point
		 */
		ViewportLayer.prototype.getWorldPointFromScene = function (point) {
			return this._world.getWorldPointFromScene(point, this.getTransformMatrix());
		};

		/**
		 * Get the World-coordinates for a scene rect
		 * @param {ShapeUtils.Rect} rect - Scene rect
		 * @returns {ShapeUtils.Rect} World rect
		 */
		ViewportLayer.prototype.getWorldRectFromScene = function (rect) {
			return this._world.getWorldRectFromScene(rect, this.getTransformMatrix());
		};

		/**
		 * Get the Scene-coordinates for a world rect
		 * @param {ShapeUtils.Rect} rect - World rect
		 * @returns {ShapeUtils.Rect} Scene rect
		 */
		ViewportLayer.prototype.getSceneRectFromWorld = function (rect) {
			return this._world.getSceneRectFromWorld(rect, this.getTransformMatrix());
		};

		/**
		 * Get the Scene-coordinates for a world point
		 * @param {ShapeUtils.Point} point - World point
		 * @returns {ShapeUtils.Point} Scene point
		 */
		ViewportLayer.prototype.getScenePointFromWorld = function (point) {
			return this._world.getScenePointFromWorld(point, this.getTransformMatrix());
		};

		/**
		 * Function to get the layer transformation within the world
		 * @returns {Transformation}
		 */
		ViewportLayer.prototype.getLayerTransform = function () {
			var l = this._layerModel.getTransform().copy();

			var t = new Transformation();

			var offsetX = this._world.getWorldSize().width / 2 - l.e;
			var offsetY = this._world.getWorldSize().height / 2 - l.f;

			t.translate(offsetX, offsetY);
			t.concatenate(l);
			t.translate(-offsetX, -offsetY);

			return t;
		};

		/**
		 * Function to convert a rect from a layer to world coordinates
		 * @returns {ShapeUtils.Rect}
		 */
		ViewportLayer.prototype.convertLayerRectToWorld = function (rect) {
			return this.getLayerTransform().transformedRect(rect);
		};

		/**
		 * Function to convert a world rect to layer coordinates
		 * @returns {ShapeUtils.Rect}
		 */
		ViewportLayer.prototype.convertWorldRectToLayer = function (rect) {
			return this.getLayerTransform().createInverse().transformedRect(rect);
		};

		/**
		 * Function to convert a point from a layer to world coordinates
		 * @returns {ShapeUtils.Point}
		 */
		ViewportLayer.prototype.convertLayerPointToWorld = function (point) {
			return this.getLayerTransform().transformedPoint(point);
		};

		/**
		 * Function to convert a world point to layer coordinates
		 * @returns {ShapeUtils.Point}
		 */
		ViewportLayer.prototype.convertWorldPointToLayer = function (point) {
			return this.getLayerTransform().createInverse().transformedPoint(point);
		};

		/**
		 * Calculate the matrix that is needed to draw objects on the canvas
		 * @returns {Transformation} the transformation that needs to be applied for this layer
		 */
		ViewportLayer.prototype.getTransformMatrix = function () {
			var t = new Transformation();

			if (this._layerModel) {
				/* Layer transform */
				var layerworld = this._layerModel.getTransform();

				t.preConcatenate(layerworld);
			}

			/* View transform */
			var offsetX = this._world.getWorldSize().width / 2 - this._transform.e;
			var offsetY = this._world.getWorldSize().height / 2 - this._transform.f;

			t.preTranslate(-offsetX, -offsetY);
			t.preConcatenate(this._transform);
			t.preTranslate(offsetX, offsetY);

			/* World transform */
			t.preConcatenate(this._world.getTransformation());

			return t;
		};

		/**
		 * @returns {float} the current zoom of the layer
		 */
		ViewportLayer.prototype.getZoom = function () {
			var t = this._world.getTransformation();
			return Math.sqrt(t.c * t.c + t.d * t.d);
		};


		/**************************************************************
		 * FRAGMENT management
		 **************************************************************/

		/**
		 * Add/preload a fragment on the viewportfragmenetmanager
		 * @param {ShapeUtils.Rect} viewrect - Rect to fetch from the viewserver
		 * @param {float} zoom - zoom/resolution of the fragment to fetch
		 */
		ViewportLayer.prototype.addFragment = function (bounds, zoom) {
			if (isNaN(zoom)) {
				return;
			}

			var maxr = this._layerModel.getMaximumResolution();
			if (maxr >= 0) {
				zoom = Math.min(maxr, zoom);
			}

			this._fragmentManager.preloadFragment(bounds, zoom, true);
		};

		/**
		 * Generate the url parameters for a certain fragment
		 * @param {Utils.Fragment} fragment - the fragment for which the URL is generated
		 * @returns {String}
		 */
		ViewportLayer.prototype.generateFragmentURL = function (fragment) {
			var context = this._layerModel.getContextForFragment(fragment);
			var colorMask = context.params.color;

			//move the diff parameter to the file parameter, ViewerServer doesn't like undefined for file (current file is blank)
			if (context.params.file === undefined && context.params.diff !== undefined) {
				context.params.file = context.params.diff;
				delete context.params.diff; //make sure to delete the difference file (this is blank)
				context.params.rs = context.params.rs == 1 ? 2 : 1; //switch the location of the 'active' page
			}
			context.params = _.extend(context.params, this._overwriteOptions);
			delete context.params.colorBITWISE;
			if (this._overwriteOptions.colorBITWISE !== undefined) {
				if (colorMask !== undefined) {
					// color parameter is overriden by bitwise and
					colorMask = context.params.color & this._overwriteOptions.colorBITWISE;
				} else {
					colorMask = this._overwriteOptions.colorBITWISE;
				}
				context.params.color = colorMask;
			}

			// no need to send isDefaultView to the server
			delete context.params.isDefaultView;

			// generate URL
			var url = HTTPUtils.constructURL(context.servlet, context.params);

			// ViewServer can't handle urlencoded commas, fix commas 
			url = url.replace(/%2C/g, ",");

			return url;
		};

		/**
		 * Get fragments that need to be rendered
		 * @param {ShapeUtils.Rect} viewrect - frame that needs to be fetched
		 * @param {float} resolution - resolution of the layer
		 * @returns the fragments that are relevant for the viewrect and the resolution
		 */
		ViewportLayer.prototype.getApplicableFragments = function (viewrect, resolution) {
			return this._fragmentManager.getApplicableFragments(viewrect, resolution);
		};

		/**
		 * Mark all loaded fragments as dirty
		 */
		ViewportLayer.prototype.markAllFragmentsAsDirty = function () {
			if (!this._dirty) {
				this._dirty = true;
				if (this.canFetchFragments()) {
					this.f.emit("viewportLayer.startImageDataRequest", this._id);
				}
			}
			this._fragmentManager.markAllAsDirty();
			this.repaint();
		};

		/**
		 * Cleanup all fragments
		 */
		ViewportLayer.prototype.cleanupFragments = function () {
			this._fragmentManager.cleanupFragments();
			this.repaint();
		};

		/**************************************************************
		 * RENDERING
		 **************************************************************/

		/**
		 * Repaint the layer, it is possible to set a new world and transformation
		 * @param {ViewportModule.ViewportWorld} [world] - the world of the Viewport
		 * @param {Transformation} [viewTransform] - transformation dictated by the view
		 */
		ViewportLayer.prototype.repaint = function (world, transform) {
			if (world) {
				this._world = world;
			}

			if (transform) {
				this._transform = transform;
			}

			if (this._canvasIsShown != this.isVisible()) {
				this._canvasIsShown = this.isVisible();
				this._canvas.toggle(this._canvasIsShown);
			}

			this.debouncedRepaint();
			this.throttledRender();
		};

		/**
		 * Is the layer allowed to fetch fragments?
		 * @returns {Boolean}
		 */
		ViewportLayer.prototype.canFetchFragments = function () {
			return this._layerModel !== null &&
				!this._interactive &&
				this.isVisible() &&
				this._world !== undefined &&
				this._world !== null &&
				this._layerModel.hasGraphics() &&
				this._world.getZoom() > 0 &&
				this.f.canFetchFragments();
		};

		/**
		 * Preload the fragments to draw the document with the current world settingss
		 */
		ViewportLayer.prototype.preloadViewportFragment = function () {
			if (this.canFetchFragments()) {
				var zoom = this._world.getZoom();
				var targetViewport = this._world.getAnimationDestinationTargetViewPort();

				// make sure we are requesting an image with integer values for width/height in the pixel domain
				targetViewport.size.width = Math.floor(targetViewport.size.width * zoom) / zoom;
				targetViewport.size.height = Math.floor(targetViewport.size.height * zoom) / zoom;


				if (!this._layerModel.shouldAutoFetchFragments()) {
					// let the model load the fragment
					this._layerModel.preloadFragment(targetViewport, zoom);
				} else {
					// autoload the fragment in the Layer
					this.addFragment(targetViewport, zoom);

					if (!this._world.isAnimating()) {
						this._fragmentManager.updateInitialFragment();
					}
				}
			}
		};

		/**
		 * Zooming started, this is used for interactive zooming (pinching)
		 */
		ViewportLayer.prototype.zoomingStarted = function () {
			this._isZooming = true;
		};

		/**
		 * Zooming stopped, used for interactive zooming (pinching)
		 */
		ViewportLayer.prototype.zoomingStopped = function () {
			this._isZooming = false;
		};

		/**
		 * Calculate the buffer for the linework (if any) of this layer if panning started
		 */
		ViewportLayer.prototype.panningStarted = function () {
			if (this._layerModel !== null && this._layerModel.hasLinework(this._view.getPanel().getModel().getName()) && !this._layerModel.hasBuffer(this._view.getPanel().getModel().getName()) && this._canvasIsShown) {
				var z = this._world.getZoom();
				var rect = this._world.getTargetViewPort(true);
				var rotation = this._world.getRotation();
				var isFlipped = this._world.isFlip();
				var isViewFromBack = this._world.isFlipViewFromBack(); // is flipped view from back
				if (isViewFromBack) {
					if (rotation === 90 || rotation === 270) {
						rotation = (rotation + 180) % 360;
					}
					isFlipped = !isFlipped;
				}

				rotation = (360 + rotation) % 360;

				//we'll have to calculate a new origin when the file is rotated from 0 to 90 degrees
				//and the size of the rect of the fragment needs to change when rotation is 90 or 270
				var width = rect.size.width;
				var height = rect.size.height;
				if (rotation % 360 === 270) {
					if (height > width && !isFlipped) {
						rect.origin.x -= (height - width);
					}
					rect.size = new ShapeUtils.Size(height, width);
				} else if (rotation % 360 === 90) {
					if (width > height) {
						rect.origin.y += (height - width); //needs to be added to origin
					} else if (height > width && isFlipped) {
						rect.origin.x -= (height - width);
					}
					rect.size = new ShapeUtils.Size(height, width);
				}

				//construct fragment for linework			
				var fragment = new Fragment(rect, z);
				var sceneBounds = this._world.getSceneBounds();
				//we'll rotate the image so the world and the layer can use the correct rotation this way the transformation don't have to be altered	
				//first create a canvas element and make sure there is enough space to rotate the image (nothing gets lost)
				var canvas = document.createElement("canvas");
				var squareSize = Math.max(sceneBounds.size.width, sceneBounds.size.height);
				if (rotation % 360 === 90 || rotation % 360 === 270) {
					canvas.width = squareSize;
					canvas.height = squareSize;
				} else {
					canvas.width = sceneBounds.size.width;
					canvas.height = sceneBounds.size.height;
				}

				//do the rotation
				var context = canvas.getContext("2d");
				context.translate(canvas.width / 2, canvas.height / 2);
				if (isFlipped) {
					context.scale(-1, 1);
				}
				context.rotate((360 - rotation % 360) / 180 * Math.PI);

				context.translate(-canvas.width / 2, -canvas.height / 2);
				context.drawImage(this._canvas[0], 0, 0); //draw the canvas, prevent performance issues by not creating images

				//set the canvas in the fragment as image that will be used as buffer
				fragment.image = canvas;
				fragment.fetched = true;

				if (this._layerModel.setBuffer !== undefined) {
					this._layerModel.setBuffer(fragment);
				}
			}
		};

		/**
		 * Remove the buffer for lineworks (if any)
		 */
		ViewportLayer.prototype.panningStopped = function () {
			if (this._layerModel !== null && this._layerModel.hasLinework(this._view.getPanel().getModel().getName()) && this._layerModel.setBuffer !== undefined) {
				this._layerModel.setBuffer(null);
				this.throttledRender();
			}
		};

		/**
		 * Render the layer
		 * !! note this method should be avoided, use throttledRender() instead!
		 */
		ViewportLayer.prototype.render = function () {
			if (!this._layerModel ||
				!this._world ||
				!this._transform ||
				(this._world.getWorldSize().width === 0 && this._world.getWorldSize().height === 0) ||
				!this.isVisible()) {

				// we do not want to draw if there is no this._layerModel, this._world, this._transform
				// or if the worldsize is 0x0
				// or if we are hidden

				return;
			}

			// hide all popovers/popups when an animation is ongoing
			if (this._world.isAnimating()) {
				this.hideAllPopovers();
				this.hideAllPopups();
			}

			// get the canvas drawing context
			var ctx = this.getCanvasContext();

			// set the default for the context
			ctx.setTransform(1, 0, 0, 1, 0, 0);

			// clear canvas
			ctx.clearRect(0, 0, this._canvas.width(), this._canvas.height());

			var t = this.getTransformMatrix();
			ctx.transform(t.a, t.b, t.c, t.d, t.e, t.f);

			//get the current zoom level, depends on the state of the transformation
			var zoom = t.getScaleX();

			// calculate the rectangle in which can be drawn
			var drawrect = this._world.getWorldRectFromScene(this._world.getSceneBounds(), t);

			// render
			this._renderBackground(ctx);
			this._renderFragments(ctx, drawrect, zoom);
			//make sure the zoom is depending on the real zoom so make sure the initial zoom isn't involved 
			this._renderLinework(ctx, drawrect, zoom, t);

			this._layerModel.render(ctx, drawrect, zoom, t);
			// show popups if the world is not animating
			if (!this._world.isAnimating()) {
				this.drawPopovers();
				this.drawPopups();
			}
		};


		/**
		 * Draw the background as it is defined in the canvas
		 * @param ctx
		 */
		ViewportLayer.prototype._renderBackground = function (ctx) {
			var bgoptions = this._layerModel.getBackgroundOptions();
			if (_.size(bgoptions) > 0) {
				ctx.save();

				for (var k in bgoptions) {
					if (bgoptions.hasOwnProperty(k)) {
						ctx[k] = bgoptions[k];
					}
				}
				var bounds = this._world.getWorldBounds();
				var tr = this._layerModel.getTransform();
				var layerTransform = tr.createInverse();
				bounds = layerTransform.transformedRect(bounds);
				ctx.fillRect(bounds.origin.x, bounds.origin.y, bounds.size.width, bounds.size.height);

				ctx.restore();
			}
		};

		/**
		 * Render all fragments on the canvas
		 * @param {Context2D} ctx
		 * @param {ShapeUtils.Rect} drawrect
		 * @param {Number} zoom
		 */
		ViewportLayer.prototype._renderFragments = function (ctx, drawrect, zoom) {
			var fragments;
			if (this._layerModel.hasGraphics()) {
				if (!this._layerModel.shouldAutoFetchFragments()) {
					// rely on the model for the fragments
					fragments = this._layerModel.fragments(drawrect, zoom);
				} else {
					// auto fetching is enabled, get the fragments from the layer 
					fragments = this.getApplicableFragments(drawrect, zoom);
				}

				// pixel perfect drawing, requires us to draw in pixel domain, not in the document domain (mm)

				// first store the current transformation
				ctx.save();

				// scale to pixel coordinates
				zoom = this._world.getZoom();
				ctx.scale(1 / zoom, 1 / zoom);

				// draw fragments in pixel domain
				for (var i = 0; i < fragments.length; i++) {
					var fragment = fragments[i];

					if (fragment.isFetched()) {
						var fragmentRect = fragment.getRect();

						var x = Math.floor(fragmentRect.origin.x * zoom);
						var y = Math.floor(fragmentRect.origin.y * zoom);
						var width = Math.ceil(fragmentRect.size.width * zoom);
						var height = Math.ceil(fragmentRect.size.height * zoom);

						if (Math.abs(fragment.getZoom() - zoom) < 0.0005) {
							// we are drawing in the same zoomlevel, draw at the natural height/width of the fragment
							width = fragment.getNaturalWidth();
							height = fragment.getNaturalHeight();
						}
						ctx.drawImage(fragment.image, x, y, width, height);
					}
				}

				// restore the transformation
				ctx.restore();

				/* un-uncomment for drawing rectangles around fragments (can be used easily see which fragments are drawn) 
			 * normal fragments have red border + fill
			 * perfect fit is shown in green
			 * dirty fragments have blue border + fill
			 * in the corner of every fragment the draw sequence number is shown
			 *
			for(var i=0; i<fragments.length; i++){
				var fragment = fragments[i];
				if (fragment.isFetched()){
					
					var fragmentRect = fragment.getRect();
		            ctx.beginPath();
		            ctx.rect(fragmentRect.origin.x, fragmentRect.origin.y, fragmentRect.size.width, fragmentRect.size.height);
		            ctx.lineWidth = 2/zoom;
		            if(fragment.isInitial){
		            	ctx.strokeStyle = 'yellow';
		            	ctx.fillStyle = "rgba(0, 255, 255, 0.2)";
		            } else {
		            	 if (fragment.perfectFit){
			            	ctx.strokeStyle = 'green';
			            	ctx.fillStyle = "rgba(0, 255, 0, 0.2)";
			            	fragment.perfectFit = false;
			            } else if (fragment.isDirty()){
			            	ctx.strokeStyle = 'blue';
			            	ctx.fillStyle = "rgba(0, 0, 255, 0.1)";
			            } else {
			            	ctx.strokeStyle = 'red';
			            	ctx.fillStyle = "rgba(255, 0, 0, 0.1)";
			            }
		            }
		            
		           
		            
		            ctx.fill();
		            ctx.stroke();	 
		            ctx.font = (10 / zoom) +"pt Verdana";
		            ctx.fillStyle = "black";
		            ctx.fillText(i, fragmentRect.origin.x + (2 / zoom)  , fragmentRect.origin.y + (12 / zoom));
		            
				}
			}
			/* */


			}
		};

		/**
		 * Render all the linework (use a buffer if needed)
		 * @param {Context2D} ctx
		 * @param {ShapeUtils.Rect} drawrect
		 * @param {Number} zoom
		 * @param {Transformation} t - transformation for this layer
		 */
		ViewportLayer.prototype._renderLinework = function (ctx, drawrect, zoom, t) {
			if ((!this._layerModel.hasBuffer(this._view.getPanel().getModel().getName()) || !this._layerModel.hasLinework(this._view.getPanel().getModel().getName()))) { //only don't draw linework if CAD and panning	

				var linework = this._layerModel.getLinework(this._world.isAnimating(), this._world.isAnimatingRotation(), drawrect, this._view.getModel().getID(), this._view.getPanel().getModel().getName(), this._isZooming);

				if (linework && linework.length > 0) {
					// reset the buffer
					this._buffer = null;

					DrawUtils.drawOverlay(this.f, ctx, linework, drawrect, _.bind(this.throttledRender, this), zoom, t, this._layerModel.getLineworkColor(), this._layerModel.getAlpha());

					//>>excludeStart("PExclude", pragmas.productionExclude);
					if (this._debugBuffer) {
						ctx.fillStyle = 'rgba(1, 0, 0, 0.4)';
						ctx.fillRect(drawrect.origin.x, drawrect.origin.y, drawrect.size.width, drawrect.size.height);
					}
					//>>excludeEnd("PExclude");

				}

			} else if (this._layerModel.hasLinework(this._view.getPanel().getModel().getName()) && this._layerModel.hasBuffer(this._view.getPanel().getModel().getName())) {
				//if there is Cad and a a buffer render the buffer
				this._renderBuffer(ctx, zoom, t);
			}
		};

		/**
		 * Renders the buffer of the layer if there is a buffer (used for buffer of lineworks)
		 * @param {Context2D} ctx
		 * @param {Number} zoom
		 * @param {Transformation} t - transformation for this layer
		 */
		ViewportLayer.prototype._renderBuffer = function (ctx, zoom, t) {
			var fragment = this._layerModel.getBuffer();

			if (fragment !== null) {
				ctx.save();
				zoom = this._world.getZoom();

				t = this._world.getTransformation();
				ctx.setTransform(1, 0, 0, 1, 0, 0);
				ctx.transform(t.a, t.b, t.c, t.d, t.e, t.f);
				ctx.scale(1 / zoom, 1 / zoom);

				if (fragment.isFetched()) {
					var fragmentRect = fragment.getRect();

					var x = fragmentRect.origin.x * zoom;
					var y = fragmentRect.origin.y * zoom;
					var width = fragmentRect.size.width * zoom;
					var height = fragmentRect.size.height * zoom;

					var imageCanvas = fragment.getImage();
					if (Math.abs(fragment.getZoom() - zoom) < 0.0005) {
						// we are drawing in the same zoomlevel, draw at the natural height/width of the fragment
						width = imageCanvas.width;
						height = imageCanvas.height;
					}
					ctx.drawImage(imageCanvas, x, y, width, height);
				}

				ctx.restore();
			}
		};


		/**************************************************************
		 * POPOVERS
		 **************************************************************/

		/**
		 * A popover was added
		 * @param {String} id - the id of the popover
		 */
		ViewportLayer.prototype.onPopoverAdded = function (id) {
			var popoverModel = this._layerModel.getPopoverModel(id);

			//first check if the the popover is already added to the layer model:
			if (this._layerModel.getPopover(id) === undefined) {
				var popoverObj = {};

				// Create a div for the popover
				var popoverDiv = $("<div>").css("pointer-events", "none"); // ignore mouse events for the dummy popover div

				// Add the div to the viewport
				this.f.getContext().find(".viewport").append(popoverDiv);
				popoverObj.popoverDiv = popoverDiv;

				// Create the popover object
				var popover = new Popover(popoverDiv, popoverModel.getContext(), this._canvas, popoverModel.getPreferredPosition(), "#fff", popoverModel.getClass(), popoverModel.getRestrictBorder());

				//For IE9 and IE10: no support for pointer-events css!! hide the popover element and emit the event on the element below popover
				popover.element.mouseenter(function (e) {
					$(this).hide();
					var elementBelow = document.elementFromPoint(e.clientX, e.clientY);
					$(this).show();
					$(elementBelow).mouseenter();
				});

				// Add events
				if (popoverModel.onAddedToContainer) {
					popover.onAddedToContainer(popoverModel.onAddedToContainer);
				}
				if (popoverModel.onDestroy) {
					popover.onDestroy(popoverModel.onDestroy);
				}

				// Set content
				popover.setContent(popoverModel.getContent());

				popoverObj.popover = popover;

				// Store it		
				this._layerModel.savePopover(popoverObj, id);

				this.drawPopovers();
			}
		};

		/**
		 * Remove a popover (object)
		 * @param {String} id - the id of the popover
		 */
		ViewportLayer.prototype.onPopoverRemoved = function (id) {
			var popoverObj = this._layerModel.getPopover(id);
			if (popoverObj) {
				popoverObj.popover.hide();
				popoverObj.popover.destroy();
				popoverObj.popoverDiv.remove();

				this._layerModel.removePopover(id);
			}
		};

		/**
		 * Hide all popovers
		 */
		ViewportLayer.prototype.hideAllPopovers = function () {
			if (this._layerModel !== null) {
				_.each(this._layerModel.getAllPopovers(), function (popoverObj) {
					popoverObj.popover.hide();
					popoverObj.popoverDiv.hide();
				});
			}
		};

		/**
		 * Hide all popovers (divs and popover objects) that are not interactive
		 */
		ViewportLayer.prototype.hideAllNonInteractivePopovers = function () {
			if (this._layerModel !== null) {
				_.each(this._layerModel.getAllPopovers(), function (popoverObj, id) {
					var popoverModel = this._layerModel.getPopoverModel(id);
					if (!popoverModel.isInteractive()) {
						popoverObj.popover.hide();
						popoverObj.popoverDiv.hide();
					}
				}, this);
			}
		};

		/**
		 * Redraw all the popovers that are needed to be drawn
		 * (typically because the world changed and the popover needs to be displayed from another rect)
		 */
		ViewportLayer.prototype.drawPopovers = function () {
			// only draw popups if the model is set and the layer is not hidden and the view of this layer is active
			if (this._layerModel && this.isVisible() && this._view.isActive()) {
				_.each(this._layerModel.getAllPopovers(), function (popoverObj, id) {
					popoverObj.popover.restrict = this._canvas; //make sure to restrict the popover to the current view
					popoverObj.popoverDiv.show();

					var popoverModel = this._layerModel.getPopoverModel(id);
					var frame = popoverModel.getFrame();
					if (frame) {
						var r = this._world.getSceneRectFromWorld(frame, this.getTransformMatrix());

						var viewLocation = this._view.getViewLocation(); //get the location of the view in the viewport
						var titleHeight = 0;
						if (this._view.isTitleVisible()) {
							titleHeight = this._view.getTitleHeight();
						}
						/**
						 * Set the position of the popover div, so the popover can be attached to it
						 */
						popoverObj.popoverDiv.css({
							position: 'absolute',
							width: r.size.width + 10 + "px", //10 to make the popover not overlap the tool
							height: r.size.height + 10 + "px",
							top: r.origin.y + viewLocation.y + titleHeight - 5 + "px", //take the box of 10px into account
							left: r.origin.x + viewLocation.x - 5 + "px"
						});
						popoverObj.popover.setPreferredPlacement(popoverModel.getPreferredPosition());

						popoverObj.popover.show();
					}
				}, this);
			}
		};




		/**************************************************************
		 * POPUPS
		 **************************************************************/
		/**
		 * Hide all popups
		 */
		ViewportLayer.prototype.hideAllPopups = function () {
			// only do stuff when we have a layermodel
			if (this._layerModel) {
				_.each(this._layerModel.getPopupModels(), function (popupModel) {
					popupModel.getPopup(this).hide();
				}, this);
			}
		};

		/**
		 * Display all popups
		 */
		ViewportLayer.prototype.drawPopups = function () {
			// only do stuff when we have a layermodel and the layer is shown
			if (this._layerModel && this._canvasIsShown) {
				// get all popup models
				_.each(this._layerModel.getPopupModels(), function (popupModel) {
					var popup = popupModel.getPopup(this);
					if (popupModel.isShown() && !this._interactive) {
						// Convert the world rect to scene rect.
						var sceneOrigin = this.getScenePointFromWorld(popupModel.getWorldOrigin());
						var sceneSize = popupModel.getWorldSize().copy();

						// 96.0/25.4 is a constant to convert world to pixels (for a standard screen PPI)
						sceneSize.width *= ConversionConstants.MM_TO_PX_RATIO;
						sceneSize.height *= ConversionConstants.MM_TO_PX_RATIO;

						// Is there an offset defined in scene pixels?
						var sceneOffset = popupModel.getSceneOffset();
						if (sceneOffset) {
							// add the offset
							sceneOrigin.x += sceneOffset.width;
							sceneOrigin.y += sceneOffset.height;
						}

						popup.setRect(new ShapeUtils.Rect(sceneOrigin, sceneSize));
						// only add once
						if (popup.listeners("resized").length === 0) {
							popup.on("resized", _.bind(function (sceneRect, autoAdapted) {
								var sceneOffset = popupModel.getSceneOffset();
								if (sceneOffset) {
									// we have to compensate the defined offset
									sceneRect.origin.x -= sceneOffset.width;
									sceneRect.origin.y -= sceneOffset.height;
								}

								// convert the scene rect to world rect
								var worldRect = this.getWorldRectFromScene(sceneRect);
								worldRect.size.width = sceneRect.size.width * ConversionConstants.PX_TO_MM_RATIO;
								worldRect.size.height = sceneRect.size.height * ConversionConstants.PX_TO_MM_RATIO;
								var worldPoint = this.getWorldPointFromScene(sceneRect.origin);
								var worldSize = sceneRect.size.copy();
								worldSize.width *= ConversionConstants.PX_TO_MM_RATIO;
								worldSize.height *= ConversionConstants.PX_TO_MM_RATIO;


								popupModel.onResized(new ShapeUtils.Rect(worldPoint, worldSize), autoAdapted);
							}, this));
						}
						// only add once
						if (popup.listeners("dragged").length === 0) {
							popup.on("dragged", _.bind(function (scenePoint) {
								var sceneOffset = popupModel.getSceneOffset();
								if (sceneOffset) {
									// we have to compensate the defined offset
									scenePoint.x -= sceneOffset.width;
									scenePoint.y -= sceneOffset.height;
								}

								popupModel.onDragged(this.getWorldPointFromScene(scenePoint));
							}, this));
						}


						popup.show();
					} else {
						popup.hide();
					}
				}, this);
			}
		};

		return ViewportLayer;
	});

define('web-viewer/viewport/ViewportView',["underscore",
		"web-viewer/viewport/ViewportLayer",
		"utils/ShapeUtils",
		"web-viewer/utils/ListenerUtil"
	],
	function (_,
		ViewportLayer,
		ShapeUtils,
		ListenerUtil) {
		"use strict";

		/**
		 * @exports ViewportView
		 * @memberof ViewportModule
		 * @class A view is responsible for showing different layers within one div-object
		 * The view is mainly steered by its model {ViewportViewModel}
		 * @constructor
		 * @param {Core.ViewerFacade} f - Facade
		 * @param {ViewportWorld} w - the viewport world
		 * @param {String} id
		 */
		var ViewportView = function (f, w, id, viewportPanel) {
			/**
			 * Reference to the world of the viewport
			 */
			this._world = w;
			this.f = f;
			this.id = id;
			ListenerUtil.mixin(this, f);

			/**
			 * Main div for the view
			 */
			this._viewWrapper = $("<div />").addClass("view").attr("name", this.id);
			this.element = this._viewWrapper;

			/**
			 * Override title of model
			 */
			this._title = null;
			this._viewTitle = $("<div />").addClass("viewTitle");
			this._viewWrapper.append(this._viewTitle);
			this._titleShown = false;
			this._viewDiv = $("<div />").css({
				position: 'relative'
			});

			this._viewWrapper.append(this._viewDiv);

			// which borders should be shown
			this.borders = {
				left: false,
				right: false
			};

			/**
			 * is the mouse cursor in the view element
			 * @private
			 */
			this._isActive = false;

			/**
			 * location and size of the view in the viewportPanel
			 * @private
			 */
			this._rect = null;

			/**
			 * Panel
			 * @private
			 */
			this._viewportPanel = viewportPanel;

			/**
			 * Layers
			 */
			this.layers = {};
			this._disabledLayers = [];

			this._isPanning = false;

			/**
			 * Model for the view
			 */
			this._viewportViewModel = undefined;

			this._overwriteOptions = {};

			/**
			 * Render for the view with throttle
			 */
			this._throttledRender = _.throttle(this._renderView, 40);
			this.recalculateViewSize();

			this._registerListener("viewport.startInteractive", function () {
				_.each(this.layers, function (layer) {
					layer.startInteractive();
				});
			}, this);

			this._registerListener("viewport.stopInteractive", function () {
				_.each(this.layers, function (layer) {
					layer.stopInteractive();
				});
			}, this);

			//register listeners to detect if the view is active or not
			this.element.mouseenter(_.bind(function () {
				this._isActive = true;
			}, this));

			this.element.mouseleave(_.bind(function () {
				this._isActive = false;
			}, this));
		};

		/**
		 * Get a layer
		 * @param {String} name - name of the layer
		 * @return {Layer} layer with the given name
		 */
		ViewportView.prototype.getLayer = function (name) {
			return this.layers[name];
		};

		/**
		 * Checks if the view is currently active
		 * @returns {Boolean}
		 */
		ViewportView.prototype.isActive = function () {
			//if the view is not active, check if the view is the default view and no other view is active (mouse cursor is not in the viewport)
			return this._isActive || (this.isDefaultView() && !this._viewportPanel.isViewportWidgetActive());
		};

		/**
		 * Calculate the closest point given a certain point
		 * @param {ShapeUtils.Point} refpoint - Point that needs to be verified (in scene coordinates)
		 * @param {Number} distance - snap distance (in px)
		 * @param {Array.<String>} [searchFilter] - list of layernames in which should be searched
		 * @returns {Object} point - Point that is closest to the point. null if no object was found (in scene coordinates)
		 */
		ViewportView.prototype.getClosestLineworkPointFromPoint = function (refpoint, distance, searchFilter) {
			var closest = null;

			var layers;
			if (searchFilter) {
				layers = _.filter(this.layers, function (layer) {
					if (layer.getLayerModel()) {
						return _.contains(searchFilter, layer.getLayerModel().getName());
					} else {
						return false;
					}
				});
			} else {
				layers = this.layers;
			}


			_.each(layers, function (layer) {
				var point = layer.getClosestLineworkPointFromPoint(refpoint, distance);
				if (point) {
					//convert the point to the actual world
					if (!closest || point.getDistance(refpoint) < closest.getDistance(refpoint)) {
						closest = point;
						return true;
					}
				}
			});

			return closest;
		};


		/**
		 * Get the closest linework from a certain point (or area if you use fatfinger)
		 * !NOTE It will return the first linework found close to the refpoint when searching multiple layers)
		 * @param {ShapeUtils.Point} refpoint - point in scene coordinates
		 * @param {Array.<String>} [searchFilter] - list of layernames in which should be searched
		 * @param {Number} [fatfinger] - search area around refpoint (in px)
		 * @returns {LineworkWrapper} the closest linwwork to refpoint
		 */
		ViewportView.prototype.getClosestLineworkFromPoint = function (refpoint, searchFilter, fatfinger) {
			var linework = null;

			var layers;
			if (searchFilter) {
				layers = _.filter(this.layers, function (layer) {
					if (layer.getLayerModel()) {
						return _.contains(searchFilter, layer.getLayerModel().getName());
					} else {
						return false;
					}
				});
			} else {
				layers = this.layers;
			}

			_.each(layers, function (layer) {
				if (!linework) {
					linework = layer.getClosestLineworkFromPoint(refpoint, fatfinger);
				}
			});


			return linework;

		};


		/**
		 * Recalculate the viewDiv size
		 */
		ViewportView.prototype.recalculateViewSize = function () {
			var css = {
				width: this._viewWrapper.innerWidth(),
				height: this._viewWrapper.innerHeight() - this._viewTitle.outerHeight(true)
			};

			// draw a border on the left side if needed
			if (this.borders.left) {
				css.width--;
				css['border-left'] = "1px solid";
			}

			// draw a border on the right side if needed
			if (this.borders.right) {
				css.width--;
				css['border-right'] = "1px solid";
			}
			this._viewDiv.css(css);
		};

		/**
		 * Hide a layer of this view
		 * @param {String} layerName - Name of the layer
		 * @param {Boolean} hide - True if hide, false if show
		 */
		ViewportView.prototype.hideLayer = function (layerName, hide) {
			if (this.layers[layerName]) {
				this.layers[layerName].setVisible(!hide);
			}
		};

		/**
		 * Hide all layers of this view
		 * @param {Boolean} hide - True if hide, false if show
		 */
		ViewportView.prototype.hideAllLayers = function (hide) {
			_.each(this.layers, function (layer, layerName) {
				this.hideLayer(layerName, hide);
			}, this);
		};

		/**
		 * @param {ShapeUtils.Rect} rect - Rectangle
		 */
		ViewportView.prototype.setRect = function (rect) {
			this._viewWrapper.css({
				position: 'absolute',
				width: rect.size.width,
				height: rect.size.height,
				top: rect.top(),
				left: rect.left()
			});
			this._rect = rect;
			this.resizeLayers();
			this.repaint();
		};

		/**
		 * Get the location of the view in the viewport
		 * @returns {ShapeUtils.Point}
		 */
		ViewportView.prototype.getViewLocation = function () {
			var location = new ShapeUtils.Point();
			if (this._rect !== null) {
				location = this._rect.origin.copy(); //only saves the position in the panel (not in the viewport)
			}
			var panelLocation = this._viewportPanel.getPosition(); //get the location of the panel
			if (panelLocation !== undefined) { //add the location of the panel to the view location
				location.x += panelLocation.x;
				location.y += panelLocation.y;
			}
			return location;
		};

		/**
		 * Get the div of the view wrapper (including title)
		 * @returns {JQueryObj} Div of the view wrapper
		 */
		ViewportView.prototype.getWrapperContext = function () {
			return this._viewWrapper;
		};

		/**
		 * Get the div of the view only (without title)
		 * @returns {JQueryObj} Div of the view only
		 */
		ViewportView.prototype.getViewContext = function () {
			return this._viewDiv;
		};

		/**
		 * Resize all the layers
		 */
		ViewportView.prototype.resizeLayers = function () {
			this.recalculateViewSize();
			_.each(this.layers, _.bind(function (layer, name) {
				this._resizeLayer(layer);
			}, this));
		};

		/**
		 * Resize a layer
		 * @param {Layer} layer - layer to be resized
		 */
		ViewportView.prototype._resizeLayer = function (layer) {
			layer.setSize(new ShapeUtils.Size($(this._viewDiv).width(), $(this._viewDiv).height()));
		};

		/**
		 * Insert a layer at a certain index
		 * @param {Number} index
		 * @param {JQueryObject} element
		 */
		ViewportView.prototype._insertLayerAtIndex = function (index, element) {
			var lastIndex = this._viewDiv.children().size();
			if (index < 0) {
				// the title sits at index 1
				index = Math.max(0, lastIndex + 1 + index);
			}
			if (index < lastIndex) {
				this._viewDiv.children().eq(Math.max(0, index)).before(element);
			} else {
				this._viewDiv.append(element);
			}
		};

		/**
		 * Add a layer to the view
		 * @param {String} name - name of the layer (fetched from the model)
		 */
		ViewportView.prototype.addLayer = function (name) {
			var layer = this.layers[name];
			if (!layer) {
				layer = new ViewportLayer(this.f, this, this._world, this._viewportViewModel.getTransform(), this.id + "_" + name);
				layer.setOverwriteOptions(this._getOverwriteOptions());
				var index = this._viewportViewModel.indexForLayer(name, this.layers);
				this._insertLayerAtIndex(index, layer.getCanvas());
				layer.setVisible(this.isLayerVisible(name));
				this.layers[name] = layer;

			}

			var layerModel = this._viewportViewModel.modelForLayer(name);
			if (layerModel) {
				layer.setLayerModel(layerModel);
			}

			this.recalculateViewSize();
			this._resizeLayer(layer);

			return layer;
		};

		/**
		 * Returns the overwrite options
		 * @returns {Object}
		 */
		ViewportView.prototype._getOverwriteOptions = function getOverwriteOptions() {
			return this._overwriteOptions;
		};

		/**
		 * Repaint the view
		 * @param {World} [w] - the world
		 * @param {Layer} [layer] - if specified, only this layer gets repainted
		 */
		ViewportView.prototype.repaint = function (w, layer) {
			if (this._viewportViewModel.isShown()) {
				if (w !== undefined) {
					this._world = w;
				}
				this._throttledRender(layer);
			}
		};

		/**
		 * Set the model of the view.
		 *
		 * Subscribes to the events that the model might send
		 * Unsubscribes to the old model (if set)
		 */
		ViewportView.prototype.setModel = function (viewportViewModel) {
			/*
			 * Unsubscribe the old channels on viewportViewModel
			 */
			if (this._viewportViewModel) {
				this._unregisterAllListenersForObject(this._viewportViewModel);
			}
			// NOTE probably layers also need to be removed?

			/*
			 * Set the new model
			 */
			this._viewportViewModel = viewportViewModel;

			/*
			 * Subscribe on the new events
			 */
			this._registerListener("layerAdded", this._addLayerToView, this, this._viewportViewModel);
			this._registerListener("layerRemoved", this.removeLayer, this, this._viewportViewModel);
			this._registerListener("titleChanged", this._repaintTitle, this, this._viewportViewModel);
			this._registerListener("showChanged", this._showHide, this, this._viewportViewModel);

			/*
			 * Add the current situation of views
			 */
			for (var i = 0; i < this._viewportViewModel.numberOfLayers(); i++) {
				var name = this._viewportViewModel.nameForLayerAtIndex(i);
				this.addLayer(name);
			}
		};

		/**
		 * Get the model for the viewport view
		 * @returns {ViewportViewModel}
		 */
		ViewportView.prototype.getModel = function () {
			return this._viewportViewModel;
		};

		/**
		 * Removes the layer from the view, when a layer in the model was deleted
		 */
		ViewportView.prototype.removeLayer = function (layername) {
			if (this.layers[layername]) {
				this.layers[layername].destroy();
				delete this.layers[layername];
			} else {
				this.f.debug.debug("Trying to remove a layer that does not exists: " + layername);
			}
		};

		/**
		 * Add a layer on the view, when a layer in the model was added
		 */
		ViewportView.prototype._addLayerToView = function (layerName) {
			this.addLayer(layerName);
			this._throttledRender(layerName);
		};

		/**
		 * Show or hide this view
		 */
		ViewportView.prototype._showHide = function () {
			var show = this._viewportViewModel.isShown();
			if (show) {
				this._viewWrapper.show();
				this.hideAllLayers(false);
			} else {
				this.hideAllLayers(true);
				this._viewWrapper.hide();
			}
		};

		/**
		 * get the height of the title
		 * @returns {Number}
		 */
		ViewportView.prototype.getTitleHeight = function () {
			return $(this._viewTitle).outerHeight(true);
		};

		/**
		 *  Shows this view a title
		 *  @returns {Boolean}
		 */
		ViewportView.prototype.isTitleVisible = function () {
			return this._titleShown;
		};

		/**
		 * Repaint the title based on its value in the model
		 */
		ViewportView.prototype._repaintTitle = function () {
			var title = this._viewportViewModel.getTitle();
			if (this._title !== null) {
				title = this._title;
			}

			if (title) {
				this._viewTitle.text(title);
				this._viewTitle.attr("title", title);
				if (!this._titleShown) {
					this._viewTitle.show();
					this._titleShown = true;
					this.recalculateViewSize();
					this.resizeLayers();
				}
			} else {
				this._viewTitle.empty();
				if (this._titleShown) {
					this._viewTitle.hide();
					this._titleShown = false;
					this.recalculateViewSize();
					this.resizeLayers();
				}
			}
		};

		/**
		 * Render the view again
		 * @param {Layer} [layer] - if specified, only this layer gets repainted
		 */
		ViewportView.prototype._renderView = function (layer) {
			this._repaintTitle();
			if (layer) {
				var actuallayer = this.layers[layer];
				if (actuallayer) {
					actuallayer.repaint(this._world, this._viewportViewModel.getTransform());
				}
			} else {
				_.each(this.layers, function (actuallayer, name) {
					actuallayer.repaint(this._world, this._viewportViewModel.getTransform());
				}, this);
			}
		};

		/**
		 * is panning active
		 * @returns {Boolean}
		 */
		ViewportView.prototype.isPanning = function () {
			return this._isPanning;
		};


		/**
		 * Show/hide borders
		 * You should call recalculateViewSize (or a function that calls it for you like setRect)
		 * after you set this border visibility to update the CSS.
		 * @param {Boolean} left - Show border on the left
		 * @param {Boolean} right - Show border on the right
		 */
		ViewportView.prototype.setBorderVisibility = function (left, right) {
			this.borders.left = left;
			this.borders.right = right;
		};

		/**
		 * Destroy the view
		 * @param {Boolean} destroyModel - if true also destroy the model
		 */
		ViewportView.prototype.destroy = function (destroyModel) {
			this._unregisterAllListeners();

			// destroy all layers
			_.each(this.layers, function (layer) {
				layer.destroy();
			});
			// delete all created HTML elements
			this.element.remove();
		};

		/**
		 * Get the id of the view
		 * @returns {String}
		 */
		ViewportView.prototype.getID = function () {
			return this.id;
		};

		/**
		 * Set overwrite options on all layers
		 * @param {Object} options
		 */
		ViewportView.prototype.setOverwriteOptions = function (options) {
			this._overwriteOptions = options;
			_.each(this.layers, function (layer) {
				layer.setOverwriteOptions(options);
			});
		};

		/**
		 * Toggle a specific layer
		 * @param {String} layerName - name of the layer
		 * @param {Boolean} toggle
		 */
		ViewportView.prototype.toggleLayer = function (layerName, toggle) {
			var layer = this.layers[layerName];
			if (layer) {
				layer.setVisible(toggle);
			}

			if (toggle) {
				this._disabledLayers = _.without(this._disabledLayers, layerName);
			} else {
				this._disabledLayers.push(layerName);
			}
		};
		/**
		 * Toggle a set of layers
		 * @param {Array.<String>} layers - names of the layers to toggle
		 * @param {Boolean} toggle
		 */
		ViewportView.prototype.toggleLayers = function (layers, toggle) {
			_.each(layers, function (layer) {
				this.toggleLayer(layer, toggle);
			}, this);
		};

		/**
		 * is a specific layer visible?
		 * @param {String} layerName
		 * @returns {Boolean}
		 */
		ViewportView.prototype.isLayerVisible = function (layerName) {
			return _.indexOf(this._disabledLayers, layerName) < 0;
		};

		/**
		 * Override the title defined in the model
		 * @param {String|null} title
		 */
		ViewportView.prototype.setTitle = function (title) {
			this._title = title;
			this._repaintTitle();
		};

		/**
		 * is the view the default view
		 * @returns {Boolean}
		 */
		ViewportView.prototype.isDefaultView = function () {
			return this._viewportViewModel.isDefaultView() && this._overwriteOptions.isDefaultView;
		};

		/**
		 * The viewport started panning, tell all the layers that panning is started
		 */
		ViewportView.prototype.panningStarted = function () {
			this._isPanning = true;
			_.each(this.layers, function (layer) {
				layer.panningStarted();
			});
		};

		/**
		 * The viewport stopped panning, tell all the layers that panning is stopped
		 */
		ViewportView.prototype.panningStopped = function () {
			this._isPanning = true;
			_.each(this.layers, function (layer) {
				layer.panningStopped();
			});
		};

		/**
		 * The viewport started panning, tell all the layers that panning is started
		 */
		ViewportView.prototype.zoomingStarted = function () {
			this._isZooming = true;
			_.each(this.layers, function (layer) {
				layer.zoomingStarted();
			});
		};

		/**
		 * The viewport started panning, tell all the layers that panning is started
		 */
		ViewportView.prototype.zoomingStopped = function () {
			this._isZooming = false;
			_.each(this.layers, function (layer) {
				layer.zoomingStopped();
			});
		};

		/**
		 * Returns the panel this view is contained in
		 * @returns {ViewportPanel}
		 */
		ViewportView.prototype.getPanel = function () {
			return this._viewportPanel;
		};

		return ViewportView;
	});

define('web-viewer/viewport/ViewportPanel',["web-viewer/utils/ListenerUtil",
		"utils/ShapeUtils",
		"web-viewer/viewport/ViewportView",
		"web-viewer/viewport/ViewportConstants",
		"underscore",
		"jquery"
	],
	function (ListenerUtil,
		ShapeUtils,
		ViewportView,
		ViewportConstants,
		_,
		$) {
		"use strict";

		/**
		 * A viewport panel is a representation of a bunch of viewportviewmodels. It creates view objects for every viewmodel is should show
		 * ViewportPanel is responsible for the layout of its panel
		 * @mixes Utils.ListenerUtilMixin
		 * @param {Core.ViewerFacade} f
		 * @param {ViewportModule.ViewportPanelModel} panelModel - the model for the panel
		 * @param {ViewPortModule.ViewportWidgetModel} widgetModel - the widgetmodel
		 * @param {ViewportModule.ViewportWorld} viewportWorld - the viewport world
		 */
		var ViewportPanel = function (f, panelModel, widgetModel, viewportWorld) {
			this.f = f;
			ListenerUtil.mixin(this, f);

			/**
			 * @private a map of the existing ViewObjects
			 */
			this._viewObjects = {};
			/**
			 * @private
			 */
			this._widgetModel = widgetModel;
			/**
			 * @private
			 */
			this._panelModel = panelModel;
			/**
			 * @private
			 */
			this._viewportWorld = viewportWorld;

			/**
			 * This is the element in which the panel will draw itself
			 */
			this.element = $("<div/>").attr("name", panelModel.getName()).css({
				top: 0,
				left: 0
			}).addClass("viewportPanel");
			this.container = $("<div />").addClass("viewportPanelContainer");
			this.element.append(this.container);

			/**
			 * Debounced version of _layout()
			 * Only .layout() should be used
			 * @private
			 */
			this._debouncedLayout = _.debounce(this._layout, 50);
			this.updateOverwriteOptions = _.debounce(this._updateOverwriteOptions, 20);

			// start listening to the correct events
			this._registerListener("changed.overwriteOption.**", this.updateOverwriteOptions, this, this._panelModel);
			this._registerListener("changed.disabledLayers", this.updateDisabledLayers, this, this._panelModel);
			this._registerListener("changed.headerText", this.updatePanelHeader, this, this._panelModel);

			this._registerListener("changed.view.toggle.*", this.layout, this, this._widgetModel);

			this._registerListener("changed.view.order", this.layout, this, this._widgetModel);
			// changed.view.add is not necessary as a "order" is also fired
			this._registerListener("changed.view.remove", function (viewModel) {
				this._destroyView(viewModel.getID());
			}, this, this._widgetModel);

			this._registerListener("changed.layout.*", this.layout, this, this._widgetModel);

			//Listen to when panning is started
			this._registerListener("viewport.startPanning", _.bind(function () {
				this._isPanning = true;
				_.each(this._viewObjects, function (view) {
					view.panningStarted();
				});
			}, this));

			// Listen to when panning is stopped
			this._registerListener("viewport.stopPanning", _.bind(function () {
				this._isPanning = false;
				_.each(this._viewObjects, function (view) {
					view.panningStopped();
				});
			}, this));

			//
			this._registerListener("viewport.startZooming", _.bind(function () {
				this._isZooming = true;
				_.each(this._viewObjects, function (view) {
					view.zoomingStarted();
				});
			}, this));

			//
			this._registerListener("viewport.stopZooming", _.bind(function () {
				this._isZooming = false;
				_.each(this._viewObjects, function (view) {
					view.zoomingStopped();
				});
			}, this));

			this.repaint = _.throttle(_.bind(function () {
				this._repaint();
			}, this), 40);
		};

		/**
		 * Destroy the panel
		 * This removes all listeners and DOM elements
		 */
		ViewportPanel.prototype.destroy = function () {
			this._unregisterAllListeners();

			// destroy all views
			_.each(this._viewObjects, function (view) {
				view.destroy();
			});

			this.element.remove();
		};

		/**
		 * Is the viewport widget active, is the mouse cursor in the viewport at this moment.
		 * @returns {Boolean}
		 */
		ViewportPanel.prototype.isViewportWidgetActive = function () {
			return this._widgetModel.isWidgetActive();
		};

		/**
		 * Update the layout of the panel
		 */
		ViewportPanel.prototype.layout = function () {
			if (!this.hasBeenLayouted) {
				this._layout();
			} else {
				this._debouncedLayout();
			}
		};

		/**
		 * Get the model of the panel
		 * @returns {ViewportPanelModel}
		 */
		ViewportPanel.prototype.getModel = function () {
			return this._panelModel;
		};
		/**
		 * Update the layout of the panel
		 *
		 * layout will also create view objects if no object exists yet
		 * !! only use ViewportPanel.layout();
		 * @private
		 */
		ViewportPanel.prototype._layout = function () {
			var viewsLayoutMode = this._widgetModel.getResolvedViewsLayoutMode();
			var viewKeys = this._widgetModel.getShownViewKeys();
			this.viewCount = {
				total: viewKeys.length
			};

			if (viewsLayoutMode === ViewportConstants.ViewsLayoutMode.HORIZONTAL) {
				this._layoutViewsHorizontal(viewKeys);
			} else if (viewsLayoutMode === ViewportConstants.ViewsLayoutMode.VERTICAL) {
				this._layoutViewsVertical(viewKeys);
			} else {
				this._layoutViewsAutomatic(viewKeys);
			}

			if (this.viewCount.total > 0) {
				this.hasBeenLayouted = true;
			}

			this.updatePanelHeader();
		};

		/**
		 * Layout the views vertically (left to right)
		 * @param {Array.<String>} viewKeys
		 * @private
		 */
		ViewportPanel.prototype._layoutViewsVertical = function (viewKeys) {
			var divSize = new ShapeUtils.Size(this.container.width(), this.container.height());

			var x = viewKeys.length;
			var y = 1;

			this.viewCount.x = x;
			this.viewCount.y = y;

			var viewWidth = Math.ceil(divSize.width / x);
			var viewHeight = Math.ceil(divSize.height / y);

			var viewSize = new ShapeUtils.Size(viewWidth, viewHeight);
			this._widgetModel.setSizeForView(viewSize);
			_.each(viewKeys, function (viewID, index) {
				var view = this.getView(viewID);

				// show a border on the right for all views, except the last one
				view.setBorderVisibility(false, index < viewKeys.length - 1);


				view.setRect(ShapeUtils.rectMake(viewWidth * index, 0, viewWidth, viewHeight));

			}, this);
		};

		/**
		 * Layout the views horizontally (top to bottom)
		 * @param {Array.<String>} viewKeys
		 * @private
		 */
		ViewportPanel.prototype._layoutViewsHorizontal = function (viewKeys) {
			var divSize = new ShapeUtils.Size(this.container.width(), this.container.height());

			var x = 1;
			var y = viewKeys.length;

			this.viewCount.x = x;
			this.viewCount.y = y;

			var viewWidth = Math.ceil(divSize.width / x);
			var viewHeight = Math.ceil(divSize.height / y);


			var viewSize = new ShapeUtils.Size(viewWidth, viewHeight);
			this._widgetModel.setSizeForView(viewSize);
			_.each(viewKeys, function (viewID, index) {
				var view = this.getView(viewID);
				// in vertical mode we don't want to see a border
				view.setBorderVisibility(false, false);

				view.setRect(ShapeUtils.rectMake(0, viewHeight * index, viewWidth, viewHeight));
			}, this);
		};
		/**
		 * Layout the views automatically
		 * @param {Array.<String>} viewKeys
		 * @private
		 */
		ViewportPanel.prototype._layoutViewsAutomatic = function (viewKeys) {
			var divSize = new ShapeUtils.Size(this.container.width(), this.container.height());

			var viewsCount = viewKeys.length;

			var x = viewsCount;
			var y = 1;

			// Math.floor for integer dividing
			if (viewsCount > 1) {
				x = Math.floor(Math.round(Math.sqrt(viewsCount) * divSize.width / divSize.height));
				if (x < 1) {
					x = 1;
				}
				y = Math.floor((viewsCount + x - 1) / x);
				x = Math.floor((viewsCount + y - 1) / y); // calculate again for best possible spacing
				var empty = x * y - viewsCount;
				if (x > 1 && y > 1 && empty > 0) {
					// try to adjust by +1 to reduce white space
					var x1 = x + 1;
					var y1 = Math.floor((viewsCount + x1 - 1) / x1);
					var empty1 = x1 * y1 - viewsCount;
					var y2 = y + 1;
					var x2 = Math.floor((viewsCount + y2 - 1) / y2);
					var empty2 = x2 * y2 - viewsCount;
					if (Math.min(empty1, empty2) < empty) {
						if (empty1 <= empty2) {
							x = x1;
							y = y1;
						} else {
							x = x2;
							y = y2;
						}
					}
				}
			}

			this.viewCount.x = x;
			this.viewCount.y = y;

			var viewWidth = Math.ceil(divSize.width / x);
			var viewHeight = Math.ceil(divSize.height / y);
			var viewSize = new ShapeUtils.Size(viewWidth, viewHeight);
			this._widgetModel.setSizeForView(viewSize);
			_.each(viewKeys, function (viewID, index) {
				var view = this.getView(viewID);
				// show a border on the right for every view (except the last one of a row)
				view.setBorderVisibility(false, index % x < x - 1);
				view.setRect(ShapeUtils.rectMake(viewWidth * Math.floor(index % x), viewHeight * Math.floor(index / x), viewWidth, viewHeight));
			}, this);
		};


		/**
		 * Trigger a repaint for all the subviews, one of the overwrite options is changes
		 * @private
		 */
		ViewportPanel.prototype._repaint = function () {
			_.each(this._widgetModel.getShownViewKeys(), function (viewID, index) {
				var view = this.getView(viewID);
				view.repaint(this._viewportWorld);

			}, this);
		};

		/**
		 * Update the overwrite Options for all views
		 * After that repaint all views
		 * @private
		 */
		ViewportPanel.prototype._updateOverwriteOptions = function () {
			var overwriteOptions = this._panelModel.getOverwriteOptions();
			_.each(this._widgetModel.getShownViewKeys(), function (viewID, index) {
				var view = this.getView(viewID);
				view.setOverwriteOptions(overwriteOptions);
			}, this);
		};

		/**
		 * Update all layer visibilities
		 * @param {Array.<String>} previouslyDisabled - the list of previously disabled layers
		 * @param {Array.<String>} currentlyDisabled - the list of currently disabled layers
		 */
		ViewportPanel.prototype.updateDisabledLayers = function (previouslyDisabled, currentlyDisabled) {
			var toEnable = _.difference(previouslyDisabled, currentlyDisabled);
			var toDisable = currentlyDisabled;

			_.each(this._viewObjects, function (view, key) {
				view.toggleLayers(toEnable, true);
				view.toggleLayers(toDisable, false);
			});
		};


		/**
		 * Get the ViewportView object for a certain view
		 * @param {String} viewID
		 * @returns {ViewportModule.ViewportView}
		 */
		ViewportPanel.prototype.getView = function (viewID) {
			if (this._viewObjects[viewID]) {
				// view object exists
				return this._viewObjects[viewID];
			} else {
				// create a view object
				return this._createView(viewID);
			}
		};
		/**
		 * Create viewportview object this also injects the view element in to the DOM
		 * @private
		 * @param {String} viewID - The ID of the view
		 * @return {ViewportModule.ViewportView}
		 */
		ViewportPanel.prototype._createView = function (viewID) {
			var view = new ViewportView(this.f, this._viewportWorld, this._panelModel.getName() + "_" + viewID, this);

			view.setModel(this._widgetModel.getViewModel(viewID));

			view.setOverwriteOptions(this._panelModel.getOverwriteOptions());
			view.toggleLayers(this._panelModel.getDisabledLayers(), false);
			this._viewObjects[viewID] = view;
			this.container.append(view.element);

			return view;
		};

		/**
		 * Destroy a view object as it is no longer needed
		 * Note: this function also removes the element from the DOM
		 * @private
		 * @param {String} viewID
		 */
		ViewportPanel.prototype._destroyView = function (viewID) {
			if (this._viewObjects[viewID]) {
				this._viewObjects[viewID].destroy();
				delete this._viewObjects[viewID];
			}
		};

		/**
		 * Get a view at a certain point
		 * @param {ShapeUtils.Point} point
		 * @returns {ViewportView}
		 */
		ViewportPanel.prototype.getViewAt = function (point) {
			point = point.copy();
			var offset = this.element.offset();
			point.x -= offset.left;
			point.y -= offset.top;

			var viewsLayoutMode = this._widgetModel.getResolvedViewsLayoutMode();


			var viewSize = this._widgetModel.getSizeForView();
			if (viewSize.width === 0 || viewSize.height === 0) {
				return undefined;
			}

			var viewIndex = 0;

			if (viewsLayoutMode === ViewportConstants.ViewsLayoutMode.HORIZONTAL) {
				viewIndex = Math.floor(point.y / viewSize.height);
			} else if (viewsLayoutMode === ViewportConstants.ViewsLayoutMode.VERTICAL) {
				viewIndex = Math.floor(point.x / viewSize.width);
			} else {
				// automatic
				var row = Math.floor(point.y / viewSize.height);
				var column = Math.floor(point.x / viewSize.width);
				viewIndex = (row * this.viewCount.x) + column;
			}


			return this._viewObjects[this._widgetModel.getIDForShownViewsAtIndex(viewIndex)];
		};

		/**
		 * Get a view if it already exists
		 * @param {String} viewName
		 * @returns {ViewportModule.Viewportview}
		 */
		ViewportPanel.prototype.getViewIfExists = function (viewName) {
			return this._viewObjects[viewName];
		};

		/**
		 * Set the  bounds of the viewport panel
		 * @param position
		 * @param size
		 */
		ViewportPanel.prototype.setBounds = function (position, size) {
			this.element.css({
				left: position.x,
				top: position.y,
				width: size.width,
				height: size.height
			});
			this.position = position;
			this.layout();
		};

		/**
		 * Get the position of the panel in the viewport.
		 * @returns {ShapeUtils.Point}
		 */
		ViewportPanel.prototype.getPosition = function () {
			if (this.position !== undefined) {
				return this.position.copy();
			} else {
				return new ShapeUtils.Point();
			}
		};

		/**
		 * If the header text has changed, update the header of the first view to show the header text if needed
		 */
		ViewportPanel.prototype.updatePanelHeader = function () {
			var headerText = this._panelModel.getHeaderText();
			var shownViewCount = this._widgetModel.getShownViewKeys().length;
			var viewID = this._widgetModel.getIDForViewAtIndex(0);
			if (headerText !== null && shownViewCount > 0) {
				// set the header of the first view to the headerText
				this._viewObjects[viewID].setTitle(headerText);
			} else if (shownViewCount > 1) {
				// Show the composite title
				this.f.i18n.loadBundle("viewer", function () {
					this._viewObjects[viewID].setTitle(this.f.i18n.t("CaptionComposite"));
				}, this);
			} else if (shownViewCount === 1) {
				this._viewObjects[viewID].setTitle(null);
			}
		};


		return ViewportPanel;
	});

define('web-viewer/viewport/ViewportWidget',["web-viewer/layout/AbstractWidget",
		"web-viewer/viewport/ViewportWorld",
		"utils/ShapeUtils",
		"web-viewer/viewport/tools/ToolManager",
		"web-viewer/viewport/ViewportConstants",
		"web-viewer/viewport/ViewportPanel",
		"utils/OrderedObject",
		"web-viewer/utils/ListenerUtil",
		"web-viewer/layout/LayoutConstants",
		"jquery",
		"underscore"
	],
	function (AbstractWidget,
		ViewportWorld,
		ShapeUtils,
		ToolManager,
		ViewportConstants,
		ViewportPanel,
		OrderedObject,
		ListenerUtil,
		LayoutConstants,
		$,
		_) {
		"use strict";

		/**
		 *  @exports ViewportWidget
		 *  @memberof ViewportModule
		 *  @class The viewport widget is responsible to draw the content of the viewer
		 *
		 * Channels that it listens to:
		 *  - "viewport.transform" : applies a transform to the world
		 *  - "viewport.zoomToFit" : Fits the content in the widget, so it can be viewed full view (or 90%)
		 *  - "viewport.fitInRect" : Fit the viewport to a certain rectangle. E.g. zoom to an annotation
		 *  - "viewport.toggleAnimation : enable or disable animations
		 *
		 *  @param {Core.ViewerFacade} f - Facade
		 *  @param {ViewportWorld} world - the viewport world
		 */
		var ViewportWidget = function ViewPortWidget(f, world) {
			AbstractWidget.call(this, f, LayoutConstants.Widget.VIEWPORT);
			ListenerUtil.mixin(this, f);

			/**
			 * @private
			 */
			this._toolManager = new ToolManager(this.f, this);
			/**
			 * @private
			 */
			this._world = world;

			/**
			 * @private
			 */
			this._widgetModel = null;

			/**
			 * @private
			 */
			this._panelObjects = {};
			/**
			 * @private
			 */
			this._element = $("<div />").addClass("viewport");

			/**
			 * Repaint the widget
			 */
			this.repaint = _.throttle(_.bind(function () {
				this._repaint();
			}, this), 40);

			this.layout = _.debounce(_.bind(function () {
				this._layout();
			}, this), 40);

			/**
			 * Did the viewer already zoom to fit
			 * @private {Boolean}
			 */
			this._didZoomToFit = false;

			// listen to transform changed of world
			this._registerListener("transformChanged", this.repaint, this, this._world);
			this.f.getDocumentController().whenActivePageChanged(function (oldPage, newPage) {
				if (oldPage !== newPage) {
					this._didZoomToFit = false;
				}
			}, this);
			this.f.getDocumentController().whenDifferencePageChanged(function (oldPage, newPage) {
				if (oldPage !== newPage) {
					this._didZoomToFit = false;
				}
			}, this);

			this._registerListener("parameters.endUpdate", this._verifyWorld, this, this.f.getDocumentController());
			this._verifyWorld();

			//register listeners to detect if the mouse cursor is in the viewport
			this._element.mouseenter(_.bind(function () {
				this._widgetModel.setWidgetActive(true);
			}, this));

			this._element.mouseleave(_.bind(function () {
				this._widgetModel.setWidgetActive(false);
			}, this));

		};

		ViewportWidget.prototype = Object.create(AbstractWidget.prototype);
		ViewportWidget.prototype.constructor = ViewportWidget;
		ViewportWidget.widgetName = LayoutConstants.Widget.VIEWPORT;


		//------------ viewport specific  ------------------------------------------------------------------------------
		/**
		 * Set the WidgetModel
		 * @param {ViewportModule.ViewportWidgetModel} model
		 */
		ViewportWidget.prototype.setWidgetModel = function (model) {

			if (this._widgetModel) {
				// unregister all listeners registered on _widgetModel
				this._unregisterAllListenersForObject(this._widgetModel);
			}

			this._widgetModel = model;

			this._registerListener("changed.panel.order", this.layout, this, this._widgetModel);
			this._registerListener("changed.panel.remove", this._destroyPanel, this, this._widgetModel);
			this._registerListener("changed.view.size", this._verifyWorld, this, this._widgetModel);
			// we don't need to listen to changed.panel.add, as this will also trigger a "order" update 

			this._registerListener("changed.layout.*", this.layout, this, this._widgetModel);
			// trigger a repaint
			this.layout();
			this.repaint();
		};

		/**
		 * Get the size of the viewport widget
		 * @returns {ShapeUtils.Size}
		 */
		ViewportWidget.prototype.getSize = function () {
			return new ShapeUtils.Size(this._element.innerWidth(), this._element.innerHeight());
		};

		/**
		 * Get the view located at a certain mouse position
		 * @param {Event|ShapeUtils.Point|String}
		 * @returns {ViewportModule.ViewportView|undefined} The view on the given mouse position, or undefined if not found.
		 */
		ViewportWidget.prototype.requestView = function (e) {
			if (_.isString(e)) {
				return this.getFirstViewOccurrence(e);
			}

			var point = null;
			if (e instanceof ShapeUtils.Point) {
				point = e.copy();
			} else {
				if (e.pageX) {
					point = new ShapeUtils.Point(e.pageX, e.pageY);
				} else {
					point = new ShapeUtils.Point(e.clientX, e.clientY);
				}
			}

			var panel = this.getPanelAt(point);
			if (panel) {
				return panel.getViewAt(point);
			} else {
				return undefined;
			}
		};


		//------------ world  ------------------------------------------------------------------------------	
		/**
		 * Update the world if needed, this checks if all world and scene sizes are still correct, changes them if needed
		 * @returns {Boolean} true if the world has changed
		 */
		ViewportWidget.prototype._verifyWorld = function () {
			if (!this._widgetModel) { //make sure widgetModel is set
				return false;
			}
			//defines the size of the world, this is a ShapeUtils.Rect
			var worldBounds = this._widgetModel.getWorldBounds();
			if (!worldBounds) {
				return false;
			}

			//Check if the world is still OK, checks if the scene size is correct
			var changed = false;
			var sizeForView = this._widgetModel.getSizeForView();
			if (!sizeForView.equals(this._world.getSceneSize())) {
				this._world.setSceneSize(sizeForView); //set the new correct scene size
				changed = true;
			}

			// set the resolution of the world on the world (in this way we can zoom in to the same levels as the viewser generates rasterized images)
			var worldResolution = this._widgetModel.getWorldResolution();
			if (worldResolution !== undefined) {
				this._world.setWorldResolution(worldResolution);
			}

			if (!worldBounds.rectEquals(this._world.getWorldBounds())) {
				this._world.setWorldBounds(worldBounds);
				changed = true;
			}

			var worldSizeNotNull = (this._world.getWorldSize().width !== 0 && this._world.getWorldSize().height !== 0);
			var sceneSizeNotNull = (this._world.getSceneSize().width !== 0 && this._world.getSceneSize().height !== 0);

			if ((!this._didZoomToFit || (changed && !this._world.didChangeTransformSinceLastZoomToFit())) && worldSizeNotNull && sceneSizeNotNull) {
				this._world.zoomToFit(false);
				if (sizeForView.width > 0 && sizeForView.height > 0 && !this.f.isParameterUpdateOngoing()) {
					this._didZoomToFit = true;
				}
			}

			if (changed && this._world.getInitialTransformation()) {
				this._world.transform(this._world.getInitialTransformation());
			}

			return changed;
		};

		/**
		 * Get the world used in the viewport
		 * @returns {ViewportModule.ViewportWorld}
		 */
		ViewportWidget.prototype.getWorld = function () {
			return this._world;
		};

		//------------ widget specific ------------------------------------------------------------------------------	
		/**
		 * Set the context of the viewport widget
		 * @override
		 * @param {JQueryObj} context
		 */
		ViewportWidget.prototype.setContext = function (context) {
			AbstractWidget.prototype.setContext.call(this, context);

			this.context.append(this._element);
		};

		/**
		 * Viewportwidget is a "maincontent" widget
		 * @override
		 * @returns {String}
		 */
		ViewportWidget.prototype.getWidgetType = function () {
			return "maincontent";
		};

		/**
		 * The widget was resized, update all panels/views --> repaint
		 * @override
		 */
		ViewportWidget.prototype.resized = function () {
			this.layout();
		};

		//------------ Panels ----------------------------------------------------------------------
		/**
		 * Get a Panel Object with a certain name
		 * If the object is not existing, create it
		 * @param {String} name - name of the panel
		 * @returns {ViewportModule.ViewportPanel}
		 */
		ViewportWidget.prototype.getPanel = function (name) {
			if (this._panelObjects[name] !== undefined) {
				return this._panelObjects[name];
			} else {
				return this._createPanel(name);
			}
		};

		/**
		 * Create a panel for a certain Panel Model
		 * @private
		 * @param {String} name - name of the panel
		 * @returns {ViewportModule.ViewportPanel}
		 */
		ViewportWidget.prototype._createPanel = function (name) {
			var panel = new ViewportPanel(this.f, this._widgetModel.getPanelModel(name), this._widgetModel, this._world);
			this._panelObjects[name] = panel;
			this._element.append(panel.element);
			return panel;
		};

		/**
		 * Destroy a panel (remove all listeners + DOM elements)
		 * @param {ViewportModule.ViewportPanel} panel - the panel to destroy
		 */
		ViewportWidget.prototype._destroyPanel = function (panel) {
			var name = panel.getName();
			if (this._panelObjects[name] !== undefined) {
				this._panelObjects[name].destroy();
				delete this._panelObjects[name];
				this.layout();
			}
		};

		/**
		 * Get the panel sitting at specific point
		 * @param {ShapeUtils.Point} point
		 * @returns {ViewportModule.ViewportPanel} the panel at the requested point
		 */
		ViewportWidget.prototype.getPanelAt = function (point) {
			point = point.copy();
			var offset = this._element.offset();
			point.x -= offset.left;
			point.y -= offset.top;

			var panelsLayoutMode = this._widgetModel.getResolvedPanelsLayoutMode();
			var panelIndex = 0;
			if (panelsLayoutMode === ViewportConstants.PanelsLayoutMode.HORIZONTAL) {
				panelIndex = Math.floor(point.y / (Math.ceil(this._element.height() / this._widgetModel.getPanelCount()) + ViewportConstants.PanelBorderWidth));
			} else {
				panelIndex = Math.floor(point.x / (Math.ceil(this._element.width() / this._widgetModel.getPanelCount()) + ViewportConstants.PanelBorderWidth));
			}

			return this._panelObjects[this._widgetModel.getPanelNameAtIndex(panelIndex)];
		};


		/**
		 * Layout the panels respecting the settings in the WidgetModel
		 * @private
		 */
		ViewportWidget.prototype._layout = function () {
			if (this._widgetModel !== undefined && this._widgetModel.getWorldBounds()) {
				var panelOrder = this._widgetModel.getPanelOrder();
				var panelCount = panelOrder.length;
				if (panelCount > 0) {
					var panelsLayoutMode = this._widgetModel.getResolvedPanelsLayoutMode();

					if (panelsLayoutMode === ViewportConstants.PanelsLayoutMode.HORIZONTAL) {
						// vertical tiling
						this._layoutPanelsHorizontal(panelOrder, panelCount);
					} else {
						// horizontal tiling
						this._layoutPanelsVertical(panelOrder, panelCount);
					}
				}
			}
		};

		/**
		 * Layout the panels in vertical tiled mode (left to right)
		 * @private
		 * @param {Array.<String>} panelOrder - order of the panels (with names in the array)
		 * @param {Number} panelCount - total number of panels
		 */
		ViewportWidget.prototype._layoutPanelsVertical = function (panelOrder, panelCount) {
			var panelSize = new ShapeUtils.Size(Math.ceil((this._element.width() - ((panelCount - 1) * ViewportConstants.PanelBorderWidth)) / panelCount), this._element.height());

			_.each(panelOrder, function (panelName, index) {
				var panel = this.getPanel(panelName);
				panel.setBounds(new ShapeUtils.Point((panelSize.width + ViewportConstants.PanelBorderWidth) * index, 0), panelSize);
				if (index < panelCount - 1) {

					panel.element.css("border-right-width", ViewportConstants.PanelBorderWidth + "px");
					panel.element.css("border-right-style", "solid");
				}
				panel.element.css("border-bottom", 0);
			}, this);
		};

		/**
		 * Layout the panels in horizontal tiled mode (top to bottom)
		 * @private
		 * @param {Array.<String>} panelOrder - order of the panels (with names in the array)
		 * @param {Number} panelCount - total number of panels
		 */
		ViewportWidget.prototype._layoutPanelsHorizontal = function (panelOrder, panelCount) {
			var panelSize = new ShapeUtils.Size(this._element.width(), Math.ceil((this._element.height() - ((panelCount - 1) * ViewportConstants.PanelBorderWidth)) / panelCount));

			_.each(panelOrder, function (panelName, index) {
				var panel = this.getPanel(panelName);
				var point = new ShapeUtils.Point(0, (panelSize.height + ViewportConstants.PanelBorderWidth) * index);
				panel.setBounds(point, panelSize);
				if (index < panelCount - 1) {
					panel.element.css("border-bottom-width", ViewportConstants.PanelBorderWidth + "px");
					panel.element.css("border-bottom-style", "solid");
				}
				panel.element.css("border-right", 0);
			}, this);
		};


		/**
		 * Repaint all panels/views/layers (private! use .repaint())
		 * @private
		 */
		ViewportWidget.prototype._repaint = function () {
			if (this._widgetModel && this._widgetModel.getWorldBounds()) {
				this._verifyWorld();

				_.each(this._widgetModel.getPanelOrder(), function (panelName, index) {
					var panel = this.getPanel(panelName);
					panel.repaint();
				}, this);
			}
		};

		/**
		 * Get the first occurrence of a certain view
		 * @param {String} viewName - name if the view
		 * @returns {ViewportView}
		 */
		ViewportWidget.prototype.getFirstViewOccurrence = function (viewName) {
			var panelOrder = this._widgetModel.getPanelOrder();
			for (var i = 0; i < panelOrder.length; i++) {
				var panel = this._panelObjects[panelOrder[i]];
				if (panel) {
					var view = panel.getViewIfExists(viewName);
					if (view) {
						return view;
					}
				}
			}
			return null;
		};

		/**
		 * Get the element of the ViewportWidget
		 * @returns {JQueryObj} the element
		 */
		ViewportWidget.prototype.getElement = function () {
			return this._element;
		};

		/**
		 * Set the background color of the viewport widget
		 * @param {String} [hexColor] - New background color in hex format without # or none to set none
		 */
		ViewportWidget.prototype.setBackgroundColor = function (hexColor) {
			if (hexColor === undefined) {
				hexColor = "";
			} else {
				hexColor = "#" + hexColor;
			}
			this._element.css({
				'background-color': hexColor
			});
		};

		return ViewportWidget;
	});

define('web-viewer/core/AbstractModel',["web-viewer/core/PrivateChannel",
		"underscore"
	],
	function (PrivateChannel, _) {
		/**
		 * @memberof Core
		 * @class AbstractModel - abstract model, providing base model functionality
		 * @constructor
		 * @mixes Core.PrivateChannel
		 * @param {Core.ViewerFacade} f
		 * @param {String} channelName - the channel on which the model will send out events
		 */
		var AbstractModel = function (f, channelName) {
			this.f = f;
			this.channelName = channelName;

			PrivateChannel.installTo(this, this.f, this.channelName);
		};

		/**
		 * Fire a model change, if not in silent mode and oldValue and newValue are not the same.
		 * The even will be fired on channelName.changed.[what] if oldValue and newValue are not the same
		 * 
		 * @param {Boolean} silent - if true no event will be fired
		 * @param {Array.<String>} what - specific description of which property has changed, this is an array i.e. ["channel", channelID, "enabled"] the event will be fired in the channel privateChannel.changed.channel.channelID.enabled
		 * @param {Any} oldValue - the old value before the change
		 * @param {Any} newValue - the new value after the change
		 * @param {Any} extraArguments - Events can be fired with extra arguments, just put them after newValue
		 */
		AbstractModel.prototype._fireChange = function (silent, what, oldValue, newValue) {
			if (!silent) {
				var didChange = false;

				if (oldValue === newValue) {
					didChange = false; // identical objects are always the same... (this is check is here for "extra" performance)
				} else if (newValue !== undefined && oldValue !== undefined && newValue !== null && oldValue !== null && _.isFunction(newValue.equals) && newValue.constructor === oldValue.constructor) {

					// an equals function was defined, and both objects are of the same type
					didChange = !oldValue.equals(newValue);
				} else if (_.isArray(oldValue) && _.isArray(newValue)) {
					// array comparison
					if (oldValue.length === newValue.length) {
						for (var i = 0; i < oldValue.length && !didChange; i++) {
							if (oldValue[i] != newValue[i]) {
								didChange = true;
							}
						}
					} else {
						// non equal length arrays
						didChange = true;
					}
				} else {
					didChange = (oldValue !== newValue);
				}


				// if there is an equals method, use this for comparison, otherwise compare the primitive types
				if (didChange) {
					arguments[1].unshift("changed");
					this._emit.apply(this, _.rest(arguments));
				}
			}
		};


		/**
		 * Install the _fireChange method in to an object, it will use the privateChannel of object, append "channelname" and what to it
		 * @param {Object} object - the object on which the _fireChange method should be installed 
		 * @param {Array.<String>} channelName - the name of the channel for events --> [privateChannel, channelName, what]
		 */
		AbstractModel.mixin = function (object) {
			object._fireChange = function (silent, what, oldValue, newValue) {
				AbstractModel.prototype._fireChange.apply(this, silent, what, oldValue, newValue);
			};
		};

		return AbstractModel;
	});

/**
 * @namespace ViewportModule
 */
define('web-viewer/viewport/model/ViewportPanelModel',["web-viewer/core/AbstractModel",
		"underscore"
	],
	function (AbstractModel,
		_) {
		"use strict";

		/**
		 * @exports ViewportPanelModel
		 * @memberof ViewportModule
		 * @class ViewportPanelModel - a model for panels
		 * @augments Core.AbstractModel
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {String} name - the name of the panel
		 * @param {Object} [overwriteOptions] - key-value pairs with overwrite options
		 * @param {Array.<String>} [disabledLayers] - layers to disable
		 */
		var ViewportPanelModel = function (f, name, overwriteOptions, disabledLayers) {
			AbstractModel.call(this, f, ["module", "viewport", "panel", name]);
			/**
			 * @private
			 * @member {String} name of the panel
			 */
			this._name = name;

			/**
			 * @private
			 */
			this._overwriteOptions = {};
			if (overwriteOptions !== undefined) {
				this._overwriteOptions = _.clone(overwriteOptions);
			}

			/**
			 * @private
			 */
			this._disabledLayers = [];
			if (disabledLayers !== undefined) {
				this._disabledLayers = disabledLayers.slice(0);
			}

			/**
			 * @private
			 * @member {String|null} Text of the header of the panel
			 */
			this._headerText = null;
		};

		ViewportPanelModel.prototype = Object.create(AbstractModel.prototype);
		ViewportPanelModel.prototype.constructor = ViewportPanelModel;

		/**
		 * Get the name of the panel
		 * @returns {String}
		 */
		ViewportPanelModel.prototype.getName = function () {
			return this._name;
		};

		/**
		 * Toggle a layer on all views (the layer will be hidden)
		 * @param {String} name
		 * @param {Boolean} toggle
		 */
		ViewportPanelModel.prototype.toggleLayer = function (name, toggle) {
			var oldValue = this._disabledLayers.slice(0);

			if (toggle) {
				// show the layer
				this._disabledLayers = _.without(this._disabledLayers, name);
			} else {
				if (_.indexOf(this._disabledLayers, name) < 0) {
					this._disabledLayers.push(name);
				}
			}

			this._fireChange(false, ["disabledLayers"], oldValue, this._disabledLayers.slice(0));
		};

		/**
		 * Check if a layer is enabled
		 * @param {String} name - the name of the layer
		 * @returns {Boolean}
		 */
		ViewportPanelModel.prototype.isLayerEnabled = function (name) {
			return !this.isLayerDisabled(name);
		};

		/**
		 * Check if a layer is disabled
		 * @param name - the name of the layer
		 * @returns {Boolean} 
		 */
		ViewportPanelModel.prototype.isLayerdisabled = function (name) {
			return _.indexOf(this._disabledLayers, name) >= 0;
		};

		/**
		 * Get a list of disabled layers
		 * @returns {Array.<String>}
		 */
		ViewportPanelModel.prototype.getDisabledLayers = function () {
			return this._disabledLayers.slice(0);
		};


		/**
		 * Get all the overwrite options for this panel
		 * 
		 * By default a copy of these options is returned, for performance reasons "doNotClone" can be used
		 * @param {Boolean} doNotClone - if true the options are passed as reference, they should not be adapted in this case!! 
		 * @returns {object} the overwrite options as a dictionary 
		 */
		ViewportPanelModel.prototype.getOverwriteOptions = function (doNotClone) {
			if (doNotClone) {
				return this._overwriteOptions;
			} else {
				return _.clone(this._overwriteOptions);
			}
		};

		/**
		 * Get the value for a specific overwrite option
		 * @param {String} name - name of the overwite option
		 * @returns {Any} the value of the overwrite option
		 */
		ViewportPanelModel.prototype.getOverwriteOption = function (name) {
			return this._overwriteOptions[name];
		};

		/**
		 * Set a specific overwrite option
		 * @param {String} name - name of the overwrite option
		 * @param {Any} value - value of the overwrite option (probably a String value)
		 */
		ViewportPanelModel.prototype.setOverwriteOption = function (name, value) {
			var oldValue = this.getOverwriteOption(name);
			this._overwriteOptions[name] = value;

			this._fireChange(false, ["overwriteOption", name], oldValue, value);
		};

		/**
		 * Set multiple overwrite options in one go
		 * @param {Object} options - object with key/value pairs
		 */
		ViewportPanelModel.prototype.setOverwriteOptions = function (options) {
			_.each(options, function (value, name) {
				this.setOverwriteOption(name, value);
			}, this);
		};

		/**
		 * Delete a specific overwrite option
		 * @param {String} name - name of the overwrite option to delete
		 */
		ViewportPanelModel.prototype.deleteOverwriteOption = function (name) {
			var oldValue = this.getOverwriteOption(name);
			delete this._overwriteOptions[name];

			// intentional use of undefined!
			this._fireChange(false, ["overwriteOption", name], oldValue, undefined);
		};

		/**
		 * Get the header text for the panel
		 * @returns {String|null} the header text or null if not defined
		 */
		ViewportPanelModel.prototype.getHeaderText = function () {
			return this._headerText;
		};

		/**
		 * Change the header text
		 * @param {String|null} headerText
		 */
		ViewportPanelModel.prototype.setHeaderText = function (headerText) {
			var oldValue = this._headerText;

			this._headerText = headerText;
			this._fireChange(false, ["headerText"], oldValue, headerText);
		};

		return ViewportPanelModel;
	});

define('web-viewer/viewport/model/ViewportWidgetModel',["web-viewer/core/AbstractModel",
		"utils/ShapeUtils",
		"web-viewer/viewport/ViewportConstants",
		"utils/OrderedObject",
		"web-viewer/viewport/model/ViewportPanelModel",
		"underscore"
	],
	function (AbstractModel,
		ShapeUtils,
		ViewportConstants,
		OrderedObject,
		ViewportPanelModel,
		_) {
		"use strict";
		/**
		 * @exports ViewportWidgetModel
		 * @memberof ViewportModule
		 * @class A widget model is an abstract class that manages the views on one widget
		 * @constructor
		 * @param {Core.ViewerFacade} f - Facade
		 * @param {String} id - unique id
		 */
		var ViewportWidgetModel = function (f, id) {
			AbstractModel.call(this, f, ['viewport', 'widget', 'model']);

			// id of the widget
			this.id = id;

			/**
			 * collection of ViewportViewModel
			 * @private
			 * @type {OrderedObject.<ViewportModule.ViewportViewModel>}
			 */
			this._viewportViewModels = new OrderedObject();

			/**
			 * Keep track of the global layers that can be added to newly created views
			 * @private
			 */
			this._globalLayers = {};

			/**
			 * The current views layout mode
			 * @private
			 */
			this._viewsLayoutMode = ViewportConstants.ViewsLayoutMode.AUTOMATIC;
			this._panelsLayoutMode = ViewportConstants.PanelsLayoutMode.AUTOMATIC;
			this._sizeForView = new ShapeUtils.Size();

			/**
			 * The panels in the widget, by default there is only 1 panel with no special options
			 * This is a list of PanelModel!
			 * @private
			 * @type {OrderedObject.<ViewportModule.ViewportPanelModel>}
			 */
			this._panels = new OrderedObject();
			this.addPanel("default", {
				isDefaultView: true
			});

			/**
			 * Is the mouse active in the viewport at this moment.
			 */
			this._isActive = false;


		};

		ViewportWidgetModel.prototype = Object.create(AbstractModel.prototype);
		ViewportWidgetModel.prototype.constructor = ViewportWidgetModel;


		//------------ general ------------------------------------------------------------------------------

		/**
		 * @returns {Number} the size of the World
		 */
		ViewportWidgetModel.prototype.getWorldSize = function () {
			return undefined;
		};

		/**
		 * @returns {ShapeUtils.Rect} the rect of the World
		 */
		ViewportWidgetModel.prototype.getWorldBounds = function () {
			return undefined;
		};


		/**
		 * Get the resolution of the loaded document in the world
		 * @returns {Object} resolutions of the document in ppmm
		 * {x: {Number} x-resolution,
		 *  y: {Number} y-resolution }
		 */
		ViewportWidgetModel.prototype.getWorldResolution = function () {
			return {
				x: 0,
				y: 0
			};
		};

		/**
		 * Set if the mouse is currently active in the viewport widget
		 * @param isActive
		 */
		ViewportWidgetModel.prototype.setWidgetActive = function (isActive) {
			this._isActive = isActive;
		};

		/**
		 * Is the viewport widget active, it the mouse cursor in the viewport
		 * @returns {Boolean}
		 */
		ViewportWidgetModel.prototype.isWidgetActive = function () {
			return this._isActive;
		};



		//------------ views ------------------------------------------------------------------------------


		/**
		 * Create a new view with options
		 * @param {String} id - the id of the view
		 * @param {String} title - the title of the view
		 * @param {Dictionary} isDefaultView - are the view with this viewmodel default views
		 */
		ViewportWidgetModel.prototype.createView = function (id, title, isDefaultView) {
			var model = this.createModelForView(id, title, isDefaultView);
			this._addView(id, model);
		};

		/**
		 * Add a view to the model
		 * (private function: use "createView" to generate one)
		 */
		ViewportWidgetModel.prototype._addView = function (viewID, model) {
			// If it already exists
			if (this._viewportViewModels.getItemByKey(viewID)) {
				// update the view
				throw "Trying to add view with already existing name";
			} else {
				// Add it
				var oldOrder = this.getViewsOrder();
				this._viewportViewModels.addItem(viewID, model);

				// Add all global layers
				_.each(this._globalLayers, function (layer, layername) {
					this.addLayer(layername, layer, viewID);
				}, this);

				this._fireChange(false, ["view", "add"], undefined, model);
				this._fireChange(false, ["view", "order"], oldOrder, this.getViewsOrder());
			}
		};

		/**
		 * Hide or show the view with the specified id
		 * @param {String} id - Id of the view
		 * @param {Boolean} show - True if the view should be shown, false otherwise
		 */
		ViewportWidgetModel.prototype.toggleView = function (id, show) {
			if (this._viewportViewModels.getItemByKey(id) !== undefined) {
				var oldValue = this._viewportViewModels.getItemByKey(id).isShown();

				this._viewportViewModels.getItemByKey(id).setShown(show);
				this._fireChange(false, ["view", "toggle", id], oldValue, show);
			}
		};

		/**
		 * Removes a view with a certain id
		 * @param {String} viewID - id of the view
		 */
		ViewportWidgetModel.prototype.removeView = function (viewID) {
			var oldOrder = this.getViewsOrder();
			var model = this._viewportViewModels.removeByKey(viewID);
			this._fireChange(false, ["view", "remove"], model, undefined);
			this._fireChange(false, ["view", "order"], oldOrder, this.getViewsOrder());
		};

		/**
		 * @returns {Number} number of viewportViewModels in the model
		 */
		ViewportWidgetModel.prototype.getViewCount = function () {
			return this._viewportViewModels.getLength();
		};

		/**
		 * @returns {ViewportModule.ViewportViewModel} a view with a certain id
		 */
		ViewportWidgetModel.prototype.getViewModel = function (viewID) {
			return this._viewportViewModels.getItemByKey(viewID);
		};

		/**
		 * Note, if you are only interested in visible views use getIDForShownViewsAtIndex
		 * @returns {String} id of the view at a certain index
		 */
		ViewportWidgetModel.prototype.getIDForViewAtIndex = function (index) {
			return this._viewportViewModels.getKeyByIndex(index);
		};


		/**
		 * @returns {Viewportmodule.ViewportViewModel} the model for a view at a certain index
		 */
		ViewportWidgetModel.prototype.getModelForViewAtIndex = function (index) {
			return this._viewportViewModels.getItemByIndex(index);
		};

		/**
		 * Get all available views as a list
		 * @returns {Array.<ViewportModule.ViewportViewModel>}
		 */
		ViewportWidgetModel.prototype.getViews = function () {
			return this._viewportViewModels.getOrderedObjectList();
		};

		/**
		 * Get the keys of all the views that are visible
		 * @returns {Array.<String>}
		 */
		ViewportWidgetModel.prototype.getShownViewKeys = function () {
			return _.filter(this._viewportViewModels.getKeys(), function (key) {
				return this._viewportViewModels.getItemByKey(key).isShown();
			}, this);
		};

		/**
		 * This funciton will return the key of the view at a certain index
		 * This method only scans through visible views (use getIDForViewAtIndex if also hidden views should be taken into account)
		 * @param index
		 * @returns
		 */
		ViewportWidgetModel.prototype.getIDForShownViewsAtIndex = function (index) {
			// only use visible views to determine the key of the  view at position index
			var shownViewKeys = this.getShownViewKeys();
			return shownViewKeys[index];
		};

		/**
		 * Create a model for a view (Method should be overwritten by the implementation) ViewportViewModel
		 * @param {String} id - id of the view
		 * @param {String} title - Title of the view
		 * @param {Boolean} isDefaultView - are the view with this view model default views
		 */
		ViewportWidgetModel.prototype.createModelForView = function (id, title, isDefaultView) {
			throw "Not Implemented";
		};

		/**
		 * Set the order of the views
		 * Probably this method should be moved to ordered object (where it can be unit tested!)
		 *
		 * @param {Array.<String>} ordered keys
		 */
		ViewportWidgetModel.prototype.setViewsOrder = function (orderedKeys) {
			var oldOrder = this.getViewsOrder();


			var originalKeys = this._viewportViewModels.getKeys();

			var nonMentionedKeys = _.difference(originalKeys, orderedKeys, ["Main"]);

			if (_.indexOf(originalKeys, "Main") >= 0) {
				orderedKeys = _.without(orderedKeys, "Main");
				orderedKeys.unshift("Main");
			}

			orderedKeys = orderedKeys.concat(nonMentionedKeys);

			this._viewportViewModels.setOrder(orderedKeys);

			this._fireChange(false, ["view", "order"], oldOrder, this.getViewsOrder());
		};

		/**
		 * Get the order of the views (in keys)
		 * @returns {Array.<String>}
		 */
		ViewportWidgetModel.prototype.getViewsOrder = function () {
			return this._viewportViewModels.getKeys();
		};

		/**
		 * Set the currently used size for a view
		 * @param size
		 */
		ViewportWidgetModel.prototype.setSizeForView = function (size) {
			var oldSize = this._sizeForView;
			this._sizeForView = size;
			this._fireChange(false, ["view", "size"], oldSize, size);
		};

		/**
		 * Get the currently set view size (this is the same for all the views)
		 * @returns {ShapeUtils.Size}
		 */
		ViewportWidgetModel.prototype.getSizeForView = function () {
			return this._sizeForView;
		};



		//------------ views layout mode ------------------------------------------------------------------------------

		/**
		 * Get the current views layout mode (this is how views are shown inside a panel)
		 *
		 * @returns {ViewportConstants.ViewsLayoutMode} Current views layout mode
		 */
		ViewportWidgetModel.prototype.getViewsLayoutMode = function () {
			return this._viewsLayoutMode;
		};

		/**
		 * Set a new layout mode for the views
		 * @param {ViewportConstants.ViewsLayoutMode} viewsLayoutMode - The new views layout mode
		 */
		ViewportWidgetModel.prototype.setViewsLayoutMode = function (viewsLayoutMode) {
			var oldValue = this._viewsLayoutMode;

			this._viewsLayoutMode = viewsLayoutMode;
			this._fireChange(false, ["layout", "views"], oldValue, this._viewsLayoutMode);
		};

		/**
		 * Get the current panels layout mode (With AUTOMATIC resolved to the correct mode)
		 * (this depends on panelslayout mode and count of panels, if there is 1 panel just use automatic (or other setting)
		 * if more than one panel choose the best option
		 */
		ViewportWidgetModel.prototype.getResolvedViewsLayoutMode = function () {
			if (this.getPanelCount() > 1 &&
				this._viewsLayoutMode === ViewportConstants.PanelsLayoutMode.AUTOMATIC) {

				if (this._panelsLayoutMode === ViewportConstants.PanelsLayoutMode.HORIZONTAL) {
					return ViewportConstants.ViewsLayoutMode.VERTICAL;
				} else {
					return ViewportConstants.ViewsLayoutMode.HORIZONTAL;
				}

			} else {
				return this._viewsLayoutMode;
			}
		};


		//------------ panels layout mode ------------------------------------------------------------------------------
		/**
		 * Get the current panels layout mode (this is how panels are shown inside the viewport )
		 * @returns {ViewportConstants.PanelsLayoutMode} Current panels layout mode
		 */
		ViewportWidgetModel.prototype.getPanelsLayoutMode = function () {
			return this._panelsLayoutMode;
		};

		/**
		 * Get the current panels layout mode (With AUTOMATIC resolved to the correct mode)
		 * @returns {ViewportConstants.PanelsLayoutMode}
		 */
		ViewportWidgetModel.prototype.getResolvedPanelsLayoutMode = function () {
			var mode = this._panelsLayoutMode;
			if (mode === ViewportConstants.PanelsLayoutMode.AUTOMATIC) {
				if (this._viewsLayoutMode === ViewportConstants.ViewsLayoutMode.VERTICAL) {
					mode = ViewportConstants.PanelsLayoutMode.HORIZONTAL;
				} else {
					// automatic views layout or vertical
					mode = ViewportConstants.PanelsLayoutMode.VERTICAL;
				}
			}
			return mode;
		};

		/**
		 * Set a new layout mode for the panels
		 * @param {ViewportConstants.PanelsLayoutMode} panelsLayoutMode - The new panels layout mode
		 */
		ViewportWidgetModel.prototype.setPanelsLayoutMode = function (panelsLayoutMode) {
			var oldValue = this._panelsLayoutMode;

			this._panelsLayoutMode = panelsLayoutMode;
			this._fireChange(false, ["layout", "panels"], oldValue, this._panelsLayoutMode);
		};

		//------------ panels  ------------------------------------------------------------------------------
		/**
		 * Add panel, the panel is added in the end of the list
		 * if the panel already exists, the panel is moved to the end
		 * @param {String} name - name of the panel to add.
		 * @param {Object} [overwriteOptions] - key-value pairs with overwrite options
		 * @param {Array.<String>} [disabledLayers] - layers to disable
		 * @returns {ViewportModule.ViewportPanelModel}
		 */
		ViewportWidgetModel.prototype.addPanel = function (name, overwriteOptions, disabledLayers) {
			var oldOrder = this._panels.getKeys();
			var panel = this._panels.getItemByKey(name);
			if (panel !== undefined) {
				// move the item
				this._panels.moveItemByKeyToEnd(name);
			} else {
				panel = new ViewportPanelModel(this.f, name, overwriteOptions, disabledLayers);
				this._panels.addItem(name, panel);
				this._fireChange(false, ["panel", "add"], undefined, panel);
			}

			this._fireChange(false, ["panel", "order"], oldOrder, this._panels.getKeys());
			return panel;
		};

		/**
		 * Get the model for a specific panel
		 * @param {String} name - name of the panel
		 * @returns {ViewportModule.ViewportPanelModel}
		 */
		ViewportWidgetModel.prototype.getPanelModel = function (name) {
			return this._panels.getItemByKey(name);
		};

		/**
		 * Remove a panel
		 * @param {String} name - name of the panel to remove
		 */
		ViewportWidgetModel.prototype.removePanel = function (name) {
			var oldOrder = this._panels.getKeys();

			var panel = this._panels.removeByKey(name);

			this._fireChange(false, ["panel", "remove"], panel, undefined);
			this._fireChange(false, ["panel", "order"], oldOrder, this._panels.getKeys());
		};

		/**
		 * Set the order of the panels
		 * !! all panel keys should be in the new order!!
		 * @param {Array.<String>} newOrder - order defined as a list of panel names
		 */
		ViewportWidgetModel.prototype.setPanelOrder = function (newOrder) {
			var oldOrder = this._panels.getKeys();
			this._panels.setOrder(newOrder);
			this._fireChange(false, ["panel", "order"], oldOrder, this._panels.getKeys());
		};

		/**
		 * Get the order of the panels
		 * @returns {Array.<String>}
		 */
		ViewportWidgetModel.prototype.getPanelOrder = function () {
			return this._panels.getKeys();
		};

		/**
		 * Get the amount of active panels
		 * @returns {Number}
		 */
		ViewportWidgetModel.prototype.getPanelCount = function () {
			return this._panels.getLength();
		};

		/**
		 * Get the panel (name) at a certain index
		 * @param {Number} index
		 * @returns {String} the name of the panel at position "index"
		 */
		ViewportWidgetModel.prototype.getPanelNameAtIndex = function (index) {
			return this._panels.getKeyByIndex(index);
		};

		/**
		 * Get the panel model at a certain index
		 * @param {Number} index
		 * @returns {ViewportModule.PanelModel} panel model for the panel at position "index"
		 */
		ViewportWidgetModel.prototype.getPanelAtIndex = function (index) {
			return this._panels.getItemByIndex(index);
		};

		//------------ layers ------------------------------------------------------------------------------

		/**
		 * Add a layer to the views
		 * @param {String} layerName - name of the layer
		 * @param {LayerModel} layerModel - model of the layer
		 * @param {String} [viewID] - if specified, only add the layer to the view with this id. If not specified, add it to all views (and all future views)
		 */
		ViewportWidgetModel.prototype.addLayer = function (layerName, layerModel, viewID) {
			if (viewID !== undefined) {
				// Only add it to the specified view
				if (this._viewportViewModels.getItemByKey(viewID)) {
					this._viewportViewModels.getItemByKey(viewID).addLayer(layerName, layerModel);
				}
			} else {
				// Add it already to the known views
				_.each(this._viewportViewModels.getItemsAsObject(), function (view) {
					view.addLayer(layerName, layerModel);
				});

				// Save it to add it later to new views
				this._globalLayers[layerName] = layerModel;
			}
		};

		/**
		 * Remove a layer from the views
		 * @param {String} layerName - name of the layer
		 * @param {String} [viewID] - if specified, only remove the layer from the view with this id. If not specified, remove it from all views
		 */
		ViewportWidgetModel.prototype.removeLayer = function (layerName, viewID) {
			if (viewID !== undefined) {
				if (this._viewportViewModels.getItemByKey(viewID)) {
					this._viewportViewModels.getItemByKey(viewID).removeLayer(layerName);
				}
			} else {
				_.each(this._viewportViewModels.getItemsAsObject(), function (view) {
					view.removeLayer(layerName);
				});
				delete this._globalLayers[layerName];
			}
		};

		/**
		 * Show/hide a layer
		 * If view is provided, only the layer in that view will be toggled
		 * @param {String} layerName
		 * @param {Boolean} visible
		 * @param {String} [viewID] - if provided, only the layer in this view will be toggled
		 */
		ViewportWidgetModel.prototype.toggleLayer = function (layerName, visible, viewID) {
			if (viewID !== undefined) {
				if (this._viewportViewModels.getItemByKey(viewID)) {
					this._viewportViewModels.getItemByKey(viewID).toggleLayer(layerName);
				}
			} else {
				var layer = this._globalLayers[layerName];
				if (layer) {
					layer.toggle(visible);
				} else {
					// it is not a global layer, try to toggle it by looking in all views
					_.each(this._viewportViewModels.getItemsAsObject(), function (view) {
						view.toggleLayer(layerName, visible);
					});
				}

			}
		};

		return ViewportWidgetModel;
	});

/**
 *  @namespace ViewportModule
 */
define('web-viewer/viewport/ViewportModule',["web-viewer/core/AbstractModule",
		"web-viewer/viewport/ViewportWidget",
		"web-viewer/viewport/model/ViewportWidgetModel",
		"web-viewer/viewport/ViewportWorld",
		"web-viewer/utils/ViewerUtils",
		"utils/ShapeUtils",
		"utils/StringSet",
		"jquery",
		"utils/Cache",
		"web-viewer/utils/ListenerUtil",
		"underscore"
	],
	function (AbstractModule,
		ViewportWidget,
		ViewportWidgetModel,
		ViewportWorld,
		ViewerUtils,
		ShapeUtils,
		StringSet,
		$,
		Cache,
		ListenerUtil,
		_) {
		"use strict";

		/**
		 * @constant
		 */
		var IMAGE_CACHE_SIZE = 75;

		/**
		 * @exports ViewportModule
		 * @memberof ViewportModule
		 * @augments Core.AbstractModule
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {Boolean} silenced
		 */
		var ViewportModule = function (f, silenced) {
			AbstractModule.call(this, f, silenced, "viewport");
			ListenerUtil.mixin(this, f);
			this._initializeImageCache();

			this.widgetModel = null;

			this._disabledAnimationSources = [];

			// how many image requests are ongoing?
			this._ongoingImageDataLoads = 0;
			// how many layers might start image requests?
			this._pendingImageRequests = new StringSet();

			this.world = new ViewportWorld(this.f, "viewportWidget", f.getProperty('initialZoom'));
			this.widget = new ViewportWidget(f, this.world);
			f.requestModule("featureDetector", function (featureDetector) {
				// Check if canvas available
				if (featureDetector.hasCanvas()) {
					f.debug.log("Canvas feature available");
				} else {
					f.debug.log("Canvas feature NOT available");
					// Show error message with possibility to go to java viewer
					f.i18n.loadBundle("viewer", function () { // to be sure we have the message available
						f.emit("userMessage", "error", f.i18n.t("CanvasNotAvailable"), f.getProperty("javaViewerURL"));
					});
				}
			});

			// viewport widget is always created, otherwise we might run in the concurrency problems...
			this._publishWidget(ViewportWidget.widgetName, _.bind(function () {
				return this.widget;
			}, this));

			// listen to viewport. should be deprecated...
			this._registerListener("viewport.transform", this.transform, this);
			this._registerListener("viewport.toggleAnimation", this.toggleAnimations, this);
			this._registerListener("viewport.zoomToFit", function (options) {
				this.zoomToFit(options.animated);
			}, this);
			this._registerListener("viewport.zoomToFitWidth", function (options) {
				this.zoomToFitWidth(options.animated);
			}, this);
			this._registerListener("viewport.zoomToFitHeight", function (options) {
				this.zoomToFitHeight(options.animated);
			}, this);
			this._registerListener("viewport.zoomOneToOne", function (options) {
				this.zoomOneToOne(options.animated, options.worldResolution);
			}, this);
			this._registerListener("viewport.fitInRect", function (options) {
				this.fitInRect(options.rect, options.animated, options.constrain);
			}, this);
			this._registerListener("viewport.repaint", function () {
				if (this.widget) {
					this.widget.repaint();
				}
			}, this);

			this._registerListener("viewportLayer.startImageDataRequest", function (id) {
				this._pendingImageRequests.add(id);

				if (this._ongoingImageDataLoads === 0) {
					this._emit("startLoadingImageData");
				}
			}, this);

			this._registerListener("viewportLayer.finishedImageDataRequest", function (id) {
				var isRemoved = this._pendingImageRequests.remove(id);

				if (isRemoved && this._pendingImageRequests.size() === 0 && this._ongoingImageDataLoads === 0) {
					this._emit("finishedLoadingImageData");
				}
			}, this);


			this.f.getDocumentController().whenActivePageChanged(function () {
				if (this.world) {
					this.world.resetOrientation();

				}
			}, this);
			this.f.getDocumentController().whenActiveOrDifferenceOrCadOverlayChangedAndParametersAvailable(function () {
				this.world.zoomToFit(false);
			}, this);
		};


		ViewportModule.prototype = Object.create(AbstractModule.prototype);
		ViewportModule.prototype.constructor = ViewportModule;


		/**
		 * Sets the viewportwidget model
		 * @param {ViewportWidgetModel} widgetModel
		 */
		ViewportModule.prototype.setWidgetModel = function (widgetModel) {
			this.widget.setWidgetModel(widgetModel);
			this.widgetModel = widgetModel;

			if (this.widgetModel) {
				this._emit("widgetModelAvailable");
			}
		};

		/**
		 * Returns the widget model
		 * @returns {ViewportWidgetModel}
		 */
		ViewportModule.prototype.getWidgetModel = function () {
			return this.widgetModel;
		};

		/**
		 * Initialize the image cache
		 */
		ViewportModule.prototype._initializeImageCache = function () {
			this._imageCache = new Cache(IMAGE_CACHE_SIZE, true);

			// in readerspread mode we don't want to reload an image if the active/diff page have switched position
			this._imageCache.getAlternativeName = function (name) {
				// we only need to supply an alternative if we are in readerspread mode (dmode=7 and rs=1|2)
				if (name.indexOf("dmode=7") > 0 && name.indexOf("rs=") > 0) {
					// switch file and diff parameters and flip rs 1->2 or 2->1
					var file = name.replace(/.*file=([^&]+).*/, '$1');
					var diff = name.replace(/.*diff=([^&]+).*/, '$1');
					var rs = name.replace(/.*rs=([^&]+).*/, '$1');
					return name.replace(/file=([^&]+)/, "file=" + diff).replace(/diff=([^&]+)/, "diff=" + file).replace(/rs=([^&]+)/, "rs=" + (rs == '2' ? 1 : 2));
				} else {
					// no alternative found
					return null;
				}
			};

			var viewportModule = this;

			/**
			 * Load an item if it is not yet in the cache
			 */
			this._imageCache._loadItem = _.partial(function (f, name) {
				//>>excludeStart("PExclude", pragmas.productionExclude);
				var error = new Error("Error receiving image of file: " + name);
				//>>excludeEnd("PExclude");

				// create new image			
				var image = new Image();
				var loadingID = "viewportImageCache_" + name;
				// both on onload and onerror, add item to the cache
				image.onload = _.bind(function () {
					this._registerItem(name, image);
					viewportModule._stopLoadingImageData(loadingID);
				}, this);
				image.onerror = _.bind(function () {

					// mark the image as failed
					image.error = true;
					this._registerItem(name, undefined);
					viewportModule._stopLoadingImageData(loadingID);

					//>>excludeStart("PExclude", pragmas.productionExclude);
					// in production we do not want to display "error receiving image"
					f.i18n.loadBundle("viewer", function () {
						f.emit("error.SEVERE", f.i18n.t("ViewReceiveError", ""));
					});

					// we also throw an exception, so we now from where the falsy image was requested...
					if (error) {
						throw error;
					}
					//>>excludeEnd("PExclude");	
				}, this);

				// start loading image	
				viewportModule._startLoadingImageData(loadingID);
				image.src = name;
			}, this.f);


		};

		/**
		 * Indicate the start of loading an image
		 * it sends a loading event (and starTLoadingImageData event)
		 * @param {String} loadingID - unique string for the loading request
		 */
		ViewportModule.prototype._startLoadingImageData = function (loadingID) {
			this.f.emit("loading.start", loadingID);
			if (this._ongoingImageDataLoads === 0 && this._pendingImageRequests.size() > 0) {
				this._emit("startLoadingImageData");
			}

			this._ongoingImageDataLoads++;
		};

		/**
		 * Indicate that the loading of an image has finsihed (or failed)
		 * @param {String} loadingID - unique string for the loading request
		 */
		ViewportModule.prototype._stopLoadingImageData = function (loadingID) {
			this.f.emit("loading.stop", loadingID);
			this._ongoingImageDataLoads--;

			if (this._ongoingImageDataLoads === 0) {
				this._emit("finishedLoadingImageData");
			}
		};

		/**
		 * Execute a callback when all image loading has finished
		 * @param {Function} callback  - called when no long fetching imag edata
		 * @param {Any}      [binding] - binding for the callback function
		 */
		ViewportModule.prototype.whenFinishedLoadingImageData = function (callback, binding) {
			if (binding) {
				callback = _.bind(callback, binding);
			}
			if (this._ongoingImageDataLoads === 0 && this._pendingImageRequests.size() === 0) {
				callback();
			} else {
				this.once("finishedLoadingImageData", callback);
			}
		};

		/**
		 * Get a reference to the ImageCache for the viewport
		 * @returns {Utils.Cache} the ImageChache (a cache of name->Image)
		 */
		ViewportModule.prototype.getImageCache = function () {
			return this._imageCache;
		};

		/**
		 * Create a new view with options
		 * @borrows ViewportModule.ViewportWidgetModel~createView
		 * @param {String} id - the id of the view
		 * @param {String} title - the title of the view
		 * @param {Boolean} isDefaultView - is the view the default view of the viewport (used to show popovers if no view is active)
		 */
		ViewportModule.prototype.createView = function (viewID, title, isDefaultView) {
			if (this.widgetModel) {
				this.widgetModel.createView(viewID, title, isDefaultView);
			}

		};

		/**
		 * Removes a view with a certain id
		 * @borrows ViewportModule.ViewportWidgetModel~removeView as removeView
		 * @param {String} viewID - id of the view
		 */
		ViewportModule.prototype.removeView = function (viewID) {
			if (this.widgetModel) {
				this.widgetModel.removeView(viewID);
			}
		};

		/**
		 * Get the first occurence from a view with a certain name
		 * @borrows ViewportModule.ViewportWidgetModel~getFirstViewOccurrence
		 * @param {String} viewName
		 * @returns {ViewportModule.ViewportView}
		 */
		ViewportModule.prototype.getFirstViewOccurrence = function (viewName) {
			if (this.widget) {
				return this.widget.getFirstViewOccurrence(viewName);
			}
		};


		/**
		 * Hide or show the view with the specified id
		 * @borrows ViewportModule.ViewportWidgetModel~toggleView
		 * @param {String} id - Id of the view
		 * @param {Boolean} show - True if the view should be shown, false otherwise
		 */
		ViewportModule.prototype.toggleView = function (viewID, toggle) {
			if (this.widgetModel) {
				this.widgetModel.toggleView(viewID, toggle);
			}
		};

		/**
		 * Set the order of the views
		 * @borrows ViewportModule.ViewportWidgetModel~setViewsOrder
		 * @param {Array.<String>} ordered keys
		 */
		ViewportModule.prototype.setViewsOrder = function (orderedKeys) {
			if (this.widgetModel) {
				this.widgetModel.setViewsOrder(orderedKeys);
			}
		};
		/**
		 * Set a new layout mode for the views
		 * @borrows ViewportModule.ViewportWidgetModel~setViewsLayoutMode
		 * @param {ViewportConstants.ViewsLayoutMode} viewsLayoutMode - The new views layout mode
		 */
		ViewportModule.prototype.setViewsLayoutMode = function (viewsLayoutMode) {
			if (this.widgetModel) {
				this.widgetModel.setViewsLayoutMode(viewsLayoutMode);
			}
		};

		/**
		 * Add a layer (this can be for all views or for a specific view)
		 * @borrows ViewportModule.ViewportWidgetModel~addLayer
		 * @param {String} layerName - name of the layer
		 * @param {LayerModel} layerModel - model of the layer
		 * @param {String} [viewID] - if specified, only add the layer to the view with this id. If not specified, add it to all views (and all future views)
		 */
		ViewportModule.prototype.addLayer = function (layerName, layerModel, viewID) {
			if (this.widgetModel) {
				this.widgetModel.addLayer(layerName, layerModel, viewID);
			}
		};

		/**
		 * Remove a layer from the views
		 * @borrows ViewportModule.ViewportWidgetModel~removeLayer
		 * @param {String} layerName - name of the layer
		 * @param {String} [viewID] - if specified, only remove the layer from the view with this id. If not specified, remove it from all views
		 */
		ViewportModule.prototype.removeLayer = function (layerName, viewID) {
			if (this.widgetModel) {
				this.widgetModel.removeLayer(layerName, viewID);
			}
		};

		/**
		 * @borrows ViewportModule.ViewportWidgetModel~hideLayer
		 * @param {String} layerName
		 * @param {Boolean} visible
		 * @param {String} [viewName]
		 */
		ViewportModule.prototype.toggleLayer = function (layerName, visible, viewName) {
			if (this.widgetModel) {
				this.widgetModel.toggleLayer(layerName, visible, viewName);
			}
		};

		/**
		 * @borrows ViewportModule.ViewportWorld~transform
		 * @param {Object} options
		 */
		ViewportModule.prototype.transform = function (options) {
			this.world.transform(options);
		};

		/**
		 * @borrows ViewportModule.ViewportWorld~toggleAnimations
		 * @param toggle
		 */
		ViewportModule.prototype.toggleAnimations = function (source, toggle) {
			if (!toggle) {
				this._disabledAnimationSources.push(source);
			} else {
				this._disabledAnimationSources = _.without(this._disabledAnimationSources, source);
			}
			this.world.toggleAnimation(this._disabledAnimationSources.length === 0);
		};

		/**
		 * @borrows ViewportModule.ViewportWorld~zoomToFit
		 * @param {Boolean} animated
		 */
		ViewportModule.prototype.zoomToFit = function (animated) {
			this.world.zoomToFit(animated);
		};

		/**
		 * @borrows ViewportModule.ViewportWorld~zoomToFitWidth
		 * @param {Boolean} animated
		 */
		ViewportModule.prototype.zoomToFitWidth = function (animated) {
			this.world.zoomToFitWidth(animated);
		};

		/**
		 * @borrows ViewportModule.ViewportWorld~zoomToFitHeight
		 * @param {Boolean} animated
		 */
		ViewportModule.prototype.zoomToFitHeight = function (animated) {
			this.world.zoomToFitHeight(animated);
		};

		/**
		 * @borrows ViewportModule.ViewportWorld~zoomOneToOne
		 * @param {Boolean} animated
		 * @param {Number} [worldResolution]
		 */
		ViewportModule.prototype.zoomOneToOne = function (animated, worldResolution) {
			this.world.zoomOneToOne(animated, worldResolution);
		};

		/**
		 * @borrows ViewportModule.ViewportWorld~getZoomPercentage
		 * @returns {Number} The current zoom percentage
		 */
		ViewportModule.prototype.getZoomPercentage = function () {
			return this.world.getZoomPercentage();
		};

		/**
		 * @borrows ViewportModule.ViewportWorld~getNumberOfZoomLevels
		 * @returns {Number} The number of zoom levels
		 */
		ViewportModule.prototype.getNumberOfZoomLevels = function () {
			return this.world.getNumberOfZoomLevels();
		};

		/**
		 * @borrows ViewportModule.ViewportWorld~getMinZoom
		 * @returns {Number} The min zoom level of the world
		 */
		ViewportModule.prototype.getMinZoom = function () {
			return this.world.getMinZoom();
		};

		/**
		 * @borrows ViewportModule.ViewportWorld~getMaxZoom
		 * @returns {Number} The max zoom level of the world
		 */
		ViewportModule.prototype.getMaxZoom = function () {
			return this.world.getMaxZoom();
		};

		/**
		 * @borrows ViewportModule.ViewportWorld~getResolution
		 * @returns {Number}
		 */
		ViewportModule.prototype.getResolution = function () {
			return this.world.getZoom();
		};

		/**
		 * @borrows ViewportModule.ViewportWorld~getRotation
		 * @returns {Number}
		 */
		ViewportModule.prototype.getRotation = function () {
			return this.world.getRotation();
		};

		/**
		 * @borrows ViewportModule.ViewportWorld~isFlip
		 * @returns {Boolean}
		 */
		ViewportModule.prototype.isFlip = function () {
			return this.world.isFlip();
		};

		/**
		 * @borrows ViewportModule.ViewportWorld~fitInRect
		 * @param rect
		 * @param animated
		 * @param constrain
		 */
		ViewportModule.prototype.fitInRect = function (rect, animated, constrain) {
			this.world.fitInRect(rect, animated, constrain);
		};

		/**
		 * @borrows ViewportModule.ViewportWidgetModel~setPanelsLayoutMode
		 */
		ViewportModule.prototype.setPanelsLayoutMode = function (layoutMode) {
			if (this.widgetModel) {
				this.widgetModel.setPanelsLayoutMode(layoutMode);
			}
		};

		/**
		 * @borrows ViewportModule.ViewportWidgetModel~getPanelModel
		 */
		ViewportModule.prototype.getPanelModel = function (name) {
			if (this.widgetModel) {
				return this.widgetModel.getPanelModel(name);
			}
		};


		/**
		 * @borrows ViewportModule.ViewportWidgetModel~addPanel
		 */
		ViewportModule.prototype.addPanel = function (name, overwriteOptions, disabledLayers) {
			if (this.widgetModel) {
				return this.widgetModel.addPanel(name, overwriteOptions, disabledLayers);
			}
		};

		/**
		 * @borrows ViewportModule.ViewportWidgetModel~removePanel
		 */
		ViewportModule.prototype.removePanel = function (name) {
			if (this.widgetModel) {
				this.widgetModel.removePanel(name);
			}
		};

		/**
		 * Returns the visible rectangle of the viewport.
		 * @returns {ShapeUtils.Rect}
		 */
		ViewportModule.prototype.getVisibleRect = function () {
			return this.world.getTargetViewPort();
		};

		/**
		 * Returns the world bounds
		 * @returns {ShapeUtils.Rect}
		 */
		ViewportModule.prototype.getWorldBounds = function () {
			return this.world.getWorldBounds();
		};


		/**
		 * Get the middle parameter of the viewport
		 * @returns {ShapeUtils.Point} middle
		 */
		ViewportModule.prototype.getMiddle = function () {
			return this.world.getMiddle();
		};


		/**
		 * Get the zoom level of the viewport
		 * @returns {Number} zoomlevel
		 */
		ViewportModule.prototype.getZoom = function () {
			return this.world.getZoom();
		};

		/**
		 * @override
		 */
		ViewportModule.prototype.getState = function () {
			var state = {
				// world
				world: {
					rotation: this.world.getRotation(),
					offsetSize: this.world.getOffsetSize(),
					zoom: this.world.getZoom(),
					isFullView: this.world.isFullView(),
					sceneSize: this.widget.getSize(),
					worldBounds: this.world.getWorldBounds(),
					flip: this.world.isFlip(),
					flipViewFromBack: this.world.isFlipViewFromBack(),
					visibleRect: this.world.getTargetViewPort()
				}
			};
			return state;
		};

		/**
		 * @override
		 * @param state
		 */
		ViewportModule.prototype.setState = function (state) {
			this.f.getDocumentController().whenActivePageChangedAndParametersAvailable(_.bind(function () {
				this.f.whenModuleAvailable("viewport", function () {
					// This module is ready
					if (state !== undefined) {
						// World
						var worldState = state.world;
						if (worldState !== undefined) {
							this.world.setWorldBounds(worldState.worldBounds);
							this.world.setInitialTransformation({
								fixedrotation: worldState.rotation,
								fixedoffset: worldState.offsetSize,
								fixedzoom: worldState.zoom,
								flip: worldState.flip,
								flipViewFromBack: worldState.flipViewFromBack
							});
							this.world.setSceneSize(new ShapeUtils.Size(worldState.visibleRect.size.width, worldState.visibleRect.size.height));


						}
					}
				}, this);
			}, this));
		};

		/**
		 * Get the viewportworld
		 * @returns {ViewportWorld}
		 */
		ViewportModule.prototype.getWorld = function () {
			return this.world;
		};

		/**
		 * Set the background color of the widget
		 * @borrows ViewportModule.ViewportWidget~setBackgroundColor
		 * @param {String} [hexColor] - New background color in hex format without # or none to set none
		 */
		ViewportModule.prototype.setWidgetBackgroundColor = function (hexColor) {
			this.widget.setBackgroundColor(hexColor);
		};


		/**
		 * The callback is called when the widgetmodel is set and the viewport can be used...
		 * @param {Function} callback - called when the viewport is ready
		 * @param {Object}   [binding]  - binding for the callback
		 */
		ViewportModule.prototype.whenWidgetModelAvailable = function (callback, binding) {
			if (binding) {
				callback = _.bind(callback, binding);
			}

			if (this.widgetModel) {
				callback();
			} else {
				this.once("widgetModelAvailable", callback);
			}
		};

		return ViewportModule;
	});

define('web-viewer/core/ViewerModeManager',["web-viewer/core/PrivateChannel",
		"underscore"
	],
	function (PrivateChannel,
		_) {

		/**
		 * @class ViewerModeManager - The ViewerModeManager is responsible for which ViewerMode is active. it enforces that only 1 ViewerMode can be active at a certain point in time
		 * @exports ViewerModeManager
		 * @memberof {Core}
		 * @params {Core.ViewerFacade} f
		 */
		var ViewerModeManager = function (f) {
			this.f = f;

			PrivateChannel.installTo(this, f, "viewerModeManager");


			this._activeMode = null;
		};

		/**
		 * Start a ViewerMode, if another mode is active, it will be stopped
		 * 
		 * @param {Core.ViewerMode} viewerMode - the mode to start
		 */
		ViewerModeManager.prototype.startMode = function (viewerMode) {
			if (viewerMode != this._activeMode) {
				this.stopActiveMode();

				this._activeMode = viewerMode;
				this._activeMode.toggle(true);

				this._emit("start", viewerMode);
			}
		};

		/**
		 * Stop a mode, if it is the active mode
		 * 
		 * @param {Core.ViewerMode} viewerMode - the mode which should be stoped
		 */
		ViewerModeManager.prototype.stopMode = function (viewerMode) {
			if (this._activeMode === viewerMode) {
				this.stopActiveMode();
			} else {
				this.f.debug.warn("Stopping a mode which isn't the active mode");
			}
		};

		/**
		 * Stop the acitve mode
		 */
		ViewerModeManager.prototype.stopActiveMode = function () {
			if (this._activeMode) {
				var oldActiveMode = this._activeMode;
				this._activeMode.toggle(false);
				this._activeMode = null;

				this._emit("stop", oldActiveMode);
			}
		};

		/**
		 * Get a reference to the currently active mode
		 * @returns {Core.ViewerMode}
		 */
		ViewerModeManager.prototype.getActiveMode = function () {
			return this._activeMode;
		};


		return ViewerModeManager;
	});

define('web-viewer/compare/CompareConstants',[],
	function () {
		"use strict";
		/**
		 * @memberof CompareModule
		 * @exports CompareConstants
		 */
		return {
			diffMode: {
				// undefined diff display mode (-1)
				UNDEFINED: -1,
				// show current version only (0)
				CURRENTVERSION: 0,
				// show previous version only (1)
				DIFFVERSION: 1,
				// show both with changes (2)
				SHOWBOTHCHANGES: 2,
				// show both with changes - dim common parts (3)
				SHOWBOTHCHANGESHIGHLIGHT: 3,
				// show both with changes - hide common parts (4)
				SHOWBOTHCHANGESONLY: 4,
				// show both - side by side(5)
				SHOWBOTHSIDEBYSIDE: 5,
				// show recto-verso - flip horizontally (6)
				RECTOVERSOATSIDE: 6, // not used currently in HTML5 Viewer
				// show both - one on top of the other (7)
				SHOWBOTHONTOP: 7,
				// show recto-verso - flip vertically (8)
				RECTOVERSOATTOP: 8, // not used currently in HTML5 Viewer
				// show in reader-spread (9)
				READERSPREAD: 9,
				// show in blinking mode - current (10)
				BLINKING_CURRENT: 10,
				// show side by side with highlight, not in java viewer
				SHOWBOTHSIDEBYSIDEHIGHLIGHT: 11,
				// show side by side with changes only, not in java viewer
				SHOWBOTHSIDEBYSIDECHANGESONLY: 12
			},
			blinkingSpeed: 1000,
			mode: {
				SHOW_CURRENT: "showCurrent",
				SHOW_DIFFERENCE: "showDifference",
				BLINKING: "blinking",
				DIFFERENCE: "difference",
				SIDEBYSIDE: "sidebyside",
				READERSPREAD: "readerspread",
				ALIGNMENT: "alignment",
				SHOWBOTHONTOP: "showBothOnTop"
			},
			modeNameToLanguageKey: {
				showCurrent: "DiffCurrentTT",
				showDifference: "DiffPrevTT",
				blinking: "DiffBothBlinkingTT",
				difference: "compareDifferenceMode",
				sidebyside: "DiffBothSplitTT",
				showBothOnTop: "DiffShowBothOnTopTT",
				alignment: "AnchorAlignWizardTitle",
				readerspread: "ReaderSpreadModeTitle"
			},
			sideBySideToDiffMode: { //map side by side difference modes to the correct difference compare mode
				5: 2,
				11: 3,
				12: 4
			},
			colorSpace: {
				DEFAULT: 0,
				CHANNELS: 1,
				HSV: 2,
				AUTOMATIC: 3
			},
			channelsMode: {
				AUTO: 0
			},
			showDifferenceMode: {
				SHOW: 2,
				HIGHLIGHT: 3,
				ONLY: 4
			},
			diffModeToMode: {
				UNDEFINED: "SHOW_CURRENT",
				CURRENTVERSION: "SHOW_CURRENT",
				DIFFVERSION: "SHOW_DIFFERENCE",
				SHOWBOTHCHANGES: "DIFFERENCE",
				SHOWBOTHCHANGESHIGHLIGHT: "DIFFERENCE",
				SHOWBOTHCHANGESONLY: "DIFFERENCE",
				SHOWBOTHSIDEBYSIDE: "SIDEBYSIDE",
				RECTOVERSOATSIDE: null,
				SHOWBOTHONTOP: "SHOWBOTHONTOP",
				RECTOVERSOATTOP: null,
				READERSPREAD: "READERSPREAD",
				BLINKING_CURRENT: "BLINKING",
				SHOWBOTHSIDEBYSIDEHIGHLIGHT: "SIDEBYSIDE",
				SHOWBOTHSIDEBYSIDECHANGESONLY: "SIDEBYSIDE"
			},
			cornerAutoAlign: {
				TOPLEFT: 0,
				TOPCENTER: 1,
				TOPRIGHT: 2,
				CENTERLEFT: 3,
				CENTERED: 4,
				CENTERRIGHT: 5,
				BOTTOMLEFT: 6,
				BOTTOMCENTER: 7,
				BOTTOMRIGHT: 8
			},
			cornerAlignIcons: {
				0: "icon-WCR_Align-TopLeft",
				1: "icon-WCR_Align-TopCenter",
				2: "icon-WCR_Align-TopRight",
				3: "icon-WCR_Align-MiddleLeft",
				4: "icon-WCR_Align-MiddleCenter",
				5: "icon-WCR_Align-MiddleRight",
				6: "icon-WCR_Align-BottomLeft",
				7: "icon-WCR_Align-BottomCenter",
				8: "icon-WCR_Align-BottomRight"
			},
			referenceRotation: {
				NOTROTATED: 0,
				ROTATED90: 90,
				ROTATED180: 180,
				ROTATED270: 270
			},
			pagelistCompareRange: {
				CURRENT: 0,
				CURRENTANDFOLLOWING: 1,
				ALL: 2
			},
			pagelistSide: {
				LEFTPAGE: 1,
				RIGHTPAGE: 2
			},
			anchorAlignment: {
				DEFAULT: "/wcr-static/web-viewer/layout/img/crosshair.png",
				ACTIVE: "/wcr-static/web-viewer/layout/img/crosshair_active.png",
				SIZE: {
					width: 14,
					height: 14
				},
				OFFSET: {
					width: 7,
					height: 7
				}
			}
		};
	});

define('web-viewer/utils/Coordinates',["web-viewer/utils/ViewerUtils",
		"web-viewer/compare/CompareConstants"
	],
	function (ViewerUtils,
		CompareConstants) {
		"use strict";

		/**
		 * Handles conversion of coordinates to the correct system.
		 * NOTE: At this moment, only CAD coordinate conversions are used
		 *
		 * @memberof Utils
		 * @exports Coordinates
		 * @class Coordinates
		 * @param {ViewerFacade} f
		 */
		var Coordinates = function (f) {
			this.f = f;
			this._cadModule = null;
			this._viewportModule = null;
			this._compareModule = null;
		};

		/**
		 * Get the transformed point
		 * @param   {ShapeUtils.Point} point     - The point to transform
		 * @returns {ShapeUtils.Point} The transformed point
		 */
		Coordinates.prototype.getTransformedPoint = function (point) {
			if (point) {
				var docController = this.f.getDocumentController();
				var activePage = docController.getActivePage();
				var diffPage = docController.getDifferencePage();
				var cadOverlay = docController.getCadOverlay();

				if (activePage.hasCad() || cadOverlay) {
					// We have CAD
					// get CAD and ViewPort modules
					if (!this._cadModule) {
						this._cadModule = this.f.getModule("cad");
					}
					if (!this._viewportModule) {
						this._viewportModule = this.f.getModule("viewport");
					}
					if (this._cadModule && this._viewportModule) {
						var transformedPoint;
						if (activePage.hasCad()) {
							// CAD as active page
							if (!this._compareModule) {
								this._compareModule = this.f.getModule("compare");
							}

							// Check if we are in diff view or active view
							if (this._compareModule) {
								var diffMode = this._compareModule.getDiffMode();
								if (diffPage && diffPage.hasCad() && diffPage.getCad()) {
									// diff version (= show ref or blinking ref), use diff transform 
									// side by side in diff panel takes active transform, this is different than the Java Viewer though, but we don't know the panel of the given point at this point!
									if (diffMode === CompareConstants.diffMode.DIFFVERSION) {
										var diffTransform = this._cadModule.getReferenceTransform();
										if (diffTransform && diffPage.getCad()) {
											transformedPoint = diffTransform.createInverse().transformedPoint(point);

											transformedPoint.y -= diffPage.getCad().getSize().height;
											transformedPoint.y = -transformedPoint.y;
											return transformedPoint;
										}
									}
								}
							}

							// Default active view
							var currentCadModel = this._cadModule.getCurrentPageCadModel();
							if (currentCadModel && currentCadModel.getCad() && activePage.getCad()) {
								var activeOrigin = currentCadModel.getCad().getOrigin();
								transformedPoint = point.copy();
								transformedPoint.x += activeOrigin.x;
								transformedPoint.y -= activeOrigin.y;
								transformedPoint.y -= activePage.getCad().getSize().height;
								transformedPoint.y = -transformedPoint.y;
								return transformedPoint;
							}
						} else {
							// CAD + graphics, use CAD overlay
							var overlayTransform = this._cadModule.getOverlayTransform();
							if (overlayTransform && cadOverlay.getCad()) {
								transformedPoint = overlayTransform.createInverse().transformedPoint(point);
								transformedPoint.y -= cadOverlay.getCad().getSize().height;
								transformedPoint.y = -transformedPoint.y;
								return transformedPoint;
							}
						}

					}
				}
			}
			return point;
		};

		/**
		 * @private
		 * Do the calculations
		 * @param   {ShapeUtils.Point} point     - The point to transform
		 * @returns {ShapeUtils.Point} The transformed point
		 */
		Coordinates.prototype._calculateTransformedPoint = function (point) {

		};

		return Coordinates;
	});

define('web-viewer/viewport/model/ViewportPopoverModel',["underscore",
		"web-viewer/core/PrivateChannel",
		"web-viewer/utils/Coordinates"
	],
	function (_,
		PrivateChannel,
		Coordinates) {
		"use strict";

		/**
		 * @exports ViewportPopoverModel
		 * @memberof ViewportModule
		 * @class 
		 * A popover model is an abstract model for adding popovers to the viewport
		 * It provides the content for the popover, the preferred position of the popover, etc.
		 */
		var ViewportPopoverModel = function (f, IID) {
			PrivateChannel.installTo(this, f, ['popoverModel', IID]);
			this.f = f;

			/**
			 * {Coordinates} - util for coordinate conversion
			 */
			this._coordinates = new Coordinates(this.f);

			/**
			 * @returns the unique internal id of the popover
			 */
			this.getID = function () {
				return IID;
			};

			/**
			 * @returns {ShapeUtils.Rect} the frame to which the popover is attached within the viewport
			 */
			this.getFrame = function () {
				return null;
			};

			/**
			 * @returns {JQueryObj} jquery object that represents the content of the popover
			 */
			this.getContent = function () {
				return null;
			};

			/**
			 * This function will be executed when the popover is added to the container
			 * Useful for doing things after it is added to the DOM
			 */
			this.onAddedToContainer = function () {
				// override
			};

			/**
			 * This function will be executed just before the popover is destroyed
			 * Useful for cleaning up
			 */
			this.onDestroy = function () {
				// override
			};

			/**
			 * Get the preferred position of the popover
			 */
			this.getPreferredPosition = function () {
				// override
				return "left";
			};

			/**
			 * Get the preferred restrict border for the restricting element
			 */
			this.getRestrictBorder = function () {
				return undefined;
			};

			/**
			 * Get the context for the popover
			 */
			this.getContext = function () {
				return f.getContext();
			};

			/**
			 * Get the class for the popover
			 */
			this.getClass = function () {
				//override
			};

			/**
			 * Is this an interactive popover?
			 * @returns {Boolean} false
			 */
			this.isInteractive = function () {
				return false;
			};
		};
		return ViewportPopoverModel;
	});

define('web-viewer/viewport/model/ViewportLayerModel',["underscore",
		"web-viewer/core/AbstractModel",
		"utils/Transformation",
		"utils/ShapeUtils",
		"web-viewer/core/PrivateChannel",
		"web-viewer/viewport/model/ViewportPopoverModel"
	], // for build reasons
	function (_,
		AbstractModel,
		Transformation,
		ShapeUtils,
		PrivateChannel) {
		"use strict";
		/**
		 * @exports ViewportLayerModel
		 * @memberof ViewportModule
		 * @augments {Core.AbstractModel}
		 * @class The layer model keeps track of the content of the layer
		 * It manages the linework or the image fetcher and keeps a transform of the layer
		 * @constructor
		 *
		 * @param {Core.ViewerFacade} f
		 * @param {String} IID - unique identifier for the layer
		 */
		var ViewportLayerModel = function (f, IID) {
			AbstractModel.call(this, f, ['layerModel', IID]); // TODO in the future we need a better naming here... (but for now it doesn't give any problem)

			this.f = f;

			this._name = IID;

			/**
			 * {Transformation}
			 * @private
			 */
			this.transform = new Transformation();

			/**
			 * All popover models
			 * @private
			 */
			this.popoverModels = {};

			/**
			 * All popup models
			 * @private
			 */
			this.popupModels = {};

			// url parameters can be overriden by putting new values in here
			this.fragmentUrlOverwriteOptions = {};

			/**
			 * Is the layer visible
			 * @private {Boolean}
			 */
			this._visible = true;

			/**
			 * Saved popovers for the layers with this layermodel
			 * Map popover id -> object containing the popoverDiv (where the popover is attached to), and the popover itself
			 * @private {Map.<String,Object>}
			 */
			this._popovers = {};
		};

		ViewportLayerModel.prototype = Object.create(AbstractModel.prototype);
		ViewportLayerModel.prototype.constructor = ViewportLayerModel;

		/**
		 * Get the name of the layer
		 * @returns {String}
		 */
		ViewportLayerModel.prototype.getName = function () {
			return this._name;
		};

		/**
		 * Save a popover in the layermodel, this popover can be used by all the layers that use this layermodel.
		 * @param {Object} popover
		 * @param {String} id
		 */
		ViewportLayerModel.prototype.savePopover = function (popover, id) {
			this._popovers[id] = popover;
		};

		/**
		 * Get a saved popover with given id.
		 * @param {String} id
		 * @returns {Object}
		 */
		ViewportLayerModel.prototype.getPopover = function (id) {
			return this._popovers[id];
		};

		/**
		 * Returns the collection of all the saved popovers.
		 * @returns {Map.<String,Object>}
		 */
		ViewportLayerModel.prototype.getAllPopovers = function () {
			return this._popovers;
		};

		/**
		 * Remove a popover from the layer model.
		 * @param {String} id
		 */
		ViewportLayerModel.prototype.removePopover = function (id) {
			delete this._popovers[id];
		};

		/**
		 * Add a popover model to the layer model
		 * @param {ViewportPopoverModel} model of the popover
		 */
		ViewportLayerModel.prototype.addPopoverModel = function (popoverModel) {
			if (!this.popoverModels[popoverModel.getID()]) {
				this.popoverModels[popoverModel.getID()] = popoverModel;
				this._emit("popoverAdded", popoverModel.getID());
			}
		};

		/**
		 * Remove a popover from the layer
		 * @param {String} popoverID - ID of the popover model to remove
		 */
		ViewportLayerModel.prototype.removePopoverModel = function (popoverID) {
			delete this.popoverModels[popoverID];
			this._emit("popoverRemoved", popoverID);
		};

		/**
		 * Get the popover with id.
		 * @param {String} popoverID
		 */
		ViewportLayerModel.prototype.getPopoverModel = function (popoverID) {
			return this.popoverModels[popoverID];
		};

		/**
		 * Add popup model
		 * @param {ViewportPopupModel} popupModel - Popup model to add.
		 */
		ViewportLayerModel.prototype.addPopupModel = function (popupModel) {
			this.popupModels[popupModel.getId()] = popupModel;
		};

		/**
		 * Remove a popup model
		 * @param {String} popupModelId - Id of the popup model.
		 */
		ViewportLayerModel.prototype.removePopupModel = function (popupModelId) {
			var popupModel = this.popupModels[popupModelId];
			popupModel.onDestroy();
			delete this.popupModels[popupModelId];
		};

		/**
		 * Remove all the popup models
		 */
		ViewportLayerModel.prototype.removeAllPopupModels = function () {
			_.each(this.popupModels, function (popupModel) {
				this.removePopupModel(popupModel.getId());
			}, this);
		};

		/**
		 * Get the popup model for the given id
		 * @param {String} popupModelId - Id of the popup model
		 * @returns {ViewportPopupModel} - Popup model with the given Id or undefined if not found
		 */
		ViewportLayerModel.prototype.getPopupModel = function (popupModelId) {
			return this.popupModels[popupModelId];
		};

		/**
		 * Get all the popup models.
		 * @returns {Object} Object with id -> popup model mapping.
		 */
		ViewportLayerModel.prototype.getPopupModels = function () {
			return this.popupModels;
		};


		/**
		 * Get the linework that needs to be rendered on the layer
		 * @param {Boolean} [animated] - indicate if the layer is being animated at the time
		 * @param {Boolean} [rotationanimation] - indicate if the layer is being animated with a rotation animation
		 * @param {ShapeUtils.Rect} [drawrect] - the rectangle for which the layermodel needs to return linework, the visible rectangle
		 * @param {String} [viewName] - name of the view to get the grid from
		 * @param {String} [panelName] - the name of the panel this layer is part of
		 * @returns {Array} [empty]
		 *
		 */
		ViewportLayerModel.prototype.getLinework = function (animated, rotationanimation, drawrect, viewName, panelName) {
			return [];
		};

		/**
		 * Returns background options for the canvas
		 * @returns {Object} object containing the background options
		 */
		ViewportLayerModel.prototype.getBackgroundOptions = function () {
			return {};
		};

		/**
		 * Get the transform
		 * @returns {Transformation}
		 */
		ViewportLayerModel.prototype.getTransform = function () {
			return this.transform;
		};

		/**
		 * Get the rotation of the layer model (needs to implemented by specific layer model)
		 * @return {Number}
		 */
		ViewportLayerModel.prototype.getRotation = function () {
			return 0;
		};

		/**
		 * Get the flipped state of the layer model (needs to implemented by specific layer model)
		 * @return {Boolean}
		 */
		ViewportLayerModel.prototype.isFlipped = function () {
			return false;
		};

		/**
		 * Reset the transformation
		 */
		ViewportLayerModel.prototype.resetTransformation = function () {
			this.transform = new Transformation();
		};

		/**
		 * Get the maximum resolution
		 * @returns {Number} [-1]
		 */
		ViewportLayerModel.prototype.getMaximumResolution = function () {
			return -1;
		};

		/**
		 * Emit this model changed
		 */
		ViewportLayerModel.prototype.modelChanged = function () {
			this._emit("modelChanged");
		};

		/**
		 * Preload a fragment (only needed when shouldAutoLoad returns false
		 */
		ViewportLayerModel.prototype.preloadFragment = function (vp, zoom) {

		};

		/**
		 * Request the url for a fragment
		 * @param fragment
		 * @returns {Object} context for the fragment (rectangle, resolution, servlet and params)
		 */
		ViewportLayerModel.prototype.getContextForFragment = function (fragment) {
			var context = {
				rectangle: fragment.getRect(),
				resolution: fragment.getZoom(),
				// the servlet that should be used to fetch the image information
				servlet: "",
				// the parameters used for requesting the image
				params: {
					x1: 0,
					x2: 0,
					y1: 0,
					y2: 0
				}
			};

			// ask all interceptors to update the context. All listeners have access to the context object and can adapt the information that is in the object
			// there is no predefined order in which these interceptors are called, their actions thus should not overlap!
			this.f.emit("intercept.fragmentURL", context);

			// override paramers
			context.params = _.extend(context.params, this.fragmentUrlOverwriteOptions);

			return context;
		};

		/**
		 * Returns the color of the cad linework
		 */
		ViewportLayerModel.prototype.getLineworkColor = function () {
			return null;
		};

		/**
		 * Returns the transparency (alpha) of the layer
		 */
		ViewportLayerModel.prototype.getAlpha = function () {};

		/**
		 * Set an overwrite option
		 * @param {String} option - option to be overwritten
		 * @param {String} value - value for the option
		 */
		ViewportLayerModel.prototype.setOverwriteOption = function (option, value) {
			var oldValue = this.fragmentUrlOverwriteOptions[option];

			this.fragmentUrlOverwriteOptions[option] = value;
			this._fireChange(false, ["overwriteOption", option], oldValue, value);
		};

		/**
		 * Set overwrite options
		 * @param {Dictionary} options - options to be overwritten
		 */
		ViewportLayerModel.prototype.setOverwriteOptions = function (options) {
			var oldValue = this.fragmentUrlOverwriteOptions.slice(0);

			this.fragmentUrlOverwriteOptions = options;
			this._fireChange(false, ["overwriteOption"], oldValue, options.slice(0));
		};

		/**
		 * Get overwrite options
		 * For DEBUGGING
		 */
		ViewportLayerModel.prototype.getOverwriteOptions = function () {
			return this.fragmentUrlOverwriteOptions;
		};

		/**
		 * Remove an overwrite option
		 * @param {String} option to be removed
		 */
		ViewportLayerModel.prototype.removeOverwriteOption = function (option) {
			var oldValue = this.fragmentUrlOverwriteOptions[option];
			delete this.fragmentUrlOverwriteOptions[option];

			this._fireChange(false, ["overwriteOption", option], oldValue, undefined);
		};

		/**
		 * Destroy all listeners
		 */
		ViewportLayerModel.prototype.destroy = function () {

		};

		/**
		 * Should the Layer fetch the fragments himself?
		 * @returns {Boolean}
		 */
		ViewportLayerModel.prototype.shouldAutoFetchFragments = function () {
			return false;
		};

		/**
		 * @returns {Boolean} if the layer contains graphics (and these should be autofetched
		 */
		ViewportLayerModel.prototype.hasGraphics = function () {
			return false;
		};

		/**
		 * Check if the layer model has Linework
		 * @returns {Boolean}
		 */
		ViewportLayerModel.prototype.hasLinework = function () {
			return false;
		};

		/**
		 * Does the layer model have a buffer
		 * @returns {Boolean}
		 */
		ViewportLayerModel.prototype.hasBuffer = function () {
			return false;
		};

		/**
		 * Returns the buffer, if any needs to be implemented, returns null if no buffer
		 * @returns {null|Fragment}
		 */
		ViewportLayerModel.prototype.getBuffer = function () {
			return null;
		};

		/**
		 * Mark all frames as dirty
		 */
		ViewportLayerModel.prototype._markAllAsDirty = function () {
			this._emit("fragmentsDirty");
		};

		/**
		 * Clean up all fragments/frames
		 */
		ViewportLayerModel.prototype._cleanupFragments = function () {
			this._emit("cleanupFragments");
		};




		/**
		 * Hide/show layer
		 * @param toggle
		 */
		ViewportLayerModel.prototype.toggle = function (toggle) {
			var oldValue = this._visible;
			this._visible = toggle;
			this._fireChange(false, ["visible"], oldValue, toggle);
		};

		/**
		 * Is the layer visible
		 * @returns {Boolean}
		 */
		ViewportLayerModel.prototype.isVisible = function () {
			return this._visible;
		};

		/**
		 * Render the model on the canvas
		 * @param {Context2D} ctx
		 * @param {ShapeUtils.Rect} drawrect
		 * @param {Number} zoom
		 * @param {Transformation} t - transformation for this layer
		 */
		ViewportLayerModel.prototype.render = function (ctx, drawrect, zoom, t) {
			// empty implementation by default
		};

		/**
		 * Calculates the transformation the layer model, takes into account that document page can been flipped, rotated or has an offset.
		 * @param {ShapeUtils.Rect} worldBounds - bounds of the world
		 * @param {ShapeUtils.Size} pageSize - size of the document page
		 * @param {ShapeUtils.Size} offset - offset for the annotations
		 * @param {Number} rotation - the rotation of the layer (0,90,180 or 270)
		 * @param {Boolean} isFlipped - is the layer for the annotations flipped
		 * @param {Number} postRotation - the rotation of the layer, after offset has been added to the transformation (rotates around world middle)
		 * @param {Boolean} isPostFlipped - is the layer for the annotation flipped after the offset has been added
		 * @param {ShapeUtils.Point} [rotationPoint] - rotation point for the transformation, if not provided (0,0) is used to rotate
		 * @param {ShapeUtils.Point} [flipPoint] - point to flip around
		 */
		ViewportLayerModel.prototype.calculateTransformation = function (worldBounds, pageSize, offset, rotation, isFlipped, postRotation, isPostFlipped, rotationPoint, flipPoint) {
			if (worldBounds !== undefined && worldBounds.size !== undefined && pageSize !== undefined) {
				this.offset = offset; //set the offset for the layer
				//rotate the transformation around 0,0 by moving the transformation centrum to 0,0
				this.getTransform().rotate(-rotation * (Math.PI / 180), worldBounds.size.width / 2, -worldBounds.size.height / 2);

				if (isFlipped) { //if the reference document page is flipped, flip the layerModel 
					//first translate the layer to the 0,0 of the transformation
					this.getTransform().translate(worldBounds.size.width / 2, -worldBounds.size.height / 2);
					this.getTransform().flip(0, 0); //flip
				}

				var isRotated = rotation % 180 === 90;
				if (isRotated && isFlipped) { //the origin seems to have been inverted both for x- and y-axis, so invert all the translations
					this.getTransform().translate(worldBounds.size.width / 2, -worldBounds.size.height / 2);
					this.getTransform().translate(-this.offset.width, -this.offset.height); //do the translate for the offset
				} else if (isFlipped) { //redo the translations
					this.getTransform().translate(-worldBounds.size.width / 2, worldBounds.size.height / 2);
					this.getTransform().translate(this.offset.width, this.offset.height); //do the translate for the offset
				} else { //not flipped so only do the translation of the offset
					this.getTransform().translate(this.offset.width, this.offset.height);
				}

				//calculate the offset which the already constructed transformation will use to transform all to the correct position
				var transformOffset = new ShapeUtils.Size(worldBounds.size.width / 2 - this.getTransform().e, worldBounds.size.height / 2 - this.getTransform().f);
				this.getTransform().translate(-transformOffset.width, -transformOffset.height);
				this.getTransform().preTranslate(transformOffset.width, transformOffset.height);

				//if the layer needs to be flipped or rotated after an offset was added use preFlip and preRotate
				if (isPostFlipped) {
					if (flipPoint) {
						this.getTransform().preTranslate(-flipPoint.x, -flipPoint.y);
					}
					this.getTransform().preFlip(0, 0);
					if (flipPoint) {
						this.getTransform().preTranslate(flipPoint.x, flipPoint.y);
					}
				}

				if (postRotation) { //currently rotates around the middle of the view
					if (!rotationPoint) {
						this.getTransform().preRotate(postRotation * (Math.PI / 180), 0, 0);
					} else {
						this.getTransform().preRotate(postRotation * (Math.PI / 180), rotationPoint.x, rotationPoint.y);
					}

				}

			}
		};

		return ViewportLayerModel;
	});

define('web-viewer/viewport/model/ViewportViewModel',["utils/Transformation",
		"web-viewer/core/PrivateChannel",
		"underscore",
		"web-viewer/viewport/model/ViewportLayerModel"
	], // for build reasons 
	function (Transformation,
		PrivateChannel,
		_) {
		"use strict";

		/**
		 * @exports ViewportViewModel
		 * @memberof ViewportModule
		 * @class A view model class is responsible for keeping track of the layers of one view
		 * @param {Core.ViewerFacade} f - Facade
		 * @param {String} IID
		 * @param {String} title
		 * @param {Boolean} isDefaultView
		 */
		var ViewportViewModel = function (f, IID, title, isDefaultView) {
			PrivateChannel.installTo(this, f, ["viewport", "viewmodel", IID]);

			/**
			 * Layer models
			 * {ViewportLayerModel}
			 */
			this._layers = {};

			/**
			 * The title of the view
			 * @private
			 */
			this._title = title;

			/**
			 * ID of the view
			 * @private
			 */
			this._IID = IID;

			/**
			 * Indicates whether the view is shown
			 * @private
			 */
			this._shown = true;

			/**
			 * Indicates whether the this view is the default view
			 * @private
			 */
			this._isDefaultView = isDefaultView;

			/**
			 * {Transformation}
			 * @private
			 */
			this._transform = new Transformation();
		};

		/**
		 * @returns {Object.<String,ViewportLayerModel}
		 */
		ViewportViewModel.prototype.getLayers = function () {
			return this._layers;
		};

		/**
		 * Add a layer model to the view model
		 * @param {String} name
		 * @param {ViewportLayerModel} layer
		 */
		ViewportViewModel.prototype.addLayer = function (name, layer) {
			this._layers[name] = layer;
			this._emit("layerAdded", name);
		};

		/**
		 * removes a layer model from the view model
		 * @param {String} name
		 */
		ViewportViewModel.prototype.removeLayer = function (name) {
			delete this._layers[name];
			this._emit("layerRemoved", name);
		};

		/**
		 * get a layer model
		 * @param {String} name
		 * @returns {ViewportLayerModel}
		 */
		ViewportViewModel.prototype.getLayer = function (name) {
			return this._layers[name];
		};

		/**
		 * Get the number of layer models
		 * @returns {Number}
		 */
		ViewportViewModel.prototype.numberOfLayers = function () {
			return _.keys(this._layers).length;
		};

		/**
		 * returns the name of layer at a given index
		 * @param {Number} index
		 * @returns {String}
		 */
		ViewportViewModel.prototype.nameForLayerAtIndex = function (index) {
			return _.keys(this._layers)[index];
		};

		/**
		 * Get the model of a layer
		 * @param {String} name
		 * @returns {ViewportLayerModel}
		 */
		ViewportViewModel.prototype.modelForLayer = function (name) {
			return this._layers[name];
		};

		/**
		 * Get the layer model of layer at a certain index
		 * @param {Number} index
		 * @returns {ViewportLayerModel}
		 */
		ViewportViewModel.prototype.modelForLayerAtIndex = function (index) {
			return this._layers[this.nameForLayerAtIndex(index)];
		};

		/**
		 * Transform the layer
		 * @param layername
		 * @param tr
		 */
		ViewportViewModel.prototype.transformLayer = function (layername, tr) {
			var layer = this._layers[layername];
			if (layer) {
				layer.getTransform().transform(tr);
			}
		};

		/**
		 * Get the unique identifier of the view
		 * @returns {String}
		 */
		ViewportViewModel.prototype.getID = function () {
			return this._IID;
		};

		/**
		 * Returns the index for a layer
		 */
		ViewportViewModel.prototype.indexForLayer = function (layername) {
			return _.indexOf(_.keys(this._layers), layername);
		};

		/**
		 * Returns the transformation of the view
		 * @returns {Transformation}
		 */
		ViewportViewModel.prototype.getTransform = function () {
			return this._transform;
		};

		/**
		 * Set the title of the view
		 * @param {String} title - Title of the view
		 */
		ViewportViewModel.prototype.setTitle = function (title) {
			if (this._title !== title) {
				this._title = title;
				this._emit("titleChanged");
			}
		};

		/**
		 * Get the title of the view
		 * @returns {String} Title of the view
		 */
		ViewportViewModel.prototype.getTitle = function () {
			return this._title;
		};

		/**
		 * Set whether the view is shown or not
		 * @param {Boolean} shown - True if the view should be shown, false otherwise
		 */
		ViewportViewModel.prototype.setShown = function (shown) {
			if (this._shown !== shown) {
				this._shown = shown;
				this._emit("showChanged");
			}
		};

		/**
		 * Get whether the view is shown or not
		 * @returns {Boolean} True if the view is shown, false otherwise
		 */
		ViewportViewModel.prototype.isShown = function () {
			return this._shown;
		};

		/**
		 * Enable/disable a layer (if it is in the ViewModel)
		 * @param {String} layerName - name of the layer
		 * @param {Boolean} visible
		 */
		ViewportViewModel.prototype.toggleLayer = function (layerName, visible) {
			var layer = this.getLayer(layerName);
			if (layer !== undefined) {
				layer.toggle(visible);
			}
		};

		/**
		 * is this view the default view.
		 * @returns {Boolean}
		 */
		ViewportViewModel.prototype.isDefaultView = function () {
			return this._isDefaultView;
		};

		return ViewportViewModel;
	});

define('web-viewer/webviewer/WebviewerViewModel',["web-viewer/viewport/model/ViewportViewModel",
		"underscore"
	],
	function (ViewportViewModel,
		_) {
		"use strict";

		/**
		 * @memberof WebviewerModule
		 * @exports ViewserverViewModel
		 * @class 
		 * The viewserver view model is responsible to manage the layers on a view
		 * @constructor
		 * @augments Viewport.ViewportViewModel
		 * @param {Core.ViewerFacade} f - Facade
		 * @param {String} ID
		 * @param {String} title
		 * @param {Boolean} isDefaultView
		 */
		var WebviewerViewModel = function (f, ID, title, isDefaultView) {
			ViewportViewModel.call(this, f, ID, title, isDefaultView);

			var layerOrder = ["Graphics", "Cad", "currentDiffCad", "referenceDiffCad", "referenceCad", "CadOverlay", "DynContent", "DynContentPopup", "Annotations", "AnnotationPopups"];


			/**
			 * Returns the index for a layer
			 * TODO: improve this implementation
			 */
			this.indexForLayer = function (layername, layers) {
				//Use the intersection of the layers and the layerOrder, so the index of the layers can be returned
				var currentLayers = _.keys(layers);
				currentLayers.push(layername);
				var layerUnion = _.intersection(layerOrder, currentLayers);
				return _.indexOf(layerUnion, layername);
			};

		};

		// inherit
		WebviewerViewModel.prototype = Object.create(ViewportViewModel.prototype);
		// correct the constructor pointer 
		WebviewerViewModel.prototype.constructor = WebviewerViewModel;


		return WebviewerViewModel;
	});

define('web-viewer/webviewer/WebviewerWidgetModel',["underscore",
		"web-viewer/webviewer/WebviewerViewModel",
		"web-viewer/viewport/model/ViewportWidgetModel",
		"utils/ShapeUtils",
		"web-viewer/utils/ViewerUtils",
		"web-viewer/utils/ConversionConstants"
	],
	function (_,
		WebviewerViewModel,
		ViewportWidgetModel,
		ShapeUtils,
		ViewerUtils,
		ConversionConstants) {
		"use strict";

		/**
		 * Create a new WebviewerWidgetModel
		 * @memberof WebviewerModule
		 * @exports WebviewerWidgetModel
		 * @augments Viewport.ViewportWidgetModel
		 * @class
		 * The WebviewerWidget model is responsible for loading CAD or Graphics module and setting the correct properties for the viewport widget
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 */
		var WebviewerWidgetModel = function (f) {
			ViewportWidgetModel.call(this, f, "webviewer");
			this.f = f;
		};

		// inherit
		WebviewerWidgetModel.prototype = Object.create(ViewportWidgetModel.prototype);
		// correct the constructor pointer 
		WebviewerWidgetModel.prototype.constructor = WebviewerWidgetModel;

		/**
		 * @override
		 */
		WebviewerWidgetModel.prototype.createModelForView = function (ID, title, isDefaultView) {
			return new WebviewerViewModel(this.f, ID, title, isDefaultView);
		};

		/**
		 * @override
		 */
		WebviewerWidgetModel.prototype.getWorldBounds = function () {
			var activePage = this.f.getActivePage();
			var referencePage = this.f.getDocumentController().getDifferencePage();
			var cadOverlay = this.f.getDocumentController().getCadOverlay();

			if (activePage !== undefined && referencePage !== undefined && activePage.hasCad() && referencePage.hasCad()) {
				if (!this.f.getModule("cad")){
					// CAD Module isn ot yet loaded
					return undefined;
				}
				
				// compare with CAD documents
				var activeOrigin = activePage.getOrigin(true);
				var referenceOrigin = referencePage.getOrigin(true);

				var activePageSize = activePage.getPageSize();
				var referencePageSize = referencePage.getPageSize();

				if (activeOrigin !== null && referenceOrigin !== null) {
					if (referencePage.getCad() && referencePage.getCad().getLocation()) {
						referenceOrigin = referencePage.getCad().getLocation().origin;
						referencePageSize = referencePage.getCad().getLocation().size;
					} else {
						referenceOrigin = new ShapeUtils.Point(referenceOrigin.x - activeOrigin.x, (activeOrigin.y + activePageSize.height) - (referenceOrigin.y + referencePageSize.height));
					}
					var currentRect = new ShapeUtils.Rect(null, activePageSize);
					var referenceRect = new ShapeUtils.Rect(referenceOrigin, referencePageSize);

					return currentRect.calculateArea(referenceRect);
				} else {
					return undefined;
				}
			} else {
				if (activePage && activePage.hasCad()) {
					// just 1 CAD file is loaded					
					return new ShapeUtils.Rect(activePage.getOrigin(), activePage.getPageSize());
				} else if(activePage) { //Graphics (in compare and with or without cad overlay)
					// we could be in graphical compare (doesn't realy matter, viewserver gave us correct sizex and sizey)
					var graphicsWorldRect = new ShapeUtils.Rect(activePage.getOrigin(), new ShapeUtils.Size(ViewerUtils.parseFloat(activePage.getParam("sizex")), ViewerUtils.parseFloat(activePage.getParam("sizey"))));

					if (activePage.isBlankPage() && (!referencePage || referencePage.isBlankPage())) {
						var margins = activePage.getMargins();
						graphicsWorldRect.size.height += margins.top + margins.bottom;
						graphicsWorldRect.size.width += margins.left + margins.right;
						if (activePage.isReaderSpread()) {
							graphicsWorldRect.size.width *= 2;
						}
					}

					if (cadOverlay && cadOverlay.getCad && cadOverlay.getCad()) {
						// there is a CAD overlay
						var cadRect;
						if (cadOverlay.getCad() && cadOverlay.getCad().getLocation()) {
							cadRect = cadOverlay.getCad().getLocation();
						} else {
							var cadOrigin = cadOverlay.getCad().getOrigin();
							var cadOffset = new ShapeUtils.Size(ViewerUtils.parseFloat(cadOverlay.get("offsetx")), ViewerUtils.parseFloat(cadOverlay.get("offsety")));
							var cadSize = cadOverlay.getPageSize();
							cadRect = new ShapeUtils.Rect(new ShapeUtils.Point(-cadOffset.width + cadOrigin.x, cadOffset.height - (cadSize.height + cadOrigin.y)), cadSize);
						}
						var graphicsRect = new ShapeUtils.Rect(new ShapeUtils.Point(), graphicsWorldRect.size);
						return cadRect.calculateArea(graphicsRect);
					} else {
						return graphicsWorldRect;
					}
				} else {
					return undefined;
				}
			}

		};

		/**
		 * @override
		 */
		WebviewerWidgetModel.prototype.getWorldResolution = function () {
			var worldResolution = {
				x: 0,
				y: 0
			};
			var activePage = this.f.getActivePage();
			if (activePage !== undefined && activePage.isParametersAvailable()) {
				var resolutionx = activePage.getParam("resolutionx");
				var resolutiony = activePage.getParam("resolutiony");
				if (resolutionx !== undefined) {
					worldResolution.x = ViewerUtils.parseFloat(resolutionx);
				} else {
					worldResolution.x = ConversionConstants.DEFAULT_POSTSCRIPT_PPI;
				}
				if (resolutiony !== undefined) {
					worldResolution.y = ViewerUtils.parseFloat(resolutiony);
				} else {
					worldResolution.y = ConversionConstants.DEFAULT_POSTSCRIPT_PPI;
				}
				return worldResolution;
			}

		};


		return WebviewerWidgetModel;

	});

/**
 * @namespace WebviewerModule
 */
define('web-viewer/webviewer/WebviewerModule',["web-viewer/core/AbstractModule",
		"web-viewer/webviewer/WebviewerWidgetModel"
	],
	function (AbstractModule,
		WebviewerWidgetModel) {
		"use strict";

		/**
		 * @exports WebviewerModule
		 * @memberof WebviewerModule
		 * @class
		 * The WebviewerModule, this module installs the correct models for the viewport and load Graphics or Cad module
		 * @constructor
		 * @augments Core.AbstractModule
		 * @param {Core.ViewerFacade} f
		 * @param {Boolean} silenced
		 */
		var WebviewerModule = function (f, silenced) {
			AbstractModule.call(this, f, silenced, "webviewer");


			this.webviewerWidgetModel = new WebviewerWidgetModel(f);
			f.getModule("viewport").setWidgetModel(this.webviewerWidgetModel);

			var docController = this.f.getDocumentController();
			
			this._pageChanged(undefined, this.f.getActivePage());
			this._pageChanged(undefined, docController.getCadOverlay());

			
			// react when the active page has changed
			docController.whenActivePageChanged(this._pageChanged, this);
			docController.whenCadOverlayChanged(this._pageChanged, this);
		};


		// inherit
		WebviewerModule.prototype = Object.create(AbstractModule.prototype);
		// correct the constructor pointer 
		WebviewerModule.prototype.constructor = WebviewerModule;


		/**
		 * The active page or CAD overlay has changed, load the needed modules, and create a new view
		 *
		 * @param {Data.AbstractDocumentPage} oldPage
		 * @param {Data.AbstractDocumentPage} newPage
		 */
		WebviewerModule.prototype._pageChanged = function (oldPage, newPage) {
			var mainView = this.webviewerWidgetModel.getViewModel("Main");

			if (mainView === undefined || mainView === null) {
				this.webviewerWidgetModel.createView("Main", "", true);
			}

			if (newPage) {
				// create main view
				if (newPage.hasCad() || newPage.hasLinkedPage()) {
					this.f.requestModule("cad", undefined, false);
				}

				if (newPage.hasGraphics() || newPage.isBlankPage()) {
					this.f.requestModule("graphics", undefined, false);
				}

				// only load attribute category module when allowed and needed for the page
				if (this.f.getProperty("allowAttributeCategory") &&
					!this.f.isMobile() &&
					newPage.getAttributeCategoryID &&
					newPage.getAttributeCategoryID()) {
					this.f.requestModule("attributeCategory", undefined, false);
				}
				
				// load 3D module if there is a 3D perspective or a 3d perspective can be generated
				
				if (newPage.get("has3DPerspective")){
					this.f.requestModule("view3D", undefined, false);	
				}

				newPage.whenParametersAvailable(_.bind(function () {
					if ((newPage.hasGraphics() || newPage.isBlankPage()) && newPage.getDocumentVersion().isMultiPage()) {
						this.f.requestModule("pagelist", undefined, false);
					}
				}, this));
			}
		};

		return WebviewerModule;
	});

define('web-viewer/core/AbstractDebug',[],function () {
	"use strict";
	/**
	 * Create a new debugger
	 * 
	 * @exports AbstractDebug
	 * @memberof Core
	 * @class AbstractDebug Debug utility used for logging inside the viewer
	 * @param {EventEmitter2} eventBus - the event bus
	 */
	var AbstractDebug = function (eventBus) {

		// Basic debug log information in production
		eventBus.on("error.SEVERE", function () {
			if (typeof console != "undefined") {
				if (!console.error) {
					if (console.log && console.log.apply) {
						console.log.apply(console, arguments);
					} else if (console.log) {
						console.log("ERROR", arguments);
					}
				} else if (console.error.apply) {
					console.error.apply(console, arguments);
				} else {
					console.error("ERROR", arguments);
				}
			}
		});
	};

	/**
	 * Same behavior as console.log
	 */
	AbstractDebug.prototype.log = function () {};

	/**
	 * Same behavior as console.warn
	 */
	AbstractDebug.prototype.warn = function () {};

	/**
	 * Same behavior as console.error
	 */
	AbstractDebug.prototype.error = function () {};

	/**
	 * Same behavior as console.debug
	 */
	AbstractDebug.prototype.debug = function () {};

	return AbstractDebug;
});

define('web-viewer/core/DevDebug',["web-viewer/core/AbstractDebug",
		"jquery",
		"underscore"
	],
	function (AbstractDebug, $) {
		"use strict";
		/**
		 * Create new DevDebug instance.
		 * @exports DevDebug the devDebug debugger enables sending debugmessages to the console
		 * @constructor
		 *
		 * @param {EventEmitter2} eventBus - the event bus used in the App
		 */
		//>>excludeStart("PExclude", pragmas.productionExclude);
		// Fix for IE(9) that does not support console.log.apply and kindred
		if (window.console !== undefined && console.log.apply) {
			//>>excludeEnd("PExclude");
			var DevDebug = function (eventBus) {
				AbstractDebug.call(this, eventBus);

				window.gViewerDebug = window.gViewerDebug || [];
				window.gViewerDebug.push(this);

				//>>excludeStart("PExclude", pragmas.productionExclude);

				// paths the Debug will listen to
				var channels = ["*"];

				/**
				 * This function logs the event information
				 */
				var _logEventFunction = _.partial(function (that) {
					that.debug(new Date().getTime(), "E", this.event, _.tail(arguments, 1));
				}, this);

				// log errors
				eventBus.on("error.*", _.bind(function (msg, details) {
					this.error(msg, details);
				}, this));



				/**
				 * Stop listening to channels on the eventBus
				 *
				 * @param {String} [channel] - if undefined: remove all debug listeners for all channels, if string stop listening to the specific channel
				 */
				this.stopListeningTo = function stopListening(channel) {
					if (channel) {
						if (channel == "*") {
							eventBus.offAny(_logEventFunction);
						} else {
							eventBus.off(channel, _logEventFunction);
						}
						this.debug("DEBUG", "stop listening to", channel);
					} else {
						// remove all listeners
						for (var i = 0; i < channels.length; i++) {
							this.stopListeningTo(channels[i]);
						}
						channels = [];
					}
				};

				/**
				 * Start listening to a certain channel on the eventBus
				 *
				 * @param {String} channel
				 */
				this.startListeningTo = function (channel) {
					this.debug("DEBUG", "start listening to", channel);
					if (channel == "*") {
						eventBus.onAny(_logEventFunction);
					} else {
						eventBus.on(channel, _logEventFunction);
					}
				};


				// listen to the standard paths
				for (var i = 0; i < channels.length; i++) {
					this.startListeningTo(channels[i]);
				}


				//>>excludeEnd("PExclude");
			};

			DevDebug.prototype = Object.create(AbstractDebug.prototype);
			DevDebug.prototype.constructor = DevDebug;

			//>>excludeStart("PExclude", pragmas.productionExclude);

			// if you want the file name + line numbers next to your log messages, set this to true
			var addLineInfo = false;
			if (navigator.appName.indexOf("Internet Explorer") != -1) {
				addLineInfo = false;
			}


			/**
			 * Skip certain lines in stack trace containing one of those strings
			 */
			var filters = ["DevDebug", "underscore", "EventEmitter2", "ViewerFacade", "PrivateChannel"];
			/**
			 * Extract line number from stack
			 * @param stack Stacktrace
			 * @returns Stack line where the log function is called.
			 */
			var extractLineNumberFromStack = function (stack) {
				var lines = stack.split('\n');
				var line;
				for (var i = 1; i < lines.length && !line; i++) {
					var j = 0;
					while (j < filters.length && lines[i].indexOf(filters[j]) < 0) {
						j++;
					}
					// filter not found
					if (j >= filters.length) {
						line = lines[i];
					}
				}
				if (!line) {
					return "";
				} else {
					// fix for various display text
					return (line.indexOf(' (') >= 0 ? line.split(' (')[1].substring(0, line.length - 1) : line.split('at ')[1]);
				}
			};

			/**
			 * Get the suffix for a line, meaning the file name and line number of the caller of log.
			 * @param error Error containing the stacktrace.
			 * @returns File name and line number of the caller of log
			 */
			var getLineSuffix = function (error) {
				return "@" + (error.lineNumber ? error.fileName + ':' + error.lineNumber + ":1" // add arbitrary column value for chrome linking
					: extractLineNumberFromStack(error.stack));
			};

			/**
			 * Same behavior as console.log
			 */
			DevDebug.prototype.log = function () {
				var args = arguments;
				if (addLineInfo) {
					args = Array.prototype.slice.call(args, 0);
					args = args.concat([getLineSuffix(new Error())]);
				}
				if (console && console.log) {
					console.log.apply(console, args);
				}
			};

			/**
			 * Same behavior as console.warn
			 */
			DevDebug.prototype.warn = function () {
				var args = arguments;
				if (addLineInfo) {
					args = Array.prototype.slice.call(args, 0);
					args = args.concat([getLineSuffix(new Error())]);
				}
				try {
					console.warn.apply(console, args);
				} catch (e) {
					this.log.apply(this, args);
				}
			};

			/**
			 * Same behavior as console.error
			 */
			DevDebug.prototype.error = function () {
				var args = arguments;
				if (addLineInfo) {
					args = Array.prototype.slice.call(args, 0);
					args = args.concat([getLineSuffix(new Error())]);
				}
				try {
					console.error.apply(console, args);
				} catch (e) {
					this.log.apply(this, args);
				}
			};

			/**
			 * Same behavior as console.debug
			 */
			DevDebug.prototype.debug = function () {
				var args = arguments;
				if (addLineInfo) {
					args = Array.prototype.slice.call(args, 0);
					args = args.concat([getLineSuffix(new Error())]);
				}

				try {
					console.debug.apply(console, args);
				} catch (e) {
					this.log.apply(this, args);
				}
			};
			//>>excludeEnd("PExclude");

			return DevDebug;
			//>>excludeStart("PExclude", pragmas.productionExclude);
		} else {
			return AbstractDebug;
		}
		//>>excludeEnd("PExclude");
	});

/**
 * @namespace Core
 */
define('web-viewer/core/Core',["lib/EventEmitter2",
        "web-viewer/core/ViewerFacade",
        "web-viewer/data/DocumentController",
        "web-viewer/data/AbstractDocument",
        "web-viewer/data/AbstractDocumentVersion",
        "component/loadingindicator/LoadingIndicator",
        "web-viewer/layout/LayoutManager",
        "web-viewer/featureDetector/FeatureDetectorModule",
        "web-viewer/message/MessageModule",
        "web-viewer/viewport/ViewportModule",
        "web-viewer/layout/component/ViewerToolbarButton",
        "web-viewer/layout/component/ModalPopup",
        "web-viewer/layout/LayoutConstants",
        "web-viewer/core/ViewerModeManager",
        "utils/I18n",
        "utils/XMLUtils",
        "utils/ConvertUtils",
        "jquery",
        "web-viewer/webviewer/WebviewerModule",
        "web-viewer/core/DevDebug",
        "underscore"
    ],
    function (EventEmitter2,
        ViewerFacade,
        DocumentController,
        AbstractDocument,
        AbstractDocumentVersion,
        LoadingIndicator,
        LayoutManager,
        FeatureDetectorModule,
        MessageModule,
        ViewportModule,
        ViewerToolbarButton,
        ModalPopup,
        LayoutConstants,
        ViewerModeManager,
        I18n,
        XMLUtils,
        ConvertUtils,
        $,
        WebviewerModule,
        Debug,
        _) {
        "use strict";

        /**
         * EVENTS
         *
         * core.loadSourceFile
         *
         * 		Load a source file dynamically.
         * 		Use this only to load source files dynamically. Dependencies should be loaded via define.
         *
         * 		@param {String} path - Full path to the source file (e.g. menus/SimpleControl)
         * 		@param {Function} [callback] - Callback which is called on completion of loading
         *
         *
         * module.request
         *
         * 		Request the instance of a module, if the module isn't loaded, it will be loaded, the callback function is called with the module
         *
         *
         * 		@param {String} name - The name of the module
         *		@param {Function} callback - Function which is called when the module is available (or couldn't be loaded), The first argument of this callback is the instance of the module (or null)
         * 		@param {Boolean} [silenced] - If true and the module is not loaded yet, then the module won't emit widgets/tools/toolbarcomponents until it loaded with silenced false/undefined or is unsilenced
         */


        /**
         * The ViewerCore is the core of the application.
         *
         * It provides an Event Bus for communication between modules, all interaction with the core happens through the Facade.
         * The ViewerCode can load widgets and modules
         * @exports ViewerCore
         * @memberof Core
         * @constructor
         * @param {JQueryObj} context - the HTML object in which the viewer application should reside
         * @param {Core.AbstractConfig} config - configuration class
         */
        var ViewerCore = function (context, config) {

            var initialLoader = $("<div>").css({
                'background': "url('" + requirejs.toUrl('web-viewer/layout/img/loaderTransparent.gif') + "') no-repeat 2px center",
                'padding-left': '20px',
                'font': 'normal 11px/20px Verdana,Arial,Helvetica,sans-serif',
                'margin': '20px',
                'height': '20px'

            }).text("Loading...");
            context.empty().append(initialLoader);


            var maxListeners = 0; // in production environment we don't want to generate any memory leak warnings on the event bus (0=unlimited)
            //>>excludeStart("PExclude", pragmas.productionExclude);
            // during development we want maximum 20 listeners per channels, in production mode this can be much more
            maxListeners = 20;
            //>>excludeEnd("PExclude");	

            // Create event bus
            var ebus = new EventEmitter2({
                wildcard: true, // should the event emitter use wildcards.
                delimiter: '.', // the delimiter used to segment namespaces,
                newListener: true // if you want to emit the newListener event
            });

            ebus.setMaxListeners(maxListeners); // the max number of listeners that can be assigned to an event

            var modulesToLoadCount = -1;

            /**
             * The coreAPI is the API the facade can use to interact with the core
             * but all other components can only communicate with the core through the
             * facade
             */
            var coreAPI = {
                getProperty: function (key) {
                    return config.get(key);
                },
                getConfig: function () {
                    return config;
                },
                getAppContext: function () {
                    return context;
                }
            };


            /**
             * The currently logged in user.
             * This variable is initialized later during the initialization phase
             */
            var user;
            /**
             * The dataAdapter.
             * This variable is initialized later during initialization
             */
            var dataAdapter;
            /**
             * The documentController
             * This variable is initialized later during initalization
             */
            var docController;


            // now configure Ajax so it has a timeout, and emits loading events
            $.ajaxSetup({
                beforeSend: function (jqXHR, settings) {
                    jqXHR["_viewerLoadKey"] = settings.url;
                    ebus.emit("loading.start", settings.url);

                },
                complete: function (jqXHR, textStatus) {
                    var url = jqXHR["_viewerLoadKey"];
                    ebus.emit("loading.stop", url);
                },
                timeout: 60000
            }); // 1 min



            // initialize debugger
            var debug = new Debug(ebus);
            debug.log("CORE", "Debugger activated", debug);



            // initialize facade
            debug.log("CORE", "Initializing ViewerFacade", debug);
            var f = new ViewerFacade(coreAPI, ebus, debug);

            // create the loading indicator (we do this here, because we want it to listen before all loads start)
            // listen to all global loading.start/stop events
            f.on("loading.start", function (what) {
                LoadingIndicator.start(what);
            });
            f.on("loading.stop", function (what) {
                LoadingIndicator.stop(what);
            });

            //>>excludeStart("PExclude", pragmas.productionExclude);
            // only expose the ViewerFacade and loading indicator during development
            window.gF = f;
            debug.log("CORE", "The ViewerFacade is now exposed as window.gF");

            window.gLoadingIndicator = LoadingIndicator;
            debug.log("CORE", "The loading indicator is now exposed as window.gLoadingIndicator");
            //>>excludeEnd("PExclude");


            // initialize viewermode manager
            debug.log("CORE", "Initializing viewerModeManager");
            var viewerModeManager = new ViewerModeManager(f);
            var isWaitingForCompare = false;

            coreAPI.stopMode = function (viewerMode) {
                return viewerModeManager.stopMode(viewerMode);
            };

            coreAPI.startMode = function (viewerMode) {
                return viewerModeManager.startMode(viewerMode);
            };

            coreAPI.stopActiveMode = function () {
                return viewerModeManager.stopActiveMode();
            };

            coreAPI.getActiveMode = function () {
                return viewerModeManager.getActiveMode();
            };

            coreAPI.canFetchFragments = function () {
                return !docController.isParameterUpdateOngoing() && !isWaitingForCompare;
            };



            /**
             * Load a source in to memory and instantiate it
             *
             * @param {String} path - Full path to the source file (e.g. menus/SimpleControl)
             * @param {Function} [callback] - This callback is called when the source is loaded. The loaded object is given as first argument, or null if failed
             */
            function loadSourceFile(path, callback) {
                //load the widget with require
                var loadingID = "loadSourceFile_" + path;
                ebus.emit("loading.start", loadingID);

                requirejs(["web-viewer/" + path], function (Class) {
                    //call callback function if available
                    if (callback) {
                        callback(Class);
                    }
                    ebus.emit("loading.stop", loadingID);
                }, function (err) {
                    //loading of source file failed...
                    f.emit("error.SEVERE", "Failed loading source " + path, err);

                    if (callback) {
                        callback(null);
                    }
                    ebus.emit("loading.stop", loadingID);
                });
            }

            // Listen to core.loadSourceFile
            // NOTE: Use this only to load source files dynamically. Dependencies should be loaded via define.
            ebus.on("core.loadSourceFile", loadSourceFile);

            // initialize module management
            var loadedModules = {};
            var loadingModules = {};
            var loadingSilencedModules = {};

            /**
             * Get the source file for a module name
             *
             * @param {String} moduleName - The name of the module
             * @returns {String} The path the the module source file
             */
            function getSrcFileForModule(moduleName) {
                return moduleName + "/" + ConvertUtils.capitaliseFirstLetter(moduleName) + "Module";
            }

            /**
             * Initialize a module
             *
             * @param {Core.AbstractModule} Module - module constructor
             * @param {String} name - the name of the module
             * @param {Boolean} silenced - initialize module in silenced mode
             * @returns {Core.AbstractModule} instantiated module
             */
            function initializeModule(Module, name, silenced) {
                ebus.emit(["module", name, "startInitialization"]);
                var moduleInstance = new Module(f, silenced);
                loadedModules[name] = moduleInstance;
                // notify possible listeners
                ebus.emit(["module", "loaded", name], moduleInstance);

                return moduleInstance;
            }

            /**
             * Request the instance of a module, if the module isn't loaded, it will be loaded, the callback function is called with the module
             * (or null if the module couldn't be loaded)
             *
             * @param {String} name - The name of the module
             * @param {Function} [callback] - Function which is called when the module is available (or couldn't be loaded), The first argument of this callback is the instance of the module (or null)
             * @param {Boolean} [silenced] - If true and the module is not loaded yet, then the module won't emit widgets/tools/toolbarcomponents until it loaded with silenced false/undefined or is unsilenced
             */
            function requestModule(name, callback, silenced) {
                silenced = silenced || false;

                if (loadedModules.hasOwnProperty(name)) {
                    // the module is already loaded
                    var module = loadedModules[name];

                    // unsilence it if needed
                    if (!silenced) {
                        module.setSilenced(false);
                    }

                    // call the callback if available
                    if (callback) {
                        callback(module);
                    }
                } else if (loadingModules.hasOwnProperty(name)) {
                    // the module is already being loaded, add callback if available
                    if (loadingSilencedModules.hasOwnProperty(name) && !silenced) {
                        delete loadingSilencedModules[name];
                    }

                    // if callback available, push it on the list
                    if (callback) {
                        loadingModules[name].push(callback);
                    }
                } else {
                    // the module is not yet loaded, request it, and register callback if any
                    loadingModules[name] = [];

                    if (silenced) {
                        // the module should be loaded silenced
                        loadingSilencedModules[name] = 1;
                    }

                    if (callback) {
                        loadingModules[name].push(callback);
                    }

                    ebus.emit(["module", "loading", name]);

                    // start loading the module
                    loadSourceFile(getSrcFileForModule(name), function (Mod) {
                        //the module is probably loaded!
                        var moduleInstance = null;
                        if (Mod) {
                            // should the module be silenced?
                            var silenced = loadingSilencedModules.hasOwnProperty(name);

                            // there is an object, instantiate it with the facade
                            moduleInstance = initializeModule(Mod, name, silenced);
                        } else {
                            // something went wrong while loading the module
                            ebus.emit("error.SEVERE", "Failed to load module " + name);
                        }

                        // now notify all registered callbacks about the result
                        var callbacks = loadingModules[name];
                        for (var i = 0; i < callbacks.length; i++) {
                            callbacks[i](moduleInstance);
                        }

                        // delete the registered callbacks, the module is loaded (or has failed loading)
                        delete loadingModules[name];
                        delete loadingSilencedModules[name];
                    });
                }
            }
            // for now the requestModule shouldn't work yet
            coreAPI.requestModule = function () {};


            /**
             * Do an Ajax call
             * @param {String} url - The URL to call
             * @param {Object} options - Object containing the options for the ajax request. See jQuery API for the meaning
             * @example
             * defaultOptions = {
             *	dataType: "xml",
             *		success: function(data, textStatus, jqXHR) {},
             *		error: function(jqXHR, textStatus, errorThrown) {},
             *		cache: false,
             *		processData: true,
             *		contentType: "application/x-www-form-urlencoded; charset=UTF-8",
             *		type: "GET",
             *		data: "",
             *		noAjaxParam: false // it will automatically add ajax=1 to the param list if this is false, setting this to true will NOT add it
             *	};
             *
             */
            coreAPI.ajax = function (url, options) {
                // define the default options
                var defaultOptions = {
                    dataType: "xml",
                    success: function (data, textStatus, jqXHR) {},
                    error: function (jqXHR, textStatus, errorThrown) {},
                    cache: false,
                    processData: true,
                    contentType: "application/x-www-form-urlencoded; charset=UTF-8",
                    type: "GET",
                    data: "",
                    noAjaxParam: false
                };

                // merge the options
                options = _.extend(defaultOptions, options);

                // add the ajax=1 parameter if noAjaxParam = false
                if (!options.noAjaxParam) {
                    if (url.indexOf("?") < 0) {
                        // no parameters
                        url += "?ajax=1";
                    } else {
                        // already has parameters
                        url += "&ajax=1";
                    }
                }

                return $.ajax(url, {
                    dataType: options.dataType,
                    success: function (data, textStatus, jqXHR) {
                        // if xml, check if login failed
                        if (data && options.dataType === "xml") {
                            var code = data.getElementsByTagName("code");
                            if (code[0] && XMLUtils.textContent(code[0]) === "001") {
                                f.i18n.loadBundle("viewer", function () {
                                    f.emit("error.SEVERE", f.i18n.t("WCSessionLostError"), data);
                                });
                                return;
                            }
                        }

                        // do general success logic
                        options.success(data, textStatus, jqXHR);
                    },
                    error: function (jqXHR, textStatus, errorThrown) {
                        // do general error handling
                        options.error(jqXHR, textStatus, errorThrown);
                    },
                    cache: options.cache,
                    processData: options.processData,
                    contentType: options.contentType,
                    type: options.type,
                    data: options.data,
                    timeout: options.timeout
                });
            };

            /**
             * Executes the callback when all modules are loaded
             * @param {Function} callback
             */
            coreAPI.whenAllModulesAreLoaded = function (callback) {
                if (modulesToLoadCount === 0) {
                    callback();
                } else {
                    f.once("allModulesAreLoaded", callback);
                }
            };

            /**
             * Returns the module is it is already loaded
             * @param {String} name - name of the module
             * @returns {Core.AbstractModule}
             */
            coreAPI.getModuleIfAvailable = function (name) {
                return loadedModules[name];
            };


            /**
             * Executes the callback when the module with the given name is loaded.
             * If it is already there, the callback is immediately executed, otherwise it waits until it is loaded.
             * @param {String} name - Name of the module
             * @param {Function} callback - Parameters: moduleInstance
             * @param {Any} [binding] - Context binding of the callback
             */
            coreAPI.whenModuleAvailable = function (name, callback, binding) {
                if (binding !== undefined) {
                    callback = _.bind(callback, binding);
                }

                var module = loadedModules[name];
                if (module !== undefined) {
                    // it is already available
                    callback(module);
                } else {
                    ebus.once(["module", "loaded", name], callback);
                }
            };

            /**
             * Get the loaded modules
             * @returns {Object}
             */
            coreAPI.getLoadedModules = function () {
                return loadedModules;
            };

            /**
             * Inject state to the viewer
             * @param {Object} state
             */
            coreAPI.injectState = function (state) {
                coreAPI.whenAllModulesAreLoaded(_.bind(function () { //wait for all modules to be loaded
                    if (state.compare !== undefined) { // first inject state for the compare module if it is not undefined 
                        var compareModule = loadedModules["compare"];
                        if (compareModule !== undefined) {
                            compareModule.setState(state.compare);
                        }
                        delete state.compare; //set to undefined to make sure it doesn't get injected again
                    }
                    var measureState = state.measure;
                    delete state.measure;
                    _.each(state, _.bind(function (moduleState, moduleName) { //inject state for all the other modules
                        if (moduleState !== undefined) { //first make sure there is a state to inject
                            coreAPI.whenModuleAvailable(moduleName, function (module) {
                                module.setState(moduleState);
                            }, this);
                        }
                    }, this));
                    if (measureState !== undefined) {
                        var measureModule = loadedModules["measure"];
                        if (measureModule !== undefined) {
                            measureModule.setState(measureState);
                        }
                    }
                }, this));
            };

            /**
             * Get the state of the current viewer, will ask the state of all loaded modules
             * @returns {Object}
             */
            coreAPI.getState = function () {
                var modules = this.getLoadedModules();
                var viewerState = {};
                _.each(modules, function (module, moduleName) {
                    viewerState[moduleName] = module.getState();
                });
                return viewerState;
            };

            debug.debug("CORE", "Initializing Message module");
            // we first start by loading the message module
            if (!MessageModule) {
                window.alert("Couldn't load Message module.");
            }

            // initialize message module instance
            var messageModule = initializeModule(MessageModule, "message", true);


            // the message module is loaded (or it failed, and the user was warned. now we start initializing other components

            // Initialize user
            debug.debug("CORE", "Initializing user object");
            user = config.getUser();
            user.setFacade(f);
            coreAPI.getCurrentUser = function () {
                return user;
            };

            // setup data adapter
            debug.debug("CORE", "Initializing DataAdapter");
            dataAdapter = config.intializeAdapter(f);

            coreAPI.getDataAdapter = function () {
                return dataAdapter;
            };


            // initialize i18n and start loading general bundle
            debug.debug("CORE", "Initializing I18N");
            f.i18n = new I18n(user.getLocale(), dataAdapter.getInstance(), config.get("languageLoadTimestamp"));
            f.i18n.loadBundle("viewer", function () {
                debug.debug("I18N", "General language bundle loaded");
                initialLoader.text(f.i18n.t("Loading"));
            });

            // setup document controller
            debug.debug("CORE", "Initializing DocumentController");
            docController = new DocumentController(f, dataAdapter);

            /**
             * Get the DocumentController
             *
             * @return {DocumentController}
             */
            coreAPI.getDocumentController = function () {
                return docController;
            };

            /**
             * Open document(s)
             * @param {any} documentInformation - String or JSON object containing information on which document to load
             */
            coreAPI.openDocuments = function openDocuments(documentsIdentifier) {
                docController.openDocuments(documentsIdentifier);
            };

            /**
             * Called before the viewer is unloaded
             */
            function onBeforeUnload() {
                var unsavedItems = "";
                _.each(loadedModules, function (module, name) {
                    if (module.hasUnsavedElements()) {
                        unsavedItems += "\r\n" + module.getUnsavedElementsName();
                    }
                });

                if (unsavedItems.length > 0) {
                    return f.i18n.t("ExitViewer") + unsavedItems;
                }
            }

            // to be sure we have strings available
            f.i18n.loadBundle("viewer", function () {
                $(window).on("beforeunload", onBeforeUnload);
            });

            f.on("viewer.request.close", function (returnURL, saveBeforeExitKey, exitWithoutSaveKey) {
                var unsavedModules = {};
                _.each(loadedModules, function (module, name) {
                    if (module.hasUnsavedElements()) {
                        unsavedModules[name] = module;
                    }
                });

                saveBeforeExitKey = saveBeforeExitKey ? saveBeforeExitKey : "SaveAndExit";
                exitWithoutSaveKey = exitWithoutSaveKey ? exitWithoutSaveKey : "ExitWithoutSave";

                // temporary function to go the the return url
                var goToReturnURL = function () {
                    if (window.location === returnURL) {
                        window.location.reload();
                    } else {
                        window.location = returnURL;
                    }
                };

                if (!_.isEmpty(unsavedModules)) {
                    f.i18n.loadBundle("viewer", function () {
                        var modalPopup = new ModalPopup(f, "unsavedItemsPopup", f.getContext());
                        modalPopup.setRemoveFromDomAfterUsage(true);
                        modalPopup.setOkBtnText(f.i18n.t(saveBeforeExitKey));

                        // extra button
                        var exitBtn = new ViewerToolbarButton(f, "exit", $("<span/>").text(f.i18n.t(exitWithoutSaveKey)), false, function () {
                            $(window).off("beforeunload", onBeforeUnload);
                            goToReturnURL();
                        });
                        modalPopup.addButton(exitBtn);

                        // cancel button
                        // nothing extra needed

                        // ok button
                        modalPopup.setOkAction(function () {
                            var callback = function (name) {
                                delete unsavedModules[name];
                                if (_.isEmpty(unsavedModules)) {
                                    $(window).off("beforeunload", onBeforeUnload);
                                    goToReturnURL();
                                }
                            };
                            if (!_.isEmpty(unsavedModules)) {
                                modalPopup.enableOkBtn(false);
                                _.each(unsavedModules, function (module, name) {
                                    module.saveAllUnsavedElements(function () {
                                        return callback(name);
                                    });
                                });
                            }
                        });

                        // Header + Content
                        modalPopup.setHeader($("<span/>").addClass("popupTitle").html(f.i18n.t("UnsavedItems")));
                        modalPopup.setContent($("<div/>").addClass("unsavedItemsContent").append(f.i18n.t("UnsavedItemsContent")));

                        modalPopup.show();
                        modalPopup.recalculateSize(400);
                    });
                } else {
                    goToReturnURL();
                }
            });


            // now start listening for module requests
            debug.debug("CORE", "Core is now listening to module requests");
            // requesting modules should also be possible from the facade
            coreAPI.requestModule = requestModule;

            // Load user options
            user.loadOptions(function () {
                // initialize feature Detector
                debug.debug("CORE", "Initializing Feature detector module");
                // initialize feature detector module instance
                var featureDetector = initializeModule(FeatureDetectorModule, "featureDetector", true);
                coreAPI.getFeatureDetector = function () {
                    return featureDetector;
                };


                    debug.debug("CORE", "Detecting layout type");
                    // detect layout type
                    var layoutType = user.getOption("LAYOUT_PREFERENCE");
                    if ((layoutType === LayoutConstants.LayoutType.MOBILE && featureDetector.hasTouch()) ||
                        (layoutType === LayoutConstants.LayoutType.AUTOMATIC && featureDetector.isMobile())) {
                        // if preference is "automatic": use mobile if device is mobile detected. If preference is "mobile": only show mobile layout if touch support is enabled
                        layoutType = LayoutConstants.LayoutType.MOBILE;
                    } else {
                        // Desktop, check which style
                        if (coreAPI.getProperty("viewerStyle") == LayoutConstants.ViewerStyle.SIMPLIFIED) {
                            layoutType = LayoutConstants.LayoutType.FLEX;
                        } else {
                            layoutType = LayoutConstants.LayoutType.DESKTOP;
                        }
                    }

                /**
                 * Return the type of layout we are in
                 * @returns {LayoutConstants.LayoutType}
                 */
                coreAPI.getLayoutType = function () {
                    return layoutType;
                };


                // if compare is activated, the viewport module needs a repaint after loading
                // after 2: we need to wait for both the layoutmanager to be loaded and the documents (if they need "compare" functionality)			
                var repaintViewportModuleIfNeeded = _.after(2, function () {
                    coreAPI.whenModuleAvailable("viewport", function (viewportModule) {
                        viewportModule.whenWidgetModelAvailable(function () {
                            requestModule("compare", function () {
                                isWaitingForCompare = false;
                                viewportModule.widget.repaint();
                            }, false); //now make it unsilenced to trigger a publish of the widget
                        });
                    });
                });


                // load the viewport first, then load the other modules
                debug.debug("CORE", "Start loading viewport module");
                if (!ViewportModule) {
                    ebus.emit("error.SEVERE", "Couldn't load viewport module!");
                }
                var viewportModule = initializeModule(ViewportModule, "viewport", true); // SILENCED (wait till LayoutManager is ready)


                // load the webviewer module, this sets the correct viewportwidget model
                debug.debug("CORE", "Start loading webviewer module, this intializes the viewport widget with the correct widgetmodel");
                var webviewerModule = initializeModule(WebviewerModule, "webviewer", true); // SILENCED (wait till LayoutManager is ready)


                // start loading compare module as soon as possible
                docController.whenDocumentsLoaded(function () {
                    if (docController.getDocumentCount() > 1 || _.some(docController.getDocuments(), function (document) {
                            return document.hasLinkedDocument() || document.hasMultipleVersions() || document.getDocumentVersion().isMultiPage() || document.getDocumentVersion().isPageList();
                        })) {
                        isWaitingForCompare = true;
                        requestModule("compare", function () {
                            isWaitingForCompare = false;
                        }, true); //true becaused silenced (so no widgets get publised, compare widget needs viewport-module)


                        repaintViewportModuleIfNeeded();
                    }
                });




                // Start up layoutmanager			
                debug.debug("CORE", "Initializing LayoutManager");
                var layoutManager = new LayoutManager(f, context, LoadingIndicator, initialLoader, _.bind(function () {

                    debug.debug("CORE", "Initialized LayoutManager", layoutManager);

                    /**
                     * Switch to another layout
                     * @param {LayoutConstants.LayoutType} layoutType - type of the layout
                     * @returns {Number}
                     */
                    coreAPI.switchLayout = function (layoutType) {
                        layoutManager.switchLayout(layoutType);
                    };


                    // unsilence the message module
                    if (messageModule) {
                        messageModule.setSilenced(false);
                    }

                    // unsilence viewport and webviewer
                    debug.debug("CORE", "Unsilence Viewport and WebViewer module");
                    viewportModule.setSilenced(false);
                    webviewerModule.setSilenced(false);

                    // we are ready for a repaint
                    repaintViewportModuleIfNeeded();

                    // start loading other modules
                    debug.debug("CORE", "Start loading other modules");

                    var modulesToLoad = config.getModules();
                    var whenAllModulesLoaded = _.after(modulesToLoad.length, function () {
                        f.emit("allModulesAreLoaded");
                    });

                    _.each(modulesToLoad, function (moduleName) {
                        requestModule(moduleName, whenAllModulesLoaded);
                    });



                }, this));

            });

            // expose the ViewerFacade as the interface to the Core
            return f;
        };

        return ViewerCore;
    });

define('web-viewer/App',["web-viewer/core/Core"],
	function (ViewerCore) {
		"use strict";

		/**
		 * The ViewerApp class is an instance of the web viewer.
		 * To launch an instance of the web viewer, a new instance of this function should be created
		 * 
		 *  new ViewerApp(......)
		 *  
		 *  @param context                {HTMLObject}          a reference to the div in which the viewer should be created
		 *  @param documentInformation    {any}                 String or JSON object containing information on which document to load
		 *  @param config                 {Core.AbstractConfig} configuration object
		 *                                                     
		 */
		var ViewerApp = function (context, documentInformation, config) {

			// we create a new instance of the viewer core facade
			this._f = new ViewerCore(context, config);

			// open the document(s)
			this._f.openDocuments(documentInformation);

		};

		/**
		 * Inject state of modules into the viewerApp. 
		 * @param {Object} state - module name is mapped on the state object for that specific module
		 */
		ViewerApp.prototype.injectState = function (state) {
			this._f.injectState(state);
		};

		return ViewerApp;
	});

/**
 * @memberof AnnotationModule
 */
define('web-viewer/annotation/AnnotationConstants',{
	AccessRight: {
		READ_ONLY: 0,
		READ_WRITE: 1
	},
	ActionPermissions: {
		DUPLICATE_WITHOUT_TEXT: "duplicateWithoutText",
		DUPLICATE: "duplicateAnnotation",
		CAN_BE_REPLIED_TO: "canBeRepliedTo"
	},
	AnnotationType: {
		ANNOT_RECT: 0,
		ANNOT_ELLIPSE: 4,
		ANNOT_HIGHLIGHT: 6,
		ANNOT_LINE: 1,
		ANNOT_ARROW: 5,
		ANNOT_PIN: 11,
		ANNOT_FREEFORM: 3
	},

	AnnotationPin: {
		normal: "/wcr-static/web-viewer/layout/img/comment-raw.gif",
		highlighted: "/wcr-static/web-viewer/layout/img/commentselected-raw.gif",
		imagehotspotx: 0,
		imagehotspoty: 0,
		imagetouchspotx: 0,
		imagetouchspoty: 0,
		imagetouchspotw: 26,
		imagetouchspoth: 24,
		imagewidth: 26,
		imageheight: 24
	},

	AnnotationElementName: {
		0: "square",
		4: "circle",
		6: "highlight",
		1: "line",
		5: "line",
		11: "text",
		3: "ink"
	},

	AnnotationReplyName: {
		0: "Rectangle",
		4: "Oval",
		6: "Highlight",
		1: "Line",
		5: "Arrow",
		11: "Note",
		3: "Freehand"
	},

	AnnotationReviewState: {
		STATUS_ACCEPTED: "Accepted",
		STATUS_REJECTED: "Rejected",
		STATUS_CANCELLED: "Cancelled",
		STATUS_COMPLETED: "Completed",
		STATUS_NONE: "None"
	},

	AnnotationReviewStateIcons: {
		STATUS_ACCEPTED: "icon-WCR_Approve",
		STATUS_REJECTED: "icon-WCR_Reject",
		STATUS_CANCELLED: "icon-WCR_Canceled",
		STATUS_COMPLETED: "icon-WCR_Approve WCR_Completed",
		STATUS_NONE: "None"
	},

	AnnotationReviewStateLocs: {
		STATUS_ACCEPTED: "AnnStatusAccepted",
		STATUS_REJECTED: "AnnStatusRejected",
		STATUS_CANCELLED: "AnnStatusCancelled",
		STATUS_COMPLETED: "AnnStatusCompleted",
		STATUS_NONE: "AnnStatusNone"
	},

	AnnotationSortType: {
		SORT_BY_MODIFICATION_DATE: 0,
		SORT_BY_CREATION_DATE: 1
	},

	StepThroughMode: {
		WITH_REPLIES: 0,
		BY_THREAD: 1,
		WITH_ZOOM: 2,
		WITH_LINKED: 3
	},

	AnnotationViewType: {
		VIEWTYPE_ALL: 0,
		VIEWTYPE_ONLYFILE: 1,
		VIEWTYPE_ONLYDIFF: 2,
		VIEWTYPE_NONE: 3
	},
	/**
	 * Translation keys for annotation type
	 */
	AnnotationComments: {
		0: "AnnCommRect",
		4: "AnnCommOval",
		6: "AnnCommHighlight",
		1: "AnnCommLine",
		5: "AnnCommArrow",
		11: "AnnCommNote",
		3: "AnnCommFreehand"
	},
	/**
	 * all annotation icons
	 */
	AnnotationIcons: {
		0: "annotRect",
		4: "annotOval",
		6: "annotHighlight",
		1: "annotLine",
		5: "annotArrow",
		11: "annotNote",
		3: "annotFreehand"
	},
	/**
	 * Annotation Type to css Icon class
	 */
	AnnotationToolIcons: {
		0: {
			icon: "icon-WCR_Annotation-Rect icon-danger"
		},
		4: {
			icon: "icon-WCR_Annotation-Oval icon-danger"
		},
		6: {
			icon: "icon icon-WCR_Annotation-Highlight-cont",
			sub: ["icon icon-WCR_Annotation-Highlight-sub icon-yellow"]
		},
		1: {
			icon: "icon-WCR_Annotation-Line icon-danger"
		},
		5: {
			icon: "icon-WCR_Annotation-Arrow icon-danger"
		},
		11: {
			icon: "addNote"
		},
		3: {
			icon: "icon-WCR_Annotation-Freehand icon-danger"
		}
	},
	/**
	 * Which mode of annotating the document is used (relationship approval/annotations)
	 */
	annotateMode: {
		NO_RESTRICTIONS: 0,
		IF_APPROVED_SHOW_WARNING: 1,
		IF_APPROVED_DISABLE: 2,
		ALWAYS_DISABLE: 3,
		IF_APPROVED_OR_REJECTED_SHOW_WARNING: 4,
		IF_APPROVED_OR_REJECTED_DISABLE: 5
	},

	/**
	 * Which people are allow to add annotations
	 */
	allowedAnnotators: {
		ALL: 0, // All Users can annotate
		ALL_APPROVERS: 1, // All people in approval setup can annotate
		ALL_APPROVERS_AND_MASTER: 2, // All people in approval setup + approval master can annotate
		PENDING_APPROVERS: 3, // Only approvers in the currently active cycle can annotate
		PENDING_APPROVERS_AND_MASTER: 4 // Only approvers in the currently active cycle + approval master can annotate
	},

	/**
	 * Operators that can be used in the custor filter popup.
	 */
	FilterOperators: {
		ALL: 0,
		ANY: 1
	},

	/**
	 * Different types for the annotation filter
	 */
	FilterTypes: {
		AUTHOR: 0,
		REVIEWSTATE: 1,
		REVIEWAUTHORANDSTATE: 2,
		TYPE: 3
	},

	/**
	 * Location keys for the filter types
	 */
	FilterTypeNameLocs: {
		AUTHOR: "AnnotFilterAuthor",
		REVIEWSTATE: "AnnotFilterReviewState",
		REVIEWAUTHORANDSTATE: "AnnotFilterReviewAuthorAndState"
	},

	/**
	 * The condition for the specific filter
	 */
	FilterCondition: {
		IS: 0,
		ISNOT: 1
	},
	
	/**
	 * Define the types of buttons available in the annotation origin selector
	 * values are of type: {
	 *  id: <String> should be the same as the key
	 *  icon: <String> icon class to display
	 *  [subIcon]: <String> optional sub icon class
	 *  langKey: <String> key used to translate the tooltip
	 *  group: <String> group to which this type belongs to
	 *  hideBtnOnEmpty: <Boolean> if the button should be shown if no annotations of this type exist
	 *  hideBtnOnGroupEmpty: <Boolean> if the button should be hidden if there are no annotations in that group 
	 * (as defined by group property). hideBtnOnEmpty has higher priority then this property
	 * }
	 *
	 */
	AnnotationOriginTypes: {
		user: {
			id: "user",
			icon: "icon-WCR_AutoReview-cont icon-red",
			subIcon: "icon-WCR_User12 icon-default",
			langKey: "userAnnotations",
			group: "user"
		},
		barcode: {
			id: "barcode",
			icon: "icon-WCR_AutoReview-cont icon-red",
			subIcon: "icon-WCR_AutoReview-Barcode-sub icon-default",
			langKey: "barcodeGVAnnotations",
			group: "GV",
			hideBtnOnGroupEmpty: true
		},
		braille: {
			id: "braille",
			icon: "icon-WCR_AutoReview-cont icon-red",
			subIcon: "icon-WCR_AutoReview-Braille-sub icon-default",
			langKey: "brailleGVAnnotations",
			group: "GV",
			hideBtnOnGroupEmpty: true
		},
		spelling: {
			id: "spelling",
			icon: "icon-WCR_AutoReview-cont icon-red",
			subIcon: "icon-WCR_AutoReview-Spell-sub icon-default",
			langKey: "spellingGVAnnotations",
			group: "GV",
			hideBtnOnGroupEmpty: true
		},
		artwork: {
			id: "artwork",
			icon: "icon-WCR_AutoReview-cont icon-red",
			subIcon: "icon-WCR_AutoReview-Artwork-sub icon-default",
			langKey: "artworkGVAnnotations",
			group: "GV",
			hideBtnOnGroupEmpty: true
		},
		text: {
			id: "text",
			icon: "icon-WCR_AutoReview-cont icon-red",
			subIcon: "icon-WCR_AutoReview-Text-sub icon-default",
			langKey: "textGVAnnotations",
			group: "GV",
			hideBtnOnGroupEmpty: true
		},
		editor: {
			id: "editor",
			icon: "icon-WCR_AutoReview-cont icon-red",
			subIcon: "icon-WCR_AutoReview-Esko-sub icon-default",
			langKey: "editorAnnotations",
			hideBtnOnEmpty: true,
			group: "ext"
		},
		acrobat: {
			id: "acrobat",
			icon: "icon-WCR_AutoReview-cont icon-red",
			subIcon: "icon-WCR_AutoReview-Acrobat-sub icon-default",
			langKey: "acrobatAnnotations",
			hideBtnOnEmpty: true,
			group: "ext"
		},
		ext: {
			id: "ext",
			icon: "icon-WCR_AutoReview-cont icon-red",
			subIcon: "icon-WCR_AutoReview-External-sub icon-default",
			langKey: "extAnnotations",
			hideBtnOnEmpty: true,
			group: "ext"
		}
	}

});

/**
 * @memberof ApprovalModule
 */
define('web-viewer/approval/ApprovalConstants',{
	ViewerStateToComplexApprovalState: {
		'00001_0000000001': "Rejected",
		'00001_0000000002': "Pending",
		'00001_0000000003': "Approved",
		'00001_0000000004': "ForceRejected",
		'00001_0000000005': "ForceApproved",
		'00001_0000000006': "ApprovedConditional",
		'00001_0000000007': "ForceApprovedConditional"
	},
	ComplexApprovalStateToViewerState: {
		'Rejected': "00001_0000000001",
		'Pending': "00001_0000000002",
		'MustApprove': "00001_0000000002",
		'Approved': "00001_0000000003",
		'ForceRejected': "00001_0000000004",
		'ForceApproved': "00001_0000000005",
		'ApprovedConditional': "00001_0000000006",
		'ForceApprovedConditional': "00001_0000000007"
	}
});

/**
 * @memberof CadModule
 */
define('web-viewer/cad/CadConstants',{
	CadLayerLocName: {
		"MAIN": "LayerMain",
		"SHEET": "LayerSheet",
		"L1": "LayerL1",
		"DIE": "LayerDie",
		"STRIP": "LayerStrip",
		"FSTRIP": "LayerFStrip",
		"MSTRIP": "LayerMStrip",
		"GUILL": "LayerGuill",
		"VARNISH": "LayerVarnish",
		"FINGERS": "LayerFingers",
		"BLANK": "LayerBlank",
		"USER1": "LayerUser1",
		"USER2": "LayerUser2",
		"USER3": "LayerUser3",
		"USERDEFINED": "LayerUser",
		"RULEPATH": "LayerRulepath"
	},

	CadClassName: {
		0: "ClassMain",
		1: "ClassManufacturing",
		2: "ClassCutouts",
		3: "ClassDimensions",
		4: "ClassOverall",
		5: "ClassAnnotation",
		6: "ClassRegistration",
		7: "ClassBleed",
		8: "ClassGraphics", // undefinedClassName,
		9: "ClassCounter",
		10: "ClassCoating1",
		11: "ClassDiagram", // undefinedClassName,
		12: "ClassOutsideGraphics", // undefinedClassName,
		13: "ClassOutputLayer", // undefinedClassName,
		14: "ClassCoating2",
		15: "ClassCoating3",
		16: "ClassInsideBleed",
		17: "ClassInsideGraphics", // undefinedClassName,
		18: "ClassInsideCoating",
		19: "ClassCross",
		20: "ClassLabelGraphics", // new
		21: "ClassShrinkWrapGraphics" // new
	},

	CadDocType: {
		OneUp: 1,
		Layout: 2
	},

	CadTypes: {
		Nick: -11
	},

	ShowBridgesNicks: {
		NONE: 1,
		BRIDGES: 2,
		NICKS: 3,
		BOTH: 4
	},

	LineTypes: {
		1: [6, 6],
		2: [6, 3, 3, 3],
		3: [3, 3],
		10: [11, 3, 3, 3, 3, 3],
		20: [11, 11],
		30: [16, 1],
		40: [14, 3],
		50: [14, 7],
		60: [11, 3, 3, 3, 3, 3, 3, 3]
	},
	CadGraphicsMode: {
		BOTH: 1,
		GRAPHICS_ONLY: 2,
		CAD_ONLY: 3
	},
	DefaultSide: {
		OUTSIDE: "-1",
		INSIDE: "1"
	},
	GrainFluteDirection: {
		VERTICAL: "1",
		HORIZONTAL: "2"
	},
	DieSide: {
		BACKSIDE: "1",
		KNIFESIDE: "-1"
	}
});

/**
 * @memberof data
 */
define('web-viewer/data/DataConstants',
	/**
	 * @exports DataConstants
	 */
	{
		documentPageType: {
			CAD: 0,
			GRAPHICS: 1,
			PAGELIST: 2
		},
		imageType: {
			OTHER: 0,
			GRAPHICS: 1
		},
		// list of user preferences that always should be character sequences
		charsequencePreferences: [
			"LAYOUT_STATE", "LAYOUT_PREFERENCE", "ANNOTATION_FILTER"
		]
	});

/**
 * @namespace Data
 */
define('web-viewer/data/AbstractDataAdapter',["web-viewer/core/PrivateChannel"],
	function (PrivateChannel) {
		"use strict";
		/**
		 * Construct the data adapter
		 * 
		 * @exports AbstractDataAdapter
		 * @memberof Data
		 * @class DataAdapter is responsible for loading new documents and (their) data.
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 */
		var AbstractDataAdapter = function (f, instance) {
			/**
			 * @protected
			 */
			this.f = f;

			/**
			 * @private
			 * instance URL for the application
			 */
			this._instance = instance;

			PrivateChannel.installTo(this, f, ["doc"]);
		};

		/**
		 * Open document(s)
		 * @param {any} documentsIdentifier - String or JSON object containing information on which document to load
		 * @param {Function} callback - Callback fired with as first argument the document (if loading succeeded, otherwise null) and as second argument whether the document should be made active. Can be fired multiple times when there are multiple documents to be loaded.
		 */
		AbstractDataAdapter.prototype.openDocuments = function (documentsIdentifier, callback) {
			// open documents based on documentsIdentifier
		};

		/**
		 * Returns the instance for the web application
		 * @returns {String}
		 */
		AbstractDataAdapter.prototype.getInstance = function () {
			return this._instance;
		};

		/**
		 * Load basic information on the document
		 * 
		 * @param {Any} fileIdentifier - identifier used by the adapter to get the needed document information
		 * @param {String} IID - internal ID of the document that is begin loaded
		 * @param {Function} callback - when the document is loaded, the callback function is called with the Document object as first argument, if loading failed the first argument is null
		 */
		AbstractDataAdapter.prototype.loadDocument = function (fileIdentifier, IID, callback) {
			// load the document
			// call the callback with the new Document object as first argument
			// if failed, have "null" as first element and send an error event
		};

		/**
		 * Load all documents of the passed object. Sets the link between graphics and CAD files.
		 * 
		 * for linked CAD files, no callback should be called
		 * @param {Object} documentsIdentifier - object with all document information (specific for webcenter)
		 * @param {Function} callback - for every generated document, the callback is executed
		 */
		AbstractDataAdapter.prototype.loadDocuments = function (documentsIdentifier, callback) {
			//information passed is an object containing all the data for initializing documents, documentversions and documentpages		
		};


		/**
		 * Makes a deep copy of a page, this means it reinitializes the document and document version. All parameters and data of the document, 
		 * document version and document page are copied to the new document, document version and document page 
		 * 
		 * @param {AbstractDocumentPage} page
		 * @returns {AbstractDocumentPage}
		 */
		AbstractDataAdapter.prototype.getDeepCopy = function (page) {
			//to be implemented by a specific dataAdapter
		};

		/**
		 * Load previous versions of the document for a given document page. If the version does not have the page with given pagenumber the version
		 * will not be loaded (only happens for pagelist documents). 
		 * 
		 * @param {Object} data
		 * @param {AbstractDocumentPage} document
		 * @param {Function} callback
		 * @param {WebCenterDocumentPage} [documentPage]
		 */
		AbstractDataAdapter.prototype.loadVersionsOfDocumentPage = function (data, document, callback, documentPage) {
			//to be implemented by a specific dataAdapter
		};


		/**
		 * Update the given pages with correct data.
		 * 
		 * !! Note !! if two consecutive calls are done, the first (ongoing) call should be cancelled, only the last call result can update the parameters
		 * @param {AbstractDocumentPage} currentPage - the currenct active page
		 * @param {AbstractDocumentPage} differencePage - the current d
		 * @param {AbstractDocumentPage} cadOverlay - the current cad overlay
		 * @param {function} callback - callback to be called when parameter call has finished
		 */
		AbstractDataAdapter.prototype.updateDocumentParameters = function (currentPage, differencePage, cadOverlay, callback) {
			//to be implemented by a specific dataAdapter
		};

		/**
		 * Update the data of the document, also updates the document reference info
		 * @param {AbstractDocumentPage}   page  
		 */
		AbstractDataAdapter.prototype.updateDocumentData = function (page) {
			//to be implemented by a specific dataAdapter
		};

		/**
		 * If a parameter update is ongoing, it should be cancelled, and it it shouldn't update any documents or trigger events
		 */
		AbstractDataAdapter.prototype.cancelDocumentParametersUpdate = function () {
			// to be implemented  by a specific dataAdapter
		};

		/**
		 * Save the CAD overlay settings
		 * @param {AbstractDocumentPage} graphicsPage - the graphics page in which the CAD reference should be stored
		 * @param {AbstractDocumentPage} cadOverlay   - the CAD document that should be set as reference
		 * @param {Number}               rotation     - rotation of the CAD overlay (0,90,180,270 degrees)
		 * @param {Boolean}              isFlipped    - is the CAD overlay flipped
		 * @param {ShapeUtils.Point}     offset       - the offset for the CAD overlay
		 * @param {Function}             [callback]   - callback function to be called when CAD overlay is saved (first parameter of the callback is a boolean indicating success)
		 */
		AbstractDataAdapter.prototype.saveCadOverlay = function (graphicsPage, CADPage, rotation, isFlipped, offset, callback) {
			// to be implemented by a specific dataAdapter
		};

		/**
		 * Open a 3D Viewer for a given page in a new browser tab
		 * @param {AbstractDocumentPage} page
		 */
		AbstractDataAdapter.prototype.show3DView = function (page) {
			//to be implemented
		};
		return AbstractDataAdapter;
	});

define('web-viewer/data/AbstractDocumentPage',["web-viewer/core/Extendable",
		"web-viewer/core/PrivateChannel",
		"web-viewer/data/DataConstants",
		"underscore",
		"utils/ShapeUtils"
	],
	function (Extendable,
		PrivateChannel,
		DataConstants,
		_,
		ShapeUtils) {
		"use strict";


		/**
		 * @class Abstract representation of a Document page. Extend this class for a specific implementation of the document page.
		 * @exports AbstractDocumentPage
		 * @memberof Data
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {Data.AbstractDocumentVersion} docVersion - the documentversion which this page belongs to
		 * @param {Number} pagenum - the number of the page
		 */
		var AbstractDocumentPage = function (f, docVersion, pagenum) {
			this.f = f;
			var privateChannel = ["doc", docVersion.getDocument().getIID()];
			privateChannel.push("version", docVersion.getVersionId());
			privateChannel.push("page", pagenum);
			PrivateChannel.installTo(this, f, privateChannel);

			/**
			 * Object containing general data of the page
			 * This information lives during the complete viewer session
			 * @private
			 */
			this._data = {};

			/**
			 * viewer parameters
			 * This information is constantly updated, and is very volatile.
			 * Caution should be taken when setting values on _parameters
			 * @private
			 */
			this._parameters = null;

			/**
			 * State data, this information is stored as long as a page is active or difference page
			 * @private
			 */
			this._state = {};
			/**
			 * Are there annotations available?
			 * @private
			 */
			this._hasAnnotations = false;

			/**
			 * The documentversion to which this page belongs
			 * @private
			 */
			this._docVersion = docVersion;

			/**
			 * The sequence number of the page (counting from 1)
			 * @private
			 */
			this._pagenum = parseInt(pagenum, 10);

			/**
			 * Is the page displayed in Readerspread
			 * @private
			 */
			this._isReaderSpread = false;

			/**
			 * Linked compare page (multi-page documents)
			 */
			this._linkedComparePage = null;

			/**
			 * Is this page a blank page, has no Graphics or CAD data
			 */
			this._isBlankPage = false;


			/**
			 * Indicates whether parameters are available or not
			 */
			this._isParametersAvailable = false;
		};


		/**
		 * Get a property of the page
		 *
		 * @param {String} key -Key of the property
		 * @returns {Any} Value of the property with the given key or undefined when not found
		 */
		AbstractDocumentPage.prototype.get = function (key) {
			if (this._data[key] !== undefined) {
				return this._data[key];
			} else {
				return undefined;
			}
		};

		/**
		 * Set a property of the page
		 *
		 * @param {String} key - Key of the property
		 * @param {Any} Value of the property with the given key
		 */
		AbstractDocumentPage.prototype.set = function (key, value) {
			var oldValue = this._data[key];
			
			this._data[key] = value;
			
			if (oldValue != value){		// != is intentional		
				this._emit(["property", key, "changed"], oldValue, value);
			}
		};

		/**
		 * reset a property of the page
		 *
		 * @param {String} key - Key of the property
		 */
		AbstractDocumentPage.prototype.reset = function (key) {
			delete this._data[key];
		};

		/**
		 * Get a copy of the page data
		 * @returns {Object} Copy of the page data
		 */
		AbstractDocumentPage.prototype.getData = function () {
			return _.clone(this._data);
		};

		/**
		 * @returns {Data.AbstractDocumentVersion} the documentversion that this page belongs to
		 */
		AbstractDocumentPage.prototype.getDocumentVersion = function () {
			return this._docVersion;
		};

		/**
		 * @returns the number of the page (counting from 1)
		 */
		AbstractDocumentPage.prototype.getPagenum = function () {
			return this._pagenum;
		};

		/**
		 * @returns {Data.AbstractDocument} the document that this page belongs to
		 */
		AbstractDocumentPage.prototype.getDocument = function () {
			return this._docVersion.getDocument();
		};

		/**
		 * Get a viewer parameter
		 * @param {String} key - Key of the property
		 * @returns {Any} Viewer parameter value
		 */
		AbstractDocumentPage.prototype.getParam = function (key) {
			if (this._parameters !== null && this._parameters[key] !== undefined) {
				return this._parameters[key];
			} else {
				return undefined;
			}
		};

		/**
		 * Returns if the document page has the param
		 * @returns {Boolean}
		 */
		AbstractDocumentPage.prototype.hasParam = function (key) {
			return this._parameters !== null && this._parameters[key] !== undefined;
		};

		/**
		 * Get a copy of all viewer parameters
		 * @returns {Object} Copy of all viewer parameters
		 */
		AbstractDocumentPage.prototype.getParams = function () {
			return _.clone(this._parameters);
		};

		/**
		 * Sets a parameter of the document page.
		 * @param {String} key
		 * @param {Any} value
		 */
		AbstractDocumentPage.prototype.setParam = function (key, value) {
			this._parameters[key] = value;
			this._emit("parameters.changed", this);
		};

		/**
		 * Reset a parameter of the document page
		 * @param {String} key
		 */
		AbstractDocumentPage.prototype.resetParam = function (key) {
			delete this._parameters[key];
			this._emit("parameters.changed", this);
		};

		/**
		 * Rotate the document page over rotation (this adds rotation to the current rotation)
		 * @param {Number} rotation
		 */
		AbstractDocumentPage.prototype.rotatePage = function (rotation) {
			this.setRotation(this.getRotation() + rotation);
		};

		/**
		 * Set the rotation of the page (used for difference pages)
		 * @param {Number} rotation
		 */
		AbstractDocumentPage.prototype.setRotation = function (rotation) {
			this.setStateParam("rotation", rotation % 360);
		};

		/**
		 * Flip the document page
		 * @param {Boolean} [flip] - if undefined just flip from the current flip state, otherwhise if true : page is flipped, if false page is not flipped
		 */
		AbstractDocumentPage.prototype.flipPage = function (flip) {
			if (flip === undefined) {
				flip = !this.getStateParam("flipped");
			}

			this.setStateParam("flipped", flip);
		};

		/**
		 * get the rotation of the document page
		 * @returns {Number}
		 */
		AbstractDocumentPage.prototype.getRotation = function () {
			var rotation = this.getStateParam("rotation");
			if (rotation) {
				return rotation * 1;
			} else {
				return 0;
			}
		};

		/**
		 * @returns {Boolean} - is the document page flipped
		 */
		AbstractDocumentPage.prototype.isFlipped = function () {
			return this.getStateParam("flipped") === undefined ? false : (this.getStateParam("flipped") && true);
		};

		/**
		 * Returns the origin of the CAD-file, only if the CAD-origin is above and/or left of the graphics' (0,0)-point
		 * if this is not the case it returns (0,0)
		 *
		 * @returns {ShapeUtils.Point}
		 */
		AbstractDocumentPage.prototype.getOrigin = function () {
			return new ShapeUtils.Point();
		};

		/**
		 * Get the size of the page (note in readerspread this is the double width of the page)
		 * @returns {ShapeUtils.Size}
		 */
		AbstractDocumentPage.prototype.getPageSize = function () {
			return new ShapeUtils.Size();
		};

		/**
		 * Get the original size of the page
		 * @returns {ShapeUtils.Size}
		 */
		AbstractDocumentPage.prototype.getOriginalSize = function () {
			return new ShapeUtils.Size();
		};

		/**
		 * Get Image type
		 * @returns {Number} image type or -1 if parameters not yet loaded
		 */
		AbstractDocumentPage.prototype.getImageType = function () {
			return this._parameters !== null ? parseInt(this._parameters.imagetype, 10) : -1;
		};

		/**
		 * Load the URL to fetch an image based on parameters
		 * @abstract
		 * @param {Object} params - Parameters for imagedata
		 * @returns {String} URL to fetch an image based on parameters
		 */
		AbstractDocumentPage.prototype.getImageURL = function (params) {
			this.f.emit("error.SEVERE", "No getImageURL implementation is provided");
			return "UNDEFINED";
		};

		/**
		 * Load the URL to fetch info based on parameters
		 * @abstract
		 * @param {Object} params - Parameters for info
		 * @returns {String} URL to fetch info based on parameters
		 */
		AbstractDocumentPage.prototype.getInfoURL = function (params) {
			this.f.emit("error.SEVERE", "No getInfoURL implementation is provided");
			return "UNDEFINED";
		};

		/**
		 * Get the margin of the document, must be implemented
		 * @abstract
		 * @returns {Object} margin{bottom, top, left, right}
		 */
		AbstractDocumentPage.prototype.getMargins = function () {
			return undefined;
		};

		/**
		 * Does the document have graphics
		 * @returns {Boolean} True if image, false otherwise
		 */
		AbstractDocumentPage.prototype.hasGraphics = function () {
			return this._data.file !== undefined;
		};

		/**
		 * Has annotations
		 * @param {String} [originType] - checks if annotations are available for this origin type, if omitted will check if there are annotations in general
		 * @returns {Boolean} True if it has annotations, false otherwise
		 */
		AbstractDocumentPage.prototype.hasAnnotations = function (originType) {
			if(originType && this._hasAnnotationsByOrigin) {
				return this._hasAnnotationsByOrigin[originType];
			}
			return this._hasAnnotations;
		};

		/**
		 * Has CAD
		 * @returns {Boolean} True if has cad, false otherwise
		 */
		AbstractDocumentPage.prototype.hasCad = function () {
			return (this._data.cad ? true : false);
		};

		/**
		 * Checks if the document of this page is linked with another document (Graphics that are linked with CAD)
		 * @returns {Boolean}
		 */
		AbstractDocumentPage.prototype.hasLinkedPage = function () {
			return this.getDocument().hasLinkedDocument();
		};

		/**
		 * returns the linked document page
		 * @returns {AbstractDocumentPage}
		 */
		AbstractDocumentPage.prototype.getLinkedPage = function () {
			return this.getDocument().getLinkedDocumentPage();
		};

		/**
		 * Get the unique ID for this page, build as follows: "docIID_versionID_pagenum"
		 * @returns {String}
		 */
		AbstractDocumentPage.prototype.getUniqueID = function () {
			return this._docVersion.getDocument().getIID() + "_" + this._docVersion.getVersionId() + "_" + this._pagenum;
		};

		/**
		 * Are the parameters available
		 * @returns {Boolean}
		 */
		AbstractDocumentPage.prototype.isParametersAvailable = function () {
			return this._parameters !== null && this._isParametersAvailable;
		};

		/**
		 * Execute some code when the parameters for this page are available
		 * @param {Function} callback - first parameter of the callback is the page itself
		 * @param {Object} [binding] - binding for callback (optional)
		 */
		AbstractDocumentPage.prototype.whenParametersAvailable = function (callback, binding) {
			if (binding) {
				callback = _.bind(callback, binding);
			}

			if (this.isParametersAvailable()) {
				callback(this);
			} else {
				this.once("parameters.available", _.bind(function () {
					callback(this);
				}, this));
			}
		};

		/**
		 * Set the parameters.
		 * @param {Object} parameters
		 */
		AbstractDocumentPage.prototype.setParameters = function (parameters) {
			this._parameters = parameters;
			this._emit("parameters.changed", this);
		};

		/**
		 * Resets all the parameters so they need be loaded again
		 */
		AbstractDocumentPage.prototype.resetParameters = function () {
			this._parameters = null;
		};

		/**
		 * Inject the needed parameters for the page when a fragment url is requested
		 * @params {Object} c - the context of intercept.fragmentURL
		 */
		AbstractDocumentPage.prototype.interceptFragmentURL = function () {
			// nothing is added by default
		};

		/**
		 * Set if the page has annotations or not
		 * @param {Boolean} hasAnnotations
		 * @param {Object} [hasAnnotationsByOrigin] - used to specify which annotation is available based on the origin
		 */
		AbstractDocumentPage.prototype.setHasAnnotations = function (hasAnnotations, hasAnnotationsByOrigin) {
			if (hasAnnotations !== this._hasAnnotations) {
				this._hasAnnotations = hasAnnotations;
				this._emit("hasAnnotations.changed", this);
			}
			this._hasAnnotationsByOrigin = hasAnnotationsByOrigin;
		};

		/**
		 * Does this page equals another page?
		 * @param {Data.AbstractDocumentPage} other
		 */
		AbstractDocumentPage.prototype.equals = function (other) {
			if (other && this.getUniqueID() == other.getUniqueID()) {
				if (this._docVersion == other._docVersion) {
					if (this._hasAnnotations == other._hasAnnotations) {
						if (_.every(this._data, function (value, key) {
								return value == other._data[key];
							})) {
							return true;
						}
					}
				}
			}
			return false;
		};

		/**
		 * Indicate that parameters are available for this file
		 * @param {Boolean} available - true if parameters are set
		 */
		AbstractDocumentPage.prototype.setParametersAvailable = function (available) {
			this._isParametersAvailable = available;

			if (available) {
				this._emit("parameters.available");
			}
		};

		/**
		 * Notify that (raw) CAD data is avaialble
		 * NOTE!! Should only be used by datadapter/documentcontroler
		 */
		AbstractDocumentPage.prototype.setCadDataAvailable = function () {
			this._emit("cadData.available");
		};



		/**
		 * Set if the page is in reader spread or not
		 * @param {Boolean} isReaderSpread
		 */
		AbstractDocumentPage.prototype.setReaderSpread = function (isReaderSpread) {
			this._isReaderSpread = isReaderSpread;
		};

		/**
		 * Is the current document page in reader spread
		 * @returns {Boolean}
		 */
		AbstractDocumentPage.prototype.isReaderSpread = function () {
			return this._isReaderSpread;
		};

		/**
		 * Get the next page in the document version with respect to this page
		 * @param {Number} [times] How many times next?
		 * @param {Boolean} [skipBlankPages] indicates if the next page can or cannot be an empty page
		 * @returns {Data.AbstractDocumentPage} The next page in the document version with respect to this page. The next of the last page, stays the last page.
		 */
		AbstractDocumentPage.prototype.getNextPage = function (times, skipBlankPages) {
			var documentVersion = this.getDocumentVersion();
			if (times === undefined) {
				times = 1;
			}
			var nextPageNumber = Math.min(this.getPagenum() + times, documentVersion.getPageCount());
			var nextPage = documentVersion.getPage(nextPageNumber);
			while ((nextPage === undefined || (skipBlankPages && nextPage.isBlankPage())) && ++nextPageNumber <= documentVersion.getPageCount()) {
				nextPage = documentVersion.getPage(nextPageNumber);
			}
			return nextPage;
		};

		/**
		 * Get the previous page with respect to the current active page
		 * @param {Number} [times] How many times previous?
		 * @param {Boolean} [skipBlankPages] indicates if the next page can or cannot be an empty page
		 * @returns {Data.AbstractDocumentPage} The previous page with respect to the current active page. The previous of the first page, stays the first page.
		 */
		AbstractDocumentPage.prototype.getPreviousPage = function (times, skipBlankPages) {
			var documentVersion = this.getDocumentVersion();
			if (times === undefined) {
				times = 1;
			}
			var previousPageNumber = Math.max(this.getPagenum() - times, 1);
			var previousPage = documentVersion.getPage(previousPageNumber);
			while ((previousPage === undefined || (skipBlankPages && previousPage.isBlankPage())) && --previousPageNumber >= 1) {
				previousPage = documentVersion.getPage(previousPageNumber);
			}
			return previousPage;
		};


		/**
		 * Get the linked compare page of this page
		 * @returns {AbstractDocumentPage} Linked compare page or null if none set.
		 */
		AbstractDocumentPage.prototype.getLinkedComparePage = function () {
			return this._linkedComparePage;
		};

		/**
		 * Set the linked compare page of this page
		 * @param {AbstractDocumentPage} linkedComparePage New linked compare page of this page.
		 */
		AbstractDocumentPage.prototype.setLinkedComparePage = function (linkedComparePage) {
			this._linkedComparePage = linkedComparePage;
		};

		/**
		 * Reset the linked compare page
		 */
		AbstractDocumentPage.prototype.resetLinkedComparePage = function () {
			this._linkedComparePage = null;
		};

		/**
		 * Has this page a linked compare page?
		 * @returns {Boolean} True if it has a linked compare page, false otherwise
		 */
		AbstractDocumentPage.prototype.hasLinkedComparePage = function () {
			return this._linkedComparePage !== null;
		};

		/**
		 * Is the rectangle on the page, only for part on the page returns true
		 * @param {ShapeUtils.Rect} rect
		 * @returns {Boolean}
		 */
		AbstractDocumentPage.prototype.isRectOnPage = function (rect) {
			//to be implemented 
		};

		/**
		 * Return the version Id for this page (used for page list document)
		 * @returns {Number} - version id for this page
		 */
		AbstractDocumentPage.prototype.getPageVersionId = function () {
			//to be implemented, only use case is page list documents
			return null;
		};

		/**
		 * Is this page a blank page?, has no Graphics or CAD data
		 * @returns {Boolean}
		 */
		AbstractDocumentPage.prototype.isBlankPage = function () {
			return this._isBlankPage;
		};

		/**
		 * Check if the page is a digital film
		 * @returns {Boolean}
		 */
		AbstractDocumentPage.prototype.isDigitalFilmPage = function () {
			return false;
		};

		/**
		 * Set a state parameter. This is information lives as long as a page is active or set as difference page
		 * @param {String} key - Key of the state parameter
		 * @param {Object} value - Value for the state parameter
		 */
		AbstractDocumentPage.prototype.setStateParam = function (key, value) {
			this._state[key] = value;
			this._emit("stateParameter.changed", this, key);
		};

		/**
		 * Checks if the state parameter with the key is set in this document page
		 * @param   {String} key - Key of the state parameter
		 * @returns {Boolean}
		 */
		AbstractDocumentPage.prototype.hasStateParam = function (key) {
			return this._state[key] !== undefined;
		};

		/**
		 * Get a state parameter
		 * @param {String} key - Key of the state parameter
		 * @returns {Object} value for the given key or undefined if not found
		 */
		AbstractDocumentPage.prototype.getStateParam = function (key) {
			return this._state[key];
		};

		/**
		 * Reset the value of a state param
		 * @param key
		 */
		AbstractDocumentPage.prototype.resetStateParam = function (key) {
			delete this._state[key];
		};


		/**
		 * Reset all data stored in the state
		 */
		AbstractDocumentPage.prototype.resetState = function () {
			this._state = {};
		};


		/**
		 * Get the url for fetching tumbnails (50x50)
		 * @returns {String} - url for fetching the thumbdata of the page
		 */
		AbstractDocumentPage.prototype.getThumbnailUrl = function () {
			//to be implemented in subclass
			return null;
		};

		/**
		 * Returns the information needed for a DocumentDropdownmenu to add an document to the dropdown menu
		 * @returns {Object}
		 */
		AbstractDocumentPage.prototype.getInformationForDocumentSelector = function () {
			//to be implemented in subclass
			return null;
		};

		/**
		 * Can the CAD overlay be saved for this file?
		 * @returns {Boolean}
		 */
		AbstractDocumentPage.prototype.canSaveCadOverlay = function () {
			// to be implemented in subclass
			return false;
		};

		/**
		 * Does the document page need an update of its data
		 * @returns {Boolean}
		 */
		AbstractDocumentPage.prototype.needsDataUpdate = function () {
			// to be implemented in subclass
			return false;
		};

		return AbstractDocumentPage;
	});

define('web-viewer/data/PreferencesConstants',{
	PREFERENCES_UNIT_SIZE: 1,
	PREFERENCES_UNIT_RESOLUTION: 2,
	PREFERENCES_VIEW_ZOOM_MODE: 3,
	PREFERENCES_ANNOTATION_NEXT_MODE: 4,
	PREFERENCES_CHANNELS_SINGESEPBLACK: 5,
	PREFERENCES_FILEOPEN_RECENTPATH: 6,
	PREFERENCES_FILEOPEN_FILTER: 7,
	PREFERENCES_IMAGE_BACKGROUNDCOLOR: 8,
	PREFERENCES_WINDOW_BACKGROUNDCOLOR: 9,
	PREFERENCES_UNIT_SCREEN: 10,
	PREFERENCES_UNIT_ANGLE: 11,
	PREFERENCES_DISPLAY_MARGINS: 12,
	PREFERENCES_MERGEFILES: 13,
	PREFERENCES_HELPBLOCKERWARNING: 14,
	PREFERENCES_DIFFMODE: 15,
	PREFERENCES_RULER_DISPLAY_MODE: 16,
	PREFERENCES_RULER_DISPLAY_COLOR: 17,
	PREFERENCES_UPDATE_PALETTE: 18,
	PREFERENCES_ENABLE_BACKGROUNDCOLOR: 19,
	PREFERENCES_SCREEN_ORIENTATION: 20,
	PREFERENCES_DIFFMODE_COMPARE: 21,
	PREFERENCES_DOCUMENTHISTORYDIALOG_DETAILSMODE: 22,
	PREFERENCES_DOCUMENTHISTORYDIALOG_DOCKED: 23,
	PREFERENCES_DOCUMENTSELECTDIALOG_DOCKED: 24,
	PREFERENCES_RULER_ANGLE_MODE: 25,
	PREFERENCES_RECTOVERSO_MODE: 26,
	PREFERENCES_CHANNELNAME_MODE: 27,
	PREFERENCES_COLORMATCHING: 28,
	PREFERENCES_COLORMATCHING_PROMPT: 29,
	PREFERENCES_DOCUMENTHISTORYDIALOG_REOPEN: 30,
	PREFERENCES_DOCUMENTSELECTDIALOG_REOPEN: 31,
	PREFERENCES_PAGELISTDIALOG_REOPEN: 32,
	PREFERENCES_PAGELISTDIALOG_DOCKED: 33,
	PREFERENCES_PAGELIST_ICON_SIZE: 34,
	PREFERENCES_READOUTDIALOG_SCREEN_VISIBLE: 35,
	PREFERENCES_READOUTDIALOG_DOT_VISIBLE: 36,
	PREFERENCES_SHOWBRIDGES_MODE: 37,
	PREFERENCES_ANNOTATION_NOTE_COLOR: 38,
	PREFERENCES_ANNOTATION_RECTANGLE_COLOR: 39,
	PREFERENCES_DOCUMENTANNOTATIONDIALOG_REOPEN: 40,
	PREFERENCES_DOCUMENTANNOTATIONDIALOG_DOCKED: 41,
	PREFERENCES_ANNOTATION_FITVIEW: 42,
	PREFERENCES_ANNOTATION_SHOWSELECTEDONLY: 43,
	PREFERENCES_ZOOMTOOLS_VISIBLE: 44,
	PREFERENCES_TOOLSPANEL_VISIBLE: 45,
	PREFERENCES_MARGIN_CLIP_MODE: 46,
	PREFERENCES_MARGIN_DISPLAY_MODE: 47,
	PREFERENCES_PANELMAINTOOLS_SKOWMASK: 48,
	PREFERENCES_TOOLSPANELTOP_VISIBLE: 49,
	PREFERENCES_ANNOTATION_SHOWCOUNT: 50,
	PREFERENCES_ANNOTATION_SHOWSELECTEDPOPUPONLY: 51,
	PREFERENCES_ANNOTATIONTOOLS_VISIBLE: 52,
	PREFERENCES_ANNOTATION_TOOLMODE: 53,
	PREFERENCES_ANNOTATION_DATESORT_MODE: 54,
	PREFERENCES_READOUTDIALOG_BARCODE_VISIBLE: 55,
	PREFERENCES_DIFF_SPLITMODE: 56,
	PREFERENCES_INKBOOKPATH: 57,
	PREFERENCES_READOUTDIALOG_BRAILLE_VISIBLE: 58,
	PREFERENCES_VIEWX_PARAMETERS: 59,
	PREFERENCES_VIEWXTOOLS_VISIBLE: 60,
	PREFERENCES_BRAILLE_LANGUAGE: 61,
	PREFERENCES_MERGE_SIMILAR_INKS: 62,
	PREFERENCES_VIEWXTOOLS_ENABLE1BIT: 63,
	PREFERENCES_ANTIALIASING: 64,
	PREFERENCES_ANNOTATION_SEQLABEL_VISIBLE: 65,
	PREFERENCES_PILOT_SELECTSERVER: 66,
	PREFERENCES_PILOT_SERVERNAME: 67,

	// In the Java Viewer, the 3 settings below are the custom annotation filter 
	PREFERENCES_RULER_DISPLAY_MODE_DA: 68,
	PREFERENCES_RULER_DISPLAY_MODE_XY: 69,
	PREFERENCES_RULER_DISPLAY_MODE_WH: 70,


	PREFERENCES_CALIPER_DISPLAY_MODE: 71, // boolean; true=Fixed size caliper
	PREFERENCE_MONITOR_RES: 72, // double; def 72/25.4 dpmm
	PREFERENCES_ZOOM_REALSIZE: 73, // boolean; def false=do not force OneToOne to be real size always

	// Print preferences 70 to 89
	PREFERENCES_PRINT_SPLITVIEW: 78,
	PREFERENCES_PRINT_MAGNIFICATION: 79,
	PREFERENCES_PRINT_MARG_TOP: 80,
	PREFERENCES_PRINT_MARG_LEFT: 81,
	PREFERENCES_PRINT_MARG_BOTTOM: 82,
	PREFERENCES_PRINT_MARG_RIGHT: 83,
	PREFERENCES_PRINT_ORIENTATION: 84,
	PREFERENCES_PRINT_TYPE: 85,
	PREFERENCES_PRINT_AREA: 86,
	PREFERENCES_PRINT_CHANNELS: 87,
	PREFERENCES_PRINT_RESOLUTION: 88,
	PREFERENCES_PRINT_SCALETOFIT: 89,
	// Highlight color preferences 90 to 99 (90 : overprint, 91 : diff add, 92 : diff sub, 93 : seamless guide lines, 94 : margins, 95 : margins diff)
	PREFERENCES_HIGHLIGHTCOLORS: 90,
	PREFERENCES_HIGHLIGHTCOLORS_DIFF_ADD: 91,
	PREFERENCES_HIGHLIGHTCOLORS_DIFF_SUB: 92,
	PREFERENCES_HIGHLIGHTCOLORS_SEAMLESS_GUIDELINES: 93,
	PREFERENCES_HIGHLIGHTCOLORS_MARGINS: 94,
	PREFERENCES_HIGHLIGHTCOLORS_MARGINS_DIFF: 95,

	// Layout in standard mode 100 to 199
	LAYOUT_EXPAND_PAGETOOLS_STATE: 100,
	LAYOUT_EXPAND_CHANNELS_STATE: 101,
	LAYOUT_EXPAND_NAVIGATOR_STATE: 102,
	LAYOUT_WINDOW_SIZE_X: 103,
	LAYOUT_WINDOW_SIZE_Y: 104,
	LAYOUT_MODE: 105,
	LAYOUT_MAIN_PANEL_RIGHT_SPLITTER: 106,
	LAYOUT_EXPAND_ANNOTATIONS_STATE: 107,
	LAYOUT_EXPAND_APPROVAL_STATE: 108,
	LAYOUT_NAVIGATOR_SIZE_X: 109,
	LAYOUT_NAVIGATOR_SIZE_Y: 110,
	LAYOUT_EXPAND_CAD_STATE: 111,
	LAYOUT_LAYERSTABLE_SIZE: 112,
	LAYOUT_CLASSESTABLE_SIZE: 113,
	LAYOUT_FRAME_SIZE_X: 114,
	LAYOUT_FRAME_SIZE_Y: 115,
	LAYOUT_FRAME_MAXIMIZED: 116,
	LAYOUT_EXPAND_PAGEVIEW_STATE: 117,
	LAYOUT_EXPAND_INFO_STATE: 118,
	LAYOUT_LOOK_AND_FEEL: 119,
	LAYOUT_CHANNELEDITDIALOG_WIDTH: 120,
	LAYOUT_DOCUMENTSELECTDIALOG_WIDTH: 121,
	LAYOUT_DOCUMENTSELECTDIALOG_HEIGHT: 122,
	LAYOUT_DOCUMENTHISTORYDIALOG_WIDTH: 123,
	LAYOUT_DOCUMENTHISTORYDIALOG_HEIGHT: 124,
	LAYOUT_MAIN_PANEL_LEFT_SPLITTER: 125,
	LAYOUT_EXPAND_DIFFADJUST: 126,
	LAYOUT_PAGELISTDIALOG_WIDTH: 127,
	LAYOUT_PAGELISTDIALOG_HEIGHT: 128,
	LAYOUT_DOCUMENTANNOTATIONDIALOG_WIDTH: 129,
	LAYOUT_DOCUMENTANNOTATIONDIALOG_HEIGHT: 130,

	// MRU list reserves IDs from 200 to 299
	MRU_COUNT: 200,
	MRU_ENTRY1: 201,
	// Path map list (automatic location of diff files) reserves IDs from 300 to 399
	PML_ENTRY1: 301,
	PML_MAXENTRY: 399,

	// Application configuration settings 400 to 499 (only read)
	CONFIG_ISGRAPHOLAS: 400,
	CONFIG_GRAPHOLASHOME: 401,
	CONFIG_SHOWDIFFADJUST: 402,
	CONFIG_COLORMATCHING: 403,
	CONFIG_TOOLTIP_INITIAL: 404,
	CONFIG_TOOLTIP_RESHOW: 405,
	CONFIG_TOOLTIP_DISMISS: 406,
	CONFIG_DIFF_PERCENTAGE: 407,
	CONFIG_DIFF_COMPAREMODE: 408,
	CONFIG_GRAPHOLAS_CENTERLINE: 409,
	CONFIG_GRAPHOLAS_MICROCROSS: 410,

	//  500: caliper tool count
	//  501: caliper tool first entry

	/**
	 * HTML5 Viewer settings 1000 and more
	 */
	// Layout state
	LAYOUT_STATE: 1000,
	LAYOUT_PREFERENCE: 1010,

	// Caliper presets
	CALIPER_PRESETS: 1100,

	// Eye dropper units
	PREFERENCES_UNIT_EYEDROPPER: 1200, // not persisted
	// Screen ruling unit (lpmm, lpcm, lpi)
	PREFERENCES_UNIT_SCREENRULING: 1201, // persisted

	ANNOTATION_FILTER: 1300,
	PREFERENCES_DIFFMODE_THRESHOLD: 1400,
	PREFERENCES_DYNCONTENTTOOLS_VISIBLE: 1500
});

define('web-viewer/data/PreferencesDefaults',{
	PREFERENCES_ANNOTATION_NEXT_MODE: 1,
	PREFERENCES_UNIT_RESOLUTION: 2,
	PREFERENCES_READOUTDIALOG_DOT_VISIBLE: 1,
	PREFERENCES_DISPLAY_MARGINS: 0,
	LAYOUT_EXPAND_PAGEVIEW_STATE: 1,
	PREFERENCES_ANNOTATIONTOOLS_VISIBLE: 1,
	PREFERENCES_DYNCONTENTTOOLS_VISIBLE: 1,
	PREFERENCES_DOCUMENTSELECTDIALOG_DOCKED: 1,
	PREFERENCES_READOUTDIALOG_SCREEN_VISIBLE: 1,
	PREFERENCES_MERGE_SIMILAR_INKS: -1,
	PREFERENCES_ANNOTATION_FITVIEW: 1,
	PREFERENCES_PANELMAINTOOLS_SKOWMASK: 127,
	PREFERENCES_DOCUMENTANNOTATIONDIALOG_REOPEN: 1,
	LAYOUT_EXPAND_CHANNELS_STATE: 1,
	PREFERENCES_ZOOMTOOLS_VISIBLE: 0,
	PREFERENCES_MERGEFILES: 1,
	LAYOUT_MODE: 1,
	PREFERENCES_CHANNELNAME_MODE: 0,
	PREFERENCES_ANNOTATION_SEQLABEL_VISIBLE: 0,
	LAYOUT_NAVIGATOR_SIZE_Y: 128,
	PREFERENCES_ANNOTATION_RECTANGLE_COLOR: -65536,
	PREFERENCES_HIGHLIGHTCOLORS_DIFF_ADD: -16776961,
	PREFERENCES_HIGHLIGHTCOLORS_MARGINS: -16744193,
	PREFERENCES_MARGIN_CLIP_MODE: 1,
	PREFERENCES_DOCUMENTHISTORYDIALOG_DOCKED: 1,
	PREFERENCES_ANNOTATION_SHOWSELECTEDONLY: 1,
	LAYOUT_NAVIGATOR_SIZE_X: 128,
	LAYOUT_MAIN_PANEL_RIGHT_SPLITTER: 203,
	PREFERENCES_WINDOW_BACKGROUNDCOLOR: 5592405,
	PREFERENCES_ANTIALIASING: -1,
	PREFERENCES_MARGIN_DISPLAY_MODE: 0,
	PREFERENCES_HIGHLIGHTCOLORS_DIFF_SUB: -65536,
	PREFERENCES_HIGHLIGHTCOLORS: -1,
	PREFERENCES_ANNOTATION_DATESORT_MODE: 0,
	PREFERENCES_DOCUMENTSELECTDIALOG_REOPEN: 0,
	PREFERENCES_DIFFMODE: 1,
	PREFERENCES_DIFFMODE_COMPARE: 2,
	PREFERENCES_PAGELISTDIALOG_DOCKED: 1,
	PREFERENCES_RULER_DISPLAY_MODE: 1,
	LAYOUT_EXPAND_NAVIGATOR_STATE: 1,
	LAYOUT_MAIN_PANEL_LEFT_SPLITTER: 209,
	PREFERENCES_TOOLSPANEL_VISIBLE: 0,
	PREFERENCES_VIEW_ZOOM_MODE: 0,
	PREFERENCES_IMAGE_BACKGROUNDCOLOR: -1,
	LAYOUT_EXPAND_DIFFADJUST: 0,
	MRU_COUNT: 0,
	PREFERENCES_ANNOTATION_SHOWCOUNT: 0,
	PREFERENCES_ANNOTATION_SHOWSELECTEDPOPUPONLY: 1,
	LAYOUT_CLASSESTABLE_SIZE: 83,
	LAYOUT_EXPAND_PAGETOOLS_STATE: 1,
	PREFERENCES_UNIT_ANGLE: 0,
	LAYOUT_EXPAND_APPROVAL_STATE: 1,
	LAYOUT_EXPAND_INFO_STATE: 1,
	CONFIG_SHOWDIFFADJUST: 0,
	PREFERENCES_RULER_DISPLAY_COLOR: -13022894,
	PREFERENCES_HIGHLIGHTCOLORS_SEAMLESS_GUIDELINES: -16711936,
	PREFERENCES_VIEWXTOOLS_ENABLE1BIT: 0,
	PREFERENCES_PAGELISTDIALOG_REOPEN: 0,
	LAYOUT_LOOK_AND_FEEL: 0,
	PREFERENCES_ANNOTATION_NOTE_COLOR: -256,
	PREFERENCES_UNIT_SCREEN: 2,
	PREFERENCES_VIEWX_PARAMETERS: "",
	PREFERENCES_DIFF_SPLITMODE: 4,
	PREFERENCES_DOCUMENTHISTORYDIALOG_REOPEN: 0,
	CONFIG_DIFF_COMPAREMODE: 0,
	PREFERENCES_SHOWBRIDGES_MODE: 2,
	LAYOUT_EXPAND_CAD_STATE: 1,
	PREFERENCES_TOOLSPANELTOP_VISIBLE: 1,
	PREFERENCES_READOUTDIALOG_BRAILLE_VISIBLE: 1,
	LAYOUT_LAYERSTABLE_SIZE: 83,
	PREFERENCES_COLORMATCHING: 2,
	PREFERENCES_READOUTDIALOG_BARCODE_VISIBLE: 1,
	PREFERENCES_ANNOTATION_TOOLMODE: 0,
	PREFERENCES_VIEWXTOOLS_VISIBLE: 0,
	LAYOUT_EXPAND_ANNOTATIONS_STATE: 1,
	PREFERENCES_CHANNELS_SINGESEPBLACK: 0,
	PREFERENCES_HIGHLIGHTCOLORS_MARGINS_DIFF: -32768,
	PREFERENCES_DOCUMENTANNOTATIONDIALOG_DOCKED: 1,
	PREFERENCES_BRAILLE_LANGUAGE: 0,
	PREFERENCES_RULER_DISPLAY_MODE_DA: 1,
	PREFERENCES_RULER_DISPLAY_MODE_XY: 0,
	PREFERENCES_RULER_DISPLAY_MODE_WH: 0,
	PREFERENCE_MONITOR_RES: 72 / 25.4 + "", //dpmm
	PREFERENCES_ZOOM_REALSIZE: 0,
	PREFERENCES_PRINT_TYPE: 0,
	PREFERENCES_PRINT_AREA: 0,
	PREFERENCES_PRINT_CHANNELS: 0,
	PREFERENCES_PRINT_RESOLUTION: 150 / 25.4 + "",
	PREFERENCES_PAGELIST_ICON_SIZE: 1, // MEDIUM icons
	LAYOUT_STATE: "",
	LAYOUT_PREFERENCE: "automatic",
	PREFERENCES_UNIT_EYEDROPPER: 5, // percent
	PREFERENCES_SCREEN_ORIENTATION: 1, // CW
	ANNOTATION_FILTER: "",
	PREFERENCES_DIFFMODE_THRESHOLD: 0
});

define('web-viewer/data/User',["underscore",
		"web-viewer/core/Extendable",
		"web-viewer/core/PrivateChannel",
		"web-viewer/utils/ViewerUtils",
		"web-viewer/data/PreferencesDefaults"
	],
	function (_,
		Extendable,
		PrivateChannel,
		ViewerUtils,
		PreferencesDefaults) {
		"use strict";

		/**
		 * Represents a user
		 * 
		 * @exports User
		 * @memberof Data
		 * @constructor
		 * @param {String} id - Id of the user
		 * @param {String} username - User name of the user
		 * @param {String} firstname - The first name of the user
		 * @param {String} lastname - Last name of the user
		 * @parma {String} [locale] - the locale of the user (defaults to "en")
		 */
		var User = function User(id, username, first, lastname, locale) {
			/**
			 * @private
			 */
			this.f = null;

			/**
			 * Debounced saving
			 */
			this.persistDebounced = _.debounce(_.bind(this._persistUserOptions, this), 1000);



			Extendable.installTo(this);

			/**
			 * In _options all user preferences are stored. This state could be
			 * persisted by a LayoutManager
			 * 
			 * All changes to the _options should be done with User.setOption so the
			 * correct events are sent out
			 * 
			 * @access private
			 */
			this._options = {};

			/**
			 * In here previous values are stored when an option is not protected
			 * @private
			 */
			this._originalNonPersistedValues = {};

			/**
			 * @returns {String} Id of the user
			 */
			this.getID = function () {
				return id;
			};

			/**
			 * @returns {String} Username of the user
			 */
			this.getUsername = function () {
				return username;
			};

			this.lastname = lastname;

			this.first = first;

			if (locale === undefined) {
				locale = "en";
			}
			this.locale = locale;

			/**
			 * Set the facade of the user, can be executed only once!
			 */
			this.setFacade = _.once(function (facade) {
				this.f = facade;
				PrivateChannel.installTo(this, this.f, ["user", id]);
			});


		};

		/**
		 * Get full name of the user
		 * @returns {String} Full name of the user
		 */
		User.prototype.getFullName = function () {
			return this.getUsername();
		};

		/**
		 * Get last name of the user
		 * @returns {String} Last name of the user
		 */
		User.prototype.getLastName = function () {
			return this.lastname;
		};

		/**
		 * Get the first name of the user
		 * @returns {String} first name of the user
		 */
		User.prototype.getFirstName = function () {
			return this.first;
		};

		/**
		 * Get locale of the user
		 * @returns {String} Locale of the user
		 */
		User.prototype.getLocale = function () {
			return this.locale;
		};

		/**
		 * Change an option of the user
		 * 
		 * If an option is changed, an event is send on
		 * <user.[ID].options.changed.[key]>
		 * 
		 * @param {String} key 
		 * @param {Object} newValue 
		 * @param {Boolean} [doNotPersist] if true the new value is not persisted!
		 * @param {Function} [onPersistedCallback] - Callback executed when the option is persisted. Called with parameter boolean successful (true) saved or not (false)                                  
		 */
		User.prototype.setOption = function (key, newValue, doNotPersist, onPersistedCallback) {
			var oldValue = this._options[key];
			if (oldValue !== newValue) {
				this._options[key] = newValue;
				this._emit("options.changed." + key, oldValue, newValue);
				if (!doNotPersist) {
					delete this._originalNonPersistedValues[key];
					if (onPersistedCallback) {
						this._persistUserOptions(onPersistedCallback);
					} else {
						this.persistDebounced();
					}
				} else if (this._originalNonPersistedValues[key] === undefined) {
					this._originalNonPersistedValues[key] = oldValue;
				}
			} else if (onPersistedCallback) {
				onPersistedCallback(true);
			}
		};


		/**
		 * Change multiple options of the user. This can be used to inject the
		 * state of the user
		 * 
		 * @param options
		 *            {key: value, key2: value2}
		 */
		User.prototype.setOptions = function (options) {
			_.each(options, function (value, key) {
				this.setOption(key, value);
			}, this);
		};


		/**
		 * Initialize an option (this doesn't trigger  a persist operation)
		 * @param {String} key 
		 * @param {Object} newValue 
		 */
		User.prototype.initializeOption = function (key, value) {
			var oldValue = this._options[key];
			if (oldValue !== value) {
				this._options[key] = value;
				this._emit("options.changed." + key, oldValue, value);
			}
		};

		/**
		 * intialize the options of the user. This can be used to inject the
		 * state of the user
		 * 
		 * @param options
		 *            {key: value, key2: value2}
		 */
		User.prototype.initializeOptions = function (options) {
			_.each(options, function (value, key) {
				this.initializeOption(key, value);
			}, this);
		};


		/**
		 * Get the options of the user. This method can be used to save the
		 * state of a user
		 * 
		 * @returns object with all options of the user
		 */
		User.prototype.getOptions = function () {
			return this._options;
		};

		/**
		 * Get an option of the user
		 * @param {String} option - Name of the option
		 * @returns value of the option
		 */
		User.prototype.getOption = function (option) {
			return this._options[option];
		};

		/**
		 * Get a hex color from an option
		 */
		User.prototype.getColor = function (option) {
			return "#" + ViewerUtils.hexFromInt(this.getOption(option));
		};

		/**
		 * Set a hex color
		 */
		User.prototype.setColor = function (option, hexcolor) {
			if (hexcolor.indexOf("#") > -1) {
				hexcolor = hexcolor.replace("#", "");
			}
			this.setOption(option, ViewerUtils.intFromHex(hexcolor));
		};

		/**
		 * persist the options to the backend
		 * @param {Object} options - {key: value, key2: value2}
		 * @param {Function} [onPersistedCallback] - Callback executed when the options are persisted. Called with parameter boolean successful (true) saved or not (false).
		 */
		User.prototype.persistOptions = function (options, onPersistedCallback) {
			// override
		};

		/**
		 * Load/initialize the user options from the backend
		 * @param {Function} onLoadedCallback - Callback executed when the options are loaded
		 */
		User.prototype.loadOptions = function (onLoadedCallback) {
			// override
		};

		/**
		 * Reset an option to its default value
		 * @param {String} option - the option to reset
		 */
		User.prototype.resetOption = function (option) {
			this.setOption(option, PreferencesDefaults[option]);
		};

		/**
		 * Get the given option default value
		 * @param {String} option - Name of the option
		 * @returns {String | Number} Value of the option
		 */
		User.prototype.getOptionDefault = function (option) {
			return PreferencesDefaults[option];
		};

		/**
		 * Save the user preferences
		 * 
		 * @param {function} [onPersistedCallback] - called when preferences are saved
		 */
		User.prototype._persistUserOptions = function (onPersistedCallback) {
			// clone the current options
			var options = _.clone(this._options);
			// possible some options were overridden but shouldn't be persisted, restore these
			_.each(this._originalNonPersistedValues, function (value, key) {
				options[key] = value;
			});

			this.persistOptions(options, onPersistedCallback);
		};


		return User;
	});

define('web-viewer/data/DataModule',["web-viewer/data/AbstractDataAdapter",
		"web-viewer/data/AbstractDocument",
		"web-viewer/data/AbstractDocumentPage",
		"web-viewer/data/AbstractDocumentVersion",
		"web-viewer/data/DataConstants",
		"web-viewer/data/DocumentController",
		"web-viewer/data/PreferencesConstants",
		"web-viewer/data/PreferencesDefaults",
		"web-viewer/data/User"
	],
	function () {
		// this module should have all the commonly used data components as a dependency
	});

/**
 * @memberof Graphics
 */
define('web-viewer/graphics/GraphicsConstants',{
	ChannelViewMode: {
		COMPOSITE_VIEW: 0,
		AUTOMATIC_TILING: 1,
		HORIZONTAL_TILING: 2,
		VERTICAL_TILING: 3
	},
	ChannelDiffMode: {
		CURRENT: 0,
		PREVIOUS: 1,
		IN_CURRENT_AND_MATCHED_WITH_PREVIOUS: 2,
		IN_PREVIOUS_AND_MATCHED_WITH_CURRENT: 3
	},
	ChannelViewModeLayoutMode: {
		COMPOSITE_VIEW: "AUTOMATIC",
		AUTOMATIC_TILING: "AUTOMATIC",
		HORIZONTAL_TILING: "HORIZONTAL",
		VERTICAL_TILING: "VERTICAL"
	},
	SingleChannelMode: {
		IN_CHANNEL_COLOR: 0,
		IN_BLACK: 1,
		IN_BLACK_WITH_SHOW_ALL_EFFECT: 2
	},
	ResetToDefaultMode: {
		RESET_ORDER: 0,
		RESET_COLORS: 1,
		RESET_BOTH: 2
	},
	ChannelMatching: {
		IDENTICAL_MATCHING: 0,
		NAME_MATCHING: 1,
		SMART_NAME_MATCHING: 2
	}

});

define('web-viewer/layout/component/ColorPicker',["web-viewer/utils/ViewerUtils",
		"component/tooltip/Tooltip",
		"underscore",
		"jquery",
		"colorpicker"
	],
	function (ViewerUtils,
		Tooltip,
		_,
		$) {
		"use strict";

		/**
		 * @exports ColorPicker
		 * @constructor
		 * @param {Function} onColorPicked - callback called when a color is picked, parameter: {String} (hex)color without #
		 * @param {HTMLObject} [element] - the element where the colorpicker needs to be added to
		 */
		var ColorPicker = function (onColorPicked, element) {

			// Callback on color picked
			this._onColorPicked = onColorPicked;

			// Current color (without #)
			this._color = undefined;

			// The color block
			this._colorBlock = undefined;
			if (element === undefined) {
				this._colorBlock = $("<div />").addClass("colorblock");
			} else {
				this._colorBlock = element;
			}

			// Public element
			this.element = this._colorBlock;


			this._lazyOnChange = _.debounce(_.bind(this._onChange, this), 50);
			// Init colorpicker
			this.element.ColorPicker({
				'onChange': this._lazyOnChange,
				'onBeforeShow': _.bind(function () {
					$(this).ColorPickerSetColor(this._color);
				}, this),
				'onSubmit': function (hsb, hex, rgb, el) {
					$(el).ColorPickerHide();
				}
			});
		};

		/**
		 * On Change of the color picker
		 * @param {Object} hsb - HSB values
		 * @param {String}   hex - Hex value
		 * @param {Object} rgb - RGB values
		 */
		ColorPicker.prototype._onChange = function (hsb, hex, rgb) {
			this._color = hex.toUpperCase();
			this._updateColor();
			this._onColorPicked(this._color);
		};

		/**
		 * Update the color of the color block if it was added by this class
		 */
		ColorPicker.prototype._updateColor = function () {
			if (this._colorBlock.hasClass("colorblock")) {
				this._colorBlock.css('background-color', "#" + this._color);
			}
		};

		/**
		 * Set the color of the colorpicker.
		 * @param {String} hexColor New color in hex format (with or without #)
		 */
		ColorPicker.prototype.setColor = function (hexColor) {
			if (ViewerUtils.startsWith(hexColor, "#")) {
				hexColor = hexColor.substring(1);
			}
			if (this._color != hexColor) {
				this._color = hexColor;
				this._updateColor();
			}
		};

		/**
		 * Get the color of the colorpicker
		 * @return {String} current color in hex format (without #)
		 */
		ColorPicker.prototype.getColor = function () {
			return this._color;
		};

		/**
		 * Show the color picker
		 */
		ColorPicker.prototype.show = function () {
			this.element.ColorPickerShow();
		};

		/**
		 * Hide the color picker
		 */
		ColorPicker.prototype.hide = function () {
			this.element.ColorPickerHide();
		};

		/**
		 * Destroy the color picker
		 */
		ColorPicker.prototype.destroy = function () {
			this.element.ColorPickerHide();
			$("#" + this.element.data('colorpickerId')).detach();
		};

		/**
		 * Set Tooltip for the color picker element.
		 * @param {String}         tooltipOrBundle - tooltip to show if langKey is undefined or language bundle
		 * @param {String}         [langKey]       - not required
		 * @param {Array.<String>} [stringArgs]    - not required, parameters that need to be filled in the language string
		 */
		ColorPicker.prototype.setTooltip = function (f, tooltipOrBundle, langKey, stringArgs) {
			if (this.tooltip === undefined) {
				this.tooltip = new Tooltip(f.i18n, this._colorBlock, f.getContext(), f.getContext(), f.isMobile(), tooltipOrBundle, langKey, stringArgs);
			} else {
				this.tooltip.setTooltipContent(tooltipOrBundle, langKey, stringArgs);
			}
		};

		return ColorPicker;
	});

define('web-viewer/layout/component/DropdownMiniButton',["web-viewer/layout/component/MiniButton",
		"component/dropdownmenu/DropdownMenu",
		"jquery",
		"underscore"
	],
	function (
		MiniButton,
		DropdownMenu,
		$,
		_
	) {
		"use strict";

		/** 
		 *
		 * A DropdownMiniButton is a Minibutton that when clicked shows a dropdownmenu.
		 *
		 * @exports Layout.Component.DropdownMiniButton
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {String} name - the name of the button
		 * @param {String} icon - the icon of the button (css class for 16px * 16px icon)
		 * @param {Array.<String>} [restrictedPositions] - array with all the positions that are allowed
		 * @param {Function} [leftClickAction] - Action executed when left clicking. The dropdown will be shown on right click. If undefined the dropdown will be shown when left clicking.
		 * Parameters are a boolean indicating whether the button is activated (true) or deactivated (false). The button itself is NOT ACTIVATED!
		 */
		var DropdownMiniButton = function (f, name, icon, restrictedDropdownPositions, leftClickAction) {
			MiniButton.call(this, f, name, icon, false, this.toggle);
			this.element.addClass("dropdown-mini-button");

			this.dropdown = new DropdownMenu(this.f.i18n, this.f.getContext(), this.f.isMobile(), "bottom", "wcr-dropdown", undefined, restrictedDropdownPositions);
			this._dropdownEnabled = true;

			this.leftClickAction = leftClickAction;
			if (this.leftClickAction === undefined) {
				// only left
				this.leftClickAction = _.bind(function () {
					if (this.isEnabled() && this._dropdownEnabled) {
						if (!this.dropdown.isActive()) {
							this.showDropdown();
						} else {
							this.dropdown.hide();
						}
					}
				}, this);
			} else {
				// we have both right/left
				this.element.on("contextmenu", _.bind(function (e) {
					e.preventDefault();
					e.stopPropagation();

					this.hideTooltip();

					if (this.isEnabled() && this._dropdownEnabled) {
						this.showDropdown();
					}
				}, this));
			}
			this.setAction(this.leftClickAction);
		};


		DropdownMiniButton.prototype = Object.create(MiniButton.prototype);
		DropdownMiniButton.prototype.constructor = DropdownMiniButton;

		DropdownMiniButton.prototype.isOpen = function () {
			return this.dropdown.isActive();
		};

		/**
		 * @override
		 */
		DropdownMiniButton.prototype.show = function () {
			this.element.css("display", "inline-block");
		};

		/**
		 * Show or hide the button
		 * @param {Boolean} show - True to show the button, false to hide it
		 */
		DropdownMiniButton.prototype.toggleButton = function (show) {
			if (show) {
				this.show();
			} else {
				this.dropdown.hide();
				this.hide();
			}
		};

		/**
		 * Set the text of a specific button by name
		 * @param {String} name - Name of the button
		 * @param {String} text - Text of the button
		 */
		DropdownMiniButton.prototype.setActionText = function (name, text) {
			this.dropdown.setActionText(name, text);
		};

		DropdownMiniButton.prototype.toggleOption = function (name, toggle) {
			this.dropdown.toggleOption(name, toggle);
		};

		DropdownMiniButton.prototype.getOptionValue = function (name) {
			return this.dropdown.getOptionValue(name);
		};

		DropdownMiniButton.prototype.setActionEnabled = function (name, enabled) {
			this.dropdown.setActionEnabled(name, enabled);
		};

		DropdownMiniButton.prototype.isActionEnabled = function (name) {
			return this.dropdown.isActionEnabled(name);
		};
		/**
		 *
		 */
		DropdownMiniButton.prototype.setWarning = function () {
			this.badge.addClass("warning");
			this.setBadge("!");
		};

		DropdownMiniButton.prototype.removeWarning = function () {
			this.badge.removeClass("warning");
			this.setBadge("");
		};

		DropdownMiniButton.prototype.addAction = function (action) {
			this.dropdown.addAction(action);
		};

		DropdownMiniButton.prototype.addOption = function (option) {
			this.dropdown.addOption(option);
		};

		DropdownMiniButton.prototype.addDivider = function () {
			this.dropdown.addDivider();
		};

		DropdownMiniButton.prototype.addHeader = function (headerText) {
			this.dropdown.addHeader(headerText);
		};
		/**
		 * Clear out the dropdown list
		 */
		DropdownMiniButton.prototype.clear = function () {
			this.dropdown.clear();
		};

		/**
		 * Enable the dropdown
		 */
		DropdownMiniButton.prototype.enableDropdown = function () {
			this._dropdownEnabled = true;
		};

		/**
		 * Disable the dropdown
		 */
		DropdownMiniButton.prototype.disableDropdown = function () {
			this.dropdown.hide();
			this._dropdownEnabled = false;
		};

		/**
		 * Show the dropdownmenu
		 */
		DropdownMiniButton.prototype.showDropdown = function () {
			this.dropdown.show(this.element);
		};


		/**
		 * Show/hide the element with the given name
		 * @param {String}   elementName - Name of the element to identify it. If the element is not found, this action does nothing.
		 * @param {Boolean} show - Show or hide the action
		 */
		DropdownMiniButton.prototype.toggleElementVisibility = function (elementName, show) {
			if (show) {
				this.dropdown.showElement(elementName);
			} else {
				this.dropdown.hideElement(elementName);
			}
		};

		return DropdownMiniButton;
	});

define('web-viewer/layout/component/SubtoolbarButton',["web-viewer/layout/component/ViewerToolbarButton",
		"jquery"
	],
	function (ViewerToolbarButton, $) {
		/**
		 * A Button is an interface element which provides an action and can have a text and icon description
		 * 
		 * @exports SubtoolbarButton
		 * @constructor
		 * @augments Layout.Button
		 * @param {String} Name - The name of the button
		 * @param {String} icon - The class which is used for displaying the icon
		 * @param {Boolean} [forMobile] - If true the button is useful on a mobile device
		 * @param {Function} [action] - The function which is called when the button is pressed. This parameter is optional as you can also use listenToTool.
		 */
		var SubtoolbarButton = function (f, name, icon, forMobile, action) {
			ViewerToolbarButton.call(this, f, name, icon, forMobile, action);

			// Overwrite the styling of the element
			this.element.removeClass("btn").addClass("subtoolbar-btn");
		};

		SubtoolbarButton.prototype = Object.create(ViewerToolbarButton.prototype);
		SubtoolbarButton.prototype.constructor = SubtoolbarButton;

		return SubtoolbarButton;
	});

define('web-viewer/layout/component/DropdownSubtoolbarButton',["web-viewer/layout/component/SubtoolbarButton",
		"component/dropdownmenu/DropdownMenu",
		"jquery",
		"underscore"
	],
	function (
		SubtoolbarButton,
		DropdownMenu,
		$,
		_) {
		"use strict";
		/** 
		 *
		 * @class A DropdownSubtoolbarButton is a subtoolbar button that when left clicked shows a dropdownmenu (toggled).
		 *
		 * @exports Layout.Component.DropdownSubtoolbarButton
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {String} name - the name of the button
		 * @param {String | JQueryObj } icon - The class which is used for displaying the icon, or your own jQuery element
		 * @param {Boolean} forMobile - is the button needed for mobile usage?
		 */
		var DropdownSubtoolbarButton = function (f, name, icon, forMobile) {
			SubtoolbarButton.call(this, f, name, icon, forMobile, this.toggle);
			this.dropdown = new DropdownMenu(this.f.i18n, this.f.getContext(), this.f.isMobile(), "bottom", "wcr-dropdown");
		};

		DropdownSubtoolbarButton.prototype = Object.create(SubtoolbarButton.prototype);
		DropdownSubtoolbarButton.prototype.constructor = DropdownSubtoolbarButton;

		/**
		 * Toggle the button
		 */
		DropdownSubtoolbarButton.prototype.toggle = function () {
			if (!this.isOpen()) {
				this.show();
			} else {
				this.hide();
			}
		};

		/**
		 * Check whether the dropdown is open
		 * @returns {Boolean} True if it is open, false otherwise
		 */
		DropdownSubtoolbarButton.prototype.isOpen = function () {
			return this.dropdown.isActive();
		};

		/**
		 * Show the dropdownmenu
		 */
		DropdownSubtoolbarButton.prototype.show = function () {
			this.dropdown.show(this.element);
		};

		/**
		 * Hide the dropdownmenu
		 */
		DropdownSubtoolbarButton.prototype.hide = function () {
			this.dropdown.hide();
		};

		/**
		 * Set the text of a specific button by name
		 * @param {String} name - Name of the button
		 * @param {String} text - Text of the button
		 */
		DropdownSubtoolbarButton.prototype.setActionText = function (name, text) {
			this.dropdown.setActionText(name, text);
		};

		/**
		 * Toggle an option
		 * @param {String} name - Name of the option
		 * @param {Boolean} toggle - True if should be 'checked', false otherwise
		 */
		DropdownSubtoolbarButton.prototype.toggleOption = function (name, toggle) {
			this.dropdown.toggleOption(name, toggle);
		};

		/**
		 * Get the value of an option
		 * @param {String} name - Name of the option
		 * @returns {Boolean} Value of the option
		 */
		DropdownSubtoolbarButton.prototype.getOptionValue = function (name) {
			return this.dropdown.getOptionValue(name);
		};

		/**
		 * Set whether the action is enabled or not
		 * @param {String} name - Name of the action
		 * @param {Boolean} enabled - True if it should be enabled, false otherwise
		 */
		DropdownSubtoolbarButton.prototype.setActionEnabled = function (name, enabled) {
			this.dropdown.setActionEnabled(name, enabled);
		};

		/**
		 * Check whether the action is enabled or not
		 * @param {String} name - Name of hte action
		 * @returns {Boolean} True if it is enabled, false otherwise
		 */
		DropdownSubtoolbarButton.prototype.isActionEnabled = function (name) {
			return this.dropdown.isActionEnabled(name);
		};

		/**
		 * Add an action
		 * @param {Object} action - Description of the action
		 * @example
		 * {
		 * 	name: 'nameOfButton',
		 * 	content: Text|JQueryObj,
		 * 	icon: undefined|''|'nameOfIcon',
		 * 	action: function{},
		 *  enabled: Boolean,
		 * 	[optional: doNotHideOnClick]
		 * }
		 */
		DropdownSubtoolbarButton.prototype.addAction = function (action) {
			this.dropdown.addAction(action);
		};

		/**
		 * Add an option
		 * @param {Object} option - Description of the option
		 * @example
		 * {
		 * 	name,
		 * 	content,
		 *  enabled,
		 * 	selected,
		 * 	onchange,
		 * 	[doNotHideOnClick]
		 *}
		 */
		DropdownSubtoolbarButton.prototype.addOption = function (option) {
			this.dropdown.addOption(option);
		};

		/**
		 * Add a divider line
		 */
		DropdownSubtoolbarButton.prototype.addDivider = function () {
			this.dropdown.addDivider();
		};

		return DropdownSubtoolbarButton;
	});

define('web-viewer/layout/component/DropdownToolbarButton',["web-viewer/layout/component/ViewerToolbarButton",
		"component/dropdownmenu/DropdownMenu",
		"jquery",
		"underscore"
	],
	function (
		ToolbarButton,
		DropdownMenu,
		$,
		_) {
		"use strict";

		/** 
		 *
		 * @class A DropdownToolbarButton is a ToolbarButton that when clicked shows a dropdownmenu.
		 *
		 * @exports Layout.Component.DropdownToolbarButton
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {String} name - the name of the button
		 * @param {String | JQueryObj } icon - The class which is used for displaying the icon (css class for 16px * 16px icon), or your own jQuery element
		 * @param {Boolean} forMobile - is the button needed for mobile usage?
		 * @param {Function} [leftClickAction] - Action executed when left clicking. If undefined the dropdown will be shown when left clicking.
		 * Parameters are a boolean indicating whether the button is activated (true) or deactivated (false). The button itself is NOT ACTIVATED!
		 */
		var DropdownToolbarButton = function (f, name, icon, forMobile, leftClickAction) {
			ToolbarButton.call(this, f, name, icon, forMobile, this.toggle);

			this.leftClickAction = leftClickAction;

			if (this.leftClickAction === undefined) {
				this.setAction(_.bind(function () {
					if (this.isEnabled() && this._dropdownEnabled) {
						if (!this.dropdown.isActive()) {
							this.showDropdown();
						} else {
							this.dropdown.hide();
						}
					}
				}, this));
			} else {
				this.element.on("contextmenu", _.bind(function (e) {
					e.preventDefault();
					e.stopPropagation();
					if (this.isEnabled() && this._dropdownEnabled) {
						this.showDropdown();
					}
				}, this));
			}

			this._dropdownEnabled = true;

			this.dropdown = new DropdownMenu(this.f.i18n, this.f.getContext(), this.f.isMobile(), "bottom", "wcr-dropdown");
		};


		DropdownToolbarButton.prototype = Object.create(ToolbarButton.prototype);
		DropdownToolbarButton.prototype.constructor = DropdownToolbarButton;

		/**
		 * Toggle the button
		 */
		DropdownToolbarButton.prototype.toggle = function () {
			if (this.isActive()) {
				this.leftClickAction(false);
			} else {
				this.leftClickAction(true);
			}
		};

		/**
		 * Check whether the dropdown is open
		 * @returns {Boolean} True if it is open, false otherwise
		 */
		DropdownToolbarButton.prototype.isOpen = function () {
			return this.dropdown.isActive();
		};

		/**
		 * Show the dropdownmenu
		 */
		DropdownToolbarButton.prototype.showDropdown = function () {
			this.dropdown.show(this.element);
		};

		/**
		 * Set the text of a specific button by name
		 * @param {String} name - Name of the button
		 * @param {String} text - Text of the button
		 */
		DropdownToolbarButton.prototype.setActionText = function (name, text) {
			this.dropdown.setActionText(name, text);
		};

		/**
		 * Toggle an option
		 * @param {String} name - Name of the option
		 * @param {Boolean} toggle - True if should be 'checked', false otherwise
		 */
		DropdownToolbarButton.prototype.toggleOption = function (name, toggle) {
			this.dropdown.toggleOption(name, toggle);
		};

		/**
		 * Get the value of an option
		 * @param {String} name - Name of the option
		 * @returns {Boolean} Value of the option
		 */
		DropdownToolbarButton.prototype.getOptionValue = function (name) {
			return this.dropdown.getOptionValue(name);
		};

		/**
		 * Set whether the action is enabled or not
		 * @param {String} name - Name of hte action
		 * @param {Boolean} enabled - True if it should be enabled, false otherwise
		 */
		DropdownToolbarButton.prototype.setActionEnabled = function (name, enabled) {
			this.dropdown.setActionEnabled(name, enabled);
		};

		/**
		 * Check whether the action is enabled or not
		 * @param {String} name - Name of hte action
		 * @returns {Boolean} True if it is enabled, false otherwise
		 */
		DropdownToolbarButton.prototype.isActionEnabled = function (name) {
			return this.dropdown.isActionEnabled(name);
		};

		/**
		 * Add an action
		 * @param {Object} action - Description of the action
		 */
		DropdownToolbarButton.prototype.addAction = function (action) {
			this.dropdown.addAction(action);
		};

		/**
		 * Add an option
		 * @param {Object} option - Description of the option
		 */
		DropdownToolbarButton.prototype.addOption = function (option) {
			this.dropdown.addOption(option);
		};

		/**
		 * Add a divider line
		 */
		DropdownToolbarButton.prototype.addDivider = function () {
			this.dropdown.addDivider();
		};

		/**
		 * Enable the dropdown
		 */
		DropdownToolbarButton.prototype.enableDropdown = function () {
			this._dropdownEnabled = true;
		};

		/**
		 * Disable the dropdown
		 */
		DropdownToolbarButton.prototype.disableDropdown = function () {
			this._dropdownEnabled = false;
		};

		/**
		 * Clear the contents of the dropdown menu
		 */
		DropdownToolbarButton.prototype.clear = function () {
			this.dropdown.clear();
		};

		return DropdownToolbarButton;
	});

define('web-viewer/layout/component/SubToolbarButton',["web-viewer/layout/component/ViewerToolbarButton",
		"jquery"
	],
	function (ViewerToolbarButton, $) {
		/**
		 * A Button is an interface element which provides an action and can have a text and icon description
		 * 
		 * @exports SubtoolbarButton
		 * @constructor
		 * @augments Layout.Button
		 * @param {String} Name - The name of the button
		 * @param {String} icon - The class which is used for displaying the icon
		 * @param {Boolean} [forMobile] - If true the button is useful on a mobile device
		 * @param {Function} [action] - The function which is called when the button is pressed. This parameter is optional as you can also use listenToTool.
		 */
		var SubtoolbarButton = function (f, name, icon, forMobile, action) {
			ViewerToolbarButton.call(this, f, name, icon, forMobile, action);

			// Overwrite the styling of the element
			this.element.removeClass("btn").addClass("subtoolbar-btn");
		};

		SubtoolbarButton.prototype = Object.create(ViewerToolbarButton.prototype);
		SubtoolbarButton.prototype.constructor = SubtoolbarButton;

		return SubtoolbarButton;
	});

define('web-viewer/layout/LayoutModule',["web-viewer/layout/AbstractWidget",
		"web-viewer/layout/LayoutManager",
		"Template",
		"tpl",
		"component/button/Button",
		"component/toolbar/ButtonGroup",
		"web-viewer/layout/component/ColorPicker",
		"component/dropdownmenu/DropdownMenu",
		"web-viewer/layout/component/DropdownMiniButton",
		"web-viewer/layout/component/DropdownSubtoolbarButton",
		"web-viewer/layout/component/DropdownToolbarButton",
		"web-viewer/layout/component/SubtoolbarButton",
		"web-viewer/layout/component/ViewerToolbarButton",
		"web-viewer/layout/component/MiniButton",
		"web-viewer/layout/component/ModalPopup",
		"component/popover/Popover",
		"web-viewer/layout/component/Popup",
		"web-viewer/layout/component/SubToolbarButton",
		"web-viewer/layout/component/ViewerToolbarButton",
		"component/toolbar/ToolbarComponent",
		"component/tooltip/Tooltip"
	],
	function () {
		// this module should have all the commonly used layout components as a dependency
	});

/**
 * @memberof MeasureModule
 */
define('web-viewer/measure/MeasureConstants',{
	MeasureType: {
		Ruler: 0,
		EyeDropper: 1,
		Barcode: 2,
		Braille: 3,
		Caliper: 4,
		ScreenRuling: 5
	},
	BrailleLanguage: {
		Automatic: 0, // Note: this is not a supported viewserver language, but gets resolved later
		English: 1,
		French: 2,
		Polish: 3,
		Portuguese: 4,
		Russian: 5,
		Spanish: 6,
		Finnish: 7,
		German: 8,
		Italian: 9,
		Latvian: 10,
		Romanian: 11,
		Slovene: 12,
		Swedish: 13,
		Hungarian: 14,
		Dutch: 15,
		Maltese: 16
	},
	LanguageTranslationPrefix: "Lanuguage",
	LanguageAutomatic: "Automatic",
	LanguageDefault: "English",
	/**
	 * Existing mappings from WebCenter languages to Braille languages.
	 * Note: some languages in WebCenter don't have a Braille language and vice versa
	 */
	LanguageLocales: {
		en: "English",
		fr: "French",
		pl: "Polish",
		br: "Portuguese",
		ru: "Russian",
		es: "Spanish",
		de: "German",
		it: "Italian",
		hu: "Hungarian",
		nl: "Dutch"
	},
	CaliperStyle: {
		Freeform: 0,
		Fixed: 1,
		Preset: 2
	},
	RulerPoint: {
		DEFAULT: "/wcr-static/web-viewer/layout/img/crosshair_fine.png",
		OFFSET: {
			width: 7,
			height: 7
		},
		CLOSETODISTANCE: 15
	},
	MeasureDrawType: {
		RECTANGLE: 0,
		LINE: 1,
		WIDTH: 2,
		HEIGHT: 3
	},
	ScreenRulingMode: {
		LINES: 0,
		ANGLE: 1,
		BOTH: 2
	},
	ScreenRulingOrientation: {
		XMP: 0, // used for bitmap viewer. Probably not used in WebCenter according to KSJ
		CW: 1,
		CCW: 2
	},
	/**
	 * Size of selection when clicking for eyedropper/screen ruling tool/etc..
	 */
	ClickSize: {
		NORMAL: 1,
		DIGITAL_FILM: 64
	},

	ServerInfoFlags: {
		barCode: 0x200,
		barCodeGV: 0x2000,
		braille: 0x400
	}
});

define('web-viewer/navigation/PanTool',["underscore",
		"web-viewer/viewport/tools/AbstractTool",
		"utils/ShapeUtils",
		"web-viewer/utils/GEOUtils",
		"web-viewer/core/CoreConstants"
	],
	function (_,
		AbstractTool,
		ShapeUtils,
		GEOUtils,
		CoreConstants) {
		"use strict";

		/**
		 * @exports PanTool
		 * @memberof NavigationModule
		 * @augments Viewport.AbstractTool
		 *
		 * @class
		 * The pan tool enables the user to pan in the document. Scrolling and double click/right click can be used to zoom
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 */
		var PanTool = function Pan(f) {
			AbstractTool.call(this, f, CoreConstants.TOOLS.PAN);

			/**
			 * Start point, set when dragging is started
			 */
			var startPoint = new ShapeUtils.Point(0, 0);
			/**
			 * the offset needed for dragging
			 */
			var startOffset = new ShapeUtils.Size();

			var deactivateAfterPanAction = false;

			/**
			 * Update the world
			 *
			 * @param {ShapeUtils.Point} to - to where has the user dragged?
			 * @param {Boolean} isEndOfInteraction
			 */
			var updateWorld = _.bind(function (to, isEndOfInteraction) {
				var draggedSize = startPoint.sizeTo(to);

				if (Math.abs(draggedSize.width) > 0 || Math.abs(draggedSize.height) > 0 || isEndOfInteraction) {
					var worldOffset = this.sceneToWorld(startPoint).sizeTo(this.sceneToWorld(to));
					worldOffset.add(startOffset);

					var animation = undefined;
					if (isEndOfInteraction) {
						animation = {
							duration: 200,
							type: "ease-out"
						};
						if (deactivateAfterPanAction) {
							f.emit("toolManager.deactivateTool", this);
							deactivateAfterPanAction = false;
						}
					}
					this._widget.getWorld().setOffsetSize(worldOffset, animation);
				}
			}, this);


			/**
			 * DEFAULT STATE
			 */
			this.registerState("default", {
				/*
				 * on mouse down, start dragging
				 */
				mousedown: _.bind(function (e) {
					if (e.which === 1) {
						startPoint = this.getScenePoint(e);
						startOffset = this._widget.getWorld().getOffsetSize();
						this.changeState("panning");

						e.preventDefault();
					}
				}, this),

				mouseup: function () {},

				mousemove: function () {},

				mouseout: function () {},

				mouseleave: function () {},

				/**
				 * on double click zoom in
				 */
				dblclick: _.bind(function (e) {
					this.changeState("default");
					if (this._widget.getWorld().isSmallerThanFullView()) {
						this._widget.getWorld().zoomToFit(true);
					} else {
						// zoom in
						this.zoom(1, this.getWorldPoint(e), true);
					}
				}, this),

				/**
				 * on right click zoom out
				 */
				contextmenu: _.bind(function (e) {
					e.preventDefault();
					e.stopPropagation();

					//zoom out
					this.zoom(-1, this.getWorldPoint(e), true);
				}, this)
			});


			/**
			 * a reference to timer for the re-enabling of text selection
			 */
			var textSelectionTimeOut = undefined;
			/**
			 * last mouse position is used to handle mouseleave and mouseout correctly
			 */
			var lastMousePosition = new ShapeUtils.Point(0, 0);

			this.registerState("panning", _.defaults({
				/**
				 * when starting to drag, notify viewport on interactive behavior and disable text selection
				 */
				enterState: function () {
					if (textSelectionTimeOut) {
						clearTimeout(textSelectionTimeOut);
						textSelectionTimeOut = null;
					}
					f.emit("viewport.startInteractive");
					f.emit("viewport.startPanning");
					f.emit("layout.disableTextSelection");
				},

				/**
				 * When dragging has stopped, notify viewport and enable text selection again
				 */
				leaveState: function () {
					startPoint = new ShapeUtils.Point(0, 0);
					f.emit("viewport.stopInteractive");
					f.emit("viewport.stopPanning");
					textSelectionTimeOut = setTimeout(function () {
						f.emit("layout.enableTextSelection");
					}, 500);
				},

				/**
				 * On mouse up, panning has stopped , update the world a last time, and go back to the default state
				 */
				mouseup: _.bind(function (e) {
					e.preventDefault();
					updateWorld(this.getScenePoint(e), true);
					this.changeState("default");
				}, this),

				/**
				 * Mouse is moving, update panning/world
				 */
				mousemove: _.bind(function (e) {
					lastMousePosition = this.getScenePoint(e);
					updateWorld(lastMousePosition, false);
				}, this),

				/**
				 * panning has stopped, update world, go back to default state
				 */
				mouseout: _.bind(function (e) {
					e.preventDefault();
					updateWorld(lastMousePosition, true);
					this.changeState("default");
				}, this),

				/**
				 * panning has stopped, update world, go back to default state
				 */
				mouseleave: _.bind(function (e) {
					e.preventDefault();
					updateWorld(lastMousePosition, true);
					this.changeState("default");
				}, this)

			}, this.getState("default")));


			// define some key events
			this.passiveKeyEvents = [];
			this.activeKeyEvents = [];

			/**
			 * Initialize the tool
			 */
			this._override("initialize", function (widget, toolManager) {
				this._super(widget, toolManager);

				// register the needed events
				this.registerActiveEvents(["mousedown", "mousemove", "mouseup", "mouseout", "mouseleave", "contextmenu", "dblclick"]);

				/**
				 * Passive events
				 */
				var requestedInterrupt = false;
				// on space down request interruption
				this.passiveKeyEvents.push(["space", _.bind(function (e) {
					if (!requestedInterrupt) {
						requestedInterrupt = true;
						f.emit("toolManager.request.activation", this);
					}
				}, this), "keydown"]);

				// on space up request interruption
				this.passiveKeyEvents.push(["space", function (e) {
					requestedInterrupt = false;
				}, "keyup"]);



				/**
				 * Active events
				 */
				// on space up, and in interruption mode, request deactivation
				this.activeKeyEvents.push(["space", _.bind(function (e) {
					if (requestedInterrupt) {
						requestedInterrupt = false;
						f.emit("toolManager.request.deactivation", this);
					}
				}, this), "keyup"]);

				// always activate the tool when pressing H
				this.passiveKeyEvents.push(["h", _.bind(function (e) {
					e.preventDefault();
					f.emit("toolManager.activateTool", this);
				}, this), "keypress"]);

				this.passiveKeyEvents.push(["mod+up", _.bind(function (e) {
					e.preventDefault();
					e.stopPropagation();
					this._setPanStart();
					var sceneSize = this._widget.getWorld().getSceneSize();
					var toPoint = new ShapeUtils.Point(startPoint.x, startPoint.y + sceneSize.height / 4);
					updateWorld(toPoint, true);
				}, this), "keypress"]);

				this.passiveKeyEvents.push(["mod+down", _.bind(function (e) {
					e.preventDefault();
					e.stopPropagation();
					this._setPanStart();
					var sceneSize = this._widget.getWorld().getSceneSize();
					var toPoint = new ShapeUtils.Point(startPoint.x, startPoint.y - sceneSize.height / 4);
					updateWorld(toPoint, true);
				}, this), "keypress"]);

				this.passiveKeyEvents.push(["mod+right", _.bind(function (e) {
					e.preventDefault();
					e.stopPropagation();
					this._setPanStart();
					var sceneSize = this._widget.getWorld().getSceneSize();
					var toPoint = new ShapeUtils.Point(startPoint.x - sceneSize.width / 4, startPoint.y);
					updateWorld(toPoint, true);
				}, this), "keypress"]);

				this.passiveKeyEvents.push(["mod+left", _.bind(function (e) {
					e.preventDefault();
					e.stopPropagation();
					this._setPanStart();
					var sceneSize = this._widget.getWorld().getSceneSize();
					var toPoint = new ShapeUtils.Point(startPoint.x + sceneSize.width / 4, startPoint.y);
					updateWorld(toPoint, true);
				}, this), "keypress"]);

				// activate pan tool until one pan action is done
				this.passiveKeyEvents.push(["f7", _.bind(function (e) {
					e.preventDefault();
					e.stopPropagation();
					deactivateAfterPanAction = true;
					f.emit("toolManager.activateTool", this, true);
				}, this)]);

			});

			this._setPanStart = function () {
				var sceneSize = this._widget.getWorld().getSceneSize();
				startPoint = new ShapeUtils.Point(sceneSize.width / 2, sceneSize.height / 2);
				startOffset = this._widget.getWorld().getOffsetSize();
			};


			// always show the hand cursor when active
			this.getDefaultCursor = function () {
				return "cursor-move";
			};

			/**
			 * The tool can be interrupted when not in "panning" state
			 */
			this.canBeInterrupted = function () {
				return this.getCurrentStateName() != "panning";
			};

			this._override("doActivation", function () {
				this._super();
				this.changeState("default");
			});

			this._override("deactivate", function (interruptRequest) {
				this._super(interruptRequest);
				this.changeState("default");
			});

			/**
			 * Zoom in/out
			 *
			 * @param step {int} if positive zoom in, if negative zoom out
			 * @param worldMiddle {Point} what is the center of the zoom action
			 * @param animated {boolean} show animation
			 */
			this.zoom = function (step, worldMiddle, animated) {
				this._widget.getWorld().transform({
					zoom: step,
					middle: worldMiddle,
					animated: animated
				});
			};

		};

		// inherit
		PanTool.prototype = Object.create(AbstractTool.prototype);
		// correct the constructor pointer 
		PanTool.prototype.constructor = PanTool;

		return PanTool;
	});

define('web-viewer/utils/TouchUtil',["underscore",
		"WV3Dlib/hammer", /* WV3Dlib is used to avoid a conflict with the 3D collada viewer... this is the easiest solution to load Hammer without breaking the 3D viewer*/
		"utils/ShapeUtils"
	],
	function (_,
		Hammer,
		ShapeUtils) {
		"use strict";

		/**
		 * Handle touch events with the use of HammerJS
		 * @memberof Utils
		 * @exports TouchUtil
		 * @class TouchUtil
		 * @param {HTMLElement} ctx - context in which the touch events are registered
		 */
		var TouchUtil = function (ctx) {
			this.ctx = ctx;
			// Initialize HammeJS and enable pinch, double tap and pan
			var touchMgr = new Hammer.Manager(ctx);
			var pinch = new Hammer.Pinch();
			var singleTap = new Hammer.Tap({
				event: "singletap"
			});
			var doubleTap = new Hammer.Tap({
				event: "doubletap",
				taps: 2
			});
			var pan = new Hammer.Pan();
			touchMgr.add([pinch, singleTap, doubleTap, pan]);
			pinch.recognizeWith(pan);
			pinch.requireFailure(pan);
			doubleTap.recognizeWith(singleTap);

			// Keep track of all listeners
			this.listeners = {};

			// register events HammerJS should listen to
			touchMgr.on("singletap doubletap pinchstart pinchmove pinchend panstart panmove panend pancancel", _.bind(this.handleTouchEvent, this));
		};

		TouchUtil.prototype.handleTouchEvent = function (event) {
			var type = event.type;
			if (this.listeners[type] && this.listeners[type].length > 0) {
				_.each(this.listeners[type], function (callback) {
					callback(event);
				});
			}
		};

		// translate adding of listeners to the HammerJS library events
		TouchUtil.prototype.addSingleTapListener = function (callback) {
			this._addListener("singletap", callback);
		};

		TouchUtil.prototype.addDoubleTapListener = function (callback) {
			this._addListener("doubletap", callback);
		};

		TouchUtil.prototype.addPinchStartListener = function (callback) {
			this._addListener("pinchstart", callback);
		};

		TouchUtil.prototype.addPinchMoveListener = function (callback) {
			this._addListener("pinchmove", callback);
		};

		TouchUtil.prototype.addPinchEndListener = function (callback) {
			this._addListener("pinchend", callback);
			this._addListener("pinchcancel", callback);
		};

		TouchUtil.prototype.addPanStartListener = function (callback) {
			this._addListener("panstart", callback);
		};

		TouchUtil.prototype.addPanMoveListener = function (callback) {
			this._addListener("panmove", callback);
		};

		TouchUtil.prototype.addPanEndListener = function (callback) {
			this._addListener("panend", callback);
			this._addListener("pancancel", callback);
		};

		// translate removing of listeners to the HammerJS library events
		TouchUtil.prototype.removeDoubleTapListener = function (callback) {
			this._removeListener("doubletap", callback);
		};

		TouchUtil.prototype.removePinchStartListener = function (callback) {
			this._removeListener("pinchstart", callback);
		};

		TouchUtil.prototype.removePinchMoveListener = function (callback) {
			this._removeListener("pinchmove", callback);
		};

		TouchUtil.prototype.removePinchEndListener = function (callback) {
			this._removeListener("pinchcancel", callback);
			this._removeListener("pinchend", callback);
		};

		TouchUtil.prototype.removePanStartListener = function (callback) {
			this._removeListener("panstart", callback);
		};

		TouchUtil.prototype.removePanMoveListener = function (callback) {
			this._removeListener("panmove", callback);
		};

		TouchUtil.prototype.removePanEndListener = function (callback) {
			this._removeListener("panend", callback);
			this._removeListener("pancancel", callback);
		};

		/**
		 * Remove all listeners
		 */
		TouchUtil.prototype.removeAllListeners = function () {
			this.listeners = {};
		};

		/**
		 * Add listener of a specific event type
		 * @param {String} Type - type of event, e.g. pinchstart, pinchmove
		 * @param {Function} callback - method that will be called when event is triggered
		 */
		TouchUtil.prototype._addListener = function (type, callback) {
			if (!this.listeners[type]) {
				this.listeners[type] = [];
			}
			if (_.indexOf(this.listeners[type], callback) < 0) {
				this.listeners[type].push(callback);
			}
		};

		/**
		 * Remove specific listener for a type of event
		 * @param {String} Type - type of event, e.g. pinchstart, pinchmove
		 * @param {Function} callback - method that will be removed from listener
		 */
		TouchUtil.prototype._removeListener = function (type, callback) {
			if (this.listeners[type]) {
				this.listeners[type] = _.without(this.listeners[type], callback);
			}
		};

		/**
		 * Data Getters and Event handler actions
		 * We place these here, because we don't want the TouchTool to know anything about the object obtained by HammerJS
		 */
		TouchUtil.prototype.preventDefault = function (e) {
			e.preventDefault();
		};
		TouchUtil.prototype.getScale = function (e) {
			return e.scale;
		};
		TouchUtil.prototype.getRotation = function (e) {
			return e.rotation;
		};

		/**
		 * Get the Center of a Touch Event
		 *
		 * Note! the doesn't take in to account multiple views in the widget... It is safer to use getScenePointFromTouch
		 *
		 * @param {TouchEvent} event - the touch event
		 * @param {Object} offset - {top: topOffset, left: leftOffset{
		 * @returns {ShapeUtils.Point} center point
		 */
		TouchUtil.prototype.getCenter = function (e, offset) {
			var center = new ShapeUtils.Point();
			center.x = e.center.x;
			center.y = e.center.y;
			if (offset) {
				center.x -= offset.left;
				center.y -= offset.top;
			}
			return center;
		};

		/**
		 * Get the center scene point of a TouchEvent
		 * @param {TouchEvent} event - the touch event
		 * @param {ViewportWidget} widget - the ViewportWidget
		 * @param {ViewportView} view - the view in which the ScenePoint should be determined
		 * @returns {Object} {view: reference to the View, point: scenePoint}
		 */
		TouchUtil.prototype.getScenePointFromTouch = function (event, widget, view) {
			var tapPoint = this.getCenter(event);
			if (!view) {
				view = widget.requestView(tapPoint);
			}

			if (view) {
				var offset = view.getViewContext().offset();
				tapPoint.offsetXY(-offset.left, -offset.top);
			}


			return {
				view: view,
				point: tapPoint
			};
		};

		return TouchUtil;
	});

define('web-viewer/navigation/TouchTool',["underscore",
		"web-viewer/viewport/tools/AbstractTool",
		"web-viewer/utils/ViewerUtils",
		"web-viewer/utils/TouchUtil",
		"web-viewer/core/CoreConstants"
	],
	function (_,
		AbstractTool,
		ViewerUtils,
		TouchUtil,
		CoreConstants) {
		"use strict";

		/**
		 * @exports TouchTool
		 * @memberof NavigationModule
		 * @augments Viewport.AbstractTool
		 *
		 * @class
		 * The touch tool enables the user to zoom, pan and rotate on the document using touch
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 */
		var TouchTool = function Touch(f) {
			AbstractTool.call(this, f, CoreConstants.TOOLS.TOUCH);

			// create touch util within viewport context
			this._touchUtil = null;

			// Allows rotation or not (only rotate when threshold is passed)
			this._allowRotation = true;
			this._unlockRotation = false;
			this._unlockRotationAngle = 25;

			this._initZoom = undefined;
			this._initRotate = undefined;
			this._initTouchCenter = undefined;
		};

		// extend from the AbstractTool
		TouchTool.prototype = Object.create(AbstractTool.prototype);
		TouchTool.prototype.constructor = TouchTool;

		/**
		 * Action the will be called when an action is started
		 * @param {Event} e - launched event
		 */
		TouchTool.prototype.startAction = function (e) {
			this._touchUtil.preventDefault(e);

			this.f.emit("viewport.startInteractive");
			this._unlockRotation = false;

			var world = this._widget.getWorld();
			this._initZoom = world.getZoom();
			this._initRotate = world.getRotation();
			var touchPoint = this._touchUtil.getScenePointFromTouch(e, this._widget);
			this._initTouchCenter = this.sceneToWorld(touchPoint.point);
			this._currentView = touchPoint.view;

		};

		/**
		 * Action the will be called when an action is stopped
		 * @param {Event} e - launched event
		 */
		TouchTool.prototype.stopAction = function (e) {
			this._touchUtil.preventDefault(e);
			delete this._currentView;
			this.f.emit("viewport.stopInteractive");
			this.releaseCanvasLayer();

			this._unlockRotation = false;
			this._widget.getWorld().transform({
				rubber: false,
				animated: {
					duration: 100,
					type: "ease-out"
				},
				constrainRotation: true
			});
		};


		/**
		 * Handle double tap: zoom in to point if fit to screen, otherwise do a fit to screen
		 * @param {Event} e - launched event
		 */
		TouchTool.prototype.doubleTap = function (e) {
			this._touchUtil.preventDefault(e);

			var atpoint = this.getScenePointFromTouch(e);
			var worldatpoint = this._widget.getWorld().getWorldPointFromScene(atpoint);
			var world = this._widget.getWorld();
			if (!world.isFullView()) {
				world.zoomToFit(true);
			} else {
				world.transform({
					zoom: 3,
					middle: worldatpoint,
					animated: true
				});
			}
		};

		/**
		 * Handle pinch move action
		 * @param {Event} e - launched event
		 */
		TouchTool.prototype.pinchMove = function (e) {
			this._touchUtil.preventDefault(e);

			// Get the center point, zoom and rotation      		
			var zoom = this._touchUtil.getScale(e);
			var rotation = this._touchUtil.getRotation(e);
			var center = this.getScenePointFromTouch(e);

			this.updateWorld(center, zoom, rotation);
		};

		/**
		 * Handle pan move action
		 * @param {Event} e - launched event
		 */
		TouchTool.prototype.panAction = function (e) {
			this._touchUtil.preventDefault(e);

			var center = this.getScenePointFromTouch(e);
			this.updateWorld(center);
		};

		/**
		 * Update the world.
		 * @param {ShapeUtils.Point} center - center point of touch (touchpoint for pan, between fingers for pinch)
		 * @param {Number} zoom - New zoom level
		 * @param {Number} rotation - New rotation (in degrees)
		 */
		TouchTool.prototype.updateWorld = function (center, zoom, rotation) {
			// Mechanism: 
			// - first, do rotation and zoom, with center point as middle
			// - afterwards, do a translation, so the "world center point" (this._initTouchCenter) is between the fingers again

			var world = this._widget.getWorld();
			var options = {};

			// Set zoom
			if (zoom) {
				var worldZoom = this._initZoom * zoom;
				options.fixedzoom = worldZoom;
			}

			// Calculate new rotation (if allowed)
			if (rotation && this._allowRotation) {
				rotation = parseInt(rotation, 10) % 360;
				// If the rotation is not unlocked yet, check if it should be unlocked
				if (!this._unlockRotation) {
					// If the new rotation is bigger than the unlock angle, unlock the rotation 
					var newRotation = rotation - ViewerUtils.closestAngle(rotation, 4);
					if (Math.abs(newRotation) > this._unlockRotationAngle) {
						this._unlockRotation = true;
					}
				}

				if (this._unlockRotation) {
					// Only rotate if the rotation is unlocked
					options.fixedrotation = rotation + this._initRotate;
				}
			}

			// use center of touches as center point of transformation
			options.middle = this._initTouchCenter;

			// Transform, but without redrawing, we still need to shift the center point back
			world.transform(options, true);

			// shift back, as we want to center point to stay on the same page
			var offsetSizeInScene = world.getSceneBounds().center().sizeTo(center);
			var offsetSizeInWorld = world.getWorldSizeFromScene(offsetSizeInScene);
			options = {};
			options.offsetX = offsetSizeInWorld.width;
			options.offsetY = offsetSizeInWorld.height;
			world.transform(options);
		};

		/**
		 * Initialize the touch tool
		 * @param {AbstractWidget} widget - the widget on which the tool is activated
		 */
		TouchTool.prototype.initialize = function (widget, toolManager) {
			AbstractTool.prototype.initialize.apply(this, arguments);
			// note this only works on the viewportwidget
			this._touchUtil = new TouchUtil(widget.getElement()[0]);
		};

		/**
		 * override the deactivation process, so all listeners are removed
		 * @override
		 */
		TouchTool.prototype.doDeactivation = function () {
			AbstractTool.prototype.doDeactivation.call(this);

			// unregister events
			this._touchUtil.removeAllListeners();
		};

		/**
		 * override the activation process, so all listeners are added
		 * @override
		 */
		TouchTool.prototype.doActivation = function () {
			AbstractTool.prototype.doActivation.call(this);

			// register events
			this._touchUtil.addPinchStartListener(_.bind(function (e) {
				this.f.emit("viewport.startZooming");
				this.startAction(e);
			}, this));
			this._touchUtil.addPinchMoveListener(_.bind(this.pinchMove, this));
			this._touchUtil.addPinchEndListener(_.bind(function (e) {
				this.f.emit("viewport.stopZooming");
				this.stopAction(e);
			}, this));

			this._touchUtil.addDoubleTapListener(_.bind(this.doubleTap, this));

			this._touchUtil.addPanStartListener(_.bind(function (e) {
				this.f.emit("viewport.startPanning");
				this.startAction(e);
			}, this));
			this._touchUtil.addPanMoveListener(_.bind(this.panAction, this));
			this._touchUtil.addPanEndListener(_.bind(function (e) {
				this.f.emit("viewport.stopPanning");
				this.stopAction(e);
			}, this));
		};

		/**
		 * Get a scene point from a touch event
		 * if a view is set with the startAction, the scene point is calculated for that view
		 * @param {Event} event - the touch event
		 * @returns {ShapeUtils.Point} the ScenePoint
		 */
		TouchTool.prototype.getScenePointFromTouch = function (event) {
			return this._touchUtil.getScenePointFromTouch(event, this._widget, this._currentView).point;
		};

		return TouchTool;

	});

define('web-viewer/navigation/TouchPreventerTool',["underscore",
		"web-viewer/viewport/tools/AbstractTool",
		"web-viewer/utils/TouchUtil",
		"web-viewer/core/CoreConstants"
	],
	function (_,
		AbstractTool,
		TouchUtil,
		CoreConstants) {
		/**
		 * @exports TouchPreventTool
		 * @memberof NavigationModule 
		 * @augments Viewport.AbstractTool
		 * 
		 * @class 
		 * The touch preventer tool prevents users to use touch. This will disable browser zooming etc.
		 * @constructor
		 * @param {Core.ViewerFacade} f 
		 */
		var TouchPreventTool = function Touch(f) {
			AbstractTool.call(this, f, CoreConstants.TOOLS.NOTOUCH);
			// create touch util within viewport context
			this._touchUtil = null;
		};

		// extend from the AbstractTool
		TouchPreventTool.prototype = Object.create(AbstractTool.prototype);
		TouchPreventTool.prototype.constructor = TouchPreventTool;

		/**
		 * override the passivation process, so all listeners are removed
		 * @override
		 */
		TouchPreventTool.prototype.doPassivation = function () {
			AbstractTool.prototype.doPassivation.call(this);

			// unregister events
			this._touchUtil.removeAllListeners();
		};

		/**
		 * Initialize the touch preventer tool
		 * @param {AbstractWidget} widget - the widget on which the tool is activated
		 */
		TouchPreventTool.prototype.initialize = function (widget, toolManager) {
			AbstractTool.prototype.initialize.apply(this, arguments);
			// note this only works on the viewportwidget
			this._touchUtil = new TouchUtil(widget.getElement()[0]);
		};

		/**
		 * Override the activation process, so all listeners are added.
		 * Listen to everything to prevent the events from doing something.
		 * @override
		 */
		TouchPreventTool.prototype.doActivation = function () {
			AbstractTool.prototype.doActivation.call(this);

			var preventDefault = _.bind(function (e) {
				this._touchUtil.preventDefault(e);
			}, this);

			// register events
			this._touchUtil.addPinchStartListener(_.bind(preventDefault, this));
			this._touchUtil.addPinchMoveListener(_.bind(preventDefault, this));
			this._touchUtil.addPinchEndListener(_.bind(preventDefault, this));

			this._touchUtil.addDoubleTapListener(_.bind(preventDefault, this));

			this._touchUtil.addPanStartListener(_.bind(preventDefault, this));
			this._touchUtil.addPanMoveListener(_.bind(preventDefault, this));
			this._touchUtil.addPanEndListener(_.bind(preventDefault, this));
		};

		return TouchPreventTool;

	});

define('web-viewer/navigation/ZoomTool',["underscore",
		"web-viewer/viewport/tools/AbstractTool",
		"utils/ShapeUtils",
		"web-viewer/utils/GEOUtils",
		"utils/Mousewheel",
		"web-viewer/utils/DrawUtils",
		"web-viewer/core/CoreConstants"
	],
	function (_,
		AbstractTool,
		ShapeUtils,
		GEOUtils,
		Mousewheel,
		DrawUtils,
		CoreConstants) {
		"use strict";

		/**
		 * @exports ZoomTool
		 * @memberof NavigationModule
		 * @augments Viewport.AbstractTool
		 *
		 * @class
		 * The zoomtool enables the user to zoom in and out on the document
		 *
		 * Several possible interaction are possible:
		 * 	zoom in: ctrl + OR left mouse click OR start dragging rectangle
		 *  zoom out: ctrl - OR right mouseclick
		 *  zoom to fit: ctrl 0
		 *
		 *  pressing alt when the tool is active changes the behavior (left click zoom out, right click zoom in)
		 *
		 *  activate zoom tool: z
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 */
		var ZoomTool = function Zoom(f) {
			AbstractTool.call(this, f, CoreConstants.TOOLS.ZOOM);

			/* Options */
			var selectionThreshold = 16;
			var defaultZoom = 1;
			var selection;
			var mouseUpsToIgnore = 0;
			var previousMouseLocation = new ShapeUtils.Point();
			var deactiveToolAfterZoomAction = false;

			/**
			 * Action to be done after zoom
			 */
			var afterZoomAction = _.bind(function () {
				if (deactiveToolAfterZoomAction) {
					f.emit("toolManager.deactivateTool", this);
					deactiveToolAfterZoomAction = false;
				}
			}, this);

			/**
			 * This is the mode when the tool is active,
			 * it will zoom on click and go the couldStartDraggingMode if left mousedown is clicked
			 */
			this.registerState("default", _.bind(function () {
				/**
				 * helper function, to change the zoom mode if altkey is pressed
				 */
				var updateZoomMode = _.bind(function (e) {
					if (e.altKey) {
						defaultZoom = -1;
					} else {
						defaultZoom = 1;
					}

					this.setCursor((defaultZoom == -1) ? "cursor-zoomOut" : "cursor-zoomIn");
				}, this);

				return {
					"mousedown": _.bind(function (e) {
						selection = new ShapeUtils.Rect(this.getScenePoint(e));


						updateZoomMode(e);

						// if left click, go to couldStartDragging mode
						if (e.which === 1) {
							this.changeState("couldStartDragging");
						}

						e.preventDefault();
					}, this),
					"mouseup": _.bind(function (e) {
						// mouseup when not dragging --> zoom in or out
						updateZoomMode(e);

						if (mouseUpsToIgnore > 0) {
							mouseUpsToIgnore--;
						} else {
							this.zoom((e.which === 1 ? 1 : -1) * defaultZoom, this.getWorldPoint(e), true);

							this.changeState("default");
							afterZoomAction();
							e.preventDefault();
						}
					}, this),
					"mousemove": function (e) {
						// mousemove when not dragging --> just update the zoom mode (cursor)
						updateZoomMode(e);
					},
					"contextmenu": function (e) {
						// prevent the context menu
						e.preventDefault();
						e.stopPropagation();
					},
					"mouseleave": _.bind(function () {
						// when the mouse goes out of the viewport, revert to default state
						this.changeState("default");
					}, this),
					"esc": _.bind(function () {
						this.changeState("default");
					}, this),
					"spacedown": function () {},
					"spaceup": function () {},
					'canBeInterrupted': true
				};
			}, this)());

			/**
			 * In this mode, we check if the selection threshold is surpassed to see if we should start showing selection rectangle
			 * if so go to draggingMode
			 */
			this.registerState("couldStartDragging", _.defaults({
				"mousemove": _.bind(function (e) {
					// mouse if moving, if the dragged distance is far enough, start the dragging mode of the selection box
					var p = this.getScenePoint(e);

					selection.update(p);

					if (Math.abs(selection.size.width) > selectionThreshold || Math.abs(selection.size.height) > selectionThreshold) {
						this.requestCanvasLayer(e);
						this.changeState("dragging");
					}
				}, this),
				"mouseleave": _.bind(function () {
					// mouse left viewport, go back to default state				
					this.changeState("default");
				}, this),
				"esc": _.bind(function () {
					mouseUpsToIgnore++;
					this.changeState("default");
				}, this),
				"canBeInterrupted": false
			}, this.getState("default")));

			/**
			 * release the canvas
			 */
			var cleanupCanvas = _.bind(function () {
				this.releaseCanvasLayer();
			}, this);

			/**
			 * Render the selection rectangle
			 */
			var renderSelection = _.throttle(_.bind(function () {
				var ctx = this.getCanvasCtx();
				if (ctx) {
					var csize = this.getCanvasSize();
					// first clear all previous drawings
					ctx.clearRect(0, 0, csize.width, csize.height);

					// draw a greyed out overlay
					ctx.beginPath();
					ctx.strokeStyle = "rgba(0, 0, 0, 1.0)";
					ctx.lineWidth = 1;
					ctx.fillStyle = "rgba(0, 0, 0, 0.40)";
					ctx.rect(0, 0, csize.width, csize.height);
					ctx.fill();
					ctx.stroke();

					// cut out the selection area
					ctx.clearRect(selection.origin.x, selection.origin.y, selection.size.width, selection.size.height);

					// draw dashed rectangle
					ctx.beginPath();
					ctx.strokeStyle = "rgba(210,210,210,0.9)";
					ctx.lineWidth = 1;
					DrawUtils.drawDashedRect(ctx, selection, [4, 3]);
					ctx.stroke();
				}
			}, this), 40);

			/**
			 * The selection rectangle is shown, the tool can't be interrupted now
			 */
			this.registerState("dragging", _.defaults({
				"enterState": function (previousState) {
					f.emit("layout.disableTextSelection");
					f.emit("viewport.startInteractive");
				},
				"leaveState": function (nextState) {
					if (nextState != "panningSelection" || nextState != "dragging") {
						setTimeout(function () {
							f.emit("layout.enableTextSelection");
						}, 500);
						f.emit("viewport.stopInteractive");
					}
				},
				"mousedown": function (e) {
					// increment mouseUp counter to ignore this right/middle click
					mouseUpsToIgnore++;
				},
				"mousemove": _.bind(function (e) {
					// update rectangle and render (if needed)
					var p = this.getScenePoint(e);

					previousMouseLocation = p;
					selection.update(p);
					renderSelection();
				}, this),
				"mouseleave": _.bind(function (e) {
					// mouse left viewport, go back to default state
					this.changeState("default");
					cleanupCanvas();
				}, this),
				"mouseup": _.bind(function (e) {
					// the mouse button is released, if no clicks need to be ignored, zoom in
					if (mouseUpsToIgnore > 0) {
						mouseUpsToIgnore--;
					} else {
						var p = this.getScenePoint(e);

						previousMouseLocation = p;
						selection.update(p);
						this.zoomToSelection(selection);
						this.changeState("default");
						afterZoomAction();
						cleanupCanvas();
					}
				}, this),
				"esc": _.bind(function (e) {
					mouseUpsToIgnore++;
					this.changeState("default");
					cleanupCanvas();
				}, this),
				"spacedown": _.bind(function () {
					this.changeState("panningSelection");
				}, this),
				"canBeInterrupted": false
			}, this.getState("default")));


			this.registerState("panningSelection", _.defaults({
				"mousemove": _.bind(function (e) {
					// update rectangle and render (if needed)
					var p = this.getScenePoint(e);


					var move = previousMouseLocation.sizeTo(p);
					previousMouseLocation = p;

					selection.origin.x += move.width;
					selection.origin.y += move.height;
					renderSelection();
				}, this),
				"spacedown": function () {

				},
				"spaceup": _.bind(function () {
					this.changeState("dragging");
				}, this)
			}, this.getState("dragging")));

			// define some key events
			this.passiveKeyEvents = [];
			this.activeKeyEvents = [];
			this.persistentKeyEvents = [];

			this._override("initialize", function (widget, toolManager) {
				this._super(widget, toolManager);

				this.registerActiveEvents(["mousedown", "mousemove", "mouseup", "mouseleave", "contextmenu"]);

				/**
				 * PERSISTENT EVENT
				 */

				// ZOOM IN
				this.persistentKeyEvents.push([
					["ctrl+plus", "f5", "ctrl+="], _.bind(function (e) {
						e.preventDefault();
						e.returnValue = false; // preventDefault() for IE
						this.zoom(1, undefined, true);
					}, this)
				]);

				// ZOOM OUT
				this.persistentKeyEvents.push([
					["ctrl+-", "f6"], _.bind(function (e) {
						e.preventDefault();
						e.returnValue = false; // preventDefault() for IE
						this.zoom(-1, undefined, true);
					}, this)
				]);


				this.persistentKeyEvents.push([
					["ctrl+0", "f9"],
					function (e) {
						if (!e.shiftKey) {
							e.preventDefault();
							f.emit("viewport.zoomToFit", {
								animated: true
							});
						}
					}
				]);


				/**
				 * PASSIVE EVENTS
				 */

				// activate zoom tool
				this.passiveKeyEvents.push(["z", _.bind(function (e) {
					e.preventDefault();
					f.emit("toolManager.activateTool", this);
				}, this), "keypress"]);

				// activate zoom tool until one zoom action is done
				this.passiveKeyEvents.push(["f8", _.bind(function (e) {
					e.preventDefault();
					deactiveToolAfterZoomAction = true;
					f.emit("toolManager.activateTool", this, true);
				}, this)]);

				/**
				 * ACTIVE EVENTS
				 */
				// cancel zoom in action
				this.activeKeyEvents.push(["esc", _.bind(function (e) {
					e.preventDefault();
					this.getCurrentState()["esc"](e);
				}, this), "keyup"]);

				// spacebar, drag zoom rectangle
				this.activeKeyEvents.push(["space", _.bind(function (e) {
					this.getCurrentState()["spacedown"](e);
				}, this), "keydown"]);

				this.activeKeyEvents.push(["space", _.bind(function (e) {
					this.getCurrentState()["spaceup"](e);
				}, this), "keyup"]);


			});

			/**
			 * Zoom in/out
			 *
			 * @param step {int} if positive zoom in, if negative zoom out
			 * @param worldMiddle {Point} what is the center of the zoom action
			 * @param animated {boolean} show animation
			 */
			this.zoom = function (step, worldMiddle, animated) {
				this._widget.getWorld().transform({
					zoom: step,
					middle: worldMiddle,
					animated: animated
				});
			};

			/**
			 * Zoom to fit to a selection
			 *
			 * @param selection {Rect}
			 */
			this.zoomToSelection = function (selection) {
				this._widget.getWorld().zoomToSceneRect(selection, true);
			};


			/**
			 * Can the tool be interrupted ?
			 */
			this.canBeInterrupted = function () {
				return this.getCurrentState().canBeInterrupted;
			};

			this.getDefaultCursor = function () {
				if (defaultZoom == -1) {
					return "cursor-zoomOut";
				} else {
					return "cursor-zoomIn";
				}
			};


			/**
			 * @override
			 */
			this._override("doActivation", function () {
				this._super();
				this.changeState("default");
			});

			/**
			 * @override
			 */
			this._override("deactivate", function (interruptRequest) {
				this._super(interruptRequest);
				this.changeState("default");
				cleanupCanvas();
			});

		};

		// extend from the AbstractTool
		ZoomTool.prototype = Object.create(AbstractTool.prototype);
		ZoomTool.prototype.constructor = ZoomTool;

		/**
		 * Get the icon offset of the zoom tool
		 * @override
		 */
		ZoomTool.prototype.getIconOffset = function () {
			return new ShapeUtils.Size(6, 6);
		};

		/**
		 * override the passivation process, register a persistent handle scroll wheel listener
		 * @override
		 */
		ZoomTool.prototype.doPassivation = function () {
			AbstractTool.prototype.doPassivation.call(this);

			// add persistent mousewheel listener
			// zoom in/out when scrolling
			Mousewheel.addListener(_.throttle(_.bind(this.handleScrollWheel, this), 150));
		};


		/**
		 * Handle scroll wheel events
		 * @param {EventObj} event
		 * @param {Number} delta
		 */
		ZoomTool.prototype.handleScrollWheel = function (event, delta) {
			// only react if scrolling in viewport
			if (this._widget.context.has(event.target).length > 0) {
				this.zoom(delta < 0 ? -1 : 1, this.getWorldPoint(event), true);
				event.preventDefault();
			}
		};


		return ZoomTool;

	});

define('web-viewer/layout/component/LogSlider',["component/tooltip/Tooltip",
		"web-viewer/utils/ViewerUtils",
		"jquery",
		"jqueryUI",
		"underscore"
	],
	function (Tooltip,
		ViewerUtils,
		$) {
		"use strict";

		/**
		 * @class A log slider is a component that has a slider bar that slides logarithmic/exponential.
		 * The minp and maxp parameters define the min/max number of the step in the slider.
		 * The minv and maxv parameters define the min/max values that can be returned in the onSliderChanged callback
		 * or can be set via setValue for the slider.
		 * The wrapping element to add to a component sits in Slider.element
		 * @exports LogSlider
		 * @constructor
		 * @param {Core.ViewerFacade} f - Viewer facade
		 * @param {Object} options -
		 *    {Number} minp: the minimum slider position (Default: 0)
		 *    {Number} maxp: the maximum slider position (Default: 100)
		 *    {Number} minv: the result minimum value (Default: 1)
		 *    {Number} maxv: the result maximum value (Default: 100000)
		 *    {Function} onSliderChanged: called when the slider value is changed. Arguments: value (Required)
		 *    {String} tooltip: Tooltip text that should be shown when you hover over the component. (Optional)
		 */
		var LogSlider = function (f, options) {
			this.f = f;

			// Set Options
			this._minp = options.minp === undefined ? 0 : options.minp;
			this._maxp = options.maxp === undefined ? 100 : options.maxp;
			this.setOptions(options);
			this._onSliderChanged = options.onSliderChanged;

			// Wrapper element
			this.element = $("<div />").addClass("sliderWrapper");

			// Add slider
			this._slider = $("<div />").addClass("slider");
			this.element.append(this._slider);
			this._slider.slider({
				slide: _.bind(function (event, ui) {
					this._fireSliderChanged(ui.value);
				}, this),
				min: this._minp,
				max: this._maxp
			});

			// Tooltip
			if (options.tooltip !== undefined) {
				this._tooltip = new Tooltip(this.f.i18n, this.element, this.f.getContext(), this.f.getContext(), this.f.isMobile(),options.tooltip);
			}
		};

		/**
		 * Calculate the value from a slider position
		 * @param position
		 * @returns
		 */
		LogSlider.prototype._toValue = function (position) {
			return Math.exp((position - this._minp) * this._scale + this._minv);
		};

		/**
		 * Calculate the slider position from the given value
		 * @param {Number} value - Value to calculate the slider position from
		 * @returns {Number} Position of the slider for the given value
		 */
		LogSlider.prototype._toPosition = function (value) {
			return this._minp + (Math.log(value) - this._minv) / this._scale;
		};

		/**
		 * Fire that the slider position has changed
		 * @param {Number} value - The slider position
		 */
		LogSlider.prototype._fireSliderChanged = function (position) {
			var v = this._toValue(position);
			this._onSliderChanged(v);
		};

		/**
		 * Set the value of the slider
		 * @param {Number} newValue - New value of the slider
		 */
		LogSlider.prototype.setValue = function (newValue) {
			var p = this._toPosition(newValue);
			this._slider.slider("value", p);
		};

		/**
		 * Update the initial options
		 * @param {Object} options -
		 *    {Number} minv: the result minimum value (Default: 1)
		 *    {Number} maxv: the result maximum value (Default: 100000)
		 */
		LogSlider.prototype.setOptions = function (options) {
			this._minv = Math.log(options.minv === undefined ? 1 : options.minv);
			this._maxv = Math.log(options.maxv === undefined ? 100000 : options.maxv);
			this._scale = (this._maxv - this._minv) / (this._maxp - this._minp);
		};

		return LogSlider;
	});

define('web-viewer/navigation/ZoomBar',["web-viewer/layout/component/LogSlider",
		"web-viewer/utils/ListenerUtil",
		"web-viewer/layout/component/MiniButton",
		"jquery",
		"underscore"
	],
	function (LogSlider,
		ListenerUtil,
		MiniButton,
		$,
		_) {


		/**
		 * @exports ZoomBar
		 * @memberof NavigationModule 
		 * @class The ZoomBar shows the zoom bar of the navigator widget
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 */
		var ZoomBar = function ZoomBar(f) {
			this.f = f;
			ListenerUtil.mixin(this, this.f);

			// the root element
			this.element = $("<div />").addClass("zoomBar clearfix");

			// min button
			this._minButton = new MiniButton(this.f, "minZoom", "icon-WCR_Minus", true, _.bind(this._minZoom, this));
			this.element.append(this._minButton.element.addClass("minZoom"));

			// zoom percentage
			this._zoomPercentage = $("<span />").addClass("zoomPercentage");
			this.element.append(this._zoomPercentage);

			// plus button
			this._plusButton = new MiniButton(this.f, "plusZoom", "icon-WCR_Plus", true, _.bind(this._plusZoom, this));
			this.element.append(this._plusButton.element.addClass("plusZoom"));

			// slider
			var viewportModule = this.f.getModule("viewport");
			if (viewportModule) {
				this._slider = new LogSlider(this.f, {
					onSliderChanged: _.bind(this._sliderChanged, this),
					minp: 1,
					maxp: viewportModule.getNumberOfZoomLevels() // this stays stable
				});
				this._updateMinMax(); // this can change
				this.element.append(this._slider.element);
			}

			this._registerListener("transform.viewportWidget.minMaxZoomChanged", this._updateMinMax, this);
			this._registerListener("transform.viewportWidget.transformChanged", this._updateZoom, this);
		};

		/**
		 * Called when the slider is changed
		 * @param {Number} sliderValue - The (new) slider value
		 */
		ZoomBar.prototype._sliderChanged = function (sliderValue) {
			var viewportModule = this.f.getModule("viewport");
			if (viewportModule) {
				viewportModule.transform({
					fixedzoom: sliderValue,
					animated: true
				});
			}
		};

		/**
		 * Called when the zoom percentage/slider should be updated
		 */
		ZoomBar.prototype._updateZoom = function () {
			var viewportModule = this.f.getModule("viewport");
			if (viewportModule) {
				// update the percentage
				var zoomPercentage = viewportModule.getZoomPercentage();
				this._zoomPercentage.text(zoomPercentage.toFixed(2) + "%");

				// update the slider
				var zoom = viewportModule.getZoom();
				this._slider.setValue(zoom);
			}
		};

		/**
		 * Update the max value of the slider
		 */
		ZoomBar.prototype._updateMinMax = function () {
			var viewportModule = this.f.getModule("viewport");
			if (viewportModule) {
				var minZoom = viewportModule.getMinZoom();
				var maxZoom = viewportModule.getMaxZoom();
				this._slider.setOptions({
					minv: minZoom,
					maxv: maxZoom
				});
			}
		};

		/**
		 * Called when the min zoom button is clicked
		 */
		ZoomBar.prototype._minZoom = function () {
			var viewportModule = this.f.getModule("viewport");
			if (viewportModule) {
				viewportModule.transform({
					zoom: -1,
					animated: true
				});
			}
		};

		/**
		 * Called when the plus zoom button is clicked
		 */
		ZoomBar.prototype._plusZoom = function () {
			var viewportModule = this.f.getModule("viewport");
			if (viewportModule) {
				viewportModule.transform({
					zoom: 1,
					animated: true
				});
			}
		};

		return ZoomBar;
	});

define('web-viewer/navigation/NavigatorPanTool',["underscore",
		"web-viewer/viewport/tools/AbstractTool",
		"utils/ShapeUtils",
		"web-viewer/core/CoreConstants"
	],
	function (_,
		AbstractTool,
		ShapeUtils,
		CoreConstants) {
		"use strict";

		/**
		 * @exports NavigatorPanTool
		 * @memberof NavigationModule
		 * @augments Viewport.AbstractTool
		 *
		 * @class
		 * The NavigatorPanTool tool enables the user to pan in the navigator widget by moving the highlighted area.
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 * @param {ViewportModule} viewport
		 */
		var NavigatorPanTool = function NavigatorPanTool(f, viewport) {
			AbstractTool.call(this, f, CoreConstants.TOOLS.NAVIGATIONPAN);

			/**
			 * Start point of the panning, in pixels
			 */
			this._startPoint = new ShapeUtils.Point();

			/**
			 * Offset of the currently active highlight in the miniature image, in world coordinates (mm)
			 */
			this._startOffset = new ShapeUtils.Size();

			/**
			 * Current center of the highlight, in pixels
			 */
			this._currentCenter = new ShapeUtils.Size();

			/**
			 * The viewport module instance
			 */
			this._viewport = viewport;

			/**
			 * Save the last mouse position while panning to trigger a rerender when leaving panning state
			 */
			this._lastMousePosition = null;

			/**
			 * DEFAULT STATE
			 */
			this.registerState("default", {
				mousedown: _.bind(function (e) {
					var visibleRect = this._viewport.getVisibleRect();
					if (!this._viewport.getWorldBounds().rectEquals(visibleRect)) {
						var offset = this._widget.getNavigatorImageOffset(); //get the offset of the image in the browser window, to get the location in the miniature image
						var coordinates = new ShapeUtils.Point(e.pageX - offset.left, e.pageY - offset.top); //coordinates of the click point in the canvas
						var worldCoordinates = this.convertScenePointToWorldPoint(coordinates); //the coordinates in the world (in mm)
						this._startOffset = this._viewport.getWorld().getOffsetSize(); //set the start offset of the currently active selection (highlighted area), in mm
						this._startPoint = coordinates; //the startpoint of the panning
						if (visibleRect.containsPoint(worldCoordinates)) { //cursor location is in the current seletion (highlighted area)
							this.changeState("panning"); //change the state to panning
						} else if (this._viewport.getWorldBounds().containsPoint(worldCoordinates)) { //if not move the visible rect to the clicked location, enable panning
							this._currentCenter = visibleRect.center(); //get the center of the current selection (highlighted area)
							this._updateViewport(coordinates, true); //update the viewport, move the selection to the clicked area
							this._startPoint = this._widget.getNavigatorTransformation().transformedPoint(this._currentCenter); //transform the world coordinates back to scene coordinates
							this.changeState("panning"); //set the state to panning
						}
					}

				}, this),

				mouseup: function () {},

				mousemove: function () {},

				mouseout: function () {},

				mouseleave: function () {},

				/**
				 * on right click don't do anything
				 */
				contextmenu: _.bind(function (e) {
					e.preventDefault();
					e.stopPropagation();
				}, this)
			});

			/**
			 * PANNING STATE
			 */
			this.registerState("panning", _.defaults({
				/**
				 * when starting to drag, notify viewport on interactive behavior and disable text selection
				 */
				enterState: _.bind(function () {
					this.setCursor("cursor-moveObject"); //set the cursor to a move icon
					this.f.emit("viewport.startInteractive");
					this.f.emit("viewport.startPanning");
					this.f.emit("layout.disableTextSelection");
				}, this),

				/**
				 * When dragging has stopped, notify viewport and enable text selection again
				 */
				leaveState: _.bind(function () {
					this.setCursor("cursor-default");
					this.f.emit("viewport.stopInteractive");
					this.f.emit("viewport.stopPanning");
					this.f.emit("layout.enableTextSelection");
					if (this._lastMousePosition) { //trigger a forced update of the viewport after panning has stopped to trigger a rerender of the viewport
						this._updateViewport(this._lastMousePosition, false, true); //force an update
					}
				}, this),

				/**
				 * On mouse up, panning has stopped , update the world a last time, and go back to the default state
				 */
				mouseup: _.bind(function (e) {
					e.preventDefault();
					this._lastMousePosition = null; //set to null to make sure a click will not trigger a forced update viewport
					this.changeState("default");
				}, this),

				/**
				 * Mouse is moving, update panning/world
				 */
				mousemove: _.bind(function (e) {
					var offset = this._widget.getNavigatorImageOffset(); //get the correct offset
					var coordinates = new ShapeUtils.Point(e.pageX - offset.left, e.pageY - offset.top); //calculate the correct coordinates
					this._lastMousePosition = coordinates; //set the last mouse position for a rerender if needed.
					this._updateViewport(coordinates, false); //update the viewport
				}, this),

				/**
				 * panning has stopped, update world, go back to default state
				 */
				mouseout: _.bind(function (e) {
					e.preventDefault();
					this.changeState("default");
				}, this),

				/**
				 * panning has stopped, update world, go back to default state
				 */
				mouseleave: _.bind(function (e) {
					e.preventDefault();
					this.changeState("default");
				}, this)

			}, this.getState("default")));

		};

		// inherit
		NavigatorPanTool.prototype = Object.create(AbstractTool.prototype);
		// correct the constructor pointer 
		NavigatorPanTool.prototype.constructor = NavigatorPanTool;

		/**
		 * Initialize the tool
		 * Only accepts NavigatorWidget!!
		 * @param {NavigatorWidget} widget
		 */
		NavigatorPanTool.prototype.initialize = function (widget, toolManager) {
			AbstractTool.prototype.initialize.call(this, widget, toolManager);

			// register the needed events
			this.registerActiveEvents(["mousedown", "mousemove", "mouseup", "mouseout", "mouseleave", "contextmenu"]);
		};

		/**
		 * Returns the default cursor, takes the current state into account
		 * @returns {String}
		 */
		NavigatorPanTool.prototype.getDefaultCursor = function () {
			if (this.getCurrentStateName() != "panning") {
				return "cursor-default";
			} else {
				return "cursor-moveObject";
			}
		};

		/**
		 * The tool can be interrupted when not in "panning" state, will not be used as the tool is managed by Navigatorwidget
		 * @returns {Boolean}
		 */
		NavigatorPanTool.prototype.canBeInterrupted = function () {
			return this.getCurrentStateName() != "panning";
		};

		/**
		 * Do the activation of the tool.
		 */
		NavigatorPanTool.prototype.doActivation = function () {
			AbstractTool.prototype.doActivation.call(this);
			this.changeState("default");
		};

		/**
		 * Do the deactivation of the tool
		 * @param {function} interruptRequest
		 */
		NavigatorPanTool.prototype.deactivate = function (interruptRequest) {
			AbstractTool.prototype.deactivate.call(this, interruptRequest);
			this.changeState("default");
		};

		/**
		 * Update the world
		 *
		 * @param {ShapeUtils.Point} to - to where has the user dragged, or where has the user clicked
		 * @param {Boolean} useCurrentCenter - should the selection be moved to 'to' from the current center
		 * @param {Boolean} [forceUpdate] - if set it will force an update
		 */
		NavigatorPanTool.prototype._updateViewport = function (to, useCurrentCenter, forceUpdate) {
			var draggedSize = this._startPoint.sizeTo(to);

			if (Math.abs(draggedSize.width) > 0 || Math.abs(draggedSize.height) > 0 || useCurrentCenter || forceUpdate) {
				var worldOffset = new ShapeUtils.Size();
				if (useCurrentCenter) {
					worldOffset = this._currentCenter.sizeTo(this.convertScenePointToWorldPoint(to));
				} else {
					worldOffset = this.convertScenePointToWorldPoint(this._startPoint).sizeTo(this.convertScenePointToWorldPoint(to));
				}

				worldOffset = new ShapeUtils.Size(-worldOffset.width, -worldOffset.height);
				worldOffset.add(this._startOffset);
				this._viewport.getWorld().setOffsetSize(worldOffset);
			}

		};

		/**
		 * convert the scene point to world coordinates
		 * @param {ShapeUtils.Point} point
		 * @returns {ShapeUtils.Point}
		 */
		NavigatorPanTool.prototype.convertScenePointToWorldPoint = function (point) {
			return this._widget.getNavigatorTransformation().createInverse().transformedPoint(point);
		};

		return NavigatorPanTool;
	});

define('web-viewer/navigation/NavigatorWidget',["web-viewer/layout/AbstractWidget",
		"web-viewer/layout/LayoutConstants",
		"web-viewer/layout/component/MiniButton",
		"web-viewer/navigation/ZoomBar",
		"jquery",
		"underscore",
		"utils/HTTPUtils",
		"web-viewer/utils/Fragment",
		"utils/Transformation",
		"web-viewer/utils/DrawUtils",
		"utils/ShapeUtils",
		"web-viewer/utils/ListenerUtil",
		"web-viewer/navigation/NavigatorPanTool",
		"web-viewer/compare/CompareConstants"
	],
	function (AbstractWidget,
		LayoutConstants,
		MiniButton,
		ZoomBar,
		$,
		_,
		HTTPUtils,
		Fragment,
		Transformation,
		DrawUtils,
		ShapeUtils,
		ListenerUtil,
		NavigatorPanTool,
		CompareConstants) {
		"use strict";

		/**
		 * @const the max height of navigator image, to prevent the widget will grow to large
		 */
		var MAX_HEIGHT = 200;

		/**
		 * @exports NavigatorWidget
		 * @memberof NavigationModule
		 * @augments Layout.AbstractWidget
		 * @class The NavigatorWidget shows the position of the user in the document and allows the user to navigate through the document
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 */
		var NavigatorWidget = function NavigatorWidget(f) {
			AbstractWidget.call(this, f, LayoutConstants.Widget.NAVIGATOR);
			ListenerUtil.mixin(this, f);

			//the container all the layers (canvas-elements) will be added to, makes it possible to center them
			this.navigatorImage = $("<div>").addClass("navigatorImage");

			//define a layer for all the graphics that will be drawn in the navigator
			this._graphicsLayer = $("<canvas>").css({
				'position': 'absolute', //makes it possible to place multiple layer on top of each other
				'top': 0,
				'left': 0
			});

			//define a layer for all the CAD lines that will be drawn in the navigator
			this._cadLayer = $("<canvas>").css({
				'position': 'absolute',
				'top': 0,
				'left': 0
			});

			//define a layer for the interactive selection rectangle
			this._interactiveLayer = $("<canvas>").css({
				'position': 'absolute',
				'top': 0,
				'left': 0
			});

			//last requested fragment will be saved to this property
			this._fragment = null;

			//instance of the viewport, so we don't need to request it all the time
			this._viewport = null;

			this._worldTranformation = null;

			this.f.loadBundle("viewer", function () {
				this.setTitle(this.f.t("NavigatorTitle"));
			}, this);
			
			this.setIcon("icon icon-WCR_Navigator");

			//make the render navigator image debounced, so it doesn't render all the time
			var debouncedRender = _.debounce(this._renderNavigatorImage, 200);

			//only renders the overlay of the current visible rect in the navigator widget
			var throttledRectRender = _.throttle(_.bind(function () {
				if (this._viewport) {
					var rotation = this._viewport.getWorld().getRotation() % 360;
					var worldBounds = this._viewport.getWorldBounds();

					//if the rotation switches width and height of the world, change them for the world as we'll use this to calculate 
					if (rotation == 90 || rotation == 270) {
						worldBounds.size = new ShapeUtils.Size(worldBounds.size.height, worldBounds.size.width);
					}
					if (worldBounds.size.width !== 0 && worldBounds.size.height !== 0) {
						//render the visible rect on to the navigator image
						this._renderVisibleRect(this._interactiveLayer[0].getContext("2d"), worldBounds, rotation);
					}
				}
			}, this), 10);

			this.resized = function () { //when the widgets gets resized render the widget again
				debouncedRender.call(this);
			};

			// Zoom bar
			this._zoomBar = null;

			//register all the listeners for events that can happen in the viewport 
			this._registerListener("transform.viewportWidget.worldBoundsChanged", debouncedRender, this);
			this._registerListener("module.compare.changed.diffMode", debouncedRender, this);
			this._registerListener("module.viewport.finishedLoadingImageData", debouncedRender, this);
			this._registerListener("cadModel.*.modelChanged", debouncedRender, this);
			this._registerListener("transform.viewportWidget.rotated", debouncedRender, this);
			this._registerListener("transform.viewportWidget.flipped", debouncedRender, this);
			this._registerListener("transform.viewportWidget.flippedViewFromBack", debouncedRender, this);
			this._registerListener("transform.viewportWidget.transformChanged", throttledRectRender, this);
			this._registerListener("modalResize", debouncedRender, this, this);
			this._registerListener("collapsed", debouncedRender, this, this);
			this._registerListener("visible", debouncedRender, this, this);

		};

		//inherit
		NavigatorWidget.prototype = Object.create(AbstractWidget.prototype);
		NavigatorWidget.prototype.constructor = NavigatorWidget;
		NavigatorWidget.widgetName = LayoutConstants.Widget.NAVIGATOR;

		/**
		 * @override
		 */
		NavigatorWidget.prototype.setContext = function (context, headerExtra) {
			AbstractWidget.prototype.setContext.call(this, context);

			context.append(this.navigatorImage);
			this.navigatorImage.append(this._graphicsLayer);
			this.navigatorImage.append(this._cadLayer);
			this.navigatorImage.append(this._interactiveLayer);

			// initial render or when viewport becomes available
			this.f.whenModuleAvailable("viewport", function (viewportModule) {
				this._viewport = viewportModule;

				this._panTool = new NavigatorPanTool(this.f, this._viewport);
				this._panTool.initialize(this);
				this._panTool.activate();

				this._zoomBar = new ZoomBar(this.f);
				this.context.addClass("navigatorWidget").append(this._zoomBar.element);
				this._renderNavigatorImage();
			}, this);
		};

		/**
		 * Render the navigator image.
		 */
		NavigatorWidget.prototype._renderNavigatorImage = function () {
			if (!this.isCollapsed() && this.isVisible() && this._viewport && this.context) {
				var page = this.f.getActivePage();
				var cadOverlay = this.f.getDocumentController().getCadOverlay();
				var size = new ShapeUtils.Size(this.context.innerWidth(), this.context.innerHeight() - this._zoomBar.element.outerHeight()); //get the current size of the widget

				var rotation = this._viewport.world.getRotation() % 360;
				var worldBounds = this._viewport.getWorldBounds();

				if (worldBounds.size.width !== 0 && worldBounds.size.height !== 0) {
					//if the rotation switches width and height of the world, change them for the world as we'll use this to calculate 
					if (rotation == 90 || rotation == 270) {
						worldBounds.size = new ShapeUtils.Size(worldBounds.size.height, worldBounds.size.width);
					}

					//calculate the correct width/height of the layers so they fit into the widget, to a maximum of 200px height and the current width of the widget
					this._pageRes = size.width / worldBounds.size.width; //first calculate the resolution of the width given the current widget width
					var newWidth = size.width; //the width stays the same for now, we'll have to check if the height isn't more than 200px
					var newHeight = worldBounds.size.height * this._pageRes; //calculate the height with the resolution of the width
					var widgetHeight = MAX_HEIGHT;
					if (this._isModal) {
						widgetHeight = size.height;
					}
					//check if the new height of the layers isn't higher than 200px or height (if modal)
					if (newHeight > widgetHeight) {
						newHeight = widgetHeight; //we'll restrict the dimensions of the navigator image to the maximum height
						this._pageRes = newHeight / worldBounds.size.height; //new resolution is needed given the fixed height of 200px 
						newWidth = worldBounds.size.width * this._pageRes; //calculate the width of the layer with this resolution
					}

					this.navigatorImage.height(Math.round(newHeight)); //set the width and height of the navigator images to make sure the widget will resize
					this.navigatorImage.width(Math.round(newWidth));
					this._fireResized();

					var newSize = new ShapeUtils.Size(newWidth, newHeight); //make sizes that take the rotation into account

					var rotatedSize = new ShapeUtils.Size(newWidth, newHeight); //make a size that doesn't take the rotation into account 
					if (rotation == 90 || rotation == 270) { //if the image is rotated, still make sure the width represents the original image width (only now in pixels)
						rotatedSize = new ShapeUtils.Size(newHeight, newWidth);
					}

					if (page.hasGraphics() || page.isBlankPage()) { //if there are graphics shown
						this._graphicsLayer.show();
						this._graphicsLayer[0].width = newWidth; //set the width and height of the canvas to the correct dimensions 
						this._graphicsLayer[0].height = newHeight;
						this._renderGraphicsImage(this._graphicsLayer[0].getContext("2d"), this._pageRes, newSize, rotatedSize, worldBounds); //render the graphics on the given canvas
					} else { //if there are no graphics, hide the graphics layer
						this._graphicsLayer.hide();
					}
					if (page.hasCad() || cadOverlay) { //if there is CAD shown in the viewport
						this._cadLayer.show();
						this._cadLayer[0].width = newWidth;
						this._cadLayer[0].height = newHeight;

						if (this.f.getModule("cad")) {
							if (page.hasCad()) { //there is only CAD, no CAD+Graphics
								this._renderCAD(this._cadLayer[0].getContext("2d"), page, this._pageRes, worldBounds, null, rotatedSize);
							} else { //there is CAD+Graphics
								//big difference is the extra offset between CAD and graphics
								this._renderCAD(this._cadLayer[0].getContext("2d"), cadOverlay, this._pageRes, worldBounds, true, rotatedSize);
							}
						}

					} else {
						this._cadLayer.hide();
					}
					this._interactiveLayer[0].width = newWidth;
					this._interactiveLayer[0].height = newHeight;
					this._renderVisibleRect(this._interactiveLayer[0].getContext("2d"), worldBounds, rotation);
				}

			}
		};

		/**
		 * Render the Graphics that are shown in the viewport onto the canvas of the navigator widget
		 * @param {CanvasRenderingContext2D} ctx - context of the canvas to drawn the image on
		 * @param {Number} pageRes - resolution of the image, will decide the size in pixels, as this says how many pixel per mm there will be rendered
		 * @param {ShapeUtils.Size} newSize - new size (in pixels) of the canvas (as visible and rotated)
		 * @param {ShapeUtils.Size} rotatedSize - new size (in pixels) of the canvas (not rotated, 90,270 degrees won't show the 'visbile' width/height)
		 * @param {ShapeUtils.Rect} worldBounds - the world bounds of the visible image (viewport) in mm
		 */
		NavigatorWidget.prototype._renderGraphicsImage = function (ctx, pageRes, newSize, rotatedSize, worldBounds) {
			if (pageRes <= 0 || this.f.isParameterUpdateOngoing()) {
				return; // we can't render if the page resolution is 0 or a parameter update is ongoing
			}
			var context = { //construct a context that is used in the intercept pattern to request fragments
				resolution: pageRes,
				rectangle: this._viewport.getWorldBounds(),
				// the servlet that should be used to fetch the image information
				servlet: "",
				// the parameters used for requesting the image
				params: {
					x1: 0,
					x2: 0,
					y1: 0,
					y2: 0
				}
			};

			//set the fragment to the requested fragment, only the last requested fragment will be set to the this._fragment object
			this._fragment = new Fragment(worldBounds, pageRes);

			//use the intercept pattern to get the same fragments as visible in the viewport
			// ask all interceptors to update the context. All listeners have access to the context object and can adapt the information that is in the object
			// there is no predefined order in which these interceptors are called, their actions thus should not overlap!
			this.f.emit("intercept.fragmentURL", context);

			if (context.params.file === undefined && context.params.diff === undefined) { //draw a white rectangle for blank pages
				ctx.setTransform(1, 0, 0, 1, 0, 0); //reset the previous transformation of the graphics layer (canvas-context)
				ctx.clearRect(0, 0, this._graphicsLayer.width(), this._graphicsLayer.height()); //make sure the canvas is clear before drawing new image on it
				this._transformCanvas(ctx, newSize, null, null); //calculate the new transformation for the graphics layer
				ctx.fillStyle = "rgba(255, 255, 255, 1)"; //make the fill color white
				DrawUtils.drawRect(ctx, 0, 0, rotatedSize.width, rotatedSize.height, 0, true, false); //draw the image, it will be rotated by the transformation to draw in rotate indepented size
			} else {
				//move the diff parameter to the file parameter, ViewerServer doesn't like undefined for file (current file is blank)
				if (context.params.file === undefined && context.params.diff !== undefined) {
					context.params.file = context.params.diff;
					delete context.params.diff; //make sure to delete the difference file (this is blank)
					context.params.rs = context.params.rs == 1 ? 2 : 1; //switch the location of the 'active' page
				} else
				if (context.params.dmode == CompareConstants.diffMode.SHOWBOTHSIDEBYSIDE ||
					context.params.dmode == CompareConstants.diffMode.SHOWBOTHSIDEBYSIDEHIGHLIGHT ||
					context.params.dmode == CompareConstants.diffMode.SHOWBOTHSIDEBYSIDECHANGESONLY) { //if there is a side-by-side compare active, only show the active document page in the navigator widget
					context.params.dmode = CompareConstants.diffMode.CURRENTVERSION;
				}
				delete context.params.colorBITWISE;
				// no need to send isDefaultView to the server
				delete context.params.isDefaultView;

				//construct an url to fetch the correct fragment from the viewserver
				var url = HTTPUtils.constructURL(context.servlet, context.params);

				// ViewServer can't handle urlencoded commas, fix commas 
				url = url.replace(/%2C/g, ",");
				//fetch the fragment, uses the viewport image cache 
				this._fragment.fetchImage(this._viewport.getImageCache(), url, _.bind(function () {
					if (this._fragment.isFetched()) { // make sure it is the current fragment that was loaded... (so it should be fetched)
						ctx.setTransform(1, 0, 0, 1, 0, 0); //reset the previous transformation of the graphics layer (canvas-context)
						ctx.clearRect(0, 0, this._graphicsLayer.width(), this._graphicsLayer.height()); //make sure the canvas is clear before drawing new image on it
						this._transformCanvas(ctx, newSize, null, null); //calculate the new transformation for the graphics layer
						ctx.drawImage(this._fragment.image, 0, 0, rotatedSize.width, rotatedSize.height); //draw the image, it will be rotated by the transformation to draw in rotate indepented size
					}
				}, this), function () {});


			}
		};

		/**
		 * Renders CAD linework in the navigator widget, on the provided canvas-context
		 * @param {CanvasRenderingContext2D} ctx - context of the canvas to drawn the image on
		 * @param {AbstractDocumentPage} page - page that has the CAD linework information, CAD can be requested from this page
		 * @param {Number} pageRes - the resolution of the CAD, will be used to set the correct scale to the canvas-context
		 * @param {ShapeUtils.Rect} worldBounds - bounds of the world, size of the CAD-file (or combination of CAD-files in compare)
		 * @param {Boolean} isCadOverlay - Does the overlay need to be rendered or not
		 * @param {ShapeUtils.Size} canvasSize - new size (in pixels) of the canvas (not rotated, 90,270 degrees won't show the 'visbile' width/height)
		 */
		NavigatorWidget.prototype._renderCAD = function (ctx, page, pageRes, worldBounds, isCadOverlay, canvasSize) {
			page.requestCad(_.bind(function () { //to make sure the CAD is present when trying to render it in the navigator widget
				var cadModule = this.f.getModule("cad"); //request the CAD-module
				//we need CadModel of both the current and reference (if any) page
				var currentCadModel;
				if (isCadOverlay) {
					currentCadModel = cadModule.getCadOverlayModel();
				} else {
					currentCadModel = cadModule.getCurrentPageCadModel();
				}
				var referenceCadModel = cadModule.getReferencePageCadModel();
				var visibleRect = this._viewport.getWorldBounds(); //the currently visible rect (represents the rectangle of the entire CAD-document)
				if (currentCadModel !== null && currentCadModel.getCad()) {
					//get the origin of the active page's CAD linework
					var origin = currentCadModel.getCad().getOrigin();
					//make sure the visibleRect is translated to the (0,0)-point (the origin) of the CAD file
					visibleRect.origin.x += origin.x; //add the x-coordinate of the origin to it
					visibleRect.origin.y -= origin.y; //as y-axis starts for the bottom (not the top), subtract the y-coordinate of the origin

					var cadLines = currentCadModel.getVisibleCadLines(); //return all the visible CAD lines

					var offset = new ShapeUtils.Size();
					if (!isCadOverlay) {
						offset = new ShapeUtils.Size(-origin.x - worldBounds.origin.x, origin.y - worldBounds.origin.y); //this is an extra offset needed for CAD 
					} else {
						offset = new ShapeUtils.Size(-worldBounds.origin.x, -worldBounds.origin.y); //this is an extra offset needed for CAD 
					}

					//reset the transform of the canvas-context
					ctx.setTransform(1, 0, 0, 1, 0, 0);
					ctx.clearRect(0, 0, this._cadLayer.width(), this._cadLayer.height()); //clear the canvas before rendering the CAD again on the canvas

					var t = this._transformCanvas(ctx, worldBounds.size, offset, pageRes); //calculate the correct transformation (include pageRes as we will drawn CAD in mm, not pixels)				

					var overlayTransform = null;
					if (isCadOverlay) {
						overlayTransform = cadModule.getOverlayTransform();
						if (overlayTransform) {
							ctx.transform(overlayTransform.a, overlayTransform.b, overlayTransform.c, overlayTransform.d, overlayTransform.e, overlayTransform.f);
						}
					} else {
						// draw background
						ctx.save();
						ctx.fillStyle = currentCadModel.getBackgroundString();
						DrawUtils.drawRect(ctx, visibleRect.origin.x, visibleRect.origin.y, visibleRect.size.width, visibleRect.size.height, 0, true, true);
						ctx.restore();
					}

					var newTransform = t.copy();

					if (cadModule.compareDiffMode !== CompareConstants.diffMode.DIFFVERSION || isCadOverlay) {
						if (overlayTransform) {
							newTransform.concatenate(overlayTransform);
						}
						var drawRect = this.f.getModule("viewport").getWorld().getWorldRectFromScene(new ShapeUtils.Rect(null, new ShapeUtils.Size(this._cadLayer.width(), this._cadLayer.height())), newTransform);
						//draw the CAD-linework on the canvas
						DrawUtils.drawOverlay(this.f, ctx, cadLines, drawRect, function () {}, canvasSize.width / visibleRect.size.width, t, currentCadModel.getCadColor(), currentCadModel.getAlpha());
					}

					//if there is CAD-linework in the reference cadmodel, CAD is in compare mode
					if (referenceCadModel !== null && referenceCadModel.getCad() !== null && cadModule.getReferenceTransform()) {
						var referenceLines = referenceCadModel.getVisibleCadLines();

						t = cadModule.getReferenceTransform();
						t.preTranslate(origin.x, -origin.y);
						//transform the canvas
						ctx.transform(t.a, t.b, t.c, t.d, t.e, t.f);

						newTransform.concatenate(t);
						var referenceDrawRect = this.f.getModule("viewport").getWorld().getWorldRectFromScene(new ShapeUtils.Rect(null, new ShapeUtils.Size(this._cadLayer.width(), this._cadLayer.height())), newTransform);
						//draw the reference document page CAD linework
						DrawUtils.drawOverlay(this.f, ctx, referenceLines, referenceDrawRect, function () {}, canvasSize.width / visibleRect.size.width, t, referenceCadModel.getCadColor(), referenceCadModel.getAlpha());
					}
				}
			}, this));
		};

		/**
		 * Renders the currently visible rect of the viewport on to the navigator image
		 * @param {CanvasRenderingContext2D} ctx - context of the canvas to drawn overlay onto
		 * @param {ShapeUtils.Rect} worldBounds - bounds of the world, size of the CAD-file/Graphics-file (or combination of CAD-files/Graphics-files in compare)
		 * @param {Number} rotation - current rotation of the viewport
		 */
		NavigatorWidget.prototype._renderVisibleRect = function (ctx, worldBounds, rotation) {
			var visibleRect = this._viewport.getVisibleRect(); //get the currently visible rectangle
			ctx.setTransform(1, 0, 0, 1, 0, 0); //first reset the transformation 
			ctx.clearRect(0, 0, this._interactiveLayer.width(), this._interactiveLayer.height()); //clear the canvas
			//transform the canvas, add an offset for the origin of CAD, this makes the transformation from scene to world coordinates correct
			this._worldTranformation = this._transformCanvas(ctx, worldBounds.size, new ShapeUtils.Size(-worldBounds.origin.x, -worldBounds.origin.y), this._pageRes);

			// draw a greyed out overlay
			ctx.beginPath();
			ctx.strokeStyle = "rgba(0, 0, 0, 1.0)";
			ctx.lineWidth = 1 / this._pageRes;
			ctx.fillStyle = "rgba(0, 0, 0, 0.50)"; //fill the rectangle	
			if (rotation == 90 || rotation == 270) { //make sure the worldbounds have the correct orientation when rotated
				ctx.rect(worldBounds.origin.x, worldBounds.origin.y, worldBounds.size.height, worldBounds.size.width);
			} else {
				ctx.rect(worldBounds.origin.x, worldBounds.origin.y, worldBounds.size.width, worldBounds.size.height);
			}
			ctx.fill();
			ctx.stroke();

			// cut out the selection area
			ctx.clearRect(visibleRect.origin.x, visibleRect.origin.y, visibleRect.size.width, visibleRect.size.height);

			// draw dashed rectangle
			ctx.beginPath();
			ctx.strokeStyle = "rgba(255,255,255,1)";
			ctx.lineWidth = 1 / this._pageRes;
			ctx.strokeStyle = "rgba(0,0,0,1)";
			if (this._pageRes) {
				DrawUtils.drawDashedRect(ctx, visibleRect, [2, 5], this._pageRes);
			}

			ctx.stroke();
		};

		/**
		 * Transforms the canvas-context to have the correct rotation, and flip of the viewport
		 * @param {CanvasRenderingContext2D} ctx
		 * @param {ShapeUtils.Size} worldSize - size of the world
		 * @param {ShapeUtils.Size} [offset] - offset to translate the transformation
		 * @param {Number} [pageRes] - resolution of the page, used to scale the canvas, provide this if you want to drawn in world-coordinates (not pixels)
		 * @returns {Transformation}
		 */
		NavigatorWidget.prototype._transformCanvas = function (ctx, worldSize, offset, pageRes) {
			var isFlipped = this._viewport.world.isFlip(); //check in the viewport if the world is flipped or rotated
			var isFlipViewFromBack = this._viewport.world.isFlipViewFromBack();
			var rotation = this._viewport.world.getRotation() % 360;

			var t = new Transformation();

			if (pageRes !== null) { //scale the transformation if needed
				t.scale(pageRes, pageRes);
			}
			t.translate(worldSize.width / 2, worldSize.height / 2); //first translate to the center to rotate and flip the contents of the canvas
			t.rotate(rotation * (Math.PI / 180)); //rotate
			if (isFlipped) {
				t.scale(-1, 1);
			}
			if (isFlipViewFromBack) {
				if (rotation % 180 === 0) {
					// not rotated
					t.scale(-1, 1);
				} else {
					// rotated, but flip horizontally
					t.scale(1, -1);
				}
			}
			if (rotation == 90 || rotation == 270) { //if the rotation is 90 or 270 (width and heights are swapped) calculate extra translate distance
				t.translate(-worldSize.width / 2 + (worldSize.width - worldSize.height) / 2, -worldSize.height / 2 - (worldSize.width - worldSize.height) / 2);
			} else {
				t.translate(-worldSize.width / 2, -worldSize.height / 2);
			}
			if (offset !== null) { //if an offset is provide, translate over this offset
				t.translate(offset.width, offset.height);
			}
			//perform the translation on the canvas context
			ctx.transform(t.a, t.b, t.c, t.d, t.e, t.f);

			return t;
		};

		/**
		 * Get the transformation for the navigator (used for the interactive layer (and CAD))
		 * @returns {Transformation}
		 */
		NavigatorWidget.prototype.getNavigatorTransformation = function () {
			return this._worldTranformation;
		};

		/**
		 * Returns the minimum size for the widget
		 * @returns {ShapeUtils.Size}
		 */
		NavigatorWidget.prototype.getMinimumSize = function () {
			return new ShapeUtils.Size(200, this.navigatorImage.height() + this._zoomBar.element.height());
		};

		/**
		 * Get the offset of the navigatorImage in the viewer
		 * @returns {ShapeUtils.Size}
		 */
		NavigatorWidget.prototype.getNavigatorImageOffset = function () {
			return this.navigatorImage.offset();
		};

		return NavigatorWidget;
	});

/**
 * @memberof NavigationModule
 */
define('web-viewer/navigation/NavigationConstants',{
	ZoomOneToOneClick: { // mapped to PREFERENCES_ZOOM_REALSIZE
		AUTO: 0, // default
		REALSIZE: 1
	}
});

define('tpl!web-viewer/navigation/AdjustMonitorResolutionPopup', function(){return function(d){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<div class="wcr-form form-labels-on-top"><div class="form-group"><label>'+
((__t=(d.i18n("ZoomOneToOneAdjustResLabel")))==null?'':_.escape(__t))+
' </label><div class="form-entry"><div class="input-group"><input value="'+
((__t=(d.monitorResolution))==null?'':_.escape(__t))+
'" type="text" size="7" name="monitorResolution" class="monitorResolution form-control"></input><span class="input-group-addon"> '+
((__t=(d.i18n(d.resolutionUnitsLangKey)))==null?'':_.escape(__t))+
'</span></div></div></div></div>\r\n\r\n<p>'+
((__t=(d.i18n('ZoomOneToOneAdjustResHint')))==null?'':_.escape(__t))+
'<br />'+
((__t=(d.i18n('ZoomOneToOneAdjustResExample')))==null?'':_.escape(__t))+
'</p>\r\n<p>'+
((__t=(d.i18n('ZoomOneToOneAdjustResMoreInfo')))==null?'':_.escape(__t))+
' <a href="'+
((__t=(d.moreInfoLink))==null?'':_.escape(__t))+
'" target="_blank">'+
((__t=(d.i18n('ZoomOneToOneAdjustResMoreInfoLink')))==null?'':_.escape(__t))+
'</a></p>';
return __p;
}});
define('web-viewer/navigation/AdjustMonitorResolutionPopup',["web-viewer/layout/component/ModalPopup",
		"Template",
		"web-viewer/utils/ViewerUtils",
		"web-viewer/core/CoreConstants",
		"tpl!web-viewer/navigation/AdjustMonitorResolutionPopup",
		"jquery",
		"underscore"
	],
	function (ModalPopup,
		Template,
		ViewerUtils,
		CoreConstants,
		tpl,
		$,
		_) {
		"use strict";
		/**
		 * @exports AdjustMonitorResolutionPopup
		 * @memberof NavigationModule
		 * @augments layout.component.ModelPopup
		 * @class Popup for adjusting monitor resolution
		 * @constructor
		 *
		 * @param {Core.ViewerFacade} f - Viewer facade
		 * @param {String} id - Id of the popup. It is used in its private channel.
		 * @param {NavigationModule} navigationModule - The navigation module
		 */
		var AdjustMonitorResolutionPopup = function (f, id, navigationModule) {
			ModalPopup.call(this, f, id, f.getContext());

			this._navigationModule = navigationModule;

			this.f.i18n.loadBundle("viewer", _.bind(function () {
				this.setHeader($("<span/>").addClass("popupTitle").html(this.f.i18n.t("ZoomOneToOneAdjustResTitle")));

				this.setOkBtnText(this.f.i18n.t("OkButton"));
				this.setOkAction(_.bind(function () {
					this._navigationModule.okMonitorResolution();
				}, this));
				this.setCancelAction(_.bind(function () {
					this._navigationModule.cancelMonitorResolution();
				}, this));
			}, this));

			this.element.addClass("adjustMonitorResolutionPopup");

			/**
			 * @member {Template} template for the popup
			 */
			this.t = new Template(this.f.i18n, tpl);
			this.t.setContext(this._popupContent.addClass("adjustMonitorResolutionPopupContent"));

			// KB article for more info
			this.t.d.moreInfoLink = "https://wiki.esko.com/display/KBA/KB134185394:+WebCenter+-+How+to+determine+the+Monitor+Resolution+for+Zoom+to+Actual+Size+in+the+HTML5+Viewer";

			//action for the input field for the resolution
			this.t.bind("focusout", "[name=monitorResolution]", _.bind(function (source, event) {
				var v = source.value;
				var parts = source.value.split("/");
				if (parts.length == 2) {
					v = ViewerUtils.parseFloat(parts[0]) / ViewerUtils.parseFloat(parts[1]);
				}
				v = ViewerUtils.parseFloat(v);
				if (isNaN(v)) { // if nothing is filled in or only letter(s), keep old valid value (in same unit as at it should be entered)		
					v = ViewerUtils.parseFloat(ViewerUtils.convertResolutionFromPpmmValue(this._navigationModule.getMonitorResolution(), this.f.getResolutionUnits()));
				}

				// convert to ppmm
				var newMonitorResolution = ViewerUtils.parseFloat(ViewerUtils.convertResolutionToPpmmValue(v, this.f.getResolutionUnits()));

				// Put the values back (in right unit)
				source.value = ViewerUtils.convertResolutionFromPpmmValue(newMonitorResolution, this.f.getResolutionUnits(), this.f.getResolutionUnitsFix());

				// set the option
				this._navigationModule.setMonitorResolution(newMonitorResolution);
			}, this));

			// Listen to resolution changes
			this.f.getCurrentUser().on("options.changed.PREFERENCES_UNIT_RESOLUTION", _.bind(function () {
				if (this.isVisible()) {
					this.render();
				}
			}, this));
		};

		AdjustMonitorResolutionPopup.prototype = Object.create(ModalPopup.prototype);
		AdjustMonitorResolutionPopup.prototype.constructor = AdjustMonitorResolutionPopup;

		/**
		 * Renders the popup for channel matching, it's a throttled function
		 */
		AdjustMonitorResolutionPopup.prototype.render = _.throttle(function () {
			this.t.d.monitorResolution = ViewerUtils.convertResolutionFromPpmmValue(this._navigationModule.getMonitorResolution(), this.f.getResolutionUnits(), this.f.getResolutionUnitsFix());
			this.t.d.resolutionUnitsLangKey = this.f.getResolutionUnitsLocName();

			this.t.render();
			if (this._isInitialized) {
				this.recalculateSize(500);
			}
		}, 40);

		/**
		 * Show the modal popup
		 */
		AdjustMonitorResolutionPopup.prototype.show = function () {
			this.render();
			ModalPopup.prototype.show.call(this);
			this.recalculateSize(500);
		};

		return AdjustMonitorResolutionPopup;
	});

/**
 * @namespace NavigationModule
 */
define('web-viewer/navigation/NavigationModule',["underscore",
		"web-viewer/core/AbstractModule",
		"web-viewer/navigation/PanTool",
		"web-viewer/navigation/TouchTool",
		"web-viewer/navigation/TouchPreventerTool",
		"web-viewer/navigation/ZoomTool",
		"web-viewer/navigation/NavigatorWidget",
		"web-viewer/navigation/NavigationConstants",
		"web-viewer/navigation/AdjustMonitorResolutionPopup",
		"component/toolbar/ToolbarComponent",
		"component/toolbar/ButtonGroup",
		"web-viewer/layout/component/ViewerToolbarButton",
		"web-viewer/layout/component/DropdownToolbarButton",
		"web-viewer/layout/LayoutConstants",
		"web-viewer/utils/ViewerUtils",
		"Mousetrap"
	],
	function (
		_,
		AbstractModule,
		PanTool,
		TouchTool,
		TouchPreventerTool,
		ZoomTool,
		NavigatorWidget,
		NavigationConstants,
		AdjustMonitorResolutionPopup,
		ToolbarComponent,
		ButtonGroup,
		ViewerToolbarButton,
		DropdownToolbarButton,
		LayoutConstants,
		ViewerUtils,
		Mousetrap) {
		"use strict";

		/**
		 * Create the NavigationModule
		 *
		 * @exports NavigationModule
		 * @memberof NavigationModule
		 * @class NavigationModule - responsible for publishing the correct tools and toolbarcomponents enabling navigation
		 * @augments Core.AbstractModule
		 * @param {Core.ViewerFacade} f
		 * @param {Boolean} silenced
		 */
		var NavigationModule = function (f, silenced) {
			AbstractModule.call(this, f, silenced, "navigation");

			this.requiresViewportAccess = true;

			this._extraRotation = 0;

			this._initializeTools(f);

			this._publishWidget(NavigatorWidget.widgetName, _.bind(function (layoutType) {
				this._navigatorWidget = new NavigatorWidget(f);
				return this._navigatorWidget;
			}, this));
		};


		// inherit
		NavigationModule.prototype = Object.create(AbstractModule.prototype);
		// correct the constructor pointer 
		NavigationModule.prototype.constructor = NavigationModule;

		/**
		 * Initializes the tools, depending on the layout type.
		 * @param {Core.ViewerFacade} f
		 */
		NavigationModule.prototype._initializeTools = function (f) {
			var layoutType = this.f.getLayoutType();

			if (layoutType === LayoutConstants.LayoutType.MOBILE) {
				var touchTool = new TouchTool(f);
				this._publishTool(touchTool);
				touchTool.requestActivation();
			} else {
				var panTool = new PanTool(f);
				var zoomTool = new ZoomTool(f);
				var noTouchTool = new TouchPreventerTool(f);
				this._publishTool(zoomTool);
				this._publishTool(panTool);
				this._publishTool(noTouchTool);


				this._publishToolbarComponent(LayoutConstants.ToolbarComponent.NAVIGATE, _.bind(function (layoutType) {
					/**
					 * Component: Navigate
					 */
					var navigateComponent = new ToolbarComponent(LayoutConstants.ToolbarComponent.NAVIGATE, f);


					/**
					 * Group: zoom + pan
					 */
					var navigateGroup = new ButtonGroup("NavigateGroup");
					navigateGroup.setTitle(f.i18n, "viewer", "NavBtnGroupTitle");
					navigateComponent.addButtonGroup(navigateGroup);


					// Zoom button
					var zoomButton = new ViewerToolbarButton(f, "ZoomButton", "icon icon-WCR_Zoom", false);
					zoomButton.setTooltip("viewer", "ZoomBtnTT");
					zoomButton.listenToTool(zoomTool);
					zoomButton.element.on("dblclick", function (e) {
						f.emit("viewport.zoomToFit", {
							animated: true
						});
						e.stopPropagation();
					});
					navigateGroup.addButton(zoomButton);


					// ZoomIn button
					var zoomInButton = new ViewerToolbarButton(f, "ZoomInButton", "icon icon-WCR_WF-ZoomIn", false, function () {
						f.emit("viewport.transform", {
							zoom: 1,
							animated: true
						});
					});
					zoomInButton.setTooltip("viewer", "ZoomInCursor");
					navigateGroup.addButton(zoomInButton);


					// ZoomOut button
					var zoomOutButton = new ViewerToolbarButton(f, "ZoomOutButton", "icon icon-WCR_WF-ZoomOut", false, function () {
						f.emit("viewport.transform", {
							zoom: -1,
							animated: true
						});
					});
					zoomOutButton.setTooltip("viewer", "ZoomOutCursor");
					navigateGroup.addButton(zoomOutButton);


					// ZoomToFit button
					var zoomToFitButton = new DropdownToolbarButton(f, "ZoomToFitButton", "icon icon-WCR_WF-ZoomFit", false, function () {
						f.emit("viewport.zoomToFit", {
							animated: true
						});
					});
					zoomToFitButton.setTooltip("viewer", "ZoomFitToPage");
					navigateGroup.addButton(zoomToFitButton);


					// ZoomOneToOne button
					this._zoomOneToOneButton = new DropdownToolbarButton(f, "ZoomOneToOne", "icon icon-WCR_Zoom1", false, _.bind(this._zoomOneToOne, this));
					navigateGroup.addButton(this._zoomOneToOneButton);

					// Pan button
					var panButton = new ViewerToolbarButton(f, "PanButton", "icon icon-WCR_Pan", false);
					panButton.setTooltip("viewer", "PanBtnTT");
					panButton.listenToTool(panTool);
					navigateGroup.addButton(panButton);



					/**
					 * Group: rotate + flip
					 */
					var rotFlipGroup = new ButtonGroup("RotFlipGroup");
					rotFlipGroup.setTitle(f.i18n, "viewer", "RotateBtnGroupTitle");
					navigateComponent.addButtonGroup(rotFlipGroup);

					// Rotate button
					this._rotateButton = new ViewerToolbarButton(f, "RotateButton", "icon icon-WCR_Orientation-Rotate1", true, _.bind(function (event) {
						var rotation = 90;
						if (event.shiftKey) {
							rotation = -90;
						}
						this.f.emit("viewport.transform", {
							rotate: rotation,
							animated: {
								interrupt: true
							}
						});
					}, this));
					rotFlipGroup.addButton(this._rotateButton);



					// Flip button
					this._flipButton = new ViewerToolbarButton(f, "FlipButton", "icon icon-WCR_Orientation-Mirror", true, _.bind(function () {
						this._doToggleFlip();
					}, this));
					rotFlipGroup.addButton(this._flipButton);

					this.f.loadBundle("viewer", function () {
						/**
						 * Zoom to Fit: submenu
						 */
						// zoomFitWidth
						zoomToFitButton.addAction({
							name: 'zoomFitWidth',
							content: f.i18n.t("ZoomFitWidth"),
							icon: "icon icon-WCR_Zoom-FitWidth",
							action: function () {
								f.emit("viewport.zoomToFitWidth", {
									animated: true
								});
							}
						});
						// zoomFitWHeight
						zoomToFitButton.addAction({
							name: 'zoomFitHeight',
							content: f.i18n.t("ZoomFitHeight"),
							icon: "icon icon-WCR_Zoom-FitHeight",
							action: function () {
								f.emit("viewport.zoomToFitHeight", {
									animated: true
								});
							}
						});

						/**
						 * Zoom One To One: submenu
						 */
						this._zoomOneToOneButton.addAction({
							name: 'zoomOneToOnePixel',
							content: f.i18n.t("ZoomOneToOnePixel"),
							icon: "icon icon-WCR_ZoomPx",
							action: _.bind(this._zoomOneToOnePixel, this)
						});
						this._zoomOneToOneButton.addAction({
							name: 'zoomOneToOneRealSize',
							content: f.i18n.t("ZoomOneToOneRealSize"),
							icon: "icon icon-WCR_Zoom1",
							action: _.bind(this._zoomOneToOneRealSize, this)
						});
						this._zoomOneToOneButton.addDivider();
						this._zoomOneToOneButton.addAction({
							name: 'adjustMonitorResolution',
							content: f.i18n.t("ZoomOneToOneAdjustRes"),
							action: _.bind(this._adjustMonitorResolution, this)
						});

						this._updateZoomOneToOneButton();

						// for further updates
						this.f.getCurrentUser().on("options.changed.PREFERENCES_ZOOM_REALSIZE", _.bind(this._updateZoomOneToOneButton, this));
						this.f.getDocumentController().whenActivePageChanged(_.bind(this._updateZoomOneToOneButton, this));

						this._updateRotateButton();
						this.f.on("transform.viewportWidget.rotated", _.bind(this._updateRotateButton, this));

						this._updateFlipButton();
						this.f.on("transform.viewportWidget.flipped", _.bind(this._updateFlipButton, this));
					}, this);

					return navigateComponent;
				}, this));

				// Keyboard shortcuts
				Mousetrap.bind("ctrl+m", _.bind(function (e) {
					this._doToggleFlip();

					e.preventDefault();
					e.stopPropagation();
				}, this), "keydown");

				Mousetrap.bind(["ctrl+alt+plus", "ctrl+alt+="], _.bind(function (e) {
					var rotation = 90;

					this.f.emit("viewport.transform", {
						rotate: rotation,
						animated: {
							interrupt: true
						}
					});

					e.preventDefault();
					e.stopPropagation();
				}, this));

				Mousetrap.bind("ctrl+alt+-", _.bind(function (e) {
					var rotation = -90;

					this.f.emit("viewport.transform", {
						rotate: rotation,
						animated: {
							interrupt: true
						}
					});

					e.preventDefault();
					e.stopPropagation();
				}, this));
			}
		};

		/**
		 * Toggle the flip state of the viewport
		 */
		NavigationModule.prototype._doToggleFlip = function () {
			var viewportModule = this.f.getModule("viewport");
			if (viewportModule) {
				var rotation = viewportModule.getRotation();
				if (rotation % 180 === 0) {
					// not rotated
					this.f.emit("viewport.transform", {
						flip: true,
						animated: false
					});
				} else {
					// rotated
					this.f.emit("viewport.transform", {
						rotate: 180,
						flip: true,
						animated: false
					});
				}
			}
		};

		/**
		 * Update the rotation button on rotate events
		 */
		NavigationModule.prototype._updateRotateButton = function () {
			var rotationClasses = ["icon icon-WCR_Orientation-Rotate1", "icon icon-WCR_Orientation-Rotate2", "icon icon-WCR_Orientation-Rotate3", "icon icon-WCR_Orientation-Rotate4"];
			var rotationTTs = ["Rotate90TT", "Rotate180TT", "Rotate270TT", "Rotate0TT"];
			var viewportModule = this.f.getModule("viewport");
			if (viewportModule && this._rotateButton) {
				var rotation = viewportModule.getRotation() + this._extraRotation;
				var i = (rotation / 90) % 4;
				this._rotateButton.setIcon(rotationClasses[i]);
				this._rotateButton.setTooltip("viewer", rotationTTs[i]);

			}
		};

		/**
		 * Set an extra rotation to the icon in the top toolbar
		 * @param {Number} extraRotation
		 */
		NavigationModule.prototype.setExtraRotation = function (extraRotation) {
			this._extraRotation = extraRotation;
			this._updateRotateButton();
		};

		/**
		 * Reset the extra rotation for the rotation icon
		 */
		NavigationModule.prototype.resetExtraRotation = function () {
			this._extraRotation = 0;
			this._updateRotateButton();
		};

		/**
		 * Update the flip button on flip events
		 */
		NavigationModule.prototype._updateFlipButton = function () {
			var viewportModule = this.f.getModule("viewport");
			if (viewportModule && this._flipButton) {
				if (viewportModule.isFlip()) {
					// on
					this._flipButton.setIcon("icon icon-WCR_Orientation-MirrorReverse");
					this._flipButton.setTooltip("viewer", "MirrorTT2");
				} else {
					// off
					this._flipButton.setIcon("icon icon-WCR_Orientation-Mirror");
					this._flipButton.setTooltip("viewer", "MirrorTT");
				}
			}
		};

		/**
		 * Check whether we have to zoom one to one pixel or not
		 * @returns {Boolean}
		 */
		NavigationModule.prototype._isZoomOneToOnePixel = function () {
			// only valid for graphics and if we need to
			return this.f.getActivePage().hasGraphics() && this.f.getOption("PREFERENCES_ZOOM_REALSIZE") === NavigationConstants.ZoomOneToOneClick.AUTO;
		};

		/**
		 * Called when the zoom one to one button behavior should be updated
		 */
		NavigationModule.prototype._updateZoomOneToOneButton = function () {
			if (this._zoomOneToOneButton) {
				if (this._isZoomOneToOnePixel()) {
					this._zoomOneToOneButton.setTooltip("viewer", "ZoomOneToOnePixel");
					this._zoomOneToOneButton.setIcon("icon icon-WCR_ZoomPx");
				} else {
					this._zoomOneToOneButton.setTooltip("viewer", "ZoomOneToOneRealSize");
					this._zoomOneToOneButton.setIcon("icon icon-WCR_Zoom1");
				}
				if (this.f.getActivePage().hasGraphics()) {
					this._zoomOneToOneButton.setActionEnabled("zoomOneToOnePixel", true);
				} else {
					this._zoomOneToOneButton.setActionEnabled("zoomOneToOnePixel", false);
				}
			}
		};

		/**
		 * Called when user wants to adjust monitor resolution
		 */
		NavigationModule.prototype._adjustMonitorResolution = function () {
			if (this._adjustMonitorResolutionPopup === undefined) {
				this._adjustMonitorResolutionPopup = new AdjustMonitorResolutionPopup(this.f, "adjustMonitorResulutionPopup", this);
			}
			this._adjustMonitorResolutionPopup.show();
		};

		/**
		 * Zoom 1:1: called when toolbar button clicked
		 */
		NavigationModule.prototype._zoomOneToOne = function () {
			if (this._isZoomOneToOnePixel()) {
				this._zoomOneToOnePixel();
			} else {
				this._zoomOneToOneRealSize();
			}
		};

		/**
		 * Zoom 1:1 Pixel: called when button clicked in menu/toolbar
		 */
		NavigationModule.prototype._zoomOneToOnePixel = function () {
			if (this.f.getActivePage().hasGraphics()) {
				// Shouldn't be possible for CAD (disabled), but extra check 
				this.f.setOption("PREFERENCES_ZOOM_REALSIZE", NavigationConstants.ZoomOneToOneClick.AUTO);
			}
			this.f.emit("viewport.zoomOneToOne", {
				animated: true
			});
		};

		/**
		 * Zoom 1:1 Real Size: called when button clicked in menu/toolbar
		 */
		NavigationModule.prototype._zoomOneToOneRealSize = function () {
			if (this.f.getActivePage().hasGraphics()) {
				// Only save the preference for Graphic files
				this.f.setOption("PREFERENCES_ZOOM_REALSIZE", NavigationConstants.ZoomOneToOneClick.REALSIZE);
			}
			this.f.emit("viewport.zoomOneToOne", {
				animated: true,
				worldResolution: this.getMonitorResolution()
			});
		};

		/**
		 * The monitor resolution in ppmm
		 * @returns {Number}
		 */
		NavigationModule.prototype.getMonitorResolution = function () {
			if (this._monitorResolution !== undefined) {
				return this._monitorResolution;
			} else {
				return ViewerUtils.parseFloat(this.f.getOption("PREFERENCE_MONITOR_RES"));
			}
		};

		/**
		 * Set the new monitor resolution (in ppmm)
		 * @param {Number} monitorResolution
		 */
		NavigationModule.prototype.setMonitorResolution = function (monitorResolution) {
			// keep it temporary available and only save on OK
			this._monitorResolution = monitorResolution;
		};

		/**
		 * Called when the ok button of the monitor resolution popup is clicked
		 */
		NavigationModule.prototype.okMonitorResolution = function () {
			this.f.setOption("PREFERENCE_MONITOR_RES", this.getMonitorResolution() + "");
			this._monitorResolution = undefined;
			this._zoomOneToOneRealSize();
			this._adjustMonitorResolutionPopup.hide();
		};

		/**
		 * Called when the cancel button of the monitor resolution popup is clicked
		 */
		NavigationModule.prototype.cancelMonitorResolution = function () {
			this._monitorResolution = undefined;
		};

		return NavigationModule;
	});

/**
 * @memberof Pagelist
 */
define('web-viewer/pagelist/PagelistConstants',{
	viewerThumbSize: {
		SMALL: 0,
		MEDIUM: 1,
		LARGE: 2
	},
	thumbWidth: {
		SMALL: 75,
		MEDIUM: 100,
		LARGE: 150
	},
	binding: {
		RIGHT: 0,
		LEFT: 1
	}
});

/**
 * @memberof PrintModule 
 */
define('web-viewer/print/PrintConstants',{
	PrintType: {
		PRINTCURRENTVIEW: 0,
		PRINTIMAGEAREA: 1,
		PRINTANNOTATIONOVERVIEW: 2
	},
	PrintArea: {
		PRINTENTIREIMAGE: 0,
		PRINTCURRENTVIEWAREA: 1,
		PRINTCURRENTSELECTION: 2
	},
	PrintChannels: {
		PRINTALLCHANNELS: 0,
		PRINTSELECTEDCHANNELS: 1
	},
	PrintAnnotationsSource: {
		PRINTANNOTATIONSCURRENT: 0,
		PRINTANNOTATIONSREFERENCE: 1
	}
});

define('web-viewer/utils/LineworkPoint',["underscore"],
	function (_, c) {
		/**
		 * @memberof Utils
		 * @exports LineworkPoint
		 * @class Describes a linework point and how the linework point is reached from the previous point
		 */
		var LineworkPoint = function () {
			this.type = -1;
			this.x = 0;
			this.y = 0;

			// the values depend on the type
			this.v1 = 0;
			this.v2 = 0;
			this.v3 = 0;
			this.v4 = 0;
			this.v5 = 0;
		};

		/**
		 * Move to the point, no extra parameters are specified, there is no line between the point and the previous point
		 * @param {Number} x
		 * @param {Number} y
		 */
		LineworkPoint.prototype.makeMoveTo = function (x, y) {
			this.type = 0;
			this.x = x;
			this.y = y;
		};

		/**
		 * Makes a straight line to the point
		 * @param {Number} x
		 * @param {Number} y
		 */
		LineworkPoint.prototype.makeLineTo = function (x, y) {
			this.type = 1;
			this.x = x;
			this.y = y;
		};

		/**
		 * Makes a bezier curve to the point, it needs two control points to calculate the line between the points
		 * @param {Number} x - x-coordinate to move to
		 * @param {Number} y - y-coordinate to move to
		 * @param {Number} x1 - x-coordinate for the first control point
		 * @param {Number} y1 - y-coordinate for the first control point
		 * @param {Number} x2 - x-coordinate for the second control point
		 * @param {Number} y2 - y-coordinate for the second control point
		 */
		LineworkPoint.prototype.makeBezier = function (x, y, x1, y1, x2, y2) {
			this.type = 2;
			this.x = x;
			this.y = y;
			this.v1 = x1;
			this.v2 = y1;
			this.v3 = x2;
			this.v4 = y2;
		};

		/**
		 * Makes a curve to the point, it needs extra parameters to define the arc
		 * @param {Number} x - x-coordinate to move to
		 * @param {Number} y - y-coordinate to move to
		 * @param {Number} rx - radius of the circle
		 * @param ry - not used
		 * @param {Number} angle - Specifies whether the drawing should be counterclockwise or clockwise. true indicates counter-clockwise.
		 * @param largeArc - not used
		 * @param sweepFlag - not used
		 */
		LineworkPoint.prototype.makeArc = function (x, y, rx, ry, angle, largeArc, sweepFlag) {
			this.type = 3;
			this.x = x;
			this.y = y;
			this.v1 = rx;
			this.v2 = ry;
			this.v3 = angle;
			this.v4 = largeArc;
			this.v5 = sweepFlag;
		};

		/**
		 * Make a nick
		 * @param {Number} x - x-coordinate to move to
		 * @param {Number} y - y-coordinate to move to
		 * @param {Number} v1 - number of the nick that will get drawn
		 * @param {Number} v2 - y-distance for the second point used to draw the nick
		 * @param {Number} v3 - x-distance for the second point used to draw the nick
		 */
		LineworkPoint.prototype.makeNick = function (x, y, v1, v2, v3) {
			this.type = 4;
			this.x = x;
			this.y = y;
			this.v1 = v1;
			this.v2 = v2;
			this.v3 = v3;
		};

		return LineworkPoint;
	});

define('web-viewer/utils/LineWorkWrapper',["underscore",
		"utils/ShapeUtils",
		"web-viewer/core/Extendable",
		"web-viewer/utils/DrawUtils",
		"web-viewer/utils/LineworkPoint"
	],
	function (_,
		ShapeUtils,
		Extendable,
		DrawUtils,
		LineworkPoint) {
		"use strict";

		/**
		 * @memberof Utils
		 * @exports LineworkWrapper
		 * @class
		 * A lineworkwrapper is used to draw linework
		 * @constructor
		 */
		var LineworkWrapper = function () {
			Extendable.installTo(this);

			this.color = null;
			this.c = null;
			this.fillcolor = null;
			this.pointArray = [];
			this.rect = new ShapeUtils.Rect();
			this.type = 0;
			this.linewidth = 1;
			this.imagehotspot = null;
			this.imagetouchspot = null;
			this.imageRef = null;

			this.treatAsOutline = false; //treat the object as an outline object for selecting

			this.textoptions = {};
			this.selected = false;

			this.showIndicator = false;
			this.indicatorText = null;
			this.indicatorTextColor = null;
			this.indicatorColor = null;

			this.snapable = false; //if snapable, the user can snap to the points of the wrapper	
		};

		/**
		 * Move the lineworkwrapper over a size
		 * @param {ShapeUtils.Size} size
		 */
		LineworkWrapper.prototype.move = function (size) {
			this.rect.origin.x += size.width;
			this.rect.origin.y += size.height;
			if (this.pointArray && this.pointArray.length > 0) {
				for (var j = 0; j < this.pointArray.length; j++) {
					var point = this.pointArray[j];
					point.x += size.width;
					point.y += size.height;
				}
			}
		};

		/**
		 * Sets the rectangle of the lineworkwrapper
		 * @param {ShapeUtils.Rect} rect
		 * @param {Boolean} updateStartEnd
		 */
		LineworkWrapper.prototype.setRect = function (rect, updateStartEnd) {
			if (updateStartEnd) {
				if (this.pointArray && this.pointArray.length == 2) {
					var p1 = rect.topleft();
					var p2 = rect.bottomright();

					if ((this.pointArray[0].x == p1.x && this.pointArray[0].y == p1.y) || this.pointArray[0].x == this.pointArray[1].x && this.pointArray[0].y == this.pointArray[1].y) {
						this.pointArray[0].x = p1.x;
						this.pointArray[0].y = p1.y;

						this.pointArray[1].x = p2.x;
						this.pointArray[1].y = p2.y;
					} else {
						this.pointArray[1].x = p1.x;
						this.pointArray[1].y = p1.y;

						this.pointArray[0].x = p2.x;
						this.pointArray[0].y = p2.y;
					}
				}
			}
			this.rect = rect;
		};

		/**
		 * Set the start and the end point for the lineworkwrapper
		 * @param {ShapeUtils.Point} start
		 * @param {ShapeUtils.Point} end
		 */
		LineworkWrapper.prototype.setStartEnd = function (start, end) {
			if (this.pointArray && this.pointArray.length == 2) {
				if (start) {
					this.pointArray[0].x = start.x;
					this.pointArray[0].y = start.y;
				} else {
					start = new ShapeUtils.Point(this.pointArray[0].x, this.pointArray[0].y);
				}
				if (end) {
					this.pointArray[1].x = end.x;
					this.pointArray[1].y = end.y;
				} else {
					end = new ShapeUtils.Point(this.pointArray[1].x, this.pointArray[1].y);
				}

				this.setRect(new ShapeUtils.Rect(start, start.sizeTo(end)));
			}
		};

		/**
		 * Get the rgba color of the line work with alpha
		 * @param {Number} alpha
		 * @returns {String}
		 */
		LineworkWrapper.prototype.rgbaColor = function (alpha) {
			var rgb = this.rgbColor();
			return "rgba(" + rgb.r + "," + rgb.g + "," + rgb.b + "," + alpha + ")";
		};

		/**
		 * Get the rgb color
		 * @returns {Object}
		 */
		LineworkWrapper.prototype.rgbColor = function () {
			var result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(this.color);
			return result ? {
				r: parseInt(result[1], 16),
				g: parseInt(result[2], 16),
				b: parseInt(result[3], 16)
			} : null;
		};

		/**
		 * Calculate the rectangle from all the points of the point array of the linework
		 */
		LineworkWrapper.prototype.calculateRectFromPointArray = function () {
			var rectx1 = null;
			var rectx2 = null;
			var recty1 = null;
			var recty2 = null;

			for (var i = 0; i < this.pointArray.length; i++) {
				var p = this.pointArray[i];

				if (!rectx1 || p.x < rectx1) {
					rectx1 = p.x;
				}
				if (!rectx2 || p.x > rectx2) {
					rectx2 = p.x;
				}
				if (!recty1 || p.y < recty1) {
					recty1 = p.y;
				}
				if (!recty2 || p.y > recty2) {
					recty2 = p.y;
				}
			}
			var r = new ShapeUtils.Rect(new ShapeUtils.Point(rectx1, recty1), new ShapeUtils.Size(rectx2 - rectx1, recty2 - recty1));
			r.normalize();
			this.rect = r;
		};

		/**
		 * Set the selection anchor point
		 * @param {ShapeUtils.Point} point
		 * @param fillColor
		 * @param strokeColor
		 * @param subtype
		 * @returns {LineworkWrapper}
		 */
		LineworkWrapper.prototype.selectionAnchorPoint = function (point, fillColor, strokeColor, subtype) {
			var anchor = new LineworkWrapper();

			anchor.type = DrawUtils.LWTYPE_ELLIPSE;
			anchor.color = strokeColor;
			anchor.fillcolor = fillColor;
			anchor.staticsize = true;
			anchor.linewidth = 1;
			var size = 10;
			anchor.rect = new ShapeUtils.Rect(new ShapeUtils.Point(point.x - size / 2, point.y - size / 2), new ShapeUtils.Size(size, size));
			anchor.anchor = subtype;
			anchor.linework = this;

			return anchor;
		};

		/**
		 * Get the selection box of the linework wrapper
		 * @returns {Array.<LineworkWrapper>}
		 */
		LineworkWrapper.prototype.getSelectionBox = function () {
			var selectionFillColor = 'rgb(153,152,187)';
			var selectionStrokeColor = 'white';

			if (this.pointArray && this.pointArray.length >= 2) {
				return [
					this.selectionAnchorPoint(new ShapeUtils.Point(this.pointArray[0].x, this.pointArray[0].y), selectionFillColor, selectionStrokeColor, DrawUtils.LWANCHOR_START),
					this.selectionAnchorPoint(new ShapeUtils.Point(this.pointArray[this.pointArray.length - 1].x, this.pointArray[this.pointArray.length - 1].y), selectionFillColor, selectionStrokeColor, DrawUtils.LWANCHOR_END)
				];
			} else {
				var anchors = [
					this.selectionAnchorPoint(this.rect.topleft(), selectionFillColor, selectionStrokeColor, DrawUtils.LWANCHOR_TOPLEFT),
					this.selectionAnchorPoint(this.rect.topright(), selectionFillColor, selectionStrokeColor, DrawUtils.LWANCHOR_TOPRIGHT),
					this.selectionAnchorPoint(this.rect.bottomleft(), selectionFillColor, selectionStrokeColor, DrawUtils.LWANCHOR_BOTTOMLEFT),
					this.selectionAnchorPoint(this.rect.bottomright(), selectionFillColor, selectionStrokeColor, DrawUtils.LWANCHOR_BOTTOMRIGHT),
					this.selectionAnchorPoint(this.rect.topcenter(), selectionFillColor, selectionStrokeColor, DrawUtils.LWANCHOR_TOPCENTER),
					this.selectionAnchorPoint(this.rect.bottomcenter(), selectionFillColor, selectionStrokeColor, DrawUtils.LWANCHOR_BOTTOMCENTER),
					this.selectionAnchorPoint(this.rect.centerleft(), selectionFillColor, selectionStrokeColor, DrawUtils.LWANCHOR_CENTERLEFT),
					this.selectionAnchorPoint(this.rect.centerright(), selectionFillColor, selectionStrokeColor, DrawUtils.LWANCHOR_CENTERRIGHT)
				];

				return _.union(anchors);
			}
		};

		/**
		 * Scale the linework wrapper with a given factor (both horizontal as vertical)
		 * @param {ShapeUtils.Point} scaleFactor
		 */
		LineworkWrapper.prototype.scale = function (scaleFactor) {
			if (scaleFactor) {
				_.each(this.pointArray, function (point) {
					point.x *= scaleFactor.x;
					point.y *= scaleFactor.y;
				});

				this.rect.origin.x *= scaleFactor.x;
				this.rect.origin.y *= scaleFactor.y;
				this.rect.size.width *= scaleFactor.x;
				this.rect.size.height *= scaleFactor.y;
			}
		};

		return LineworkWrapper;
	});

define('web-viewer/core/AbstractConfig',["web-viewer/data/AbstractDataAdapter",
		"web-viewer/core/Extendable"
	],
	function (AbstractDataAdapter, Extendable) {
		"use strict";
		/**
		 * Create new AbstractConfig
		 * 
		 * @exports AbstractConfig
		 * @memberof Core
		 * @class Configuration for Web Viewer. A configuration provides the correct DataAdapter (to interact with the server) and sets up the debugger
		 */
		var AbstractConfig = function () {
			Extendable.installTo(this);

			/**
			 * @private
			 */
			this.properties = {};
		};

		/**
		 * Set a property in the config
		 * 
		 * @param {String} key - The name of the property
		 * @param {Any} value - The value of the property
		 */
		AbstractConfig.prototype.set = function (key, value) {
			this.properties[key] = value;
		};

		/**
		 * Get a property from the config
		 * 
		 * @param {String} key - The name of the property for which we want the value
		 * @returns {Any} The value of the property
		 */
		AbstractConfig.prototype.get = function (key) {
			return this.properties[key];
		};


		/**
		 * Initialize the DataAdapter
		 * 
		 * (This method should be overridden by the a specific DataAdapter)
		 * 
		 * @param {Core.ViewerFacade} f
		 */
		AbstractConfig.prototype.initializeAdapter = function (f) {
			this.adapter = new AbstractDataAdapter(f);

			return this.adapter;
		};

		/**
		 * Get the user who is logged in
		 * 
		 * @returns {User} the currently logged in user
		 */
		AbstractConfig.prototype.getUser = function () {
			return undefined;
		};


		/**
		 * Get the modules to load 
		 * 
		 * @return {Array.<String>} array of modules to load
		 */
		AbstractConfig.prototype.getModules = function () {
			return [];
		};

		return AbstractConfig;
	});

/**
 * @memberof ViewControlsModule 
 */
define('web-viewer/viewControls/ViewControlsConstants',{
	compensateDist: {
		DISTONLY: 1,
		SCALEONLY: 2,
		DISTANDSCALE: 3
	},
	trimboxMode: {
		ONCURRENTPAGE: 0,
		ONBOTHINREADERSPREAD: 1
	},
	marginClipMode: {
		CLIP_OFF: 0,
		CLIP_IN_READERSPREAD: 1,
		CLIP: 2
	}
});

define('web-viewer/webcenter/WebCenterDocumentPage',["web-viewer/data/AbstractDocumentPage",
		"utils/XMLUtils",
		"web-viewer/utils/ViewerUtils",
		"utils/ShapeUtils",
		"web-viewer/data/DataConstants",
		"web-viewer/viewControls/ViewControlsConstants",
		"underscore"
	],
	function (AbstractDocumentPage,
		XMLUtils,
		ViewerUtils,
		ShapeUtils,
		DataConstants,
		ViewControlsConstants,
		_) {
		"use strict";

		/**
		 * @exports WebCenterDocumentPage
		 * @augments Data.AbstractDocumentPage
		 * @memberof WebCenter
		 * @constructor
		 *
		 * @param {Core.ViewerFacade} f
		 * @param {AbstractDocumentVersion} docVersion
		 * @param {Number} pagenum
		 * @param {Object} info
		 * @param {AbstractDataAdapter} adapter
		 */
		var WebCenterDocumentPage = function (f, docVersion, pagenum, fileInfo, info, adapter) {
			AbstractDocumentPage.call(this, f, docVersion, pagenum);

			/**
			 * @private
			 */
			this._adapter = adapter;

			//check what type the document is before initializing.
			this.documentType = this.getDocument().getDocumentType();

			this._pageVersionId = null;

			if (info !== undefined && info.pageindex !== undefined) { //only for pagelist documents 
				this._pageVersionId = info.docversion;
			}

			if (info !== undefined && info.blank !== undefined) {
				this._isBlankPage = true;
			}


			if (info !== undefined) {
				this.set("canChangeAnnotations", info.can_change_annotations);
				this.set("canEditAttributes", info.can_edit_attributes);
				this.set("attributeCategoryID", info.attribute_category_id);
			}

			if ((this.documentType === DataConstants.documentPageType.GRAPHICS) && fileInfo !== undefined && info !== undefined) {
				//set the correct data coming from the server
				this.set("hasDefaultData", info.hasDefaultData);
				this.set("margbot", info.margbot);
				this.set("margtop", info.margtop);
				this.set("margleft", info.margleft);
				this.set("margright", info.margright);
				this.set("vsize", info.vsize);
				this.set("hsize", info.hsize);
				this.set("hasCorrectMargin", !docVersion.isPageList());
				this.set("docversionid", fileInfo.docversionid);
				this.set("pageDocversionid", info.docversionid);
				if (fileInfo.file === undefined) {
					this.set("file", info.file);
				} else {
					this.set("file", fileInfo.file);
				}
				this.set("offsetx", info.offsetx);
				this.set("offsety", info.offsety);
				this.set("pageindex", info.pageindex);
				this.set("pagename", info.pagename);
				this.set("hscale", info.hscale);
				this.set("vscale", info.vscale);
				this.set("hdist", info.hdist);
				this.set("vdist", info.vdist);

				this.set("has3DPerspective", fileInfo.has3DPerspective == "1" ? true : false);
				this.set("canGenerate3D", fileInfo.canGenerate3D == "1" ? true : false);

				this.set("document_approval_code", info.document_approval_code);
				this.set("user_approval_code", info.user_approval_code);
				if (info.cadregedit !== undefined) {
					this.set("cadregedit", info.cadregedit);
					this.set("isCadAlignmentSaved", info.saved == 1 ? true : false);
				}
			} else if (this.documentType === DataConstants.documentPageType.CAD && fileInfo !== undefined && info !== undefined) {
				this.set("cad", fileInfo.file);
				this.set("pageDocversionid", info.docversionid);
				if (_.isArray(info.style)) {
					this.set("style", info.style[0]);
				} else {
					this.set("style", info.style);
				}
				if (info.metadata && (info.metadata.design || info.metadata.layout)) {
					var cadData = info.metadata.design || info.metadata.layout;
					this.set("grainDirection", cadData.grain_direction);
					this.set("defaultSide", cadData.default_side);
					this.set("dieSide", cadData.die_side);
				}
				//for Graphics + CAD
				this.setPageAlignmentData(info);
				this.set("docversionid", fileInfo.docversionid);
				this.set("has3DPerspective", fileInfo.has3DPerspective == "1" ? true : false);
				this.set("canGenerate3D", fileInfo.canGenerate3D == "1" ? true : false);

			} else if (fileInfo !== undefined && fileInfo.file !== undefined) { //for pages with only the file as data (multipage or pagelist)
				this.set("file", fileInfo.file);
				this._pageVersionId = fileInfo.docversion;
				this.set("hasCorrectMargin", false);
			}

			if (info !== undefined) {
				this.set("diffversion", info.diffversion ? true : false);
			}
		};

		WebCenterDocumentPage.prototype = Object.create(AbstractDocumentPage.prototype);
		WebCenterDocumentPage.prototype.constructor = WebCenterDocumentPage;

		/**
		 * Set the alignment data for this page
		 * @param {object} alignData
		 * @example
		 *  {
		 *      offsetx: ,
		 *      offsety: ,
		 *      orientation:
		 *  }
		 */
		WebCenterDocumentPage.prototype.setPageAlignmentData = function (alignData) {
			this.set("offsetx", alignData.offsetx);
			this.set("offsety", alignData.offsety);
			if (ViewerUtils.isCloseTo(this._data.offsetx, 0, 7)) { //viewserver returns other values for close to 0
				this.set("offsetx", 0); //set to 0 if close to 0
			}
			if (ViewerUtils.isCloseTo(this._data.offsety, 0, 7)) {
				this.set("offsety", 0);
			}
			// parse orientation info
			// ViewServer uses a bitmask: 
			//      first two bits: rotation (0=0, 1=270, 2=180, 3=180)
			//      third bit: flipped
			if (alignData.orientation !== undefined) {
				this.set("overlayIsFlipped", (+alignData.orientation & 4) > 0);
				this.set("overlayRotation", ((4 - (alignData.orientation % 4)) * 90) % 360);
			}
		};

		/**
		 * Get the margin of the document
		 * @returns {Object} margins{bottom, top, left, right}
		 */
		WebCenterDocumentPage.prototype.getMargins = function () {
			var margins = {};

			margins.bottom = ViewerUtils.parseFloat(this._data['margbot'] ? this._data['margbot'] : 0);
			margins.top = ViewerUtils.parseFloat(this._data['margtop'] ? this._data['margtop'] : 0);
			margins.left = ViewerUtils.parseFloat(this._data['margleft'] ? this._data['margleft'] : 0);
			margins.right = ViewerUtils.parseFloat(this._data['margright'] ? this._data['margright'] : 0);

			return margins;
		};

		/**
		 * Get the origin of the document page, calculates this from the trimbox origin and the margins
		 * @returns {ShapeUtils.Point}
		 */
		WebCenterDocumentPage.prototype.getDocumentPageOrigin = function () {
			var rotation = this.getRotation() % 360;
			var isFlipped = this.isFlipped();
			var margin = this.getMargins();

			var left = this.getParam("fileminx");
			var top = this.getParam("fileminy");
			if (rotation === 90) {
				left -= margin.bottom;
				if (isFlipped) {
					top -= margin.right;
				} else {
					top -= margin.left;
				}

			} else if (rotation === 180) {
				top -= margin.bottom;
				if (isFlipped) {
					left -= margin.left;
				} else {
					left -= margin.right;
				}
			} else if (rotation === 270) {
				left -= margin.top;
				if (isFlipped) {
					top -= margin.left;
				} else {
					top -= margin.right;
				}
			} else {
				top -= margin.top;
				if (isFlipped) {
					left -= margin.right;
				} else {
					left -= margin.left;
				}
			}
			return new ShapeUtils.Point(left, top);
		};

		/**
		 * get the trimbox rectangle for the page
		 * @returns {ShapeUtils.Rect}
		 */
		WebCenterDocumentPage.prototype.getTrimboxRect = function () {
			var origin, size;
			if (this.isBlankPage()) {
				var margins = this.getMargins();
				origin = new ShapeUtils.Point(margins.left, margins.top);
				size = new ShapeUtils.Size(ViewerUtils.parseFloat(this.get("hsize")), ViewerUtils.parseFloat(this.get("vsize")));
			} else {
				origin = new ShapeUtils.Point(ViewerUtils.parseFloat(this.getParam("fileminx")), ViewerUtils.parseFloat(this.getParam("fileminy")));
				size = new ShapeUtils.Size(ViewerUtils.parseFloat(this.getParam("filemaxx")) - origin.x, ViewerUtils.parseFloat(this.getParam("filemaxy")) - origin.y);
			}
			return new ShapeUtils.Rect(origin, size);
		};

		/**
		 * Return the size of the document page
		 * @returns {ShapeUtils.Size}
		 */
		WebCenterDocumentPage.prototype.getPageSize = function () {
			var margins = this.getMargins();

			if (this.hasGraphics()) {
				// graphics document
				if (Math.abs(ViewerUtils.parseFloat(this.get("hsize"))) > 0.000000001) { // float, can't compare safely with 0
					var graphicsSize = new ShapeUtils.Size(ViewerUtils.parseFloat(this.get("hsize")) + margins.left + margins.right, ViewerUtils.parseFloat(this.get("vsize")) + margins.bottom + margins.top);
					var scaling = this.getCurrentScaling();
					if (scaling) {
						graphicsSize.width *= scaling.x;
						graphicsSize.height *= scaling.y;
					}

					return graphicsSize;
				} else {
					// hsize and vsize are not defined in the metadata... return data we got from the parameters
					return new ShapeUtils.Size(ViewerUtils.parseFloat(this.getParam("sizex")), ViewerUtils.parseFloat(this.getParam("sizey")));
				}

			} else if (this.isBlankPage()) {
				// blank page
				if (this.isReaderSpread()) { //calculation of size is different for when in readerspread
					var margins = this.getMargins();
					var trimboxSize = this.getTrimboxSize();
					return new ShapeUtils.Size(trimboxSize.width + margins.left + margins.right, trimboxSize.height + margins.top + margins.bottom);
				} else {
					return new ShapeUtils.Size(this.getParam("filemaxx") + margins.left + margins.right, this.getParam("filemaxy") + margins.top + margins.bottom);
				}
			} else if (this.hasCad()) {
				// CAD
				if (this.isCadAvailable && this.isCadAvailable()) {
					return this.getCad().getSize();
				} else {
					// CAD is not yet available
					return new ShapeUtils.Size();
				}
			}
		};

		/**
		 * Get the mediabox of this page. For blank pages this becomes 0.
		 * @returns {ShapeUtils.Rect}
		 */
		WebCenterDocumentPage.prototype.getMediaBoxRect = function () {
			var origin = new ShapeUtils.Point(ViewerUtils.parseFloat(this.getParam("filemedminx")), ViewerUtils.parseFloat(this.getParam("filemedminy")));
			var size = new ShapeUtils.Size(ViewerUtils.parseFloat(this.getParam("filemedmaxx")) - origin.x, ViewerUtils.parseFloat(this.getParam("filemedmaxy")) - origin.y);
			return new ShapeUtils.Rect(origin, size);
		};

		/**
		 * Get the current scaling of the document page, takes the compensation state (dist or scale or both) into account, returns null if no compensation present
		 * @returns {ShapeUtils.Point|null}
		 */
		WebCenterDocumentPage.prototype.getCurrentScaling = function () {
			var yComp = 1;
			var xComp = 1;
			if (this.getStateParam("compensateDist") === ViewControlsConstants.compensateDist.DISTONLY || this.getStateParam("compensateDist") === ViewControlsConstants.compensateDist.DISTANDSCALE) {
				yComp = yComp * 1 / (ViewerUtils.parseFloat(this.get("vdist")));
				xComp = xComp * 1 / (ViewerUtils.parseFloat(this.get("hdist")));
			}
			if (this.getStateParam("compensateDist") === ViewControlsConstants.compensateDist.SCALEONLY || this.getStateParam("compensateDist") === ViewControlsConstants.compensateDist.DISTANDSCALE) {
				yComp = yComp * 1 / (ViewerUtils.parseFloat(this.get("vscale")));
				xComp = xComp * 1 / (ViewerUtils.parseFloat(this.get("hscale")));
			}

			if (yComp === 1 && xComp === 1) {
				return null;
			} else {
				return new ShapeUtils.Point(xComp, yComp);
			}
		};

		/**
		 * @override
		 */
		WebCenterDocumentPage.prototype.getInfoURL = function (params) {
			return this._adapter.ViewServer("info.jsp", params);
		};

		/**
		 * Is the rectangle on the page, only for part on the page returns true
		 * @param {ShapeUtils.Rect} rect
		 * @returns {Boolean}
		 */
		WebCenterDocumentPage.prototype.isRectOnPage = function (rect) {
			var margin = this.getMargins();
			var pageSize = this.getPageSize();
			var pageRect = new ShapeUtils.Rect(new ShapeUtils.Point(0, 0), pageSize);
			pageRect.origin.offset(new ShapeUtils.Size(-margin.left, -margin.bottom));
			return pageRect.rectIntersect(rect);
		};

		/**
		 * Return the version Id for this page (used for page list document)
		 * @returns {Number} - version id for this page
		 */
		WebCenterDocumentPage.prototype.getPageVersionId = function () {
			return this._pageVersionId;
		};

		/**
		 * Check if the page is a digital film
		 * @returns {Boolean}
		 * @override
		 */
		WebCenterDocumentPage.prototype.isDigitalFilmPage = function () {
			// imagetype parameter is 0 for digital film and CAD, 1 for graphics
			return this.hasGraphics() && this.getParam("imagetype") === DataConstants.imageType.OTHER;
		};

		/**
		 * get the size of the trimbox, return null if the trimbox size is not known by the page (need to use margins)
		 * @return {ShapeUtils.Size}
		 */
		WebCenterDocumentPage.prototype.getTrimboxSize = function () {
			var hsize = ViewerUtils.parseFloat(this.get("hsize"));
			var vsize = ViewerUtils.parseFloat(this.get("vsize"));
			if (!hsize || !vsize) { // undefined or 0
				return new ShapeUtils.Size();
			} else {
				return new ShapeUtils.Size(hsize, vsize);
			}
		};

		/**
		 * Get the url for fetching tumbnails (50x50)
		 * @returns {String} - url for fetching the thumbdata of the page
		 */
		WebCenterDocumentPage.prototype.getThumbnailUrl = function () {
			var options;
			if (this.hasGraphics()) {
				//http://localhost:8081/ViewServer/thumbdata.jsp?file=000000\00002_0000001004\82b0fc110a1f8506202dd44c45aa7a91.0&px=50&py=50
				options = {
					file: this._data.file,
					px: 50,
					py: 50
				};
				return this.f.getDataAdapter().ViewServer("thumbdata.jsp", options);
			} else if (this.hasCad()) {
				options = {
					docverid: this._docVersion.get("docversionid")
				};
				return this.f.getDataAdapter().WebCenter("docgetthumbdata.jsp", options);
			}
		};

		/**
		 * Returns the information needed for a DocumentDropdownmenu to add an document to the dropdown menu
		 * @returns {Object}
		 */
		WebCenterDocumentPage.prototype.getInformationForDocumentSelector = function () {
			var document = this.getDocument();
			var docVersion = this.getDocumentVersion();
			var pageInfo = {};
			pageInfo.documentName = document.get("docname");
			pageInfo.project = document.get("projectname");
			if (document.hasMultipleVersions()) {
				pageInfo.versionNumber = this.getPageVersionId();
				if (pageInfo.versionNumber === undefined || pageInfo.versionNumber === null) {
					pageInfo.versionNumber = docVersion.getVersionId();
				}
				if (pageInfo.versionNumber && docVersion.get("docRevision")) {
					pageInfo.versionNumber += "(" + docVersion.get("docRevision") + ")";
				}
				pageInfo.versionId = docVersion.getVersionId();
			}
			var pageCount = docVersion.getPageCount();
			if (pageCount > 1) {
				pageInfo.pageCount = pageCount;
				pageInfo.pageNumber = this.getPagenum();
			}
			pageInfo.thumbnailUrl = this.getThumbnailUrl();
			pageInfo.pageId = this.getUniqueID();
			pageInfo.docType = document.getDocumentType();

			return pageInfo;
		};

		/**
		 * Is it possible to change the CAD overlay settings for this page?
		 * @override
		 * @returns {Boolean}
		 */
		WebCenterDocumentPage.prototype.canSaveCadOverlay = function () {
			return this.get("cadregedit") == 1;
		};

		/**
		 * Is the user allowed the edit/save te attributes of this page?
		 * @returns {Boolean}
		 */
		WebCenterDocumentPage.prototype.canEditAttributes = function () {
			return this.get("canEditAttributes") == 1;
		};

		/**
		 * Get the Attribute Category ID for the current document
		 * @returns {String} Category ID of the current document
		 */
		WebCenterDocumentPage.prototype.getAttributeCategoryID = function () {
			return this.get("attributeCategoryID");
		};

		/**
		 * Does the document page need an update of its data
		 * @returns {Boolean}
		 */
		WebCenterDocumentPage.prototype.needsDataUpdate = function () {
			return this.get("hasDefaultData");
		};

		/**
		 * Can the annotations be changed for this document page (not latest version, pharma settings for non-active projects, ...)
		 * @returns {Boolean}
		 */
		WebCenterDocumentPage.prototype.canChangeAnnotations = function () {
			return this.get("canChangeAnnotations") == 1;
		};

		return WebCenterDocumentPage;
	});

define('web-viewer/webcenter/WebCenterDocumentVersion',["web-viewer/data/AbstractDocumentVersion",
		"web-viewer/data/AbstractDocumentPage",
		"web-viewer/webcenter/WebCenterDocumentPage",
		"utils/XMLUtils",
		"web-viewer/utils/ViewerUtils",
		"underscore"
	],
	function (AbstractDocumentVersion,
		AbstractDocumentPage,
		WebCenterDocumentPage,
		XMLUtils,
		ViewerUtils,
		_) {
		/**
		 * @class The implementation of AbstractDocumentVersion for WebCenter. Initializes a version and constructs
		 * the page(s) in the documentversion.
		 * @exports WebCenterDocumentVersion
		 * @augments Data.AbstractDocumentVersion
		 * @memberof WebCenter
		 * @constructor
		 *
		 * @param {Core.ViewerFacade} f
		 * @param {AbstractDocument} doc - reference to the document
		 * @param {Number} versionId - id of the version of the document
		 */
		var WebCenterDocumentVersion = function (f, doc, versionId, versionInfo, info, adapter) {
			AbstractDocumentVersion.call(this, f, doc, versionId);

			this._adapter = adapter;

			this._versionData.docversionid = versionInfo.docversionid;

			this._versionData.docversion = versionId;

			this._versionData.isMultiPage = versionInfo.isMultiPage !== undefined ? versionInfo.isMultiPage : false;
			if (_.isString(this._versionData.isMultiPage)) {
				this._versionData.isMultiPage = this._versionData.isMultiPage === "true" ? true : false;
			}

			this._versionData.isPageList = versionInfo.isPageList !== undefined ? versionInfo.isPageList : false;
			if (_.isString(this._versionData.isPageList)) {
				this._versionData.isPageList = this._versionData.isPageList === "true" ? true : false;
			}

			this._versionData.docRevision = versionInfo.docrevision;

			if (info !== undefined && info.pagecount !== undefined) {
				this._versionData.pagecount = info.pagecount;
			}

			if (!_.isArray(versionInfo) && versionInfo !== undefined) {
				var pageNumber = 1;
				if (info) {
					pageNumber = info.pageno || 1;
				}
				var page = new WebCenterDocumentPage(f, this, pageNumber, versionInfo, info, adapter);
				this.addPage(pageNumber, page);
			}
		};

		WebCenterDocumentVersion.prototype = Object.create(AbstractDocumentVersion.prototype);
		WebCenterDocumentVersion.prototype.constructor = WebCenterDocumentVersion;

		/**
		 * Add a document to the document version with a data string
		 * @param {String} data
		 */
		WebCenterDocumentVersion.prototype.addDocumentPageFromData = function (data) {
			var page = new WebCenterDocumentPage(this._f, this, data.pageno, this._versionData, data, this._adapter);
			this.addPage(page.getPagenum(), page);
			if (data.pagecount !== undefined) {
				this._versionData.pagecount = parseInt(data.pagecount);
			}
		};

		/**
		 * Check if the document version has multiple pages
		 * @returns {Boolean}
		 */
		WebCenterDocumentVersion.prototype.isMultiPage = function () {
			return this.getPageCount() > 1 || this._versionData.isMultiPage;
		};

		/**
		 * @returns {String} The WebCenter document version id
		 */
		WebCenterDocumentVersion.prototype.getDocumentVersionID = function () {
			return this.get("docversionid");
		};

		/**
		 * Page count of the document
		 * @returns {Number} Page count of the document
		 */
		WebCenterDocumentVersion.prototype.getPageCount = function () {
			if (this.get("pagecount")) {
				return this.get("pagecount");
			} else if (this._versionData.isMultiPage) {
				return this._pages.length - 1;
			} else {
				return 1;
			}
		};

		return WebCenterDocumentVersion;
	});

define('web-viewer/webcenter/WebCenterDocument',["web-viewer/data/AbstractDocument",
		"web-viewer/data/DataConstants",
		"underscore",
		"web-viewer/webcenter/WebCenterDocumentVersion",
		"web-viewer/webcenter/WebCenterDocumentPage",
		"utils/XMLUtils"
	],
	function (
		AbstractDocument,
		DataConstants,
		_,
		WebCenterDocumentVersion,
		WebCenterDocumentPage,
		XMLUtils
	) {
		"use strict";

		/**
		 * @class Specific implementation of a document for WebCenter.
		 * @exports WebCenterDocument
		 * @augments Data.AbstractDocument
		 * @memberof WebCenter
		 *
		 * @param {Core.ViewerFacade} f
		 * @param {Object} documentData - information of the document
		 * @param {Number} IID - id of the document
		 * @param {DataConstants.documentPageType} type
		 * @param {AbstractAdapter} adapter -
		 */
		var WebCenterDocument = function (f, documentData, IID, type, adapter) {
			AbstractDocument.call(this, f, IID, type);

			this.f = f;

			this._adapter = adapter;

			if (documentData !== undefined) {
				this._initializeWithDocInfo(documentData);
			}
		};

		WebCenterDocument.prototype = Object.create(AbstractDocument.prototype);
		WebCenterDocument.prototype.constructor = WebCenterDocument;

		/**
		 * Use a data string to initialize a document page and add it to the correct documentVersion
		 * @param {String} pageData
		 */
		WebCenterDocument.prototype.addDocumentPageFromData = function (pageData) {
			//first fetch the correct version from data where the page should be added to
			var documentVersion = this.getDocumentVersion(pageData.docversion);
			if (documentVersion !== undefined && pageData.docversionid === documentVersion._versionData.docversionid) { //make sure the document version exists
				documentVersion.addDocumentPageFromData(pageData);
			} else if (pageData.isMultiPage && pageData.pageindex !== undefined) { //for pagelist the docversion isn't always correct
				documentVersion = this.getDocumentVersion(); //get the latest document version for this document
				documentVersion.addDocumentPageFromData(pageData); //add the document page to the latest version
				//if the page data added has a higher version number change the version data in the document version
				if (pageData.docversion > documentVersion.getVersionId()) {
					var previousId = documentVersion.getVersionId();
					documentVersion._versionData.docversionid = pageData.docversionid;
					documentVersion._versionData.docversion = pageData.docversion;
					documentVersion._versionId = pageData.docversion;
					this._documentVersions[pageData.docversion] = documentVersion;
					delete this._documentVersions[previousId];
				} else if (pageData.docversion === documentVersion.getVersionId() && pageData.docversionid > documentVersion._versionData.docversionid) {
					documentVersion._versionData.docversionid = pageData.docversionid;
					this._documentVersions[pageData.docversion] = documentVersion;
				}
			}
			if (pageData.diff !== undefined) { //a default reference is supplied for this page, (the page with same pagenumber of a previous version)
				var diffDocumentVersion = this.getDocumentVersion(pageData.diffversion);
				if (diffDocumentVersion === undefined) { //if not defined make a temp document version and fill the docversionid in later
					diffDocumentVersion = new WebCenterDocumentVersion(this.f, this, pageData.diffversion, {}, undefined, this._adapter);
					this.addDocumentVersion(diffDocumentVersion);
				}
				var diffDocumentPage = new WebCenterDocumentPage(this.f, diffDocumentVersion, pageData.pageno, {
					file: pageData.diff
				}, undefined, this._adapter);
				diffDocumentVersion.addPage(pageData.pageno, diffDocumentPage);
			}
		};

		/**
		 * The viewer is initialized with document info provided
		 * @private
		 * @param {Object} data
		 */
		WebCenterDocument.prototype._initializeWithDocInfo = function (data) {
			this.set("docid", data.docid);

			this.set("projectid", data.projectid);
			this.set("projectname", data.projectname + "");
			this.set("binding", data.binding);

			if (this._documentType === DataConstants.documentPageType.GRAPHICS) {
				//first add the current version
				var currentVersion = {};
				currentVersion.docversionid = data.docversionid;
				this.set("docname", data.docname + "");
				this.set("projectname", data.projectname + "");
				currentVersion.file = data.file;
				currentVersion.isPageList = data.pageindex !== undefined;
				currentVersion.docrevision = data.docrevision;
				var docVersion = new WebCenterDocumentVersion(this.f, this, data.docversion, currentVersion, data, this._adapter);
				this.addDocumentVersion(docVersion);

				if (data.cadversionid) {
					// this document has a linked cad, store this ID so we can decide on showing the compare controls or not...
					this.set("linkedCADVersionID", data.cadversionid);
				}

				if (data.isMultiPage && data.versionlist === undefined && data.pageindex === undefined) { //should only happen for multipage
					//first add a documentVersion for each version before calling the viewserver for info
					for (var i = 1; i < parseInt(data.docversion, 10); i++) { //only versions besides the current version
						var tempVersion = new WebCenterDocumentVersion(this.f, this, i, {
							isMultiPage: data.isMultiPage
						}, undefined, this._adapter);
						this.addDocumentVersion(tempVersion);
					}

					if (data.diff !== undefined) { //default reference page for this page (page with same page number of a previous version)
						var diffDocumentVersion = this.getDocumentVersion(data.diffversion);
						var diffDocumentPage = new WebCenterDocumentPage(this.f, diffDocumentVersion, data.pageno, {
							file: data.diff
						}, undefined, this._adapter);
						diffDocumentVersion.addPage(data.pageno, diffDocumentPage);
					}

					//get the versionlist with getdocumentinfo, this is not supplied in the data string
					data.pageNumber = 1;
					this._adapter.loadVersionsOfDocumentPage(data, this, _.bind(function () {
						this.f.getDocumentController().loadedDocumentChanged();
					}, this));
				}

				//multiple versions, make multiple AbstractDocumentVersions
				if (data.versionlist !== undefined) {
					if (!_.isArray(data.versionlist)) { //check if versionlist is an array (more than 2 versions) are object (2 versions)
						data.versionlist = [data.versionlist]; //make it an array if it's only one element
					}
					_.each(data.versionlist, _.bind(function (version) {
						var docVersion = new WebCenterDocumentVersion(this.f, this, version.docversion, version, undefined, this._adapter);
						this.addDocumentVersion(docVersion);
					}, this));
				}
			} else if (this._documentType === DataConstants.documentPageType.CAD) { //the file is CAD
				if (data.caddocname) {
					this.set("docname", data.caddocname + "");
				} else {
					this.set("docname", data.docname + "");
				}
				//multiple versions, make multiple AbstractDocumentVersions
				if (data.cadversionlist !== undefined) { //if CAD with graphics, the cad versionlist is defined in cadversionlist
					if (!_.isArray(data.cadversionlist)) { //check if cadversionlist is an array
						data.cadversionlist = [data.cadversionlist];
					}
					_.each(data.cadversionlist, _.bind(function (version) {
						var docVersion = new WebCenterDocumentVersion(this.f, this, version.docversion, version, data, this._adapter);
						this.addDocumentVersion(docVersion);
					}, this));
				} else if (data.versionlist !== undefined) {
					if (!_.isArray(data.versionlist)) { //check if versionlist is an array
						data.versionlist = [data.versionlist];
					}
					_.each(data.versionlist, _.bind(function (version) {
						var docVersion = new WebCenterDocumentVersion(this.f, this, version.docversion, version, data, this._adapter);
						this.addDocumentVersion(docVersion);
					}, this));
				}
			}
		};

		/**
		 * Loads document pages of previous versions with he same document page number
		 * @param {WebCenterDocumentPage} documentPage
		 * @param {Function} callback
		 */
		WebCenterDocument.prototype.loadVersionsForDocumentPage = function (documentPage, callback) {
			var data = {
				docid: documentPage.getDocument().getDocumentID(),
				docversionid: documentPage.getDocumentVersion().getDocumentVersionID(),
				diffversion: null,
				pageNumber: documentPage.getPagenum(),
				isPageList: documentPage.getDocumentVersion().isPageList()
			};
			this._adapter.loadVersionsOfDocumentPage(data, this, callback, documentPage);
		};


		/**
		 * Get the WebCenter docID of the document
		 * @returns {String}
		 */
		WebCenterDocument.prototype.getDocumentID = function () {
			return this.get("docid");
		};

		/**
		 * Get the WebCenter Project ID of the document
		 * @returns {String}
		 */
		WebCenterDocument.prototype.getProjectID = function () {
			return this.get("projectid");
		};

		return WebCenterDocument;
	});

define('web-viewer/utils/LineworkWrapper',["underscore",
		"utils/ShapeUtils",
		"web-viewer/core/Extendable",
		"web-viewer/utils/DrawUtils",
		"web-viewer/utils/LineworkPoint"
	],
	function (_,
		ShapeUtils,
		Extendable,
		DrawUtils,
		LineworkPoint) {
		"use strict";

		/**
		 * @memberof Utils
		 * @exports LineworkWrapper
		 * @class
		 * A lineworkwrapper is used to draw linework
		 * @constructor
		 */
		var LineworkWrapper = function () {
			Extendable.installTo(this);

			this.color = null;
			this.c = null;
			this.fillcolor = null;
			this.pointArray = [];
			this.rect = new ShapeUtils.Rect();
			this.type = 0;
			this.linewidth = 1;
			this.imagehotspot = null;
			this.imagetouchspot = null;
			this.imageRef = null;

			this.treatAsOutline = false; //treat the object as an outline object for selecting

			this.textoptions = {};
			this.selected = false;

			this.showIndicator = false;
			this.indicatorText = null;
			this.indicatorTextColor = null;
			this.indicatorColor = null;

			this.snapable = false; //if snapable, the user can snap to the points of the wrapper	
		};

		/**
		 * Move the lineworkwrapper over a size
		 * @param {ShapeUtils.Size} size
		 */
		LineworkWrapper.prototype.move = function (size) {
			this.rect.origin.x += size.width;
			this.rect.origin.y += size.height;
			if (this.pointArray && this.pointArray.length > 0) {
				for (var j = 0; j < this.pointArray.length; j++) {
					var point = this.pointArray[j];
					point.x += size.width;
					point.y += size.height;
				}
			}
		};

		/**
		 * Sets the rectangle of the lineworkwrapper
		 * @param {ShapeUtils.Rect} rect
		 * @param {Boolean} updateStartEnd
		 */
		LineworkWrapper.prototype.setRect = function (rect, updateStartEnd) {
			if (updateStartEnd) {
				if (this.pointArray && this.pointArray.length == 2) {
					var p1 = rect.topleft();
					var p2 = rect.bottomright();

					if ((this.pointArray[0].x == p1.x && this.pointArray[0].y == p1.y) || this.pointArray[0].x == this.pointArray[1].x && this.pointArray[0].y == this.pointArray[1].y) {
						this.pointArray[0].x = p1.x;
						this.pointArray[0].y = p1.y;

						this.pointArray[1].x = p2.x;
						this.pointArray[1].y = p2.y;
					} else {
						this.pointArray[1].x = p1.x;
						this.pointArray[1].y = p1.y;

						this.pointArray[0].x = p2.x;
						this.pointArray[0].y = p2.y;
					}
				}
			}
			this.rect = rect;
		};

		/**
		 * Set the start and the end point for the lineworkwrapper
		 * @param {ShapeUtils.Point} start
		 * @param {ShapeUtils.Point} end
		 */
		LineworkWrapper.prototype.setStartEnd = function (start, end) {
			if (this.pointArray && this.pointArray.length == 2) {
				if (start) {
					this.pointArray[0].x = start.x;
					this.pointArray[0].y = start.y;
				} else {
					start = new ShapeUtils.Point(this.pointArray[0].x, this.pointArray[0].y);
				}
				if (end) {
					this.pointArray[1].x = end.x;
					this.pointArray[1].y = end.y;
				} else {
					end = new ShapeUtils.Point(this.pointArray[1].x, this.pointArray[1].y);
				}

				this.setRect(new ShapeUtils.Rect(start, start.sizeTo(end)));
			}
		};

		/**
		 * Get the rgba color of the line work with alpha
		 * @param {Number} alpha
		 * @returns {String}
		 */
		LineworkWrapper.prototype.rgbaColor = function (alpha) {
			var rgb = this.rgbColor();
			return "rgba(" + rgb.r + "," + rgb.g + "," + rgb.b + "," + alpha + ")";
		};

		/**
		 * Get the rgb color
		 * @returns {Object}
		 */
		LineworkWrapper.prototype.rgbColor = function () {
			var result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(this.color);
			return result ? {
				r: parseInt(result[1], 16),
				g: parseInt(result[2], 16),
				b: parseInt(result[3], 16)
			} : null;
		};

		/**
		 * Calculate the rectangle from all the points of the point array of the linework
		 */
		LineworkWrapper.prototype.calculateRectFromPointArray = function () {
			var rectx1 = null;
			var rectx2 = null;
			var recty1 = null;
			var recty2 = null;

			for (var i = 0; i < this.pointArray.length; i++) {
				var p = this.pointArray[i];

				if (!rectx1 || p.x < rectx1) {
					rectx1 = p.x;
				}
				if (!rectx2 || p.x > rectx2) {
					rectx2 = p.x;
				}
				if (!recty1 || p.y < recty1) {
					recty1 = p.y;
				}
				if (!recty2 || p.y > recty2) {
					recty2 = p.y;
				}
			}
			var r = new ShapeUtils.Rect(new ShapeUtils.Point(rectx1, recty1), new ShapeUtils.Size(rectx2 - rectx1, recty2 - recty1));
			r.normalize();
			this.rect = r;
		};

		/**
		 * Set the selection anchor point
		 * @param {ShapeUtils.Point} point
		 * @param fillColor
		 * @param strokeColor
		 * @param subtype
		 * @returns {LineworkWrapper}
		 */
		LineworkWrapper.prototype.selectionAnchorPoint = function (point, fillColor, strokeColor, subtype) {
			var anchor = new LineworkWrapper();

			anchor.type = DrawUtils.LWTYPE_ELLIPSE;
			anchor.color = strokeColor;
			anchor.fillcolor = fillColor;
			anchor.staticsize = true;
			anchor.linewidth = 1;
			var size = 10;
			anchor.rect = new ShapeUtils.Rect(new ShapeUtils.Point(point.x - size / 2, point.y - size / 2), new ShapeUtils.Size(size, size));
			anchor.anchor = subtype;
			anchor.linework = this;

			return anchor;
		};

		/**
		 * Get the selection box of the linework wrapper
		 * @returns {Array.<LineworkWrapper>}
		 */
		LineworkWrapper.prototype.getSelectionBox = function () {
			var selectionFillColor = 'rgb(153,152,187)';
			var selectionStrokeColor = 'white';

			if (this.pointArray && this.pointArray.length >= 2) {
				return [
					this.selectionAnchorPoint(new ShapeUtils.Point(this.pointArray[0].x, this.pointArray[0].y), selectionFillColor, selectionStrokeColor, DrawUtils.LWANCHOR_START),
					this.selectionAnchorPoint(new ShapeUtils.Point(this.pointArray[this.pointArray.length - 1].x, this.pointArray[this.pointArray.length - 1].y), selectionFillColor, selectionStrokeColor, DrawUtils.LWANCHOR_END)
				];
			} else {
				var anchors = [
					this.selectionAnchorPoint(this.rect.topleft(), selectionFillColor, selectionStrokeColor, DrawUtils.LWANCHOR_TOPLEFT),
					this.selectionAnchorPoint(this.rect.topright(), selectionFillColor, selectionStrokeColor, DrawUtils.LWANCHOR_TOPRIGHT),
					this.selectionAnchorPoint(this.rect.bottomleft(), selectionFillColor, selectionStrokeColor, DrawUtils.LWANCHOR_BOTTOMLEFT),
					this.selectionAnchorPoint(this.rect.bottomright(), selectionFillColor, selectionStrokeColor, DrawUtils.LWANCHOR_BOTTOMRIGHT),
					this.selectionAnchorPoint(this.rect.topcenter(), selectionFillColor, selectionStrokeColor, DrawUtils.LWANCHOR_TOPCENTER),
					this.selectionAnchorPoint(this.rect.bottomcenter(), selectionFillColor, selectionStrokeColor, DrawUtils.LWANCHOR_BOTTOMCENTER),
					this.selectionAnchorPoint(this.rect.centerleft(), selectionFillColor, selectionStrokeColor, DrawUtils.LWANCHOR_CENTERLEFT),
					this.selectionAnchorPoint(this.rect.centerright(), selectionFillColor, selectionStrokeColor, DrawUtils.LWANCHOR_CENTERRIGHT)
				];

				return _.union(anchors);
			}
		};

		/**
		 * Scale the linework wrapper with a given factor (both horizontal as vertical)
		 * @param {ShapeUtils.Point} scaleFactor
		 */
		LineworkWrapper.prototype.scale = function (scaleFactor) {
			if (scaleFactor) {
				_.each(this.pointArray, function (point) {
					point.x *= scaleFactor.x;
					point.y *= scaleFactor.y;
				});

				this.rect.origin.x *= scaleFactor.x;
				this.rect.origin.y *= scaleFactor.y;
				this.rect.size.width *= scaleFactor.x;
				this.rect.size.height *= scaleFactor.y;
			}
		};

		return LineworkWrapper;
	});

define('web-viewer/cad/CadLine',["web-viewer/utils/LineworkWrapper",
		"web-viewer/utils/LineworkPoint",
		"utils/ShapeUtils",
		"utils/XMLUtils",
		"web-viewer/utils/ViewerUtils",
		"web-viewer/utils/DrawUtils",
		"web-viewer/cad/CadConstants"
	],
	function (LineworkWrapper,
		LineworkPoint,
		ShapeUtils,
		XMLUtils,
		ViewerUtils,
		DrawUtils,
		CadConstants) {
		"use strict";

		/**
		 * @exports CadLine
		 * @memberof CadModule
		 * @class CadLine is a LineworkWrapper object that is used to display a line within a CadFile
		 * @augments Utils.LineworkWrapper
		 * @constructor
		 * @param {XMLElement} node - xml to construct the CadLine
		 * @param {Object} styles - plotting styles to use to construct the CadFile (colors/width/...)
		 * @param {Number} height - height of the cad file to calculate the point on normal axis
		 */
		var CadLine = function (node, styles, height) {
			LineworkWrapper.call(this);

			var linename = node.nodeName;
			if (linename == 'Nicks') {
				this.type = CadConstants.CadTypes.Nick;
				this.linewidth = 1;
				this.color = 'black';
			} else {
				this.type = XMLUtils.attributeValue(node, 'linetype');
			}
			var bopt = XMLUtils.attributeValue(node, 'bridgeopt');
			if (bopt) {
				this.bridgeopt = parseInt(bopt, 10);
			}

			if (this.type === null) {
				var strokeColor = XMLUtils.attributeValue(node, 'stroke');
				if (strokeColor !== null) {
					this.color = strokeColor;
				}
			} else {
				var pstyle = styles[this.type];
				if (pstyle) {
					this.c = pstyle.color; //color
					this.color = '#' + ViewerUtils.rgbToHex(this.c.r, this.c.g, this.c.b);
					this.linewidth = pstyle.width / 4;
					if (pstyle.style !== "0") {
						this.dash = CadConstants.LineTypes[pstyle.style];
					}
				}
			}

			//test if fill color is defined for the CADLine, specify a fill color for the lineworkWrapper
			this.fillcolor = XMLUtils.attributeValue(node, 'fill');
			if (this.fillcolor === "none") { //if "none" is specified as fill color set fillcolor to null
				this.fillcolor = null;
			}
			//if there is a stroke defined as none set the linewidth to the minimum (linethickness 1)
			if (this.fillcolor && XMLUtils.attributeValue(node, 'stroke') === "none") {
				this.linewidth = 1;
			}

			var path = XMLUtils.attributeValue(node, 'd');
			this.pointArray = [];

			var rectx1 = null;
			var rectx2 = null;
			var recty1 = null;
			var recty2 = null;

			var cmdRegEx = /[a-z][^a-z]*/ig;
			var pathchuncs = path.match(cmdRegEx);
			for (var i = 0; i < pathchuncs.length; i++) {
				var p = new LineworkPoint();

				var chunc = ViewerUtils.trim(pathchuncs[i]);

				var minXPoint = null;
				var maxXPoint = null;
				var minYPoint = null;
				var maxYPoint = null;

				var command = chunc.charAt(0);
				if (command == 'A') {
					//A 22.00 22.00 0 0 0 13.585 323.300
					var arcregex = new RegExp(/A\s(.+)\s(.+)\s(.+)\s(.+)\s(.+)\s(.+)\s(.+)/);
					var arccommand = arcregex.exec(chunc);

					var rx = ViewerUtils.parseFloat(arccommand[1]);
					var ry = ViewerUtils.parseFloat(arccommand[2]);
					var angle = ViewerUtils.parseFloat(arccommand[3]);
					var largeArcFlag = ViewerUtils.parseFloat(arccommand[4]);
					var sweepFlag = ViewerUtils.parseFloat(arccommand[5]);
					var x1 = ViewerUtils.parseFloat(arccommand[6]);
					var y1 = ViewerUtils.parseFloat(arccommand[7]);

					p.makeArc(x1, height - y1, rx, ry, angle, largeArcFlag, sweepFlag);

					minXPoint = p.x;
					maxXPoint = p.x;
					minYPoint = p.y;
					maxYPoint = p.y;
				} else if (command == 'C') {
					var circleregex = new RegExp(/C\s(.+)\s(.+)\s(.+)\s(.+)\s(.+)\s(.+)/);
					var circlecommand = circleregex.exec(chunc);

					var x1 = ViewerUtils.parseFloat(circlecommand[1]);
					var y1 = ViewerUtils.parseFloat(circlecommand[2]);
					var x2 = ViewerUtils.parseFloat(circlecommand[3]);
					var y2 = ViewerUtils.parseFloat(circlecommand[4]);
					var x3 = ViewerUtils.parseFloat(circlecommand[5]);
					var y3 = ViewerUtils.parseFloat(circlecommand[6]);

					//calculate the min and max x/y points for the rectangle that contains the bezier-curve
					//A bezier curve is completely contained in the convex hull of its control points
					minXPoint = Math.min(x1, x2, x3);
					maxXPoint = Math.max(x1, x2, x3);
					minYPoint = Math.min(height - y1, height - y2, height - y3); //the origin of CAD has inverted Y-axis
					maxYPoint = Math.max(height - y1, height - y2, height - y3);

					p.makeBezier(x1, height - y1, x2, height - y2, x3, height - y3);
				} else if (command == 'N') {

					var nickregex = new RegExp(/N(.+)\s(.+)\s(.+)\s(.+)\s(.+)/);
					var nickcommand = nickregex.exec(chunc);
					var v1 = ViewerUtils.parseFloat(nickcommand[1]);
					var v2 = ViewerUtils.parseFloat(nickcommand[2]);
					var v3 = ViewerUtils.parseFloat(nickcommand[3]);
					var v4 = ViewerUtils.parseFloat(nickcommand[4]);
					var v5 = ViewerUtils.parseFloat(nickcommand[5]);

					p.makeNick(v1, height - v2, v3, v4, v5);

					minXPoint = p.x;
					maxXPoint = p.x;
					minYPoint = p.y;
					maxYPoint = p.y;
				} else if (command == 'L' || command == 'M') {
					var lineregex = new RegExp(/.(.+)\s(.+)/);
					var linecommand = lineregex.exec(chunc);
					var x = ViewerUtils.parseFloat(linecommand[1]);
					var y = height - ViewerUtils.parseFloat(linecommand[2]);

					if (command == 'L') {
						p.makeLineTo(x, y);
					} else {
						p.makeMoveTo(x, y);
					}

					minXPoint = p.x;
					maxXPoint = p.x;
					minYPoint = p.y;
					maxYPoint = p.y;
				}
				if (p.type != -1) {
					this.pointArray.push(p);

					if (rectx1 === null || minXPoint < rectx1) {
						rectx1 = minXPoint;
					}
					if (rectx2 === null || maxXPoint > rectx2) {
						rectx2 = maxXPoint;
					}
					if (recty1 === null || minYPoint < recty1) {
						recty1 = minYPoint;
					}
					if (recty2 === null || maxYPoint > recty2) {
						recty2 = maxYPoint;
					}
				}
			}
			var r = new ShapeUtils.Rect(new ShapeUtils.Point(rectx1, recty1), new ShapeUtils.Size(rectx2 - rectx1, recty2 - recty1));
			r.normalize();
			this.rect = r;
		};
		// inherit
		CadLine.prototype = Object.create(LineworkWrapper.prototype);
		// correct the constructor pointer 
		CadLine.prototype.constructor = CadLine;

		return CadLine;

	});

define('web-viewer/cad/CadText',["web-viewer/utils/LineworkWrapper",
		"utils/ShapeUtils",
		"utils/XMLUtils",
		"web-viewer/utils/ViewerUtils"
	],
	function (LineworkWrapper,
		ShapeUtils,
		XMLUtils,
		ViewerUtils) {
		"use strict";

		/**
		 * @exports CadText
		 * @memberof CadModule
		 * @class CadText is a LineworkWrapper object that is used to display a text within a CadFile
		 * @augments Utils.LineworkWrapper
		 * @constructor
		 * @param {XMLElement} node - xml to construct the CadText
		 * @param {Object} styles - plotting styles to use to construct the CadFile (colors/width/...)
		 * @param {Number} tspanIndex - index of the tspan node
		 * @param {Number} height - height of the cad file to calculate the point on normal axis
		 */
		var CadText = function (node, styles, tspanIndex, height) {
			LineworkWrapper.call(this);

			this.type = -21;
			//		this.staticsize = true;
			this.textoptions['rotation'] = 0;

			for (var i = 0; i < node.attributes.length; i++) {
				var attrib = node.attributes[i];
				this.textoptions[attrib.nodeName] = attrib.nodeValue;
			}
			if (this.textoptions.rotate == null) {
				this.textoptions.rotate = 0;
			} else {
				this.textoptions.rotate = ViewerUtils.parseFloat(this.textoptions.rotate);
			}

			var tspannode = node.getElementsByTagName('Tspan')[tspanIndex];
			this.textoptions['text'] = tspannode.firstChild.nodeValue;
			var x = XMLUtils.attributeValue(tspannode, 'x');
			var y = XMLUtils.attributeValue(tspannode, 'y');
			if (this.textoptions.rotate !== 0) {
				var rotate = -this.textoptions.rotate / 180 * Math.PI;

				// SVG store text position pre-rotate, we prefer post-rotate
				var newX = x * Math.cos(rotate) - y * Math.sin(rotate);
				var newY = x * Math.sin(rotate) + y * Math.cos(rotate);
				x = newX;
				y = newY;
			}

			this.rect = new ShapeUtils.Rect(new ShapeUtils.Point(0, 0), new ShapeUtils.Size(0, 0));
			this.rect.origin.x = x;
			this.rect.origin.y = height - y;

			var pstyle = styles[this.textoptions.filltype];
			if (pstyle) {
				this.c = pstyle.color;
				this.color = '#' + ViewerUtils.rgbToHex(this.c.r, this.c.g, this.c.b);
			}

			if (this.textoptions['fill'] !== undefined) {
				this.color = this.textoptions['fill'];
			}
		};

		// inherit
		CadText.prototype = Object.create(LineworkWrapper.prototype);
		// correct the constructor pointer 
		CadText.prototype.constructor = CadText;

		return CadText;

	});

define('web-viewer/cad/CadFile',["underscore",
		"utils/ShapeUtils",
		"utils/XMLUtils",
		"web-viewer/cad/CadLine",
		"web-viewer/cad/CadText",
		"web-viewer/cad/CadConstants",
		"web-viewer/utils/ViewerUtils"
	],
	function (_,
		ShapeUtils,
		XMLUtils,
		CadLine,
		CadText,
		CadConstants,
		ViewerUtils) {
		"use strict";

		/**
		 * @exports CadLayer
		 * @memberof CadModule
		 * @class CadLayer is an object that contains the information of a layer + its lines and oneups
		 * @constructor
		 * @param {String} layerclass - class of the layer
		 * @param {String} layertype - type of the layer
		 * @param {String} layername - name of the layer
		 * @param {Boolean} [oneup] - if true, the xml node describes a oneup CadFile within a layout CadFile
		 */
		var CadLayer = function (layerclass, layertype, layername, oneup) {
			/**
			 * Class of the layer
			 */
			this.layerclass = layerclass;

			/**
			 * Type of the layer
			 */
			this.layertype = layertype;

			/**
			 * Layer name
			 */
			this.layername = layername;

			/**
			 * Array of CadLine/CadText objects
			 */
			this.lines = [];

			/**
			 * Array of oneups (CadFile objects)
			 */
			this.oneUps = [];


		};

		/**
		 * @exports CadFile
		 * @memberof CadModule
		 * @class CadFile is an object that contains all needed info for a cad file (layers, styles...)
		 * @constructor
		 * @param {CadStyle} style - style to use to construct the CadFile (colors/width/...)
		 * @param {XMLElement} node - xml to construct the CadFile
		 * @param {ShapeUtils.Rect} [rect] - Fixed rectangle (if not, its calculated from the xml node)
		 * @param {Boolean} [oneup] - if true, the xml node describes a oneup CadFile within a layout CadFile
		 */
		var CadFile = function (style, node, rect, oneup) {

			/**
			 * Plotting styles
			 */
			this.cstyles = style;

			/**
			 * Name of the CadFile (only used for a oneup CadFile)
			 */
			this.name = null;

			/**
			 * Cad layers (Array.<CadLayer>)
			 */
			this.cadlayers = [];

			/**
			 * Oneup layers (undefined in the beginning, but getOneupClasses())
			 */
			this.oneupClasses = null;

			/**
			 * {Array.<CadLayer>} objects that are selected
			 * @private
			 */
			this._selectedLayers = null;

			/**
			 * {Array.<String>} class names that are selected
			 * @private
			 */
			this._selectedClasses = null;

			/**
			 * {Number} transparency of the CadFile
			 */
			this.alpha = 1;

			/**
			 * {XMLElement} xml used to construct the CAD file
			 */
			this._xmlData = node; //used to recontruct the cad-file when getting a deep copy, prevent call to the server

			/*
			 * If the CadFile is a oneup, use the rect from its parent
			 */
			if (oneup) {
				this.rect = rect;
			} else {
				//Calculate the rect based on the attributes
				this.rect = new ShapeUtils.Rect();

				var minx = ViewerUtils.parseFloat(XMLUtils.attributeValue(node, 'minx'));
				var miny = ViewerUtils.parseFloat(XMLUtils.attributeValue(node, 'miny'));
				var maxx = ViewerUtils.parseFloat(XMLUtils.attributeValue(node, 'maxx'));
				var maxy = ViewerUtils.parseFloat(XMLUtils.attributeValue(node, 'maxy'));

				this.rect.origin.x = minx;
				this.rect.origin.y = miny;
				this.rect.size.width = maxx - minx;
				this.rect.size.height = maxy - miny;
			}

			/**
			 * Original origin of the CAD
			 */
			this.originalOrigin = this.rect.origin.copy();

			/**
			 * Doc type (1 = oneup, 2 = layout)
			 */
			this.doctype = XMLUtils.attributeValue(node, 'doctype');

			/*
			 * When the CadFile is a oneup, take the "OneupLayer"-elements, if not, take the "Layer"-elements
			 */
			var layers;
			if (oneup) {
				this.name = XMLUtils.attributeValue(node, 'name');
				layers = node.getElementsByTagName('OneupLayer');
			} else {
				layers = node.getElementsByTagName('Layer');
			}


			/*
			 * Variables used to calculate the content rect (rect that contains all the cad lines)
			 */
			var x1 = null;
			var y1 = null;
			var x2 = null;
			var y2 = null;

			/*
			 * Iterate over the layers and store them inside of the layer objects
			 */
			for (var i = 0; i < layers.length; i++) {
				var clayer = layers[i];

				var lclass = XMLUtils.attributeValue(clayer, 'class');
				var ltype = XMLUtils.attributeValue(clayer, 'type');
				var lname = XMLUtils.attributeValue(clayer, 'name');

				var l = new CadLayer(lclass, ltype, lname, oneup);

				var children = clayer.childNodes;
				for (var j = 0; j < children.length; j++) {
					var child = children[j];
					if (child.nodeType === 1) {
						var cline;
						if (child.nodeName == 'Path' || child.nodeName == 'Nicks') {
							cline = new CadLine(child, this.cstyles.plottingStyles, this.rect.size.height);

							if (child.nodeName == 'Path' && l.layerclass != "3" && l.layerclass != "4") { //Snap if it's a line, but not on layers of class 3/4 (Dimensions)
								cline.snapable = true;
							}

							l.lines.push(cline);

							//Check if the x/y values are larger than the ones we already had
							if (x1) {
								x1 = Math.min(x1, cline.rect.left());
							} else {
								x1 = cline.rect.left();
							}
							if (y1) {
								y1 = Math.min(y1, cline.rect.top());
							} else {
								y1 = cline.rect.top();
							}
							if (x2) {
								x2 = Math.max(x2, cline.rect.right());
							} else {
								x2 = cline.rect.right();
							}
							if (y2) {
								y2 = Math.max(y2, cline.rect.bottom());
							} else {
								y2 = cline.rect.bottom();
							}
						} else if (child.nodeName == 'TextGroup') {
							var texts = child.getElementsByTagName('Text');
							for (var t = 0; t < texts.length; t++) {
								var tspans = texts[t].getElementsByTagName('Tspan');
								for (var ts = 0; ts < tspans.length; ts++) {
									cline = new CadText(texts[t], this.cstyles.plottingStyles, ts, this.rect.size.height);
									l.lines.push(cline);
								}
							}
						} else if (child.nodeName == 'Oneup') {
							//If the node is a OneUp, make a new CadFile, specifically for oneups
							var oneupFile = new CadFile(style, child, this.rect, true);
							l.oneUps.push(oneupFile);
						}
					}
				}

				this.cadlayers.push(l);

				this.contentRect = new ShapeUtils.Rect(new ShapeUtils.Point(x1, y1), new ShapeUtils.Size(x2 - x1, y2 - y1));
				this.contentRect.normalize();
			}

			// Initialize the selected layers (select all by default)
			this._selectedLayers = _.clone(this.cadlayers);

			// Initialize the selected classes (select all by default)
			this._selectedClasses = _.clone(this.getOneupClasses());

		};

		/**
		 * Get the size of the CadFile
		 * @returns {ShapeUtils.Size}
		 */
		CadFile.prototype.getSize = function () {
			return this.rect.size.copy();
		};

		/**
		 * Get the origin of the CadFile
		 * @returns {ShapeUtils.Point}
		 */
		CadFile.prototype.getOrigin = function () {
			return this.rect.origin.copy();
		};

		/**
		 * Returns the original origin
		 * @returns {ShapeUtils.Point}
		 */
		CadFile.prototype.getOriginalOrigin = function () {
			return this.originalOrigin;
		};

		/**
		 * Resets the origin
		 */
		CadFile.prototype.resetOrigin = function () {
			this.rect.origin = this.originalOrigin.copy();
		};

		/**
		 *
		 * @param newX
		 * @param newY
		 */
		CadFile.prototype.setOrigin = function (newX, newY) {
			this.rect.origin.x = newX;
			this.rect.origin.y = newY;
		};

		/**
		 * get the selected layers
		 * @returns {Array.<CadLayer>}
		 */
		CadFile.prototype.getSelectedLayers = function () {
			return this._selectedLayers;
		};

		/**
		 * get the selected classes
		 * @returns {Array.<String>}
		 */
		CadFile.prototype.getSelectedClasses = function () {
			return this._selectedClasses;
		};

		/**
		 * Set the location of the CAD file in the current world
		 * @param {ShapeUtils.Rect} location
		 */
		CadFile.prototype.setLocation = function (location) {
			this._location = location;
		};

		/**
		 * Returns the location of the CAD file in the current world
		 * @returns {ShapeUtils.Rect}
		 */
		CadFile.prototype.getLocation = function () {
			return this._location;
		};

		/**
		 * Toggle the visibility of a layer
		 * @param {Number} index
		 * @param {Boolean} hideAllLayers
		 */
		CadFile.prototype.toggleLayer = function (index, hideAllLayers) {
			if (index < 0) {
				if (this._selectedLayers.length === this.cadlayers.length && hideAllLayers) {
					this._selectedLayers = [];
				} else {
					this._selectedLayers = _.clone(this.cadlayers);
				}
			} else {
				var layer = this.cadlayers[index];
				if (layer) {
					index = _.indexOf(this._selectedLayers, layer);
					if (index >= 0) {
						this._selectedLayers.splice(index, 1);
					} else {
						this._selectedLayers.push(layer);
					}
				}
			}
		};

		/**
		 * Toggle the visibility of a class
		 * @param {Number} index
		 * @param {Boolean} hideAllClasses
		 */
		CadFile.prototype.toggleClass = function (index, hideAllClasses) {
			if (index < 0) {
				if (this._selectedClasses.length === this.getOneupClasses().length && hideAllClasses) {
					this._selectedClasses = [];
				} else {
					this._selectedClasses = _.clone(this.getOneupClasses());
				}
			} else {
				var classname = this.getOneupClasses()[index];
				if (classname) {
					var i = _.indexOf(this._selectedClasses, classname);
					if (i < 0) {
						this._selectedClasses.push(classname);
					} else {
						this._selectedClasses.splice(i, 1);
					}
				}
			}
		};

		/**
		 * hide all classes
		 */
		CadFile.prototype.hideClasses = function () {
			this._selectedClasses = [];
		};

		/**
		 * hide all layers
		 */
		CadFile.prototype.hideLayers = function () {
			this._selectedLayers = [];
		};

		/**
		 * Get the union of all the layer classes of the oneups
		 * @returns {Array.String}
		 */
		CadFile.prototype.getOneupClasses = function () {
			if (!this.oneupClasses) { //calculate the layer names when requested the first time
				//Fetch all the unique layer classes first
				var classes = [];
				_.each(this.cadlayers, function (layer) {
					_.each(layer.oneUps, function (oneup) {
						_.each(oneup.cadlayers, function (oneupLayer) {
							var lname = oneupLayer.layerclass;
							if (_.indexOf(classes, lname) < 0) {
								classes.push(lname);
							}
						});
					});
				});

				this.oneupClasses = classes;
			}

			return this.oneupClasses;
		};

		return CadFile;
	});

/**
 * @namespace WebCenter
 */
define('web-viewer/webcenter/WebCenterDataAdapter',["web-viewer/data/AbstractDataAdapter",
		"web-viewer/webcenter/WebCenterDocument",
		"web-viewer/webcenter/WebCenterDocumentVersion",
		"web-viewer/webcenter/WebCenterDocumentPage",
		"web-viewer/data/DataConstants",
		"utils/HTTPUtils",
		"utils/XMLUtils",
		"web-viewer/utils/ViewerUtils",
		"web-viewer/graphics/GraphicsConstants",
		"web-viewer/viewControls/ViewControlsConstants",
		"web-viewer/annotation/AnnotationConstants",
		"web-viewer/cad/CadFile",
		"jquery",
		"underscore"
	],

	function (AbstractDataAdapter,
		WebCenterDocument,
		WebCenterDocumentVersion,
		WebCenterDocumentPage,
		DataConstants,
		HTTPUtils,
		XMLUtils,
		ViewerUtils,
		GraphicsConstants,
		ViewControlsConstants,
		AnnotationConstants,
		CadFile,
		$,
		_
	) {
		"use strict";

		/**
		 * @exports WebCenterDataAdapter
		 * @memberof WebCenter
		 * @class WebCenter specific DataAdapter <<extends AbstractDataAdapter>>
		 * @augments AbstractDataAdapter
		 * @param {Core.ViewerFacade} f
		 * @param {String} URL to the WebCenter instance
		 * @param {String} URL to the viewserver
		 */
		var WebCenterDataAdapter = function (f, instance, viewserver) {
			AbstractDataAdapter.call(this, f, instance);

			/**
			 * @private
			 */
			this.instance = instance;
			/**
			 * @private
			 */
			this.viewserver = viewserver;

			/**
			 * This counter is used to see when parameters return, they are still valid
			 */
			this._parameterUpdateToken = 0;

			this._cadStyles = {};

			/**
			 * Keep the session alive in WebCenter
			 */
			var keepSessionAlive = _.bind(function () {
				f.ajax(this.WebCenter("sessionalive.jsp"), {
					error: function (jqXHR, textStatus, errorThrown) {
						f.i18n.loadBundle("viewer", function () {
							f.emit("error.WARNING", f.i18n.t("WCSessionLostError"), jqXHR, textStatus, errorThrown);
						});
					},
					success: function (data) {
						f.debug.log("Keep WebCenter session alive");
					}
				});
			}, this);

			setInterval(keepSessionAlive, 300000); // every 5 minutes


			// inject the viewserver servlet when a fragmentURL is requested  
			this.f.on("intercept.fragmentURL", _.bind(this.interceptFragmentURL, this));

		};


		WebCenterDataAdapter.prototype = Object.create(AbstractDataAdapter.prototype);
		WebCenterDataAdapter.prototype.constructor = WebCenterDataAdapter;

		/**
		 * Returns the number of documents that are loaded with the documentsIdentifier
		 * @param {String|Object} documentsIdentifier
		 * @returns {Number}
		 */
		WebCenterDataAdapter.prototype.getNumberOfDocuments = function (documentsIdentifier) {
			if (documentsIdentifier === undefined) {
				return 0;
			} else if (_.isString(documentsIdentifier)) {
				return 1;
			} else if (_.isObject(documentsIdentifier)) {
				var data = documentsIdentifier.pagelist.page;
				if (_.isArray(data)) {
					return data.length;
				} else {
					return 1;
				}
			}
		};

		/**
		 * Open document(s)
		 * @param {any} documentsIdentifier - String or JSON object containing information on which document to load
		 * @param {Function} callback - Callback fired with as first argument the document (if loading succeeded, otherwise null) and as second argument whether the document should be made active. Can be fired multiple times when there are multiple documents to be loaded.
		 */
		WebCenterDataAdapter.prototype.openDocuments = function (documentsIdentifier, callback) {
			if (_.isString(documentsIdentifier)) {
				// just a doc version id
				this.loadDocument(documentsIdentifier, callback, true);
			} else if (_.isObject(documentsIdentifier)) {
				// an object
				this.loadDocuments(documentsIdentifier, callback);
			} else if (documentsIdentifier === undefined) {
				//no documents to load
			} else {
				this.f.i18n.loadBundle("viewer", function () {
					this.f.emit("error.SEVERE", this.f.t("UnknownDocumentError"), documentsIdentifier);
				}, this);
			}
		};

		/**
		 * Load basic information on the document, make all the needed documents
		 *
		 * SHOULD ONLY BE USED BY THE DATA ADAPTER
		 *
		 * @param {String} fileIdentifier - Identifier of the file
		 * @param {Function} callback - Executed when document is loaded
		 * @param {Boolean} makeActive - make the loaded document active
		 */
		WebCenterDataAdapter.prototype.loadDocument = function (fileIdentifier, callback, makeActive) {
			var url = this.WebCenter("getdocumentinfo.jsp", {
				docversionid: fileIdentifier
			});
			this.f.ajax(url, {
				success: _.bind(function (data) {
					data = XMLUtils.xml2obj(data);
					if (data.status.value == "0") {
						if (_.isArray(data.status.page)) {
							_.each(data.status.page, _.bind(function (page) {
								this._initDocument(page, callback, makeActive);
							}, this));
						} else {
							this._initDocument(data.status.page, callback, makeActive);
						}
					} else {
						this.f.i18n.loadBundle("viewer", function () {
							this.f.emit("error.SEVERE", this.f.i18n.t("LoadingDocumentInfoError"), data);
						}, this);
						callback(null);
					}

				}, this),
				error: _.bind(function (jqXHR, textStatus, errorThrown) {
					// error while loading the document			
					this.f.i18n.loadBundle("viewer", function () {
						this.f.emit("error.SEVERE", this.f.i18n.t("LoadingDocumentInfoError"), jqXHR, textStatus, errorThrown);
					}, this);
					callback(null);
				}, this)
			});
		};

		/**
		 * Load all documents of the passed object. Sets the link between graphics and CAD files.
		 * SHOULD ONLY BE USED BY THE DATA ADAPTER
		 * @param {Object} documentsIdentifier - object with all document information (specific for webcenter)
		 * @param {Function} callback - for every generated document, the callback is executed
		 */
		WebCenterDataAdapter.prototype.loadDocuments = function (documentsIdentifier, callback) {
			var data = documentsIdentifier.pagelist.page;
			//check if multiple documents are loaded
			if (_.isArray(data)) {
				//multiple documents need to be loaded
				_.each(data, _.bind(function (document, index) {
					// for page lists, if startpageindex is defined, activate it, otherwise, activate page 1
					var makeActive = index === 0;
					if (documentsIdentifier.pagelist.startpageindex !== undefined) {
						makeActive = index === documentsIdentifier.pagelist.startpageindex;
					}
					this._initDocument(document, callback, makeActive);
				}, this));
			} else {
				this._initDocument(data, callback, true);
			}
		};

		/**
		 * Initializes documents with single document information
		 * @param {Object} data
		 * @param {Function} callback
		 * @param {Boolean} makeActive - make a loaded document active
		 */
		WebCenterDataAdapter.prototype._initDocument = function (data, callback, makeActive) {
			var docController = this.f.getDocumentController();
			var graphicsDocument = null;
			var cadDocument = null;
			var activeVersion = data.docversion;
			var document;
			//check if file is graphics file
			if (data.file !== undefined || data.blank !== undefined) {
				//first check if the document doesn't already exist (multiple pages belonging to the same document)
				document = this.getDocumentByDocId(data.docid);
				var docType = DataConstants.documentPageType.GRAPHICS;
				data.hasDefaultData = true;
				if (data.pagecount !== undefined && data.pagecount > 1) {
					data.isMultiPage = true;
				} else {
					data.isMultiPage = false;
				}
				if (document === undefined) {
					graphicsDocument = new WebCenterDocument(this.f, data, docController.getNextIID(), docType, this);
				} else { //this is only the case for multipage and pagelists (versions are in the definition of the document in versionlist
					document.addDocumentPageFromData(data);
					graphicsDocument = document;
				}

			}
			if (data.cad !== undefined) { //it's a CAD file
				if (graphicsDocument !== null) {
					if (data.cadid !== undefined) {
						data.docid = data.cadid;
						data.docversionid = data.cadversionid;
					}
				}
				if (graphicsDocument === null || !graphicsDocument.hasLinkedDocument()) {
					cadDocument = new WebCenterDocument(this.f, data, docController.getNextIID(), DataConstants.documentPageType.CAD, this);
				}
				if (callback && graphicsDocument === null) {
					callback(cadDocument, false); // do not yet make active, see further
				}

			}
			if (data.cad === undefined && data.file === undefined && (data.pageno === undefined || parseInt(data.pageno, 10) === 1) && data.blank != 1) {
				this.loadDocument(data.docversionid, callback, makeActive);
			}
			//check if there is a CAD and Graphics loaded, there is a link between CAD and Graphics
			if (graphicsDocument !== null && cadDocument !== null) {
				// find the correct document page for the link
				var documentPage;
				var allVersions = cadDocument.getAllDocumentVersions();
				var checkPage = function (page) {
					return page && page.get("docversionid") === data.cadversionid;
				};
				for (var i = 1; !documentPage && i <= allVersions.length; i++) {
					var documentVersion = allVersions[i];
					if (documentVersion) {
						documentPage = _.find(documentVersion.getAllPages(), checkPage);
					}
				}
				graphicsDocument.setLinkedDocumentPage(documentPage);
			}

			if (callback && graphicsDocument !== null) {
				callback(graphicsDocument, false); //do not yet make active, maybe there is a link to CAD
			} else if (document) {
				callback(document, false);
			}

			if (makeActive) {
				// If there are no graphics and only CAD, activate the right cad version
				if (graphicsDocument == null && cadDocument !== null && cadDocument.getDocumentVersion(activeVersion) !== undefined && cadDocument.getDocumentVersion(activeVersion).getFirstPage() !== undefined) {
					docController.changeActivePage(cadDocument.getDocumentVersion(activeVersion).getFirstPage());
					// If there are graphics, activate the graphics version
				} else if (document && document.getDocumentVersion() !== undefined && document.getDocumentVersion().getLastPage() !== undefined) {
					// if the makeActive flag is on, but we're dealing with a page instead of a new document, activate the page.
					// since the pages are added in order, we can take the last added page to activate
					// NOTE: pagelist will always have all pages in latest version.
					docController.changeActivePage(document.getDocumentVersion().getLastPage());
				} else if (graphicsDocument !== null && graphicsDocument.getDocumentVersion(activeVersion) !== undefined && graphicsDocument.getDocumentVersion(activeVersion).getFirstPage() !== undefined) {
					//if document wasn't defined yet use graphicsDocument with the first page.
					docController.changeActivePage(graphicsDocument.getDocumentVersion(activeVersion).getFirstPage());
				}
			}
		};

		/**
		 * Construct an URL to WebCenter
		 * @example WebCenter("DownloadLocKeys.jsp", { scope : "viewer" }) - will call the DownloadLocKeys.jsp?scope="viewer" page in WebCenter
		 * @param {String} servlet - Name of the page, relative to WebCenter instance
		 * @param {Object} args - Key,Value pairs as parameters
		 * @returns {String} URL to call
		 */
		WebCenterDataAdapter.prototype.WebCenter = function (servlet, args) {
			return HTTPUtils.constructURL(servlet, args, this.instance);
		};

		/**
		 * Construct an URL to the Viewer
		 * @see WebCenter but for the ViewServer
		 * @param {String} servlet - Name of the page, relative to viewserver root
		 * @param {Object} args - Key,Value pairs as parameters
		 * @returns {String} URL to call
		 */
		WebCenterDataAdapter.prototype.ViewServer = function (servlet, args) {
			return HTTPUtils.constructURL(servlet, args, this.viewserver);
		};

		/**
		 * Get path for images referenced to WebCenter
		 * @returns {String} URL for images referenced to WebCenter
		 */
		WebCenterDataAdapter.prototype.getPathToWCRImages = function () {
			return this.instance + "images/";
		};

		/**
		 * Get path to JS directory of WebCenter
		 * @returns {String} URL to javascript directory of WebCenter
		 */
		WebCenterDataAdapter.prototype.getPathToJSFiles = function () {
			return this.instance + "javascript/";
		};

		/**
		 * Set the base parameters when a FragmentURL is generated
		 * @param {Object} c - context for interceptor
		 */
		WebCenterDataAdapter.prototype.interceptFragmentURL = function (c) {
			// set rectangle parameters
			c.params.x1 += c.rectangle.left();
			c.params.y1 += c.rectangle.top();
			c.params.x2 += c.rectangle.right();
			c.params.y2 += c.rectangle.bottom();

			// resolution
			c.params.res = c.resolution;

			// other settings
			c.params.wc = 6;

			// set correct servlet for viewdata
			c.servlet = this.viewserver + "thumbdata.jsp";
		};


		/**
		 * Check if the given update token still valid
		 * @param   {Number}  updateToken - the token generated by gneerateUpdateToken when de update was requested
		 * @returns {Boolean}
		 */
		WebCenterDataAdapter.prototype.isUpdateTokenValid = function (updateToken) {
			return this._parameterUpdateToken === updateToken;
		};

		/**
		 * Generate a new update token (this makes the previous generated update token invalid)
		 * @returns {Number} updateToken, used for checking wether an asynchronous result is still valid
		 */
		WebCenterDataAdapter.prototype.generateUpdateToken = function () {
			return ++this._parameterUpdateToken;
		};

		/**
		 * We need to cancel the current parameter update
		 */
		WebCenterDataAdapter.prototype.cancelDocumentParametersUpdate = function () {
			this.generateUpdateToken();
		};

		/**
		 * Should the CAD data and style file be requested for a certain page
		 * @param   {AbstractDocumentPage} cadPage - the file for which CAD might need to be fetched
		 * @returns {Boolean} true if CAD data and style still need to be requested
		 */
		WebCenterDataAdapter.prototype.shouldRequestCadDataForPage = function (cadPage) {
			var cadData = cadPage.get("cadData");

			return (!cadData || !cadData.loaded);
		};

		/**
		 * Should the style be requested for this page?
		 * @param   {AbstractDocumentPage} cadPage - the page for we want to have the CAD style
		 * @returns {Boolean}
		 */
		WebCenterDataAdapter.prototype.shouldRequestCadStyleForPage = function (cadPage) {
			return !this._cadStyles[cadPage.get("style")];
		};


		/**
		 * Update the given pages with correct data.
		 *
		 * !! Note !! if two consecutive calls are done, the first (ongoing) call should be cancelled, only the last call result can update the parameters
		 * @param {AbstractDocumentPage} currentPage - the currenct active page
		 * @param {AbstractDocumentPage} differencePage - the current d
		 * @param {AbstractDocumentPage} cadOverlay - the current cad overlay
		 * @param {function} callback - callback to be called when parameter call has finished
		 * @override
		 */
		WebCenterDataAdapter.prototype.updateDocumentParameters = function (currentPage, differencePage, cadOverlay, callback) {
			var updateToken = this.generateUpdateToken();

			if (currentPage.getDocumentVersion().isPageList() || !currentPage.get("hasCorrectMargin")) {
				// we are dealing with a page list, make sure we have the correct page info
				this._fetchPageInfo(currentPage, _.bind(function () {
					if (differencePage && differencePage.getDocumentVersion().isPageList()) {
						// the difference page is also a difference page is also a page list, make sure we have the correct page info
						this._fetchPageInfo(differencePage, _.bind(function () {
							this._fetchParameters(updateToken, currentPage, differencePage, cadOverlay, callback);
						}, this));
					} else {
						this._fetchParameters(updateToken, currentPage, differencePage, cadOverlay, callback);
					}
				}, this));
			} else {
				if (differencePage && (differencePage.getDocumentVersion().isPageList() || !differencePage.get("hasCorrectMargin"))) {
					// the difference page is a pagelist or we don't have correct page info
					this._fetchPageInfo(differencePage, _.bind(function () {
						this._fetchParameters(updateToken, currentPage, differencePage, cadOverlay, callback);
					}, this));
				} else {
					this._fetchParameters(updateToken, currentPage, differencePage, cadOverlay, callback);
				}
			}
		};

		/**
		 * Fetch the parameters
		 * @override
		 * @param {Number}               updateToken      - the unique token for the update
		 * @param {AbstractDocumentPage} currentPage      - current page
		 * @param {AbstractDocumentPage} [differencePage] - difference page
		 * @param {AbstractDocumentPage} [cadOverlay]     - cad Overlay (if defined)
		 * @param {Function}             callback         - callback to be called when parameters are available
		 */
		WebCenterDataAdapter.prototype._fetchParameters = function (updateToken, currentPage, differencePage, cadOverlay, callback) {
			var urlParams;
			var newActivePage;

			if (currentPage.hasGraphics() || (currentPage.isBlankPage() && differencePage && differencePage.hasGraphics())) {
				// either current or difference page has graphics
				if (currentPage.isBlankPage() && differencePage && differencePage.hasGraphics()) {
					// current page is a blank page, switch current en difference so it works correctly
					newActivePage = differencePage;
					differencePage = currentPage;
					currentPage = newActivePage;
				}

				// set requestparameters for current page
				var currentPageMargin = currentPage.getMargins();
				var currentPageTrimboxSize = currentPage.getTrimboxSize();

				urlParams = {
					wc: 6,
					file: currentPage.get("file"),
					ftbtlx: currentPageMargin.left,
					ftbtly: currentPageMargin.top,
					ftbbrx: currentPageTrimboxSize.width + currentPageMargin.left,
					ftbbry: currentPageTrimboxSize.height + currentPageMargin.top
				};

				if (currentPage.getDocumentVersion().isPageList()) {
					urlParams.pageno = currentPage.getPagenum();
				}

				if (currentPage.getDocumentVersion().isMultiPage()) {
					urlParams.fmp = 1;
				}

				// set parameters for difference pagfe

				if (differencePage) {
					var diffPageMargin = differencePage.getMargins();
					var diffPageTrimboxSize = differencePage.getTrimboxSize();

					if (differencePage.getDocumentVersion().isPageList()) {
						urlParams.diffpageno = differencePage.getPagenum();
					}
					urlParams.dtbtlx = diffPageMargin.left;
					urlParams.dtbtly = diffPageMargin.top;
					urlParams.dtbbrx = diffPageTrimboxSize.width + diffPageMargin.left;
					urlParams.dtbbry = diffPageTrimboxSize.height + diffPageMargin.top;
				}

				// do an interception for the parameters
				this.f.emit("intercept.parameters", {
					params: urlParams
				});

				//switch pages on the viewserver (the cuurentpage was a blank page)
				if (newActivePage && urlParams.rs) {
					urlParams.rs = urlParams.rs == 1 ? 2 : 1;
				}

				if (differencePage !== undefined && !differencePage.isBlankPage()) {
					urlParams.diff = differencePage.get("file");
					differencePage.resetParameters();
				}


				if (cadOverlay) {
					// there is a CAD overlay set, load the CAD information if not available
					if (this.shouldRequestCadDataForPage(cadOverlay)) {
						// we still need to request the data
						urlParams.cad = cadOverlay.get("cad");

						if (this.shouldRequestCadStyleForPage(cadOverlay)) {
							urlParams.style = cadOverlay.get("style");
						}
					}
				}
			} else if (currentPage.hasCad()) {
				// we are in CAD mode

				if (this.shouldRequestCadDataForPage(currentPage)) {
					// we still need to fetch CAD data for the current page

					urlParams = {
						wc: 6,
						cad: currentPage.get("cad")
					};

					if (this.shouldRequestCadStyleForPage(currentPage)) {
						urlParams.style = currentPage.get("style");
					}

					if (differencePage && differencePage.hasCad() && this.shouldRequestCadDataForPage(differencePage)) {
						// we also need to fetch cad for the difference
						urlParams.dcad = differencePage.get("cad");
					}
				} else if (differencePage && differencePage.hasCad() && this.shouldRequestCadDataForPage(differencePage)) {
					// we already have the cad data for the active page, but not yet for the difference page

					//switch current and difference page (note this is possible because we are not dealing with graphics!!)
					var realCurrentPage = currentPage;
					currentPage = differencePage;
					differencePage = realCurrentPage;

					urlParams = {
						wc: 6,
						cad: currentPage.get("cad")
					};

					if (this.shouldRequestCadStyleForPage(currentPage)) {
						urlParams.style = currentPage.get("style");
					}
				} else {
					// no data should be fetched, just call the callback
					callback();
				}

			}

			if (urlParams) {
				// we should fetch some data
				urlParams.afs = 1;
				var url = this.ViewServer("parameters.jsp", urlParams);
				this.f.ajax(url, {
					success: _.bind(this._handleParametersResult, this, updateToken, urlParams, currentPage, differencePage, cadOverlay, callback)
				});
			} else {
				// no new data needs to be fetched

				// it could be that CAD data is still being fetched, wait for the data to be fully fetched before calling the callback
				if (callback) {
					this._whenCadDataAvailableForAllActiveFiles(updateToken, currentPage, differencePage, cadOverlay, callback);
				}
			}
		};


		/**
		 * Wait for all ongoing CAD data requests to be finished
		 * !! note !! callback is only executed when the updateToken is still valid
		 * @param {Number}               updateToken      - the unique token for the update
		 * @param {AbstractDocumentPage} currentPage      - current page
		 * @param {AbstractDocumentPage} [differencePage] - difference page
		 * @param {AbstractDocumentPage} [cadOverlay]     - cad Overlay (if defined)
		 * @param {Function}             callback         - callback to be called when all CAD data is loaded
		 */
		WebCenterDataAdapter.prototype._whenCadDataAvailableForAllActiveFiles = function (updateToken, currentPage, differencePage, cadOverlay, callback) {
			this.f.requestModule("cad", _.bind(function () {
				if (this.isUpdateTokenValid(updateToken)) {
					// first find out if there are still ongoing CAD data calls
					var stillWaitingForCad = _.filter([currentPage, differencePage, cadOverlay], function (page) {
						if (page && page.hasCad()) {
							var cadData = page.get("cadData");
							return cadData.requested && cadData.loaded === false;
						} else {
							return false;
						}
					}, this);

					// we only want to fire the callback when the update token is still valid
					var finalcallback = _.bind(function (updateToken, callback) {
						if (this.isUpdateTokenValid(updateToken)) {
							callback();
						}
					}, this, updateToken, callback);


					if (stillWaitingForCad.length > 0) {
						// some CAD data loading is still ongoing, listen on each pending page for the cadData.available event and execute the callback when all is loaded
						finalcallback = _.after(stillWaitingForCad.length, finalcallback);

						_.each(stillWaitingForCad, function (page) {
							page.on("cadData.available", finalcallback);
						});
					} else {
						// no need to wait, we can just call the callback
						finalcallback();
					}
				}
			}, this), true); // don't publish CAD widget!!
		};


		/**
		 * Call to parameters.jsp has finished, handle the CAD files first  (and after that normal graphical parameters)
		 *
		 * @param {Number}               updateToken      - the unique token for the update
		 * @param {Object}               urlParams        - parameters used for parameters.jsp call
		 * @param {AbstractDocumentPage} currentPage      - current page
		 * @param {AbstractDocumentPage} [differencePage] - difference page
		 * @param {AbstractDocumentPage} [cadOverlay]     - cad Overlay (if defined)
		 * @param {Function}             callback         - callback to be called whe
		 * @param {Object}               result           - result returned from parameters.jsp
		 */
		WebCenterDataAdapter.prototype._handleParametersResult = function (updateToken, urlParams, currentPage, differencePage, cadOverlay, callback, result) {
			var res = XMLUtils.xml2obj(result);

			if (!res.status || res.status.value == -1) {
				if (this.isUpdateTokenValid(updateToken)) {
					// there was an error	
					this.f.i18n.loadBundle("viewer", function () {
						this.f.emit("error.SEVERE", this.f.i18n.t("ServerImageParmsError", this.f.i18n.t("file")), res.status.message, res);
					}, this);
				}
			} else {
				if (res.status.parameters.cad) {
					if (res.status.parameters.cad.data) {
						this._loadCadDataFor(cadOverlay ? cadOverlay : currentPage, res.status.parameters.cad.data);
					}

					if (res.status.parameters.cad.ddata) {
						this._loadCadDataFor(differencePage, res.status.parameters.cad.ddata);
					}

					if (res.status.parameters.cad.style) {
						this._loadCadStyle(urlParams.style, res.status.parameters.cad.style);
					}
				}


				this._whenCadDataAvailableForAllActiveFiles(updateToken, currentPage, differencePage, cadOverlay, _.bind(function () {
					this._handleGraphicsParametersResult(updateToken, currentPage, differencePage, cadOverlay, callback, res);
				}, this));
			}
		};
		/**
		 * Execute callback when a certain Cad Style is available
		 * @param {String}   styleName - name of the CAD style
		 * @param {Function} callback  - callback to be called when CAD Style is available
		 * @param {Object}   [binding]   - binding for the callback function
		 */
		WebCenterDataAdapter.prototype._whenCadStyleAvailable = function (styleName, callback, binding) {
			if (binding) {
				callback = _.bind(callback, binding);
			}

			if (this._cadStyles[styleName] && this._cadStyles[styleName] !== "LOADING") {
				callback(this._cadStyles[styleName]);
			} else {
				// wait for the style to become available
				this.once(["cadStyle", styleName, "available"], callback);
			}
		};

		/**
		 * Load a CAD style
		 * @param {String} styleName     - name of the CAD style
		 * @param {String} styleLocation - URL indication the location of the TMP file on the server
		 */
		WebCenterDataAdapter.prototype._loadCadStyle = function (styleName, styleLocation) {
			this._cadStyles[styleName] = "LOADING";

			this._fetchPreviewData(styleLocation, _.bind(function (result) {
				this._cadStyles[styleName] = result.documentElement;
				this._emit(["cadStyle", styleName, "available"], result.documentElement);
			}, this));
		};

		/**
		 * Load CAD data for a page
		 * @param {AbstractDocumentPage} page           - the page for which CAD data should be retrieved
		 * @param {String}               cadPreviewFile - URL of the location of the TMP CAD file on the server
		 * @param {Function}             [callback]       - callback function, called when CAD file is available
		 */
		WebCenterDataAdapter.prototype._loadCadDataFor = function (page, cadPreviewFile, callback) {
			var fileCadData = {
				requested: true,
				loaded: false
			};
			page.set("cadData", fileCadData);
			this._fetchPreviewData(cadPreviewFile, _.bind(function (cadData) {
				this._whenCadStyleAvailable(page.get("style"), function (cadStyle) {
					fileCadData.loaded = true;
					fileCadData.rawCad = cadData.documentElement;
					fileCadData.rawStyle = cadStyle;
					page.set("cadData", fileCadData);
					page.setCadDataAvailable();

					if (callback) {
						callback();
					}
				}, this);
			}, this));
		};

		/**
		 * Fetch PreviewData file from the ViewServer (fetch the data, remove the preview and execute the callback)
		 * @param {String}   name            - URL of the file on the ViewServer
		 * @param {Function} callback        - callback to be called when data is available (the first parameter of the callback is the needed data)
		 * @param {Function} [errorCallback] - callback to be called when an error occurs
		 */
		WebCenterDataAdapter.prototype._fetchPreviewData = function (name, callback, errorCallback) {

			this.f.ajax(this.ViewServer("getcaddata.jsp", {
				url: name
			}), {
				success: _.bind(function (result) {
					try {
						callback(result);
					} finally {
						// make sure to delete the tmp file
						this.f.ajax(this.ViewServer("previewdelete.jsp", {
							url: name
						}), {
							error: _.bind(function (jqXHR, textStatus, errorThrown) {
								this.f.i18n.loadBundle("viewer", function () {
									this.f.emit("error.SEVERE", this.f.i18n.t("PreviewDeleteError"), jqXHR, textStatus, errorThrown);
								}, this);
							}, this)
						});
					}
				}, this),
				error: _.bind(function (jqXHR, textStatus, errorThrown) {
					this.f.i18n.loadBundle("viewer", function () {
						this.f.emit("error.SEVERE", this.f.i18n.t("ServerCadReadError", this.f.i18n.t("file")), jqXHR, textStatus, errorThrown);
					});

					if (errorCallback) {
						errorCallback(jqXHR, textStatus, errorThrown);
					}
				}, this)
			});
		};

		/**
		 * This function is called whe nthe CAD data is already handled, it sets the correct parameters in the pages (page dimensions, file info, channel info)
		 *
		 * @param {Number}               updateToken      - the unique token for the update
		 * @param {AbstractDocumentPage} currentPage      - current page
		 * @param {AbstractDocumentPage} [differencePage] - difference page
		 * @param {AbstractDocumentPage} [cadOverlay]     - cad Overlay (if defined)
		 * @param {Function}             callback         - callback to be called whe
		 * @param {Object}               result           - result returned from parameters.jsp
		 */
		WebCenterDataAdapter.prototype._handleGraphicsParametersResult = function (updateToken, currentPage, differencePage, cadOverlay, callback, res) {
			if (!this.isUpdateTokenValid(updateToken)) {
				this.f.debug.log("_fetchParameters returned dirty result");
			} else {
				// the update token is still valid, we can handle the parameters result
				var currentPageParams = {
					resolutionx: ViewerUtils.parseFloat(res.status.parameters.resolutionx),
					resolutiony: ViewerUtils.parseFloat(res.status.parameters.resolutiony),
					sizex: ViewerUtils.parseFloat(res.status.parameters.sizex),
					sizey: ViewerUtils.parseFloat(res.status.parameters.sizey),
					fileminx: ViewerUtils.parseFloat(res.status.parameters.fileminx),
					fileminy: ViewerUtils.parseFloat(res.status.parameters.fileminy),
					filemaxx: ViewerUtils.parseFloat(res.status.parameters.filemaxx),
					filemaxy: ViewerUtils.parseFloat(res.status.parameters.filemaxy),
					filemedminx: ViewerUtils.parseFloat(res.status.parameters.filemedminx),
					filemedminy: ViewerUtils.parseFloat(res.status.parameters.filemedminy),
					filemedmaxx: ViewerUtils.parseFloat(res.status.parameters.filemedmaxx),
					filemedmaxy: ViewerUtils.parseFloat(res.status.parameters.filemedmaxy),
					imagetype: ViewerUtils.parseFloat(res.status.parameters.imagetype)
				};

				if (cadOverlay) {
					cadOverlay.setParameters({
						resolutionx: ViewerUtils.parseFloat(res.status.parameters.resolutionx),
						resolutiony: ViewerUtils.parseFloat(res.status.parameters.resolutiony),
						sizex: ViewerUtils.parseFloat(res.status.parameters.sizex),
						sizey: ViewerUtils.parseFloat(res.status.parameters.sizey),
						imagetype: -1
					});
				}
				var diffPageParams = {
					resolutionx: ViewerUtils.parseFloat(res.status.parameters.diffresx),
					resolutiony: ViewerUtils.parseFloat(res.status.parameters.diffresy),
					sizex: ViewerUtils.parseFloat(res.status.parameters.sizex),
					sizey: ViewerUtils.parseFloat(res.status.parameters.sizey),
					fileminx: ViewerUtils.parseFloat(res.status.parameters.diffminx),
					fileminy: ViewerUtils.parseFloat(res.status.parameters.diffminy),
					filemaxx: ViewerUtils.parseFloat(res.status.parameters.diffmaxx),
					filemaxy: ViewerUtils.parseFloat(res.status.parameters.diffmaxy),
					filemedminx: ViewerUtils.parseFloat(res.status.parameters.diffmedminx),
					filemedminy: ViewerUtils.parseFloat(res.status.parameters.diffmedminy),
					filemedmaxx: ViewerUtils.parseFloat(res.status.parameters.diffmedmaxx),
					filemedmaxy: ViewerUtils.parseFloat(res.status.parameters.diffmedmaxy)
				};

				currentPageParams.channel = [];
				diffPageParams.channel = [];

				// now automatic channel matching will be done (this is based on the information we get from the viewserver)
				// build for both the current and difference page a list of (ordered) matched channels
				// currentMatched[i] is matched to diffMatched[i]
				var currentPageMatchedChannels = [];
				var diffPageMatchedChannels = [];

				if (res.status.parameters.channel !== undefined && !_.isArray(res.status.parameters.channel)) {
					res.status.parameters.channel = [res.status.parameters.channel];
				}

				_.each(res.status.parameters.channel, _.bind(function (channel, index) {
					channel.serverID = index;

					// parse the dmode
					if (channel.dmode !== undefined || channel.dmode !== null) {
						channel.dmode = parseInt(channel.dmode, 10);
						if (isNaN(channel.dmode)) {
							channel.dmode = null;
						}
					} else {
						// no dmode is set
						channel.dmode = null;
					}

					// convert 0/1 into boolean
					channel.opaque = channel.opaque ? true : false;

					if (channel.dmode === null ||
						channel.dmode === GraphicsConstants.ChannelDiffMode.CURRENT ||
						channel.dmode === GraphicsConstants.ChannelDiffMode.IN_CURRENT_AND_MATCHED_WITH_PREVIOUS) {

						currentPageParams.channel.push(channel);

						if (channel.dmode === GraphicsConstants.ChannelDiffMode.IN_CURRENT_AND_MATCHED_WITH_PREVIOUS) {
							// the channel is in the current document and matched with difference channel add to the matches list
							currentPageMatchedChannels.push(channel);
						}
					} else if (channel.dmode === GraphicsConstants.ChannelDiffMode.PREVIOUS ||
						channel.dmode === GraphicsConstants.ChannelDiffMode.IN_PREVIOUS_AND_MATCHED_WITH_CURRENT) {

						diffPageParams.channel.push(channel);

						if (channel.dmode === GraphicsConstants.ChannelDiffMode.IN_PREVIOUS_AND_MATCHED_WITH_CURRENT) {
							// the channel is in the difference document and matched with a channel from current document add to matches list
							diffPageMatchedChannels.push(channel);
						}
					}
				}, this));

				// link the current page channel to the matched difference page channel 
				// (this link is bi-directional: the matched difference page channel is also linked to the current page channel)
				for (var i = 0; i < currentPageMatchedChannels.length; i++) {
					currentPageMatchedChannels[i].matchedWith = diffPageMatchedChannels[i].serverID;
					diffPageMatchedChannels[i].matchedWith = currentPageMatchedChannels[i].serverID;
				}

				var hasUserAnnotations;
				var hasGVAnnotations;
				var hasAnnotationsObj;
				if (differencePage !== undefined) {
					hasUserAnnotations = res.status.parameters.annotations.diffexists == 1;
					hasGVAnnotations = res.status.parameters.annotations.diffexistsext == 1;
					hasAnnotationsObj = this._createHasAnnotationsObj(hasGVAnnotations, hasUserAnnotations);

					differencePage.setParameters(diffPageParams);
					differencePage.setHasAnnotations(hasUserAnnotations || hasGVAnnotations, hasAnnotationsObj);
				}
				if (currentPage !== undefined) {
					hasUserAnnotations = res.status.parameters.annotations.exists == 1;
					hasGVAnnotations = res.status.parameters.annotations.existsext == 1;
					hasAnnotationsObj = this._createHasAnnotationsObj(hasGVAnnotations, hasUserAnnotations);

					currentPage.setHasAnnotations(hasUserAnnotations || hasGVAnnotations, hasAnnotationsObj);
					currentPage.setParameters(currentPageParams);
				}


				if (callback) {
					callback();
				}
			}
		};

		/**
		 * Creates the object to pass to setHasAnnotations
		 * @return {Object} 
		 */
		WebCenterDataAdapter.prototype._createHasAnnotationsObj = function (hasGVAnnotations, hasUserAnnotations) {
			var hasAnnotationsObj = {};
			hasAnnotationsObj[AnnotationConstants.AnnotationOriginTypes.user.id] = hasUserAnnotations;

			// These are all considered the same as they are in the same file
			hasAnnotationsObj[AnnotationConstants.AnnotationOriginTypes.barcode.id] = hasGVAnnotations;
			hasAnnotationsObj[AnnotationConstants.AnnotationOriginTypes.braille.id] = hasGVAnnotations;
			hasAnnotationsObj[AnnotationConstants.AnnotationOriginTypes.artwork.id] = hasGVAnnotations;
			hasAnnotationsObj[AnnotationConstants.AnnotationOriginTypes.spelling.id] = hasGVAnnotations;
			hasAnnotationsObj[AnnotationConstants.AnnotationOriginTypes.text.id] = hasGVAnnotations;
			hasAnnotationsObj[AnnotationConstants.AnnotationOriginTypes.editor.id] = hasGVAnnotations;
			hasAnnotationsObj[AnnotationConstants.AnnotationOriginTypes.ext.id] = hasGVAnnotations;
			hasAnnotationsObj[AnnotationConstants.AnnotationOriginTypes.acrobat.id] = hasGVAnnotations;
			return hasAnnotationsObj;
		};

		/**
		 * Fetch document parameters with the getdocumentinfo.jsp-call, used for fetching basic information of the doucment page
		 * for blank pages this is the only information that's available.
		 * @param {WebCenterDocumentPage} documentPage
		 * @param {Function} callback
		 */
		WebCenterDataAdapter.prototype._fetchPageInfo = function (documentPage, callback) {
			var url = this.WebCenter("getdocumentinfo.jsp", {
				docid: documentPage.getDocument().getDocumentID(),
				docversionid: documentPage.getDocumentVersion().getDocumentVersionID(),
				pageno: documentPage.getPagenum()
			});
			this.f.ajax(url, {
				success: _.bind(function (data) {
					data = XMLUtils.xml2obj(data);
					var pageData = data.status.page;

					if (pageData !== undefined) {
						documentPage.set("margbot", data.status.page.margbot);
						documentPage.set("margtop", data.status.page.margtop);
						documentPage.set("margleft", data.status.page.margleft);
						documentPage.set("margright", data.status.page.margright);
						documentPage.set("hsize", data.status.page.hsize);
						documentPage.set("vsize", data.status.page.vsize);
						documentPage.set("hasCorrectMargin", true);
						documentPage.set("pageno", data.status.page.pageno);
						documentPage.set("offsetx", data.status.page.offsetx);
						documentPage.set("offsety", data.status.page.offsety);
						if (data.status.page.cadregedit !== undefined) {
							documentPage.set("cadregedit", data.status.page.cadregedit);
							documentPage.set("isCadAlignmentSaved", data.status.page.saved == 1 ? true : false);
						}

						if (pageData.metadata && (pageData.metadata.design || pageData.metadata.layout)) {
							var cadData = pageData.metadata.design || pageData.metadata.layout;
							documentPage.set("grainDirection", cadData.grain_direction);
							documentPage.set("defaultSide", cadData.default_side);
							documentPage.set("dieSide", cadData.die_side);
						}
						documentPage.set("has3DPerspective", pageData.has3DPerspective == "1" ? true : false);
						documentPage.set("canGenerate3D", pageData.canGenerate3D == "1" ? true : false);

						documentPage.set("canEditAttributes", pageData.can_edit_attributes);
						documentPage.set("attributeCategoryID", pageData.attribute_category_id);
						
						documentPage.set("canChangeAnnotations", pageData.can_change_annotations);

					}

					if (documentPage.isBlankPage()) {
						var pageParams = {
							sizex: ViewerUtils.parseFloat(pageData.hsize),
							sizey: ViewerUtils.parseFloat(pageData.vsize),
							filemaxx: ViewerUtils.parseFloat(pageData.hsize),
							filemaxy: ViewerUtils.parseFloat(pageData.vsize)
						};
						documentPage.setHasAnnotations(false);
						documentPage.setParameters(pageParams);
					}
					if (callback) {
						callback(data);
					}
				}, this),
				error: _.bind(function (jqXHR, textStatus, errorThrown) {
					this.f.i18n.loadBundle("viewer", function () {
						this.f.emit("error.SEVERE", this.f.i18n.t("LoadingDocumentInfoError"), jqXHR, textStatus, errorThrown);
					}, this);
					if (callback) {
						callback();
					}
				}, this)
			});
		};

		/**
		 * Makes a deep copy of a page, this means it reinitializes the document and document version. All parameters and data of the document,
		 * document version and document page are copied to the new document, document version and document page
		 *
		 * @param {WebCenterDocumentPage} page - to page thats needs to be copied
		 * @param {WebCenterDocument} [doc] - if no new document needs to be made, add the copied page to this document. If this parameter is undefinied a deep copy of the document will be made
		 * @param {WebCenterDocumentVersion} [docVersion] - if undefined the document version will be copied, if not undefined the copied document page will be added to this document version
		 * @param {Number} [documentID] - the new ID for the document, if not defined same id as document will be used
		 * @returns {WebCenterDocumentPage}
		 */
		WebCenterDataAdapter.prototype.getDeepCopy = function (page, doc, docVersion, documentID) {
			var oldDocumentVersion = page._docVersion; //get the old document 
			var oldDocument = oldDocumentVersion._doc; //get the old documentVersion
			var document = doc;
			if (documentID === undefined) {
				documentID = oldDocument.documentID;
			}
			if (document === undefined) {
				//make a new WebCenterDocument, use the parameters of the old document, dont specify documentData to prevent it making the version and page automatically
				document = new WebCenterDocument(this.f, undefined, documentID, page.documentType, this);
				document._data = $.extend(true, {}, oldDocument._data); //copy all the data of the old document to the new document
			}
			var documentVersion = docVersion;
			if (documentVersion === undefined) {
				//make a new document version, doesn't use any getters because it can be used in new window and not possible to access prototype-methods (internet explorer)
				documentVersion = new WebCenterDocumentVersion(this.f, document, oldDocumentVersion._versionData.docversion, {
					'docversionid': oldDocumentVersion._versionData.docversionid,
					'docRevision': oldDocumentVersion._versionData.docRevision
				}, undefined, this);
			}
			//make a new WebCenterDocumentPage, dont specify data string, this will be copied from the old document page		
			var currentPage = new WebCenterDocumentPage(this.f, documentVersion, page._pagenum, undefined, undefined, this);
			currentPage._data = $.extend(true, {}, page._data); //copy the data of the old document page 
			currentPage._parameters = $.extend(true, {}, page._parameters); //copy the parameters of the old document page
			currentPage._state = $.extend(true, {}, page._state); //copy the state parameters of the old document page

			//copy the CAD data, in case of CAD document page
			if (page._data.cadData) {
				currentPage._data.cadData = _.clone(page._data.cadData);
				if (page._data.cadData.cad) {
					// NOTE: selected layers and such are not copied
					currentPage._data.cadData.cad = new CadFile(page._data.cadData.cad.cstyles, page._data.cadData.cad._xmlData);
				}
			}

			if (currentPage._state.compareOffset) {
				currentPage._state.compareOffset = page._state.compareOffset.copy();
			}
			currentPage.setReaderSpread(page.isReaderSpread());
			//For Graphics+CAD:
			if (docVersion === undefined) {
				document.addDocumentVersion(documentVersion); //add the document version to the document
			}
			documentVersion.addPage(page.getPagenum(), currentPage); //add the page to the document version

			return currentPage;
		};

		/**
		 * Fetches all pages of a document version, callback is used after all pages are initailized and added to the document version.
		 * @param {WebCenterDocumentVersion} docVersionId
		 * @param {Function} callback
		 */
		WebCenterDataAdapter.prototype.getPagesOfDocumentVersion = function (docVersion, callback) {
			var document = docVersion.getDocument();
			var url = this.WebCenter("getdocumentinfo.jsp", {
				docid: document.getDocumentID(),
				docversionid: docVersion.getDocumentVersionID()
			});

			this.f.ajax(url, {
				success: _.bind(function (data) {
					data = XMLUtils.xml2obj(data);
					var pagelist = data.status.page;

					_.each(pagelist, _.bind(_.bind(function (page) {
						document.addDocumentPageFromData(page);
					}, this)));

					if (callback !== undefined) {
						callback();
					}
				}, this),
				error: _.bind(function (jqXHR, textStatus, errorThrown) {
					this.f.i18n.loadBundle("viewer", function () {
						this.f.emit("error.SEVERE", this.f.i18n.t("LoadingDocumentInfoError"), jqXHR, textStatus, errorThrown);
					}, this);
				}, this)
			});
		};

		/**
		 * Returns the document with the WebCenter docId (not the number but UID of the document that's saved in the data of the document)
		 * @param {String} docId - document id of a WebCenter document (obtained by calling getDocumentID())
		 * @returns {WebCenterDocument}
		 */
		WebCenterDataAdapter.prototype.getDocumentByDocId = function (docId) {
			var doc = _.find(this.f.getDocumentController().getAllDocuments(), function (doc) {
				return doc.getDocumentID() === docId;
			});

			if (doc) {
				return doc;
			} else {
				return _.find(this.f.getDocumentController().getLinkedDocuments(), function (doc) {
					return doc.getDocumentID() === docId;
				});
			}
		};

		/**
		 * Load previous versions of the document for a given document page. If the version does not have the page with given pagenumber the version
		 * will not be loaded (only happens for pagelist documents).
		 * @param {Object} data
		 * 	@example
		 * 	{
		 * 		docid: "document id used by WebCenter",
		 * 		docversionid: "document version id used by WebCenter",
		 * 		diffversion: "difference document version id used by WebCenter",
		 * 		pageno: "number of the page to be used
		 * 	}
		 * @param {WebCenterDocument} document
		 * @param {WebCenterDocumentPage} [documentPage]
		 * @param {Function} callback
		 */
		WebCenterDataAdapter.prototype.loadVersionsOfDocumentPage = function (data, document, callback, documentPage) {
			var url = this.WebCenter("getdocumentinfo.jsp", {
				docid: data.docid,
				docversionid: data.docversionid,
				diffversion: data.diffversion,
				pageno: data.pageNumber
			});
			var pageNumber = data.pageNumber;
			var isPageList = data.isPageList;
			var isMultiPage = data.isMultiPage;

			this.f.ajax(url, {
				success: _.bind(function (data) {
					data = XMLUtils.xml2obj(data);
					var versionlist = data.status.page.versionlist;

					//make sure to update margins in the data as this is only provided in this documentinfo call
					if (data.status.page !== undefined && documentPage !== undefined) {
						documentPage.set("margbot", data.status.page.margbot);
						documentPage.set("margtop", data.status.page.margtop);
						documentPage.set("margleft", data.status.page.margleft);
						documentPage.set("margright", data.status.page.margright);
						documentPage.set("hasCorrectMargin", true);
					}

					if (versionlist !== undefined) {
						//create the documentversions from the versionlist:
						if (!_.isArray(versionlist)) { //check if versionlist is an array (more than 2 versions) are object (2 versions)
							versionlist = [versionlist]; //make it an array if it's only one element
						}
						var pages = [];
						_.each(versionlist, _.bind(function (version) {
							//first check if the version already exists, if so set the docversion of this version
							var docVersion = document._documentVersions[parseInt(version.docversion, 10)];
							if (docVersion === undefined) {
								version.isPageList = isPageList;
								version.isMultiPage = isMultiPage;
								docVersion = new WebCenterDocumentVersion(this.f, document, parseInt(version.docversion, 10), version, undefined, this);
								document._documentVersions[parseInt(version.docversion, 10)] = docVersion;
							} else {
								docVersion.set('docversionid', version.docversionid);
								docVersion.set('isMultiPage', version.isMultiPage === 'true');
							}
							//add the page to this documentVersion
							var newPage = new WebCenterDocumentPage(this.f, docVersion, pageNumber, {
								file: version.file,
								versionNumber: version.docversion
							}, undefined, this);
							docVersion.addPage(pageNumber, newPage);
							pages[version.docversion] = newPage;
						}, this));
						callback(pages);
					}
				}, this),
				error: _.bind(function (jqXHR, textStatus, errorThrown) {
					this.f.i18n.loadBundle("viewer", function () {
						this.f.emit("error.SEVERE", this.f.i18n.t("LoadingDocumentInfoError"), jqXHR, textStatus, errorThrown);
					}, this);
				}, this)
			});
		};


		/**
		 * Save the CAD overlay settings
		 * @param {AbstractDocumentPage} graphicsPage - the graphics page in which the CAD reference should be stored
		 * @param {AbstractDocumentPage} cadOverlay   - the CAD document that should be set as reference
		 * @param {Number}               rotation     - rotation of the CAD overlay (0,90,180,270 degrees)
		 * @param {Boolean}              isFlipped    - is the CAD overlay flipped
		 * @param {ShapeUtils.Point}     offset       - the offset for the CAD overlay
		 * @param {Function}             [callback]   - callback function to be called when CAD overlay is saved (first parameter of the callback is a boolean indicating success)
		 */
		WebCenterDataAdapter.prototype.saveCadOverlay = function (graphicsPage, cadOverlay, rotation, isFlipped, offset, callback) {
			if (graphicsPage.canSaveCadOverlay()) {

				var cadOrientation = (4 - (rotation / 90)) % 4;
				if (isFlipped) {
					cadOrientation += 4;
				}

				var newVersionId;
				if (cadOverlay) {
					newVersionId = cadOverlay.get("docversionid");
				}

				var pageMargins = graphicsPage.getMargins();
				var url = this.WebCenter("setcadalignment.jsp", {
					docversionid: graphicsPage.get("docversionid"),
					cadversionid: newVersionId,
					cadorgx: offset.x,
					cadorgy: offset.y,
					cadorientation: cadOrientation,
					hs: graphicsPage.get("hsize"),
					vs: graphicsPage.get("vsize"),
					ml: pageMargins.left,
					mt: pageMargins.top,
					mr: pageMargins.right,
					mb: pageMargins.bottom
				});

				this.f.ajax(url, {
					dataType: "xml",
					success: _.bind(function (data) {
						var result = XMLUtils.xml2obj(data);
						var error = "";

						if (result) {
							if (result.status && result.status.value === "0") {
								this.f.loadBundle("viewer", function () {
									this.f.emit("userMessage", "success", this.f.t("SaveCadOverlaySuccess"), true);
								}, this);

								var oldCadOverlay = graphicsPage.getLinkedPage();
								if (oldCadOverlay !== this.f.getDocumentController().getCadOverlay()) {
									this.f.getDocumentController().createNewLinkedCadFromOverlay();
								}

								//persist data in the linked CAD file
								var newLinkedCad = this.f.getDocumentController().getCadOverlay();
								if (newLinkedCad) {
									newLinkedCad.set("offsetx", offset.x);
									newLinkedCad.set("offsety", offset.y);
									newLinkedCad.set("overlayRotation", rotation);
									newLinkedCad.set("overlayIsFlipped", isFlipped);
								} else {
									this.updateDocumentData(graphicsPage, undefined, true);
								}

								if (callback) {
									callback(true);
								}

								return;
							} else if (result.status === "-1") {
								error = result.status.message;
							}
						}

						// something went wrong...
						this.f.emit("userMessage", "warn", this.f.t("SaveCadOverlayFailed") + " " + error);
						if (callback) {
							callback(false);
						}

					}, this),
					error: _.bind(function () {
						this.f.loadBundle("viewer", function () {
							this.f.emit("userMessage", "warn", this.f.t("SaveCadOverlayFailed"));
						}, this);

						if (callback) {
							callback(false);
						}
					}, this)
				});
			} else {
				this.f.emit("userMessage", "warn", this.f.t("SaveCadOverlayFailed"));
				if (callback) {
					callback(false);
				}
			}
		};

		/**
		 * Show the 3D Collada viewer in a new browser tab
		 * @param {WebCenterDocumentPage} page - Graphics page to show the 3D Viewer for
		 */
		WebCenterDataAdapter.prototype.show3DView = function (page) {
			var url = this.WebCenter("doctypecolladaview.jsp", {
				'docVerID': page.get("docversionid"),
				'sourcePage': "javascript:window.close()"
			});
			window.open(url, '_blank');
		};

		/**
		 * Update the data of the document, also updates the document reference info
		 * @param {AbstractDocumentPage}   page
		 * @param {Function} callback - the callback that is executed when the document data is correctly fetched
		 * @param {Boolean} [doNotActivate] - do not activate any newly created CAD reference document pages
		 */
		WebCenterDataAdapter.prototype.updateDocumentData = function (page, callback, doNotActivate) {
			this._fetchPageInfo(page, _.bind(function (data) {
				if (data.status.page.file && data.status.page.cad && page === this.f.getActivePage()) {
					data = data.status.page; //get the data of the page into the data object
					var linkedCadId = data.cadid; //get the document id of the linked cad
					var oldCadLink = page.getLinkedPage(); //get the old linked CAD page
					data.docid = data.cadid;
					data.docversionid = data.cadversionid;
					if (!this.getDocumentByDocId(linkedCadId)) { //if the CAD document doesn't exist yet      
						//create a new document from the cad info
						var cadDocument = new WebCenterDocument(this.f, data, this.f.getDocumentController().getNextIID(), DataConstants.documentPageType.CAD, this);
						//add this new document as link the currently active document page
						this.f.getDocumentController().addNewLinkedDocument(cadDocument, doNotActivate);
					} else { //the CAD already exists in the system, create a new link to it
						var newCadLink = this.getDocumentByDocId(linkedCadId).getDocumentVersion().getPage();
						if (newCadLink !== oldCadLink) {
							this.f.getDocumentController().createNewLinkedCadFromOverlay(doNotActivate, newCadLink);
						}
						var cadOverlayPage = page.getLinkedPage();
						if (cadOverlayPage) {
							cadOverlayPage.setPageAlignmentData(data);
						}
					}
				} else if (page.getDocument().hasLinkedDocument()) {
					page.getDocument().setLinkedDocumentPage();
					this.f.getDocumentController().changeCadOverlay();
				}

				if (callback) {
					callback();
				}
			}, this));
		};

		return WebCenterDataAdapter;

	});

define('web-viewer/webcenter/WebCenterConfig',["web-viewer/core/AbstractConfig",
		"web-viewer/webcenter/WebCenterDataAdapter"
	],
	function (AbstractConfig,
		WebCenterDataAdapter) {
		/**
		 * @exports WebCenterConfig
		 * @class WebCenter configuration  <<extends AbstractConfig>>
		 * 
		 * configuration class specific for WebCenter usage
		 * @exports WebCenterConfig
		 * @memberof WebCenter
		 * @constructor
		 * @param instance - URL to WebCenter instance
		 * @param viewserver - URL to WebCenter ViewServer
		 */
		var WebCenterConfig = function (instance, viewserver, user) {
			AbstractConfig.call(this);

			/**
			 * @private
			 */
			this.modules = [];
			/**
			 * @private
			 */
			this.instance = instance;
			/**
			 * @private
			 */
			this.user = user;
			/**
			 * @private
			 */
			this.viewserver = viewserver;
		};

		WebCenterConfig.prototype = Object.create(AbstractConfig.prototype);
		WebCenterConfig.prototype.constructor = WebCenterConfig;

		/**
		 * Add a module
		 * @param {Core.AbstractModule} module
		 */
		WebCenterConfig.prototype.addModule = function (module) {
			this.modules.push(module);
		};

		/**
		 * Initialize this adapter
		 * @param {ViewerCore} f
		 * @returns {WebCenterDataAdapter} Initialized adapter
		 */
		WebCenterConfig.prototype.intializeAdapter = function (f) {
			this.adapter = new WebCenterDataAdapter(f, this.instance, this.viewserver);
			return this.adapter;
		};

		/**
		 * Get user
		 * @returns {WebCenterUser}
		 */
		WebCenterConfig.prototype.getUser = function () {
			return this.user;
		};

		/**
		 * Get modules
		 * @returns {Array.<String>}
		 */
		WebCenterConfig.prototype.getModules = function () {
			return this.modules;
		};

		return WebCenterConfig;
	});

define('web-viewer/webcenter/WebCenterUser',["web-viewer/data/User",
		"web-viewer/data/PreferencesConstants",
		"web-viewer/data/PreferencesDefaults",
		"web-viewer/data/DataConstants",
		"web-viewer/core/CoreConstants",
		"underscore"
	],
	function (User,
		PreferencesConstants,
		PreferencesDefaults,
		DataConstants,
		CoreConstants,
			 _) {
		"use strict";

		/**
		 * @exports WebCenterUser
		 * @memberof WebCenter
		 * 
		 * @class
		 * User implementation for WebCenter
		 * @augments Data.User 
		 * @constructor
		 */
		var WebCenterUser = function (id, username, first, last, locale, isAdmin) {
			User.call(this, id, username, first, last, locale);

			this.userIsAdmin = isAdmin;

			this.viewServerID = undefined;
		};


		WebCenterUser.prototype = Object.create(User.prototype);
		WebCenterUser.prototype.constructor = WebCenterUser;

		/**
		 * WebCenter way of displaying user
		 */
		WebCenterUser.prototype.getFullName = function () {
			var fullname = this.getLastName();

			if (this.first) {
				fullname += ", " + this.first;
			}

			fullname += " (" + this.getUsername().toUpperCase() + ")";

			return fullname;
		};

		WebCenterUser.prototype.isAdmin = function () {
			return this.userIsAdmin;
		};


		/**
		 * @override
		 */
		WebCenterUser.prototype.loadOptions = function (onLoadedCallback) {
			this.f.requestModule("featureDetector", _.bind(function (featureDetector) {
				var isMobile = "";
				if (featureDetector.isMobile()) {
					isMobile = "m";
				}

				var userinfo = {
					name: this.getFullName(),
					address: window.screen.width + "x" + window.screen.height + isMobile
				};

				//http://eaw11dd413:8081/ViewServer/userid.jsp?description=Java%20Version:%201.7.0_25&host=EAW11DD413&name=Cagnie%2c%20Ruben%20(RUCA <http://eaw11dd413:8081/ViewServer/userid.jsp?description=Java Version: 1.7.0_25&host=EAW11DD413&name=Cagnie, Ruben (RUCA> )
				var adapter = this.f.getDataAdapter();
				if (!adapter.ViewServer) {
					this.f.i18n.loadBundle("viewer", function () {
						this.f.emit("error.SEVERE", this.f.i18n.t("ServerSettingsReadError"), "Adapter cannot connect to Viewserver");
					}, this);
					this.loadDefaultOptions();
					onLoadedCallback();
					return;
				}
				var userid = adapter.ViewServer("userid.jsp", userinfo);
				this.f.ajax(userid, {
					success: _.bind(function (data) {
						var root = data.documentElement;
						var status = root.getElementsByTagName("value")[0].textContent || root.getElementsByTagName("value")[0].text; //ie9
						if (status == "0") {
							this.viewServerID = root.getElementsByTagName("userid")[0].textContent || root.getElementsByTagName("userid")[0].text; //ie9

							//http://eaw11dd413:8081/ViewServer/getsettings.jsp?userid=1					
							var getsettings = adapter.ViewServer("getsettings.jsp", {
								userid: this.viewServerID
							});
							this.f.ajax(getsettings, {
								success: _.bind(function (data) {
									var root = data.documentElement;
									var status = root.getElementsByTagName("value")[0].textContent || root.getElementsByTagName("value")[0].text; //ie9
									if (status == "0") {
										var settings = root.getElementsByTagName("settings")[0];
										this._handleSettings(settings);
										onLoadedCallback();
									} else {
										this.f.i18n.loadBundle("viewer", function () {
											this.f.emit("error.SEVERE", this.f.i18n.t("ServerSettingsReadError"), root.getElementsByTagName("error")[0].textContent || root.getElementsByTagName("error")[0].text); //ie9
										}, this);
										this.loadDefaultOptions();
										onLoadedCallback();
									}
								}, this),
								error: _.bind(function (jqXHR, textStatus, errorThrown) {
									this.f.i18n.loadBundle("viewer", function () {
										this.f.emit("error.SEVERE", this.f.i18n.t("ServerSettingsReadError"), jqXHR, textStatus, errorThrown);
									}, this);
									this.loadDefaultOptions();
									onLoadedCallback();
								}, this)
							});

						} else {
							this.f.i18n.loadBundle("viewer", function () {
								this.f.emit("error.SEVERE", this.f.i18n.t("ServerSettingsReadError"), root.getElementsByTagName("error")[0] ? root.getElementsByTagName("error")[0].textContent : undefined);
							}, this);
							this.loadDefaultOptions();
							onLoadedCallback();
						}
					}, this),
					error: _.bind(function (jqXHR, textStatus, errorThrown) {
						this.f.i18n.loadBundle("viewer", function () {
							this.f.emit("error.SEVERE", this.f.i18n.t("ServerSettingsReadError"), jqXHR, textStatus, errorThrown);
						}, this);
						this.loadDefaultOptions();
						onLoadedCallback();
					}, this)
				});
			}, this));
		};

		WebCenterUser.prototype.loadDefaultOptions = function () {
			this.setOptions(_.clone(PreferencesDefaults));
		};

		/**
		 * Read out result of getsettings call. Initialize the options for the current user
		 * @param {Element} settings - XML node with the settings
		 */
		WebCenterUser.prototype._handleSettings = function (settings) {
			var preferences = _.invert(PreferencesConstants);

			var options = _.clone(PreferencesDefaults);

			var child = settings.firstChild;
			while (child) {
				if (child.nodeType == 1) {
					var type = child.nodeName;

					var id = child.getElementsByTagName("id")[0].textContent || child.getElementsByTagName("id")[0].text; //ie9
					var value = child.getElementsByTagName("value")[0].textContent || child.getElementsByTagName("value")[0].text; //ie9
					if (type == "settingint") {
						value = parseInt(value, 10);
					}

					// this check will ignore settingsint that should by charsequences
					if (type == "settingchar" || !_.contains(DataConstants.charsequencePreferences, preferences[id])) {
						options[preferences[id]] = value;
					}
				}
				child = child.nextSibling;
			}

			// set unit options based on units property
			var units = this.f.getUnits();
			if (units === CoreConstants.SizeUnits.INCH) {
				options["PREFERENCES_UNIT_SIZE"] = CoreConstants.SizeUnits.INCH;
				options["PREFERENCES_UNIT_RESOLUTION"] = CoreConstants.ResolutionUnits.PPI;
				options["PREFERENCES_UNIT_SCREENRULING"] = CoreConstants.ResolutionUnits.PPI;
			} else {
				options["PREFERENCES_UNIT_SIZE"] = CoreConstants.SizeUnits.MM;
				options["PREFERENCES_UNIT_RESOLUTION"] = CoreConstants.ResolutionUnits.PPCM;
				options["PREFERENCES_UNIT_SCREENRULING"] = CoreConstants.ResolutionUnits.PPCM;
			}

			this.initializeOptions(options);
		};

		/**
		 * Persist the settings by POST-ing an XML to the setsettings.jsp servlet
		 */
		WebCenterUser.prototype.persistOptions = function (options, onPersistedCallback) {
			var adapter = this.f.getDataAdapter();
			if (!adapter.ViewServer) {
				this.f.i18n.loadBundle("viewer", function () {
					this.f.emit("error.SEVERE", this.f.i18n.t("ServerSettingsWriteError"), "Adapter cannot connect to Viewserver");
				}, this);
				if (onPersistedCallback) {
					onPersistedCallback(false);
				}
				return;
			}

			//Create a settings XML document
			var settings = document.implementation.createDocument("", 'settings', null);
			_.each(options, function (value, key) {
				var setting = settings.createElementNS(null, _.isNumber(value) ? "settingint" : "settingchar");

				var idNode = settings.createElementNS(null, "id");
				idNode.textContent = PreferencesConstants[key];

				var valueNode = settings.createElementNS(null, "value");
				valueNode.textContent = value;

				setting.appendChild(idNode);
				setting.appendChild(valueNode);

				settings.documentElement.appendChild(setting);

			});
			
			// because of a bug in IE9, we need to serialize the XML Document to a string ourselves
			// if we don't do this step, IE9 might add newline characters at places where spaces are being used
			// this can break JSONs stored in the user preferences
			var serializedSettings = new XMLSerializer().serializeToString(settings);
	
			var url = adapter.ViewServer("setsettings.jsp", {
				userid: this.viewServerID
			});
			this.f.ajax(url, {
				error: _.bind(function (jqXHR, textStatus, errorThrown) {
					this.f.i18n.loadBundle("viewer", function () {
						this.f.emit("error.SEVERE", this.f.i18n.t("ServerSettingsWriteError"), jqXHR, textStatus, errorThrown);
					}, this);
					if (onPersistedCallback) {
						onPersistedCallback(false);
					}
				}, this),
				success: function (data, textStatus, jqXHR) {
					if (onPersistedCallback) {
						onPersistedCallback(true);
					}
				},
				processData: false,
				contentType: 'text/xml',
				data: serializedSettings,
				type: 'POST'
			});
		};


		return WebCenterUser;

	});

define('web-viewer/webcenter/WebCenterModule',["web-viewer/webcenter/WebCenterConfig",
		"web-viewer/webcenter/WebCenterDataAdapter",
		"web-viewer/webcenter/WebCenterDocument",
		"web-viewer/webcenter/WebCenterDocumentPage",
		"web-viewer/webcenter/WebCenterDocumentVersion",
		"web-viewer/webcenter/WebCenterModule",
		"web-viewer/webcenter/WebCenterUser"
	],
	function () {
		// this module should have all the commonly used webcenter components as a dependency
	});
