
define('packaging-content-management/text-content-utils/LanguageUtils',["utils/Map",
        "underscore",
        "jquery"],
	function (Map,
			_,
			$) {
	
		"use strict";
		
		var LanguageUtils = function () {
			
		};
		
		
		LanguageUtils.prototype.removeFrom = function (langs, langsToRemove) {
			for (var i=0; i <langs.length; i++) {
				for (var j=0; j <langsToRemove.length; j++) {
					if (langs[i].langCode === langsToRemove[j].langCode) {
						langs.splice(i, 1); // remove the entry
					}
				}
			}
		};
		
		LanguageUtils.prototype.getLangForDescription = function (langs, langDescription) {
			for (var i=0; i <langs.length; i++) {
				if (langs[i].langDesc === langDescription) {
					return langs[i];
				}
			}
		};
		
		LanguageUtils.prototype.getLangForCode = function (langs, langCode) {
			for (var i=0; i <langs.length; i++) {
				if (langs[i].langCode === langCode) {
					return langs[i];
				}
			}
		};
		
		LanguageUtils.prototype.getLangsForCodes = function (langs, langCodes) {
			var matchedLangs = [];
			for (var i=0; i <langCodes.length; i++) {
				var lang = this.getLangForCode(langs, langCodes[i]);
				if (lang) {
					matchedLangs.push(lang);
				}
			}
			return matchedLangs;
		};
		
		LanguageUtils.prototype.isAProjectAttrLang = function(projectLangs, langCode) {
			for(var i=0; i<projectLangs.length; i++) {
				if(projectLangs[i].langCode === langCode) {
					return true;
					break;
				}
			}
			return false;
		};
		
		/**
		 * Combines two array of language objects in single Map. <br>
		 * This method is being used basically for merging master and translation languages.
		 * @param masterLanguages {Array of objects}	- 	{langCode : "en-US", langDesc : "English - United States", langFlag : "US.png", langIndex : "1"}
		 * @param translationLanguages {Array of objects}	-	{langCode : "nl-BE", langDesc : "Dutch - Belgium", langFlag : "BE.png", langIndex : "2"}
		 * @returns {Map} - Map  with language code as key and language object as value
		 * 	E.g. key = "en-US" and value = {langDesc : "English - United States", langFlag : "US.png", langIndex : "1"}
		 */
		LanguageUtils.prototype.mergeBothLanguagesTogether = function (masterLanguages, translationLanguages) {
			var completeBothLanguage = new Map();
			
			$(masterLanguages).each(function(lang) {
				completeBothLanguage.add(masterLanguages[lang].langCode, {langCode : masterLanguages[lang].langCode, langDesc : masterLanguages[lang].langDesc, langFlag : masterLanguages[lang].langFlag, langIndex : masterLanguages[lang].langIndex});
			});
			
			$(translationLanguages).each(function(lang) {
				completeBothLanguage.add(translationLanguages[lang].langCode, {langCode : translationLanguages[lang].langCode, langDesc : translationLanguages[lang].langDesc, langFlag : translationLanguages[lang].langFlag, langIndex : translationLanguages[lang].langIndex});
			});
			return completeBothLanguage;
		};
		
		/**
		 * This method will search for language code in map of Languages and returns lang object 
		 * {langCode : "en-US", langDesc : "English - United States", langFlag : "US.png", langIndex : "1"} 
		 * @param langsMap - Map of languages where language code is key and language object is value
		 * @ returns LangObject
		 */
		LanguageUtils.prototype.getLangForCodeFromMap = function (langsMap, languageCode) {
			if(languageCode == "")
				return;
			var langObj ; 
			_.each(langsMap.entries(), _.bind(function(langCode){
				//if languageCode is found in map, set the langObj and return false to break the loop
				if(langCode == languageCode){
					langObj = langsMap.get(langCode);
					return false;
				}
			}));
			
			return langObj;
		};
		
		return new LanguageUtils();
		
});

define('web-viewer/textContent/DynContentEntryView',["jquery",
		"underscore",
		"packaging-content-management/ContentConstants"],
	function (
		$,
		_,
		ContentConstants) {
		"use strict";

		var DynContentEntryView = function (masterText, transText, masterStatus, transStatus, masterLangCode, transLangCode, languagesList, elementTypeColor, elementHumanReadable, isUnsaved) {
			this._masterText = masterText;
			this._transText = transText;
			this._masterStatus = masterStatus;
			this._transStatus = transStatus;
			this._languagesList = languagesList;
			this._masterLangCode = masterLangCode;
			this._transLangCode = transLangCode;
			this._elementTypeColor = elementTypeColor;
			this._elementHumanReadable = elementHumanReadable;
			this._isUnsaved = isUnsaved;

			// All the callbacks that need to be executed when the view is clicked.
			this._onClicked = [];
			this._onElementClicked = [];

			// Create the wrapping div.
			this._wrapper = $("<div>", {
				"class": "DynContentEntry"
			});

			// Create the wrapping div.
			this.init();

			// When the wrapping div is clicked ...
			this._wrapper.on("click", function () {
				for (var i = 0; i < this._onClicked.length; i++) {
					this._onClicked[i]();
				}
			}.bind(this));
		};

		DynContentEntryView.prototype.init = function () {

			// Empty the wrapper.
			this._wrapper.empty();

			// Create a lang table.
			this._langTable = $("<table>", {
				"class": "LangTable"
			});

			// Add the "Unsaved"  class if the entry is currently unsaved.
			if (this._isUnsaved) {
				this._langTable.addClass("Unsaved");
			}

			// Create the elements for the master text
			this._masterTextRow = $("<tr>", {
				"class": "MasterTextRow"
			});
			this._masterTextFlagCell = $("<td>", {
				"class": "MasterTextFlagCell"
			});
			this._masterTextFlagCell.append($("<img>", {
				"src": this._getLangFlag(true, this._masterLangCode)
			}));
			this._masterTextStateCell = $("<td>", {
				"class": "MasterTextStateCell"
			});
			this._masterTextStateCell.append($("<i>", {
				"class": this._getStateIconClass(this._masterStatus)
			}));
			this._masterTextContentCell = $("<td>", {
				"class": "MasterTextContentCell"
			});
			this._masterTextContentContainer = $("<div>", {
				"class": "MasterTextContentContainer"
			});

			// If master language is still not added, then we get master text as undefined
			if (undefined !== this._masterText && this._masterText.length > 0) {
				this._masterTextContentContainer.text(this._masterText);
			} else {
				this._masterTextContentContainer.html("&nbsp;");
			}
			this._masterTextContentCell.append(this._masterTextContentContainer);
			this._masterTextFlagCell.on("click", function (event) {
				event.stopPropagation();
				this._execOnElementClicked(this._masterLangCode);
			}.bind(this));
			this._masterTextStateCell.on("click", function (event) {
				event.stopPropagation();
				this._execOnElementClicked(this._masterLangCode);
			}.bind(this));
			this._masterTextContentContainer.on("click", function (event) {
				event.stopPropagation();
				this._execOnElementClicked(this._masterLangCode);
			}.bind(this));

			// Add them to the wrapper.
			this._langTable.append(this._masterTextRow);
			this._masterTextRow.append(this._masterTextFlagCell);
			this._masterTextRow.append(this._masterTextStateCell);
			this._masterTextRow.append(this._masterTextContentCell);

			if (this._transLangCode) {
				// Create the elements for the translation
				this._transTextRow = $("<tr>", {
					"class": "TransTextRow"
				});
				this._transTextFlagCell = $("<td>", {
					"class": "TransTextFlagCell"
				});
				this._transTextFlagCell.append($("<img>", {
					"src": this._getLangFlag(false, this._transLangCode)
				}));
				this._transTextStateCell = $("<td>", {
					"class": "TransTextStateCell"
				});
				this._transTextStateCell.append($("<i>", {
					"class": this._getStateIconClass(this._transStatus)
				}));
				this._transTextContentCell = $("<td>", {
					"class": "TransTextContentCell"
				});
				this._transTextContentContainer = $("<div>", {
					"class": "TransTextContentContainer"
				});
				if (this._transText.length > 0) {
					this._transTextContentContainer.text(this._transText);
				} else {
					this._transTextContentContainer.html("&nbsp;");
				}
				this._transTextContentCell.append(this._transTextContentContainer);
				this._transTextStateCell.on("click", function (event) {
					event.stopPropagation();
					this._execOnElementClicked(this._transLangCode);
				}.bind(this));
				this._transTextFlagCell.on("click", function (event) {
					event.stopPropagation();
					this._execOnElementClicked(this._transLangCode);
				}.bind(this));
				this._transTextContentContainer.on("click", function (event) {
					event.stopPropagation();
					this._execOnElementClicked(this._transLangCode);
				}.bind(this));

				// Add them to the wrapper.
				this._langTable.append(this._transTextRow);
				this._transTextRow.append(this._transTextFlagCell);
				this._transTextRow.append(this._transTextStateCell);
				this._transTextRow.append(this._transTextContentCell);
			}

			// If the element type of this contentstatement has a particular color ...
			if (this._elementTypeColor) {
				// Create a new wrapping table that will hold the color handle
				var mainTable = $("<table>", {
					"class": "MainTable"
				});
				var mainRow = $("<tr>", {
					"class": "MainRow"
				});
				var colorCell = $("<td>", {
					"class": "ColorCell",
					"style": "background-color: " + this._elementTypeColor + "; cursor: pointer"
				});
				var langCell = $("<td>", {
					"class": "LangCell"
				});


				// When hovering over the color cell, make it lighter ...
				colorCell.hover(function () {
					colorCell.css("opacity", 0.3);
				}.bind(this), function () {
					colorCell.css("opacity", 1);
				}.bind(this));

				// Add the element type description as a tooltip.
				if (this._elementHumanReadable) {
					colorCell.attr('title', this._elementHumanReadable);
				}

				// Put everything together.
				langCell.append(this._langTable);
				mainRow.append(colorCell);
				mainRow.append(langCell);
				mainTable.append(mainRow);
				this._wrapper.append(mainTable);
			} else {
				this._wrapper.append(this._langTable);
			}
		};

		DynContentEntryView.prototype.highlight = function () {
			this._wrapper.addClass("highlight");
			// Try to scroll the element into view.
			try {
				this._wrapper[0].scrollIntoView();
			} catch (e) {}
		};

		DynContentEntryView.prototype.unHighlight = function () {
			this._wrapper.removeClass("highlight");
		};

		DynContentEntryView.prototype.setMasterText = function (masterLang, masterPlainText, masterStatus) {
			this._masterText = masterPlainText;
			this._masterLangCode = masterLang;
			this._masterStatus = masterStatus;
			this.init();
		};

		DynContentEntryView.prototype.setTransText = function (transLang, transPlainText, transStatus) {
			this._transText = transPlainText;
			this._transLangCode = transLang;
			this._transStatus = transStatus;
			this.init();
		};

		DynContentEntryView.prototype.setUnsaved = function (isUnsaved) {
			this._isUnsaved = isUnsaved;
			// Add the "Unsaved"  class if the entry is currently unsaved.
			if (this._isUnsaved) {
				this._langTable.addClass("Unsaved");
			} else {
				this._langTable.removeClass("Unsaved");
			}
		};

		DynContentEntryView.prototype._getStateIconClass = function (state) {
			if (state == ContentConstants.State.DRAFT) {
				return "icon-WCR_Text-Status-Draft";
			} else if (state == ContentConstants.State.READY_FOR_APPROVAL) {
				return "icon-WCR_Text-Status-Pending";
			} else if (state == ContentConstants.State.APPROVED) {
				return "icon-WCR_Text-Status-Approved icon-WCR_Approve";
			} else if (state == ContentConstants.State.REJECTED) {
				return "icon-WCR_Text-Status-Rejected icon-WCR_Reject";
			}
		};

		DynContentEntryView.prototype._getLangFlag = function (isMaster, langCode) {
			if (isMaster) {
				for (var i = 0; i < this._languagesList.allMasterLangs.length; i++) {
					if (this._languagesList.allMasterLangs[i].langCode == langCode) {
						var mFlagName = this._languagesList.allMasterLangs[i].langFlag;
						return "./images/country_flags/" + mFlagName;
					}
				}
			} else {
				for (var j = 0; j < this._languagesList.allTranslationLangs.length; j++) {
					if (this._languagesList.allTranslationLangs[j].langCode == langCode) {
						var tFlagName = this._languagesList.allTranslationLangs[j].langFlag;
						return "./images/country_flags/" + tFlagName;
					}
				}
			}
		};

		DynContentEntryView.prototype._execOnElementClicked = function (langCode) {
			for (var i = 0; i < this._onElementClicked.length; i++) {
				this._onElementClicked[i](langCode);
			}
		};

		DynContentEntryView.prototype.onElementClicked = function (callback) {
			this._onElementClicked.push(callback);
		};

		DynContentEntryView.prototype.onClicked = function (callback) {
			this._onClicked.push(callback);
		};

		DynContentEntryView.prototype.getJQElement = function () {
			return this._wrapper;
		};

		return DynContentEntryView;
	});

define('web-viewer/textContent/DynContentEntry',["jquery",
		"underscore",
		"packaging-content-management/ContentConstants",
		"./DynContentEntryView"],
	function (
		$,
		_,
		ContentConstants,
		DynContentEntryView) {
		"use strict";

		var DynContentEntry = function (contentStatement, masterLang, transLang, languagesList, elementTypeList) {

			this._contentStatement = contentStatement;
			this._masterElement = this._contentStatement.getElement(masterLang, true);
			this._transElement = this._contentStatement.getElement(transLang, false);
			this._languagesList = languagesList;

			// The callbacks that need to be executed when a statement is clicked.
			this._onSelected = [];
			this._onElementSelected = [];

			var masterPlainText = this._getPlainText(this._masterElement);
			var transPlainText = this._getPlainText(this._transElement);
			var masterStatus;
			var transStatus;
			var masterElementLang;
			var transElementLang;
			var elementTypeColor;
			var elementHumanReadable = this._contentStatement.elementType;

			if (this._masterElement) {
				masterStatus = this._masterElement.state;
				// If somebody already reviewed, take that state to display.
				if (this._masterElement.approval.length > 0) {
					masterStatus = this._masterElement.approval[this._masterElement.approval.length - 1].state;
				}
				masterElementLang = this._masterElement.language;
			}
			if (this._transElement) {
				transStatus = this._transElement.state;
				// If somebody already reviewed, take that state to display.
				if (this._transElement.approval.length > 0) {
					transStatus = this._transElement.approval[this._transElement.approval.length - 1].state;
				}
				transElementLang = this._transElement.language;
			}
			var elementType = elementTypeList[this._contentStatement.elementType];
			if (elementType) {
				elementTypeColor = elementType.color;
				elementHumanReadable = elementType.elementHumanReadable;
			}

			// Create the view.
			this._view = new DynContentEntryView(masterPlainText, transPlainText, masterStatus, transStatus, masterElementLang, transElementLang, languagesList, elementTypeColor, elementHumanReadable, contentStatement.getContentChanged());

			// When the whole view is clicked
			this._view.onClicked(function () {
				for (var i = 0; i < this._onSelected.length; i++) {
					this._onSelected[i]();
				}
			}.bind(this));

			// When an element in the view is clicked
			this._view.onElementClicked(function (langCode) {
				for (var i = 0; i < this._onElementSelected.length; i++) {
					this._onElementSelected[i](langCode);
				}
			}.bind(this));
		};

		DynContentEntry.prototype.getContentStatement = function () {
			return this._contentStatement;
		};

		DynContentEntry.prototype.update = function (textElement) {
			if (textElement == this._masterElement) {
				this.setMasterLang(this._masterElement.language);
			} else if (textElement == this._transElement) {
				this.setTransLang(this._transElement.language);
			}
			if (textElement.isChanged()) {
				this._view.setUnsaved(true);
			} else {
				this._view.setUnsaved(false);
			}
		};

		DynContentEntry.prototype.onSelected = function (callback) {
			this._onSelected.push(callback);
		};

		DynContentEntry.prototype.onElementSelected = function (callback) {
			this._onElementSelected.push(callback);
		};

		DynContentEntry.prototype.highlight = function () {
			this._view.highlight();
		};

		DynContentEntry.prototype.unHighlight = function () {
			this._view.unHighlight();
		};

		DynContentEntry.prototype.setMasterLang = function (masterLang) {
			this._masterElement = this._contentStatement.getElement(masterLang, true);
			var masterPlainText = this._getPlainText(this._masterElement);
			var masterStatus = this._masterElement.state;
			// If somebody already reviewed, take that state to display.
			if (this._masterElement.approval.length > 0) {
				masterStatus = this._masterElement.approval[this._masterElement.approval.length - 1].state;
			}
			this._view.setMasterText(masterLang, masterPlainText, masterStatus);
		};

		DynContentEntry.prototype.setTransLang = function (transLang) {
			this._transElement = this._contentStatement.getElement(transLang, false);
			var transPlainText = this._getPlainText(this._transElement);
			var transStatus = this._transElement.state;
			// If somebody already reviewed, take that state to display.
			if (this._transElement.approval.length > 0) {
				transStatus = this._transElement.approval[this._transElement.approval.length - 1].state;
			}
			this._view.setTransText(transLang, transPlainText, transStatus);
		};

		DynContentEntry.prototype._getPlainText = function (element) {
			if (element) {
				var plainText = element.content.content;
				if (this._contentStatement.type == ContentConstants.TextType.RICHTEXT) {
					plainText = $("<div>").html(plainText).text();
				}
				return plainText.replace(/(\r\n|\n|\r)/gm, "");
			} else {
				return undefined;
			}
		};

		DynContentEntry.prototype.getJQElement = function () {
			return this._view.getJQElement();
		};

		return DynContentEntry;
	});

define('web-viewer/textContent/DynContentWidget',["web-viewer/layout/AbstractWidget",
		"web-viewer/layout/LayoutConstants",
		"web-viewer/layout/component/MiniButton",
		"web-viewer/layout/component/DropdownMiniButton",
		"packaging-content-management/ContentConstants",
		"packaging-content-management/text-content-utils/LanguageUtils",
		"utils/ShapeUtils",
		"jquery",
		"underscore",
		"web-viewer/textContent/DynContentEntry",
		"utils/webi18n!Viewer_Strings"],
	function (AbstractWidget,
		LayoutConstants,
		MiniButton,
		DropdownMiniButton,
		ContentConstants,
		languageUtils,
		ShapeUtils,
		$,
		_,
		DynContentEntry,
		i18nViewerStrings) {
		"use strict";

		/**
		 * @exports DynContentWidget
		 * @class DynContentWidget
		 * @param {TextContentModule} textContentModule - 
		 * @param {Core.ViewerFacade} f - 
		 * @param {Array} contentStatementList
		 * @param {Array} elementTypeList
		 * @param {Object} languagesList
		 */
		var DynContentWidget = function DynContentWidget(textContentModule, f, contentStatementList, elementTypeList, languagesList) {
			AbstractWidget.call(this, f, LayoutConstants.Widget.TEXTCONTENT);

			// The module.
			this._textContentModule = textContentModule;
			this._dynContentModel = textContentModule.dynContentModel;

			// Viewer facade
			this.f = f;

			// Set the icon
			this.setIcon("icon icon-WCR_Measure-Text");

			// Array of complete element types
			this._elementTypeList = elementTypeList;

			// Array of TextContent documents referenced to the artwork
			this._contentStatementList = contentStatementList;
			this._sortContentStatementList();

			// Array of master and translation languages
			this._languagesList = languagesList;
			this._projectMasterLangs = languagesList.projectMasterLangs;
			this._projectTranslationLangs = languagesList.projectTranslationLangs;
			this._allMasterLangs = languagesList.allMasterLangs;
			this._allTranslationLangs = languagesList.allTranslationLangs;

			// Array of langues
			this._allAvailableMasterLangs = [];
			this._allAvailableTranslationLangs = [];
			this._findAllAvailableLanguages();

			// The wrapping div
			this._wrapper = $('<div>').addClass("DynContentViewerWidget");

			// Create the widget buttons and dropdowns
			this._saveButton = null;
			this._languageSelectDropdown = null;
			this._statusFilterDropdown = null;
			this._fullWidthViewButton = null;

			// The selected languages/state to filter on.
			this._selectedMasterLanguage = '';
			this._selectedTranslationLanguage = '';
			this._selectedState = ContentConstants.State.NONE;

			// The create dynamic content entries ...
			this._dynContentEntries = [];

			// Options for the widget
			this._options = {};
			this._options.viewOptions = {
				widgetview: 'viewer'
			};
			this._options.f = this.f;

			// Set the widget title
			this.setTitle(i18nViewerStrings.t("PCMTextWidgetTitle"));

			// Construct the widget ...
			this._constructWidget();

			/*
			 * When a dynamic content annotation was selected ...
			 */
			this._dynContentModel.on("selectedDynContents", _.bind(function (dynamicContentAnnotations, isDirectSelect) {
				// Extract the dynamic content objects from the dynamic content annotations.
				var selectedDynContents = [];
				for (var i = 0; i < dynamicContentAnnotations.length; i++) {
					selectedDynContents.push(dynamicContentAnnotations[i].getDynContent());
				}
				// highlight the selected dynamic contents.
				if (selectedDynContents.length > 0) {
					this.highlight(selectedDynContents, isDirectSelect);
				}
			}, this));

			/*
			 * When a new unsaved Text Content object was added ...
			 */
			this._dynContentModel.on("unsavedTextContent", _.bind(function (dynamicContentAnnotations, isDirectSelect) {
				this.updateSaveButton();
			}, this));

			/*
			 * When a text element was changed ...
			 */
			this._dynContentModel.on("textElementChanged", _.bind(function (textElement) {

				// Find the affected dynContentEntry.
				var affected;
				for (var i = 0; i < this._dynContentEntries.length; i++) {
					var contentStatement = this._dynContentEntries[i].getContentStatement();
					if (textElement.parentStatement == contentStatement) {
						affected = this._dynContentEntries[i];
					}
				}

				// If an affected entry was found.
				if (affected) {
					// Update it.
					affected.update(textElement);
				}

			}, this));

			/*
			 * When the content of the model changed. 
			 */
			this._dynContentModel.on("modelContentChanged", _.bind(function () {
				// Extract the text content documents from the annotations in the model.
				var annotations = this._dynContentModel.getDynContents();
				this._contentStatementList = [];
				for (var i = 0; i < annotations.length; i++) {
					if (annotations[i].getDynContent().getDynContentType() == "Text") {
						var element = annotations[i].getDynContent().getTextElement();
						var contentStatement = element.parentStatement;
						if ($.inArray(contentStatement, this._contentStatementList) < 0) {
							this._contentStatementList.push(contentStatement);
						}
					}
				}
				// Update the widget for these new content statements.
				this._sortContentStatementList();
				this._createDynContentEntries();
				this.updateSaveButton();
			}, this));

		};

		DynContentWidget.prototype = Object.create(AbstractWidget.prototype);
		DynContentWidget.prototype.constructor = DynContentWidget;
		DynContentWidget.widgetName = LayoutConstants.Widget.TEXTCONTENT;

		/**
		 * 
		 */
		DynContentWidget.prototype._constructWidget = function () {
			this._createSaveButton();
			this._createLanguageSelectionButton();
			this._createStatusFilterButton();
			this._createFullWidthViewButton();
			this._createDynContentEntries();
		};

		DynContentWidget.prototype._createSaveButton = function () {
			this._saveButton = new MiniButton(this.f, "saveDynContents", "icon-WCR_Save", true, _.bind(function () {
				// start saving annotations
				this._dynContentModel.save();
			}, this), "fontIconDisabled icon-WCR_Save");
			this.updateSaveButton();
		};

		DynContentWidget.prototype._createLanguageSelectionButton = function () {
			this._languageSelectDropdown = new DropdownMiniButton(this.f, "pcmLangSelection", "icon-WCR_Text-Translation2");
			this._languageSelectDropdown.setTooltip("viewer", "PCMSelLangTT");
			this._languageSelectDropdown.addButtonClass("pcmSelectLang");

			var onMasterLangChange = _.bind(function (selectedLangCode, toggle) {
				for (var i = 0; i < this._allAvailableMasterLangs.length; i++) {
					this._languageSelectDropdown.toggleOption(this._allAvailableMasterLangs[i].langCode, selectedLangCode === this._allAvailableMasterLangs[i].langCode);
					if (selectedLangCode === this._allAvailableMasterLangs[i].langCode) {
						this._selectedMasterLanguage = selectedLangCode;
					}
				}
				this._updateStatusFilterCount();
				this._createDynContentEntries();
			}, this);

			if (this._allAvailableMasterLangs && this._allAvailableMasterLangs.length > 0) {
				this._languageSelectDropdown.addHeader(this.f.i18n.t("PCMMasterLangMenuHeader"));

				for (var i = 0; i < this._allAvailableMasterLangs.length; i++) {
					var language = this._allAvailableMasterLangs[i];

					var selected = false;
					if (i === 0) {
						this._selectedMasterLanguage = language.langCode;
						selected = true;
					}

					this._languageSelectDropdown.addOption({
						name: language.langCode,
						content: language.langDesc,
						onchange: _.bind(onMasterLangChange, this, language.langCode),
						selected: selected,
						userCannotUncheck: true
					});
				}
			}

			var onTranslationLangChange = _.bind(function (selectedLangCode, toggle) {
				for (var i = 0; i < this._allAvailableTranslationLangs.length; i++) {
					this._languageSelectDropdown.toggleOption(this._allAvailableTranslationLangs[i].langCode, selectedLangCode === this._allAvailableTranslationLangs[i].langCode);
					if (selectedLangCode === this._allAvailableTranslationLangs[i].langCode) {
						this._selectedTranslationLanguage = selectedLangCode;
					}
				}
				this._dynContentModel.setCurrentTransLangCode(this._selectedTranslationLanguage);
				this._updateStatusFilterCount();
				this._createDynContentEntries();
			}, this);

			if (this._allAvailableTranslationLangs && this._allAvailableTranslationLangs.length > 0) {
				this._languageSelectDropdown.addHeader(this.f.i18n.t("PCMTranslationLangMenuHeader"));

				for (var i = 0; i < this._allAvailableTranslationLangs.length; i++) {
					var language = this._allAvailableTranslationLangs[i];

					if (language) {
						var selected = false;
						if (i === 0) {
							this._selectedTranslationLanguage = language.langCode;
							this._dynContentModel.setCurrentTransLangCode(this._selectedTranslationLanguage);
							selected = true;
						}

						this._languageSelectDropdown.addOption({
							name: language.langCode,
							content: language.langDesc,
							onchange: _.bind(onTranslationLangChange, this, language.langCode),
							selected: selected,
							userCannotUncheck: true
						});

					}
				}
			}

			if ((!this._allAvailableMasterLangs || this._allAvailableMasterLangs.length === 0) && (!this._allAvailableTranslationLangs || this._allAvailableTranslationLangs.length === 0)) {
				this._languageSelectDropdown.addHeader(this.f.i18n.t("PCMLangNotAddedTT"));
			}
		};

		DynContentWidget.prototype._createStatusFilterButton = function () {
			this._statusFilterDropdown = new DropdownMiniButton(this.f, "pcmStatusFilter", "icon-WCR_Filter");
			this._statusFilterDropdown.setTooltip("viewer", "PCMStatusFilterTT");
			this._statusFilterDropdown.addButtonClass("textStatusFilter");
			this._statusFilterDropdown.element.click(_.bind(function () {
				this._updateStatusFilterCount();
			}, this));

			this._statusFilterDropdown.addAction({
				name: "pcmDraftCountText",
				content: this.f.i18n.t("PCMDraftBtnTT") + " (0)",
				icon: "icon-WCR_Text-Status-Draft",
				enabled: true,
				action: _.bind(function (event) {
					if (!$(event.target).hasClass("selectedFilterState")) {
						$(this._statusFilterDropdown.dropdown.menu).find(".selectedFilterState").removeClass("selectedFilterState");
						$(event.target).addClass("selectedFilterState");
						this._selectedState = ContentConstants.State.DRAFT;
						this._statusFilterDropdown.setWarning();
						this._statusFilterDropdown.setTooltip("viewer", "PCMStatusFilterActiveTT");
					} else {
						$(event.target).removeClass("selectedFilterState");
						this._selectedState = ContentConstants.State.NONE;
						this._statusFilterDropdown.removeWarning();
						this._statusFilterDropdown.setTooltip("viewer", "PCMStatusFilterTT");
					}
					this._dynContentModel.setDynContentFilterState(this._selectedState);
					this._createDynContentEntries();
				}, this)
			});

			this._statusFilterDropdown.addAction({
				name: "pcmRFApprovalCountText",
				content: this.f.i18n.t("PCMPendingApprovalBtnTT") + " (0)",
				icon: "icon-WCR_Text-Status-Pending",
				enabled: true,
				action: _.bind(function (event) {
					if (!$(event.target).hasClass("selectedFilterState")) {
						$(this._statusFilterDropdown.dropdown.menu).find(".selectedFilterState").removeClass("selectedFilterState");
						$(event.target).addClass("selectedFilterState");
						this._selectedState = ContentConstants.State.READY_FOR_APPROVAL;
						this._statusFilterDropdown.setWarning();
						this._statusFilterDropdown.setTooltip("viewer", "PCMStatusFilterActiveTT");
					} else {
						$(event.target).removeClass("selectedFilterState");
						this._selectedState = ContentConstants.State.NONE;
						this._statusFilterDropdown.removeWarning();
						this._statusFilterDropdown.setTooltip("viewer", "PCMStatusFilterTT");
					}
					this._dynContentModel.setDynContentFilterState(this._selectedState);
					this._createDynContentEntries();
				}, this)
			});

			this._statusFilterDropdown.addAction({
				name: "pcmApprovedCountText",
				content: this.f.i18n.t("PCMTextSignedOff") + " (0)",
				icon: "icon-WCR_Text-Status-Approved icon-WCR_Approve",
				enabled: true,
				action: _.bind(function (event) {
					if (!$(event.target).hasClass("selectedFilterState")) {
						$(this._statusFilterDropdown.dropdown.menu).find(".selectedFilterState").removeClass("selectedFilterState");
						$(event.target).addClass("selectedFilterState");
						this._selectedState = ContentConstants.State.APPROVED;
						this._statusFilterDropdown.setWarning();
						this._statusFilterDropdown.setTooltip("viewer", "PCMStatusFilterActiveTT");
					} else {
						$(event.target).removeClass("selectedFilterState");
						this._selectedState = ContentConstants.State.NONE;
						this._statusFilterDropdown.removeWarning();
						this._statusFilterDropdown.setTooltip("viewer", "PCMStatusFilterTT");
					}
					this._dynContentModel.setDynContentFilterState(this._selectedState);
					this._createDynContentEntries();
				}, this)
			});

			this._statusFilterDropdown.addAction({
				name: "pcmRejectedCountText",
				content: this.f.i18n.t("PCMTextSignoffRejected") + " (0)",
				icon: "icon-WCR_Text-Status-Rejected icon-WCR_Reject",
				enabled: true,
				action: _.bind(function (event) {
					if (!$(event.target).hasClass("selectedFilterState")) {
						$(this._statusFilterDropdown.dropdown.menu).find(".selectedFilterState").removeClass("selectedFilterState");
						$(event.target).addClass("selectedFilterState");
						this._selectedState = ContentConstants.State.REJECTED;
						this._statusFilterDropdown.setWarning();
						this._statusFilterDropdown.setTooltip("viewer", "PCMStatusFilterActiveTT");
					} else {
						$(event.target).removeClass("selectedFilterState");
						this._selectedState = ContentConstants.State.NONE;
						this._statusFilterDropdown.removeWarning();
						this._statusFilterDropdown.setTooltip("viewer", "PCMStatusFilterTT");
					}
					this._dynContentModel.setDynContentFilterState(this._selectedState);
					this._createDynContentEntries();
				}, this)
			});
		};

		/**
		 * Update the count in the filter.
		 */
		DynContentWidget.prototype._updateStatusFilterCount = function () {
			var statusCount = {
				draft: 0,
				pending: 0,
				approved: 0,
				rejected: 0
			};
			for (var i = 0; i < this._contentStatementList.length; i++) {
				for (var j = 0; j < this._contentStatementList[i].element.length; j++) {
					var state = this._contentStatementList[i].element[j].state;
					var approvals = this._contentStatementList[i].element[j].approval;
					if (state == ContentConstants.State.DRAFT && (this._selectedMasterLanguage == this._contentStatementList[i].element[j].language
							|| this._selectedTranslationLanguage == this._contentStatementList[i].element[j].language)) {
						statusCount.draft++;
						break;
					}
					// Check if somebody recently reviewed it to this state.
					else if (approvals.length > 0 && approvals[approvals.length - 1].state == ContentConstants.State.DRAFT) {
						statusCount.draft++;
						break;
					}
				}
			}
			for (var i = 0; i < this._contentStatementList.length; i++) {
				for (var j = 0; j < this._contentStatementList[i].element.length; j++) {
					var state = this._contentStatementList[i].element[j].state;
					var approvals = this._contentStatementList[i].element[j].approval;
					if (state == ContentConstants.State.READY_FOR_APPROVAL && (this._selectedMasterLanguage == this._contentStatementList[i].element[j].language
							|| this._selectedTranslationLanguage == this._contentStatementList[i].element[j].language)) {
						statusCount.pending++;
						break;
					}
					// Check if somebody recently reviewed it to this state.
					else if (approvals.length > 0 && approvals[approvals.length - 1].state == ContentConstants.State.READY_FOR_APPROVAL) {
						statusCount.pending++;
						break;
					}
				}
			}
			for (var i = 0; i < this._contentStatementList.length; i++) {
				for (var j = 0; j < this._contentStatementList[i].element.length; j++) {
					var state = this._contentStatementList[i].element[j].state;
					var approvals = this._contentStatementList[i].element[j].approval;
					if (state == ContentConstants.State.APPROVED && (this._selectedMasterLanguage == this._contentStatementList[i].element[j].language
							|| this._selectedTranslationLanguage == this._contentStatementList[i].element[j].language)) {
						statusCount.approved++;
						break;
					}
					// Check if somebody recently reviewed it to this state.
					else if (approvals.length > 0 && approvals[approvals.length - 1].state == ContentConstants.State.APPROVED) {
						statusCount.approved++;
						break;
					}
				}
			}
			for (var i = 0; i < this._contentStatementList.length; i++) {
				for (var j = 0; j < this._contentStatementList[i].element.length; j++) {
					var state = this._contentStatementList[i].element[j].state;
					var approvals = this._contentStatementList[i].element[j].approval;
					if (state == ContentConstants.State.REJECTED && (this._selectedMasterLanguage == this._contentStatementList[i].element[j].language
							|| this._selectedTranslationLanguage == this._contentStatementList[i].element[j].language)) {
						statusCount.rejected++;
						break;
					}
					// Check if somebody recently reviewed it to this state.
					else if (approvals.length > 0 && approvals[approvals.length - 1].state == ContentConstants.State.REJECTED) {
						statusCount.rejected++;
						break;
					}
				}
			}

			var draftLabel = this.f.i18n.t("PCMDraftBtnTT") + " (" + statusCount.draft + ")";
			var pendingLabel = this.f.i18n.t("PCMPendingApprovalBtnTT") + " (" + statusCount.pending + ")";
			var approvedLabel = this.f.i18n.t("PCMTextSignedOff") + " (" + statusCount.approved + ")";
			var rejectedLabel = this.f.i18n.t("PCMTextSignoffRejected") + " (" + statusCount.rejected + ")";

			var updateDOMElement = function (htmlElement, statusLabel) {
				if (htmlElement) {
					htmlElement.textContent = statusLabel;
				}
			};

			updateDOMElement($(".pcmDraftCountText a[name='btn_pcmDraftCountText']").first().contents().last()[0], draftLabel);
			updateDOMElement($(".pcmRFApprovalCountText a[name='btn_pcmRFApprovalCountText']").first().contents().last()[0], pendingLabel);
			updateDOMElement($(".pcmApprovedCountText a[name='btn_pcmApprovedCountText']").first().contents().last()[0], approvedLabel);
			updateDOMElement($(".pcmRejectedCountText a[name='btn_pcmRejectedCountText']").first().contents().last()[0], rejectedLabel);
		};

		DynContentWidget.prototype._createFullWidthViewButton = function () {
			var fullWidthViewButtonHandler = _.bind(function () {
				var callback = _.bind(function () {
					//AttributeCategoryModule is required to open the full view
					this.f.requestModule("attributeCategory", _.bind(function (acatModule) {
						//load the CSS required for attribute category layout
						//	this._loadCSS();
						//Opening TEXTVIEW view of attributes in a full width popup
						acatModule.showViewInModalPopup("TEXTVIEW", this.f.i18n.t("pcmFullViewTitle"), this.f.i18n.t("PCMFullViewSaveBtnTT"));
					}, this), false);
				}, this);

				//Before opening the Full view(Attributes View), prompt the user with unsaved text content changes popup
				this._textContentModule.promptToSaveUnsavedChanges(callback);
			}, this);


			this._fullWidthViewButton = new MiniButton(this.f, "pcmFullView", "icon-WCR_FullScreen", true, fullWidthViewButtonHandler);
			this._fullWidthViewButton.setTooltip("viewer", "PCMOpenFullViewTT");
		};


		DynContentWidget.prototype.updateSaveButton = function () {
			if (this._saveButton) {
				var unsavedCount = this._dynContentModel.getUnsavedTextContentsCount();
				if (unsavedCount > 0) {
					this._saveButton.setEnabled(true);
					this._saveButton.setBadge(unsavedCount);
					this._saveButton.setTooltip("viewer", "PCMSaveMiniBtnTT", [unsavedCount]);
				} else {
					this._saveButton.setEnabled(false);
				}
			}
		};

		/**
		 * 
		 */
		DynContentWidget.prototype.highlight = function (selectedDynContents, isDirectSelect) {

			// If this was a direct single select, we need to make sure that the language and state which was selected
			// is not hidden by a filter ... if so ... reset the filter.
			if (isDirectSelect && selectedDynContents.length == 1) {
				this.ensureVisibility(selectedDynContents[0]);
			}

			// Find the content statements that we need to highlight.
			var contentStatementsToHighlight = [];
			for (var i = 0; i < selectedDynContents.length; i++) {
				var contentStatement = selectedDynContents[i].getTextElement().parentStatement;
				if ($.inArray(contentStatement, contentStatementsToHighlight) < 0) {
					contentStatementsToHighlight.push(contentStatement);
				}
			}

			// Unhighlight/highlight the right dynContentEntries
			for (var i = 0; i < this._dynContentEntries.length; i++) {
				var contentStatement = this._dynContentEntries[i].getContentStatement();
				if ($.inArray(contentStatement, contentStatementsToHighlight) < 0) {
					this._dynContentEntries[i].unHighlight();
				} else {
					this._dynContentEntries[i].highlight();
				}
			}
		};

		/**
		 * Make sure that the passed dynContent object is visible in the DynContent Widget.
		 * 
		 * @param dynContent
		 */
		DynContentWidget.prototype.ensureVisibility = function (dynContent) {
			var element = dynContent.getTextElement();
			var filterChanged = false;
			// Check that the correct master language is displayed.
			if (element.isMaster() && element.language != this._selectedMasterLanguage) {
				this._languageSelectDropdown.toggleOption(this._selectedMasterLanguage, false);
				this._selectedMasterLanguage = element.language;
				this._languageSelectDropdown.toggleOption(this._selectedMasterLanguage, true);
				filterChanged = true;
			}
			// Check that the correct translation language is displayed.
			if (!element.isMaster() && element.language != this._selectedTranslationLanguage) {
				this._languageSelectDropdown.toggleOption(this._selectedTranslationLanguage, false);
				this._selectedTranslationLanguage = element.language;
				this._languageSelectDropdown.toggleOption(this._selectedTranslationLanguage, true);
				this._dynContentModel.setCurrentTransLangCode(this._selectedTranslationLanguage);
				filterChanged = true;
			}
			// Check that the correct state is displayed, e.g. check that the parent contentstatement has an element witht the selected state.
			if (this._selectedState != ContentConstants.State.NONE && !this._containsState(element.parentStatement, this._selectedState)) {
				this._selectedState = ContentConstants.State.NONE;
				$(this._statusFilterDropdown.dropdown.menu).find(".selectedFilterState").removeClass("selectedFilterState");
				this._statusFilterDropdown.removeWarning();
				this._statusFilterDropdown.setTooltip("viewer", "PCMStatusFilterTT");
				filterChanged = true;
			}

			// If the filter changed.
			if (filterChanged) {
				this._dynContentModel.setDynContentFilterState(this._selectedState);
				this._createDynContentEntries();
			}
		};

		/**
		 * Check if the contentStatement contains an element with the given state.
		 */
		DynContentWidget.prototype._containsState = function (contentStatement, state) {
			var foundElementWithState = false;
			for (var j = 0; j < contentStatement.element.length; j++) {
				var textElement = contentStatement.element[j];
				if (textElement.state == state && (textElement.language == this._selectedMasterLanguage || 
						textElement.language == this._selectedTranslationLanguage)) {
					foundElementWithState = true;
					break;
				} else if (textElement.approval.length > 0) {
					var lastApprovedState = textElement.approval[textElement.approval.length - 1].state;
					if (lastApprovedState == state  && (textElement.language == this._selectedMasterLanguage || 
							textElement.language == this._selectedTranslationLanguage)) {
						foundElementWithState = true;
						break;
					}
				}
			}
			return foundElementWithState;
		};

		/**
		 * Create the dynamic content entries.
		 */
		DynContentWidget.prototype._createDynContentEntries = function () {

			// Empty the dynamic content entries
			this._wrapper.empty();
			this._dynContentEntries = [];

			// Create a new wrapper for the dynamic content entries.
			this._entryWrapper = $('<div>').addClass("DynContentEntries");
			this._wrapper.append(this._entryWrapper);

			if (this._dynContentModel.getDynContentsLoading()) {
				this._entryWrapper.append($('<div>', {
					"class": "NoTextContentsAvailable"
				}).text(i18nViewerStrings.t("PCMDynContentLoading")));
				return;
			}

			// Make sure languages were selected.
			if (this._allAvailableMasterLangs.length === 0 && this._allAvailableTranslationLangs.length === 0) {
				this._entryWrapper.append($('<div>', {
					"class": "NoTextContentsAvailable"
				}).text(i18nViewerStrings.t("PCMNoLangSelected")));
				return;
			}

			if ((this._allAvailableMasterLangs && this._allAvailableMasterLangs.length > 0) || 
					(this._allAvailableTranslationLangs && this._allAvailableTranslationLangs.length > 0)) {

				var buildSelectedCallback = function (dynContentEntry) {
					return function (langCode) {
						// When the dynamic content entry is selected do ...
						var contentStatement = dynContentEntry.getContentStatement();
						// Ask the module to select on dyn content annotations based on this statement.
						this._textContentModule.selectDynContentBasedOn(contentStatement, langCode);
					}.bind(this);
				}.bind(this);

				for (var i = 0; i < this._contentStatementList.length; i++) {
					var contentStatement = this._contentStatementList[i];

					// Check if there's a filter value on state.
					if (this._selectedState != ContentConstants.State.NONE && !this._containsState(contentStatement, this._selectedState)) {
						// If we did not find element with the selected state, skip this contentstatement.
						continue;
					}

					var dynContentEntry = new DynContentEntry(contentStatement, this._selectedMasterLanguage, 
							this._selectedTranslationLanguage, this._languagesList, this._elementTypeList);
					dynContentEntry.onSelected(buildSelectedCallback(dynContentEntry));
					dynContentEntry.onElementSelected(buildSelectedCallback(dynContentEntry));
					this._entryWrapper.append(dynContentEntry.getJQElement());
					this._dynContentEntries.push(dynContentEntry);
				}
			}

			// no cards are available display no text contents message
			if (this._entryWrapper.children().length === 0) {
				this._entryWrapper.append($('<div>', {
					"class": "NoTextContentsAvailable"
				}).text(i18nViewerStrings.t("PCMNoDynContentFound")));
			}
		};

		/**
		 * Sort the content statements on color and element type name.
		 */
		DynContentWidget.prototype._sortContentStatementList = function () {

			// Define a function which compare two contentstatements ...
			var compare = function (c1, c2) {

				if (!this._elementTypeList[c1.elementType]) {
					return -1;
				}

				var color1 = this._elementTypeList[c1.elementType].color;
				var name1 = this._elementTypeList[c1.elementType].elementHumanReadable;
				var color2 = this._elementTypeList[c2.elementType] !== undefined ? this._elementTypeList[c2.elementType].color : "";
				var name2 = this._elementTypeList[c2.elementType] !== undefined ? this._elementTypeList[c2.elementType].elementHumanReadable : c2.elementType;

				// First sort on color ...
				if (color1 != color2) {
					// If the colors differ ... return wether we have the biggest color.
					if (color1 < color2) {
						return -1;
					} else {
						return 1;
					}
				} else {
					// If it's the same color ... sort on name ...
					if (name1 < name2) {
						return -1;
					} else if (name1 > name2) {
						return 1;
					} else {
						return 0;
					}
				}
			}.bind(this);
			this._contentStatementList.sort(compare);
		};

		/**
		 * Find all the available languages in the current content statement list.
		 */
		DynContentWidget.prototype._findAllAvailableLanguages = function () {
			// Iterate over all the contentStatements.
			for (var i = 0; i < this._contentStatementList.length; i++) {
				var contentStatement = this._contentStatementList[i];
				// Iterate over all the text elements.
				for (var j = 0; j < contentStatement.element.length; j++) {
					var element = contentStatement.element[j];
					// Check whether the element is a master or translation language.
					if (element.isMaster()) {
						// Get the master language of the element.
						var masterLang = languageUtils.getLangForCode(this._projectMasterLangs, element.language);
						// Check whether this master language of the element is already added to the "_allAvailableMasterLangs" array.
						if (masterLang && $.inArray(masterLang, this._allAvailableMasterLangs) < 0) {
							this._allAvailableMasterLangs.push(masterLang);
						}
					} else {
						// Get the master language of the element.
						var transLang = languageUtils.getLangForCode(this._projectTranslationLangs, element.language);
						// Check whether this translation language of the element is already added to the "_allAvailbeTransLangs" array.
						if (transLang && $.inArray(transLang, this._allAvailableTranslationLangs) < 0) {
							this._allAvailableTranslationLangs.push(transLang);
						}
					}
				}
			}
		};

		/**
		 * Set the context of the widget
		 * @override
		 * @param context
		 * @param headerExtra
		 */
		DynContentWidget.prototype.setContext = function (context, headerExtra) {
			AbstractWidget.prototype.setContext.call(this, context, headerExtra);
			this.context.addClass("enableTextSelection");
			this.headerExtra.append(this._saveButton.element).append(this._languageSelectDropdown.element).append(this._statusFilterDropdown.element).append(this._fullWidthViewButton.element);
			this._wrapper.appendTo(this.context);
		};

		/**
		 * The text content  widget is greedy, and wants to have all available extra space
		 * @override
		 * @returns {Boolean} true
		 */
		DynContentWidget.prototype.willFill = function () {
			return true;
		};

		/**
		 * Get the preferred size
		 * @override
		 */
		DynContentWidget.prototype.getPreferredSize = function () {
			return new ShapeUtils.Size(this._entryWrapper.outerWidth(true), this._entryWrapper.outerHeight(true));
		};

		/**
		 * Get the preferred size
		 * @override
		 */
		DynContentWidget.prototype.getMinimumSize = function () {
			return new ShapeUtils.Size(200, 100);
		};


		return DynContentWidget;
	});

define('web-viewer/textContent/DynContentAnnotation',["web-viewer/utils/LineworkWrapper",
		"web-viewer/utils/LineworkPoint",
		"utils/ShapeUtils",
		"web-viewer/utils/ViewerUtils",
		"web-viewer/utils/DrawUtils",
		"packaging-content-management/ContentConstants",
		"underscore"
	],
	function (LineworkWrapper,
		LineworkPoint,
		ShapeUtils,
		ViewerUtils,
		DrawUtils,
		ContentConstants,
		_) {
		"use strict";


		/**
		 * @class Represents an dynamic content object. It includes the linework and the data of the dynamic content.
		 * @exports DynContent
		 * @memberof TextContentModule
		 * @augments Objects.LineworkWrapper
		 * @constructor
		 * @param {Core.ViewerFacade} f
		 */
		var DynContentAnnotation = function (f, dynContent, dynContentLocation, pageSize) {
			LineworkWrapper.call(this);
			this.f = f;
			this._dynContent = dynContent;
			this._dynContentLocation = dynContentLocation;
			this._pageSize = pageSize;

			// Keep track wether this annotation has been selected or not.
			this.selected = false;

			// Boolean indicating whether this is a full page annotation. 
			// (check whether a location is provided)
			this._fillPage = dynContentLocation === undefined;

			this.calcRect();
			this.calcStyle();

		};

		// inherit
		DynContentAnnotation.prototype = Object.create(LineworkWrapper.prototype);
		// correct the constructor pointer 
		DynContentAnnotation.prototype.constructor = DynContentAnnotation;

		/**
		 * Calculate the position of the rectangle.
		 */
		DynContentAnnotation.prototype.calcRect = function () {

			if (this._dynContent.getDynContentType() == "Text") {
				// All text dyn contents annotations are rectangle.
				this.type = DrawUtils.LWTYPE_RECT;
				// Check whether this annotation fills the whole page.
				if (!this._fillPage) {
					// Create the rectangle at the correct position.
					var point = new ShapeUtils.Point(this._dynContentLocation.orgx, this._dynContentLocation.orgy - this._dynContentLocation.sizey);
					var size = new ShapeUtils.Size(this._dynContentLocation.sizex, this._dynContentLocation.sizey);
					this.rect = new ShapeUtils.Rect(point, size);
					// Find out wether a rotation was provided ... (not 360 or 0 degrees).
					var epsilon = 2.2204460492503130808472633361816E-16;
					if (Math.abs(this._dynContentLocation.rotation - 360) > 2 * epsilon && Math.abs(this._dynContentLocation.rotation) > 2 * epsilon) {
						// A rotation was provided, but we don't support rotated rectangles ... let's draw the bounding rectangle instead.
						// Calculate the points of the rectangle ...
						var point1 = point;
						var point2 = new ShapeUtils.Point(point.x + size.width, point.y);
						var point3 = new ShapeUtils.Point(point.x, point.y + size.height);
						var point4 = new ShapeUtils.Point(point.x + size.width, point.y + size.height);
						var center = new ShapeUtils.Point(point.x + size.width / 2, point.y + size.height / 2);

						// Rotate the points of the rectangle ...
						point1 = this._rotatePoint(point1, center, this._dynContentLocation.rotation);
						point2 = this._rotatePoint(point2, center, this._dynContentLocation.rotation);
						point3 = this._rotatePoint(point3, center, this._dynContentLocation.rotation);
						point4 = this._rotatePoint(point4, center, this._dynContentLocation.rotation);
						var rotPoints = [point1, point2, point3, point4];

						// Calculate the bounding rectangle of the rotated points ...
						var smallestX = Number.MAX_VALUE;
						var smallestY = Number.MAX_VALUE;
						var biggestX = Number.MIN_VALUE;
						var biggestY = Number.MIN_VALUE;
						for (var i = 0; i < rotPoints.length; i++) {
							if (rotPoints[i].x < smallestX) {
								smallestX = rotPoints[i].x;
							}
							if (rotPoints[i].x > biggestX) {
								biggestX = rotPoints[i].x;
							}
							if (rotPoints[i].y < smallestY) {
								smallestY = rotPoints[i].y;
							}
							if (rotPoints[i].y > biggestY) {
								biggestY = rotPoints[i].y;
							}
						}

						// Based on this, create the bounding rectangle ...
						var newPoint = new ShapeUtils.Point(smallestX, smallestY);
						var newSize = new ShapeUtils.Size(biggestX - smallestX, biggestY - smallestY);
						this.rect = new ShapeUtils.Rect(newPoint, newSize);
					}
				} else {
					// No location was provided for the DynContentAnnotation
					var origin = new ShapeUtils.Point(0, 0);
					this.rect = new ShapeUtils.Rect(origin, this._pageSize);
				}

				// Calc the location of the popup of this annotation.
				var popupOriginX = this.rect.origin.x;
				var popupOriginY = this.rect.origin.y - 38;
				var popupOrigin = new ShapeUtils.Point(popupOriginX, popupOriginY);
				this.popuprect = new ShapeUtils.Rect(popupOrigin, new ShapeUtils.Size(120, 38));
			}
		};

		/**
		 * Calculate the style of the annotation.
		 */
		DynContentAnnotation.prototype.calcStyle = function () {

			if (this._dynContent.getDynContentType() == "Text") {

				// Color according to the current state ...
				var state = this._dynContent.getTextElement().state;
				// However ... if somebody just reviewed ... color the soon to be state.
				var approvals = this._dynContent.getTextElement().approval;
				if (this._dynContent.getTextElement().approvalChanged && approvals && approvals.length > 0) {
					var lastApproval = approvals[approvals.length - 1];
					if (lastApproval.state) {
						state = lastApproval.state;
					}
				}

				// Determine the color based on the approval state of the element of the dynContent.
				if (state == ContentConstants.State.APPROVED) {
					this.color = "#4BE24B";
					if (!this._fillPage) {
						if (this.selected) {
							this.fillcolor = "rgba(75, 226, 75, 0.3)";
						} else {
							this.fillcolor = "rgba(75, 226, 75, 0.15)";
						}
					}
				} else if (state == ContentConstants.State.REJECTED) {
					this.color = "#DC2B24";
					if (!this._fillPage) {
						if (this.selected) {
							this.fillcolor = "rgba(220, 43, 36, 0.3)";
						} else {
							this.fillcolor = "rgba(220, 43, 36, 0.15)";
						}
					}
				} else if (state == ContentConstants.State.READY_FOR_APPROVAL) {
					this.color = "#52DAF7";
					if (!this._fillPage) {
						if (this.selected) {
							this.fillcolor = "rgba(82, 218, 247, 0.3)";
						} else {
							this.fillcolor = "rgba(82, 218, 247, 0.15)";
						}
					}
				} else {
					this.color = "#C7C7C7";
					if (!this._fillPage) {
						if (this.selected) {
							this.fillcolor = "rgba(199, 199, 199, 0.3)";
						} else {
							this.fillcolor = "rgba(199, 199, 199, 0.15)";
						}
					}
				}

				if (this.selected) {
					this.linewidth = 4;
				} else {
					this.linewidth = 2;
				}
			}
		};


		/**
		 * Recalculate the style and return wether there was a change.
		 */
		DynContentAnnotation.prototype.reCalcStyle = function () {

			// Remember the old style.
			var oldFillcolor = this.fillcolor;
			var oldColor = this.color;
			var oldLineWidth = this.linewidth;

			// Calculate the style again.
			this.calcStyle();

			// Return whether the style changed.
			if (oldFillcolor !== this.fillcolor || oldColor !== this.color || oldLineWidth !== this.linewidth) {
				return true;
			} else {
				return false;
			}
		};


		/**
		 * Return the dynContent.
		 */
		DynContentAnnotation.prototype.getDynContent = function () {
			return this._dynContent;
		};

		/**
		 * Indicate that the dynContent is selected.
		 */
		DynContentAnnotation.prototype.setSelected = function (isSelected) {
			this.selected = isSelected;
			this.calcStyle();
		};

		/**
		 * Utility function to calculate the rotation of the point
		 */
		DynContentAnnotation.prototype._rotatePoint = function (point, origin, angle) {
			angle = angle * Math.PI / 180.0;
			var x = Math.cos(angle) * (point.x - origin.x) - Math.sin(angle) * (point.y - origin.y) + origin.x;
			var y = Math.sin(angle) * (point.x - origin.x) + Math.cos(angle) * (point.y - origin.y) + origin.y;
			return new ShapeUtils.Point(x, y);
		};

		/**
		 * Set the scaling of the dynamic content annotation, all points of the annotation (linework) are scaled witht his value
		 * @param {ShapeUtils.Point|null} scaling
		 */
		DynContentAnnotation.prototype.setScale = function (scaling) {
			this.scale(scaling);
			this._scaling = scaling;
		};

		return DynContentAnnotation;
	});

/**
 * DynContent object
 * 
 * @author KADA
 */
define('packaging-content-management/dyn-content/DynContent',["jquery"],	function ($) {
	
		"use strict";
		
		var DynContent = function (type, id) {
			this._dynContentType = type;
			this._id = id;
		};

		DynContent.prototype.getDynContentType = function () {
			return this._dynContentType;
		};
		
		DynContent.prototype.getId = function () {
			return this._id;
		};
		
		return DynContent;
});

/**
 * ContentStatement object
 * @author HUBU
 */
define('packaging-content-management/dyn-content/TextDynContent',["./DynContent",
        "../ContentConstants",
		"../ContentStatement",
		"../ContentStatementElement",
		"jquery"
	],
	function (DynContent,
			ContentConstants,
			ContentStatement,
			ContentStatementElement,
			$) {
	
		"use strict";
		
		
		var TextDynContent = function (contentStatementElement, locations) {
			
			// Generate an id for this TextDynContent.
			var elementLang = contentStatementElement.language;
			var elementType = contentStatementElement.parentStatement.elementType;
			var instanceSeq = contentStatementElement.parentStatement.instanceId;
			if (!instanceSeq) {
				instanceSeq = "";
			}
			var random = Math.floor(Math.random()*11);
			var id = elementType+"_I"+instanceSeq+"_"+elementLang+"_"+random;
			// Call the super
			DynContent.call(this, "Text", id);
			
			this._textElement = contentStatementElement;
			if (locations) {
				this._locations = locations;
			} else {
				this._locations = [];
			}
		};
		
		// inherit
		TextDynContent.prototype = Object.create(DynContent.prototype);
		// correct the constructor pointer 
		TextDynContent.prototype.constructor = TextDynContent;
		
		/**
		 * 
		 */
		TextDynContent.prototype.setTextElement = function (textElement) {
			this._textElement = textElement;
		};
		
		/**
		 * 
		 */
		TextDynContent.prototype.getTextElement = function () {
			return this._textElement;
		};
		
		/**
		 * 
		 */
		TextDynContent.prototype.getLocations = function () {
			return this._locations;
		};

		return TextDynContent;
});

define('packaging-content-management/dyn-content-services/dynContentService',["jquery",
        "packaging-content-management/ContentStatement",
        "packaging-content-management/dyn-content/TextDynContent"], 
        function ($, 
        		ContentStatement,
        		TextDynContent) {
	
	
	
	/**
	 * 
	 */
	var DynContentService = function () {

		this._isFetchingDynContentFor = {};
		
		this._dynContentsByDocVerID = [];
	};
	
	/**
	 * 
	 * @param docVersionId
	 */
	DynContentService.prototype.getDynContentFor = function(docVersionId, forceUpdate) {

		// force the update of the cache
		if (forceUpdate) this._dynContentsByDocVerID[docVersionId] = null;

		var cacheItem = this._dynContentsByDocVerID[docVersionId];

		if (!cacheItem) {
			// create a new cache item for the data to be loaded
			cacheItem = {};
			this._dynContentsByDocVerID[docVersionId] = cacheItem;
		}

		if (!cacheItem.request) {
			// create a new Deferred object representing the result of this method (not only the AJAX call!)
			cacheItem.request = $.Deferred();
		} else {
			// return the Promise object in the cache (representing the request), which already contains 
			// the result or is waiting for the executed AJAX call to finish
			return cacheItem.request.promise();
		}

		// The parameters of the call
		var params = {
				"ajax": "1",
				"docverid": docVersionId
		};
		// The baseUrl of the service.
		var jspBaseUrl = window.webcenterConfig.jspBaseUrl;
		var url = jspBaseUrl+"GetDynContent.jsp?json=1&" + $.param(params, true);

			// Get the ajax JSON.
		$.getJSON(url, function(json) {

			// json = {
			//			textDynContents: [
			//				{
			//					contentStatement: <json>,
			//					locations: {
			//						fr-FR: [...],
			//						en-US: [...],
			//						...
			//					}
			//				},
			//				{
			//					...
			//				}, ...
			//			];
			// }

			var dynContents = [];

			// Parse and generate the text dynamic content objects. 
			for (var i=0; i < json.length; i++) {
				var contentStatement = new ContentStatement(json[i]);
				for (var j=0; j < contentStatement.element.length; j++) {
					var textElement = contentStatement.element[j];
					var locations = [];
					if (contentStatement.locationList && contentStatement.locationList[textElement.language]) {
						locations = contentStatement.locationList[textElement.language];
					}
					var textDynContent = new TextDynContent(textElement, locations);
					dynContents.push(textDynContent);
				}
			}

			// return the parsed value for the request.done() method.
			cacheItem.request.resolve(dynContents);

		}.bind(this)).fail(function(jqXHR, textStatus, errorThrown) {
			// TODO
		});

		// value not yet prepared in the cache (the loading is in progress) - return a Promise object
		// to be able to register callbacks in the caller(s)
		return cacheItem.request.promise();
	};
	
	return new DynContentService();
});

define('packaging-content-management/text-content-services/contentStatementService',["jquery",
        "packaging-content-management/ContentStatement",
        "packaging-content-management/ContentStatementElement",
        "services/passwordValidationService",
        "utils/webi18n!TextContentStrings"], 
        function ($, 
        		ContentStatement,
        		ContentStatementElement,
        		passwordValidationService,
        		TextContentStrings) {

	
	
	var ContentStatementService = function () {
		
        // These private variables take care of preventing duplicate requests.
        this._contentStatementIsBeingRetrieved = {};
        this._onContentStatementRetrieved = {};
        
	};


	/**
	 * Request an existing contentent statement from the backend.
	 * 
	 * @param {String} docVerId - The ID of the document version of the the contentstatement that should be returned.
	 * @param {Function} callback - Will be executed on succesfull return from the backend (and be passed the content statement).
	 */
	ContentStatementService.prototype.getContentStatement = function (docVerId, callback, errorCallback) {
		
		var params = {
				"ajax": "1",
				"docverid": docVerId,
		};
		
		// The baseUrl of the service.
        var jspBaseUrl = window.webcenterConfig.jspBaseUrl;
		var url = jspBaseUrl+"GetDocTypeTextContent.jsp?json=1&" + $.param(params, true);
		$.getJSON(url, function(jsonData) {
        	var contentStatement = new ContentStatement(jsonData);
        	callback(contentStatement);
        }.bind(this)).fail(function(jqXHR, textStatus, errorThrown) {
        	if(errorCallback) {
            	var errorCode = null;
            	errorCode = $(jqXHR.responseText).find('error').find('code').text();
            	
            	if(errorCode && ('INVALIDPARAMETER_TAG' === errorCode || '002' === errorCode || '003' === errorCode)) {
            		errorCallback(TextContentStrings.t("TEXTCONTENT_DOCVERSION_NOT_FOUND"));
            		return;
            	}
        		errorCallback(TextContentStrings.t("TEXTCONTENT_LOAD_ERROR"));
        	}
        });
	};
	
	
	/**
	 * Request a new contentent statement in a different project from the backend. (does not yet exist)
	 * 
	 * @param {String} docVerId - The ID of the document version from wich the content statement should be created/copied.
	 * @param {String} targetProjID - The ID of the project in which the new content statement should be created/copied.
	 * @param {Function} callback - Will be executed on succesfull return from the backend (and be passed the content statement).
	 */
	ContentStatementService.prototype.getNewContentStatementFor = function (docVerId, targetProjID, retainStatus, callback, errorCallback) {
		
		var params = {
				"ajax": "1",
				"isNew": "1",
				"docverid": docVerId,
				"retainstatus" : retainStatus
		};
		
		// Set also a projectID in the parameters when specified. 
		if (targetProjID && targetProjID != null) {
			params.targetprojid = targetProjID;
		}

		// The baseUrl of the service.
        var jspBaseUrl = window.webcenterConfig.jspBaseUrl;
		var url = jspBaseUrl+"GetDocTypeTextContent.jsp?json=1&" + $.param(params, true);
		$.getJSON(url, function(jsonData) {
        	var contentStatement = new ContentStatement(jsonData);
        	callback(contentStatement);
        }.bind(this)).fail(function(jqXHR, textStatus, errorThrown) {
        	if(errorCallback) {
            	var errorCode = null;
            	errorCode = $(jqXHR.responseText).find('error').find('code').text();
            	
            	if(errorCode && ('INVALIDPARAMETER_TAG' === errorCode || '002' === errorCode || '003' === errorCode)) {
            		errorCallback(TextContentStrings.t("TEXTCONTENT_DOCVERSION_NOT_FOUND"));
            		return;
            	}
        		errorCallback(TextContentStrings.t("TEXTCONTENT_LOAD_ERROR"));
        	}
        });
	};
	
	
	/**
	 * Save the changes in a Content Statement object.
	 * 
	 * The Content Statement should already exist!
	 * 
	 * @param {ContentStatement} contentStatement - The Content Statement object to be saved.
	 * @param {Function} callback - To be executed on save action (Will be passed the updated content statement or
	 * 								the content statement for which the update failed and a possible error message).
	 */
	ContentStatementService.prototype.saveContentStatement = function (contentStatement, callback) {
		this.saveContentStatements([contentStatement], callback);
	};
	
	/**
	 * Save the changes in multiple Content Statement objects.
	 * 
	 * These Content Statements should already exist!
	 * 
	 * @param {ContentStatement} contentStatements - The Content Statements objects to be saved.
	 * @param {Function} callback - To be executed on save action (Will be passed the updated contentStatements, 
	 * 								the contentStatements for which the update failed and error messages).
	 */
	ContentStatementService.prototype.saveContentStatements = function (contentStatements, callback) {
		
		var password = passwordValidationService.getValidatedPassword();
		
		var saveJsons = [];
		var docVersionIds = [];
		var docVersionIdMap = [];
		for (var i=0; i < contentStatements.length; i++) {
			saveJsons.push(contentStatements[i].getUnsavedContentJSON(password));
			docVersionIds.push(contentStatements[i].docVersionId);
			docVersionIdMap[contentStatements[i].docVersionId] = contentStatements[i];
		}
		
		var url = window.webcenterConfig.jspBaseUrl + 'updateDocTypeTextContent.jsp';
		$.ajax(url, {
			dataType: "json",
			cache: false,
			noAjaxParam: false,
			contentType: "application/x-www-form-urlencoded; charset=UTF-8",
			type: "POST",
			data: {
				updatejsons: saveJsons,
				docverids : docVersionIds
			},
			success: function (json) {
				
				// Get all the error messages.
				var errorMsgs = new Array();
				// Get all the updated content statements;
				var updatedContentStatements = {};
				// Get all the failed content statements;
				var failedContentStatements = {};
				
				// If a general error occured.
				if (json.error) {
					// Add the general error occured.
					errorMsgs.push(json.error);
					// All content statements failed.
					failedContentStatements = contentStatements.slice(0, contentStatements.length);
				}
				// Find out which updates succeeded and get the new updated content statements.
				else if (json["updates"]) {
					// Iterate over all update entries ...
					for (var i=0; i < json.updates.length; i++) {
						var update = json.updates[i];
						// Check if an error occured during the update ...
						if (update["error"]) {
							errorMsgs[update["id"]] = update["error"];
						}
						// Check if we have a newly updated version of the content statement.
						if (update["new"]) {
							updatedContentStatements[update["id"]] = new ContentStatement(update["new"]);
						}
						// If no such newly updated version could be found ...
						else {
							// We now know that this contentStatement could not be updated.
							// Add the original content statement to the list of failed content statements.
							var failedContentStatement = docVersionIdMap[update["id"]];
							failedContentStatements[update["id"]] = failedContentStatement;
						}
					}
				}
				
				// If a callback was specified.
				if (callback) {
					callback(updatedContentStatements, failedContentStatements, errorMsgs);
				}
				
			},
			error: function (jqXHR, textStatus, errorThrown) {
				if(callback) {
					callback([], contentStatements.slice(0, contentStatements.length), [TextContentStrings.t("TEXTCONTENT_SAVE_FAILED")]);
				}
			}
		});
		
	};
	
	/**
	 * Get the Content statement element language when a new language has been added
	 */
	ContentStatementService.prototype.getContentStatementElement = function (docVerId, language, isMaster, callback, errorCallback) {
		
		var langCode = language;
		if (typeof langCode == "object") {
			langCode = language.langCode;
		}
		
		var params = {
				"ajax": "1",
				"docverid": docVerId,
				"language": langCode,
				"contentType": isMaster,
		};

		// The baseUrl of the service.
        var jspBaseUrl = window.webcenterConfig.jspBaseUrl;
		var url = jspBaseUrl+"CreateDocTypeTextContentElement.jsp?" + $.param(params, true);
		$.getJSON(url, function(jsonData) {
        	var element = new ContentStatementElement(jsonData);
        	callback(element);
        }.bind(this)).fail(function(jqXHR, textStatus, errorThrown) {
        	if(errorCallback) {
            	var errorCode = null;
            	errorCode = $(jqXHR.responseText).find('error').find('code').text();
            	
            	if(errorCode && ('INVALIDPARAMETER_TAG' === errorCode || '002' === errorCode || '003' === errorCode)) {
            		errorCallback(TextContentStrings.t("TEXTCONTENT_DOCVERSION_NOT_FOUND"));
            		return;
            	}
        		errorCallback(TextContentStrings.t("TEXTCONTENT_LOAD_ERROR"));
        	}
        });
		
		
	};
	
	
	/**
	 * Get the Content statement element language when a new language has been added in case of new project
	 */
	ContentStatementService.prototype.getNewContentStatementElementFor = function (docVerId, language, isMaster, targetProjId, retainStatus, callback, errorCallback) {
		
		var langCode = language;
		if (typeof langCode == "object") {
			langCode = language.langCode;
		}
		
		var params = {
				"ajax": "1",
				"isNew": "1",
				"docverid": docVerId,
				"language": langCode,
				"contentType": isMaster,
				"retainstatus": retainStatus
		};

		// Set also a projectID in the parameters when specified. 
		if (targetProjId && targetProjId != null) {
			params.targetprojid = targetProjId;
		}
		
		// The baseUrl of the service.
        var jspBaseUrl = window.webcenterConfig.jspBaseUrl;
		var url = jspBaseUrl+"CreateDocTypeTextContentElement.jsp?" + $.param(params, true);
		$.getJSON(url, function(jsonData) {
        	var element = new ContentStatementElement(jsonData);
        	callback(element);
        }.bind(this)).fail(function(jqXHR, textStatus, errorThrown) {
        	if(errorCallback) {
            	var errorCode = null;
            	errorCode = $(jqXHR.responseText).find('error').find('code').text();
            	
            	if(errorCode && ('INVALIDPARAMETER_TAG' === errorCode || '002' === errorCode || '003' === errorCode)) {
            		errorCallback(TextContentStrings.t("TEXTCONTENT_DOCVERSION_NOT_FOUND"));
            		return;
            	}
        		errorCallback(TextContentStrings.t("TEXTCONTENT_LOAD_ERROR"));
        	}
        });
		
		
	};

	
	/**
	 * Request an existing contentent statement from the backend.
	 * 
	 * @param {String} docVerId - The ID of the document version of the the contentstatement that should be returned.
	 * @param {Function} callback - Will be executed on succesfull return from the backend (and be passed the content statement).
	 */
	ContentStatementService.prototype.getContentStatementList = function (docVerId, callback, errorCallback) {
		
		var params = {
				"ajax": "1",
				"docverid": docVerId,
		};
		
		// The baseUrl of the service.
        var jspBaseUrl = window.webcenterConfig.jspBaseUrl;
		var url = jspBaseUrl+"GetDocTypeTextContent.jsp?json=1&" + $.param(params, true);
		$.getJSON(url, function(jsonData) {
			var contentStatementList = [];
			for(var i=0;i<jsonData.length;i++) {
				var contentStatement = new ContentStatement(jsonData[i]);
				contentStatementList.push(contentStatement);
			}
			callback(contentStatementList);
		}.bind(this)).fail(function(jqXHR, textStatus, errorThrown) {
			if(errorCallback) {
            	var errorCode = null;
            	errorCode = $(jqXHR.responseText).find('error').find('code').text();
            	
            	if(errorCode && ('INVALIDPARAMETER_TAG' === errorCode || '002' === errorCode || '003' === errorCode)) {
            		errorCallback(TextContentStrings.t("TEXTCONTENT_DOCVERSION_NOT_FOUND"));
            		return;
            	}
        		errorCallback(TextContentStrings.t("TEXTCONTENT_LOAD_ERROR"));
        	}
        });
	};
	
	return new ContentStatementService();
	
});

define('web-viewer/textContent/DynContentModel',["web-viewer/core/PrivateChannel",
		"utils/ShapeUtils",
		"web-viewer/utils/ListenerUtil",
		"web-viewer/utils/ViewerUtils",
		"web-viewer/layout/component/ModalPopup",
		"web-viewer/layout/component/ViewerToolbarButton",
		"web-viewer/layout/LayoutConstants",
		"web-viewer/textContent/DynContentAnnotation",
		"packaging-content-management/dyn-content-services/dynContentService",
		"packaging-content-management/text-content-services/contentStatementService",
		"packaging-content-management/ContentConstants",
		"component/passwordpopup/PasswordValidationPopup",
		"underscore",
		"jquery"
	],
	function (PrivateChannel,
		ShapeUtils,
		ListenerUtil,
		ViewerUtils,
		ModalPopup,
		ViewerToolbarButton,
		LayoutConstants,
		DynContentAnnotation,
		dynContentService,
		contentStatementService,
		ContentConstants,
		PasswordValidationPopup,
		_,
		$) {
		"use strict";

		/**
		 * Constructs a new Dynamic Content model
		 * @exports DynContentModel
		 * @memberof TextContentModule
		 * @class
		 * The dynamic content model is the main model within the viewer that keeps track of the dynamic content objects and their selection. The model has its main function "setDynamicContents".
		 * @constructor
		 * @param {Core.ViewerFacade} f - Viewer facade
		 * @param {String} IID - Unique channel ID of the dynamic content model
		 */
		var DynContentModel = function (f, IID) {
			PrivateChannel.installTo(this, f, ['dynContentModel', IID]);
			ListenerUtil.mixin(this, f);

			/**
			 * Facade
			 */
			this.f = f;

			/**
			 * The page to which the current annotations belong
			 * @private
			 * @type {Data.AbstractDocumentPage}
			 */
			this._page = null;

			/**
			 * Dynamic content annotations of the main document.
			 */
			this._dynContentAnnotations = [];

			/**
			 * Flag to specify loading dyn content 
			 */
			this._isDynContentsLoading = false;

			/**
			 * Keeps the list of dynamic content annotations that are selected.
			 */
			this._selectedDynContentAnnotations = [];

			/**
			 * An array containing all the annotations that are currently visible.
			 */
			this._visibleDynContentAnnotations = [];

			/**
			 * Keeps the list of Text Content objects that are unsaved.
			 */
			this._unsavedTextContents = [];

			/**
			 *  The currently active scaling, all annotation are scaled with this scaling. This is used to initialize new annotations.
			 */
			this._currentScaling = null;

			/**
			 * Indicates whether we're currently in the process of saving the dynamic content.
			 */
			this._isSavingDynamicContents = false;

			/**
			 * The state on which we're filtering in the viewer.
			 */
			this._dynContentFilterState = ContentConstants.State.NONE;

			/**
			 * By default the model is not disabled.
			 */
			this._disabled = false;
		};

		/**
		 * Toggle the annotation visibility
		 * @param {Boolean} [toggle] - force visibility
		 */
		DynContentModel.prototype.toggleVisibility = function (toggle) {
			if (toggle === undefined) {
				toggle = this.f.getOption("PREFERENCES_DYNCONTENTTOOLS_VISIBLE") == 1;
				toggle = !toggle;
			} else if (toggle && this.f.getOption("PREFERENCES_DYNCONTENTTOOLS_VISIBLE") == 1) { // yes! we compare with == and not with === (we are not sure what is in the preference (true, false)
				// nothing changed
				return;
			}

			if (!toggle) {
				this.f.setOption("PREFERENCES_DYNCONTENTTOOLS_VISIBLE", 0);
				this.unselectAllDynContents(false, true);
			} else {
				this.f.setOption("PREFERENCES_DYNCONTENTTOOLS_VISIBLE", 1);
			}
			this.modelChanged();
		};

		/**
		 * Get the page for which the dynamic content is visible
		 * @returns {Data.AbstractDocumentPage}
		 */
		DynContentModel.prototype.getDynContentPage = function () {
			return this._page;
		};

		/**
		 * Disable the model, e.g. clear all the annotations and prevent them to be set.
		 */
		DynContentModel.prototype.disable = function () {
			this._disabled = true;
			this._dynContentAnnotations = [];
			this._visibleDynContentAnnotations = [];
			this.modelContentChanged();
		};

		/**
		 * Set the annotations objects for this model
		 * @param {Array<DynContentAnnotation>} dynContents - List of annotations
		 */
		DynContentModel.prototype.setDynContents = function (dynContents) {
			if (!this._disabled) {
				this.scaleAllDynContents(null);
				this._dynContentAnnotations = dynContents;
				this._visibleDynContentAnnotations = dynContents.slice(0, dynContents.length);
				//TODO allow the reference page the compensated, the Java Viewer does not allow this, as it returns wrong parameter values
				if (this._page && this._page !== this.f.getDocumentController().getDifferencePage()) {
					this.scaleAllDynContents(this._page.getCurrentScaling());
				}
				this.modelContentChanged();
			}
		};

		/**
		 * @returns {Array.<DynContentAnnotation>} dyn content of the model (unfiltered)
		 */
		DynContentModel.prototype.getDynContents = function () {
			return this._dynContentAnnotations;
		};

		/**
		 * @returns {Boolean} - True if loading dyn content is in progress 
		 */
		DynContentModel.prototype.getDynContentsLoading = function () {
			return this._isDynContentsLoading;
		};

		/**
		 * Reload the list of dynamic contents.
		 */
		DynContentModel.prototype.reload = function (silent) {
			//TODO, not needed atm for dynamic content.
		};

		/**
		 * Save 1 or all dynamic content.
		 * 
		 * @param {ContentStatementElement} [textElement] if specified, only this statements will be saved. If not, all statements will be saved
		 * @param {Function} [callbackFunction] will be called when the annotations have been saved
		 */
		DynContentModel.prototype.save = function (textElement, callbackFunction) {
			if (!this._isSavingDynamicContents) {
				this._emit("save.started");
				this._isSavingDynamicContents = true;
				if (this._page) {
					this.unselectAllDynContents(false, true);

					var unsavedTextContents = [];
					if (textElement) {
						// First make sure if the text content of this passed dynamic content is in the array of unsaved text contents.
						var textContent = textElement.parentStatement;
						if ($.inArray(textContent, this._unsavedTextContents) < 0) {
							// Attempting to save a text content that is not in the list of unsaved.
							// Stop here.
							this._isSavingDynamicContents = false;
							if (callbackFunction) {
								callbackFunction();
							}
							this._emit("save.finished");
							this.modelChanged();
							this.modelContentChanged();
							return;
						} else {
							unsavedTextContents.push(textContent);
						}
					} else {
						unsavedTextContents = this.getUnsavedTextContents();
					}

					// Find out if we need password validation.
					var isPasswordRequired = false;
					for (var i = 0; i < unsavedTextContents.length; i++) {
						isPasswordRequired = unsavedTextContents[i].isPasswordRequiredForApproval();
						if (isPasswordRequired) {
							break;
						}
					}

					// Create a function that will send the actual content statement save request.
					var doSaveContentStatements = function () {

						// Save the content statements
						contentStatementService.saveContentStatements(unsavedTextContents, function (updatedTextContents, failedTextContents, errorMsgs) {

							// Make sure no annotations are selected.
							this.unselectAllDynContents(false, true);

							// Show the errors ...
							var numErrors = Object.keys(failedTextContents).length;
							var errorString = this.f.i18n.t("PCMDynContentSaveFailed", numErrors);
							$.each(errorMsgs, function (id, errorMsg) {
								errorString += "\n - " + errorMsg;
							}.bind(this));
							if (numErrors > 0) {
								this.f.emit("userMessage", "warn", errorString, false);
							} else {
								this.f.emit("userMessage", "success", this.f.i18n.t("PCMDynContentSaveSucces"), true);
							}

							// Find out which text content documents were successfully saved and remove them from the array.
							$.each(updatedTextContents, function (id, updatedTextContent) {
								for (var i = 0; i < this._unsavedTextContents.length; i++) {
									if (this._unsavedTextContents[i].docVersionId == id) {
										this._unsavedTextContents.splice(i, 1);
										i--;
									}
								}
							}.bind(this));

							// Update the annotations of the successfully updated text content docuemnts.
							$.each(updatedTextContents, function (id, updatedTextContent) {
								for (var i = 0; i < this._dynContentAnnotations.length; i++) {
									var parentStatement = this._dynContentAnnotations[i].getDynContent().getTextElement().parentStatement;
									if (parentStatement.docVersionId == id) {
										var oldTextElement = this._dynContentAnnotations[i].getDynContent().getTextElement();
										var updatedTextElement = updatedTextContent.getElement(oldTextElement.language, oldTextElement.isMaster());
										// Update the Text Element.
										this._dynContentAnnotations[i].getDynContent().setTextElement(updatedTextElement);
										this._dynContentAnnotations[i].reCalcStyle();
									}
								}
							}.bind(this));

							// Trigger the calculation of the visble annotations.
							this.setDynContentFilterState(this._dynContentFilterState);

							// Finish the saving action ...
							this._isSavingDynamicContents = false;
							if (callbackFunction) {
								callbackFunction();
							}
							this._emit("save.finished");
							this.modelChanged();
							this.modelContentChanged();

						}.bind(this));

					}.bind(this);


					// Query the password ...
					if (isPasswordRequired) {
						var options = {};
						options.f = this.f;
						options.viewType = "viewer";
						this.passwordValidationPopup = new PasswordValidationPopup(options);

						// When the validation of the password was successfull ...
						this.passwordValidationPopup.onSubmit(function () {
							// Finally save the content statements.
							doSaveContentStatements();
						}.bind(this));

						// When the validation of the password failed ...
						this.passwordValidationPopup.onCancel(function () {
							// Indicate to the user that he supplied the wrong password.
							this.f.emit("userMessage", "warn", this.f.i18n.t("PCMDynContentSaveFailed", unsavedTextContents.length), false);
							// We're no longer saving.
							this._isSavingDynamicContents = false;
							// we didn't need to save, notify that the safe action is finsihed
							this._emit("save.finished");
						}.bind(this));

						// Show the password validation popup
						this.passwordValidationPopup.open();

					} else {
						// No need for password validation first ... just save them.
						doSaveContentStatements();
					}

				} else {
					this._isSaving = false;
					// we didn't need to save, notify that the safe action is finsihed
					this._emit("save.finished");
				}
			} else {
				this.f.emit("error.WARNING", this.f.i18n.t("PCMDynContentAlreadyBeingSaved"), true);
			}
		};

		/**
		 * Get the currently selected translation language.
		 */
		DynContentModel.prototype.setDynContentFilterState = function (filterState) {

			// If there's no new filter state just return.
			if (filterState == this._dynContentFilterState) {
				return;
			}

			// Set the new state.
			this._dynContentFilterState = filterState;

			// Check if the "NO FILTER" state is set.
			if (!filterState || filterState == ContentConstants.State.NONE) {
				this._visibleDynContentAnnotations = this._dynContentAnnotations;
			} else {
				// A new filter state was specified, calculate the newly visible annotations.
				this._visibleDynContentAnnotations = [];
				for (var i = 0; i < this._dynContentAnnotations.length; i++) {
					var approvals = this._dynContentAnnotations[i].getDynContent().getTextElement().approval;
					var state = this._dynContentAnnotations[i].getDynContent().getTextElement().state;
					if (state == filterState) {
						this._visibleDynContentAnnotations.push(this._dynContentAnnotations[i]);
					} else if (approvals.length > 0 && approvals[approvals.length - 1].state == filterState) {
						this._visibleDynContentAnnotations.push(this._dynContentAnnotations[i]);
					}
				}
				// Check if we have to deselect the currently selected annotation as they might be no longer visible.
				for (i = 0; i < this._selectedDynContentAnnotations.length; i++) {
					var selectedAnnotation = this._selectedDynContentAnnotations[i];
					if ($.inArray(selectedAnnotation, this._visibleDynContentAnnotations) < 0) {
						// One of the selected annotations is no longer visible, deselect the annotations.
						this.unselectAllDynContents(false, false);
						break;
					}
				}
			}

			this._emit("visibleDynContentChanged", this._visibleDynContentAnnotations);
			this.modelChanged();
		};

		/**
		 * Get the currently selected translation language.
		 */
		DynContentModel.prototype.getDynContentFilterState = function (state) {
			return this._dynContentFilterState;
		};

		/**
		 * Get the currently selected translation language.
		 */
		DynContentModel.prototype.setCurrentTransLangCode = function (langCode) {
			this._currentTransLangCode = langCode;
			this._emit("translationLangChanged", langCode);
		};

		/**
		 * Get the currently selected translation language.
		 */
		DynContentModel.prototype.getCurrentTransLangCode = function () {
			return this._currentTransLangCode;
		};

		/**
		 *
		 * @returns {Number} the count of all unsaved text content objects.
		 */
		DynContentModel.prototype.getUnsavedTextContentsCount = function () {
			return this.getUnsavedTextContents().length;
		};

		/**
		 * Get an array of unsaved text content objects.
		 * @returns {Array.<Annotation>} Array of unsaved annotations
		 */
		DynContentModel.prototype.getUnsavedTextContents = function () {
			return this._unsavedTextContents;
		};

		/**
		 * @returns {Array.<DynContentAnnotation>} the visible annotations to be displayed on the viewport
		 */
		DynContentModel.prototype.getVisibleDynContents = function () {
			// if annotation tools are disabled, or the module is temporarily disabled, do not show any annotations
			if (this.f.getOption("PREFERENCES_DYNCONTENTTOOLS_VISIBLE") === 0) {
				return [];
			} else {
				return this._visibleDynContentAnnotations;
			}
		};

		/**
		 * Get annotations for the given text element
		 * @param {ContentStatementElement} textElement - The text element
		 * @returns {Array.<<DynContentAnnotation>>} the dynamic content annotations linked to a specific text element
		 */
		DynContentModel.prototype.getDynContentAnnotationsForTextElement = function (textElement) {
			var forTextElement = [];
			for (var i = 0; i < this._dynContentAnnotations.length; i++) {
				if (this._dynContentAnnotations[i].getDynContent().getTextElement() == textElement) {
					forTextElement.push(this._dynContentAnnotations[i]);
				}
			}
			return forTextElement;
		};

		/**
		 * When a text element changed ...
		 * 
		 * @param {ContentStatementElement} textElement - The text element which got changed.
		 * @param {Object} fromWidget - The source of the changed.
		 */
		DynContentModel.prototype.textElementChanged = function (textElement, fromWidget) {
			this._emit("textElementChanged", textElement, fromWidget);

			// Put the parent text object in the list of text content objects that need to be saved.
			var existingTCIndex = $.inArray(textElement.parentStatement, this._unsavedTextContents);
			if ( existingTCIndex < 0) {
				this._unsavedTextContents.push(textElement.parentStatement);
				this._emit("unsavedTextContent", this._unsavedTextContents);
			} else {
				//Remove text content object from  array if it is not changed(status reverted)
				var existingTC = this._unsavedTextContents[existingTCIndex];
				if(!existingTC.getContentChanged()){
					this._unsavedTextContents.splice(existingTCIndex, 1);
					this._emit("unsavedTextContent", this._unsavedTextContents);
				}
			}

			// Check to see if any dynamic content annotations are affected (e.g. they need to change color or ...).
			var affectedDynContentAnnotations = this.getDynContentAnnotationsForTextElement(textElement);
			var changed = false;
			for (var i = 0; i < affectedDynContentAnnotations.length; i++) {
				var annotation = affectedDynContentAnnotations[i];
				// Recalculate the style for the annotation.
				var styleChanged = annotation.reCalcStyle();
				changed = changed || styleChanged;
			}

			// If the style for any annotation changed ...
			if (changed) {
				this.modelChanged();
			}
		};


		/**
		 * 
		 * @param contentStatement
		 */
		DynContentModel.prototype.selectDynContentBasedOn = function (contentStatement, langCode) {
			this.unselectAllDynContents(false, true);
			for (var i = 0; i < this._visibleDynContentAnnotations.length; i++) {
				var dynContentAnnotation = this._visibleDynContentAnnotations[i];
				var dynContent = dynContentAnnotation.getDynContent();
				if (dynContent.getTextElement().parentStatement == contentStatement) {
					// If a langCode was specified, filter on that ... but only when one was specified.
					if (langCode && langCode == dynContent.getTextElement().language) {
						this._selectedDynContentAnnotations.push(dynContentAnnotation);
						dynContentAnnotation.setSelected(true);
					} else if (!langCode) {
						this._selectedDynContentAnnotations.push(dynContentAnnotation);
						dynContentAnnotation.setSelected(true);
					}
				}
			}

			//If user has selected state filter and then selects the text in text content widget that is other than the filter, 
			//then we dont zoom or show the text highlight on the canvas, as the annotations are removed for other states than filter.
			//Eg: filter is set to draft, and in widget user select statement that is signed-off, then we dont do anything on canvas

			if (this._selectedDynContentAnnotations.length > 0) {
				// Indicate that we should zoom to these dynContents
				this.f.emit("module.dyncontent.zoomToDynContents", this._selectedDynContentAnnotations);

				// Indicate that these dynContents are selected 
				this._emit("selectedDynContents", this._selectedDynContentAnnotations, true);

				this.ensureVisibility();
			}

			this.modelChanged();
		};


		/**
		 * Select a dynamic content. All other selected dynamic contents will be unselected first
		 * 
		 * @param {DynContentAnnotation} dynContentAnnotation - Dynamic Content to be selected
		 * @param {Boolean} force - makes the unselectAllDynamicContents forced
		 */
		DynContentModel.prototype.selectDynContent = function (dynContentAnnotation, force) {

			if (!dynContentAnnotation) {
				return;
			}

			if (this._visibleDynContentAnnotations.length === 1) {
				var sDyn = this._visibleDynContentAnnotations[0];
				if (dynContentAnnotation == sDyn) {
					return;
				}
			}

			this.unselectAllDynContents(false, force);
			// Select the targeted one.
			this._selectedDynContentAnnotations.push(dynContentAnnotation);
			dynContentAnnotation.setSelected(true);
			// Select also all other dyncontent annotation with the same text element.
			var textElement = dynContentAnnotation.getDynContent().getTextElement();
			// Loop over all dyncontent annotations.
			for (var i = 0; i < this._visibleDynContentAnnotations.length; i++) {
				// If the dyncontent has the same text element as the selected one ...
				if (this._visibleDynContentAnnotations[i].getDynContent().getTextElement() == textElement) {
					// Select that annotation if it's not already.
					if ($.inArray(this._visibleDynContentAnnotations[i], this._selectedDynContentAnnotations) < 0) {
						this._selectedDynContentAnnotations.push(this._visibleDynContentAnnotations[i]);
						this._visibleDynContentAnnotations[i].setSelected(true);
					}
				}
			}
			this.ensureVisibility();

			// On mobile devices ... also zoom 
			if (this._layoutType == LayoutConstants.LayoutType.MOBILE) {
				this.f.emit("module.dyncontent.zoomToDynContents", this._selectedDynContentAnnotations);
			}

			// Indicate that we selected a dynContent annotation.
			this._emit("selectedDynContents", this._selectedDynContentAnnotations, true);
			this.modelChanged();
		};

		/**
		 * Make sure that the annotations are visible. If not, set the visible property back to 1
		 */
		DynContentModel.prototype.ensureVisibility = function () {
			if (!this.f.getOption("PREFERENCES_DYNCONTENTTOOLS_VISIBLE")) {
				this.f.setOption("PREFERENCES_DYNCONTENTTOOLS_VISIBLE", 1);
			}
		};

		/**
		 * Get an annotation based on the annotation name
		 * @param {Number} i - The index of the dynamic content.
		 */
		DynContentModel.prototype.getDynamicContent = function (i) {
			return this._dynContentAnnotations[i];
		};

		/**
		 * Unselect all the dynamic contents
		 * @param {Boolean} [donotbroadcast] - if set, the model will not broadcast that this happened (only used when the caller will call modelChanged() itself)
		 * @param {Boolean} [force] - if set, it will ignore the lockSelection flag
		 */
		DynContentModel.prototype.unselectAllDynContents = function (donotbroadcast, force) {
			if (this._selectedDynContentAnnotations.length <= 0) {
				return;
			}

			var selectedDynContents = this._selectedDynContentAnnotations;
			this._selectedDynContentAnnotations = [];

			_.each(selectedDynContents, _.bind(function (dynContent) {
				dynContent.setSelected(false);
			}, this));

			if (!donotbroadcast) {
				this.modelChanged();
				this._emit("unselectedDynContents", selectedDynContents, force);
			}
		};

		/**
		 * Get the selected dynamic contents
		 * 
		 * @returns {Array.<DynContentAnnotation>} list of selected annotations
		 */
		DynContentModel.prototype.getSelectedDynamicContents = function () {
			return this._selectedDynContentAnnotations;
		};

		/**
		 * Inform the listeners that the model has changed.
		 * 
		 * @param {DynContentAnnotation} dynContent - the dynamic content that has been updated
		 */
		DynContentModel.prototype.modelChanged = function () {
			this._emit("modelChanged");
		};

		/**
		 * Inform the listeners that the complete content has changed for the model (i.o.w, the _annotations are changed)
		 */
		DynContentModel.prototype.modelContentChanged = function () {
			this._emit("modelContentChanged");
		};

		/**
		 * Set the page for which dynamic content  should be displayed (this method will fetch the annotation for the page if necessary)
		 * @param {Data.AbstractDocumentPage} newPage - page for which dynamic content should be displayed
		 * @param {Function} [callback] - called when the page has changed
		 */
		DynContentModel.prototype.setPage = function (newPage, callback, forceUpdate) {
			if (newPage != this._page || forceUpdate) {

				// reset annotations
				this._isDynContentsLoading = true;
				this.setDynContents([]);

				this._emit("changed.page");
				this.unselectAllDynContents(false, true);
				this._page = newPage;

				if (newPage) {
					// load the annotations for the newPage
					var dynContentReq = dynContentService.getDynContentFor(newPage.getDocumentVersion().getDocumentVersionID(), forceUpdate);
					$.when(dynContentReq).done(function (dynContents) {
							var dynContentAnnotations = [];
							var pageSize = this._page.getPageSize();
							for (var i = 0; i < dynContents.length; i++) {
								var dynContLocations = dynContents[i].getLocations();
								if (dynContLocations.length > 0) {
									for (var j = 0; j < dynContLocations.length; j++) {
										var loc = dynContLocations[j];
										loc.orgy = loc.orgy + pageSize.height;
										dynContentAnnotations.push(new DynContentAnnotation(this.f, dynContents[i], loc));
									}
								} else {
									dynContentAnnotations.push(new DynContentAnnotation(this.f, dynContents[i], undefined, pageSize));
								}
							}
							this._isDynContentsLoading = false;
							this.setDynContents(dynContentAnnotations);
							if (callback) {
								callback(dynContents);
							}
						}.bind(this))
						.fail(function (errorMsg) { // on error
							//TODO
						});

				}
			} else {
				if (callback !== undefined) {
					callback([]);
				}
			}
			this.modelChanged();
		};

		/**
		 * Get the page for which annotations are shown currently
		 * @returns {Data.AbstractDocumentPage}
		 */
		DynContentModel.prototype.getPage = function () {
			return this._page;
		};

		/**
		 * Prompt to save unsaved dynamic contents if there are any
		 * if there are unsaved dynamic contents, save them
		 *
		 * NOTE: this is not a pure model function, but to limit regressions, we keep it this way...
		 * @param {Function} callbackOnOK - function which is called when the popup is closed and annotations are saved
		 * @param {Function} [callbackOnCancel] - if provided, this callback is called when the user presses cancel
		 * @param {Function} [callbackOnDontSave] - adds the possibility to not save unsaved annotation but to continue with the action, on the contrary, cancel will cancel the action and don't save. Note that reloading the annotations should be done by the caller!
		 * @returns {Boolean} true if there are unsaved items and the popup is shown, false if not
		 */
		DynContentModel.prototype.promptToSaveUnsavedDynamicContent = function (callbackOnOK, callbackOnCancel, callbackOnDontSave) {
			if (callbackOnCancel === undefined) {
				callbackOnCancel = callbackOnOK;
			}

			if (this._page && this.getUnsavedDynContentCount() > 0) {
				this.f.loadBundle("viewer", function () {
					// show popup if there are unsaved items
					var modalPopup = new ModalPopup(this.f, "unsavedDynContentsPopup", this.f.getContext());
					modalPopup.setRemoveFromDomAfterUsage(true);
					modalPopup.setOkAction(_.bind(function () {
						modalPopup.enableOkBtn(false);

						// save annotations
						this.save(null, _.bind(function () {
							// saving succeeded, so hide
							modalPopup.hide();
							if (callbackOnOK) {
								callbackOnOK();
							}
							this.toggleVisibility(true);
						}, this));
					}, this));

					modalPopup.setCancelAction(_.bind(function () {
						if (callbackOnCancel) {
							callbackOnCancel();
						}
						this.toggleVisibility(true);
					}, this));

					if (callbackOnDontSave) {
						var dontSaveButton = new ViewerToolbarButton(this.f, "dontSaveDynContents", $("<span>").append(this.f.t("RevertDynChanges")), false, _.bind(function () {
							modalPopup.hide();
							this.toggleVisibility(true);

							if (callbackOnDontSave) {
								callbackOnDontSave();
							}
						}, this));
						modalPopup.addButton(dontSaveButton);
					}

					// Header + Content
					modalPopup.setHeader($("<span/>").addClass("popupTitle").text(this.f.i18n.t("AnnTitle")));
					modalPopup.setContent($("<div/>").addClass("unsavedItemsContent").text(this.f.i18n.t("DynSavePrompt")));
					modalPopup.setOkBtnText(this.f.i18n.t("Save"));
					this.toggleVisibility(false);
					modalPopup.show();
					modalPopup.recalculateSize(600);
				}, this);

				return true;
			} else {
				return false;
			}
		};

		/**
		 * Set the layout type, the widget needs to be rendered for
		 * @param layoutType
		 */
		DynContentModel.prototype.setLayoutType = function (layoutType) {
			this._layoutType = layoutType;
		};

		/**
		 * Scale all the dynamic contents of the model with a given factor, both horizontal as vertical, changes all the points and rectangle of the annotation,
		 * first undos the scaling and than applies the new scaling, if no scaleFactor is provided the current scaling is undone
		 * @param {ShapeUtils.Point} scaleFactor
		 */
		DynContentModel.prototype.scaleAllDynContents = function (scaleFactor) {
			if (this._currentScaling) {
				_.each(this._dynContentAnnotations, function (dynContent) {
					if (dynContent.isScaled()) {
						dynContent.scale(new ShapeUtils.Point(1 / dynContent.getScale().x, 1 / dynContent.getScale().y));
					}
				}, this);
			}
			this._currentScaling = scaleFactor;
			_.each(this._dynContentAnnotations, function (dynContent) {
				dynContent.setScale(scaleFactor);
			});
		};

		/**
		 * Get the currently active scaling
		 * @returns {ShapeUtils.Point}
		 */
		DynContentModel.prototype.getCurrentScaling = function () {
			return this._currentScaling;
		};

		return DynContentModel;

	});

define('web-viewer/textContent/DynContentLayerModel',["web-viewer/viewport/model/ViewportLayerModel",
        "web-viewer/textContent/DynContentAnnotation",
		"utils/ShapeUtils",
		"web-viewer/utils/ListenerUtil",
		"web-viewer/viewControls/ViewControlsConstants",
		"underscore"
	],
	function (ViewportLayerModel,
		DynContentAnnotation,
		ShapeUtils,
		ListenerUtil,
		ViewControlsConstants,
		_
	) {
		"use strict";

		/**
		 * Create a new annotation layer model
		 * @exports DynContentLayerModel
		 * @memberof TextContentModule
		 *
		 * @class This class is responsible for drawing annotations	 *
		 * @constructor
		 * @param {Core.ViewerFacade} f - Viewerfacade
		 * @param {String} ID - Id of the layer
		 * @param {DynContentModel} dynContentModel - Model of the dynamic content objects.
		 */
		var DynContentLayerModel = function (f, ID, dynContentModel) {
			ViewportLayerModel.call(this, f, ID);
			ListenerUtil.mixin(this, f);

			this.f = f;
			this.dynContentModel = dynContentModel;

			this._registerListener("modelChanged", this._calculateTransformation, this, this.dynContentModel);
			this._registerListener("modelContentChanged", this.modelChanged, this, this.dynContentModel);
			this._registerListener("parameters.changed", this._calculateTransformation, this, this.f.getDocumentController());
			this._registerListener("transform.viewportWidget.transformChanged", this._calculateTransformation, this);
		};
		
		DynContentLayerModel.prototype = Object.create(ViewportLayerModel.prototype);
		DynContentLayerModel.prototype.constructor = DynContentLayerModel;

		/**
		 * Calculates the transformation for the dynamic content layer, takes compensation for distortion and scaling into account
		 */
		DynContentLayerModel.prototype._calculateTransformation = function () {
			var page = this.dynContentModel.getDynContentPage();
			if (page !== null && page !== undefined) {
				this.resetTransformation();
				this.getTransform().scale(1, -1);

				var rotation = page.getRotation();

				var worldBounds = this.f.getModule("viewport").getWorldBounds();

				//calculate the rest of the tranformation, for rotation, flipped state
				this.calculateTransformation(worldBounds, page.getPageSize(), new ShapeUtils.Size(), rotation, page.isFlipped());
			}
			this.modelChanged();
		};
		
		/**
		 * @override
		 */
		DynContentLayerModel.prototype.getLinework = function (animated, rotationanimation) {
			
			// Get all visible dyncontent annotations.
			var dynContentAnnotations = this.dynContentModel.getVisibleDynContents();
			// Make a quick shallow copy of this array.
			dynContentAnnotations = dynContentAnnotations.slice(0,dynContentAnnotations.length);
			
			// Get the selected annotations.
			var selectedDynContents = this.dynContentModel.getSelectedDynamicContents();
			// Generate the selection boxes for the selected annotations.
			var selectionBoxes = [];
			for (var i=0; i < selectedDynContents.length; i++) {
				// Only show selection boxes if the dynContent is visible.
				if ($.inArray(selectedDynContents[i], dynContentAnnotations) > -1) {
					selectionBoxes.push(selectedDynContents[i].getSelectionBox());
				}
			}
			
			// Make sure we don't double fill annotations on top of each other.
			// Group the annotations based on their location.
			var annotationsGrouped = [];
			while (dynContentAnnotations.length > 0) {
				var annotation1 = dynContentAnnotations.shift();
				var annotationGroup = [annotation1];
				for (var i=0; i < dynContentAnnotations.length; i++) {
					var annotation2 = dynContentAnnotations[i];
					if (annotation1.rect.rectEquals(annotation2.rect)) {
						annotationGroup.push(annotation2);
						dynContentAnnotations.splice(i, 1);
						i--;
					}
				}
				annotationsGrouped.push(annotationGroup);
			}
			
			// Iterate over every annotation group (all annotations in a group or on the same location).
			for (var i=0; i < annotationsGrouped.length; i++) {
				var annotationGroup = annotationsGrouped[i];
				
				// Check if there are multiple annotations in this group.
				if (annotationGroup.length > 1) {
					
					var transLang = this.dynContentModel.getCurrentTransLangCode();
					annotationGroup.sort(function(annotation1, annotation2) {
						// First sort on selected.
						if (annotation1.selected && !annotation2.selected) {
							return 1;
						} else if (!annotation1.selected && annotation2.selected) {
							return -1;
						} else if (annotation1.selected && annotation2.selected) {
							return 0;
						} else {
							// Sort on language.
							var lang1 = annotation1.getDynContent().getTextElement().language;
							var lang2 = annotation2.getDynContent().getTextElement().language;
							if (lang1 == transLang) {
								return 1;
							} else if (lang2 == transLang) {
								return -1;
							} else {
								return 0;
							}
						}
					});
					
					// Now disable the fill color of every annotation but the last.
					for (var j=0; j < annotationGroup.length; j++) {
						if (j != annotationGroup.length-1) {
							annotationGroup[j].fillcolor = undefined;
						} else {
							annotationGroup[j].reCalcStyle();
						}
					}
				}
			}
			// After removing any potential anoying fills ... flatten our grouped annotations again.
			dynContentAnnotations = _.flatten(annotationsGrouped);
			
			var linework = _.union(dynContentAnnotations, _.without(_.flatten(selectionBoxes), undefined, null));
			return linework;
			
		};
		
		return DynContentLayerModel;
});

define('web-viewer/viewport/model/ViewportPopupModel',["underscore",
		"web-viewer/core/PrivateChannel"
	],
	function (_,
		PrivateChannel) {
		/**
		 * Construct a new viewport popup model
		 * @exports ViewportPopupModel
		 * @memberof ViewportModule
		 * @class
		 * This model is responsible for the state (shown/not shown) of the popup.
		 * It also contains the popup itself.
		 * @param {Core.ViewerFacade} f - Viewerfacade
		 * @param {String} id - Id of the viewport popup model
		 */
		var ViewportPopupModel = function (f, id) {
			this.f = f;
			this.id = id;
			this.shown = true;
		};

		/**
		 * Get the id of the popup model
		 * @returns {String} The id of the popup model
		 */
		ViewportPopupModel.prototype.getId = function () {
			return this.id;
		};

		/**
		 * Indicate that the popup should be shown.
		 */
		ViewportPopupModel.prototype.show = function () {
			this.shown = true;
		};

		/**
		 * Indicate that the popup should be hidden.
		 */
		ViewportPopupModel.prototype.hide = function () {
			this.shown = false;
		};

		/**
		 * Check whether the popup is shown.
		 * @returns {Boolean} True if the popup is shown, false otherwise/
		 */
		ViewportPopupModel.prototype.isShown = function () {
			return this.shown;
		};

		/**
		 * Get the popup belonging to this model.
		 * Override this function.
		 * @param {ViewportLayer} viewportLayer - The layer where the popups should be shown.
		 * @returns {Popup} The popup belonging to this model.
		 */
		ViewportPopupModel.prototype.getPopup = function (viewportLayer) {
			// override
			return undefined;
		};

		/**
		 * Called when the popup model is destroyed.
		 * Override this function.
		 * Useful to do some cleanup.
		 */
		ViewportPopupModel.prototype.onDestroy = function () {
			// override to do some cleanup
		};

		/**
		 * Return the world origin of the popup
		 * Override this function.
		 * @returns {ShapeUtils.Point} - Origin of the popup.
		 */
		ViewportPopupModel.prototype.getWorldOrigin = function () {
			// override
			return undefined;
		};

		/**
		 * Returns the world size of the popup
		 * Override this function.
		 * @returns {ShapeUtils.Size} - Size of the popup.
		 */
		ViewportPopupModel.prototype.getWorldSize = function () {
			// override
			return undefined;
		};

		/**
		 * When there needs to be added an offset to the popup location (in scene pixels), return the offset here
		 * @returns {ShapeUtils.Size} offset in pixels that needs to be added to the popuplocation
		 */
		ViewportPopupModel.prototype.getSceneOffset = function () {
			return undefined;
		};

		/**
		 * Function called by the layer after translating the scene rectangle to world coordinates.
		 * Override this function.
		 * @param {ShapeUtils.Rect} worldRect - Rectangle in world coordinates
		 * @param {Boolean} autoAdapted - True if the event is fired because the content was auto adapted (e.g. from a makeMeVisible trigger from an element in the content), false otherwise
		 */
		ViewportPopupModel.prototype.onResized = function (worldRect, autoAdapted) {
			// override
		};

		/**
		 * Function called by the layer after translating the scene point to world coordinates.
		 * Override this function.
		 * @param {ShapeUtils.Point} worldPoint - Point in world coordinates
		 */
		ViewportPopupModel.prototype.onDragged = function (worldPoint) {
			// override
		};

		return ViewportPopupModel;
	});

define('utils/webcss-text!lib/summernote/summernote.css',[],function () { return '@font-face{font-family:"summernote";font-style:normal;font-weight:normal;src:url("font/summernote.eot?ad8d7e2d177d2473aecd9b35d16211fb");src:url("font/summernote.eot?#iefix") format("embedded-opentype"),url("font/summernote.woff?ad8d7e2d177d2473aecd9b35d16211fb") format("woff"),url("font/summernote.ttf?ad8d7e2d177d2473aecd9b35d16211fb") format("truetype")}[class^="note-icon-"]:before,[class*=" note-icon-"]:before{display:inline-block;font:normal normal normal 14px summernote;font-size:inherit;-webkit-font-smoothing:antialiased;text-decoration:inherit;text-rendering:auto;text-transform:none;vertical-align:middle;speak:none;-moz-osx-font-smoothing:grayscale}.note-icon-align-center:before{content:"\\f101"}.note-icon-align-indent:before{content:"\\f102"}.note-icon-align-justify:before{content:"\\f103"}.note-icon-align-left:before{content:"\\f104"}.note-icon-align-outdent:before{content:"\\f105"}.note-icon-align-right:before{content:"\\f106"}.note-icon-align:before{content:"\\f107"}.note-icon-arrows-alt:before{content:"\\f108"}.note-icon-bold:before{content:"\\f109"}.note-icon-caret:before{content:"\\f10a"}.note-icon-chain-broken:before{content:"\\f10b"}.note-icon-circle:before{content:"\\f10c"}.note-icon-close:before{content:"\\f10d"}.note-icon-code:before{content:"\\f10e"}.note-icon-eraser:before{content:"\\f10f"}.note-icon-font:before{content:"\\f110"}.note-icon-frame:before{content:"\\f111"}.note-icon-italic:before{content:"\\f112"}.note-icon-link:before{content:"\\f113"}.note-icon-magic:before{content:"\\f114"}.note-icon-menu-check:before{content:"\\f115"}.note-icon-minus:before{content:"\\f116"}.note-icon-orderedlist:before{content:"\\f117"}.note-icon-pencil:before{content:"\\f118"}.note-icon-picture:before{content:"\\f119"}.note-icon-question:before{content:"\\f11a"}.note-icon-redo:before{content:"\\f11b"}.note-icon-special-character:before{content:"\\f11c"}.note-icon-square:before{content:"\\f11d"}.note-icon-strikethrough:before{content:"\\f11e"}.note-icon-subscript:before{content:"\\f11f"}.note-icon-summernote:before{content:"\\f120"}.note-icon-superscript:before{content:"\\f121"}.note-icon-table:before{content:"\\f122"}.note-icon-text-height:before{content:"\\f123"}.note-icon-trash:before{content:"\\f124"}.note-icon-underline:before{content:"\\f125"}.note-icon-undo:before{content:"\\f126"}.note-icon-unorderedlist:before{content:"\\f127"}.note-icon-video:before{content:"\\f128"}.note-editor{position:relative}.note-editor .note-dropzone{position:absolute;z-index:100;display:none;color:#87cefa;background-color:white;opacity:.95;pointer-event:none}.note-editor .note-dropzone .note-dropzone-message{display:table-cell;font-size:28px;font-weight:bold;text-align:center;vertical-align:middle}.note-editor .note-dropzone.hover{color:#098ddf}.note-editor.dragover .note-dropzone{display:table}.note-editor .note-editing-area{position:relative}.note-editor .note-editing-area .note-editable{outline:0}.note-editor .note-editing-area .note-editable sup{vertical-align:super}.note-editor .note-editing-area .note-editable sub{vertical-align:sub}.note-editor.note-frame{border:1px solid #a9a9a9}.note-editor.note-frame.codeview .note-editing-area .note-editable{display:none}.note-editor.note-frame.codeview .note-editing-area .note-codable{display:block}.note-editor.note-frame .note-editing-area{overflow:hidden}.note-editor.note-frame .note-editing-area .note-editable{padding:10px;overflow:auto;color:#000;background-color:#fff}.note-editor.note-frame .note-editing-area .note-editable[contenteditable="false"]{background-color:#e5e5e5}.note-editor.note-frame .note-editing-area .note-codable{display:none;width:100%;padding:10px;margin-bottom:0;font-family:Menlo,Monaco,monospace,sans-serif;font-size:14px;color:#ccc;background-color:#222;border:0;-webkit-border-radius:0;-moz-border-radius:0;border-radius:0;box-shadow:none;-webkit-box-sizing:border-box;-moz-box-sizing:border-box;-ms-box-sizing:border-box;box-sizing:border-box;resize:none}.note-editor.note-frame.fullscreen{position:fixed;top:0;left:0;z-index:1050;width:100%}.note-editor.note-frame.fullscreen .note-editable{background-color:white}.note-editor.note-frame.fullscreen .note-resizebar{display:none}.note-editor.note-frame .note-statusbar{background-color:#f5f5f5;border-bottom-right-radius:4px;border-bottom-left-radius:4px}.note-editor.note-frame .note-statusbar .note-resizebar{width:100%;height:8px;padding-top:1px;cursor:ns-resize}.note-editor.note-frame .note-statusbar .note-resizebar .note-icon-bar{width:20px;margin:1px auto;border-top:1px solid #a9a9a9}.note-editor.note-frame .note-placeholder{padding:10px}.note-popover.popover{max-width:none}.note-popover.popover .popover-content a{display:inline-block;max-width:200px;overflow:hidden;text-overflow:ellipsis;white-space:nowrap;vertical-align:middle}.note-popover.popover .arrow{left:20px!important}.note-popover .popover-content,.panel-heading.note-toolbar{padding:0 0 5px 5px;margin:0}.note-popover .popover-content>.btn-group,.panel-heading.note-toolbar>.btn-group{margin-top:5px;margin-right:5px;margin-left:0}.note-popover .popover-content .btn-group .note-table,.panel-heading.note-toolbar .btn-group .note-table{min-width:0;padding:5px}.note-popover .popover-content .btn-group .note-table .note-dimension-picker,.panel-heading.note-toolbar .btn-group .note-table .note-dimension-picker{font-size:18px}.note-popover .popover-content .btn-group .note-table .note-dimension-picker .note-dimension-picker-mousecatcher,.panel-heading.note-toolbar .btn-group .note-table .note-dimension-picker .note-dimension-picker-mousecatcher{position:absolute!important;z-index:3;width:10em;height:10em;cursor:pointer}.note-popover .popover-content .btn-group .note-table .note-dimension-picker .note-dimension-picker-unhighlighted,.panel-heading.note-toolbar .btn-group .note-table .note-dimension-picker .note-dimension-picker-unhighlighted{position:relative!important;z-index:1;width:5em;height:5em;background:url(\'data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABIAAAASAgMAAAAroGbEAAAACVBMVEUAAIj4+Pjp6ekKlAqjAAAAAXRSTlMAQObYZgAAAAFiS0dEAIgFHUgAAAAJcEhZcwAACxMAAAsTAQCanBgAAAAHdElNRQfYAR0BKhmnaJzPAAAAG0lEQVQI12NgAAOtVatWMTCohoaGUY+EmIkEAEruEzK2J7tvAAAAAElFTkSuQmCC\') repeat}.note-popover .popover-content .btn-group .note-table .note-dimension-picker .note-dimension-picker-highlighted,.panel-heading.note-toolbar .btn-group .note-table .note-dimension-picker .note-dimension-picker-highlighted{position:absolute!important;z-index:2;width:1em;height:1em;background:url(\'data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAABIAAAASAgMAAAAroGbEAAAACVBMVEUAAIjd6vvD2f9LKLW+AAAAAXRSTlMAQObYZgAAAAFiS0dEAIgFHUgAAAAJcEhZcwAACxMAAAsTAQCanBgAAAAHdElNRQfYAR0BKwNDEVT0AAAAG0lEQVQI12NgAAOtVatWMTCohoaGUY+EmIkEAEruEzK2J7tvAAAAAElFTkSuQmCC\') repeat}.note-popover .popover-content .note-style h1,.panel-heading.note-toolbar .note-style h1,.note-popover .popover-content .note-style h2,.panel-heading.note-toolbar .note-style h2,.note-popover .popover-content .note-style h3,.panel-heading.note-toolbar .note-style h3,.note-popover .popover-content .note-style h4,.panel-heading.note-toolbar .note-style h4,.note-popover .popover-content .note-style h5,.panel-heading.note-toolbar .note-style h5,.note-popover .popover-content .note-style h6,.panel-heading.note-toolbar .note-style h6,.note-popover .popover-content .note-style blockquote,.panel-heading.note-toolbar .note-style blockquote{margin:0}.note-popover .popover-content .note-color .dropdown-toggle,.panel-heading.note-toolbar .note-color .dropdown-toggle{width:20px;padding-left:5px}.note-popover .popover-content .note-color .dropdown-menu,.panel-heading.note-toolbar .note-color .dropdown-menu{min-width:340px}.note-popover .popover-content .note-color .dropdown-menu .btn-group,.panel-heading.note-toolbar .note-color .dropdown-menu .btn-group{margin:0}.note-popover .popover-content .note-color .dropdown-menu .btn-group:first-child,.panel-heading.note-toolbar .note-color .dropdown-menu .btn-group:first-child{margin:0 5px}.note-popover .popover-content .note-color .dropdown-menu .btn-group .note-palette-title,.panel-heading.note-toolbar .note-color .dropdown-menu .btn-group .note-palette-title{margin:2px 7px;font-size:12px;text-align:center;border-bottom:1px solid #eee}.note-popover .popover-content .note-color .dropdown-menu .btn-group .note-color-reset,.panel-heading.note-toolbar .note-color .dropdown-menu .btn-group .note-color-reset{width:100%;padding:0 3px;margin:3px;font-size:11px;cursor:pointer;-webkit-border-radius:5px;-moz-border-radius:5px;border-radius:5px}.note-popover .popover-content .note-color .dropdown-menu .btn-group .note-color-row,.panel-heading.note-toolbar .note-color .dropdown-menu .btn-group .note-color-row{height:20px}.note-popover .popover-content .note-color .dropdown-menu .btn-group .note-color-reset:hover,.panel-heading.note-toolbar .note-color .dropdown-menu .btn-group .note-color-reset:hover{background:#eee}.note-popover .popover-content .note-para .dropdown-menu,.panel-heading.note-toolbar .note-para .dropdown-menu{min-width:216px;padding:5px}.note-popover .popover-content .note-para .dropdown-menu>div:first-child,.panel-heading.note-toolbar .note-para .dropdown-menu>div:first-child{margin-right:5px}.note-popover .popover-content .dropdown-menu,.panel-heading.note-toolbar .dropdown-menu{min-width:90px}.note-popover .popover-content .dropdown-menu.right,.panel-heading.note-toolbar .dropdown-menu.right{right:0;left:auto}.note-popover .popover-content .dropdown-menu.right::before,.panel-heading.note-toolbar .dropdown-menu.right::before{right:9px;left:auto!important}.note-popover .popover-content .dropdown-menu.right::after,.panel-heading.note-toolbar .dropdown-menu.right::after{right:10px;left:auto!important}.note-popover .popover-content .dropdown-menu.note-check li a i,.panel-heading.note-toolbar .dropdown-menu.note-check li a i{color:deepskyblue;visibility:hidden}.note-popover .popover-content .dropdown-menu.note-check li a.checked i,.panel-heading.note-toolbar .dropdown-menu.note-check li a.checked i{visibility:visible}.note-popover .popover-content .note-fontsize-10,.panel-heading.note-toolbar .note-fontsize-10{font-size:10px}.note-popover .popover-content .note-color-palette,.panel-heading.note-toolbar .note-color-palette{line-height:1}.note-popover .popover-content .note-color-palette div .note-color-btn,.panel-heading.note-toolbar .note-color-palette div .note-color-btn{width:20px;height:20px;padding:0;margin:0;border:1px solid #fff}.note-popover .popover-content .note-color-palette div .note-color-btn:hover,.panel-heading.note-toolbar .note-color-palette div .note-color-btn:hover{border:1px solid #000}.note-dialog>div{display:none}.note-dialog .form-group{margin-right:0;margin-left:0}.note-dialog .note-modal-form{margin:0}.note-dialog .note-image-dialog .note-dropzone{min-height:100px;margin-bottom:10px;font-size:30px;line-height:4;color:lightgray;text-align:center;border:4px dashed lightgray}@-moz-document url-prefix(){.note-image-input{height:auto}}.note-placeholder{position:absolute;display:none;color:gray}.note-handle .note-control-selection{position:absolute;display:none;border:1px solid black}.note-handle .note-control-selection>div{position:absolute}.note-handle .note-control-selection .note-control-selection-bg{width:100%;height:100%;background-color:black;-webkit-opacity:.3;-khtml-opacity:.3;-moz-opacity:.3;opacity:.3;-ms-filter:alpha(opacity=30);filter:alpha(opacity=30)}.note-handle .note-control-selection .note-control-handle{width:7px;height:7px;border:1px solid black}.note-handle .note-control-selection .note-control-holder{width:7px;height:7px;border:1px solid black}.note-handle .note-control-selection .note-control-sizing{width:7px;height:7px;background-color:white;border:1px solid black}.note-handle .note-control-selection .note-control-nw{top:-5px;left:-5px;border-right:0;border-bottom:0}.note-handle .note-control-selection .note-control-ne{top:-5px;right:-5px;border-bottom:0;border-left:none}.note-handle .note-control-selection .note-control-sw{bottom:-5px;left:-5px;border-top:0;border-right:0}.note-handle .note-control-selection .note-control-se{right:-5px;bottom:-5px;cursor:se-resize}.note-handle .note-control-selection .note-control-se.note-control-holder{cursor:default;border-top:0;border-left:none}.note-handle .note-control-selection .note-control-selection-info{right:0;bottom:0;padding:5px;margin:5px;font-size:12px;color:white;background-color:black;-webkit-border-radius:5px;-moz-border-radius:5px;border-radius:5px;-webkit-opacity:.7;-khtml-opacity:.7;-moz-opacity:.7;opacity:.7;-ms-filter:alpha(opacity=70);filter:alpha(opacity=70)}.note-hint-popover{min-width:100px;padding:2px}.note-hint-popover .popover-content{max-height:150px;padding:3px;overflow:auto}.note-hint-popover .popover-content .note-hint-group .note-hint-item{display:block!important;padding:3px}.note-hint-popover .popover-content .note-hint-group .note-hint-item.active,.note-hint-popover .popover-content .note-hint-group .note-hint-item:hover{display:block;clear:both;font-weight:400;line-height:1.4;color:white;text-decoration:none;white-space:nowrap;cursor:pointer;background-color:#428bca;outline:0}';});

define('utils/webcss-text!packaging-content-management/text-content-element-widget/RichTextEditor.css',[],function () { return '.WcrPageContent .RichTextEditor {\r\n\twidth: 100%;\r\n\theight: 100%;\r\n\toverflow-y: auto;\r\n\tpadding: 3px;\r\n\tcursor: text;\r\n\tbackground-color: white;\r\n}\r\n.WcrPageContent .RichTextEditor.Disabled {\r\n\tbackground-color: #f2f2f2;\r\n}\r\n.WcrPageContent .RichTextEditor p {\r\n\tmargin: 0px !important;\r\n\tmargin-top: 3px !important;\r\n\tpadding: 0px !important;\r\n}';});

define('packaging-content-management/text-content-element-widget/RichTextEditor',["jquery",
        "lib/summernote/summernote",
        "utils/webcss!lib/summernote/summernote.css",
        "utils/webcss!./RichTextEditor.css"],
	function ($) {
	
	
	var RichTextEditor = function (content, height) {
		
		// The array holding all the "onChange" callbacks.
		this._onChange = [];
		
		// The wrapping div of the editor.
		this._wrapper = $("<div>");
		this._wrapper.addClass("RichTextEditor");
		
		// Create the div that will be hidden and function as the handle for summernote.
		this._handle = $("<div>");
		
		// Parse the content.
		var parsedContent = undefined;
		try {
			parsedContent = $(content);
		} catch (exception) {
			// This is not valid html ... ugh ... just insert in the editor and hope for the best.
			this._handle.html(content);
		}
		// Check if the content could be parsed.
		if (parsedContent && parsedContent.length == 0) {
			// Parsed content could not be parsed, just dump it in the editor and hope for the best.
			this._handle.html(content);
		}
		// Check if the root element of content is "<body>" (we can't insert that in the editor).
		else if (parsedContent && parsedContent.length == 1 && parsedContent.is("body")) {
			// We need to insert all of the parsed content except the body tag itself.
			this._handle.html(parsedContent.html());
		} 
		// If we get here, then we have perfectly fine rich text. 
		else {
			this._handle.html(content);
		}
						
		// Add the handle to the wrapper.
		this._wrapper.append(this._handle);
		
		// Create the editor.
		this._handle.summernote({
			airMode: true,
			disableDragAndDrop: true,
			popover: {
				air: [['style', ['bold', 'italic', 'underline', 'clear']],
				      ['font', ['strikethrough', 'superscript', 'subscript']]]
			},
			keyMap: {
				  pc: {
				    'ENTER': 'insertParagraph',
				    'CTRL+Z': 'undo',
				    'CTRL+Y': 'redo',
				    'TAB': 'tab',
				    'SHIFT+TAB': 'untab',
				    'CTRL+B': 'bold',
				    'CTRL+I': 'italic',
				    'CTRL+U': 'underline',
				    'CTRL+SHIFT+S': 'strikethrough',
				    'CTRL+BACKSLASH': 'removeFormat'
				  },
				  mac: {
				    'ENTER': 'insertParagraph',
				    'CMD+Z': 'undo',
				    'CMD+SHIFT+Z': 'redo',
				    'TAB': 'tab',
				    'SHIFT+TAB': 'untab',
				    'CMD+B': 'bold',
				    'CMD+I': 'italic',
				    'CMD+U': 'underline',
				    'CMD+SHIFT+S': 'strikethrough',
				    'CMD+BACKSLASH': 'removeFormat',
				    'CMD+SHIFT+NUM7': 'insertUnorderedList',
				    'CMD+SHIFT+NUM8': 'insertOrderedList',
				    'CMD+LEFTBRACKET': 'outdent',
				    'CMD+RIGHTBRACKET': 'indent'
				  }
			},
			callbacks: {
				onPaste: this._handlePaste.bind(this) // Handle the paste event.
			}
		});
		this._handle.summernote("setHeight", (height-10)+"px");
		this._handle.summernote("setStyle", "word-wrap", "break-word");
		
		// When text is not wrapped with p tags then it required to give padding for text with superscript style.
		this._handle.summernote("setStyle", 'padding-top', '5px');
		
		// When the wrapper is clicked ... transfer focus to the editor.
		this._wrapper.on("click", function () {
			this._handle.summernote("focus");
		}.bind(this));
		
		// Execute change call-backs when content is changed in editor.
		this._handle.on('summernote.change', function () {
			$.each(this._onChange, function (index, changeCallback) {
				changeCallback();
			}.bind(this));
		}.bind(this));
	};
	
	/**
	 * Filter the god damn frigging paste event.
	 */
	RichTextEditor.prototype._handlePaste = function(e) {
		// Defer the handling of this event.
		window.setTimeout(function (e) {
			var allowedTags = ["p", "b", "i", "u", "sup", "br", "sub", "strike", "s"];
			var allowedEmptyTags = ["br"];
			var allowedAttributes = [];
			var changed = this._handle.summernote('cleanEditable', allowedTags, allowedEmptyTags, allowedAttributes);
			if (changed) {
				$.each(this._onChange, function (index, changeCallback) {
					changeCallback();
				}.bind(this));
			}
		}.bind(this, e), 0);
	};

	
	/**
	 * Disable editing.
	 */
	RichTextEditor.prototype.disable = function () {
		this._wrapper.addClass("Disabled");
		this._handle.summernote('disable');
	};
	
	/**
	 * Enable editing.
	 */
	RichTextEditor.prototype.enable = function () {
		this._wrapper.removeClass("Disabled");
		this._handle.summernote('enable');
	};
	
	/**
	 * Set the god damn content.
	 * 
	 * @returns
	 */
	RichTextEditor.prototype.setContent = function (content) {
		this._handle.summernote('code', content);
	};
	
	/**
	 * Get the god damn content.
	 * 
	 * @returns
	 */
	RichTextEditor.prototype.focus = function () {
		this._handle.summernote('focus');
		this._handle.summernote('restoreRange');
	};
	
	/**
	 * Get the god damn content.
	 * 
	 * @returns
	 */
	RichTextEditor.prototype.blur = function () {
		this._handle.summernote('saveRange');
		this._handle.summernote('blur');
		// Also clear the current selection.
		if (window.getSelection()) {
			window.getSelection().removeAllRanges();
		} else if (document.selection) {
			document.selection.empty();
		}
	};
	
	/**
	 * Get the god damn content.
	 * 
	 * @returns
	 */
	RichTextEditor.prototype.getContent = function () {
		
		var newContent = this._handle.summernote('code');
		var parsedNewContent = $.parseHTML(newContent);
		// Check if newCotent has one single root tag.
		if (parsedNewContent && parsedNewContent.length > 1) {
			// Create a "body" root tag to contain all the elements.
			var root = $("<body>");
			root.append(parsedNewContent);
			return root[0].outerHTML;
		} else {
			return newContent;
		}
		
		
	};
	
	/**
	 * Register an "onChange" callback.
	 * 
	 * @param callback
	 * @returns {Array}
	 */
	RichTextEditor.prototype.onChange = function (callback) {
		this._onChange.push(callback);
	};
	
	/**
	 * Return the wrapping jQuery element.
	 * @returns
	 */
	RichTextEditor.prototype.getJQElement = function () {
		return this._wrapper;
	};
	
	return RichTextEditor;
});

define('utils/webcss-text!packaging-content-management/text-content-element-widget/ContentStatementElementWidgetView.css',[],function () { return '\r\n.WcrPageContent .ContentStatementElementWidget {\r\n\theight: 120px;\r\n\tposition: relative;\r\n\twidth: 100%;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .TextContent-ApprovalCol {\r\n\tposition: absolute;\r\n\tleft: 0px;\r\n\ttop: 0px;\r\n\tbottom: 0px;\r\n\twidth: 60px;\r\n\ttext-align: center;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .TextContentElement-Status, .ContentStatementElementWidget .TextElementStatus_Delete {\r\n\tvertical-align: middle;\r\n\tcolor: white;\r\n\tpadding: 5px;\r\n\theight: 100%;\r\n\tposition: absolute;\r\n\ttop: 0px;\r\n\tbottom: 0px;\r\n\tleft: 0px;\r\n\tright: 0px;\r\n\tbackground-color: #D3D3D3;\r\n\tborder: 0px;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .TextContent-ApprovalStatus {\r\n\tposition: relative;\r\n\ttop: 50%;\r\n\ttransform: translateY(-50%);\r\n\t-webkit-transform: translateY(-50%);\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .TextContent-StatusIcon i {\r\n\tfont-size: 32px;\r\n\theight: 32px;\r\n\twidth: 32px;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .TextContent-StatusLabel {\r\n\twhite-space: normal;\r\n\tfont-size: 10px;\r\n\tmargin-top: -8px;\r\n\tpadding-top: 15px;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .ElementStatusDraft {\r\n\tbackground-color: #a7a9ac;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .ElementStatusPending {\r\n\tbackground-color: #46b8da;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .ElementStatusApproved {\r\n\tbackground-color: #4cae4c;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .ElementStatusRejected {\r\n\tbackground-color: #d43f3a;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .ElementStatusIconAndLabel {\r\n\tcolor: #fff;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .DeleteIconAndLabel {\r\n\tcolor: #5F5F5F;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .TextContent-Approval-NextStates {\r\n\tposition: absolute;\r\n\theight: 100%;\r\n\ttop: 0px;\r\n\tbottom: 0px;\r\n\tleft: 0px;\r\n\tright: 0px;\r\n\tborder: 0px;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .TextContent-ApprovalAction {\r\n\tcolor: white;\r\n\tcursor: pointer;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .ApprovalAction-InnerDiv {\r\n\tposition: relative;\r\n\ttop: 50%;\r\n\ttransform: translateY(-50%);\r\n\t-webkit-transform: translateY(-50%);\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .TextContent-ApprovalLabel {\r\n\tfont-size: 10px;\r\n\twhite-space: normal;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .ApproveText {\r\n\tbackground-color: #dff0d8;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .ApproveText:hover {\r\n\tbackground-color: #c3e3b5;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .ApproveTextIconAndLabel {\r\n\tcolor: #3c763d;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .RejectText {\r\n\tbackground-color: #ebccd1;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .RejectText:hover {\r\n\tbackground-color: #e2b6bd;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .RejectTextIconAndLabel {\r\n\tcolor: #a94442;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .StartTextApproval {\r\n\tbackground-color: #bce8f1;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .StartTextApproval:hover {\r\n\tbackground-color: #96dbe9;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .StartTextApprovalIconAndLabel {\r\n\tcolor: #31708f;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .RevertTextToDraft {\r\n\tbackground-color: #a7a9ac;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .RevertTextToDraft:hover {\r\n\tbackground-color: #96989c;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .RevertTextToDraftIconAndLabel {\r\n\tcolor: #e5e6e6;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .RevertApprovalAction {\r\n\tbackground-color: #e5e5e5;\r\n\tcursor: pointer;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .RevertApprovalAction:hover {\r\n\tbackground-color: #d9d9d9;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .RevertApprovalActionIconAndLabel {\r\n\tcolor: #4d4d4d;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .TextContent-1ApprovalAction {\r\n\tposition: relative;\r\n\theight: 100%;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .TextContent-1ApprovalAction i {\r\n\tfont-size: 32px;\r\n\theight: 32px;\r\n\twidth: 32px;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .TextContent-2ApprovalActions {\r\n\tposition: relative;\r\n\theight: 50%;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .TextContent-2ApprovalActions i {\r\n\tfont-size: 32px;\r\n\theight: 32px;\r\n\twidth: 32px;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .TextContent-3ApprovalActions {\r\n\tposition: relative;\r\n\theight: 33.33%;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .TextContent-3ApprovalActions i {\r\n\tfont-size: 32px;\r\n\theight: 32px;\r\n\twidth: 32px;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .TextContent-ContentCol {\r\n\tposition: absolute;\r\n\ttop: 0px;\r\n\tleft: 60px;\r\n\tright: 0px;\r\n\tbottom: 0px;\r\n\theight: 100%;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .TextContent-InnerContentCol {\r\n\tposition: relative;\r\n\theight: 100%;\r\n\twidth: 100%;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .TextContent-TitleRow {\r\n\tposition: absolute;\r\n    top: 0px;\r\n    right: 0px;\r\n    left: 0px;\r\n    height: 33px;\r\n\tcolor: #5f7e7e;\r\n\tbackground-color: #f2f2f2;\r\n\toverflow: hidden;\r\n\tborder-top: 1px solid #dddddd;\r\n\tborder-right: 1px solid #dddddd;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .TextHeaderDraft {\r\n\tbackground-color: #f2f2f2;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .TextHeaderPending {\r\n\tbackground-color: #eaf8fb;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .TextHeaderApproved {\r\n\tbackground-color: #f0f8ed;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .TextHeaderRejected {\r\n\tbackground-color: #f8edee;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .TextContent-TitleRow .TextContent-Title {\r\n\tfloat: left;\r\n\tpadding: 8px 12px;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .TextContent-TitleRow .TextContent-Title img {\r\n\tpadding-right: 5px;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .TextContent-TitleRow .TextContent-Action {\r\n\tpadding-top: 8px;\r\n\tpadding-bottom: 7px;\r\n\tpadding-left: 10px;\r\n\tpadding-right: 10px;\r\n\tborder-left: 1px solid #dddddd;\r\n\tfloat: right;\r\n\tcursor: pointer;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .TextContent-TitleRow .TextContent-Action i {\r\n\tfont-size: 16px;\r\n\theight: 16px;\r\n\twidth: 16px;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .TextContent-TitleRow .TextContent-Action:hover {\r\n\tbackground-color: #E5E5E5;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .Content-ContentRow {\r\n\tposition: absolute;\r\n\ttop: 33px;\r\n\tleft: 0px;\r\n\tright: 0px;\r\n\tbottom: 0px;\r\n\tborder: 1px solid #dddddd;\r\n\tborder-left: 0px;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .ContentArea {\r\n\tresize: none;\r\n\twidth: 100%;\r\n\theight:100%;\r\n\tpadding: 3px;\r\n\tborder: none;\r\n\toverflow-x: auto;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .ContentArea.NotEditable {\r\n\tbackground-color: #f2f2f2;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .TextComment {\r\n\tposition: absolute;\r\n\ttop: 120px;\r\n\tleft: 60px;\r\n\tcolor: #5f7e7e;\r\n\tborder: 1px solid #dddddd;\r\n\tborder-top: 0px;\r\n\toverflow: hidden;\r\n\tmax-height: 200px;\r\n\theight: auto;\r\n\tpadding: 2px;\r\n\toverflow-y: auto;\r\n\tbackground-color: white;\r\n\tz-index: 100000;\r\n\tright: 0px;\r\n\t-webkit-box-shadow: 0px 4px 10px 2px rgba(0,0,0,0.17);\r\n\t-moz-box-shadow: 0px 4px 10px 2px rgba(0,0,0,0.17);\r\n\tbox-shadow: 0px 4px 10px 2px rgba(0,0,0,0.17);\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .TextComment .TextComments {\r\n\twidth: 100%;\r\n\tmargin-top: 5px;\r\n\tfont-size: 11px;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .TextComment .Comment {\r\n\toverflow: hidden;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .TextComment .CommentAuthor {\r\n\tcolor: #5f7e7e;\r\n\tmax-width: 150px;\r\n\tvertical-align: top;\r\n\ttext-align: right;\r\n\tfont-weight: bold;\r\n\tpadding: 8px;\r\n\tpadding-right: 5px;\r\n\tpadding-top: 5px;\r\n\twhite-space: nowrap;\r\n\ttext-overflow: ellipsis;\r\n\toverflow: hidden;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .TextComment .CommentContent {\r\n\tcolor: #5f7e7e;\r\n\toverflow: hidden;\r\n\tpadding: 5px;\r\n\tfont-style: italic;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .TextComment .CommentText {\r\n\twhite-space: pre-wrap;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .TextComment .CommentContent .CommentDate {\r\n\tcolor: #D1D1D1;\r\n\tfont-size: 10px;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .TextComment .CommentField {\r\n\tborder: 1px solid #dddddd;\r\n\twidth: 100%;\r\n\theight: 40px;\r\n\tresize: vertical;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .TextComment .CommentFieldButton {\r\n\twidth: 100%;\r\n\ttext-align: right;\r\n\tfont-style: normal;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .MyApprovalStatus {\r\n\tpadding-top: 9px;\r\n\tfloat: left;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .MyApprovalStatus .MyStatusIconStack {\r\n\tposition: relative;\r\n\tdisplay: inline-block;\r\n\twidth: 24px;\r\n\theight: 24px;\r\n\tline-height: 24px;\r\n\tvertical-align: middle;\r\n\tmargin-top: -5px;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .MyStatusIconStack i {\r\n\tposition: absolute;\r\n\ttext-align: center;\r\n\tfont-size: 16px;\r\n\twidth: 100%;\r\n\tcursor: default;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .MyStatusApprovedIcon {\r\n\tpadding-top: 2px;\r\n\tpadding-left: 4px;\r\n\tcolor: #4CCC4C;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .MyStatusRejectedIcon {\r\n\tfont-size: 14px;\r\n\tpadding-top: 3px;\r\n\tpadding-left: 4px;\r\n\tcolor: #FF4C4C;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget .MyStatusPendingIcon {\r\n\tfont-size: 14px;\r\n\tpadding-top: 0px;\r\n\tpadding-left: 10px;\r\n\ttransform: rotate(45deg);\r\n\tdisplay: inline-block;\r\n\tmargin-top: -2px;\r\n\tcolor: #0600FF;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget-Restricted-Popover {\r\n\tfont-family: \'Open Sans\', \'Helvetica Neue\', Helvetica, \'hiragino kaku gothic pro\', \'ms pgothic\', Arial;\r\n\tfont-size: 12px;\r\n\tpadding: 4px 8px 4px 8px;\r\n\tcolor: grey;\r\n}\r\nbody > .TextContent-Restricted-Popover > div.content {\r\n\tmargin: 3px;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget.DocTableXView {\r\n\theight: 78px;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget.DocTableXView .TextContent-RightCol {\r\n\tcolor: #5f7e7e;\r\n\tbackground-color: #f2f2f2;\r\n\tborder: 1px solid #dddddd;\r\n\tborder-left: 0px;\r\n\twidth: 33px;\r\n\tfloat: right;\r\n\theight: 100%;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget.DocTableXView .TextContent-ActionDocX {\r\n\tfont-size: 16px;\r\n\ttext-align: center;\r\n\theight: 50%;\r\n\tpadding-top: 8px;\r\n\tcursor: pointer;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget.DocTableXView .TextContent-ActionBorder {\r\n\tborder-bottom: 1px solid #dddddd;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget.DocTableXView .TextContent-Action:hover {\r\n\tbackground-color: #E5E5E5;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget.DocTableXView .TextContent-ContentCol {\r\n\tright: 33px;\r\n\tleft: 52px;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget.DocTableXView .Content-ContentRow {\r\n\ttop: 0px;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget.DocTableXView .TextComment {\r\n\ttop: 78px;\r\n\tleft: 52px;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget.DocTableXView .TextContent-StatusLabel {\r\n\tpadding-top: 10px;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget.DocTableXView .TextContent-ApprovalCol {\r\n\twidth: 52px;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget.DocTableXView .TextContent-StatusIcon i {\r\n\tfont-size: 24px;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget.DocTableXView .ActionIcon-1ApprovalAction i {\r\n\tfont-size: 28px;\r\n\theight: 28px;\r\n\twidth: 28px;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget.DocTableXView .ActionIcon-2ApprovalActions i {\r\n\tfont-size: 24px;\r\n\theight: 24px;\r\n\twidth: 24px;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget.DocTableXView .ActionIcon-3ApprovalActions i {\r\n\tfont-size: 20px;\r\n\theight: 20px;\r\n\twidth: 20px;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget.Viewer {\r\n\tmin-height: 78px;\r\n\theight: 100%\r\n}\r\n.WcrPageContent .ContentStatementElementWidget.Viewer .TextContent-ContentCol {\r\n\tright: 0px;\r\n\tleft: 52px;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget.Viewer .Content-ContentRow {\r\n\ttop: 0px;\r\n\tborder-top: 0px;\r\n\tborder-bottom: 0px;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget.Viewer .TextContent-StatusLabel {\r\n\tpadding-top: 1px;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget.Viewer .TextContent-ApprovalCol {\r\n\twidth: 52px;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget.Viewer .TextContent-StatusIcon i {\r\n\tfont-size: 24px;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget.Viewer .ActionIcon-1ApprovalAction i {\r\n\tfont-size: 28px;\r\n\theight: 28px;\r\n\twidth: 28px;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget.Viewer .ActionIcon-2ApprovalActions i {\r\n\tfont-size: 24px;\r\n\theight: 24px;\r\n\twidth: 24px;\r\n}\r\n.WcrPageContent .ContentStatementElementWidget.Viewer .ActionIcon-3ApprovalActions i {\r\n\tfont-size: 20px;\r\n\theight: 20px;\r\n\twidth: 20px;\r\n}';});

define('packaging-content-management/text-content-element-widget/ContentStatementElementWidgetView',["underscore",
        "jquery",
        "utils/webi18n!TextContentStrings",
        "utils/ConvertUtils",
        "../ContentConstants",
        "component/modal/Modal",
        "../../component/tooltip/Tooltip",
        "./RichTextEditor",
        "utils/webcss!./ContentStatementElementWidgetView.css"],
	function (_,
			$,
			TextContentStrings,
			ConvertUtils,
			ContentConstants,
			Modal,
			Tooltip,
			RichTextEditor) {
	
		"use strict";

		/**
		 * Create the default ContentStatementElementWidgetView.
		 * 
		 * @param {Object} content - The content object from the ContentStatementElementWidget represented by this widget.
		 * @param {Boolean} isMaster - Flag which specifies whether the language is a Master or Translation
		 * @param {Array} comments - The array of Comment objects from the ContentStatementElementWidget represented by this widget.
		 * @param {String} state - The state of the ContentStatementElementWidget represented by this widget.
		 * @param {String} language - The language object of the ContentStatementElementWidget represented by this widget.
		 * @param {Boolean} canEdit - Whether or not the ContentStatementElementWidget represented by this widget can be edited by the current user.
		 * @param {Boolean} whyCanNotEdit - Why the ContentStatementElementWidget represented by this widget can't bed edited by the current user.
		 * @param {Boolean} canReview - Whether or not the ContentStatementElementWidget represented by this widget can be reviewed by the current user.
		 * @param {Boolean} whyCanNotReview - Why the ContentStatementElementWidget represented by this widget can't be reviewed by the current user.
		 * @param {Boolean} isContentManagerView - Specifies whether the current user is a Content_Manager
		 * @param {String} myApprovalStatus - Approval status of TextContent element for a Approver
		 * @param {Object} [viewOptions] - An object containing some specific variables indicating how the widget will be rendered.
		 * @param {String} markedApprovalState - Approval action taken if any, Current status otherwise
		 * @param {String} type - text type as "richtext" or "simple".
		 */
		var ContentStatementElementWidgetView = function (content, isMaster, comments, state, languageCode, canEdit, whyCanNotEdit, canReview, whyCanNotReview, isContentManagerView, myApprovalStatus, viewOptions, markedApprovalState, type) {
			
			this._content = content;
			if (!this._content.content) {
				this._content.content = "";
			}
			this._isMaster = isMaster;
			this._comments = comments;
			this._state = state;
			this._langaugeCode = languageCode;
			this._canEdit = canEdit;
			this._whyCanNotEdit = whyCanNotEdit;
			this._canReview = canReview;
			this._whyCanNotReview = whyCanNotReview;
			this._isContentManagerView = isContentManagerView;
			this._myApprovalStatus = myApprovalStatus;
			this._type = type;
			
			// store the updated changes for approval
			this._markedApprovalState = markedApprovalState;
			
			// Let the viewOptions override the default, extend one these default values.
			this._viewOptions = {
				// Some configuration might override this variable to disable the editing capabilities of this widget.
				editable: true,
				// Some configuration might override this variable to disable the review capabilities of this widget.
				reviewable: true 
			};
			$.extend(this._viewOptions, viewOptions);
			
			// Create the callback arrays
			this._onApproveCallbacks = [];
			this._onRejectCallbacks = [];
			this._onStartApprovalCallbacks = [];
			this._onRevertToDraftCallbacks = [];
			this._onDeleteCallbacks = [];
			this._onCommentCallbacks = [];
			this._onRevertApprovalCallbacks = [];
			this._onApprovalInfoCallbacks = [];
			this._onMainContentCallbacks = [];
			
			// Create the common HTML elements for the view.
			this._approveButton = null;
			this._rejectButton = null;
			this._startApprovalButton = null;
			this._revertToDraftButton = null;
			this._deleteButton = null;
			this._richTextEditor = null;
			this._plainTextEditor = null;
			this._commentTextArea = null;
			this._commentButton = null;
			this._approvalInfoButton = null;
			this._revertApprovalButton = null;
			
			// The height of the content.
			this._contentHeight = 85;
			// Wether to show popovers on why the user does not have certain rights. 
			this._showTooltips = true;
			
			// Create the wrapping div of this widget.
			this._wrapper = $("<div>").addClass("ContentStatementElementWidget");
			
			if(this._viewOptions.widgetview === 'default') {
				// Start constructing the default view
				this._constructView();
			} else {
				return;
			}
			
		};
		
		/**
		 * Construct the HTML elements of this view.
		 */
		ContentStatementElementWidgetView.prototype._constructView = function () {
			this._wrapper.empty();
			
			this._initializeDOMElements();
			
			// create widget elements and append to the wrapper
			this._wrapper.append(
					this._constructApprovalDiv(),
					$('<div>').addClass("TextContent-ContentCol").append(
							$('<div>').addClass("TextContent-InnerContentCol").append(
									this._constructHeaderDiv(),
									this._constructTextDiv()
							)
					),
					this._constructCommentsDiv()
			);
		};
		
		/**
		 * Provides a dialog for Default view to show approval info
		 * @param approvalInfoContent {JQuery} - Approval information as jquery elements
		 */
		ContentStatementElementWidgetView.prototype.showApprovalInfoDialog = function (approvalInfoContent) {
			// Condition check to avoid creating multiple approval info modal
			if (!this._approvalInfoModal) {
				this._approvalInfoModal = new Modal(approvalInfoContent, {"customClass" : "ApprovalOverviewWidgetModal"});

				this._approvalInfoModal.setTitle(TextContentStrings.t("PCM_APPROVAL_INFO_POPUP_TITLE"));
			} else {
				this._approvalInfoModal.setContent(approvalInfoContent);
			}
			this._approvalInfoModal.open();
		};
		
		/**
		 * Method to update the widget and repaint
		 * 
		 * @param text
		 * @param isMaster
		 * @param comments
		 * @param state
		 * @param languageCode
		 * @param canEdit
		 * @param whyCanNotEdit
		 * @param canReview
		 * @param whyCanNotReview
		 * @param isContentManagerView
		 * @param myApprovalStatus
		 * @param markedApprovalState
		 * @param type
		 */
		ContentStatementElementWidgetView.prototype.updateWidgetView = function(content, isMaster, comments, state, languageCode, canEdit, whyCanNotEdit, canReview, whyCanNotReview, isContentManagerView, myApprovalStatus, markedApprovalState, type) {
			this._content = content;
			this._isMaster = isMaster;
			this._comments = comments;
			this._state = state;
			this._langaugeCode = languageCode;
			this._canEdit = canEdit;
			this._whyCanNotEdit = whyCanNotEdit;
			this._canReview = canReview;
			this._whyCanNotReview = whyCanNotReview;
			this._isContentManagerView = isContentManagerView;
			this._myApprovalStatus = myApprovalStatus;
			this._markedApprovalState = markedApprovalState;
			this._type = type;
			
			this._constructView();
		};
		
		/**
		 * Initialize the DOM elements to repaint the widget
		 */
		ContentStatementElementWidgetView.prototype._initializeDOMElements = function() {
			// create the DOM elements
			this._approveButton = $('<div>').attr({"name": "approveButton"});
			this._rejectButton = $('<div>').attr({"name": "rejectButton"});
			this._startApprovalButton = $('<div>').attr({"name": "startApprovalButton"});
			this._revertToDraftButton = $('<div>').attr({"name": "revertToDraftButton"});
			this._deleteButton = $('<div>').attr({"name": "deleteButton"});

			// Create the editors based on text type
			if (ContentConstants.TextType.RICHTEXT === this._type) {
				// Create a rich text editor.
				this._richTextEditor = new RichTextEditor(this._content.content, this._contentHeight);
				this._richTextEditor.getJQElement().addClass("ContentArea");
				
				// Register content change event in rich text editor
				this._richTextEditor.onChange(_.bind(function () {
					var richText = this._richTextEditor.getContent();
					this._executeContentChange(richText);
				}, this));
			} else {
				// Create a plain text editor (e.g. a normal "textarea").
				this._plainTextEditor = $('<textarea>').text(this._content.content);
				this._plainTextEditor.addClass("ContentArea");
				// Register content change event in simple text editor
				this._plainTextEditor.on("input", _.bind(function () {
					var plainText = (this._plainTextEditor.val());
					this._executeContentChange(plainText);
				}, this));
			}
			
			this._commentTextArea = $('<textarea>').attr({"name": "addCommentTextArea"});
			this._commentButton = $('<div>').attr({"name": "commentButton"});
			this._approvalInfoButton = $('<div>').attr({"name": "approvalInfoButton"});
			this._revertApprovalButton = $('<div>').attr({"name": "revertApprovalButton"});
			
			// Bind events to execute callbacks for each action
			this._approveEventHandler = _.bind(function(event) {
				_.each(this._onApproveCallbacks, _.bind(function(callback) {
					callback();
				}, this));
			}, this);
			this._approveButton.click(this._approveEventHandler);
			
			this._rejectEventHandler = _.bind(function(event) {
				_.each(this._onRejectCallbacks, _.bind(function(callback) {
					callback();
				}, this));
			}, this);
			this._rejectButton.click(this._rejectEventHandler);
			
			this._startApprovalEventHandler = _.bind(function(event) {
				_.each(this._onStartApprovalCallbacks, _.bind(function(callback) {
					callback();
				}, this));
			}, this);
			this._startApprovalButton.click(this._startApprovalEventHandler);
			
			this._revertToDraftEventHandler = _.bind(function(event) {
				_.each(this._onRevertToDraftCallbacks, _.bind(function(callback) {
					callback();
				}, this));
			}, this);
			this._revertToDraftButton.click(this._revertToDraftEventHandler);
			
			this._deleteButton.click(_.bind(function(event) {
				_.each(this._onDeleteCallbacks, _.bind(function(callback) {
					callback();
				}, this));
			}, this));
			
			this._commentTextArea.on("input propertychange paste", _.bind(function(event) {
				var commentText = $.trim($(event.target).val());
				_.each(this._onCommentCallbacks, _.bind(function(commentText, callback) {
					callback(commentText);
				}, this, commentText));
			}, this));
			
			this._revertApprovalEventHandler = _.bind(function(event) {
				_.each(this._onRevertApprovalCallbacks, _.bind(function(callback) {
					callback();
				}, this));
			}, this);
			this._revertApprovalButton.click(this._revertApprovalEventHandler);
		};
		
		/**
		 * Method is to update content if its actually changed.
		 * @param currentContent {String} - content from content area
		 */
		ContentStatementElementWidgetView.prototype._executeContentChange = function(currentContent) {
			// Avoid update if content is not modified
			if (currentContent !== this._content.content) {
				_.each(this._onMainContentCallbacks, _.bind(function(currentContent, callback) {
					callback(currentContent);
				}, this, currentContent));
				
				//Ok what are we doing this? Current status is Rejected and user starts typing in text area, automatically the status of the
				//Text Element should change to Draft
				//Important Note: User doesn't get revert option in this case
				if(this._state === ContentConstants.State.REJECTED){
					this._markedApprovalState = ContentConstants.State.DRAFT;
					$("div.TextContent-ApprovalCol", this._wrapper).empty().append(this._constructElementStatus());
				}
			}
		};
		
		/**
		 * Set the language indicator of the view.
		 * 
		 * @param language - language object {langCode, langDesc, langFlag}
		 */
		ContentStatementElementWidgetView.prototype.setLanguage = function (language) {
			var languageDescDiv = this._wrapper.find(".TextContent-TitleRow div[name='languageDescDiv']");
			
			languageDescDiv.addClass("TextContent-Title");
			
			var languageLabel = null;
			if(language && language.langDesc) {
				languageLabel = language.langDesc;
			} else if(language && language.langCode) {
				languageLabel = language.langCode;
			} else {
				languageLabel = this._langaugeCode;
			}
			
			if(this._isMaster) {
				languageDescDiv.append($('<b>').text(languageLabel));
			} else {
				languageDescDiv.append($('<span>').text(languageLabel));
			}
			
			if(language && language.langFlag) {
				languageDescDiv.prepend($('<img>').attr({"src": "images/country_flags/" + language.langFlag}));
			}
			
		};
		
		/**
		 * Create delete button and append to the header
		 * 
		 * @param revertDeleteHandler - click hander for revert button when delete action is performed
		 */
		ContentStatementElementWidgetView.prototype.createDeleteButton = function(revertDeleteHandler) {
			this._deleteButton.addClass("TextContent-Action").attr({"title": TextContentStrings.t("PCM_DELETE_LANGUAGE")}).append($('<i>').addClass("icon icon-WCR_Trash"));
			
			this._changeToDeleteState = function() {
				var deleteStatus = $('<div>').attr({"name": "TextContent-StatusDiv"}).css({'opacity': '1'});
				
				// Display the language status
				deleteStatus.addClass("TextElementStatus_Delete").append(
						$('<div>').addClass("TextContent-ApprovalStatus").append(
								$('<div>').addClass("TextContent-StatusIcon").append($('<i>').addClass("icon icon-WCR_Trash").addClass("DeleteIconAndLabel")),
								$('<div>').addClass("TextContent-StatusLabel DeleteIconAndLabel").text(TextContentStrings.t("PCM_DELETE_LANGUAGE"))
						)
				);
				
				return deleteStatus;
			};
			
			
			this._deleteButton.click(_.bind(function() {
				this._deleteButton.hide();
				
				this._createApprovalDivHandlers();
				$("div.TextContent-ApprovalCol", this._wrapper).empty().append(this._changeToDeleteState()).append(this._constructRevertApprovalAction());
				
				// unbind all events on revert approval button and bind necessary click event
				this._revertApprovalButton.unbind();
				this._revertApprovalButton.click(_.bind(function() {
					this._deleteButton.show();
					$("div.TextContent-ApprovalCol", this._wrapper).empty().append(this._constructElementStatus());
					this._revertApprovalButton = $('<div>').click(this._revertApprovalEventHandler);
				}, this));
				
				// bind the revert approval action handler
				this._revertApprovalButton.click(revertDeleteHandler);
				
			}, this));
			
			// Append the delete button to the header
			this._deleteButton.insertBefore(this._wrapper.find(".TextContent-TitleRow:first div:last"));
		};
		
		/**
		 * Construct the approval column of the widget
		 * @returns approvalDiv - approval column container
		 */
		ContentStatementElementWidgetView.prototype._constructApprovalDiv = function () {
			// Container for Approval column
			var approvalDiv = $('<div>').addClass("TextContent-ApprovalCol");
			
			// add element status
			approvalDiv.append(this._constructElementStatus());
			
			if((this._canEdit && this._state === ContentConstants.State.DRAFT) || 
					((this._canReview || this._canEdit) && this._state === ContentConstants.State.READY_FOR_APPROVAL) || 
					(this._canEdit && this._state === ContentConstants.State.REJECTED)) {
				// create the hover and mouseleave listeners for the actions div
				this._createApprovalDivHandlers();
				
				// If no approval action is taken add approval buttons, add revert button otherwise
				if(this._state === this._markedApprovalState) {
					approvalDiv.append(this._constructApprovalActions());
				} else {
					approvalDiv.append(this._constructRevertApprovalAction());
				}
			} else {
				$("div[name='TextContent-StatusDiv']", approvalDiv).css({'cursor': 'default'});
			}
			
			return approvalDiv;
		};
		
		/**
		 * Creates hover and mouseleave event listeners for the actions
		 */
		ContentStatementElementWidgetView.prototype._createApprovalDivHandlers = function() {
			// create a handler when mouseover element status to view approval actions
			if(!this._actionsDivHandlerIn) {
				this._actionsDivHandlerIn = _.bind(function () {
					this._wrapper.find("div[name='TextContent-StatusDiv']").stop();
					this._wrapper.find("div[name='TextContent-ActionsDiv']").stop();
					this._wrapper.find("div[name='TextContent-StatusDiv']").animate({opacity: 0}, 400);
					this._wrapper.find("div[name='TextContent-ActionsDiv']").animate({opacity: 1}, 400);
				}, this);
			}
			// create a handler when mouseout element status to hide approval actions
			if(!this._actionsDivHandlerOut) {
				this._actionsDivHandlerOut = _.bind(function () {
					this._wrapper.find("div[name='TextContent-StatusDiv']").stop();
					this._wrapper.find("div[name='TextContent-ActionsDiv']").stop();
					this._wrapper.find("div[name='TextContent-ActionsDiv']").animate({opacity: 0}, 400);
					this._wrapper.find("div[name='TextContent-StatusDiv']").animate({opacity: 1}, 400);	
				}, this);
			}
		};
		
		/**
		 * Create TextContent status
		 * 
		 * @returns elementStatusDiv - jQuery element for TextContent status
		 */
		ContentStatementElementWidgetView.prototype._constructElementStatus = function () {
			var elementStatusDiv = $('<div>').attr({"name": "TextContent-StatusDiv"}).css({'opacity': '1'});
			
			if(!this._canEdit && !this._canReview) {
				elementStatusDiv.css({'cursor': 'default'});
			} else {
				elementStatusDiv.css({'cursor': 'pointer'});
				// On mouseleave create the approval actions div after a approval action is taken
				// unbind this event after mouseleave!
				
				/* 
				 * This event shouldn't trigger when the widget is created (creating issue in Firefox browser)!
				 * During widget creation the approval actions are readily available 
				 * But when an approval action is taken the user should move the mouse out of status and move in to get the revert approval button (should be same conversely)
				 */
				
				// TODO: Need a better fix. see jira bug id GRCCOMMONS-2749 (fixing it now by clearing the icons)
				
				elementStatusDiv.mouseleave(_.bind(function() {
					if(this._markedApprovalState === this._state) {
						$("div.TextContent-ApprovalCol", this._wrapper).append(this._constructApprovalActions());
					} else if(this._state !== ContentConstants.State.REJECTED){
						//Dont construct Revert option once the state is changed to DRAFT from rejected mode
						$("div.TextContent-ApprovalCol", this._wrapper).append(this._constructRevertApprovalAction());
					}
					this._wrapper.find('div[name="TextContent-StatusDiv"]').unbind();
				}, this));
			}
			
			var elementStatusIconClass = null;
			var elementStatusClass = null;
			var elementStatusLabelText = null;
			var elementStatusLabelClass = null;
			if(this._state === this._markedApprovalState) {
				switch(this._state) {
				case ContentConstants.State.DRAFT:
					elementStatusIconClass = "icon icon-WCR_Text-Status-Draft ElementStatusIconAndLabel";
					elementStatusClass = "TextContentElement-Status ElementStatusDraft";
					elementStatusLabelClass = "ElementStatusIconAndLabel";
					break;
				case ContentConstants.State.READY_FOR_APPROVAL:
					if(this._canReview && this._myApprovalStatus === ContentConstants.State.READY_FOR_APPROVAL) {
						elementStatusIconClass = "icon icon-WCR_Text-Status-Pending-You ElementStatusIconAndLabel";
						elementStatusLabelClass = "ElementStatusIconAndLabel";
					} else if(this._myApprovalStatus === ContentConstants.State.APPROVED) {
						elementStatusIconClass = "icon icon-WCR_Text-Status-Pending-YouApproved ElementStatusIconAndLabel";
						elementStatusLabelClass = "ElementStatusIconAndLabel";
					} else if(this._myApprovalStatus === ContentConstants.State.REJECTED) {
						elementStatusIconClass = "icon icon-WCR_Text-Status-Pending-YouRejected ElementStatusIconAndLabel";
						elementStatusLabelClass = "ElementStatusIconAndLabel";
					} else {
						elementStatusIconClass = "icon icon-WCR_Text-Status-Pending ElementStatusIconAndLabel";
						elementStatusLabelClass = "ElementStatusIconAndLabel";
					}
					elementStatusClass = "TextContentElement-Status ElementStatusPending";
					break;
				case ContentConstants.State.APPROVED:
					elementStatusIconClass = "icon icon-WCR_Text-Status-Approved ElementStatusIconAndLabel";
					elementStatusClass = "TextContentElement-Status ElementStatusApproved";
					elementStatusLabelClass = "ElementStatusIconAndLabel";
					break;
				case ContentConstants.State.REJECTED:
					elementStatusIconClass = "icon icon-WCR_Text-Status-Rejected ElementStatusIconAndLabel";
					elementStatusClass = "TextContentElement-Status ElementStatusRejected";
					elementStatusLabelClass = "ElementStatusIconAndLabel";
					break;
				default :
					elementStatusIconClass = "icon icon-WCR_Text-Status-NA ElementStatusIconAndLabel";
					elementStatusClass = "TextContentElement-Status";
					elementStatusLabelClass = "ElementStatusIconAndLabel";
					break;	
				}
				
				elementStatusLabelText = this._getLocalizedContentState(this._state, this._myApprovalStatus);
				
			} else {
				switch(this._markedApprovalState) {
				case ContentConstants.State.DRAFT:
					elementStatusIconClass = "icon icon-WCR_Text-Action-Draft RevertTextToDraftIconAndLabel";
					elementStatusClass = "TextContentElement-Status RevertTextToDraft";
					elementStatusLabelClass = "RevertTextToDraftIconAndLabel";
					break;
				case ContentConstants.State.READY_FOR_APPROVAL:
					elementStatusIconClass = "icon icon-WCR_Text-Action-Start StartTextApprovalIconAndLabel";
					elementStatusClass = "TextContentElement-Status StartTextApproval";
					elementStatusLabelClass = "StartTextApprovalIconAndLabel";
					break;
				case ContentConstants.State.APPROVED:
					elementStatusIconClass = "icon icon-WCR_Text-Action-Approve ApproveTextIconAndLabel";
					elementStatusClass = "TextContentElement-Status ApproveText";
					elementStatusLabelClass = "ApproveTextIconAndLabel";
					break;
				case ContentConstants.State.REJECTED:
					elementStatusIconClass = "icon icon-WCR_Text-Action-Reject RejectTextIconAndLabel";
					elementStatusClass = "TextContentElement-Status RejectText";
					elementStatusLabelClass = "RejectTextIconAndLabel";
					break;
				default :
					break;
				}
				
				elementStatusLabelText = this._getLocalizedContentAction(this._markedApprovalState);
			}
			
			
			// Display the language status
			elementStatusDiv.removeClass().addClass(elementStatusClass).append(
					$('<div>').addClass("TextContent-ApprovalStatus").append(
							$('<div>').addClass("TextContent-StatusIcon").append($('<i>').addClass(elementStatusIconClass)),
							$('<div>').addClass("TextContent-StatusLabel").addClass(elementStatusLabelClass).text(elementStatusLabelText)
					)
			);
			
			return elementStatusDiv;
		};
		
		/**
		 * Create Approval action buttons
		 * 
		 * @returns approvalActionsDiv - jQuery element for approval action buttons
		 */
		ContentStatementElementWidgetView.prototype._constructApprovalActions = function () {
			var approvalActionsDiv = $('<div>').attr({"name": "TextContent-ActionsDiv"}).addClass("TextContent-Approval-NextStates").css({'opacity': '0'});
			
			// Create StartApproval button
			this._startApprovalButton.empty().addClass("TextContent-ApprovalAction StartTextApproval").append($('<div>').addClass("ApprovalAction-InnerDiv").append(
					$('<div>').addClass("TextContent-ApprovalSymbol").append($('<i>').addClass("icon icon-WCR_Text-Action-Start").addClass("StartTextApprovalIconAndLabel")),
					$('<div>').addClass("TextContent-ApprovalLabel StartTextApprovalIconAndLabel").text(this._getLocalizedContentAction(ContentConstants.State.READY_FOR_APPROVAL))
					)
			);
			
			// Create RevertToDraft button
			this._revertToDraftButton.empty().addClass("TextContent-ApprovalAction RevertTextToDraft").append($('<div>').addClass("ApprovalAction-InnerDiv").append(
					$('<div>').addClass("TextContent-ApprovalSymbol").append($('<i>').addClass("icon icon-WCR_Text-Action-Draft").addClass("RevertTextToDraftIconAndLabel")),
					$('<div>').addClass("TextContent-ApprovalLabel RevertTextToDraftIconAndLabel").text(this._getLocalizedContentAction(ContentConstants.State.DRAFT))
					)
			);
			
			// Create Approve button
			this._approveButton.empty().addClass("TextContent-ApprovalAction ApproveText").append($('<div>').addClass("ApprovalAction-InnerDiv").append(
					$('<div>').addClass("TextContent-ApprovalSymbol").append($('<i>').addClass("icon icon-WCR_Text-Action-Approve").addClass("ApproveTextIconAndLabel")),
					$('<div>').addClass("TextContent-ApprovalLabel ApproveTextIconAndLabel").text(this._getLocalizedContentAction(ContentConstants.State.APPROVED))
					)
			);
			
			// Create Reject button
			this._rejectButton.empty().addClass("TextContent-ApprovalAction RejectText").append($('<div>').addClass("ApprovalAction-InnerDiv").append(
					$('<div>').addClass("TextContent-ApprovalSymbol").append($('<i>').addClass("icon icon-WCR_Text-Action-Reject").addClass("RejectTextIconAndLabel")),
					$('<div>').addClass("TextContent-ApprovalLabel RejectTextIconAndLabel").text(this._getLocalizedContentAction(ContentConstants.State.REJECTED))
					)
			);
			
			
			// Bind the click events to each approval action button
			this._actionsCallback = _.bind(function() {
				this._revertApprovalButton = $('<div>').click(this._revertApprovalEventHandler);
				$("div.TextContent-ApprovalCol", this._wrapper).empty().append(this._constructElementStatus());
			}, this);
			
			this._startApprovalButton.click(_.bind(function(event) {
				this._markedApprovalState = ContentConstants.State.READY_FOR_APPROVAL;
				this._actionsCallback();
			}, this));
			
			this._revertToDraftButton.click(_.bind(function(event) {
				this._markedApprovalState = ContentConstants.State.DRAFT;
				this._actionsCallback();
			}, this));
			
			this._approveButton.click(_.bind(function(event) {
				this._markedApprovalState = ContentConstants.State.APPROVED;
				this._actionsCallback();
			}, this));
			
			this._rejectButton.click(_.bind(function(event) {
				this._markedApprovalState = ContentConstants.State.REJECTED;
				this._actionsCallback();
			}, this));
			
			
			// If the language is ready for approval, show Approve, Reject and RevertToDraft buttons based on the permissions
			if(this._state === ContentConstants.State.READY_FOR_APPROVAL) {
				// If User is have both EDITOR and APPROVER permissions and TextContent is in Pending-Signoff - add Draft, Approve and Reject buttons
				if(this._canReview && this._canEdit) {
					approvalActionsDiv.append(this._approveButton, this._rejectButton, this._revertToDraftButton);
					approvalActionsDiv.find(".TextContent-ApprovalAction").addClass("TextContent-3ApprovalActions");
					approvalActionsDiv.find(".TextContent-ApprovalSymbol").addClass("ActionIcon-3ApprovalActions");
					approvalActionsDiv.find(".TextContent-ApprovalLabel").addClass("ActionLabel-3ApprovalActions").hide();
				}
				// If User has APPROVER permission - add Approve and Reject buttons
				else if(this._canReview) {
					approvalActionsDiv.append(this._approveButton, this._rejectButton);
					approvalActionsDiv.find(".TextContent-ApprovalAction").addClass("TextContent-2ApprovalActions");
					approvalActionsDiv.find(".TextContent-ApprovalSymbol").addClass("ActionIcon-2ApprovalActions");
					approvalActionsDiv.find(".TextContent-ApprovalLabel").addClass("ActionLabel-2ApprovalActions").hide();
				}
				// If the User has EDITOR permission - add the RevertToDraft button
				else if (this._canEdit) {
					approvalActionsDiv.append(this._revertToDraftButton);
					approvalActionsDiv.find(".TextContent-ApprovalAction").addClass("TextContent-1ApprovalAction");
					approvalActionsDiv.find(".TextContent-ApprovalSymbol").addClass("ActionIcon-1ApprovalAction");
					approvalActionsDiv.find(".TextContent-ApprovalLabel").addClass("ActionLabel-1ApprovalAction").hide();
				}
			}
			else if (this._state === ContentConstants.State.DRAFT) {
				// If the language is in Draft and user has both EDITOR and APPROVER permissions, then add Start Approval & Approve button (User can skip start approval)
				if(this._canEdit && this._canReview) {
					approvalActionsDiv.append(this._approveButton, this._startApprovalButton);
					approvalActionsDiv.find(".TextContent-ApprovalAction").addClass("TextContent-2ApprovalActions");
					approvalActionsDiv.find(".TextContent-ApprovalSymbol").addClass("ActionIcon-2ApprovalActions");
					approvalActionsDiv.find(".TextContent-ApprovalLabel").addClass("ActionLabel-2ApprovalActions").hide();
				} 
				// If the language is in Draft and user has EDITOR permission, then add StartApproval button
				else if(this._canEdit) {
					approvalActionsDiv.append(this._startApprovalButton);
					approvalActionsDiv.find(".TextContent-ApprovalAction").addClass("TextContent-1ApprovalAction");
					approvalActionsDiv.find(".TextContent-ApprovalSymbol").addClass("ActionIcon-1ApprovalAction");
					approvalActionsDiv.find(".TextContent-ApprovalLabel").addClass("ActionLabel-1ApprovalAction").hide();
				}
			}
			// If the language is rejected, add a explict move to draft button
			else if(this._canEdit && this._state === ContentConstants.State.REJECTED) {
				approvalActionsDiv.append(this._revertToDraftButton);
				approvalActionsDiv.find(".TextContent-ApprovalAction").addClass("TextContent-1ApprovalAction");
				approvalActionsDiv.find(".TextContent-ApprovalSymbol").addClass("ActionIcon-1ApprovalAction");
				approvalActionsDiv.find(".TextContent-ApprovalLabel").addClass("ActionLabel-1ApprovalAction");
			}
			
			// add the hover handler for approval actions
			approvalActionsDiv.hover(this._actionsDivHandlerIn, this._actionsDivHandlerOut);
			
			return approvalActionsDiv;
		};
		
		/**
		 * Create the Revert Approval button
		 * @returns approvalActionsDiv - jQuery element for revert approval action button
		 */
		ContentStatementElementWidgetView.prototype._constructRevertApprovalAction = function () {
			var approvalActionsDiv = $('<div>').attr({"name": "TextContent-ActionsDiv"}).addClass("TextContent-Approval-NextStates").css({'opacity': '0'});
			
			// Create revert approval action button
			this._revertApprovalButton.empty().removeClass().addClass("RevertApprovalAction TextContent-1ApprovalAction").append($('<div>').addClass("ApprovalAction-InnerDiv").append(
					$('<div>').addClass("TextContent-ApprovalSymbol ActionIcon-1ApprovalAction").append($('<i>').addClass("icon icon-WCR_Revert").addClass("RevertApprovalActionIconAndLabel")),
					$('<div>').addClass("TextContent-ApprovalLabel ActionLabel-1ApprovalAction RevertApprovalActionIconAndLabel").text(TextContentStrings.t("PCM_REVERT"))
					)
			);
			this._revertApprovalButton.click(_.bind(function(event) {
				this._markedApprovalState = this._state;
				this._startApprovalButton = $('<div>').click(this._startApprovalEventHandler);
				this._revertToDraftButton = $('<div>').click(this._revertToDraftEventHandler);
				this._approveButton = $('<div>').click(this._approveEventHandler);
				this._rejectButton = $('<div>').click(this._rejectEventHandler);
				
				$("div.TextContent-ApprovalCol", this._wrapper).empty().append(this._constructElementStatus());
			}, this));
			
			approvalActionsDiv.append(this._revertApprovalButton);
			
			// add the hover handler for approval actions
			approvalActionsDiv.hover(this._actionsDivHandlerIn, this._actionsDivHandlerOut);
			
			return approvalActionsDiv;
		};
		
		/**
		 * Construct the header of the widget
		 * 
		 * @returns headerDiv - jQuery element for Language header and other actions
		 */
		ContentStatementElementWidgetView.prototype._constructHeaderDiv = function () {
			
			var headerClass = null;
			if(ContentConstants.State.DRAFT === this._state) {
				headerClass = "TextContent-TitleRow TextHeaderDraft";
			} else if(ContentConstants.State.READY_FOR_APPROVAL === this._state) {
				headerClass = "TextContent-TitleRow TextHeaderPending";
			} else if(ContentConstants.State.APPROVED === this._state) {
				headerClass = "TextContent-TitleRow TextHeaderApproved";
			} else if(ContentConstants.State.REJECTED === this._state) {
				headerClass = "TextContent-TitleRow TextHeaderRejected";
			} else {
				headerClass = "TextContent-TitleRow";
			}
			
			// create the header div
			var headerDiv = $('<div>').addClass(headerClass);
			
			// Add language description div to header
			headerDiv.append($('<div>').attr({'name': 'languageDescDiv'}));
			
			
			if(this._state === ContentConstants.State.APPROVED || this._state === ContentConstants.State.REJECTED || this._state === ContentConstants.State.READY_FOR_APPROVAL || this._state === ContentConstants.State.DRAFT) {
				// create and append comment button to header
				this._createCommentButton();
				headerDiv.append(this._commentButton);
				
				// create and append approval info button to header
				this._createApprovalInfoButton();
				headerDiv.append(this._approvalInfoButton);
			}
			
			headerDiv.append($('<div>').css({'clear': 'both'}));
			
			return headerDiv;
		};
		
		/**
		 * Create comment button
		 */
		ContentStatementElementWidgetView.prototype._createCommentButton = function() {
			// Add the Comment button based on the permissions
			var commentIcon = null;
			if(this._comments.length > 0) {
				commentIcon = "icon icon-WCR_Text-Comment-full";
			} else {
				commentIcon = "icon icon-WCR_Text-Comment-outline";
			}
			
			this._commentButton.attr({"title": TextContentStrings.t("PCM_COMMENT")}).addClass("ToggleComment TextContent-Action").append($('<i>').addClass(commentIcon));
			
			if((this._canEdit || this._canReview || this._isContentManagerView) || (this._comments.length > 0)) {
				// click event handler for comment button
				this._commentButton.click(_.bind(function(event) {
					if(this._commentsDiv.hasClass("commentshidden")) {
						this._showCommentSection();
					} else {
						this._hideCommentSection();
					}
				}, this));
				
				// hide the comments when clicked anywhere on the page except the comment button
				$(document).mouseup(_.bind(function (e) {
					if (!this._commentsDiv.is(e.target) 
							&& this._commentsDiv.has(e.target).length === 0 && !this._commentButton.is(e.target) && !this._commentButton.find("i:first").is(e.target)) {
						// Check if we should hide the comments section.
						if(!this._commentsDiv.hasClass("commentshidden")) {
							this._hideCommentSection();
						}
					}
					e.stopPropagation();
				}, this));
			} else {
				this._commentButton.prop({'disabled': 'disabled'}).css({'cursor': 'not-allowed'});
			}
		};
		
		/**
		 * Show the comments section
		 */
		ContentStatementElementWidgetView.prototype._showCommentSection = function() {
			this._commentsDiv.removeClass("commentshidden").slideDown(200);
		};

		/**
		 * Hide the comments section
		 */
		ContentStatementElementWidgetView.prototype._hideCommentSection = function() {
			this._commentsDiv.addClass("commentshidden").slideUp(200);
		};
		
		/**
		 * Create Approval Info button
		 */
		ContentStatementElementWidgetView.prototype._createApprovalInfoButton = function() {
			this._approvalInfoButton.addClass("TextContent-Action").attr({"title": TextContentStrings.t("PCM_APPROVAL_INFO")}).append($('<i>').addClass("icon icon-WCR_Info"));
			
			// What needs to happen when the approval info button is clicked.
			this._approvalInfoButton.on("click", function (){
				// Execute all the callbacks ...
				for (var i=0; i < this._onApprovalInfoCallbacks.length; i++) {
					this._onApprovalInfoCallbacks[i]();
				}
			}.bind(this));
			
		};
		
		/**
		 * Construct the TextContent area
		 * @returns contentDiv
		 */
		ContentStatementElementWidgetView.prototype._constructTextDiv = function () {
			
			// Create the div that will be holding the content.
			this._contentDiv = $('<div>').addClass("Content-ContentRow").attr({"name": "Content-ContentAreaDiv"});
			
			if (ContentConstants.TextType.RICHTEXT === this._type) {
				// Add the rich text area to the content div.
				this._contentDiv.append(this._richTextEditor.getJQElement());
			} else {
				// Add the plain text area to the content div.
				this._contentDiv.append(this._plainTextEditor);
			}
			
			// Reset the "editability" tooltip.
			this._nonEditableTooltip = undefined;
			this.applyEditPermission();

			return this._contentDiv;
			
		};
		
		/**
		 * Apply edit permission if allowed or provide tool tip message on disabled field.
		 */
		ContentStatementElementWidgetView.prototype.applyEditPermission = function () {
			// We can only edit when you have the edit permission and the state is "DRAFT" or "REJECTED".
			if (!(this._canEdit && (this._state === ContentConstants.State.DRAFT || this._state === ContentConstants.State.REJECTED))) {
				// Disable content editing.
				if (ContentConstants.TextType.RICHTEXT === this._type) {
					this._richTextEditor.disable();
				} else {
					this._plainTextEditor.prop("readonly", "readonly");
					this._plainTextEditor.addClass("NotEditable");
				}

				// If we can't edit because we lack permissions, say why ...
				var msgTag = undefined;
				if (!this._canEdit) {
					// Get the message tag.
					msgTag = this._whyCanNotEdit;
					// If the state is "EMPTY", make sure the "NO_PCM_PERM_NOT_ADDED" message is shown.
					if (this._state === ContentConstants.State.EMPTY) {
						msgTag = "NO_PCM_PERM_NOT_ADDED";
					}
				} else if (this._state === ContentConstants.State.READY_FOR_APPROVAL) {
					msgTag = "NO_PCM_PERM_EDIT_READY_FOR_APPROVAL";
				} else if (this._canEdit && this._state === ContentConstants.State.EMPTY) {
					msgTag = "NO_PCM_PERM_NOT_ADDED";
				}
				
				// If we have a reason, display it. 
				if (msgTag && this._showTooltips) {
					if (!this._nonEditableTooltip ) {
						// Create the tooltip
						this._nonEditableTooltip = new Tooltip(undefined, this._contentDiv, this._wrapper, $("body"), false, TextContentStrings.t(msgTag), undefined, undefined, {color: "#EBEBE4", cssClass : "TextContent-Restricted-Popover", restrictBorder : 8});
					} else {
						this._nonEditableTooltip.setTooltipContent(TextContentStrings.t(msgTag));
					}
				}
			} else {
				// Enable content editing.
				if (ContentConstants.TextType.RICHTEXT === this._type) {
					this._richTextEditor.enable();
				} else {
					this._plainTextEditor.prop("readonly", false);
				}
			}
		};	
		
		/**
		/**
		 * update the TextContent area
		 * @param content {"content": content} - An object for the text content statement
		 * @returns textDiv - jQuery element for TextContent textarea
		 */
		ContentStatementElementWidgetView.prototype.updateContent = function (content) {
			
			if (!content) {
				content = {content : ""};
			}
			this._content = content;
			
			// Determine whether we should be able to edit the statement or not.
			if (ContentConstants.TextType.RICHTEXT === this._type) {
				this._richTextEditor.setContent(content.content);
			} else {
				this._plainTextEditor.text(content.content);
			}
			
			this.applyEditPermission();
		};
		
		
		/**
		 * Construct the comments column of the widget
		 * 
		 * @returns commentDiv - jQuery element for comments column
		 */
		ContentStatementElementWidgetView.prototype._constructCommentsDiv = function () {
			// If user does not have permissions and there are no existing comments, then do not create comments div
			if((!this._canEdit && !this._canReview && !this._isContentManagerView) && (this._comments.length === 0)) {
				return;
			}
			
			this._commentsDiv = $('<div>').attr({"name": "commentsDiv"});
			
			// hide the comments initially
			this._commentsDiv.addClass("TextComment commentshidden").hide();
			this._commentsDiv.append(
					$('<table>').addClass("TextComments").append(
							$('<tbody>')
					)
			);
			
			// Show the Add comment content area if the user has permissions to add comment
			if(this._canEdit || this._canReview || this._isContentManagerView) {
				this._commentsDiv.find("tbody:first").append($('<tr>').addClass("Comment").append(
						$('<td>').addClass("CommentAuthor").text(TextContentStrings.t("PCM_COMMENT_CURRENT_USER")),
						$('<td>').addClass("CommentContent").append(
								this._commentTextArea.addClass("CommentField").attr({"placeholder": TextContentStrings.t("PCM_ADD_COMMENT_PLACEHOLDER")})
						)
				));
			}
			
			// Add the existing comments to the comments div
			if(this._comments.length > 0) {
				var textcomments = $.extend(true, [], this._comments);
				textcomments.reverse();
				
				var newComment = null;
				_.each(textcomments, _.bind(function(comment) {
					if(!comment.isNew) {
						this._commentsDiv.find("tbody:first").append(
								$('<tr>').addClass("Comment").append(
										$('<td>').addClass("CommentAuthor").text(comment.user),
										$('<td>').addClass("CommentContent").append(
												$('<div>').addClass("CommentText").text(comment.content),
												$('<div>').addClass("CommentDate").text(ConvertUtils.getLangFormattedDateAndTime(TextContentStrings, new Date(WCXML.xmlencode(comment.created))))
										)
								)
						);
					} else {
						newComment = comment;
					}
				}, this));
				
				if(newComment) {
					this._commentTextArea.text(newComment.content);
				}
			}
			
			return this._commentsDiv;
		};
		
		/**
		 * Get the localized value for a text content state
		 * 
		 * @param {String} [stateCode] 
		 * @returns {String} [localizedState]
		 */
		ContentStatementElementWidgetView.prototype._getLocalizedContentState = function(state, myapprovalstatus) {
			var localizedState = state;
			switch (state) {
			case ContentConstants.State.APPROVED:
				localizedState = TextContentStrings.t("PCM_CONTENT_STATUS_APPROVED");
				break;
			case ContentConstants.State.NEW:
				localizedState = TextContentStrings.t("PCM_CONTENT_STATUS_NEW");
				break;
			case ContentConstants.State.DRAFT:
				localizedState = TextContentStrings.t("PCM_CONTENT_STATUS_DRAFT");
				break;
			case ContentConstants.State.READY_FOR_APPROVAL:
				if(myapprovalstatus === ContentConstants.State.APPROVED) {
					localizedState = TextContentStrings.t("PCM_CONTENT_STATUS_PENDING_YOUAPPROVED");
				} else if(myapprovalstatus === ContentConstants.State.REJECTED) {
					localizedState = TextContentStrings.t("PCM_CONTENT_STATUS_PENDING_YOUREJECTED");
				} else if(myapprovalstatus === ContentConstants.State.READY_FOR_APPROVAL) {
					localizedState = TextContentStrings.t("PCM_CONTENT_STATUS_PENDING_YOUPENDING");
				} else {
					localizedState = TextContentStrings.t("PCM_CONTENT_STATUS_PENDING");
				}
				break;
			case ContentConstants.State.EMPTY:
				localizedState = TextContentStrings.t("PCM_CONTENT_STATUS_EMPTY");
				break;
			case ContentConstants.State.REJECTED:
				localizedState = TextContentStrings.t("PCM_CONTENT_STATUS_REJECTED");
				break;
			default:
				break;
			}
			
			return localizedState;
		};
		
		/**
		 * Get the localized action value for a text content state
		 * 
		 * @param {String} [stateCode] 
		 * @returns {String} [localizedAction]
		 */
		ContentStatementElementWidgetView.prototype._getLocalizedContentAction = function(state) {
			var localizedAction = state;
			switch (state) {
			case ContentConstants.State.APPROVED:
				localizedAction = TextContentStrings.t("PCM_ACTION_APPROVE");
				break;
			case ContentConstants.State.DRAFT:
				localizedAction = TextContentStrings.t("PCM_ACTION_DRAFT");
				break;
			case ContentConstants.State.READY_FOR_APPROVAL:
				localizedAction = TextContentStrings.t("PCM_ACTION_START_APPROVAL");
				break;
			case ContentConstants.State.REJECTED:
				localizedAction = TextContentStrings.t("PCM_ACTION_REJECT");
				break;
			default:
				break;
			}
			
			return localizedAction;
		};
		
		/**
		 * Focus on the widget.
		 */
		ContentStatementElementWidgetView.prototype.focus = function (focus) {
			// Check on which type of editor to focus.
			if (ContentConstants.TextType.RICHTEXT === this._type) {
				if (focus) {
					// Focus on the rich text editor.
					this._richTextEditor.focus();
				} else {
					// Blur the plain rich editor
					this._richTextEditor.blur();
				}
			} else {
				if (focus) {
					// Focus on the plain text editor
					this._plainTextEditor.focus();
				} else {
					// Blur the plain text editor
					this._plainTextEditor.blur();
				}
			}
		};
		
		/**
		 * Enable/Disable the widget.
		 */
		ContentStatementElementWidgetView.prototype.enable = function (enable) {
			// Check which type of editor to disable/enable.
			if (ContentConstants.TextType.RICHTEXT === this._type) {
				if (enable) {
					// Focus on the rich text editor.
					this._richTextEditor.enable();
				} else {
					// Blur the plain rich editor
					this._richTextEditor.disable();
				}
			} else {
				if (enable) {
					// Focus on the plain text editor
					this._plainTextEditor.prop('readonly', false);
				} else {
					// Blur the plain text editor
					this._plainTextEditor.prop("readonly", "readonly");
				}
			}
			// TODO also prevent the approval of the widget, @Anil?
		};
		
		/**
		 * Add callback for Start Approval action
		 * @param callback - callback function to perform the start approval action
		 * @returns
		 */
		ContentStatementElementWidgetView.prototype.onStartApproval = function (callback) {
			return this._onStartApprovalCallbacks.push(callback);
		};
		
		/**
		 * Add callback for Reject action
		 * @param callback - callback function to perform the reject action
		 * @returns
		 */
		ContentStatementElementWidgetView.prototype.onReject = function (callback) {
			return this._onRejectCallbacks.push(callback);
		};
		
		/**
		 * Add callback for Approve action
		 * @param callback - callback function to perform the approve action
		 * @returns
		 */
		ContentStatementElementWidgetView.prototype.onApprove = function (callback) {
			return this._onApproveCallbacks.push(callback);
		};
		
		/**
		 * Add callback for Revert to Draft action
		 * @param callback - callback function to perform the revert to draft action
		 * @returns
		 */
		ContentStatementElementWidgetView.prototype.onRevertToDraft = function (callback) {
			return this._onRevertToDraftCallbacks.push(callback);
		};
		
		/**
		 * Add callback for Delete action
		 * @param callback - callback function to peform delete action using delete icon
		 * @returns
		 */
		ContentStatementElementWidgetView.prototype.onDelete = function (callback) {
			return this._onDeleteCallbacks.push(callback);
		};
		
		/**
		 * Add callback for adding comment
		 * @param - callback function to add comment using comment textarea
		 */
		ContentStatementElementWidgetView.prototype.onComment = function (callback) {
			return this._onCommentCallbacks.push(callback);
		};
		
		/**
		 * Add callback for reverting the approval
		 * @param callback - callback function to revert the approval action (approve/reject/revert-to-draft/start-approval)
		 */
		ContentStatementElementWidgetView.prototype.onRevertApproval = function (callback) {
			return this._onRevertApprovalCallbacks.push(callback);
		};
		
		/**
		 * Add callback that should be executed when the approval info button is clicked.
		 * 
		 * @param callback - callback function to update the textcontent
		 */
		ContentStatementElementWidgetView.prototype.onApprovalInfo = function (callback) {
			return this._onApprovalInfoCallbacks.push(callback);
		};
		
		/**
		 * Add callback that should be executed when the main content is clicked.
		 * 
		 * @param callback - callback function to update the textcontent
		 */
		ContentStatementElementWidgetView.prototype.onMainContent = function (callback) {
			return this._onMainContentCallbacks.push(callback);
		};
		
		/**
		 * Method to return the wrapper jQuery element of the widget
		 * 
		 * @returns wrapper - Return the wrapping jquery element.
		 */
		ContentStatementElementWidgetView.prototype.getJQElement = function () {
			return this._wrapper;
		};
		
		/**
		 * Method to check if the element state is EMPTY
		 * 
		 * @returns {Boolean}
		 */
		ContentStatementElementWidgetView.prototype.isEmptyElement = function () {
			return this._state === ContentConstants.State.EMPTY;
		};
		
		// Return the constructor.
		return ContentStatementElementWidgetView;
});

define('packaging-content-management/text-content-element-widget/DocXContentStatementElementWidgetView',["underscore",
        "jquery",
        "./ContentStatementElementWidgetView",
        "../ContentConstants",
        "utils/webi18n!TextContentStrings"],
	function (_,
			$,
			ContentStatementElementWidgetView,
			ContentConstants,
			TextContentStrings) {
	
		"use strict";

		/**
		 * Create the default DocXContentStatementElementWidgetView.
		 * 
		 * @param {Object} text - The content object from the ContentStatementElementWidget represented by this widget.
		 * @param {Array} comments - The array of Comment objects from the ContentStatementElementWidget represented by this widget.
		 * @param {String} state - The state of the ContentStatementElementWidget represented by this widget.
		 * @param {String} language - The language object of the ContentStatementElementWidget represented by this widget.
		 * @param {Boolean} canEdit - Whether or not the ContentStatementElementWidget represented by this widget can be edited by the current user.
		 * @param {Boolean} whyCanNotEdit - Why the ContentStatementElementWidget represented by this widget can't be edited by the current user.
		 * @param {Boolean} canReview - Whether or not the ContentStatementElementWidget represented by this widget can be reviewed by the current user.
		 * @param {Boolean} whyCanNotReview - Why the ContentStatementElementWidget represented by this widget can't be reviewed by the current user.
		 * @param {Boolean} isContentManagerView - Specifies whether the current user is a Content_Manager
		 * @param {String} myApprovalStatus - Approval status of TextContent element for a Approver
		 * @param {Object} [viewOptions] - An object containing some specific variables indicating how the widget will be rendered.
		 * @param {String} markedApprovalState - Approval action taken if any, Current status otherwise
		 * @param {String} type - text type as "richtext" or "simple".
		 */
		var DocXContentStatementElementWidgetView = function (text, isMaster, comments, state, languageCode, canEdit, whyCanNotEdit, canReview, whyCanNotReview, isContentManagerView, myApprovalStatus, viewOptions, markedApprovalState, type) {
			
			// call the parent prototype object to inherit properties
			ContentStatementElementWidgetView.call(this, text, isMaster, comments, state, languageCode, canEdit, whyCanNotEdit, canReview, whyCanNotReview, isContentManagerView, myApprovalStatus, viewOptions, markedApprovalState, type);
			
			// call the overridden function to render the doctablex view of the widget
			this._constructView();
		};
		
		DocXContentStatementElementWidgetView.prototype = Object.create(ContentStatementElementWidgetView.prototype);
		DocXContentStatementElementWidgetView.prototype.constructor = DocXContentStatementElementWidgetView;
		DocXContentStatementElementWidgetView.prototype.parent = ContentStatementElementWidgetView.prototype;
		
		/**
		 * First override the content height before executing the initializeDOMElements method.
		 */
		DocXContentStatementElementWidgetView.prototype._initializeDOMElements = function () {
			this._contentHeight = 76;
			ContentStatementElementWidgetView.prototype._initializeDOMElements.call(this);
		};
		
		/**
		 * Construct the HTML elements of this view.
		 * 
		 */
		DocXContentStatementElementWidgetView.prototype._constructView = function () {
			this._wrapper.empty();
			
			this._initializeDOMElements();
			
			this._wrapper.addClass("DocTableXView");
			
			// create widget elements and append to the wrapper
			this._wrapper.append(
					this._constructApprovalDiv(),
					$('<div>').addClass("TextContent-ContentCol TextContent-ContentColDocX").append(
							$('<div>').addClass("TextContent-InnerContentCol").append(
									this._constructTextDiv()
							)
					),
					this._constructRightColDiv(),
					this._constructCommentsDiv()
			);
			
			// Update css for TextContent textarea div
			$("div[name='Content-ContentAreaDiv']", this._wrapper).addClass("Content-ContentRowDocX");
			
			// Update css for comments div
			$(".TextComment", this._wrapper).addClass("TextCommentDocX");
		};
		
		/**
		 * Construct the right column with comment, approval info, mystatus buttons
		 * 
		 * @returns rightPanel - jQuery element for the right side actions column of the widget
		 */
		DocXContentStatementElementWidgetView.prototype._constructRightColDiv = function() {
			var rightPanel = $('<div>').addClass("TextContent-RightCol");
			
			// If the element status is empty - don't add comment, approvalinfo, mystatus buttons
			if(!this.isEmptyElement()) {
				this._createCommentButton();
				rightPanel.append(this._commentButton.removeClass().addClass("TextContent-ActionDocX TextContent-ActionBorder"));
				
				this._createApprovalInfoButton();
				rightPanel.append(this._approvalInfoButton.removeClass().addClass("TextContent-ActionDocX"));
			}
			
			return rightPanel;
		};
		
		/**
		 * Show the comments section
		 */
		DocXContentStatementElementWidgetView.prototype._showCommentSection = function() {
			
			// When showing the comments section in the last row of the DocTableX, it could be hidden as DocTableX hides vertical overflow.
			// Check if this is the case and otherwise make some room by adding some padding to the bottom row. 
			// (While not ideal, it's the best place for this at the moment).
			
			// Select the DocTableX row in which this widget is sitting.
			var docXRow = this._wrapper.closest("tbody.DocEntry");
			// Check if we found a row and wether it's the last row of the table.
			if (docXRow.length > 0 && docXRow.is(':last-child')) {
				// Get the DocTableX cell in which this widget is sitting.
				var parentCell = this._wrapper.parent();
				// Make sure we found a cell.
				if (parentCell.length > 0 && parentCell.prop("tagName").toLowerCase() == "td") {
					// Set the bottom padding to the height of the comments div to make place for it.
					docXRow.find("> tr:first-child > td").css({
						"padding-bottom" : this._commentsDiv.height()+6+"px"
					});	
				}
			}
			
			this._commentsDiv.removeClass("commentshidden").slideDown(200);
		};

		/**
		 * Hide the comments section
		 */
		DocXContentStatementElementWidgetView.prototype._hideCommentSection = function() {
			
			// When showing the comments section in the last row of the DocTableX, it could be hidden as DocTableX hides vertical overflow.
			// Check if this is the case and otherwise make some room by adding some padding to the bottom row.
			// When hiding the comment section we need to remove that space again.
			
			// Select the DocTableX row in which this widget is sitting.
			var docXRow = this._wrapper.closest("tbody.DocEntry");
			// Check if we found a row and wether it's the last row of the table.
			if (docXRow.length > 0 && docXRow.is(':last-child')) {
				// Get the DocTableX cell in which this widget is sitting.
				var parentCell = this._wrapper.parent();
				// Make sure we found a cell.
				if (parentCell.length > 0 && parentCell.prop("tagName").toLowerCase() == "td") {
					// Reset the bottom padding of 6px.
					docXRow.find("> tr:first-child > td").css({
						"padding-bottom" : ""
					});	
				}
			}
			
			this._commentsDiv.addClass("commentshidden").slideUp(200);
		};
		
		
		return DocXContentStatementElementWidgetView;
		
});

define('packaging-content-management/text-content-element-widget/ViewerContentStatementElementWidgetView',["underscore",
        "jquery",
        "./ContentStatementElementWidgetView",
        "../ContentConstants",
        "utils/webi18n!TextContentStrings"],
	function (_,
			$,
			ContentStatementElementWidgetView,
			ContentConstants,
			TextContentStrings) {
	
		"use strict";

		/**
		 * Create the ViewerContentStatementElementWidgetView.
		 * 
		 * @param {Object} text - The content object from the ContentStatementElementWidget represented by this widget.
		 * @param {Array} comments - The array of Comment objects from the ContentStatementElementWidget represented by this widget.
		 * @param {String} state - The state of the ContentStatementElementWidget represented by this widget.
		 * @param {String} language - The language object of the ContentStatementElementWidget represented by this widget.
		 * @param {Boolean} canEdit - Whether or not the ContentStatementElementWidget represented by this widget can be edited by the current user.
		 * @param {Boolean} whyCanNotEdit - Why the ContentStatementElementWidget represented by this widget can't be edited by the current user.
		 * @param {Boolean} canReview - Whether or not the ContentStatementElementWidget represented by this widget can be reviewed by the current user.
		 * @param {Boolean} whyCanNotReview - Why the ContentStatementElementWidget represented by this widget can't be reviewed by the current user.
		 * @param {Boolean} isContentManagerView - Specifies whether the current user is a Content_Manager
		 * @param {String} myApprovalStatus - Approval status of TextContent element for a Approver
		 * @param {Object} [viewOptions] - An object containing some specific variables indicating how the widget will be rendered.
		 * @param {String} markedApprovalState - Approval action taken if any, Current status otherwise
		 * @param {String} type - text type as "richtext" or "simple".
		 */
		var ViewerContentStatementElementWidgetView = function (text, isMaster, comments, state, languageCode, canEdit, whyCanNotEdit, canReview, whyCanNotReview, isContentManagerView, myApprovalStatus, viewOptions, markedApprovalState, type) {
			
			// call the parent prototype object to inherit properties
			ContentStatementElementWidgetView.call(this, text, isMaster, comments, state, languageCode, canEdit, whyCanNotEdit, canReview, whyCanNotReview, isContentManagerView, myApprovalStatus, viewOptions, markedApprovalState, type);
			
			// Do not show tooltips in the viewer.
			this._showTooltips = false;
			
			// call the overridden function to render the doctablex view of the widget
			this._constructView();
		};
		
		ViewerContentStatementElementWidgetView.prototype = Object.create(ContentStatementElementWidgetView.prototype);
		ViewerContentStatementElementWidgetView.prototype.constructor = ViewerContentStatementElementWidgetView;
		ViewerContentStatementElementWidgetView.prototype.parent = ContentStatementElementWidgetView.prototype;
		
		/**
		 * First override the content height before executing the initializeDOMElements method.
		 */
		ViewerContentStatementElementWidgetView.prototype._initializeDOMElements = function () {
			this._contentHeight = 76;
			ContentStatementElementWidgetView.prototype._initializeDOMElements.call(this);
		};
		
		/**
		 * Construct the HTML elements of this view.
		 * 
		 */
		ViewerContentStatementElementWidgetView.prototype._constructView = function () {
			this._wrapper.empty();
			
			this._initializeDOMElements();
			
			this._wrapper.addClass("Viewer");
			this._wrapper.addClass("enableTextSelection");
			
			// create widget elements and append to the wrapper
			this._wrapper.append(
					this._constructApprovalDiv(),
					$('<div>').addClass("TextContent-ContentCol").append(
							$('<div>').addClass("TextContent-InnerContentCol").append(this._constructTextDiv())
					)
			);
		};
		
		return ViewerContentStatementElementWidgetView;
		
});

define('packaging-content-management/PCMEventBus',["jquery",
        "underscore",
        "lib/EventEmitter2"],
		function($, _,	EventEmitter2) {
			
		"use strict";
		
		if (!window.pcmEventBus) {
			window.pcmEventBus = new EventEmitter2({
				wildcard: true, // should the event emitter use wildcards.
				delimiter: '.', // the delimiter used to segment namespaces,
				newListener: true // if you want to emit the newListener event
			});
			
			window.pcmEventBus.setMaxListeners(999); 
		}
		
		return window.pcmEventBus;
});

define('packaging-content-management/text-content-services/languageService',["jquery",
        "packaging-content-management/PCMEventBus",
        "packaging-content-management/text-content-utils/LanguageUtils"], function ($, pcmEventBus, LanguageUtils) {

	
	
	var LanguageService = function () {
		
		// These private variables will cache the returned languages and smartnames
        this._cachedLanguages = {};
        
        // These private variables take care of preventing duplicate requests for project languages.
        this._langsAreBeingRetrieved = {};
        this._onLangsRetrieved = {};
        
        // These private variables take care of preventing duplicate request for all languages.
        this._allLangsAreBeingRetrieved = false;
        this._onAllLangsRetrieved = [];
        
        // Add a listener to update our cache when a language was removed/added from the project.
        pcmEventBus.on("PCM.projMasterLangChanged", _.bind(function (langAttrName, addedLangDesc, removedLangDesc, projectID) {
        	
			if(this._cachedLanguages[projectID]) {
				var isMaster = true;
				// Determine wether we need to work on Master/Translation text.
				var projLangs = this._cachedLanguages[projectID].master;
				var allLangs = this._cachedLanguages.all.master;
				if ('MASTER_TEXT_LANGUAGE' !== langAttrName) {
					isMaster = false;
					projLangs = this._cachedLanguages[projectID].translations;
					allLangs = this._cachedLanguages.all.translations;
				}
				
				// Let's start by mapping the passed language descriptions to real language objects.
					
				// Start searching for the passed language description in the "All Languages" array.
				var addedLang = [];
				var removedLang = [];
				for (var i=0; i < addedLangDesc.length;i++) {
					var lang = LanguageUtils.getLangForDescription(allLangs, addedLangDesc[i]);
					if (lang) {
						addedLang.push(lang);
					}
				}
				for (var i=0; i < removedLangDesc.length;i++) {
					var lang = LanguageUtils.getLangForDescription(allLangs, removedLangDesc[i]);
					if (lang) {
						removedLang.push(lang);
					}
				}
				
				// Now add or remove them to the project languages.
				if(addedLang.length > 0) {
					if(isMaster) {
						this._cachedLanguages[projectID].master = projLangs.concat(addedLang);
					} else {
						this._cachedLanguages[projectID].translations = projLangs.concat(addedLang);
					}
					
				}
				if(removedLang.length > 0) {
					if(isMaster) {
						LanguageUtils.removeFrom(this._cachedLanguages[projectID].master, removedLang);
					} else {
						LanguageUtils.removeFrom(this._cachedLanguages[projectID].translations, removedLang);
					}
				}
			}
        }, this));
        
	};
	

	/**
	 * Return the project master or translation languages for a certain text content document.
	 * 
	 * @param projectId
	 * @param isMaster
	 * @param onProjectLangsCallback
	 * @returns
	 */
	LanguageService.prototype.getProjectLanguages = function (projectId, isMaster, onProjectLangsCallback) {
		if (this._cachedLanguages[projectId]) {
			// We have a cache entry for that projectId, return from cache.
			this._returnCachedLanguagesFor(projectId, isMaster, true, onProjectLangsCallback);
		} else {
			// We don't have a cache entry, this means that they still need to be fetched.
			// Execute the "onProjectLangsCallback" when the're retrieved by adding it to the "this._onLangsRetrieved" array.
			if (!this._onLangsRetrieved[projectId]) {
				this._onLangsRetrieved[projectId] = [];
			}
			this._onLangsRetrieved[projectId].push(_.bind(function () {
				this._returnCachedLanguagesFor(projectId, isMaster, true, onProjectLangsCallback);
			}, this));
			
			// Fetch the languages.
			this._getLanguages(projectId);
		}
	};

	
	/**
	 * Return all the master or translation languages for whole WebCenter.
	 * 
	 * @param projectId
	 * @param isMaster
	 * @param onAllLangsCallback
	 * @returns
	 */
	LanguageService.prototype.getAllLanguages = function (projectId, isMaster, onAllLangsCallback) {
		if (this._cachedLanguages.all) {
			// We have a cache entry for that projectId, return from cache.
			this._returnCachedLanguagesFor(projectId, isMaster, false, onAllLangsCallback);
		} else {
			// We don't have a cache entry, this means that they still need to be fetched.
			// Execute the "onProjectLangsCallback" when the're retrieved by adding it to the "this._onLangsRetrieved" array.
			if (!this._onLangsRetrieved[projectId]) {
				this._onLangsRetrieved[projectId] = [];
			}
			this._onLangsRetrieved[projectId].push(_.bind(function () {
				this._returnCachedLanguagesFor(projectId, isMaster, false, onAllLangsCallback);
			}, this));
			
			// Fetch the languages.
			this._getLanguages(projectId);
		}
	};
	
	/**
	 * Get both the translation and master languages.
	 */
	LanguageService.prototype.getBothAllLanguages = function (onAllLangsCallback) {
		
		if (this._cachedLanguages.all){
			onAllLangsCallback(this._cachedLanguages.all.master, this._cachedLanguages.all.translations);
		} else {
			
			// We don't have a cache entry, this means that they still need to be fetched.
			// Execute the "onProjectLangsCallback" when the're retrieved by adding it to the "this._onLangsRetrieved" array.
			this._onAllLangsRetrieved.push(_.bind(function () {
				onAllLangsCallback(this._cachedLanguages.all.master, this._cachedLanguages.all.translations);
			}, this));
			
			// We need to fetch all the languages.
			this._getAllLanguages();
		}
	};
	
	/**
	 * A utility function to return some cached languages.
	 * 
	 * @param projectId
	 * @param isMaster
	 * @param onlyProject
	 * @param onLangsCallback
	 * @returns
	 */
	LanguageService.prototype._returnCachedLanguagesFor = function (projectId, isMaster, onlyProject, onLangsCallback) {
		
		// Check if need for all the languages or just the project languages.
		if (onlyProject) {
			// Get the cached languages for that project.
			var docVerLang =  this._cachedLanguages[projectId];
			if (isMaster) {
				onLangsCallback(docVerLang.master);
			} else {
				onLangsCallback(docVerLang.translations);
			}
		} else {
			// Get the cached languages for whole WebCenter.
			var allLang = this._cachedLanguages.all;
			if (isMaster) {
				onLangsCallback(allLang.master);
			} else {
				onLangsCallback(allLang.translations);
			}
		}
	};
	
	/**
	 * 
	 * @param projectId
	 * @param onLangsCallback
	 * @param textContentsLangs - In Viewer TextContentWidget when no Project language attributes are present then the languages in Textcontents are considered
	 */
	LanguageService.prototype.getLanguages = function(projectId, onLangsCallback, textContentsLangs) {
		
		if (this._cachedLanguages.all && this._cachedLanguages[projectId]) {
			// We have a cache entry for these languages.
			var allLang = this._cachedLanguages.all;
			var docVerLang =  this._cachedLanguages[projectId];
			onLangsCallback(docVerLang.master, docVerLang.translations, allLang.master, allLang.translations);
		} else {
			// We don't have a cache entry for the languages, this means that they still need to be fetched.
			// Execute the "onLangsCallback" when the're retrieved by adding it to the "this._onLangsRetrieved" array.
			if (!this._onLangsRetrieved[projectId]) {
				this._onLangsRetrieved[projectId] = [];
			}
			this._onLangsRetrieved[projectId].push(_.bind(function () {
				var allLang = this._cachedLanguages.all;
				var docVerLang =  this._cachedLanguages[projectId];
				onLangsCallback(docVerLang.master, docVerLang.translations, allLang.master, allLang.translations);
			}, this));
			
			// Fetch the languages.
			this._getLanguages(projectId, textContentsLangs);
		}
	};
	
	
	/**
	 * Make a language retrieval request to WebCenter.
	 */
	LanguageService.prototype._getLanguages = function (projectId, languages) {
    	
		// Check if we're not already retrieving the languages for this projectId.
		if (this._langsAreBeingRetrieved[projectId] === undefined || this._langsAreBeingRetrieved[projectId] === false) {
			
    		// Indicate that they're being fetched now for this projectId 
			this._langsAreBeingRetrieved[projectId] = true;
    		
			// The baseUrl of the service.
	        var jspBaseUrl = window.webcenterConfig.jspBaseUrl;
			
	        var url = jspBaseUrl+"GetProjectLanguageAttr.jsp?json=1&projectId="+projectId;
	        
	        if(languages) {
	        	url += "&languages=" + languages;
	        }
	        
    		// Do the async call to the service
    		$.getJSON(url, _.bind(function( data ) {
    		
    			// Parse the json string.
    			this._cachedLanguages.all = {};
    			this._cachedLanguages.all.master = this._parseLangs(data.languages.complete_source_language.language);
    			this._cachedLanguages.all.translations = this._parseLangs(data.languages.complete_translation_language.language);
    			this._cachedLanguages[projectId] = {};
    			
    			if (data.languages.selected_source_language) {
        			this._cachedLanguages[projectId].master = this._parseLangs(data.languages.selected_source_language.language);
    			} else {
    				this._cachedLanguages[projectId].master = [];
    			}
    			
    			if (data.languages.selected_translation_language) {
    				this._cachedLanguages[projectId].translations = this._parseLangs(data.languages.selected_translation_language.language);
    			} else {
    				this._cachedLanguages[projectId].translations = [];
    			}
    			
    			// Execute the onLangsRetrieved callbacks.
    			var callbacks = this._onLangsRetrieved[projectId];
    			if (callbacks) {
    				for (var i=0; i < callbacks.length; i++) {
        				callbacks[i]();
        			}
    			}
    			
        	}, this)).fail(function(jqxhr, textStatus, error) {
        		
        		
        	});
        }
	};
	
	
	/**
	 * Make a language retrieval request to WebCenter without providing prjectId. <br>
	 * Useful for those functionality where projectId is not available like- Search.
	 */
	LanguageService.prototype._getAllLanguages = function () {
		
		if (this._allLangsAreBeingRetrieved === false) {
			
			// Indicate that they're being fetched now 
			this._allLangsAreBeingRetrieved = true;
    		
			// The baseUrl of the service.
	        var jspBaseUrl = window.webcenterConfig.jspBaseUrl;
			
    		// Do the async call to the service
    		$.getJSON( jspBaseUrl+"GetProjectLanguageAttr.jsp?json=1", _.bind(function( data ) {
    			
    			// Parse the json string.
    			this._cachedLanguages.all = {};
    			this._cachedLanguages.all.master = this._parseLangs(data.languages.complete_source_language.language);
    			this._cachedLanguages.all.translations = this._parseLangs(data.languages.complete_translation_language.language);
    			
    			// Execute the onLangsRetrieved callbacks.
    			for (var i=0; i < this._onAllLangsRetrieved.length; i++) {
    				this._onAllLangsRetrieved[i]();
        		}
  
    			
    			this._allLangsAreBeingRetrieved = false;
    			
    		}, this)).fail(function(jqxhr, textStatus, error) {
        		
        		
        	});
		}
	};
	
	/**
	 * Make a language retrieval request to WebCenter.
	 */
	LanguageService.prototype._parseLangs = function (langsFromXml) {
		var langs = [];
		
		if (langsFromXml instanceof Array) {
			for (var i=0; i <langsFromXml.length; i++) {
				var lang = {};
				lang.langCode = langsFromXml[i].language_code;
				lang.langDesc = langsFromXml[i].descriptor;
				lang.langFlag = langsFromXml[i].flag_file_name;
				lang.langIndex = langsFromXml[i].language_index;
				langs.push(lang);
			}
		} else if (langsFromXml instanceof Object) {
			var lang = {};
			lang.langCode = langsFromXml.language_code;
			lang.langDesc = langsFromXml.descriptor;
			lang.langFlag = langsFromXml.flag_file_name;
			lang.langIndex = langsFromXml.language_index;
			langs.push(lang);
		}
		
		return langs;
	};
	
    return new LanguageService();

});

define('packaging-content-management/text-content-services/elementTypeService',["jquery"], function ($) {

	
	
	var ElementTypeService = function () {
		
		// These private variables will cache the returned languages and smartnames
        this._cachedElementTypes = undefined;
        
        this._cachedElementTypesWithoutDelimiter = undefined;
        
        // These private variables take care of preventing duplicate requests.
        this._elementTypesAreBeingRetrieved = false;
        
        this._onElementTypesRetrieved = [];
	};
	
	/**
	 * Method is to get readable format of the element type code without worried about delimiter "_" present in element type code string.
	 * 
	 * @param {String} elementTypeCode : Element type code for which readable format is required.
	 */
	ElementTypeService.prototype.getReadableElementType = function (elementTypeCode, callback) {
		if (elementTypeCode) {
			elementTypeCode = elementTypeCode.trim().replace(/\_/g, '');
			
			if (this._cachedElementTypesWithoutDelimiter) {
				if (this._cachedElementTypesWithoutDelimiter[elementTypeCode]) {
					callback(this._cachedElementTypesWithoutDelimiter[elementTypeCode]);
				} else {
					callback(elementTypeCode);
				}
			} else {
				this.getElementTypes(function (elementTypes) {
					var elementTypeReadable = elementTypes[elementTypeCode];
					if (elementTypeReadable) {
						callback(elementTypeReadable);
					} else {
						callback(elementTypeCode);
					}
				});
			}
		}
	};
	
	/**
	 * 
	 */
	ElementTypeService.prototype.getElementTypes = function (callback) {
		
		// Check if we have cached element types.
		if (this._cachedElementTypes) {
			
			callback(this._cachedElementTypes);
			
		} else {
			
			this._onElementTypesRetrieved.push(callback);
			
			if (!this._elementTypesAreBeingRetrieved) {
				
				this._elementTypesAreBeingRetrieved = true;
				
				// The baseUrl of the service.
		        var jspBaseUrl = window.webcenterConfig.jspBaseUrl;
				
				$.ajax({
					url: jspBaseUrl+"GetRowsFromList.jsp?listType=WCR_DB_List&indexed=1",
					type: "GET",
					dataType : "text",
					data : {
						outputType:"JSON",
						list : "ELEMENT_TYPES",
						maxrows : 200,
						maxcolrows : 200
					},
					success: function(jsonContent) {
						
						this._cachedElementTypes = {};
						this._cachedElementTypesWithoutDelimiter = {};
						
						try {
							var jsonObj;
							try {
								jsonObj = jsonParse(jsonContent);
							} catch(err) {			// if jsonParse fails, fall back on eval (which is slower)
								jsonObj = eval('(' + jsonContent + ')');
							}
							if (!jsonObj.success || jsonObj.success == "0") {
								callback('Cannot get values ' + jsonObj.error.code + ' : '+ jsonObj.error.message);
							} else {
								
								var elementTypesValues = [];
								var elementTypesColIndex = -1;
								var humanReadableValues = [];
								var humanReadableColIndex = -1;
								var elementCategoryValues = [];
								var elementCategoryColIndex = -1;
								var colorValues = [];
								var colorColIndex = -1;

								for (var i=0; i < jsonObj.columns.length; i++) {
									if (jsonObj.columns[i].name == "ELEMENT_TYPES") {
										elementTypesValues = jsonObj.columns[i].values;
										elementTypesColIndex = i;
									} else if (jsonObj.columns[i].name == "HUMAN_READABLE_FORM") {
										humanReadableValues = jsonObj.columns[i].values;
										humanReadableColIndex = i;
									} else if (jsonObj.columns[i].name == "ELEMENT_CATEGORY") {
										elementCategoryValues = jsonObj.columns[i].values;
										elementCategoryColIndex = i;
									} else if (jsonObj.columns[i].name == "COLOR") {
										colorValues = jsonObj.columns[i].values;
										colorColIndex = i;
									} 
								}
								var combinations = jsonObj.combinations;
								
								$.each(combinations, function(index, value) {
									
									var elementDetails = {};
									var combinationsArr = value.split('~');

									var indexElemType = combinationsArr[elementTypesColIndex];
									this._cachedElementTypes[elementTypesValues[indexElemType]] = elementDetails;
									
									// Adding element type details for element type code without delimiter.
									this._cachedElementTypesWithoutDelimiter[elementTypesValues[indexElemType].trim().replace(/\_/g, '')] = elementDetails;
									
									if (humanReadableColIndex >= 0) {
										elementDetails.elementHumanReadable = humanReadableValues[combinationsArr[humanReadableColIndex]];
									}
									
									if (elementCategoryColIndex >= 0) {
										elementDetails.elementCat = elementCategoryValues[combinationsArr[elementCategoryColIndex]];
									}
									
									if (colorColIndex >= 0) {
										elementDetails.color = colorValues[combinationsArr[colorColIndex]];
									}
									
								}.bind(this));
							}
						} catch (err) {
							alert(err);
						}
						
						// Execute the onLangsRetrieved callbacks.
		    			for (var i=0; i < this._onElementTypesRetrieved.length; i++) {
		    				this._onElementTypesRetrieved[i](this._cachedElementTypes);
		    			}
		    			
					}.bind(this)
				});
			}
		}
	};
	
	return new ElementTypeService();
	
});

define('tpl!packaging-content-management/text-content-element-widget/ApprovalOverviewWidgetView', function(){return function(d){
var __t,__p='',__j=Array.prototype.join,print=function(){__p+=__j.call(arguments,'');};
__p+='<table class="table ApprovalOverviewTable">\r\n	<tbody>\r\n	';
 if (d.approvals != undefined && d.approvals.length != 0) { 
__p+='\r\n		';
 _.each(d.approvals, function(approval, index){ 
__p+='\r\n			<tr>\r\n				<td style="text-align: left;">\r\n					';
 if (approval.state == "draft") { 
__p+='\r\n					<i class="icon icon-WCR_Text-Status-Draft" style="color: #a7a9ac"></i>&nbsp;\r\n					<span>'+
((__t=(d.i18n("PCM_STATUS_MARK_DRAFT")))==null?'':_.escape(__t))+
'</span>\r\n					';
 } else if (approval.state == "ready-for-approval") { 
__p+='\r\n					<i class="icon icon-WCR_Text-Status-Pending" style="color: #46b8da"></i>&nbsp;\r\n					<span>'+
((__t=(d.i18n("PCM_START_APPROVAL")))==null?'':_.escape(__t))+
'</span>\r\n					';
 } else if (approval.state == "signed-off") { 
__p+='\r\n					<i class="icon icon-WCR_Text-Status-Approved" style="color: #4cae4c"></i>&nbsp;\r\n					<span>'+
((__t=(d.i18n("PCM_APPROVE")))==null?'':_.escape(__t))+
'</span>\r\n					';
 } else if (approval.state == "signoff-rejected") { 
__p+='\r\n					<i class="icon icon-WCR_Text-Reject" style="color: #d43f3a"></i>&nbsp;\r\n					<span>'+
((__t=(d.i18n("PCM_REJECT")))==null?'':_.escape(__t))+
'</span>\r\n					';
 } 
__p+='\r\n				</td>\r\n				<td>\r\n					'+
((__t=(approval.user))==null?'':_.escape(__t))+
'\r\n				</td>\r\n				<td style="text-align: right;">\r\n					'+
((__t=(approval.ts))==null?'':_.escape(__t))+
'\r\n				</td>\r\n			</tr>\r\n		';
 }); 
__p+='\r\n	';
 } else { 
__p+='\r\n		<tr>\r\n			<td colspan="3">'+
((__t=(d.i18n("PCM_NO_ONGOING_APPROVAL_CYCLE")))==null?'':_.escape(__t))+
'</td>\r\n		</tr>\r\n	';
 } 
__p+='\r\n	</tbody>\r\n</table>';
return __p;
}});
define('utils/webcss-text!packaging-content-management/text-content-element-widget/ApprovalOverviewWidgetStyle.css',[],function () { return '.WcrPageContent .ApprovalOverviewWidgetModal > .modal-dialog > .modal-content > .modal-body {\r\n\tmargin:0px;\r\n\tpadding: 0px;\r\n}\r\n.WcrPageContent .ApprovalOverviewWidgetModal .table.ApprovalOverviewTable {\r\n\tborder:0px;\r\n}\r\n.WcrPageContent .ApprovalOverviewWidgetModal .table.ApprovalOverviewTable > tbody > tr:first-child > td {\r\n\tborder:0px;\r\n}\r\n.WcrPageContent .ApprovalOverviewWidgetModal .table.ApprovalOverviewTable >  tbody > tr > td {\r\n\tpadding:15px;\r\n}';});

define('packaging-content-management/text-content-element-widget/ApprovalOverviewWidgetView',["underscore",
        "jquery",
        "Template",
        "utils/ConvertUtils",
        "tpl!./ApprovalOverviewWidgetView",
        "utils/webi18n!TextContentStrings",
        "../ContentConstants",
        "utils/webcss!./ApprovalOverviewWidgetStyle.css"],
	function (_, 
			$,
			Template, 
			ConvertUtils,
			tpl,
			textContentStrings,
			ContentConstants,
			ApprovalOverviewWidgetStyle) {
		
		"use strict";

		/**
		 * Create a widget to display the approval overview. 
		 */
		var ApprovalOverviewWidgetView = function (approvals) {
			
			this._callbacks = [];
			
			this._approvalsArr = [];
			// Create the wrapper.
			this._wrapper = $("<div>");
			
			// Create the template
			this._t = new Template(textContentStrings, tpl);
			
			// Add the template to the wrapper.
			this._t.setContext(this._wrapper);
			
			for (var i = 0; i < approvals.length; i++) {
				var formattedTime = ConvertUtils.getLangFormattedDateAndTime(textContentStrings, new Date(approvals[i].ts));
				var approvalObj = {'state':approvals[i].state, 'ts':formattedTime, 'user':approvals[i].user};
				this._approvalsArr.push(approvalObj);
			}
			
			// Assign the formatted approvals in the template
			this._t.d.approvals = this._approvalsArr;
			
			// Render
			this._t.render();
			
		};
		
		ApprovalOverviewWidgetView.prototype.update = function (approvals) {
			// Update the approvals in the template.
			this._t.d.approvals = approvals;
			// Render
			this._t.render();
		};
		
		ApprovalOverviewWidgetView.prototype.getJQElement = function () {
			return this._wrapper;
		};
		
		return ApprovalOverviewWidgetView;
});

define('packaging-content-management/text-content-element-widget/ApprovalOverviewWidget',["underscore",
        "jquery",
        "./ApprovalOverviewWidgetView",
        "utils/webi18n!TextContentStrings",
        "../ContentConstants"],
	function (_, 
			$,
			ApprovalOverviewWidgetView,
			TextContentStrings,
			ContentConstants) {
		
		"use strict";

		/**
		 * Create a widget to display the approval overview. 
		 */
		var ApprovalOverviewWidget = function (contentStatementElement) {
			
			this._csElement = contentStatementElement;
			this._view = new ApprovalOverviewWidgetView(this._csElement.originalApproval);
		};
		
		ApprovalOverviewWidget.prototype.update = function () {
			this._view.update(this._csElement.approval);
		};
		
		ApprovalOverviewWidget.prototype.getJQElement = function () {
			return this._view.getJQElement();
		};
		
		return ApprovalOverviewWidget;
});

define('packaging-content-management/text-content-element-widget/ContentStatementElementWidget',["underscore",
        "jquery",
        "./ContentStatementElementWidgetView",
        "./DocXContentStatementElementWidgetView",
        "./ViewerContentStatementElementWidgetView",
        "../text-content-services/languageService",
        "../text-content-services/elementTypeService",
        "../text-content-utils/LanguageUtils",
        "./ApprovalOverviewWidget",
        "utils/webi18n!TextContentStrings",
        "../ContentConstants"],
	function (_, 
			$, 
			ContentStatementElementWidgetView,
			DocXContentStatementElementWidgetView,
			ViewerContentStatementElementWidgetView,
			languageService,
			elementTypeService,
			LanguageUtils,
			ApprovalOverviewWidget,
			TextContentStrings,
			ContentConstants) {
		
		"use strict";

		/**
		 * Create a Widget in order to display and potentially edit the contents of a 'ContentStatementElement'.
		 * 
		 * @param {ContentStatementElement} contentStatementElement - The statement for this widget.
		 * @param {Object} options - An object containing some variables specifying how the widget should behave.
		 * 
		 */
		var ContentStatementElementWidget = function (contentStatementElement, options) {
			
			this._textElement = contentStatementElement;
			this._elementTypeCode = contentStatementElement.parentStatement.elementType;
			this._type = contentStatementElement.parentStatement.type;
			this._options = options;
			
			// Extract the fields from the contentStatementElementWidget that are needed for the view.
			
			var content = this._textElement.content;			
			var languageCode = this._textElement.language;
			var comments = this._textElement.comment;
			var state = this._textElement.state;
			var canEdit = this._textElement.isEditable === 'true'; // Want this to be a boolean.
			var whyCanNotEdit = this._textElement.whyNonEditable;
			var canReview = this._textElement.isReviewable  === 'true'; // Want this to be a boolean.
			var whyCanNotReview = this._textElement.whyNonReviewable;
			var isMaster = this._textElement.master === 'true';
			var isContentManagerView = this._textElement.parentStatement.contentManagerView === 'true';
			var myApprovalStatus = this._textElement.myApprovalStatus;
			
			var markedApprovalState = state;
			if(this._textElement.approvalChanged) {
				markedApprovalState = this._textElement.approval[0].state;
			}

			// Extract the 'viewOptions' from the passed options.
			var viewOptions = {};
			if (this._options && typeof this._options.viewOptions === 'object') {
				viewOptions = this._options.viewOptions;
			}
			
			// callbacks to be executed when any changes happen in the widget
			this._onChangeCallbacks = [];
			
			// create the view of the widget based on the viewOptions
			switch(viewOptions.widgetview) {
			case 'default':
				this._view = new ContentStatementElementWidgetView(content, isMaster, comments, state, languageCode, canEdit, whyCanNotEdit, canReview, whyCanNotReview, isContentManagerView, myApprovalStatus, viewOptions, markedApprovalState, this._type);
				this._updateView(isMaster, isContentManagerView);
				break;
			case 'doctablex':
				this._view = new DocXContentStatementElementWidgetView(content, isMaster, comments, state, languageCode, canEdit, whyCanNotEdit, canReview, whyCanNotReview, isContentManagerView, myApprovalStatus, viewOptions, markedApprovalState, this._type);
				break;
			case 'viewer':
				this._view = new ViewerContentStatementElementWidgetView(content, isMaster, comments, state, languageCode, canEdit, whyCanNotEdit, canReview, whyCanNotReview, isContentManagerView, myApprovalStatus, viewOptions, markedApprovalState, this._type);
				this._setLanguageHeader(isMaster, isContentManagerView);
				break;
			default:
				break;
			}

			// Specify what should happen on Approval actions
			this._view.onApprove(_.bind(function () {
				this._textElement.approval.push({"state": ContentConstants.State.APPROVED});
				this._textElement.approvalChanged = true;
				this._textElement.doneApproveORReject = true;
				this._executeOnChange(ContentConstants.OnChangeCallbackType.APPROVED);
			}, this));
			
			this._view.onReject(_.bind(function () {
				this._textElement.approval.push({"state": ContentConstants.State.REJECTED});
				this._textElement.approvalChanged = true;
				this._textElement.doneApproveORReject = true;
				this._executeOnChange(ContentConstants.OnChangeCallbackType.REJECTED);
			}, this));
			
			this._view.onStartApproval(_.bind(function () {
				this._textElement.approval.push({"state": ContentConstants.State.READY_FOR_APPROVAL});
				this._textElement.approvalChanged = true;
				this._executeOnChange(ContentConstants.OnChangeCallbackType.STARTED_APPROVAL);
			}, this));
			
			this._view.onRevertToDraft(_.bind(function () {
				this._textElement.approval.push({"state": ContentConstants.State.DRAFT});
				this._textElement.approvalChanged = true;
				this._executeOnChange(ContentConstants.OnChangeCallbackType.REVERTED_TO_DRAFT);
			}, this));
			
			// Specify what should happen when the approval action is reverted
			this._view.onRevertApproval(_.bind(function() {
				this._textElement.cancelStatusChange();
				this._executeOnChange(ContentConstants.OnChangeCallbackType.APPROVAL_REVERTED);
			}, this));
			
			// Specify what should happen when a comment is added/edited
			this._view.onComment(_.bind(function(commentText) {
				if($.trim(commentText)) {
					var newComment = {
							content : commentText,
							ts : new Date(),
							user : "you",
							isNew : true
							// add a variable to differentiate between normal and reject comment, if required
					};
					
					// If new comment exists update it, add a new comment otherwise
					if(this._textElement.commentChanged) {
						for(var i=0 ; i < this._textElement.comment.length; i++) {
							if(this._textElement.comment[i].isNew) {
								this._textElement.comment[i] = newComment;
								break;
							}
						}
					} else {
						this._textElement.comment.push(newComment);
						this._textElement.commentChanged = true;
					}
					
					this._executeOnChange(ContentConstants.OnChangeCallbackType.COMMENTED);
				}
			}, this));
			
			// What needs to happen when the approval info button is clicked
			this._view.onApprovalInfo(_.bind(function () {
				// Create a new approval overview widget.
				this._aprovalOverviewWidget = new ApprovalOverviewWidget(this._textElement);
				this._view.showApprovalInfoDialog(this._aprovalOverviewWidget.getJQElement(), this._options);
			}, this));
			
			// What needs to happen when content is changed in editor.
			this._view.onMainContent(_.bind(function (content) {
				var currentContent = {'content': content};
				this.updateCurrentContent(currentContent);
			}, this));
			

		};
		
		ContentStatementElementWidget.prototype.updateCurrentContent = function (currentContent) {
			// Update should be done only if content is actually changed
			if (currentContent.content != this._textElement.content.content) {
				this._textElement.content = currentContent;
				this._textElement.contentChanged = true;
				this._executeOnChange(ContentConstants.OnChangeCallbackType.TEXT_UPDATED);
			}
		};
		
		/**
		 * Make sure the widget is updated with the latest state of the passed contentStatementElement.
		 */
		ContentStatementElementWidget.prototype.update = function (contentStatementElement) {
			this._textElement = contentStatementElement;
			
			// Extract the fields from the contentStatementElementWidget that are needed for the view.
			var content = contentStatementElement.content;
			var languageCode = contentStatementElement.language;
			var comments = contentStatementElement.comment;
			var state = contentStatementElement.state;
			var canEdit = contentStatementElement.isEditable === 'true'; // Want this to be a boolean.
			var whyCanNotEdit = contentStatementElement.whyNonEditable;
			var canReview = contentStatementElement.isReviewable  === 'true'; // Want this to be a boolean.
			var whyCanNotReview = contentStatementElement.whyNonReviewable;
			var isMaster = contentStatementElement.master === 'true';
			var isContentManagerView = contentStatementElement.parentStatement.contentManagerView === 'true';
			var myApprovalStatus = contentStatementElement.myApprovalStatus;
			var type = contentStatementElement.parentStatement.type;
			
			var markedApprovalState = state;
			if(this._textElement.approvalChanged) {
				markedApprovalState = this._textElement.approval[0].state;
			}
			
			this._view.updateWidgetView(content, isMaster, comments, state, languageCode, canEdit, whyCanNotEdit, canReview, whyCanNotReview, isContentManagerView, myApprovalStatus, markedApprovalState, type);
		};
		
		/**
		 * To update the language in header for default view and add a delete icon for non-project languages
		 * @param isMaster
		 * @param isContentManagerView
		 */
		ContentStatementElementWidget.prototype._updateView = function (isMaster, isContentManagerView) {
			this._addDeleteOption(isMaster, isContentManagerView);
			this._setLanguageHeader(isMaster, isContentManagerView);
		};
		
		/**
		 * Adds a delete button in widget when the language is not a project attribute language
		 * 
		 * @param isMaster
		 * @param isContentManagerView
		 */
		ContentStatementElementWidget.prototype._addDeleteOption = function(isMaster, isContentManagerView) {
			// Create delete button if the language is not present in project attribute and user has content manager permission
			var projId = this._textElement.parentStatement.projectId;
			if(isContentManagerView) {
				languageService.getProjectLanguages(projId, isMaster, _.bind(function(langs) {
					var isProjectLang = LanguageUtils.isAProjectAttrLang(langs, this._textElement.language);
					if(!isProjectLang) {
						var revertDeleteHandler = _.bind(function() {
							this._textElement.deleted = false;
						}, this);
						this._view.createDeleteButton(revertDeleteHandler);
						
						// Specify what should happen on Delete action
						this._view.onDelete(_.bind(function () {
							this._textElement.deleted = true;
							this._executeOnChange(ContentConstants.OnChangeCallbackType.DELETED);
						}, this));
					}
				}, this));
			}
		};
		
		/**
		 * Update the language header of the widget
		 * 
		 * @param isMaster
		 * @param isContentManagerView
		 */
		ContentStatementElementWidget.prototype._setLanguageHeader = function(isMaster, isContentManagerView) {
			// Retrieve the languages for this text element.
			var projId = this._textElement.parentStatement.projectId;
			languageService.getAllLanguages(projId, isMaster, function (allLangs) {
				this._view.setLanguage(LanguageUtils.getLangForCode(allLangs, this._textElement.language));
			}.bind(this));
		};
		
		/**
		 * Execute the onchange callback when anything changes in the widget
		 */
		ContentStatementElementWidget.prototype._executeOnChange = function (changetype) {
			_.each(this._onChangeCallbacks, _.bind(function(changetype, callback) {
				callback(changetype, this._textElement);
			}, this, changetype));
		};
		
		/**
		 * Add callback to be executed when any changes is done in the widget
		 */
		ContentStatementElementWidget.prototype.onChange = function (callback) {
			this._onChangeCallbacks.push(callback);
		};
		
		/**
		 * Focus/Blur the widget.
		 */
		ContentStatementElementWidget.prototype.focus = function (focus) {
			this._view.focus(focus);
		};
		
		/**
		 * Disable the widget.
		 */
		ContentStatementElementWidget.prototype.disable = function () {
			this._view.enable(false);
		};
		
		/**
		 * Enable the widget.
		 */
		ContentStatementElementWidget.prototype.enable = function () {
			this._view.enable(true);
		};
		
		/**
		 * Get the current text element
		 */
		ContentStatementElementWidget.prototype.getContentStatementElement = function () {
			return this._textElement;
		};
		
		/**
		 * Return the jQuery HTML element of the widget
		 * @returns
		 */
		ContentStatementElementWidget.prototype.getJQElement = function () {
			return this._view.getJQElement();
		};
		
		// Return the constructor.
		return ContentStatementElementWidget;
		
});

/**
 * ContentStatementElement object
 * @author HUBU
 */
define('web-viewer/textContent/TextElementDynContentWidget',["jquery",
        "packaging-content-management/text-content-element-widget/ContentStatementElementWidget",
        "packaging-content-management/text-content-services/languageService",
		"packaging-content-management/text-content-utils/LanguageUtils",
		"utils/ShapeUtils"
	],
	function ($,
			ContentStatementElementWidget,
			languageService,
			languageUtils,
			ShapeUtils) {
		"use strict";
		
		var TextElementDynContentWidget = function (dynContentModel, dynContentPopupModel, textElement) {
			
			this.dynContentModel = dynContentModel;
			this.dynContentPopupModel = dynContentPopupModel;
			this.textElement = textElement;
			this.popup = this.dynContentPopupModel.popup;
			this.transTextElement;
			
			// Create the wrapping table.
			this._wrapper = $("<table>").css("height","100%").css("width","100%");
			
			// Initialize the widget.
			this.init();
		};
		
		/**
		 * Initialize the widget.
		 */
		TextElementDynContentWidget.prototype.init = function () {

			// Create a ContentStatementElementWidget.
			this._widget = new ContentStatementElementWidget(this.textElement, {
				viewOptions: {
					widgetview : "viewer"
				}
			});
			this._widget.onChange(function () {
				// Show the save button if it's not already there.
				if (this._rightOptionCell.has(this._saveButton)) {
					this._rightOptionCell.append(this._saveButton);
				}
				this.dynContentModel.textElementChanged(this.textElement, this);
			}.bind(this));
			// Add the ContentStatementElementWidget to the wrapper.
			this._masterTextWidgetRow = $("<tr>", {"style" : "height: 100%;"});
			this._masterTextWidgetCell = $("<td>", {"style" : "height: 100%;"});
			this._wrapper.append(this._masterTextWidgetRow.append(this._masterTextWidgetCell.append(this._widget.getJQElement())));
			
			// Add a hidden row to show the header of a potential translation language.
			this._translationHeaderCell = $("<td>");
			this._translationHeaderRow = $("<tr>",{"class" : "TransHeaderRow"}).hide();
			this._wrapper.append(this._translationHeaderRow.append(this._translationHeaderCell));
			
			// Add a hidden row to show the widget of a potential translation language.
			this._transWidget;
			this._translationWidgetCell = $("<td>", {"style" : "height: 100%;"});
			this._translationWidgetRow = $("<tr>", {"style" : "height: 100%;"}).hide();
			this._wrapper.append(this._translationWidgetRow.append(this._translationWidgetCell));
			
			// Create a bottom bar to show additional options.
			this._leftOptionCell = $("<td>", {"class" : "LeftOption"});
			this._rightOptionCell = $("<td>", {"class" : "RightOption"});
			this._optionBar = $("<table>", {"class" : "OptionBar"}).append($("<tr>").append(this._leftOptionCell).append(this._rightOptionCell));
				
			// Add the save button.
			this._saveButton = $("<a>", {"class": "btn btn-default btn-mini btn-success"});
			this._saveButton.append($("<span>", {"class" : "icon icon-WCR_Save12 marginRight5 icon-normal", "style" : "color: #333;"}));
			this._saveButton.append($("<span>").text("Save"));
			this._saveButton.on("click", function (e) {
				this.dynContentModel.save(this.textElement);
			}.bind(this));
			if (this.textElement.isChanged()) {
				this._rightOptionCell.append(this._saveButton);
			}
			
			
			// Add the option bar to the wrapper.
			this._wrapper.append($("<tr>", {"class" : "OptionRow"}).append($("<td>").append(this._optionBar)));
			
			// Show the option to translate the statement.
			this._updateShowTranslationOption();
			
			// When the translation language changes ... update also the translation option.
			this.dynContentModel.on("translationLangChanged", function () {
				this._updateShowTranslationOption();
				this.hideTranslation();
			}.bind(this));
			
			// When a text element was changed from within a widget.
			this.dynContentModel.on("textElementChanged", function (textElement, fromWidget) {
				if (fromWidget == this) {
					return false;
				}
				if (textElement == this.textElement) {
					this._widget.update(textElement);
					this._rightOptionCell.empty();
					this._rightOptionCell.append(this._saveButton);
					return true;
				} else if (textElement == this.transTextElement) {
					this._transWidget.update(textElement);
					this._rightOptionCell.empty();
					this._rightOptionCell.append(this._saveButton);
					return true;
				}
			}.bind(this));
			
			// Disable the widget when save operation starts.
			this.dynContentModel.on("save.started", function (textElement, fromWidget) {
				// Disable the widgets and buttons.
				if (this._widget) {
					this._widget.disable();
				}
				if (this._transWidget) {
					this._transWidget.disable();
				}
				if (this._saveButton) {
					this._saveButton.prop('disabled', true);
				}
			}.bind(this));
			
			// Enable the widget when save operation finishes.
			this.dynContentModel.on("save.finished", function (textElement, fromWidget) {
				// Disable the widgets and buttons.
				if (this._widget) {
					this._widget.enable();
				}
				if (this._transWidget) {
					this._transWidget.enable();
				}
				if (this._saveButton) {
					this._saveButton.prop('disabled', false);
				}
			}.bind(this));
			
		};
		
		/**
		 * Update the widget with a different text element.
		 * 
		 * @param textElement
		 */
		TextElementDynContentWidget.prototype.update = function (textElement) {
			// Update the textElement.
			this.textElement = textElement;
			this.transTextElement = undefined;
			// Re-init the widget.
			this._wrapper.empty();
			this.init();
		};
		
		/**
		 * Show another translation language allong the master language.
		 * 
		 * @param langCode
		 */
		TextElementDynContentWidget.prototype.showTranslation = function (transTextElement) {
			
			// Check if this is a master language.
			if (!this.textElement.isMaster()) {
				return;
			}
			
			// If we're already showing a translation.
			if (this.showingTranslation) {
				return;
			}
			
			// If this is a new transTextElement.
			if (transTextElement != this.transTextElement) {
				this.transTextElement = transTextElement;
				
				// Create a ContentStatementElementWidget for this translation.
				this._transWidget = new ContentStatementElementWidget(transTextElement, {
					viewOptions: {
						widgetview : "viewer"
					}
				});
				this._transWidget.onChange(function () {
					// Show the save button if it's not already there.
					if (this._rightOptionCell.has(this._saveButton)) {
						this._rightOptionCell.append(this._saveButton);
					}
					this.dynContentModel.textElementChanged(this.transTextElement, this);
				}.bind(this));
				this._translationWidgetCell.empty();
				this._translationWidgetCell.append(this._transWidget.getJQElement());
				
				// Create a header for this translation.
				languageService.getLanguages(this.textElement.parentStatement.projectId, function(projectMasterLangs, projectTranslationLangs, allMasterLangs, allTranslationLangs) {
					// Get the translang object.
					var language = languageUtils.getLangForCode(allTranslationLangs, transTextElement.language);

					// Generate the language header.
					var transHeader = $("<span>", {"class": "TransHeader"});
					transHeader.append($("<img>", {"src": "./images/country_flags/"+language.langFlag, "style" : "margin-top: -3px; margin-right: 5px;"}));
					transHeader.append($("<span>", {"style":"font-weight: bold"}).text(language.langDesc));
					
					// Add it to the row
					this._translationHeaderCell.empty();
					this._translationHeaderCell.append(transHeader);
				}.bind(this));
				
			} else {
				// This translation language is already set, just show it again.
			}
			
			// Set the variable on true
			this.showingTranslation = true;
			
			// Make room to show the translation.
			this._resizeToShowTranslation();
			
			// Apply the correct CSS to fill.
			this._translationWidgetCell.attr("style", "height: 45%;)");
			this._translationWidgetRow.attr("style", "height: 45%;");
			this._masterTextWidgetRow.attr("style", "height: 45%;");
			this._masterTextWidgetCell.attr("style", "height: 45%;");
			
			// Show/Hide the necessary elements.
			this._translationHeaderRow.show();
			this._translationWidgetRow.show();
			this._hideTransButton.show();
			this._showTransButton.hide();
		};
		
		/**
		 * Hide the translation language for this master language.
		 * 
		 * @param langCode
		 */
		TextElementDynContentWidget.prototype.hideTranslation = function () {
			
			// Check if this is a master language.
			if (!this.textElement.isMaster()) {
				return;
			}
			
			// If we're already hiding the translation.
			if (!this.showingTranslation) {
				return;
			}
			
			// Set the variable on false.
			this.showingTranslation = false;
			
			// Clear room to show only the master text.
			this._resizeToHideTranslation();
			
			// Apply the correct CSS to fill.
			this._translationWidgetCell.attr("style", "height: 100%;");
			this._translationWidgetRow.attr("style", "height: 100%;");
			this._masterTextWidgetRow.attr("style", "height: 100%;");
			this._masterTextWidgetCell.attr("style", "height: 100%;");
			
			// Show/Hide the necessary elements.
			this._translationHeaderRow.hide();
			this._translationWidgetRow.hide();
			this._hideTransButton.hide();
			this._showTransButton.show();
		};
		
		/**
		 * Resize the containing popup so that there's room to show the translation.
		 */
		TextElementDynContentWidget.prototype._resizeToShowTranslation = function () {
			// Only make vertically room.
			var curHeight = this.popup.element.height();
			// How many of that space is taken up by the master text?
			var masterTextHeight = Math.max(curHeight-(26+36+2), 78);
			// We'll have to be carefull to not make the widget to big, therefore ceil the masterTextHeight to 120px;
			var ceiledMasterTextHeight = Math.min(120, masterTextHeight);
			// The new height should be twice the height of the master text + space of the translation header.
			var newHeight = 26 + ceiledMasterTextHeight + 26 + ceiledMasterTextHeight + 36 + 2;
			// Create the new size.
			var newSize = new ShapeUtils.Size(this.popup.element.width()+2, newHeight);
			// Calculate the new min size.
			this.popup.minHeight = this.popup.minHeight + 78 + 26;
			var minSize = new ShapeUtils.Size(this.popup.minWidth, this.popup.minHeight);
			
			// Set the new size.
			this.popup.setMinSize(minSize);
			this.popup.setSize(newSize);
			var ui = {
					position: {
						left: this.popup.element.position().left,
						top: this.popup.element.position().top
					},
					size: {
						width: newSize.width,
						height: newSize.height
					}
				};
			this.popup._fireResized(ui, false);
		};
		
		/**
		 * Resize the containing popup so that room will free up since the translation no longer needs to be shown.
		 */
		TextElementDynContentWidget.prototype._resizeToHideTranslation = function () {
			// Only make vertically room.
			var curHeight = this.popup.element.height();
			// How many space do both texts take up?
			var bothTextHeight = Math.max(curHeight-(26+26+36), 2*78);
			// We'll have to be carefull to not make the widget to big, therefore ceil the masterTextHeight to 120px;
			var masterTextHeight = Math.floor(120, bothTextHeight/2);
			// The new height should be twice the height of the master text + space of the translation header.
			var newHeight = 26 + masterTextHeight + 36 + 2;
			// Create the new size.
			var newSize = new ShapeUtils.Size(this.popup.element.width()+2, newHeight);			
			// Calculate the new min size.
			this.popup.minHeight = 144;
			var minSize = new ShapeUtils.Size(this.popup.minWidth, this.popup.minHeight);			
			// Set the new size.
			this.popup.setMinSize(minSize);
			this.popup.setSize(newSize);
			var ui = {
					position: {
						left: this.popup.element.position().left,
						top: this.popup.element.position().top
					},
					size: {
						width: newSize.width,
						height: newSize.height
					}
				};
			this.popup._fireResized(ui, false);
		};
		
		
		/**
		 * This will possibly generate a button to display the text of current translation language along with the current master text of this text element.
		 * (Will only show when this is a popup for a master text).
		 */
		TextElementDynContentWidget.prototype._updateShowTranslationOption = function () {
			
			if (this.textElement.isMaster()) {
				// Get the current translation language.
				var curTransLang = this.dynContentModel.getCurrentTransLangCode();
				
				// Check if this master statement has the translation attached of the current translation language.
				var transTextElement = undefined;
				for (var i=0; i < this.textElement.parentStatement.element.length; i++) {
					var aTextElement = this.textElement.parentStatement.element[i];
					if (aTextElement.language == curTransLang && !aTextElement.isMaster()) {
						transTextElement = aTextElement;
						break;
					}
				}
				
				// If such a translation was found, generate a button to show it in this popup.
				if (transTextElement) {
					// Generate the language header.
					languageService.getLanguages(this.textElement.parentStatement.projectId, function(projectMasterLangs, projectTranslationLangs, allMasterLangs, allTranslationLangs) {
						// Find the language object for this text element.
						
						var language = languageUtils.getLangForCode(allTranslationLangs, curTransLang);
						
						// Generate the show translate button.
						this._showTransButton = $("<a>", {"class": "btn btn-default btn-mini"});
						this._showTransButton.append($("<span>").text("Show"));
						this._showTransButton.append($("<img>", {"src": "./images/country_flags/"+language.langFlag, "style" : "margin-top: -2px; margin-left: 5px;"}));
						
						// Generate the hide translate button.
						this._hideTransButton = $("<a>", {"class": "btn btn-default btn-mini"});
						this._hideTransButton.append($("<span>").text("Hide"));
						this._hideTransButton.append($("<img>", {"src": "./images/country_flags/"+language.langFlag, "style" : "margin-top: -2px; margin-left: 5px;"}));
						this._hideTransButton.hide();
						
						// When the show translate button is clicked.
						this._showTransButton.on("click", function () {
							this.showTranslation(transTextElement);
						}.bind(this));
						
						// When the show translate button is clicked.
						this._hideTransButton.on("click", function () {
							this.hideTranslation();
						}.bind(this));
						
						// Add it.
						this._leftOptionCell.empty();
						this._leftOptionCell.append(this._showTransButton);
						this._leftOptionCell.append(this._hideTransButton);
						
						//if there was already a translation that was opened on canvas, then close that and show the new language that was clicked from filter
						if(this.showingTranslation){
							this.hideTranslation();		
							this.showTranslation(transTextElement);
						}
							
					}.bind(this));
				} else {
									
					this._leftOptionCell.empty();
				}
			} else {
				this._leftOptionCell.empty();
			}
			
		};
		
		/**
		 * Focus on this popup widget.
		 */
		TextElementDynContentWidget.prototype.focus = function (focus) {
			this._widget.focus(focus);
		};
		
		/**
		 * 
		 * @returns
		 */
		TextElementDynContentWidget.prototype.onChange = function (callback) {
			this._widget.onChange(callback);
		};
		
		/**
		 * 
		 * @returns
		 */
		TextElementDynContentWidget.prototype.getJQElement = function () {
			return this._wrapper;
		};
		
		return TextElementDynContentWidget;
});

define('web-viewer/textContent/DynContentPopup',["web-viewer/layout/component/Popup",
		"web-viewer/layout/component/MiniButton",
		"./TextElementDynContentWidget",
		"packaging-content-management/text-content-services/languageService",
		"packaging-content-management/text-content-utils/LanguageUtils",
		"jquery",
		"underscore"
	],
	function (Popup,
		MiniButton,
		TextElementDynContentWidget,
		languageService,
		languageUtils,
		$,
		_) {
		"use strict";

		/**
		 * Create an annotation popup. 
		 * Initial (re)paint should be called by caller
		 * 
		 * @exports DynContentPopup
		 * @memberof TextContentModule
		 * @class
		 * The dyncontent annotation popup controls the popup for dyncontent annotation details
		 * @constructor	  
		 * @augments Layout.Popup
		 * @param {Core.ViewerFacade} f - Viewerfacade
		 * @param {DynContentModel} dynContentModel - The dynamic content model
		 * @param {String} id - Id of the popup used for its private channel
		 * @param {List<DynContentAnnotation>} dynContentAnnotations - DynContentAnnotations this popup belongs to
		 * @param {JQueryObj} container - The container where the popup should stay in
		 */
		var DynContentPopup = function (f, id, dynContentModel, dynContentPopupModel, textElement, container) {
			this.f = f;
			Popup.call(this, f, id, container);

			this.dynContentPopupModel = dynContentPopupModel;
			this.dynContentModel = dynContentModel;
			this.textElement = textElement;
			this.element.addClass("DynContentPopup");

			// By default, the popup is not visible.
			this.visible = false;

			// The text element widget
			this._textElementWidget = undefined;

			this.minWidth = 300;
			this.minHeight = 144;
		};

		DynContentPopup.prototype = Object.create(Popup.prototype);
		DynContentPopup.prototype.constructor = DynContentPopup;

		/**
		 * Initialize the popup, create needed DOM events, listeners
		 * 
		 * @private
		 * @override
		 */
		DynContentPopup.prototype._initialize = function () {
			if (!this._isInitialized) {
				Popup.prototype._initialize.call(this);

				// Generate the language header.
				languageService.getLanguages(this.textElement.parentStatement.projectId, function (projectMasterLangs, projectTranslationLangs, allMasterLangs, allTranslationLangs) {
					// Find the language object for this text element.
					var language;
					if (this.textElement.isMaster()) {
						language = languageUtils.getLangForCode(allMasterLangs, this.textElement.language);
					} else {
						language = languageUtils.getLangForCode(allTranslationLangs, this.textElement.language);
					}

					// Generate the language header.
					var langHeader = $("<span>", {
						"class": "TransHeader"
					});
					langHeader.append($("<img>", {
						"src": "./images/country_flags/" + language.langFlag,
						"style": "margin-top: -3px; margin-right: 5px;"
					}));
					langHeader.append($("<span>", {
						"style": "font-weight: bold"
					}).text(language.langDesc));

					// Add it to the header
					this.header.append(langHeader);
				}.bind(this));

				// Create the TextElementDynContentWidget
				this._textElementWidget = new TextElementDynContentWidget(this.dynContentModel, this.dynContentPopupModel, this.textElement);
				this._outerContent.empty();
				this._outerContent.append(this._textElementWidget.getJQElement());

				// Initial paint
				DynContentPopup.prototype.repaint.call(this);
			}
		};

		/**
		 * Update the popup with a new textElement.
		 * 
		 * @param textElement
		 */
		DynContentPopup.prototype.update = function (textElement) {

			// Only update if a different text element is passed.
			if (textElement == this.textElement) {
				return;
			}

			// Set the new text element.
			this.textElement = textElement;
			if (this._textElementWidget) {
				this._textElementWidget.update(textElement);
			}
		};

		/**
		 * Repaint the popup.
		 */
		DynContentPopup.prototype.repaint = function () {
			if (this._isInitialized) {

			}
		};

		/**
		 * Set focus on or off on the widget.
		 */
		DynContentPopup.prototype.focus = function (focus) {
			if (this._textElementWidget) {
				this._textElementWidget.focus(focus);
			}
		};

		/**
		 * Remove this popup
		 * @override
		 */
		DynContentPopup.prototype.remove = function () {
			Popup.prototype.remove.call(this);
		};

		return DynContentPopup;
	});

define('web-viewer/textContent/DynContentPopupModel',["web-viewer/viewport/model/ViewportPopupModel",
		"web-viewer/textContent/DynContentPopup",
		"utils/ShapeUtils",
		"underscore"
	],
	function (ViewportPopupModel,
		DynContentPopup,
		ShapeUtils,
		_) {
		"use strict";

		/**
		 * Create a new DynContent Popup Model.
		 * @exports DynContentPopupModel
		 * @memberof TextContentModule
		 * @class
		 * The dyncontent popup model is responsible for a dyncontent popup and its content.
		 * It binds the events of the popup and reacts on them.
		 * @augments Viewport.ViewportPopupModel
		 * @constructor
		 * @param {Core.ViewerFacade} f - Viewerfacade
		 * @param {String} id - Id of the popup model.
		 * @param {DynContentAnnotation} dynContentAnnotation - The dycontent annotation this model belongs to.
		 * @param {DynContentModel} dynContentModel - Model of the dyncontents.
		 */
		var DynContentPopupModel = function (f, id, dynContentAnnotations, dynContentModel) {
			ViewportPopupModel.call(this, f, id);

			this.dynContentAnnotations = dynContentAnnotations;
			this.targetedDynContentAnnotation = dynContentAnnotations[0];
			this.dynContentModel = dynContentModel;
			this.popup = null;

			// When the content of the model changed.
			this.dynContentModel.on("modelContentChanged", function () {
				if (this.popup) {
					this.popup.update(this.targetedDynContentAnnotation.getDynContent().getTextElement());
				}
			}.bind(this));
		};

		DynContentPopupModel.prototype = Object.create(ViewportPopupModel.prototype);
		DynContentPopupModel.prototype.constructor = DynContentPopupModel;

		/**
		 * Repaint the popup
		 */
		DynContentPopupModel.prototype.repaint = function (refDynContentAnnotation) {
			if (this.popup != null) {
				this.popup.repaint(refDynContentAnnotation);
			}
		};

		/**
		 * Set the targeted annotation.
		 */
		DynContentPopupModel.prototype.setTargetAnnotation = function (annotation) {
			this.targetedDynContentAnnotation = annotation;
		};

		/**
		 * Set the targeted annotation.
		 */
		DynContentPopupModel.prototype.focus = function (focus) {
			this.popup.focus(focus);
		};

		/**
		 * @override
		 * This method saves the origin in the annotation.
		 * @param {ShapeUtils.Point} point - Point (origin) of the popup in world coordinates
		 */
		DynContentPopupModel.prototype.onDragged = function (worldPoint) {
			var point = worldPoint.copy();
			point.y -= this.targetedDynContentAnnotation.popuprect.size.height;

			if (!this.targetedDynContentAnnotation.popuprect.origin.equals(point)) {
				this.targetedDynContentAnnotation.popuprect.origin = point;
			}
		};

		/**
		 * @override
		 * This method saves the rectangle in the annotation.
		 * @param {ShapeUtils.Rect} rect - Rectangle (origin/size) of the popup in world coordinates
		 * @param {Boolean} autoAdapted - True if the event is fired because the content was auto adapted (e.g. from a makeMeVisible trigger from an element in the content), false otherwise
		 */
		DynContentPopupModel.prototype.onResized = function (worldRect, autoAdapted) {
			var copy = worldRect.copy();
			// compensate for origin in left bottom corner
			copy.origin.y -= copy.size.height;

			if (!this.targetedDynContentAnnotation.popuprect.rectEquals(copy)) {
				this.targetedDynContentAnnotation.popuprect = copy;
			}
		};

		/**
		 * Get the DynContent popup
		 * @param {ViewportLayer} viewportLayer - The layer where the popups should be shown.
		 * @returns {DynContentPopup} The dyncontent popup of this model.
		 */
		DynContentPopupModel.prototype.getPopup = function (viewportLayer) {
			var container = viewportLayer.getCanvas();
			if (this.popup === null) {
				this._createPopup(container);
			} else {
				this.popup.setContainment(container);
			}
			return this.popup;
		};

		/**
		 * Create a dyncontent annotation popup {DynContentPopup} for the given layer
		 * and bind all events.
		 * @param {JQueryObj} container - Container for the popup
		 */
		DynContentPopupModel.prototype._createPopup = function (container) {

			// Get the text element.
			var textElement = this.targetedDynContentAnnotation.getDynContent().getTextElement();
			// Create the popup.
			this.popup = new DynContentPopup(this.f, this.id, this.dynContentModel, this, textElement, container);

			// we want the resized in world rect instead of scene rect, so don't bind them here, but in the layer, and let the layer call the model functions
			this.popup.on("closed", _.bind(this.onClosed, this));
			this.popup.on("clicked", _.bind(this.onClicked, this));
		};

		/**
		 * Method called when the model is destroyed.
		 * Useful for some cleanup in the popup/content
		 */
		DynContentPopupModel.prototype.onDestroy = function () {
			if (this.popup != null) {
				this.popup.remove();
			}
		};

		/**
		 * This function is bind to the "closed" popup event and called when the popup is closed.
		 */
		DynContentPopupModel.prototype.onClosed = function () {
			this.popup.focus(false);
			this.dynContentModel.unselectAllDynContents(false, true);
		};

		/**
		 * This function is bind to the "clicked" dyncontent annotation popup event and called when the popup is clicked.
		 */
		DynContentPopupModel.prototype.onClicked = function () {
			//this.dynContentModel.selectDynContent(this.targetedDynContentAnnotation);
		};

		/**
		 * Fire that a dyncontent annotation has changed
		 * @param {String} what - indicate what was changed
		 * @returns {Boolean} True when fired, false otherwise
		 */
		DynContentPopupModel.prototype._fireDynContentAnnotationChanged = function (what) {
			for (var i = 0; i < this.dynContentAnnotations.length; i++) {
				this.dynContentAnnotations[i].setModified(what);
			}
			this.dynContentModel.modelChanged();
		};

		/**
		 * Get the origin of the popup in world coordinates
		 * @returns {ShapeUtils.Point} - The origin of the popup in world coordinates
		 */
		DynContentPopupModel.prototype.getWorldOrigin = function () {
			// origin in left bottom corner
			return new ShapeUtils.Point(this.targetedDynContentAnnotation.popuprect.left(), this.targetedDynContentAnnotation.popuprect.bottom());
		};


		/**
		 * Get the size of the popup in world coordinates
		 * @returns {ShapeUtils.Size} - The size of the popup in world coordinates
		 */
		DynContentPopupModel.prototype.getWorldSize = function () {
			return this.targetedDynContentAnnotation.popuprect.size;
		};

		return DynContentPopupModel;

	});

define('web-viewer/textContent/DynContentPopupsLayerModel',["web-viewer/viewport/model/ViewportLayerModel",
		"web-viewer/textContent/DynContentPopupModel",
		"utils/ShapeUtils",
		"underscore"
	],
	function (ViewportLayerModel,
		DynContentPopupModel,
		ShapeUtils,
		_) {
		"use strict";

		/**
		 * @exports DynContentPopupsLayerModel
		 * @memberof TextContentModule
		 * @class
		 * Create a new dynamic content layer model for dynamic content popups
		 * @constructor
		 * @param {Core.ViewerFacade} f - Viewerfacade
		 * @param {String} ID - Id of the layer
		 * @param {DynContentModel} dynContentModel - Model of the dynamic contents.
		 */
		var DynContentPopupsLayerModel = function (f, ID, dynContentModel) {
			ViewportLayerModel.call(this, f, ID);

			this.f = f;
			this.dynContentModel = dynContentModel;
			
			// New annotations received
			this._createModels();
			this.dynContentModel.on("modelContentChanged", _.bind(function () {
				this.removeAllPopupModels();
				this._createModels();
			}, this));
			
			// DynContent model changed events
			this.dynContentModel.on("modelChanged", _.bind(function () {
				var page = this.dynContentModel.getDynContentPage();
				if (page !== null && page !== undefined) {
					this.resetTransformation();
					this.getTransform().scale(1, -1);
					var rotation = page.getRotation();
					var worldBounds = this.f.getModule("viewport").getWorldBounds();
					this.calculateTransformation(worldBounds, page.getPageSize(), new ShapeUtils.Size(), rotation, page.isFlipped());
				}
				this.modelChanged();
			}, this));

			/*
			 * When dynamic content annotations are selected.
			 */
			this.dynContentModel.on("selectedDynContents", _.bind(function (dynamicContentAnnotations, isDirectSelect) {
				// Only show a popup when annotations are selected for a single dynContent object.
				var singleDynContentSelected = false;
				if (dynamicContentAnnotations && dynamicContentAnnotations.length > 0) {
					var selectedDynContent =  dynamicContentAnnotations[0].getDynContent();
					singleDynContentSelected = true;
					for (var i=1; i < dynamicContentAnnotations.length; i++) {
						if (selectedDynContent != dynamicContentAnnotations[i].getDynContent()) {
							singleDynContentSelected = false;
							break;
						}
					}
				}
				// If a single dynamic content element is selected.
				if (singleDynContentSelected) {
					// The selected annotation.
					var annotation = dynamicContentAnnotations[0];
					var popupModel = this.getPopupModel(annotation.getDynContent().getId());
					popupModel.setTargetAnnotation(annotation);
					popupModel.show();
					this._updateModels();
					this.modelChanged();
				}
			}, this));

			/*
			 * When dynamic content annotations are deselected.
			 */ 	
			this.dynContentModel.on("unselectedDynContents", _.bind(function (dynamicContentAnnotations, force) {
				// Hide the popups for all these dynamic contents.
				for (var i=0; i < dynamicContentAnnotations.length; i++) {
					var annotation = dynamicContentAnnotations[i];
					var popup = this.getPopupModel(annotation.getDynContent().getId());
					popup.focus(false);
					popup.hide();
				}
				this.modelChanged();
			}, this));
		};
		
		DynContentPopupsLayerModel.prototype = Object.create(ViewportLayerModel.prototype);
		DynContentPopupsLayerModel.prototype.constructor = DynContentPopupsLayerModel;
		
		
		/**
		 * Update the state of the models based on the options.
		 * Does the show/hide logic for annotation popups
		 */
		DynContentPopupsLayerModel.prototype._updateModels = function () {
			
		};
		
		/**
		 * Create the annotation popup models
		 */
		DynContentPopupsLayerModel.prototype._createModels = function () {
			// Get all the textElements for all the dynamic contents.
			var textElements = [];
			_.each(this.dynContentModel.getDynContents(), function (dynContentAnnotation) {
				var textElement = dynContentAnnotation.getDynContent().getTextElement();
				if ($.inArray(textElement, textElements) < 0) {
					textElements.push(textElement);
				}
			});
			// Create a popup for each textElement and it's linked annotations.
			_.each(textElements, function (textElement) {
				var annotations = this.dynContentModel.getDynContentAnnotationsForTextElement(textElement);
				this._createModel(annotations);
			}, this);
			this._updateModels();
			this.modelChanged();
		};
		
		/**
		 * Create the annotation popup model for the given annotation
		 * @param {Annotation} annotation - Annotation to create the model of
		 */
		DynContentPopupsLayerModel.prototype._createModel = function (dynContentAnnotations) {
			// This will create the appropriate popup (AnnotationPopupNew vs AnnotationPopupExisting)
			var popupModel = new DynContentPopupModel(this.f, dynContentAnnotations[0].getDynContent().getId(), dynContentAnnotations, this.dynContentModel);
			this.addPopupModel(popupModel);
			// Hide the popupModels by default ...
			popupModel.hide();
		};
		
		return DynContentPopupsLayerModel;
		
});

define('web-viewer/textContent/DynContentTool',["underscore",
		"web-viewer/viewport/tools/AbstractTool",
		"web-viewer/textContent/DynContentAnnotation",
		"utils/ShapeUtils",
		"web-viewer/utils/DrawUtils",
		"web-viewer/utils/ViewerUtils",
		"web-viewer/core/CoreConstants"
	],
	function (_,
		AbstractTool,
		DynContentAnnotation,
		ShapeUtils,
		DrawUtils,
		ViewerUtils,
		CoreConstants) {
		"use strict";
		
		/**
		 * Construct a new Dynamic Content Tool
		 * @exports DynContentTool
		 * @memberof TextContentModule
		 *
		 * @class Tool to select and edit annotations
		 * @constructor
		 * @param {Core.ViewerFacade} f - Viewer facade
		 * @param {AnnotationModel} annotationModel - Annotation model
		 */
		var DynContentTool = function (f, dynContentModel) {
			AbstractTool.call(this, f, CoreConstants.TOOLS.DYNCONTENT);

			/**
			 * Different states constants
			 */
			this._defaultState = "defaultState";
			this._hoverState = "hoverState";
			
			this._targetDynContent = null;
			this._dynContentModel = dynContentModel;
			
			/**
			 * Variable to keep track of where interaction started
			 */
			this._touchDown = null;
			
			/**
			 * The default state
			 */
			this.registerState(this._defaultState, (function (that) {
				return {
					"mousemove": function (e) {
						var l = that._getDynContentLayer(e);
						if (!l || !l.isVisible()) {
							return;
						}

						that._verifyObjects(e, l);
						if (that._targetDynContent || that._anchor) {
							that.requestActivation();
						}
					},
					"mouseup": function (e) {

					},
					"mousedown": function (e) {
						var l = that._getDynContentLayer(e);
						if (!l) {
							return;
						}

						dynContentModel.unselectAllDynContents();
						e.originalEvent.preventDefault();
					},
					"mouseout": function (e) {
						if (that.isActive()) {
							f.emit("toolManager.request.deactivation", that);
						}
					},
					"mouseleave": function () {
						if (that.isActive()) {
							f.emit("toolManager.request.deactivation", that);
						}
					},
					"contextmenu": function (e) {
						e.preventDefault();
						e.stopPropagation();
						return false;
					}
				};
			}(this)));

			/**
			 * The hover state
			 */
			this.registerState(this._hoverState, _.defaults(function (that) {
				return {
					"mousedown": function (e) {
						if (e.which === 1) {
							that._verifyObjects(e);
							dynContentModel.selectDynContent(that._targetDynContent);
						}
						e.originalEvent.preventDefault();
					},
					"mousemove": function (e) {
						that._verifyObjects(e);
						that.cursorShouldChange();
					}
				};
			}(this), this.getState(this._defaultState)));

			this._override("initialize", _.bind(function (widget, toolManager) {
				this._super(widget, toolManager);
				this.changeState(this._defaultState);
				this.registerPassiveEvents(["mousemove", "mousedown"]);
				this.registerActiveEvents(["mousemove", "mousedown", "mouseup", "mouseout", "mouseleave", "contextmenu"]);
			}, this));

			this._override("activate", _.bind(function (interruptRequest) {
				this._super(interruptRequest);
				this.changeState(this._hoverState);
			}, this));

			this._override("deactivate", _.bind(function (interruptRequest) {
				this._super(interruptRequest);
				this.changeState(this._defaultState);
			}, this));

			this.f = f;
		};
		
		// inherit
		DynContentTool.prototype = Object.create(AbstractTool.prototype);
		// correct the constructor pointer 
		DynContentTool.prototype.constructor = DynContentTool;

		
		/**
		 * Calculate the default cursor
		 * 	- when an anchor is selected:
		 * 		- line or arrow annotation: move-cursor
		 * 		- show correct resize-cursor
		 *  - when an annotation is selected
		 *  	- show move-cursor unless the annotation can't be moved -> then show a not-allowed cursor
		 *  - when nothing is selected: show select arrow
		 */
		DynContentTool.prototype.getDefaultCursor = function () {
			if (this._anchor) {
				return this._cursorForAnchor();
			} else if (this.getCurrentStateName() == this._draggingState) {
				if (this._dynContentModel.canChangeAnnotation(this._targetDynContent)) {
					return "cursor-moveObject";
				} else {
					return "cursor-notAllowed";
				}
			} else {
				return "cursor-select";
			}
		};

		/**
		 * Deactivate the current Dynamic Content tool
		 * @param that
		 * @param e
		 */
		DynContentTool.prototype._deActivate = function (e) {
			if (this.isActive()) {
				this.changeState(this._defaultState);
				this.f.emit("toolManager.request.deactivation", this);
			}
		};
		
		/**
		 * @returns {String} the cursor for the anchor state
		 */
		DynContentTool.prototype._cursorForAnchor = function () {
			return "cursor-select";
		};
		
		/**
		 * Get the dynamic content layer based on a point
		 * @param {String|Event} either the name of the view or an event that will calculate the view of the dynamic content layer
		 * @returns {ViewportLayer} Annotations layer of null if not found
		 */
		DynContentTool.prototype._getDynContentLayer = function (e) {
			var v = this._widget.requestView(e);
			if (v) {
				var l = v.getLayer("Dyncontent");
				if (l) {
					return l;
				}
			}
			return null;
		};
		
		/**
		 * Get the closest linework from a given point
		 * @param {Event} e - mouse event to calcuate the point to
		 * @param {ViewportLayer} [layer] - if the layer was already fetched
		 * @returns {LineworkWrapper} the closest lineworkwrapper to a certain point
		 */
		DynContentTool.prototype._getClosestLineworkFromPoint = function (e, layer) {
			var l = layer;
			if (!l) {
				l = this._getDynContentLayer(e);
			}

			if (l) {
				return l.getClosestLineworkFromPoint(this.getScenePoint(e));
			} else {
				return null;
			}
		};

		/**
		 * Converts a point from an event to the world, makes sure the point is containt in the document page
		 * @param {Event} e - mouse event
		 * @returns {ShapeUtils.Point} point in world coordinates
		 */
		DynContentTool.prototype._getWorldPoint = function (e) {
			var activePage = this._dynContentModel.getDynContentPage();
			var worldPoint = this.keepPointWithinWorldBounds(this.getWorldPoint(e), false);

			//convert the coordinate system from world coordinates to annotation coordinates
			worldPoint.y = activePage.getPageSize().height - worldPoint.y;

			var origin = activePage.getDocumentPageOrigin();

			worldPoint.offset(new ShapeUtils.Size(-origin.x, origin.y));

			var trimboxSize = activePage.getTrimboxRect().size;
			var missingHeight = 0; //pages with wrong XMP-data (page size) can have a missing part in the height, annotationlocation needs to be compensated for this
			if (trimboxSize.width !== 0 && trimboxSize.height !== 0) {
				missingHeight = activePage.getPageSize().height - (trimboxSize.height + activePage.getMargins().top + activePage.getMargins().bottom);
			}

			var margin = activePage.getMargins();
			if (activePage.isReaderSpread() && activePage.get("isRightPage")) { //check if the active page is on the right side of reader spread
				margin.left = -margin.left;
			}
			worldPoint.offset(new ShapeUtils.Size(-margin.left, -margin.bottom - missingHeight));

			return worldPoint;
		};

		/**
		 * Verify the current situation, so calcutate where the mouse is for the moment on the canvas
		 * @param {Event} e - mouse event to calcuate the point to
		 * @param {ViewportLayer} [layer] - if the layer was already fetched
		 */
		DynContentTool.prototype._verifyObjects = function (e, layer) {
			if (!layer) {
				layer = this._getDynContentLayer(e);
				if (!layer) {
					return;
				}
			}
			var object = this._getClosestLineworkFromPoint(e, layer);

			this._targetDynContent = null;
			this._anchor = null;
			if (object) {
				if (object instanceof DynContentAnnotation) {
					this._targetDynContent = object;
				} else if (object.anchor) {
					this._anchor = object;
					this._targetDynContent = object.linework;
				}
			}
			if (!object && !this._targetDynContent) {
				this._deActivate(e);
			}
		};
		
		
		return DynContentTool;
});

define('web-viewer/textContent/DynContentToolbarComponent',["component/toolbar/ToolbarComponent",
		"component/toolbar/ButtonGroup",
		"web-viewer/layout/component/ViewerToolbarButton",
		"web-viewer/layout/component/SubtoolbarButton",
		"web-viewer/layout/LayoutConstants",
	],
	function (ToolbarComponent,
		ButtonGroup,
		ViewerToolbarButton,
		SubtoolbarButton,
		LayoutConstants) {
		"use strict";

		
		/**
		 * @exports DynContentToolbarComponent
		 * @memberof DynContentModule
		 * @constructor
		 * @param {Core.ViewerFacade} f - Viewer facade
		 * @param {DynContentModel} dynContentModel - Dynamic Content model
		 * @param {DynContentTool} dynContentTool - The dynamic content tool
		 */
		var DynContentToolbarComponent = function (f, dynContentModel, dynContentTool) {
			ToolbarComponent.call(this, LayoutConstants.ToolbarComponent.DYNCONTENT, f);
			this.f = f;
			this.dynContentModel = dynContentModel;
			/**
			 * Group: annotate
			 */
			var dynContentGroup = new ButtonGroup("DynContentGroup");
			dynContentGroup.setTitle(f.i18n, "viewer", "DynContentBtnGroupTitle");
			this.addButtonGroup(dynContentGroup);
			
			// DynContent ViewerToolbarButton
			var toggleDynContentButton = new ViewerToolbarButton(f, "ToggleDynContentButton", f.getOption("PREFERENCES_DYNCONTENTTOOLS_VISIBLE") ? "icon-WCR_Annotation-DC-Hide" : "icon-WCR_Annotation-DC-Show", true, function () {
				dynContentModel.toggleVisibility();
			});
			// set initial tooltip
			toggleDynContentButton.setTooltip("viewer", f.getOption("PREFERENCES_DYNCONTENTTOOLS_VISIBLE") ? "HideDynContentsBtnTT" : "ShowDynContentsBtnTT");
			// react on preference changes
			f.getCurrentUser().on("options.changed.PREFERENCES_DYNCONTENTTOOLS_VISIBLE", function (oldValue, newValue) {
				if (newValue) {
					toggleDynContentButton.setTooltip("viewer", "HideDynContentsBtnTT");
					toggleDynContentButton.setIcon("icon-WCR_Annotation-DC-Hide");
				} else {
					toggleDynContentButton.setTooltip("viewer", "ShowDynContentsBtnTT");
					toggleDynContentButton.setIcon("icon-WCR_Annotation-DC-Show");
				}
			});
			dynContentGroup.addButton(toggleDynContentButton);
		};
		
		DynContentToolbarComponent.prototype = Object.create(ToolbarComponent.prototype);
		DynContentToolbarComponent.prototype.constructor = DynContentToolbarComponent;
		DynContentToolbarComponent.toolbarName = LayoutConstants.ToolbarComponent.DYNCONTENT;
		
		
		return DynContentToolbarComponent;
});
		

define('web-viewer/textContent/TextContentModule',["web-viewer/core/AbstractModule",
		"web-viewer/utils/ListenerUtil",
		"utils/ShapeUtils",
		"web-viewer/textContent/DynContentWidget",
		"web-viewer/layout/component/ModalPopup",
		"web-viewer/textContent/DynContentModel",
		"web-viewer/textContent/DynContentLayerModel",
		"web-viewer/textContent/DynContentPopupsLayerModel",
		"web-viewer/textContent/DynContentTool",
		"web-viewer/textContent/DynContentToolbarComponent",
		"packaging-content-management/ContentConstants",
		"underscore",
		"packaging-content-management/text-content-services/languageService",
		"packaging-content-management/text-content-services/elementTypeService"
	],
	function (AbstractModule,
		ListenerUtil,
		ShapeUtils,
		DynContentWidget,
		ModalPopup,
		DynContentModel,
		DynContentLayerModel,
		DynContentPopupsLayerModel,
		DynContentTool,
		DynContentToolbarComponent,
		ContentConstants,
		_,
		languageService,
		elementTypeService) {

		"use strict";
		/**
		 * This module is responsible for adding the text content widget
		 *
		 * @exports TextContentModule
		 * @constructor
		 * @augments Core.AbstractModule
		 * @param {Core.ViewerFacade} f
		 * @param {Boolean} silenced
		 */
		var TextContentModule = function (f, silenced) {
			AbstractModule.call(this, f, silenced, "dyncontent");
			ListenerUtil.mixin(this, f);

			/**
			 * Layer Models
			 */
			this.layerModels = {};

			/**
			 * The dynamic content model
			 */
			this.dynContentModel = new DynContentModel(f, 'dyncontent');

			/**
			 * Component: dynamic content
			 */
			this._dynContentComponent = null; // set the annotation tool bar component as unavailable


			this.f.getDocumentController().whenActivePageChanged(_.bind(this._whenActivePageChanged, this));

			/*
			 * What to do when we need to zoom to certain dyn contents.
			 */
			this.on("zoomToDynContents", _.bind(function (dynamicContentAnnotations) {
				var options = {};
				// Constrain the zoom so it never goes lower than 2 cm
				var zoomRect;
				if (dynamicContentAnnotations && dynamicContentAnnotations.length == 1) {
					// Single annotation selected, zoom on the single annotation.
					zoomRect = dynamicContentAnnotations[0].rect.copy();
				} else {
					// Multiple annotation selected, zoom on the bounding box.
					var smallestX = Number.MAX_VALUE;
					var smallestY = Number.MAX_VALUE;
					var biggestX = Number.MIN_VALUE;
					var biggestY = Number.MIN_VALUE;
					for (var i = 0; i < dynamicContentAnnotations.length; i++) {
						var rectOrigin = dynamicContentAnnotations[i].rect.origin;
						var rectSize = dynamicContentAnnotations[i].rect.size;
						var rectBiggestX = rectOrigin.x + rectSize.width;
						var rectSmallestX = rectOrigin.x;
						var rectBiggestY = rectOrigin.y + rectSize.height;
						var rectSmallestY = rectOrigin.y;
						if (smallestX > rectSmallestX) {
							smallestX = rectSmallestX;
						}
						if (smallestY > rectSmallestY) {
							smallestY = rectSmallestY;
						}
						if (biggestX < rectBiggestX) {
							biggestX = rectBiggestX;
						}
						if (biggestY < rectBiggestY) {
							biggestY = rectBiggestY;
						}
					}
					var origin = new ShapeUtils.Point(smallestX, smallestY);
					var size = new ShapeUtils.Size(biggestX - smallestX, biggestY - smallestY);
					zoomRect = new ShapeUtils.Rect(origin, size);
				}

				var minSize = 20;

				var diff = null;
				if (Math.abs(zoomRect.size.width) < minSize) {
					diff = (minSize - zoomRect.size.width) / 2;
					zoomRect.origin.x -= diff;
					zoomRect.size.width = minSize;
				}
				if (Math.abs(zoomRect.size.height) < minSize) {
					diff = (minSize - zoomRect.size.height) / 2;
					zoomRect.origin.y -= diff;
					zoomRect.size.height = minSize;
				}

				// Convert annotation rect to world
				var transform = this.layerModels['DynContentLayer'].getTransform().copy();
				options.rect = transform.transformedRect(zoomRect);
				options.constrain = true;
				options.animated = true;

				this.dynContentModel.ensureVisibility();

				f.emit("viewport.fitInRect", options);
			}, this));

			this._docVersionID = this.f.getActiveDocument().getDocumentVersion().get("docversionid");
			this._projectID = this.f.getActiveDocument().getProjectID();
		};

		// inherit
		TextContentModule.prototype = Object.create(AbstractModule.prototype);
		TextContentModule.prototype.constructor = TextContentModule;

		/**
		 * When the dynamic contents are loaded ...
		 */
		TextContentModule.prototype.onDynContentsLoaded = function (dynContents) {
			// Check if actually dyn contents were loaded.
			if (dynContents && dynContents.length === 0) {
				return;
			}
			this._dynContents = dynContents;
			var contentStatementList = [];
			for (var i = 0; i < dynContents.length; i++) {
				if (dynContents[i].getDynContentType() == "Text") {
					var element = dynContents[i].getTextElement();
					var contentStatement = element.parentStatement;
					if ($.inArray(contentStatement, contentStatementList) < 0) {
						contentStatementList.push(contentStatement);
					}
				}
			}
			var textContentsLangs = "";
			_.each(contentStatementList, _.bind(function (contentStatement) {
				if (contentStatement && contentStatement.element && contentStatement.element.length > 0) {
					_.each(contentStatement.element, _.bind(function (csElement) {
						if (csElement && csElement.master === 'true') {
							textContentsLangs += csElement.language + ":m,";
						} else if (csElement) {
							textContentsLangs += csElement.language + ":t,";
						}
					}, this));
				}
			}, this));
			$.when(this._loadElementTypeList(), this._loadLanguages(textContentsLangs)).done(_.bind(function (csList, elementTypeList, languagesList) {

				// Check if at least one master language was selected in the project.
				var foundMasterLang = false;
				if (languagesList && languagesList.projectMasterLangs) {
					foundMasterLang = Object.keys(languagesList.projectMasterLangs).length;
				}

				// Check if a master language was found before publishing the tools and the widget.
				if (foundMasterLang) {

					// Publish the dynamic content widget.
					this._publishWidget(DynContentWidget.widgetName, _.bind(function (layoutType) {
						this._dynContentWidget = new DynContentWidget(this, this.f, contentStatementList, elementTypeList, languagesList);
						return this._dynContentWidget;
					}, this, contentStatementList, elementTypeList, languagesList));

					// Publish the dynamic content tool.
					this._selectTool = new DynContentTool(this.f, this.dynContentModel);
					this._publishTool(this._selectTool);

					// publish annotate tool bar component
					this._publishToolbarComponent(DynContentToolbarComponent.toolbarName, _.bind(function (layoutType) {
						this._dynContentComponent = new DynContentToolbarComponent(this.f, this.dynContentModel, this._selectTool);
						return this._dynContentComponent;
					}, this));

				} else {
					// Disable the dynamic content annotations.
					this.dynContentModel.disable();
				}


			}, this, contentStatementList));
		};

		/**
		 * Are there still unsaved Text?
		 * @override
		 * @returns {Boolean}
		 */
		TextContentModule.prototype.hasUnsavedElements = function () {
			return this.dynContentModel.getUnsavedTextContentsCount() > 0;
		};

		/**
		 * @override
		 * @returns {String}
		 */
		TextContentModule.prototype.getUnsavedElementsName = function () {
			return this.f.i18n.t("PCMTextWidgetTitle");
		};

		/**
		 * Prompt the user the save unsaved text content changes (if there are unsaved text content changes)
		 * @param {Function} callbackOnOk     - called when everything is OK (text contents saved)
		 * @param {Function} callbackOnCancel - called when the user cancels the operation (text contents are not saved)
		 * @param {Boolean} [canNotBeCancelled] - indicates the action cannot be cancelled, it is either revert or save
		 * @returns {Boolean} true if a prompt is shown
		 */
		TextContentModule.prototype.promptToSaveUnsavedChanges = function (callbackOnOk, callbackOnCancel, canNotBeCancelled) {
			if (this.hasUnsavedElements()) {
				this.f.loadBundle("viewer", function () {

					var modalPopup = new ModalPopup(this.f, "saveUnsavedTextContentChanges", this.f.getContext(), canNotBeCancelled);
					modalPopup.setRemoveFromDomAfterUsage(true);

					// the user wants to Save attributes before the callbackOnOk is executed
					modalPopup.setOkAction(_.bind(function () {
						// indicate to user that save operation is ongoing
						var loadingIndicator = $("<div>").css({
							'background': "url('" + requirejs.toUrl('web-viewer/layout/img/loaderTransparent.gif') + "') no-repeat 2px center",
							'padding-left': '20px',
							'line-height': '20px'
						}).addClass("unsavedItemsContent").text(this.f.t("PCMSaveOngoing"));
						modalPopup.setContent(loadingIndicator);

						//disable buttons while saving
						modalPopup.toggleButtons(false);
						modalPopup.toggleHeaderIcons(false);

						var validatePasswordAndSaveCallback = this._dynContentWidget.getValidatePasswordAndSaveHandler();

						$.when(validatePasswordAndSaveCallback()).done(function (status) {
							if (status === ContentConstants.ServerCall.SUCCESS) {
								// text contents are saved successfully!
								callbackOnOk();
							} else {
								// save operation failed
								if (callbackOnCancel) {
									callbackOnCancel();
								}
							}
							modalPopup.hide();
						}).fail(function () {
							if (callbackOnCancel) {
								callbackOnCancel();
							}
							modalPopup.hide();
						});

					}, this));

					if (!canNotBeCancelled) {
						// cancel the pending action
						modalPopup.setCancelAction(_.bind(function () {
							if (callbackOnCancel) {
								callbackOnCancel();
							}
						}, this));
						modalPopup.setCancelBtnText(this.f.i18n.t("Cancel"));
					}

					// Header + Content
					modalPopup.setHeader($("<span/>").addClass("popupTitle").text(this.f.t("PCMUnsavedWarningTitle")));
					modalPopup.setContent($("<div/>").addClass("unsavedItemsContent").text(this.f.t("PCMUnsavedWarningMsg")));
					modalPopup.setOkBtnText(this.f.i18n.t("Save"));
					modalPopup.show();
				}, this);
				return true;
			} else {
				// everything is OK, go on with OK action
				callbackOnOk();
				return false;
			}
		};

		TextContentModule.prototype._loadElementTypeList = function () {
			var deferred = $.Deferred();

			// load element type list
			elementTypeService.getElementTypes(_.bind(function (elementTypes) {
				deferred.resolve(elementTypes);
			}, deferred));

			return deferred.promise();
		};

		TextContentModule.prototype._loadLanguages = function (textContentsLangs) {
			var deferred = $.Deferred();

			// load language list
			languageService.getLanguages(this._projectID, _.bind(function (projectMasterLangs, projectTranslationLangs, allMasterLangs, allTranslationLangs) {
				var languagesList = {};
				languagesList.projectMasterLangs = projectMasterLangs;
				languagesList.projectTranslationLangs = projectTranslationLangs;
				languagesList.allMasterLangs = allMasterLangs;
				languagesList.allTranslationLangs = allTranslationLangs;

				deferred.resolve(languagesList);
			}, deferred), textContentsLangs);

			return deferred.promise();
		};

		/**
		 * When the active page has changed, react on it:
		 *
		 * Load text content data, unregister/unregister listners and display the widget if needed
		 *
		 * @param {AbstractDocumentPage} oldPage - previous active page
		 * @param {AbstractDocumentPage} newPage - new active page
		 */
		TextContentModule.prototype._whenActivePageChanged = function (oldPage, newPage) {

			//handle new page
			if (this.layerModels['DynContentLayer'] === undefined) {
				// Create the annotations layer
				this.layerModels['DynContentLayer'] = new DynContentLayerModel(this.f, "DynContent", this.dynContentModel);
				this.layerModels['DynContentPopupsLayer'] = new DynContentPopupsLayerModel(this.f, "DynContentPopups", this.dynContentModel);

				var viewportModule = this.f.getModule("viewport");
				// Global layer
				viewportModule.addLayer("Dyncontent", this.layerModels['DynContentLayer']);
				viewportModule.addLayer("Dyncontentpopups", this.layerModels['DynContentPopupsLayer']);
			}

			// unregister listeners
			if (oldPage) {
				this._unregisterAllListenersForObject(oldPage);
			}

			if (newPage) {
				//When attribute save happens in any AttributeCategoryModule(in the Attributes - Text Content View)
				//reload the widget with updated text contents
				this._registerListener("attributeCategory.saveFinished", function () {
					this._attributeSaveHappened = true;
				}, this, newPage);

				this._registerListener("attributeCategory.modalPopupClosed", function () {
					if (this._attributeSaveHappened === true) {
						this._attributeSaveHappened = false;
						this.dynContentModel.setPage(newPage, function (dynContents) {
							this.onDynContentsLoaded(dynContents);
						}.bind(this), true);
						this.dynContentModel.modelChanged();
					}
				}, this, newPage);

				this.dynContentModel.setPage(newPage, function (dynContents) {
					this.onDynContentsLoaded(dynContents);
				}.bind(this));
				this.dynContentModel.modelChanged();
			}
		};

		/**
		 * Select all the dynamic content annotations for a given Content Statement.
		 * 
		 * @param contentStatement
		 */
		TextContentModule.prototype.selectDynContentBasedOn = function (contentStatement, langCode) {
			this.dynContentModel.selectDynContentBasedOn(contentStatement, langCode);
		};

		/**
		 * Reload the dynamic content (throwing away unsaved actions!)
		 */
		TextContentModule.prototype.reload = function () {
			this.dynContentModel.reload(true);
		};

		return TextContentModule;
	});
